searchData={"items":[{"type":"module","title":"Ash","doc":"The primary interface to call actions and interact with resources.","ref":"Ash.html"},{"type":"type","title":"Ash.actor/0","doc":"The actor performing the action - can be any term.","ref":"Ash.html#t:actor/0"},{"type":"type","title":"Ash.aggregate/0","doc":"Aggregate specification for queries.\n\nCan be an `Ash.Query.Aggregate` struct, a `{name, kind}` tuple, or a `{name, kind, opts}` tuple with options.","ref":"Ash.html#t:aggregate/0"},{"type":"function","title":"Ash.aggregate/3","doc":"Runs an aggregate or aggregates over a resource query\n\nIf you pass an `%Ash.Query.Aggregate{}`, gotten from `Ash.Query.Aggregate.new()`,\nthe query provided as the first argument to this function will not apply. For this\nreason, it is preferred that you pass in the tuple format, i.e\n\nPrefer this:\n`Api.aggregate(query, {:count_of_things, :count})`\n\nOver this:\n`Api.aggregate(query, Ash.Query.Aggregate.new(...))`","ref":"Ash.html#aggregate/3"},{"type":"function","title":"Examples - Ash.aggregate/3","doc":"iex> MyApp.Post |> Ash.aggregate({:count, :count})\n    {:ok, %{count: 42}}\n\n    iex> query |> Ash.aggregate([{:avg_likes, :avg, field: :likes}, {:count, :count}])\n    {:ok, %{avg_likes: 10.5, count: 42}}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.aggregate({:sum_views, :sum, field: :view_count})\n    {:ok, %{sum_views: 1542}}","ref":"Ash.html#aggregate/3-examples"},{"type":"function","title":"See also - Ash.aggregate/3","doc":"- `aggregate!/3` for the raising version\n- `count/2` for counting records specifically\n- `sum/3` for summing field values\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#aggregate/3-see-also"},{"type":"function","title":"Options - Ash.aggregate/3","doc":"* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#aggregate/3-options"},{"type":"function","title":"Ash.aggregate!/3","doc":"Runs an aggregate or aggregates over a resource query, returning the result or raising an error.\n\nThis is the bang version of `aggregate/3` that raises an error if the operation fails.","ref":"Ash.html#aggregate!/3"},{"type":"function","title":"Examples - Ash.aggregate!/3","doc":"iex> MyApp.Post |> Ash.aggregate!({:count, :count})\n    42\n\n    iex> query |> Ash.aggregate!([{:avg_likes, :avg, field: :likes}, {:count, :count}])\n    %{avg_likes: 10.5, count: 42}","ref":"Ash.html#aggregate!/3-examples"},{"type":"function","title":"See also - Ash.aggregate!/3","doc":"- `aggregate/3` for the non-raising version\n- `count!/2` for counting records specifically\n- `sum!/3` for summing field values\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#aggregate!/3-see-also"},{"type":"function","title":"Ash.avg/3","doc":"Fetches the average of all values of a given field.","ref":"Ash.html#avg/3"},{"type":"function","title":"Examples - Ash.avg/3","doc":"iex> MyApp.Post |> Ash.avg(:view_count)\n    {:ok, 42.5}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.avg(:likes)\n    {:ok, 15.8}","ref":"Ash.html#avg/3-examples"},{"type":"function","title":"See also - Ash.avg/3","doc":"- `avg!/3` for the raising version\n- `sum/3` for getting the total sum\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#avg/3-see-also"},{"type":"function","title":"Ash.avg!/3","doc":"Fetches the average of all values of a given field or raises an error.","ref":"Ash.html#avg!/3"},{"type":"function","title":"Examples - Ash.avg!/3","doc":"iex> MyApp.Post |> Ash.avg!(:view_count)\n    42.5\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.avg!(:likes)\n    15.8","ref":"Ash.html#avg!/3-examples"},{"type":"function","title":"See also - Ash.avg!/3","doc":"- `avg/3` for the non-raising version\n- `sum!/3` for getting the total sum\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#avg!/3-see-also"},{"type":"function","title":"Ash.bulk_create/4","doc":"Creates many records.","ref":"Ash.html#bulk_create/4"},{"type":"function","title":"Assumptions - Ash.bulk_create/4","doc":"We assume that the input is a list of changesets all for the same action, or a list of input maps for the\nsame action with the `:resource` and `:action` option provided to illustrate which action it is for.","ref":"Ash.html#bulk_create/4-assumptions"},{"type":"function","title":"Performance/Feasibility - Ash.bulk_create/4","doc":"The performance of this operation depends on the data layer in question.\nData layers like AshPostgres will choose reasonable batch sizes in an attempt\nto handle large bulk actions, but that does not mean that you can pass a list of\n500k inputs and expect things to go off without a hitch (although it might).\nIf you need to do large data processing, you should look into projects like\nGenStage and Broadway. With that said, if you want to do things like support CSV upload\nand you place some reasonable limits on the size this is a great tool. You'll need to\ntest it yourself, YMMV.\n\nPassing `return_records?: true` can significantly increase the time it takes to perform the operation,\nand can also make the operation completely unreasonable due to the memory requirement. If you want to\ndo very large bulk creates and display all of the results, the suggestion is to annotate them with a\n\"bulk_create_id\" in the data layer, and then read the records with that `bulk_create_id` so that they can\nbe retrieved later if necessary.","ref":"Ash.html#bulk_create/4-performance-feasibility"},{"type":"function","title":"Changes/Validations - Ash.bulk_create/4","doc":"Changes will be applied in the order they are given on the actions as normal. Any change that exposes\nthe `bulk_change` callbacks will be applied on the entire list.","ref":"Ash.html#bulk_create/4-changes-validations"},{"type":"function","title":"After Action Hooks - Ash.bulk_create/4","doc":"The following requirements must be met for `after_action` hooks to function properly. If they are not met,\nand an after_action hook being applied to a changeset in a `change`.\n\n1. `return_records?` must be set to `true`.\n2. The changeset must be setting the primary key as part of its changes, so that we know which result applies to which\n   changeset.\n\nIt is possible to use `after_action` hooks with `bulk_change/3`, but you need to return the hooks along with the changesets.\nThis allows for setting up `after_action` hooks that don't need access to the returned record,\nor `after_action` hooks that can operate on the entire list at once.  See the documentation for that callback for more on\nhow to do accomplish that.","ref":"Ash.html#bulk_create/4-after-action-hooks"},{"type":"function","title":"See also - Ash.bulk_create/4","doc":"- `bulk_create!/4` for the raising version\n- `create/3` for creating single records\n- [Create Actions Guide](/documentation/topics/actions/create-actions.md) for understanding create operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#bulk_create/4-see-also"},{"type":"function","title":"Options - Ash.bulk_create/4","doc":"* `:upsert?` (`t:boolean/0`) - If a conflict is found based on the primary key, the record is updated in the database (requires upsert support) The default value is `false`.\n\n* `:upsert_identity` (`t:atom/0`) - The identity to use when detecting conflicts for `upsert?`, e.g. `upsert_identity: :full_name`. By default, the primary key is used. Has no effect if `upsert?: true` is not provided\n\n* `:upsert_fields` - The fields to upsert. If not set, the action's `upsert_fields` is used. Unlike singular `create`, `bulk_create` with `upsert?` requires that `upsert_fields` be specified explicitly in one of these two locations.\n\n* `:after_action` (function of arity 2) - An after_action hook to be added to each processed changeset\n\n* `:upsert_condition` (`t:term/0`) - An expression to check if the record should be updated when there's a conflict.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n* `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n* `:load` (`t:term/0`) - A load statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Valid values are :filter, :error The default value is `:filter`.\n\n* `:context` (`t:map/0`) - Context to set on each changeset\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n* `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n* `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n* `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n* `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n  or `max_page_size` of the action, or 100.\n\n* `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.  \n  Potential elements:  \n  `{:notification, notification}` - if `return_notifications?` is set to `true`\n  `{:ok, record}` - if `return_records?` is set to `true`\n  `{:error, error}` - an error that occurred. May be changeset or an individual error. The default value is `false`.\n\n* `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.  \n  Only relevant if `return_stream?` is set to `true` and all other\n  `return_*?` options are set to `false`. The default value is `false`.\n\n* `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n  will be skipped. The default value is `true`.\n\n* `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n  the results from each batch. This may be intensive for large bulk actions.  \n  Notifications will be automatically sent unless `return_notifications?` is set to `true`. The default value is `false`.\n\n* `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.  \n  Keep in mind:  \n  `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n  *inside* the transaction if you choose `transaction: :all`.\n   Valid values are :all, :batch, false The default value is `:batch`.\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"Ash.html#bulk_create/4-options"},{"type":"function","title":"Ash.bulk_create!/4","doc":"Creates many records, raising any errors that are returned. See `bulk_create/4` for more.","ref":"Ash.html#bulk_create!/4"},{"type":"function","title":"Ash.bulk_destroy/4","doc":"Destroys all items in the provided enumerable or query with the provided input.\n\nThe input is a map of valid inputs for the action. The input will be applied to all records in the enumerable/query.\n\nIf the data layer supports destroying from a query, and the destroy action can be done fully atomically,\nit will be updated in a single pass using the data layer.\n\nOtherwise, this will stream each record and update it.","ref":"Ash.html#bulk_destroy/4"},{"type":"function","title":"Options - Ash.bulk_destroy/4","doc":"* `:resource` (`Ash.Resource`) - The resource being destroyed. This must be provided if the input given is a stream, so we know ahead of time what the resource being updated is.\n\n* `:stream_batch_size` (`t:integer/0`) - Batch size to use if provided a query and the query must be streamed\n\n* `:authorize_query?` (`t:boolean/0`) - If a query is given, determines whether or not authorization is run on that query. The default value is `true`.\n\n* `:strategy` - The strategy or strategies to enable. :stream is used in all cases if the data layer does not support atomics. Valid values are :atomic, :atomic_batches, :stream The default value is `:atomic`.\n\n* `:filter` (`t:term/0`) - A filter to apply to records. This is also applied to a stream of inputs.\n\n* `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read The default value is `:keyset`.\n\n* `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n* `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n* `:load` (`t:term/0`) - A load statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Valid values are :filter, :error The default value is `:filter`.\n\n* `:context` (`t:map/0`) - Context to set on each changeset\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n* `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n* `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n* `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n* `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n  or `max_page_size` of the action, or 100.\n\n* `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.  \n  Potential elements:  \n  `{:notification, notification}` - if `return_notifications?` is set to `true`\n  `{:ok, record}` - if `return_records?` is set to `true`\n  `{:error, error}` - an error that occurred. May be changeset or an individual error. The default value is `false`.\n\n* `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.  \n  Only relevant if `return_stream?` is set to `true` and all other\n  `return_*?` options are set to `false`. The default value is `false`.\n\n* `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n  will be skipped. The default value is `true`.\n\n* `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n  the results from each batch. This may be intensive for large bulk actions.  \n  Notifications will be automatically sent unless `return_notifications?` is set to `true`. The default value is `false`.\n\n* `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.  \n  Keep in mind:  \n  `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n  *inside* the transaction if you choose `transaction: :all`.\n   Valid values are :all, :batch, false The default value is `:batch`.\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"Ash.html#bulk_destroy/4-options"},{"type":"function","title":"Ash.bulk_destroy!/4","doc":"Destroys all items in the provided enumerable or query with the provided input.\n\nSee `bulk_destroy/4` for more.","ref":"Ash.html#bulk_destroy!/4"},{"type":"function","title":"Ash.bulk_update/4","doc":"Updates all items in the provided enumerable or query with the provided input.\n\nThe input is a map of valid inputs for the action. The input will be applied to all records in the enumerable/query.\n\nIf the data layer supports updating from a query, and the update action can be done fully atomically,\nit will be updated in a single pass using the data layer.\n\nOtherwise, this will stream each record and update it.","ref":"Ash.html#bulk_update/4"},{"type":"function","title":"Options - Ash.bulk_update/4","doc":"* `:resource` (`Ash.Resource`) - The resource being updated. This must be provided if the input given is a stream, so we know ahead of time what the resource being updated is.\n\n* `:atomic_update` (`t:map/0`) - A map of atomic updates to apply. See `Ash.Changeset.atomic_update/3` for more.\n\n* `:stream_batch_size` (`t:integer/0`) - Batch size to use if provided a query and the query must be streamed\n\n* `:authorize_query?` (`t:boolean/0`) - If a query is given, determines whether or not authorization is run on that query. The default value is `true`.\n\n* `:filter` (`t:term/0`) - A filter to apply to records. This is also applied to a stream of inputs.\n\n* `:strategy` - The strategy or strategies to enable. :stream is used in all cases if the data layer does not support atomics. Valid values are :atomic, :atomic_batches, :stream The default value is `[:atomic]`.\n\n* `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read The default value is `:keyset`.\n\n* `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n* `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n* `:load` (`t:term/0`) - A load statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n* `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set. Valid values are :filter, :error\n\n* `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Valid values are :filter, :error The default value is `:filter`.\n\n* `:context` (`t:map/0`) - Context to set on each changeset\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n* `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n* `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n* `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n* `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n  or `max_page_size` of the action, or 100.\n\n* `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.  \n  Potential elements:  \n  `{:notification, notification}` - if `return_notifications?` is set to `true`\n  `{:ok, record}` - if `return_records?` is set to `true`\n  `{:error, error}` - an error that occurred. May be changeset or an individual error. The default value is `false`.\n\n* `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.  \n  Only relevant if `return_stream?` is set to `true` and all other\n  `return_*?` options are set to `false`. The default value is `false`.\n\n* `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n  will be skipped. The default value is `true`.\n\n* `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n  the results from each batch. This may be intensive for large bulk actions.  \n  Notifications will be automatically sent unless `return_notifications?` is set to `true`. The default value is `false`.\n\n* `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.  \n  Keep in mind:  \n  `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n  *inside* the transaction if you choose `transaction: :all`.\n   Valid values are :all, :batch, false The default value is `:batch`.\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"Ash.html#bulk_update/4-options"},{"type":"function","title":"Ash.bulk_update!/4","doc":"Updates all items in the provided enumerable or query with the provided input.\n\nSee `bulk_update/4` for more.","ref":"Ash.html#bulk_update!/4"},{"type":"function","title":"Ash.calculate/3","doc":"Evaluates the calculation on the resource.\n\nIf a record is provided, its field values will be used to evaluate the calculation.","ref":"Ash.html#calculate/3"},{"type":"function","title":"Examples - Ash.calculate/3","doc":"iex> Ash.calculate(post, :word_count)\n    {:ok, 142}\n\n    iex> Ash.calculate(MyApp.User, :age, args: %{birth_date: ~D[1990-01-01]})\n    {:ok, 34}","ref":"Ash.html#calculate/3-examples"},{"type":"function","title":"See also - Ash.calculate/3","doc":"- `calculate!/3` for the raising version\n- `d:Ash.Resource.Dsl.calculations` for defining calculations on resources\n- [Calculations Guide](/documentation/topics/resources/calculations.md) for understanding calculations","ref":"Ash.html#calculate/3-see-also"},{"type":"function","title":"Options - Ash.calculate/3","doc":"* `:args` (`t:map/0`) - Values for arguments referenced by the calculation. The default value is `%{}`.\n\n* `:refs` (`t:map/0`) - Values for references used by the calculation. The default value is `%{}`.\n\n* `:actor` (`t:term/0`) - The actor for handling `^actor/1` templates, supplied to calculation context.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant, supplied to calculation context.\n\n* `:context` (`t:map/0`) - Context to set on the calculation input.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the request is being authorized, provided to calculation context. The default value is `true`.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer, provided to the calculation context.\n\n* `:record` (`t:term/0`) - A record to use as the base of the calculation\n\n* `:data_layer?` (`t:boolean/0`) - Set to `true` to require that the value be computed within the data layer. Only works for calculations that define an expression.\n\n* `:reuse_values?` (`t:boolean/0`) - Set to `true` to reuse existing values on any provided record. Only necessary if providing a record as the basis for calculation. The default value is `false`.\n\n* `:domain` (`Ash.Domain`) - The domain to use for the action","ref":"Ash.html#calculate/3-options"},{"type":"function","title":"Ash.calculate!/3","doc":"Evaluates the calculation on the resource or raises an error. See `calculate/3` for more.","ref":"Ash.html#calculate!/3"},{"type":"function","title":"Examples - Ash.calculate!/3","doc":"iex> Ash.calculate!(post, :word_count)\n    142\n\n    iex> Ash.calculate!(MyApp.User, :age, args: %{birth_date: ~D[1990-01-01]})\n    34","ref":"Ash.html#calculate!/3-examples"},{"type":"function","title":"See also - Ash.calculate!/3","doc":"- `calculate/3` for the non-raising version\n- `d:Ash.Resource.Dsl.calculations` for defining calculations on resources\n- [Calculations Guide](/documentation/topics/resources/calculations.md) for understanding calculations","ref":"Ash.html#calculate!/3-see-also"},{"type":"function","title":"Ash.can/3","doc":"Returns whether or not the user can perform the action, or `:maybe`, returning any errors.\n\nIn cases with \"runtime\" checks (checks after the action), we may not be able to determine\nan answer, and so the value `:maybe` will be returned from `can/2`. The `can?` function assumes that\n`:maybe` means `true`. Keep in mind, this is just for doing things like \"can they do this\" in a UI,\nso assuming `:maybe` is `true` is fine. The actual action invocation will be properly checked regardless.\nIf you have runtime checks, you may need to use `can` instead of `can?`, or configure what `:maybe` means.","ref":"Ash.html#can/3"},{"type":"function","title":"Accepted inputs - Ash.can/3","doc":"You can pass many different inputs as the subject to `can/3`.\n\n```elixir\n# Can this user run this query.\nAsh.Query.t()\n\n# Can this user run this changeset.\nAsh.Changeset.t()\n\n# Can this user run this action.\nAsh.ActionInput.t()\n\n# Can this user run this action.\n{Ash.Resource.t(), :action}\n\n# Can this user run this action.\n{Ash.Resource.t(), %Action{}}\n\n# Can this user run this action with this input.\n{Ash.Resource.t(), :atom, %{...input}}\n\n# Can this user run this action with this input.\n{Ash.Resource.t(), %Action{}, %{...input}}\n```","ref":"Ash.html#can/3-accepted-inputs"},{"type":"function","title":"Examples - Ash.can/3","doc":"```elixir\n# no actor\nAsh.can?({MyApp.Accounts.Organization, :create}, nil)\n# => false\n\n# admin user actor\nAsh.can?({MyApp.Accounts.Organization, :create}, %MyApp.Accounts.User{role: :admin})\n# => true\n\n# check for permission to update a specific thing\nuser = MyApp.Accounts.get_post_by_id!(«uuid»)\nAsh.can?({user, :update}, %{role: :user})\n# => false\n\n# read actions\n# no logged in user. Will say `true` because the action\n# is allowed, but will just be filtered\nAsh.can?({MyApp.Accounts.Organization, :read}, nil)\n# => true\n\n# check for permission to read a specific thing\nAsh.can?({organization, :read}, nil)\n# => false\n```","ref":"Ash.html#can/3-examples"},{"type":"function","title":"Code Interfaces - Ash.can/3","doc":"When you define code interfaces, they provide `can_*` functions, which can be used like so:\n\n```elixir\n# no actor\nMyApp.Accounts.can_create_organization?(nil)\n# => false\n\n# admin user actor\nMyApp.Accounts.can_create_organization?(%MyApp.Accounts.User{role: :admin})\n# => true\n\n# check for permission to update a specific thing\nuser = MyApp.Accounts.get_post_by_id!(«uuid»)\nMyApp.Accounts.can_update_user(user, %{role: :user})\n# => false\n\n# read actions\n# no logged in user. Will say `true` because the action\n# is allowed, but will just be filtered\nMyApp.Accounts.can_read_organizations?(nil)\n# => true\n\n# check for permission to read a specific thing\nMyApp.Accounts.can_read_organizations?(nil, data: organization)\n# => false\n```","ref":"Ash.html#can/3-code-interfaces"},{"type":"function","title":"See also - Ash.can/3","doc":"- `can?/3` for the raising version that returns true/false\n- `d:Ash.Policy.Authorizer.policies` for defining authorization policies\n- [Actors and Authorization Guide](/documentation/topics/security/actors-and-authorization.md) for understanding authorization\n- [Policies Guide](/documentation/topics/security/policies.md) for defining authorization policies","ref":"Ash.html#can/3-see-also"},{"type":"function","title":"Options - Ash.can/3","doc":"* `:maybe_is` (`t:term/0`) - If the actor *may* be able to perform the action, what value should be returned. The default value is `:maybe`.\n\n* `:filter_with` - If set to `:error`, the query will raise an error on a match. If set to `:filter` the query will filter out unauthorized access. Valid values are :filter, :error The default value is `:filter`.\n\n* `:validate?` (`t:boolean/0`) - Whether or not to treat an invalid action as a non-allowed action. The default value is `false`.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether or not loaded data like aggregates, calculations and relationships should be checked in memory if possible, instead of querying. No effect if `pre_flight?` is `false`. The default value is `false`.\n\n* `:pre_flight?` (`t:boolean/0`) - Whether or not this is a pre_flight check (which may perform optimized in-memory checks) or the final proper check. The default value is `true`.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:run_queries?` (`t:boolean/0`) - Whether or not to run queries. If set to `true`, `:maybe` will not be returned. The default value is `true`.\n\n* `:data` - The record or records specifically attempting to be acted upon.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant to use for authorization\n\n* `:alter_source?` (`t:boolean/0`) - If set to `true`, the source being authorized is returned so it can be run. The default value is `false`.\n\n* `:base_query` (`t:term/0`) - A base query on which to apply an generated filters\n\n* `:no_check?` (`t:boolean/0`) - Whether or not authorization must pass at the strict/filter step, or if post-checks are allowed to be run The default value is `false`.\n\n* `:on_must_pass_strict_check` (`t:term/0`) - Override the value returned when `no_check?` is `true` but a check must be run.\n\n* `:atomic_changeset` (`t:term/0`) - A base query on which to apply an generated filters\n\n* `:return_forbidden_error?` (`t:boolean/0`) - Whether or not to return a forbidden error in cases of not being authorized. The default value is `false`.","ref":"Ash.html#can/3-options"},{"type":"function","title":"Ash.can?/3","doc":"Returns whether or not the user can perform the action, or raises on errors.\n\nCalls `can/3` with a `maybe_is: true`. See `can/3` for more info.","ref":"Ash.html#can?/3"},{"type":"function","title":"Examples - Ash.can?/3","doc":"iex> Ash.can?({MyApp.Post, :create}, actor)\n    true\n\n    iex> Ash.can?({MyApp.Post, :read}, nil)\n    true\n\n    iex> Ash.can?({post, :update}, actor)\n    false","ref":"Ash.html#can?/3-examples"},{"type":"function","title":"See also - Ash.can?/3","doc":"- `can/3` for the non-raising version that returns detailed results\n- `d:Ash.Policy.Authorizer.policies` for defining authorization policies\n- [Actors and Authorization Guide](/documentation/topics/security/actors-and-authorization.md) for understanding authorization\n- [Policies Guide](/documentation/topics/security/policies.md) for defining authorization policies","ref":"Ash.html#can?/3-see-also"},{"type":"function","title":"Options - Ash.can?/3","doc":"* `:maybe_is` (`t:term/0`) - If the actor *may* be able to perform the action, what value should be returned. The default value is `true`.\n\n* `:filter_with` - If set to `:error`, the query will raise an error on a match. If set to `:filter` the query will filter out unauthorized access. Valid values are :filter, :error The default value is `:filter`.\n\n* `:validate?` (`t:boolean/0`) - Whether or not to treat an invalid action as a non-allowed action. The default value is `false`.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether or not loaded data like aggregates, calculations and relationships should be checked in memory if possible, instead of querying. No effect if `pre_flight?` is `false`. The default value is `false`.\n\n* `:pre_flight?` (`t:boolean/0`) - Whether or not this is a pre_flight check (which may perform optimized in-memory checks) or the final proper check. The default value is `true`.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:run_queries?` (`t:boolean/0`) - Whether or not to run queries. If set to `true`, `:maybe` will not be returned. The default value is `true`.\n\n* `:data` - The record or records specifically attempting to be acted upon.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant to use for authorization\n\n* `:alter_source?` (`t:boolean/0`) - If set to `true`, the source being authorized is returned so it can be run. The default value is `false`.\n\n* `:base_query` (`t:term/0`) - A base query on which to apply an generated filters\n\n* `:no_check?` (`t:boolean/0`) - Whether or not authorization must pass at the strict/filter step, or if post-checks are allowed to be run The default value is `false`.\n\n* `:on_must_pass_strict_check` (`t:term/0`) - Override the value returned when `no_check?` is `true` but a check must be run.\n\n* `:atomic_changeset` (`t:term/0`) - A base query on which to apply an generated filters\n\n* `:return_forbidden_error?` (`t:boolean/0`) - Whether or not to return a forbidden error in cases of not being authorized. The default value is `false`.","ref":"Ash.html#can?/3-options"},{"type":"function","title":"Ash.context_to_opts/2","doc":"See  Ash.Context.to_opts/2 .","ref":"Ash.html#context_to_opts/2"},{"type":"function","title":"Ash.count/2","doc":"Fetches the count of results that would be returned from a given query.","ref":"Ash.html#count/2"},{"type":"function","title":"Examples - Ash.count/2","doc":"iex> MyApp.Post |> Ash.count()\n    {:ok, 42}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.count()\n    {:ok, 15}","ref":"Ash.html#count/2-examples"},{"type":"function","title":"See also - Ash.count/2","doc":"- `count!/2` for the raising version\n- `aggregate/3` for running multiple aggregates\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#count/2-see-also"},{"type":"function","title":"Ash.count!/2","doc":"Fetches the count of results that would be returned from a given query, or raises an error.","ref":"Ash.html#count!/2"},{"type":"function","title":"Examples - Ash.count!/2","doc":"iex> MyApp.Post |> Ash.count!()\n    42\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.count!()\n    15","ref":"Ash.html#count!/2-examples"},{"type":"function","title":"See also - Ash.count!/2","doc":"- `count/2` for the non-raising version\n- `aggregate!/3` for running multiple aggregates\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#count!/2-see-also"},{"type":"function","title":"Ash.create/3","doc":"Create a record.","ref":"Ash.html#create/3"},{"type":"function","title":"Examples - Ash.create/3","doc":"iex> Ash.create(MyApp.Post, %{title: \"Hello World\", content: \"...\"})\n    {:ok, %MyApp.Post{id: 1, title: \"Hello World\", content: \"...\"}}\n\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create, %{name: \"John\"})\n    iex> Ash.create(changeset)\n    {:ok, %MyApp.User{id: 1, name: \"John\"}}\n\n    iex> Ash.create(MyApp.Post, %{title: \"New Post\"}, return_notifications?: true)\n    {:ok, %MyApp.Post{id: 2, title: \"New Post\"}, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#create/3-examples"},{"type":"function","title":"See also - Ash.create/3","doc":"- `create!/3` for the raising version\n- `d:Ash.Resource.Dsl.actions.create` for defining create actions\n- `d:Ash.Resource.Dsl.attributes` for defining attributes\n- [Create Actions Guide](/documentation/topics/actions/create-actions.md) for understanding create operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#create/3-see-also"},{"type":"function","title":"Options - Ash.create/3","doc":"* `:upsert?` (`t:boolean/0`) - If a conflict is found based on the primary key, the record is updated in the database (requires upsert support) The default value is `false`.\n\n* `:return_skipped_upsert?` (`t:boolean/0`) - If `true`, and a record was *not* upserted because its filter prevented the upsert, the original record (which was *not* upserted) will be returned. The default value is `false`.\n\n* `:upsert_identity` (`t:atom/0`) - The identity to use when detecting conflicts for `upsert?`, e.g. `upsert_identity: :full_name`. By default, the primary key is used. Has no effect if `upsert?: true` is not provided\n\n* `:upsert_fields` - The fields to upsert. If not set, the action's upsert_fields is used, and if that is not set, then any fields not being set to defaults are written.\n\n* `:upsert_condition` (`t:term/0`) - An expression to check if the record should be updated when there's a conflict.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset","ref":"Ash.html#create/3-options"},{"type":"function","title":"Ash.create!/3","doc":"Create a record. See `create/2` for more information.","ref":"Ash.html#create!/3"},{"type":"function","title":"Examples - Ash.create!/3","doc":"iex> Ash.create!(MyApp.Post, %{title: \"Hello World\", content: \"...\"})\n    %MyApp.Post{id: 1, title: \"Hello World\", content: \"...\"}\n\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create, %{name: \"John\"})\n    iex> Ash.create!(changeset)\n    %MyApp.User{id: 1, name: \"John\"}\n\n    iex> Ash.create!(MyApp.Post, %{title: \"New Post\"}, return_notifications?: true)\n    {%MyApp.Post{id: 2, title: \"New Post\"}, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#create!/3-examples"},{"type":"function","title":"See also - Ash.create!/3","doc":"- `create/3` for the non-raising version\n- [Create Actions Guide](/documentation/topics/actions/create-actions.md) for understanding create operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#create!/3-see-also"},{"type":"type","title":"Ash.data_layer_query/0","doc":"A data layer query structure with execution and counting functions.\n\nContains the query that would be executed along with functions for counting,\nrunning the query, and loading any runtime data needed for the operation.","ref":"Ash.html#t:data_layer_query/0"},{"type":"function","title":"Ash.data_layer_query/2","doc":"Gets the full query and any runtime calculations that would be loaded","ref":"Ash.html#data_layer_query/2"},{"type":"function","title":"Examples - Ash.data_layer_query/2","doc":"iex> query = MyApp.Post |> Ash.Query.filter(published: true)\n    iex> Ash.data_layer_query(query)\n    {:ok, %{query: #Ecto.Query<...>, ash_query: %Ash.Query{}, count: #Function<...>, run: #Function<...>, load: #Function<...>}}\n\n    iex> MyApp.Post |> Ash.Query.limit(10) |> Ash.data_layer_query()\n    {:ok, %{query: #Ecto.Query<...>, ...}}","ref":"Ash.html#data_layer_query/2-examples"},{"type":"function","title":"See also - Ash.data_layer_query/2","doc":"- `data_layer_query!/2` for the raising version","ref":"Ash.html#data_layer_query/2-see-also"},{"type":"function","title":"Ash.data_layer_query!/2","doc":"Gets the full query and any runtime calculations that would be loaded, raising any errors.","ref":"Ash.html#data_layer_query!/2"},{"type":"function","title":"Examples - Ash.data_layer_query!/2","doc":"iex> query = MyApp.Post |> Ash.Query.filter(published: true)\n    iex> Ash.data_layer_query!(query)\n    %{query: #Ecto.Query<...>, ash_query: %Ash.Query{}, count: #Function<...>, run: #Function<...>, load: #Function<...>}\n\n    iex> MyApp.Post |> Ash.Query.limit(10) |> Ash.data_layer_query!()\n    %{query: #Ecto.Query<...>, ...}","ref":"Ash.html#data_layer_query!/2-examples"},{"type":"function","title":"See also - Ash.data_layer_query!/2","doc":"- `data_layer_query/2` for the non-raising version\n\nSee `data_layer_query/2` for more.","ref":"Ash.html#data_layer_query!/2-see-also"},{"type":"function","title":"Ash.destroy/2","doc":"Destroy a record.","ref":"Ash.html#destroy/2"},{"type":"function","title":"Examples - Ash.destroy/2","doc":"iex> Ash.destroy(post)\n    :ok\n\n    iex> changeset = Ash.Changeset.for_destroy(user, :archive)\n    iex> Ash.destroy(changeset)\n    {:ok, :ok}\n\n    iex> Ash.destroy(post, return_destroyed?: true)\n    {:ok, %MyApp.Post{id: 1, title: \"Deleted Post\"}}\n\n    iex> Ash.destroy(user, return_notifications?: true)\n    {:ok, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#destroy/2-examples"},{"type":"function","title":"See also - Ash.destroy/2","doc":"- `destroy!/2` for the raising version\n- `d:Ash.Resource.Dsl.actions.destroy` for defining destroy actions\n- [Destroy Actions Guide](/documentation/topics/actions/destroy-actions.md) for understanding destroy operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#destroy/2-see-also"},{"type":"function","title":"Options - Ash.destroy/2","doc":"* `:return_destroyed?` (`t:boolean/0`) - If true, the destroyed record is included in the return result, e.g `{:ok, destroyed}` or `{:ok, destroyed, notifications}` The default value is `false`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset","ref":"Ash.html#destroy/2-options"},{"type":"function","title":"Ash.destroy!/2","doc":"Destroy a record. See `destroy/2` for more information.","ref":"Ash.html#destroy!/2"},{"type":"function","title":"Examples - Ash.destroy!/2","doc":"iex> Ash.destroy!(post)\n    :ok\n\n    iex> changeset = Ash.Changeset.for_destroy(user, :archive)\n    iex> Ash.destroy!(changeset)\n    :ok\n\n    iex> Ash.destroy!(post, return_destroyed?: true)\n    %MyApp.Post{id: 1, title: \"Deleted Post\"}\n\n    iex> Ash.destroy!(user, return_notifications?: true)\n    [%Ash.Notifier.Notification{}]","ref":"Ash.html#destroy!/2-examples"},{"type":"function","title":"See also - Ash.destroy!/2","doc":"- `destroy/2` for the non-raising version\n- `d:Ash.Resource.Dsl.actions.destroy` for defining destroy actions\n- [Destroy Actions Guide](/documentation/topics/actions/destroy-actions.md) for understanding destroy operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#destroy!/2-see-also"},{"type":"function","title":"Ash.exists/2","doc":"Returns whether or not the query would return any results.","ref":"Ash.html#exists/2"},{"type":"function","title":"Examples - Ash.exists/2","doc":"iex> MyApp.Post |> Ash.exists()\n    {:ok, true}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: false) |> Ash.exists()\n    {:ok, false}","ref":"Ash.html#exists/2-examples"},{"type":"function","title":"See also - Ash.exists/2","doc":"- `exists?/2` for the raising version\n- `count/2` for getting the actual count\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations","ref":"Ash.html#exists/2-see-also"},{"type":"function","title":"Ash.exists?/2","doc":"Returns whether or not the query would return any results, or raises an error.","ref":"Ash.html#exists?/2"},{"type":"function","title":"Examples - Ash.exists?/2","doc":"iex> MyApp.Post |> Ash.exists?()\n    true\n\n    iex> MyApp.Post |> Ash.Query.filter(published: false) |> Ash.exists?()\n    false","ref":"Ash.html#exists?/2-examples"},{"type":"function","title":"See also - Ash.exists?/2","doc":"- `exists/2` for the non-raising version\n- `count!/2` for getting the actual count\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations","ref":"Ash.html#exists?/2-see-also"},{"type":"function","title":"Ash.first/3","doc":"Fetches the first value for a given field.","ref":"Ash.html#first/3"},{"type":"function","title":"Examples - Ash.first/3","doc":"iex> MyApp.Post |> Ash.first(:title)\n    {:ok, \"Hello World\"}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.first(:view_count)\n    {:ok, 42}","ref":"Ash.html#first/3-examples"},{"type":"function","title":"See also - Ash.first/3","doc":"- `first!/3` for the raising version\n- `list/3` for getting all values of a field\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#first/3-see-also"},{"type":"function","title":"Ash.first!/3","doc":"Fetches the first value for a given field, or raises an error.","ref":"Ash.html#first!/3"},{"type":"function","title":"Examples - Ash.first!/3","doc":"iex> MyApp.Post |> Ash.first!(:title)\n    \"Hello World\"\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.first!(:view_count)\n    42","ref":"Ash.html#first!/3-examples"},{"type":"function","title":"See also - Ash.first!/3","doc":"- `first/3` for the non-raising version\n- `list!/3` for getting all values of a field\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#first!/3-see-also"},{"type":"function","title":"Ash.get/3","doc":"Get a record by an identifier.\n\nFor a resource with a composite primary key, pass a keyword list or map, e.g\n`Ash.get(MyResource, %{first_key: 1, second_key: 2})`\n\nAdditionally, a keyword list or map of keys matching an identity can be provided.","ref":"Ash.html#get/3"},{"type":"function","title":"Examples - Ash.get/3","doc":"iex> Ash.get(MyApp.Post, 1)\n    {:ok, %MyApp.Post{id: 1, title: \"Hello World\"}}\n\n    iex> Ash.get(MyApp.User, %{email: \"user@example.com\"})\n    {:ok, %MyApp.User{id: 5, email: \"user@example.com\"}}\n\n    iex> Ash.get(MyApp.Post, %{first_key: 1, second_key: 2})\n    {:ok, %MyApp.Post{first_key: 1, second_key: 2}}","ref":"Ash.html#get/3-examples"},{"type":"function","title":"See also - Ash.get/3","doc":"- `get!/3` for the raising version\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations","ref":"Ash.html#get/3-see-also"},{"type":"function","title":"Options - Ash.get/3","doc":"* `:error?` (`t:boolean/0`) - Whether or not an error should be returned or raised when the record is not found. If set to false, `nil` will be returned. The default value is `true`.\n\n* `:load` (`t:term/0`) - Fields or relationships to load in the query. See `Ash.Query.load/2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#get/3-options"},{"type":"function","title":"Ash.get!/3","doc":"Get a record by an identifier, or raises an error. See `get/3` for more.","ref":"Ash.html#get!/3"},{"type":"function","title":"Examples - Ash.get!/3","doc":"iex> Ash.get!(MyApp.Post, 1)\n    %MyApp.Post{id: 1, title: \"Hello World\"}\n\n    iex> Ash.get!(MyApp.User, %{email: \"user@example.com\"})\n    %MyApp.User{id: 5, email: \"user@example.com\"}\n\n    iex> Ash.get!(MyApp.Post, %{first_key: 1, second_key: 2})\n    %MyApp.Post{first_key: 1, second_key: 2}","ref":"Ash.html#get!/3-examples"},{"type":"function","title":"See also - Ash.get!/3","doc":"- `get/3` for the non-raising version\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations","ref":"Ash.html#get!/3-see-also"},{"type":"function","title":"Ash.list/3","doc":"Fetches a list of all values of a given field.","ref":"Ash.html#list/3"},{"type":"function","title":"Examples - Ash.list/3","doc":"iex> MyApp.Post |> Ash.list(:title)\n    {:ok, [\"Hello World\", \"Another Post\", \"Final Post\"]}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.list(:view_count)\n    {:ok, [42, 15, 89]}","ref":"Ash.html#list/3-examples"},{"type":"function","title":"See also - Ash.list/3","doc":"- `list!/3` for the raising version\n- `first/3` for getting just the first value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#list/3-see-also"},{"type":"function","title":"Ash.list!/3","doc":"Fetches a list of all values of a given field or raises an error.","ref":"Ash.html#list!/3"},{"type":"function","title":"Examples - Ash.list!/3","doc":"iex> MyApp.Post |> Ash.list!(:title)\n    [\"Hello World\", \"Another Post\", \"Final Post\"]\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.list!(:view_count)\n    [42, 15, 89]","ref":"Ash.html#list!/3-examples"},{"type":"function","title":"See also - Ash.list!/3","doc":"- `list/3` for the non-raising version\n- `first!/3` for getting just the first value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#list!/3-see-also"},{"type":"function","title":"Ash.load/3","doc":"Load fields or relationships on already fetched records.\n\nAccepts a list of non-loaded fields and loads them on the provided records or a query, in\nwhich case the loaded fields of the query are used. Relationship loads can be nested, for\nexample: `Ash.load(record, [posts: [:comments]])`.","ref":"Ash.html#load/3"},{"type":"function","title":"Examples - Ash.load/3","doc":"iex> Ash.load(post, :comments)\n    {:ok, %MyApp.Post{comments: [%MyApp.Comment{}, ...]}}\n\n    iex> Ash.load(posts, [:author, :comments])\n    {:ok, [%MyApp.Post{author: %MyApp.User{}, comments: [...]}, ...]}\n\n    iex> Ash.load(user, [posts: [:comments]])\n    {:ok, %MyApp.User{posts: [%MyApp.Post{comments: [...]}]}}","ref":"Ash.html#load/3-examples"},{"type":"function","title":"See also - Ash.load/3","doc":"- `load!/3` for the raising version\n- `d:Ash.Resource.Dsl.relationships` for defining relationships to load\n- `d:Ash.Resource.Dsl.calculations` for defining calculations to load","ref":"Ash.html#load/3-see-also"},{"type":"function","title":"Options - Ash.load/3","doc":"* `:lazy?` (`t:boolean/0`) - If set to true, values will only be loaded if the related value isn't currently loaded. The default value is `false`.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#load/3-options"},{"type":"function","title":"Ash.load!/3","doc":"Load fields or relationships on already fetched records. See `load/3` for more information.","ref":"Ash.html#load!/3"},{"type":"function","title":"Examples - Ash.load!/3","doc":"iex> Ash.load!(post, :comments)\n    %MyApp.Post{comments: [%MyApp.Comment{}, ...]}\n\n    iex> Ash.load!(posts, [:author, :comments])\n    [%MyApp.Post{author: %MyApp.User{}, comments: [...]}, ...]\n\n    iex> Ash.load!(user, [posts: [:comments]])\n    %MyApp.User{posts: [%MyApp.Post{comments: [...]}]}","ref":"Ash.html#load!/3-examples"},{"type":"function","title":"See also - Ash.load!/3","doc":"- `load/3` for the non-raising version\n- `d:Ash.Resource.Dsl.relationships` for defining relationships to load\n- `d:Ash.Resource.Dsl.calculations` for defining calculations to load\n- [Relationships Guide](/documentation/topics/resources/relationships.md) for understanding relationships\n- [Calculations Guide](/documentation/topics/resources/calculations.md) for understanding calculations","ref":"Ash.html#load!/3-see-also"},{"type":"type","title":"Ash.load_statement/0","doc":"Load statement for relationships and calculations.\n\nCan be a query, a list of atoms, a single atom, keywords, or a list of atoms and tuples with options.","ref":"Ash.html#t:load_statement/0"},{"type":"function","title":"Ash.max/3","doc":"Fetches the greatest of all values of a given field.","ref":"Ash.html#max/3"},{"type":"function","title":"Examples - Ash.max/3","doc":"iex> MyApp.Post |> Ash.max(:view_count)\n    {:ok, 1542}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.max(:created_at)\n    {:ok, ~U[2023-12-25 10:30:00Z]}","ref":"Ash.html#max/3-examples"},{"type":"function","title":"See also - Ash.max/3","doc":"- `max!/3` for the raising version\n- `min/3` for getting the minimum value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#max/3-see-also"},{"type":"function","title":"Ash.max!/3","doc":"Fetches the greatest of all values of a given field or raises an error.","ref":"Ash.html#max!/3"},{"type":"function","title":"Examples - Ash.max!/3","doc":"iex> MyApp.Post |> Ash.max!(:view_count)\n    1542\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.max!(:created_at)\n    ~U[2023-12-25 10:30:00Z]","ref":"Ash.html#max!/3-examples"},{"type":"function","title":"See also - Ash.max!/3","doc":"- `max/3` for the non-raising version\n- `min!/3` for getting the minimum value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#max!/3-see-also"},{"type":"function","title":"Ash.min/3","doc":"Fetches the least of all values of a given field.","ref":"Ash.html#min/3"},{"type":"function","title":"Examples - Ash.min/3","doc":"iex> MyApp.Post |> Ash.min(:view_count)\n    {:ok, 5}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.min(:created_at)\n    {:ok, ~U[2023-01-01 08:00:00Z]}","ref":"Ash.html#min/3-examples"},{"type":"function","title":"See also - Ash.min/3","doc":"- `min!/3` for the raising version\n- `max/3` for getting the maximum value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#min/3-see-also"},{"type":"function","title":"Ash.min!/3","doc":"Fetches the least of all values of a given field or raises an error.","ref":"Ash.html#min!/3"},{"type":"function","title":"Examples - Ash.min!/3","doc":"iex> MyApp.Post |> Ash.min!(:view_count)\n    5\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.min!(:created_at)\n    ~U[2023-01-01 08:00:00Z]","ref":"Ash.html#min!/3-examples"},{"type":"function","title":"See also - Ash.min!/3","doc":"- `min/3` for the non-raising version\n- `max!/3` for getting the maximum value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#min!/3-see-also"},{"type":"function","title":"Ash.page/2","doc":"Fetch a page relative to the provided page.","ref":"Ash.html#page/2"},{"type":"function","title":"Examples - Ash.page/2","doc":"iex> Ash.page(page, :next)\n    {:ok, %Ash.Page.Offset{results: [...], more?: true}}\n\n    iex> Ash.page(page, :prev)\n    {:ok, %Ash.Page.Offset{results: [...], more?: false}}\n\n    iex> Ash.page(page, 3)\n    {:ok, %Ash.Page.Offset{results: [...], offset: 40}}","ref":"Ash.html#page/2-examples"},{"type":"function","title":"See also - Ash.page/2","doc":"- `page!/2` for the raising version","ref":"Ash.html#page/2-see-also"},{"type":"function","title":"Ash.page!/2","doc":"Fetch a page relative to the provided page or raises an error","ref":"Ash.html#page!/2"},{"type":"function","title":"Examples - Ash.page!/2","doc":"iex> Ash.page!(page, :next)\n    %Ash.Page.Offset{results: [...], more?: true}\n\n    iex> Ash.page!(page, :prev)\n    %Ash.Page.Offset{results: [...], more?: false}\n\n    iex> Ash.page!(page, 3)\n    %Ash.Page.Offset{results: [...], offset: 40}","ref":"Ash.html#page!/2-examples"},{"type":"function","title":"See also - Ash.page!/2","doc":"- `page/2` for the non-raising version","ref":"Ash.html#page!/2-see-also"},{"type":"type","title":"Ash.page_request/0","doc":"Page request options for paginated queries.\n\nCan be atoms for navigation (`:next`, `:prev`, `:first`, `:last`, `:self`) or an integer for specific page numbers.","ref":"Ash.html#t:page_request/0"},{"type":"function","title":"Ash.read/2","doc":"Runs an `Ash.Query`.\n\nFor more information on building a query, see `Ash.Query`.","ref":"Ash.html#read/2"},{"type":"function","title":"Examples - Ash.read/2","doc":"iex> Ash.read(MyApp.Post)\n    {:ok, [%MyApp.Post{id: 1, title: \"Hello\"}, %MyApp.Post{id: 2, title: \"World\"}]}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.read()\n    {:ok, [%MyApp.Post{id: 1, title: \"Hello\", published: true}]}\n\n    iex> MyApp.Post |> Ash.Query.limit(5) |> Ash.read()\n    {:ok, [%MyApp.Post{}, %MyApp.Post{}, ...]}","ref":"Ash.html#read/2-examples"},{"type":"function","title":"See also - Ash.read/2","doc":"- `read!/2` for the raising version\n- `d:Ash.Resource.Dsl.actions.read` for defining read actions\n- `d:Ash.Resource.Dsl.actions.read.pagination` for pagination configuration\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#read/2-see-also"},{"type":"function","title":"Options - Ash.read/2","doc":"* `:page` - Pagination options, see `Ash.read/2` for more.\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#read/2-options"},{"type":"function","title":"Pagination - Ash.read/2","doc":"#### Limit/offset pagination\n* `:offset` (`t:non_neg_integer/0`) - The number of records to skip from the beginning of the query\n\n* `:limit` (`t:pos_integer/0`) - The number of records to include in the page\n\n* `:filter` (`t:term/0`) - A filter to apply for pagination purposes, that should not be considered in the full count.  \n  This is used by the liveview paginator to only fetch the records that were *already* on the\n  page when refreshing data, to avoid pages jittering.\n\n* `:count` (`t:boolean/0`) - Whether or not to return the page with a full count of all records\n\n\n\n#### Keyset pagination\n* `:before` (`t:String.t/0`) - Get records that appear before the provided keyset (mutually exclusive with `after`)\n\n* `:after` (`t:String.t/0`) - Get records that appear after the provided keyset (mutually exclusive with `before`)\n\n* `:limit` (`t:pos_integer/0`) - How many records to include in the page\n\n* `:filter` (`t:term/0`) - See the `filter` option for offset pagination, this behaves the same.\n\n* `:count` (`t:boolean/0`) - Whether or not to return the page with a full count of all records","ref":"Ash.html#read/2-pagination"},{"type":"function","title":"Ash.read!/2","doc":"Run an `Ash.Query`. See `read/2` for more.","ref":"Ash.html#read!/2"},{"type":"function","title":"Examples - Ash.read!/2","doc":"iex> Ash.read!(MyApp.Post)\n    [%MyApp.Post{id: 1, title: \"Hello\"}, %MyApp.Post{id: 2, title: \"World\"}]\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.read!()\n    [%MyApp.Post{id: 1, title: \"Hello\", published: true}]\n\n    iex> MyApp.Post |> Ash.Query.limit(5) |> Ash.read!()\n    [%MyApp.Post{}, %MyApp.Post{}, ...]","ref":"Ash.html#read!/2-examples"},{"type":"function","title":"See also - Ash.read!/2","doc":"- `read/2` for the non-raising version\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations","ref":"Ash.html#read!/2-see-also"},{"type":"function","title":"Ash.read_first/2","doc":"Runs a query on a resource, returning a first result, nil, or an error.\n\nQuery is automatically limited to only return one result, unlike `read_one/3`","ref":"Ash.html#read_first/2"},{"type":"function","title":"Examples - Ash.read_first/2","doc":"iex> Ash.read_first(MyApp.Post)\n    {:ok, %MyApp.Post{id: 1, title: \"First Post\"}}\n\n    iex> MyApp.Post |> Ash.Query.sort(:created_at) |> Ash.read_first()\n    {:ok, %MyApp.Post{id: 1, created_at: ~U[2023-01-01 00:00:00Z]}}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: false) |> Ash.read_first()\n    {:ok, nil}","ref":"Ash.html#read_first/2-examples"},{"type":"function","title":"See also - Ash.read_first/2","doc":"- `read_first!/2` for the raising version","ref":"Ash.html#read_first/2-see-also"},{"type":"function","title":"Options - Ash.read_first/2","doc":"* `:not_found_error?` (`t:boolean/0`) - Whether or not to return an `Ash.Error.Query.NotFound` if no record is found. The default value is `false`.\n\n* `:page` - Pagination options, see `Ash.read/2` for more.\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#read_first/2-options"},{"type":"function","title":"Ash.read_first!/2","doc":"Runs an Ash query, returning the first result or nil, or raising an error. See `read_first/2` for more.","ref":"Ash.html#read_first!/2"},{"type":"function","title":"Examples - Ash.read_first!/2","doc":"iex> Ash.read_first!(MyApp.Post)\n    %MyApp.Post{id: 1, title: \"First Post\"}\n\n    iex> MyApp.Post |> Ash.Query.sort(:created_at) |> Ash.read_first!()\n    %MyApp.Post{id: 1, created_at: ~U[2023-01-01 00:00:00Z]}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: false) |> Ash.read_first!()\n    nil","ref":"Ash.html#read_first!/2-examples"},{"type":"function","title":"See also - Ash.read_first!/2","doc":"- `read_first/2` for the non-raising version","ref":"Ash.html#read_first!/2-see-also"},{"type":"function","title":"Ash.read_one/2","doc":"Runs a query on a resource, returning a single result, nil, or an error.\n\nIf more than one result would be returned, an error is returned instead.","ref":"Ash.html#read_one/2"},{"type":"function","title":"Examples - Ash.read_one/2","doc":"iex> Ash.read_one(MyApp.User, email: \"user@example.com\")\n    {:ok, %MyApp.User{id: 1, email: \"user@example.com\"}}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.read_one()\n    {:ok, %MyApp.Post{id: 1, published: true}}\n\n    iex> MyApp.User |> Ash.Query.filter(email: \"nonexistent@example.com\") |> Ash.read_one()\n    {:ok, nil}","ref":"Ash.html#read_one/2-examples"},{"type":"function","title":"See also - Ash.read_one/2","doc":"- `read_one!/2` for the raising version","ref":"Ash.html#read_one/2-see-also"},{"type":"function","title":"Options - Ash.read_one/2","doc":"* `:not_found_error?` (`t:boolean/0`) - Whether or not to return an `Ash.Error.Query.NotFound` if no record is found. The default value is `false`.\n\n* `:page` - Pagination options, see `Ash.read/2` for more.\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#read_one/2-options"},{"type":"function","title":"Ash.read_one!/2","doc":"Runs an ash query, returning a single result or raise an error. See `read_one/2` for more.","ref":"Ash.html#read_one!/2"},{"type":"function","title":"Examples - Ash.read_one!/2","doc":"iex> Ash.read_one!(MyApp.User, email: \"user@example.com\")\n    %MyApp.User{id: 1, email: \"user@example.com\"}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.read_one!()\n    %MyApp.Post{id: 1, published: true}\n\n    iex> MyApp.User |> Ash.Query.filter(email: \"nonexistent@example.com\") |> Ash.read_one!()\n    nil","ref":"Ash.html#read_one!/2-examples"},{"type":"function","title":"See also - Ash.read_one!/2","doc":"- `read_one/2` for the non-raising version","ref":"Ash.html#read_one!/2-see-also"},{"type":"type","title":"Ash.record_or_records/0","doc":"A single record or a list of records.","ref":"Ash.html#t:record_or_records/0"},{"type":"function","title":"Ash.reload/2","doc":"Refetches a record by primary key. See `get/2` for more.","ref":"Ash.html#reload/2"},{"type":"function","title":"Examples - Ash.reload/2","doc":"iex> Ash.reload(post)\n    {:ok, %MyApp.Post{id: 1, title: \"Updated Title\", updated_at: ~U[2023-12-25 10:30:00Z]}}\n\n    iex> Ash.reload(user, load: [:posts])\n    {:ok, %MyApp.User{id: 1, posts: [%MyApp.Post{}, ...]}}","ref":"Ash.html#reload/2-examples"},{"type":"function","title":"See also - Ash.reload/2","doc":"- `reload!/2` for the raising version","ref":"Ash.html#reload/2-see-also"},{"type":"function","title":"Ash.reload!/2","doc":"Refetches a record by primary key or raises an error. See `reload/2` for more.","ref":"Ash.html#reload!/2"},{"type":"function","title":"Examples - Ash.reload!/2","doc":"iex> Ash.reload!(post)\n    %MyApp.Post{id: 1, title: \"Updated Title\", updated_at: ~U[2023-12-25 10:30:00Z]}\n\n    iex> Ash.reload!(user, load: [:posts])\n    %MyApp.User{id: 1, posts: [%MyApp.Post{}, ...]}","ref":"Ash.html#reload!/2-examples"},{"type":"function","title":"See also - Ash.reload!/2","doc":"- `reload/2` for the non-raising version\n- `d:Ash.Resource.Dsl.relationships` for defining relationships to load","ref":"Ash.html#reload!/2-see-also"},{"type":"function","title":"Ash.run_action/2","doc":"Runs a generic action.","ref":"Ash.html#run_action/2"},{"type":"function","title":"Examples - Ash.run_action/2","doc":"iex> input = Ash.ActionInput.for_action(MyApp.Post, :send_email, %{email: \"test@example.com\"})\n    iex> Ash.run_action(input)\n    {:ok, :ok}\n\n    iex> input = Ash.ActionInput.for_action(MyApp.Calculator, :calculate_tax, %{amount: 100})\n    iex> Ash.run_action(input)\n    {:ok, 8.25}","ref":"Ash.html#run_action/2-examples"},{"type":"function","title":"See also - Ash.run_action/2","doc":"- `run_action!/2` for the raising version\n- `d:Ash.Resource.Dsl.actions.action` for defining generic actions\n- [Generic Actions Guide](/documentation/topics/actions/generic-actions.md) for understanding generic actions\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#run_action/2-see-also"},{"type":"function","title":"Options - Ash.run_action/2","doc":"* `:actor` (`t:term/0`) - The actor for handling `^actor/1` templates, supplied to calculation context.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant, supplied to calculation context.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the request should be authorized.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer, provided to the calculation context.\n\n* `:domain` (`Ash.Domain`) - The domain to use for the action\n\n* `:context` (`t:map/0`) - Context to set on the action input\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"Ash.html#run_action/2-options"},{"type":"function","title":"Ash.run_action!/2","doc":"Runs a generic action or raises an error. See `run_action/2` for more","ref":"Ash.html#run_action!/2"},{"type":"function","title":"Examples - Ash.run_action!/2","doc":"iex> input = Ash.ActionInput.for_action(MyApp.Post, :send_email, %{email: \"test@example.com\"})\n    iex> Ash.run_action!(input)\n    :ok\n\n    iex> input = Ash.ActionInput.for_action(MyApp.Calculator, :calculate_tax, %{amount: 100})\n    iex> Ash.run_action!(input)\n    8.25","ref":"Ash.html#run_action!/2-examples"},{"type":"function","title":"See also - Ash.run_action!/2","doc":"- `run_action/2` for the non-raising version\n- `d:Ash.Resource.Dsl.actions.action` for defining generic actions\n- [Generic Actions Guide](/documentation/topics/actions/generic-actions.md) for understanding generic actions\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#run_action!/2-see-also"},{"type":"function","title":"Ash.stream!/2","doc":"Streams the results of a query.","ref":"Ash.html#stream!/2"},{"type":"function","title":"Strategies - Ash.stream!/2","doc":"There are three strategies supported, and the best one available is always chosen. They are,\nin order from best to worst:\n\n- `:keyset`\n- `:offset`\n- `:full_read`\n\nBy default, only `:keyset` is supported. If you want to allow worse strategies to be used, pass\nthe worst one you wish to allow as the `allow_stream_with` option, i.e `allow_stream_with: :full_read`.\nIf you wish to specify a specific strategy to use, pass `stream_with: :strategy_name`.","ref":"Ash.html#stream!/2-strategies"},{"type":"function","title":"Keyset - Ash.stream!/2","doc":"This utilizes keyset pagination to accomplish this stream. The action must support keyset pagination.\nThis is the most efficient way to stream a query, because it works by using filters which can benefit\nfrom indexes in the data layer.","ref":"Ash.html#stream!/2-keyset"},{"type":"function","title":"Offset - Ash.stream!/2","doc":"This utilizes offset/limit to accomplish this stream. If the action supports offset pagination, that will\nbe used. Otherwise, if the data layer supports limit/offset, then explicit limits/offsets will be used.\nThis is a much less efficient way of streaming a resource than `keyset`. To use limit/offset to reliably\nstream, a sort must always be applied, and limit/offset in the data layer will generally require sorting\nthe entire table to figure out what is in each batch.","ref":"Ash.html#stream!/2-offset"},{"type":"function","title":"Full Read - Ash.stream!/2","doc":"This reads the entire table into memory with no limit. This is, generally speaking, the least efficient.","ref":"Ash.html#stream!/2-full-read"},{"type":"function","title":"Examples - Ash.stream!/2","doc":"iex> MyApp.Post |> Ash.stream!() |> Enum.take(10)\n    [%MyApp.Post{}, %MyApp.Post{}, ...]\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.stream!(strategy: :keyset) |> Enum.map(& &1.title)\n    [\"Hello World\", \"Another Post\", ...]\n\n    iex> MyApp.Post |> Ash.stream!(strategy: :offset, batch_size: 50) |> Stream.filter(& &1.likes > 10) |> Enum.to_list()\n    [%MyApp.Post{likes: 15}, ...]","ref":"Ash.html#stream!/2-examples"},{"type":"function","title":"See also - Ash.stream!/2","doc":"- `read/2` for non-streaming reads\n- `d:Ash.Resource.Dsl.actions.read` for defining read actions\n- `d:Ash.Resource.Dsl.actions.read.pagination` for pagination configuration","ref":"Ash.html#stream!/2-see-also"},{"type":"function","title":"Options - Ash.stream!/2","doc":"* `:batch_size` (`t:integer/0`) - How many records to request in each query run. Defaults to the pagination limits on the resource, or 250.\n\n* `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read The default value is `:keyset`.\n\n* `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more. Valid values are :keyset, :offset, :full_read\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.","ref":"Ash.html#stream!/2-options"},{"type":"function","title":"Ash.sum/3","doc":"Fetches the sum of a given field.","ref":"Ash.html#sum/3"},{"type":"function","title":"Examples - Ash.sum/3","doc":"iex> MyApp.Post |> Ash.sum(:view_count)\n    {:ok, 1542}\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.sum(:likes)\n    {:ok, 238}","ref":"Ash.html#sum/3-examples"},{"type":"function","title":"See also - Ash.sum/3","doc":"- `sum!/3` for the raising version\n- `avg/3` for getting the average value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Aggregates Guide](/documentation/topics/resources/aggregates.md) for resource-level aggregates","ref":"Ash.html#sum/3-see-also"},{"type":"function","title":"Ash.sum!/3","doc":"Fetches the sum of a given field or raises an error.","ref":"Ash.html#sum!/3"},{"type":"function","title":"Examples - Ash.sum!/3","doc":"iex> MyApp.Post |> Ash.sum!(:view_count)\n    1542\n\n    iex> MyApp.Post |> Ash.Query.filter(published: true) |> Ash.sum!(:likes)\n    238","ref":"Ash.html#sum!/3-examples"},{"type":"function","title":"See also - Ash.sum!/3","doc":"- `sum/3` for the non-raising version\n- `avg!/3` for getting the average value\n- `d:Ash.Resource.Dsl.aggregates` for defining aggregates on resources","ref":"Ash.html#sum!/3-see-also"},{"type":"function","title":"Ash.transaction/3","doc":"Wraps the execution of the function in a transaction with the resource's data_layer.\nCollects notifications during the function's execution and sends them if the transaction was successful.","ref":"Ash.html#transaction/3"},{"type":"function","title":"Examples - Ash.transaction/3","doc":"iex> Ash.transaction(MyApp.Post, fn ->\n    ...>   post = Ash.create!(MyApp.Post, %{title: \"Hello\"})\n    ...>   Ash.update!(post, %{content: \"World\"})\n    ...> end)\n    {:ok, %MyApp.Post{title: \"Hello\", content: \"World\"}}\n\n    iex> Ash.transaction([MyApp.User, MyApp.Post], fn ->\n    ...>   user = Ash.create!(MyApp.User, %{name: \"John\"})\n    ...>   Ash.create!(MyApp.Post, %{title: \"Hello\", author_id: user.id})\n    ...> end)\n    {:ok, %MyApp.Post{title: \"Hello\"}}\n\n    iex> Ash.transaction(MyApp.Post, fn ->\n    ...>   Ash.create!(MyApp.Post, %{title: \"Test\"})\n    ...> end, return_notifications?: true)\n    {:ok, %MyApp.Post{title: \"Test\"}, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#transaction/3-examples"},{"type":"function","title":"See also - Ash.transaction/3","doc":"- [Actions Guide](/documentation/topics/actions/actions.md) for understanding action concepts\n- [Development Testing Guide](/documentation/topics/development/testing.md) for testing with transactions","ref":"Ash.html#transaction/3-see-also"},{"type":"function","title":"Options - Ash.transaction/3","doc":"* `:timeout` (`t:timeout/0`) - The time in milliseconds (as an integer) to wait for the transaction to finish or `:infinity` to wait indefinitely.  \n  If not specified then default behaviour is adapter specific - for `Ecto`-based data layers it will be `15_000`.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you want to manually handle sending notifications.  \n  If true the returned tuple will contain notifications list as the last element.  \n  To send notifications use `Ash.Notifier.notify(notifications)`. It sends any notifications that can be sent, and returns the rest. The default value is `false`.","ref":"Ash.html#transaction/3-options"},{"type":"function","title":"Ash.update/3","doc":"Update a record.","ref":"Ash.html#update/3"},{"type":"function","title":"Examples - Ash.update/3","doc":"iex> Ash.update(post, %{title: \"Updated Title\"})\n    {:ok, %MyApp.Post{id: 1, title: \"Updated Title\"}}\n\n    iex> changeset = Ash.Changeset.for_update(user, :update, %{name: \"Jane\"})\n    iex> Ash.update(changeset)\n    {:ok, %MyApp.User{id: 1, name: \"Jane\"}}\n\n    iex> Ash.update(post, %{content: \"New content\"}, return_notifications?: true)\n    {:ok, %MyApp.Post{id: 1, content: \"New content\"}, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#update/3-examples"},{"type":"function","title":"See also - Ash.update/3","doc":"- `update!/3` for the raising version\n- `d:Ash.Resource.Dsl.actions.update` for defining update actions\n- `d:Ash.Resource.Dsl.changes` for defining changes\n- [Update Actions Guide](/documentation/topics/actions/update-actions.md) for understanding update operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#update/3-see-also"},{"type":"function","title":"Options - Ash.update/3","doc":"* `:params` (`t:map/0`) - Parameters to supply, ignored if the input is a changeset, only used when an identifier is given.\n\n* `:atomic_upgrade?` (`t:boolean/0`) - If true the action will be done atomically if it can (and is configured to do so), ignoring the in memory transformations and validations. You should not generally need to disable this. The default value is `true`.\n\n* `:domain` (`Ash.Domain`) - The domain to use.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset","ref":"Ash.html#update/3-options"},{"type":"function","title":"Ash.update!/3","doc":"Update a record. See `update/2` for more information.","ref":"Ash.html#update!/3"},{"type":"function","title":"Examples - Ash.update!/3","doc":"iex> Ash.update!(post, %{title: \"Updated Title\"})\n    %MyApp.Post{id: 1, title: \"Updated Title\"}\n\n    iex> changeset = Ash.Changeset.for_update(user, :update, %{name: \"Jane\"})\n    iex> Ash.update!(changeset)\n    %MyApp.User{id: 1, name: \"Jane\"}\n\n    iex> Ash.update!(post, %{content: \"New content\"}, return_notifications?: true)\n    {%MyApp.Post{id: 1, content: \"New content\"}, [%Ash.Notifier.Notification{}]}","ref":"Ash.html#update!/3-examples"},{"type":"function","title":"See also - Ash.update!/3","doc":"- `update/3` for the non-raising version\n- `d:Ash.Resource.Dsl.actions.update` for defining update actions\n- [Update Actions Guide](/documentation/topics/actions/update-actions.md) for understanding update operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.html#update!/3-see-also"},{"type":"module","title":"Ash.ActionInput","doc":"Input for a custom action\n\nMuch like an `Ash.Query` and `Ash.Changeset` are used to provide inputs into\nCRUD actions, this struct provides the inputs required to execute a generic\naction.","ref":"Ash.ActionInput.html"},{"type":"function","title":"Ash.ActionInput.add_error/3","doc":"Adds an error to the errors list and marks the action input as invalid.\n\nThis function allows you to add validation errors or other issues to the\naction input. Once an error is added, the input will be marked as invalid\nand action execution will be prevented.","ref":"Ash.ActionInput.html#add_error/3"},{"type":"function","title":"Examples - Ash.ActionInput.add_error/3","doc":"# Add a simple string error\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{})\n    ...> |> Ash.ActionInput.add_error(\"Missing required configuration\")\n    iex> input.valid?\n    false\n\n    # Add an error with a specific path\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:process_data, %{})\n    ...> |> Ash.ActionInput.add_error(\"Invalid format\", [:data, :format])\n    iex> input.errors |> List.first() |> Map.get(:path)\n    [:data, :format]\n\n    # Add multiple errors\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:complex_action, %{})\n    ...> |> Ash.ActionInput.add_error([\"Error 1\", \"Error 2\"])\n    iex> length(input.errors)\n    2\n\n    # Add structured error with keyword list\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:validate_input, %{})\n    ...> |> Ash.ActionInput.add_error(field: :email, message: \"is invalid\")","ref":"Ash.ActionInput.html#add_error/3-examples"},{"type":"function","title":"See also - Ash.ActionInput.add_error/3","doc":"- `Ash.Error.to_ash_error/3` for more on supported error values\n- Action implementations can use this to add custom validation errors\n- `set_argument/3` automatically adds errors for invalid argument values","ref":"Ash.ActionInput.html#add_error/3-see-also"},{"type":"function","title":"Ash.ActionInput.fetch_argument/2","doc":"Fetches the value of an argument provided to the input.\n\nReturns `{:ok, value}` if the argument exists, or `:error` if not found.\nThis is the safer alternative to `get_argument/2` when you need to distinguish\nbetween a `nil` value and a missing argument.","ref":"Ash.ActionInput.html#fetch_argument/2"},{"type":"function","title":"Examples - Ash.ActionInput.fetch_argument/2","doc":"# Fetch an argument that exists\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{priority: :high})\n    ...> |> Ash.ActionInput.fetch_argument(:priority)\n    {:ok, :high}\n\n    # Fetch an argument that doesn't exist\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{})\n    ...> |> Ash.ActionInput.fetch_argument(:missing_arg)\n    :error\n\n    # Distinguish between nil and missing arguments\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:example, %{optional_field: nil})\n    ...> |> Ash.ActionInput.fetch_argument(:optional_field)\n    {:ok, nil}\n\n    # Use in conditional logic\n    iex> input = MyApp.Post |> Ash.ActionInput.for_action(:process, %{})\n    iex> case Ash.ActionInput.fetch_argument(input, :mode) do\n    ...>   {:ok, mode} -> \"Processing in #{mode} mode\"\n    ...>   :error -> \"Using default processing mode\"\n    ...> end\n    \"Using default processing mode\"","ref":"Ash.ActionInput.html#fetch_argument/2-examples"},{"type":"function","title":"See also - Ash.ActionInput.fetch_argument/2","doc":"- `get_argument/2` for simpler argument access\n- `set_argument/3` for setting argument values\n- `for_action/4` for providing initial arguments","ref":"Ash.ActionInput.html#fetch_argument/2-see-also"},{"type":"function","title":"Ash.ActionInput.for_action/4","doc":"Creates a new input for a generic action.\n\nThis is the primary way to create action inputs for generic actions. It validates\nthe action exists, sets up the input with proper defaults, and validates any\nprovided arguments according to the action's argument definitions.","ref":"Ash.ActionInput.html#for_action/4"},{"type":"function","title":"Examples - Ash.ActionInput.for_action/4","doc":"# Create input for a simple action\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{message: \"Hello\"})\n    %Ash.ActionInput{action: %{name: :send_notification}, arguments: %{message: \"Hello\"}, ...}\n\n    # Create input with options\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:complex_action, %{data: \"test\"},\n    ...>   actor: current_user, authorize?: true)\n    %Ash.ActionInput{arguments: %{data: \"test\"}, ...}\n\n    # Create input and then modify it\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:example, %{})\n    ...> |> Ash.ActionInput.set_context(%{source: \"api\"})\n    iex> input.action.name\n    :example","ref":"Ash.ActionInput.html#for_action/4-examples"},{"type":"function","title":"Options - Ash.ActionInput.for_action/4","doc":"* `:domain` (`Ash.Domain`) - The domain to use for the action. The resource's domain is used by default.\n\n* `:context` (`t:map/0`) - Context to set on the action input. The default value is `%{}`.\n\n* `:authorize?` - Whether or not to run authorization on the action. Default behavior of this option is controlled by the domain.\n\n* `:tenant` (`t:term/0`) - The tenant to use for the action.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:actor` (`t:term/0`) - The actor performing the action\n\n* `:skip_unknown_inputs` - A list of unknown inputs to skip. Use `:*` to skip all unknown inputs.\n\n* `:tracer` (`t:term/0`) - A tracer or list of tracers to trace action execution.\n\n* `:private_arguments` (`t:map/0`) - A list of private arguments to be set before the action is invoked. The default value is `%{}`.","ref":"Ash.ActionInput.html#for_action/4-options"},{"type":"function","title":"See also - Ash.ActionInput.for_action/4","doc":"- `new/2` for creating basic inputs\n- `set_argument/3` for adding arguments after creation\n- `Ash.run_action/2` for executing the action with the input\n- `d:Ash.Resource.Dsl.actions.action` for defining generic actions\n- [Generic Actions Guide](/documentation/topics/actions/generic-actions.md) for understanding generic actions\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.ActionInput.html#for_action/4-see-also"},{"type":"function","title":"Ash.ActionInput.get_argument/2","doc":"Gets the value of an argument provided to the input.\n\nReturns the argument value if found, or `nil` if not found. Arguments are\nvalidated and cast according to the action's argument definitions when set.","ref":"Ash.ActionInput.html#get_argument/2"},{"type":"function","title":"Examples - Ash.ActionInput.get_argument/2","doc":"# Get an argument that exists\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_email, %{recipient: \"user@example.com\"})\n    ...> |> Ash.ActionInput.get_argument(:recipient)\n    \"user@example.com\"\n\n    # Get an argument that doesn't exist returns nil\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_email, %{})\n    ...> |> Ash.ActionInput.get_argument(:missing_arg)\n    nil\n\n    # Arguments can be accessed by string or atom key\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:example, %{\"message\" => \"hello\"})\n    ...> |> Ash.ActionInput.get_argument(:message)\n    \"hello\"","ref":"Ash.ActionInput.html#get_argument/2-examples"},{"type":"function","title":"See also - Ash.ActionInput.get_argument/2","doc":"- `fetch_argument/2` for safer argument access with explicit error handling\n- `set_argument/3` for setting argument values\n- `for_action/4` for providing initial arguments","ref":"Ash.ActionInput.html#get_argument/2-see-also"},{"type":"function","title":"Ash.ActionInput.new/2","doc":"Creates a new action input from a resource.\n\nThis creates a basic action input struct that can be used as a starting point\nfor building inputs for generic actions. Use `for_action/4` to create an input\nbound to a specific action.","ref":"Ash.ActionInput.html#new/2"},{"type":"function","title":"Examples - Ash.ActionInput.new/2","doc":"# Create a new action input for a resource\n    iex> Ash.ActionInput.new(MyApp.Post)\n    %Ash.ActionInput{resource: MyApp.Post, domain: nil, ...}\n\n\n    # Usually you'll want to use for_action/4 instead\n    iex> MyApp.Post |> Ash.ActionInput.for_action(:send_notification, %{message: \"Hello\"})\n    %Ash.ActionInput{action: %{name: :send_notification}, arguments: %{message: \"Hello\"}, ...}","ref":"Ash.ActionInput.html#new/2-examples"},{"type":"function","title":"See also - Ash.ActionInput.new/2","doc":"- `for_action/4` for creating action-specific inputs\n- `set_argument/3` for adding arguments\n- `set_context/2` for adding context","ref":"Ash.ActionInput.html#new/2-see-also"},{"type":"function","title":"Ash.ActionInput.set_argument/3","doc":"Sets an argument value on the action input.\n\nThe argument value is validated and cast according to the action's argument\ndefinition. If validation fails, errors will be added to the input and it\nwill be marked as invalid.","ref":"Ash.ActionInput.html#set_argument/3"},{"type":"function","title":"Examples - Ash.ActionInput.set_argument/3","doc":"# Set a simple argument\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{})\n    ...> |> Ash.ActionInput.set_argument(:message, \"Hello World\")\n    ...> |> Ash.ActionInput.get_argument(:message)\n    \"Hello World\"\n\n    # Set multiple arguments by chaining\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:complex_action, %{})\n    ...> |> Ash.ActionInput.set_argument(:priority, :high)\n    ...> |> Ash.ActionInput.set_argument(:batch_size, 100)\n    iex> Ash.ActionInput.get_argument(input, :priority)\n    :high\n\n    # Arguments are validated according to type\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:schedule_job, %{})\n    ...> |> Ash.ActionInput.set_argument(:run_at, ~U[2024-01-01 10:00:00Z])\n    iex> input.valid?\n    true","ref":"Ash.ActionInput.html#set_argument/3-examples"},{"type":"function","title":"See also - Ash.ActionInput.set_argument/3","doc":"- `get_argument/2` for retrieving argument values\n- `set_private_argument/3` for setting private arguments\n- `for_action/4` for providing initial arguments","ref":"Ash.ActionInput.html#set_argument/3-see-also"},{"type":"function","title":"Ash.ActionInput.set_context/2","doc":"Deep merges the provided map into the input context.\n\nContext is used to pass additional information through the action pipeline\nthat can be accessed by action implementations, changes, and validations.\nThe context is merged deeply, so nested maps will be combined rather than replaced.\n\nDo not use the `private` key in your custom context, as that is reserved for\ninternal use.","ref":"Ash.ActionInput.html#set_context/2"},{"type":"function","title":"Examples - Ash.ActionInput.set_context/2","doc":"# Set simple context values\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.new()\n    ...> |> Ash.ActionInput.set_context(%{source: \"api\", user_id: 123})\n    ...> |> then(& &1.context.source)\n    \"api\"\n\n    # Context is merged deeply\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.new()\n    ...> |> Ash.ActionInput.set_context(%{metadata: %{version: 1}})\n    ...> |> Ash.ActionInput.set_context(%{metadata: %{trace_id: \"abc123\"}})\n    iex> input.context.metadata\n    %{version: 1, trace_id: \"abc123\"}\n\n    # Use context in action implementations\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:process_data, %{data: \"test\"})\n    ...> |> Ash.ActionInput.set_context(%{\n    ...>   request_id: \"req_456\",\n    ...>   feature_flags: %{new_algorithm: true}\n    ...> })","ref":"Ash.ActionInput.html#set_context/2-examples"},{"type":"function","title":"See also - Ash.ActionInput.set_context/2","doc":"- `for_action/4` for setting context when creating inputs\n- Action implementations can access context for custom logic\n- `set_tenant/2` for tenant-specific context","ref":"Ash.ActionInput.html#set_context/2-see-also"},{"type":"function","title":"Ash.ActionInput.set_private_argument/3","doc":"Sets a private argument value on the action input.\n\nPrivate arguments are not exposed in the public API and can only be set\ninternally. This function will only work for arguments marked as `public?: false`\nin the action definition.","ref":"Ash.ActionInput.html#set_private_argument/3"},{"type":"function","title":"Examples - Ash.ActionInput.set_private_argument/3","doc":"# Set a private argument (assuming :internal_flag is private)\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:example, %{})\n    ...> |> Ash.ActionInput.set_private_argument(:internal_flag, true)\n    ...> |> Ash.ActionInput.get_argument(:internal_flag)\n    true\n\n    # Attempting to set a public argument as private will error\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:example, %{})\n    ...> |> Ash.ActionInput.set_private_argument(:public_arg, \"value\")\n    iex> input.valid?\n    false\n\n    # Use in action implementations for internal state\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:complex_workflow, %{data: \"user_data\"})\n    ...> |> Ash.ActionInput.set_private_argument(:workflow_step, 1)","ref":"Ash.ActionInput.html#set_private_argument/3-examples"},{"type":"function","title":"See also - Ash.ActionInput.set_private_argument/3","doc":"- `set_argument/3` for setting public arguments\n- `get_argument/2` for retrieving argument values\n- Action argument definitions with `public?: false`","ref":"Ash.ActionInput.html#set_private_argument/3-see-also"},{"type":"function","title":"Ash.ActionInput.set_tenant/2","doc":"Sets the tenant to use when calling the action.\n\nIn multitenant applications, this configures which tenant's data the action\nshould operate on. The tenant value is used for data isolation and access control.","ref":"Ash.ActionInput.html#set_tenant/2"},{"type":"function","title":"Examples - Ash.ActionInput.set_tenant/2","doc":"# Set tenant using a string identifier\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.new()\n    ...> |> Ash.ActionInput.set_tenant(\"org_123\")\n    ...> |> then(& &1.tenant)\n    \"org_123\"\n\n    # Set tenant using a struct that implements Ash.ToTenant\n    iex> org = %MyApp.Organization{id: 456}\n    iex> input = MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:send_notification, %{})\n    ...> |> Ash.ActionInput.set_tenant(org)\n    iex> input.tenant == org\n    true\n\n    # Use with action execution\n    iex> MyApp.Post\n    ...> |> Ash.ActionInput.for_action(:cleanup, %{})\n    ...> |> Ash.ActionInput.set_tenant(\"tenant_456\")\n    ...> |> Ash.run_action()","ref":"Ash.ActionInput.html#set_tenant/2-examples"},{"type":"function","title":"See also - Ash.ActionInput.set_tenant/2","doc":"- `for_action/4` for setting tenant when creating inputs\n- `Ash.ToTenant` protocol for custom tenant conversion\n- `set_context/2` for adding tenant to action context","ref":"Ash.ActionInput.html#set_tenant/2-see-also"},{"type":"type","title":"Ash.ActionInput.t/0","doc":"An action input struct for generic (non-CRUD) actions.\n\nContains all the information needed to execute a generic action including\narguments, context, tenant information, and validation state. Built using\n`for_action/4` and modified with functions like `set_argument/3` and `set_context/2`.","ref":"Ash.ActionInput.html#t:t/0"},{"type":"module","title":"Ash.Changeset","doc":"Changesets are used to create and update data in Ash.\n\nCreate a changeset with `new/1` or `new/2`, and alter the attributes\nand relationships using the functions provided in this module.  Nothing in this module\nactually incurs changes in a data layer. To commit a changeset, see `Ash.create/2`\nand `Ash.update/2`.\n\n# Changeset lifecycle","ref":"Ash.Changeset.html"},{"type":"module","title":"Action Lifecycle - Ash.Changeset","doc":"The following example illustrates the hook lifecycle of a changeset.\n\n```elixir\ndefmodule AshChangesetLifeCycleExample do\n  def change(changeset, _, _) do\n    changeset\n    # execute code both before and after the transaction\n    |> Ash.Changeset.around_transaction(fn changeset, callback ->\n      callback.(changeset)\n    end)\n    # execute code before the transaction is started. Use for things like external calls\n    |> Ash.Changeset.before_transaction(fn changeset -> changeset end)\n    # execute code in the transaction, before and after the data layer is called\n    |> Ash.Changeset.around_action(fn changeset, callback ->\n      callback.(changeset)\n    end)\n    # execute code in the transaction, before the data layer is called\n    |> Ash.Changeset.before_action(fn changeset -> changeset end)\n    # execute code in the transaction, after the data layer is called, only if the action is successful\n    |> Ash.Changeset.after_action(fn changeset, result -> {:ok, result} end)\n    # execute code after the transaction, both in success and error cases\n    |> Ash.Changeset.after_transaction(fn changeset, success_or_error_result -> success_or_error_result end\n  end\nend\n```","ref":"Ash.Changeset.html#module-action-lifecycle"},{"type":"function","title":"Ash.Changeset.accessing/3","doc":"Returns a list of attributes, aggregates, relationships, and calculations that are being loaded\n\nProvide a list of field types to narrow down the returned results.","ref":"Ash.Changeset.html#accessing/3"},{"type":"function","title":"Ash.Changeset.add_error/3","doc":"Add an error to the errors list and mark the changeset as invalid.\n\nSee `Ash.Error.to_ash_error/3` for more on supported values for `error`","ref":"Ash.Changeset.html#add_error/3"},{"type":"function","title":"Ash.Changeset.after_action/3","doc":"Adds an after_action hook to the changeset.\n\nAfter action hooks run within the database transaction after the data layer action succeeds.\nThey are perfect for:\n\n- Creating related records that depend on the main action's result\n- Sending notifications with the final record data\n- Updating derived data or maintaining referential integrity\n- Triggering business logic that needs the persisted record\n- Creating audit trails with the actual saved data\n\nThe hook receives both the changeset and the successfully created/updated record.\nAny errors returned will roll back the entire transaction.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Changeset.html#after_action/3"},{"type":"function","title":"Examples - Ash.Changeset.after_action/3","doc":"# Create related audit record with the final data\n    iex> changeset = Ash.Changeset.for_update(user, :update_profile, %{email: \"new@example.com\"})\n    iex> changeset = Ash.Changeset.after_action(changeset, fn changeset, updated_user ->\n    ...>   {:ok, _audit} = MyApp.AuditLog.create(%{\n    ...>     action: \"profile_updated\",\n    ...>     user_id: updated_user.id,\n    ...>     changes: changeset.attributes,\n    ...>     actor_id: changeset.context.actor.id,\n    ...>     timestamp: DateTime.utc_now()\n    ...>   })\n    ...>   {:ok, updated_user}\n    ...> end)\n\n    # Send notification with the final record\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{total: 100.00})\n    iex> changeset = Ash.Changeset.after_action(changeset, fn _changeset, order ->\n    ...>   NotificationService.send_order_confirmation(%{\n    ...>     order_id: order.id,\n    ...>     customer_email: order.customer_email,\n    ...>     total: order.total,\n    ...>     order_number: order.number  # Generated by the database\n    ...>   })\n    ...>   {:ok, order}\n    ...> end)\n\n    # Update related records that depend on the main record\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :publish, %{title: \"My Post\"})\n    iex> changeset = Ash.Changeset.after_action(changeset, fn _changeset, post ->\n    ...>   # Update author's post count\n    ...>   {:ok, _author} = MyApp.User\n    ...>   |> Ash.Changeset.for_update(:increment_post_count, %{})\n    ...>   |> Ash.update()\n    ...>\n    ...>   {:ok, post}\n    ...> end)","ref":"Ash.Changeset.html#after_action/3-examples"},{"type":"function","title":"See also - Ash.Changeset.after_action/3","doc":"- `before_action/3` for hooks that run before the data layer action\n- `after_transaction/3` for hooks that run after the transaction commits\n- `around_action/2` for hooks that wrap the data layer action","ref":"Ash.Changeset.html#after_action/3-see-also"},{"type":"type","title":"Ash.Changeset.after_action_fun/0","doc":"Function type for after action hooks.\n\nReceives the changeset and the successfully created/updated record, and can return\nthe record optionally with notifications, or an error.","ref":"Ash.Changeset.html#t:after_action_fun/0"},{"type":"function","title":"Ash.Changeset.after_transaction/3","doc":"Adds an after_transaction hook to the changeset. Cannot be called within other hooks.\n\nAfter transaction hooks run outside the database transaction and are called regardless\nof whether the action succeeded or failed. They are essential for:\n\n- Cleanup operations that must happen regardless of success/failure\n- External service notifications that shouldn't be rolled back\n- Logging final outcomes for auditing purposes\n- Releasing external resources or locks\n- Sending emails or notifications that should persist even if the action failed\n\nThe hook receives the changeset and either `{:ok, result}` or `{:error, error}`,\nallowing you to handle both success and failure cases appropriately.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Changeset.html#after_transaction/3"},{"type":"function","title":"Examples - Ash.Changeset.after_transaction/3","doc":"# Send notification regardless of order creation success/failure\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{total: 100.00})\n    iex> changeset = Ash.Changeset.after_transaction(changeset, fn changeset, result ->\n    ...>   case result do\n    ...>     {:ok, order} ->\n    ...>       NotificationService.order_created_successfully(order.id, changeset.context.actor)\n    ...>     {:error, _error} ->\n    ...>       NotificationService.order_creation_failed(changeset.attributes, changeset.context.actor)\n    ...>   end\n    ...>   result  # Always return the original result\n    ...> end)\n\n    # Release external resources or cleanup\n    iex> changeset = Ash.Changeset.for_update(file, :process, %{status: \"processing\"})\n    iex> changeset = Ash.Changeset.after_transaction(changeset, fn changeset, result ->\n    ...>   # Always release the file lock, regardless of processing outcome\n    ...>   ExternalFileService.release_lock(changeset.data.file_path)\n    ...>\n    ...>   case result do\n    ...>     {:ok, processed_file} ->\n    ...>       Logger.info(\"File processing completed successfully: #{processed_file.id}\")\n    ...>     {:error, error} ->\n    ...>       Logger.error(\"File processing failed: #{inspect(error)}\")\n    ...>       # Could trigger retry mechanism here\n    ...>   end\n    ...>   result\n    ...> end)\n\n    # Audit final outcome for compliance\n    iex> changeset = Ash.Changeset.for_update(sensitive_record, :update, %{classification: \"top_secret\"})\n    iex> changeset = Ash.Changeset.after_transaction(changeset, fn changeset, result ->\n    ...>   outcome = case result do\n    ...>     {:ok, _} -> \"success\"\n    ...>     {:error, _} -> \"failure\"\n    ...>   end\n    ...>\n    ...>   ComplianceLogger.log_security_action(%{\n    ...>     action: :update_classification,\n    ...>     actor: changeset.context.actor,\n    ...>     resource_id: changeset.data.id,\n    ...>     outcome: outcome,\n    ...>     timestamp: DateTime.utc_now()\n    ...>   })\n    ...>   result\n    ...> end)","ref":"Ash.Changeset.html#after_transaction/3-examples"},{"type":"function","title":"See also - Ash.Changeset.after_transaction/3","doc":"- `after_action/3` for hooks that run within the transaction on success only\n- `before_transaction/3` for hooks that run before the transaction starts\n- `around_transaction/2` for hooks that wrap the entire transaction","ref":"Ash.Changeset.html#after_transaction/3-see-also"},{"type":"type","title":"Ash.Changeset.after_transaction_fun/0","doc":"Function type for after transaction hooks.\n\nReceives the changeset and the result (success or failure) of the action,\nand returns the result (potentially modified).","ref":"Ash.Changeset.html#t:after_transaction_fun/0"},{"type":"function","title":"Ash.Changeset.apply_attributes/2","doc":"Returns the original data with attribute changes merged, if the changeset is valid.\n\nOptions:\n\n* force? - applies current attributes even if the changeset is not valid","ref":"Ash.Changeset.html#apply_attributes/2"},{"type":"function","title":"Ash.Changeset.around_action/2","doc":"Adds an around_action hook to the changeset.\n\nAround action hooks wrap the data layer action execution within the transaction.\nThis is a specialized hook primarily useful for:\n\n- Debugging and development tooling that needs to wrap action execution\n- Action timing and performance monitoring for profiling\n- Testing frameworks that need to intercept action execution\n- Advanced error handling that requires wrapping the action itself\n\nYour function receives the changeset and a callback that must be called with a changeset.\nThe callback returns `{:ok, result, changeset, instructions}` or `{:error, error}`.\nYou can modify these values, but must return the same structure.\n\n**Warning**: This is an advanced hook that runs within the database transaction.\nYou *must* call the callback function. For most use cases, `before_action/3` and\n`after_action/3` are simpler and more appropriate.","ref":"Ash.Changeset.html#around_action/2"},{"type":"function","title":"Examples - Ash.Changeset.around_action/2","doc":"# Monitor action execution time for debugging\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{total: 100.00})\n    iex> changeset = Ash.Changeset.around_action(changeset, fn changeset, callback ->\n    ...>   start_time = System.monotonic_time(:microsecond)\n    ...>   result = callback.(changeset)\n    ...>   duration = System.monotonic_time(:microsecond) - start_time\n    ...>\n    ...>   Logger.debug(\"Action #{changeset.action.name} took #{duration}μs\")\n    ...>   result\n    ...> end)","ref":"Ash.Changeset.html#around_action/2-examples"},{"type":"function","title":"See also - Ash.Changeset.around_action/2","doc":"- `before_action/3` and `after_action/3` for most workflow needs\n- `around_transaction/2` for wrapping the entire transaction\n- Multi-step actions guide for complex workflow patterns","ref":"Ash.Changeset.html#around_action/2-see-also"},{"type":"type","title":"Ash.Changeset.around_action_callback/0","doc":"Callback function type for around action hooks.\n\nA function that takes a changeset and returns an around action result.","ref":"Ash.Changeset.html#t:around_action_callback/0"},{"type":"type","title":"Ash.Changeset.around_action_fun/0","doc":"Function type for around action hooks.\n\nReceives a changeset and a callback function that must be called with the changeset.","ref":"Ash.Changeset.html#t:around_action_fun/0"},{"type":"type","title":"Ash.Changeset.around_action_result/0","doc":"Result type for around action callbacks.\n\nContains the successful result with record, changeset, and notifications, or an error.","ref":"Ash.Changeset.html#t:around_action_result/0"},{"type":"function","title":"Ash.Changeset.around_transaction/2","doc":"Adds an around_transaction hook to the changeset.\n\nAround transaction hooks wrap the entire database transaction execution.\nThis is a specialized hook primarily useful for:\n\n- Debugging and development tooling that needs to wrap transaction execution\n- Transaction timing and monitoring for performance analysis\n- Testing frameworks that need to intercept transaction behavior\n- Advanced transaction management that requires wrapping all hook execution\n\nYour function receives the changeset and a callback that must be called with a changeset.\nThe callback returns `{:ok, result}` or `{:error, error}`. You can modify these values,\nbut must return the same structure.\n\n**Warning**: This is an advanced hook that controls transaction execution.\nYou *must* call the callback function. For most use cases, `before_transaction/3`\nand `after_transaction/3` are simpler and more appropriate.","ref":"Ash.Changeset.html#around_transaction/2"},{"type":"function","title":"Examples - Ash.Changeset.around_transaction/2","doc":"# Monitor transaction execution time\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{total: 100.00})\n    iex> changeset = Ash.Changeset.around_transaction(changeset, fn changeset, callback ->\n    ...>   tx_start = System.monotonic_time(:microsecond)\n    ...>   result = callback.(changeset)\n    ...>   tx_duration = System.monotonic_time(:microsecond) - tx_start\n    ...>\n    ...>   Logger.debug(\"Transaction for #{changeset.action.name} took #{tx_duration}μs\")\n    ...>   result\n    ...> end)","ref":"Ash.Changeset.html#around_transaction/2-examples"},{"type":"function","title":"See also - Ash.Changeset.around_transaction/2","doc":"- `before_transaction/3` and `after_transaction/3` for most workflow needs\n- `around_action/2` for wrapping the data layer action\n- Multi-step actions guide for complex workflow patterns","ref":"Ash.Changeset.html#around_transaction/2-see-also"},{"type":"type","title":"Ash.Changeset.around_transaction_callback/0","doc":"Callback function type for around transaction hooks.\n\nA function that takes a changeset and returns an around transaction result.","ref":"Ash.Changeset.html#t:around_transaction_callback/0"},{"type":"type","title":"Ash.Changeset.around_transaction_fun/0","doc":"Function type for around transaction hooks.\n\nReceives a changeset and a callback function that must be called with the changeset.","ref":"Ash.Changeset.html#t:around_transaction_fun/0"},{"type":"type","title":"Ash.Changeset.around_transaction_result/0","doc":"Result type for around transaction callbacks.\n\nContains either a successful result with the record or an error.","ref":"Ash.Changeset.html#t:around_transaction_result/0"},{"type":"function","title":"Ash.Changeset.atomic_defaults/1","doc":"","ref":"Ash.Changeset.html#atomic_defaults/1"},{"type":"function","title":"Ash.Changeset.atomic_ref/2","doc":"Gets a reference to a field, or the current atomic update expression of that field.","ref":"Ash.Changeset.html#atomic_ref/2"},{"type":"function","title":"Ash.Changeset.atomic_update/2","doc":"Adds multiple atomic changes to the changeset\n\nSee `atomic_update/3` for more information.","ref":"Ash.Changeset.html#atomic_update/2"},{"type":"function","title":"Ash.Changeset.atomic_update/3","doc":"Adds an atomic change to the changeset.\n\nAtomic changes are applied by the data layer, and as such have guarantees that are not\ngiven by changes that are based on looking at the previous value and updating it. Here\nis an example of a change that is not safe to do concurrently:\n\n```elixir\nchange fn changeset, _ ->\n  Ash.Changeset.change_attribute(changeset, :score, changeset.data.score + 1)\nend\n```\n\nIf two processes run this concurrently, they will both read the same value of `score`, and\nset the new score to the same value. This means that one of the increments will be lost.\nIf you were to instead do this using `atomic_update`, you would get the correct result:\n\n```elixir\nAsh.Changeset.atomic_update(changeset, :score, expr(score + 1))\n```\n\nThere are drawbacks/things to consider, however. The first is that atomic update results\nare not known until after the action is run. The following functional validation would not\nbe able to enforce the score being less than 10, because the atomic happens after the validation.\n\n```elixir\nvalidate fn changeset, _ ->\n  if Ash.Changeset.get_attribute(changeset, :score) < 10 do\n    :ok\n  else\n    {:error, field: :score, message: \"must be less than 10\"}\n  end\nend\n```\n\nIf you want to use atomic updates, it is suggested to write module-based validations & changes,\nand implement the appropriate atomic callbacks on those modules. All builtin validations and changes\nimplement these callbacks in addition to the standard callbacks. Validations will only be run atomically\nwhen the entire action is being run atomically or if one of the relevant fields is being updated atomically.","ref":"Ash.Changeset.html#atomic_update/3"},{"type":"function","title":"Examples - Ash.Changeset.atomic_update/3","doc":"# Basic atomic increment\n    iex> changeset = Ash.Changeset.for_update(post, :update)\n    iex> changeset = Ash.Changeset.atomic_update(changeset, :view_count, expr(view_count + 1))\n    iex> changeset.atomics\n    [view_count: %Ash.Expr{...}]\n\n    # Multiple atomic updates\n    iex> changeset = Ash.Changeset.for_update(user, :update)\n    iex> changeset = changeset\n    ...> |> Ash.Changeset.atomic_update(:login_count, expr(login_count + 1))\n    ...> |> Ash.Changeset.atomic_update(:last_login, expr(now()))\n    iex> length(changeset.atomics)\n    2\n\n    # Atomic update with conditional logic\n    iex> changeset = Ash.Changeset.for_update(post, :update)\n    iex> changeset = Ash.Changeset.atomic_update(changeset, :status,\n    ...>   expr(if view_count > 1000, do: \"popular\", else: \"normal\"))\n\n    # Using the map/keyword syntax for multiple atomics\n    iex> changeset = Ash.Changeset.for_update(user, :update)\n    iex> changeset = Ash.Changeset.atomic_update(changeset, %{\n    ...>   login_count: expr(login_count + 1),\n    ...>   last_seen: expr(now())\n    ...> })","ref":"Ash.Changeset.html#atomic_update/3-examples"},{"type":"function","title":"See also - Ash.Changeset.atomic_update/3","doc":"- `atomic_ref/2` for referencing atomic values in expressions\n- `fully_atomic_changeset/4` for creating fully atomic changesets\n- `change_attribute/3` for regular (non-atomic) attribute changes","ref":"Ash.Changeset.html#atomic_update/3-see-also"},{"type":"function","title":"Ash.Changeset.attribute_present?/2","doc":"Checks if an attribute is not nil, either in the original data, or that it is not being changed to a `nil` value if it is changing.\n\nThis also accounts for the `accessing_from` context that is set when using `manage_relationship`, so it is aware that a particular value\n*will* be set by `manage_relationship` even if it isn't currently being set.","ref":"Ash.Changeset.html#attribute_present?/2"},{"type":"function","title":"Ash.Changeset.before_action/3","doc":"Adds a before_action hook to the changeset.\n\nBefore action hooks are essential for multi-step workflows where you need to:\n\n- Validate complex business logic that requires database queries\n- Set computed attributes based on other changes\n- Prepare data for external service calls\n- Implement conditional logic that depends on the current state\n\nThe hook runs after validations and changes but before the data layer action is executed.\nThis gives you access to the final validated changeset while still being able to modify it.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Changeset.html#before_action/3"},{"type":"function","title":"Examples - Ash.Changeset.before_action/3","doc":"# Set computed fields based on other attributes\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{items: [%{price: 10}, %{price: 15}]})\n    iex> changeset = Ash.Changeset.before_action(changeset, fn changeset ->\n    ...>   total = changeset.attributes.items |> Enum.map(& &1.price) |> Enum.sum()\n    ...>   tax = total * 0.08\n    ...>   changeset\n    ...>   |> Ash.Changeset.change_attribute(:subtotal, total)\n    ...>   |> Ash.Changeset.change_attribute(:tax, tax)\n    ...>   |> Ash.Changeset.change_attribute(:total, total + tax)\n    ...> end)\n\n    # Assign resources based on complex business logic\n    iex> changeset = Ash.Changeset.for_create(MyApp.Ticket, :create, %{title: \"Help needed\", priority: :urgent})\n    iex> changeset = Ash.Changeset.before_action(changeset, fn changeset ->\n    ...>   case changeset.attributes.priority do\n    ...>     :urgent ->\n    ...>       # Query for available senior agents\n    ...>       agent = MyApp.Agent |> MyApp.Query.for_read(:available_senior) |> MyApp.read_one!()\n    ...>       Ash.Changeset.change_attribute(changeset, :assigned_agent_id, agent.id)\n    ...>     _ ->\n    ...>       changeset\n    ...>   end\n    ...> end)\n\n    # Validate complex business rules that require database access\n    iex> changeset = Ash.Changeset.for_update(user, :upgrade_plan, %{plan: :premium})\n    iex> changeset = Ash.Changeset.before_action(changeset, fn changeset ->\n    ...>   existing_subscription = MyApp.Subscription\n    ...>   |> MyApp.Query.filter(user_id == ^user.id, status == :active)\n    ...>   |> MyApp.read_one()\n    ...>\n    ...>   case existing_subscription do\n    ...>     {:ok, %{plan: :premium}} ->\n    ...>       Ash.Changeset.add_error(changeset, field: :plan, message: \"already on premium plan\")\n    ...>     _ ->\n    ...>       changeset\n    ...>   end\n    ...> end)","ref":"Ash.Changeset.html#before_action/3-examples"},{"type":"function","title":"See also - Ash.Changeset.before_action/3","doc":"- `after_action/3` for hooks that run after the action succeeds\n- `around_action/2` for hooks that wrap the entire action\n- `before_transaction/3` for hooks that run before the database transaction\n- Multi-step actions guide for complex workflow patterns","ref":"Ash.Changeset.html#before_action/3-see-also"},{"type":"type","title":"Ash.Changeset.before_action_fun/0","doc":"Function type for before action hooks.\n\nReceives a changeset and returns a modified changeset, optionally with notifications.","ref":"Ash.Changeset.html#t:before_action_fun/0"},{"type":"function","title":"Ash.Changeset.before_transaction/3","doc":"Adds a before_transaction hook to the changeset.\n\nBefore transaction hooks run outside the database transaction and are ideal for:\n\n- Making external API calls that shouldn't be rolled back\n- Validating external resources or permissions\n- Setting up external state that the action depends on\n- Logging or auditing that should happen regardless of transaction success\n- Preparing data from external services\n\nThese hooks run before any database transaction is started, so they can't access\nthe final result but can prepare the changeset with external data.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Changeset.html#before_transaction/3"},{"type":"function","title":"Examples - Ash.Changeset.before_transaction/3","doc":"# Validate external service availability before processing\n    iex> changeset = Ash.Changeset.for_create(MyApp.Order, :create, %{items: [%{sku: \"ABC123\"}]})\n    iex> changeset = Ash.Changeset.before_transaction(changeset, fn changeset ->\n    ...>   case ExternalInventoryService.check_availability(changeset.attributes.items) do\n    ...>     {:ok, _} ->\n    ...>       changeset\n    ...>     {:error, :out_of_stock} ->\n    ...>       Ash.Changeset.add_error(changeset, field: :items, message: \"items out of stock\")\n    ...>   end\n    ...> end)\n\n    # Fetch and set data from external service\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create, %{email: \"user@example.com\"})\n    iex> changeset = Ash.Changeset.before_transaction(changeset, fn changeset ->\n    ...>   case UserService.get_profile_data(changeset.attributes.email) do\n    ...>     {:ok, profile} ->\n    ...>       changeset\n    ...>       |> Ash.Changeset.change_attribute(:display_name, profile.name)\n    ...>       |> Ash.Changeset.change_attribute(:avatar_url, profile.avatar)\n    ...>     {:error, _} ->\n    ...>       changeset  # Continue without external data\n    ...>   end\n    ...> end)","ref":"Ash.Changeset.html#before_transaction/3-examples"},{"type":"function","title":"See also - Ash.Changeset.before_transaction/3","doc":"- `before_action/3` for hooks that run within the transaction\n- `after_transaction/3` for hooks that run after the transaction completes\n- `around_transaction/2` for hooks that wrap the entire transaction","ref":"Ash.Changeset.html#before_transaction/3-see-also"},{"type":"type","title":"Ash.Changeset.before_transaction_fun/0","doc":"Function type for before transaction hooks.\n\nReceives a changeset and returns a modified changeset.","ref":"Ash.Changeset.html#t:before_transaction_fun/0"},{"type":"function","title":"Ash.Changeset.change_attribute/3","doc":"Adds a change to the changeset, unless the value matches the existing value.","ref":"Ash.Changeset.html#change_attribute/3"},{"type":"function","title":"Examples - Ash.Changeset.change_attribute/3","doc":"# Basic attribute change\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.change_attribute(changeset, :title, \"New Title\")\n    iex> changeset.attributes\n    %{title: \"New Title\"}\n\n    # Change multiple attributes in sequence\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create)\n    iex> changeset = changeset\n    ...> |> Ash.Changeset.change_attribute(:name, \"John Doe\")\n    ...> |> Ash.Changeset.change_attribute(:email, \"john@example.com\")\n    iex> changeset.attributes\n    %{name: \"John Doe\", email: \"john@example.com\"}\n\n    # Value is cast according to the attribute type\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.change_attribute(changeset, :view_count, \"42\")\n    iex> changeset.attributes\n    %{view_count: 42}","ref":"Ash.Changeset.html#change_attribute/3-examples"},{"type":"function","title":"See also - Ash.Changeset.change_attribute/3","doc":"- `change_attributes/2` for changing multiple attributes at once\n- `force_change_attribute/3` for writing attributes not accepted by the action, or changing attributes in hooks\n- `change_new_attribute/3` for changing an attribute unless its already being changed\n- `update_change/3` for updating existing changes","ref":"Ash.Changeset.html#change_attribute/3-see-also"},{"type":"function","title":"Ash.Changeset.change_attributes/2","doc":"Calls `change_attribute/3` for each key/value pair provided.","ref":"Ash.Changeset.html#change_attributes/2"},{"type":"function","title":"Examples - Ash.Changeset.change_attributes/2","doc":"# Change multiple attributes with a map\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.change_attributes(changeset, %{title: \"Hello World\", content: \"Post content\"})\n    iex> changeset.attributes\n    %{title: \"Hello World\", content: \"Post content\"}\n\n    # Change multiple attributes with a keyword list\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create)\n    iex> changeset = Ash.Changeset.change_attributes(changeset, [name: \"John Doe\", email: \"john@example.com\", age: 30])\n    iex> changeset.attributes\n    %{name: \"John Doe\", email: \"john@example.com\", age: 30}\n\n    # Values are cast according to their attribute types\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.change_attributes(changeset, %{view_count: \"42\", published: \"true\"})\n    iex> changeset.attributes\n    %{view_count: 42, published: true}","ref":"Ash.Changeset.html#change_attributes/2-examples"},{"type":"function","title":"See also - Ash.Changeset.change_attributes/2","doc":"- `change_attribute/3` for changing individual attributes\n- `force_change_attribute/3` for writing attributes not accepted by the action, or changing attributes in hooks\n- `set_arguments/2` for setting action arguments","ref":"Ash.Changeset.html#change_attributes/2-see-also"},{"type":"function","title":"Ash.Changeset.change_default_attribute/3","doc":"The same as `change_attribute`, but annotates that the attribute is currently holding a default value.\n\nThis information can be used in changes to see if a value was explicitly set or if it was set by being the default.\nAdditionally, this is used in `upsert` actions to not overwrite existing values with the default.","ref":"Ash.Changeset.html#change_default_attribute/3"},{"type":"function","title":"Ash.Changeset.change_new_attribute/3","doc":"Change an attribute only if is not currently being changed","ref":"Ash.Changeset.html#change_new_attribute/3"},{"type":"function","title":"Ash.Changeset.change_new_attribute_lazy/3","doc":"Change an attribute if is not currently being changed, by calling the provided function.\n\nUse this if you want to only perform some expensive calculation for an attribute value\nonly if there isn't already a change for that attribute.","ref":"Ash.Changeset.html#change_new_attribute_lazy/3"},{"type":"function","title":"Ash.Changeset.changing_attribute?/2","doc":"Returns true if an attribute exists in the changes.","ref":"Ash.Changeset.html#changing_attribute?/2"},{"type":"function","title":"Examples - Ash.Changeset.changing_attribute?/2","doc":"# Check if an attribute is being changed\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.change_attribute(changeset, :title, \"New Title\")\n    iex> Ash.Changeset.changing_attribute?(changeset, :title)\n    true\n    iex> Ash.Changeset.changing_attribute?(changeset, :content)\n    false\n\n    # Works with atomic updates too\n    iex> changeset = Ash.Changeset.for_update(post, :update)\n    iex> changeset = Ash.Changeset.atomic_update(changeset, :view_count, {:atomic, expr(view_count + 1)})\n    iex> Ash.Changeset.changing_attribute?(changeset, :view_count)\n    true\n\n    # Check multiple attributes\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create)\n    iex> changeset = Ash.Changeset.change_attributes(changeset, %{name: \"John\", email: \"john@example.com\"})\n    iex> Ash.Changeset.changing_attribute?(changeset, :name)\n    true\n    iex> Ash.Changeset.changing_attribute?(changeset, :age)\n    false","ref":"Ash.Changeset.html#changing_attribute?/2-examples"},{"type":"function","title":"See also - Ash.Changeset.changing_attribute?/2","doc":"- `changing_attributes?/1` for checking if any attributes are changing\n- `changing_relationship?/2` for checking relationship changes\n- `present?/2` for checking if a value is present\n- `fetch_change/2` for getting the changed value","ref":"Ash.Changeset.html#changing_attribute?/2-see-also"},{"type":"function","title":"Ash.Changeset.changing_attributes?/1","doc":"Returns true if any attributes on the resource are being changed.","ref":"Ash.Changeset.html#changing_attributes?/1"},{"type":"function","title":"Ash.Changeset.changing_relationship?/2","doc":"Returns true if a relationship exists in the changes","ref":"Ash.Changeset.html#changing_relationship?/2"},{"type":"function","title":"Ash.Changeset.clear_change/2","doc":"Clears an attribute or relationship change off of the changeset.","ref":"Ash.Changeset.html#clear_change/2"},{"type":"function","title":"Ash.Changeset.delete_argument/2","doc":"Remove an argument from the changeset","ref":"Ash.Changeset.html#delete_argument/2"},{"type":"function","title":"Ash.Changeset.deselect/2","doc":"Ensure the the specified attributes are `nil` in the changeset results.","ref":"Ash.Changeset.html#deselect/2"},{"type":"function","title":"Ash.Changeset.ensure_selected/2","doc":"Ensures that the given attributes are selected.\n\nThe first call to `select/2` will *limit* the fields to only the provided fields.\nUse `ensure_selected/2` to say \"select this field (or these fields) without deselecting anything else\".\n\nSee `select/2` for more.","ref":"Ash.Changeset.html#ensure_selected/2"},{"type":"type","title":"Ash.Changeset.error_info/0","doc":"","ref":"Ash.Changeset.html#t:error_info/0"},{"type":"function","title":"Ash.Changeset.expand_upsert_fields/2","doc":"Turns the special case {:replace, fields}, :replace_all and {:replace_all_except, fields} upsert_fields\noptions into a list of fields","ref":"Ash.Changeset.html#expand_upsert_fields/2"},{"type":"function","title":"Ash.Changeset.fetch_argument/2","doc":"Fetches the value of an argument provided to the changeset or `:error`.","ref":"Ash.Changeset.html#fetch_argument/2"},{"type":"function","title":"Ash.Changeset.fetch_argument_or_change/2","doc":"Gets the value of an argument provided to the changeset, falling back to `Ash.Changeset.fetch_change/2` if nothing was provided.","ref":"Ash.Changeset.html#fetch_argument_or_change/2"},{"type":"function","title":"Ash.Changeset.fetch_change/2","doc":"Gets the new value for an attribute, or `:error` if it is not being changed.","ref":"Ash.Changeset.html#fetch_change/2"},{"type":"function","title":"Ash.Changeset.filter/2","doc":"Adds a filter for a record being updated or destroyed.\n\nUsed by optimistic locking. See `Ash.Resource.Change.Builtins.optimistic_lock/1` for more.","ref":"Ash.Changeset.html#filter/2"},{"type":"function","title":"Ash.Changeset.for_action/4","doc":"Constructs a changeset for a given action, and validates it.\n\nCalls `for_create/4`, `for_update/4` or `for_destroy/4` based on the type of action passed in.\n\nSee those functions for more explanation.","ref":"Ash.Changeset.html#for_action/4"},{"type":"function","title":"Ash.Changeset.for_create/4","doc":"Constructs a changeset for a given create action, and validates it.\n\nAnything that is modified prior to `for_create/4` is validated against the rules of the action, while *anything after it is not*.\nThis runs any `change`s contained on your action. To have your logic execute *only* during the action, you can use `after_action/2`\nor `before_action/2`.\n\nMultitenancy is *not* validated until an action is called. This allows you to avoid specifying a tenant until just before calling\nthe domain action.","ref":"Ash.Changeset.html#for_create/4"},{"type":"function","title":"Params - Ash.Changeset.for_create/4","doc":"`params` may be attributes, relationships, or arguments. You can safely pass user/form input directly into this function.\nOnly public attributes and relationships are supported. If you want to change private attributes as well, see the\nCustomization section below. `params` are stored directly as given in the `params` field of the changeset, which can be\nused to retrieve the originally input value.","ref":"Ash.Changeset.html#for_create/4-params"},{"type":"function","title":"Options - Ash.Changeset.for_create/4","doc":"* `:require?` (`t:boolean/0`) - If set to `false`, values are only required when the action is run (instead of immediately). The default value is `false`.\n\n* `:actor` (`t:term/0`) - set the actor, which can be used in any `Ash.Resource.Change`s configured on the action. (in the `context` argument)\n\n* `:authorize?` (`t:term/0`) - set authorize?, which can be used in any `Ash.Resource.Change`s configured on the action. (in the `context` argument)\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer to use. Will be carried over to the action. For more information see `Ash.Tracer`.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - set the tenant on the changeset\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - Data to load on the result after running the action.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set before validations and changes. The default value is `%{}`.\n\n* `:return_skipped_upsert?` (`t:boolean/0`) - If `true`, and a record was *not* upserted because its filter prevented the upsert, the original record (which was *not* upserted) will be returned. The default value is `false`.","ref":"Ash.Changeset.html#for_create/4-options"},{"type":"function","title":"Customization - Ash.Changeset.for_create/4","doc":"A changeset can be provided as the first argument, instead of a resource, to allow\nsetting specific attributes ahead of time.\n\nFor example:\n\n    MyResource\n    |> Ash.Changeset.new()\n    |> Ash.Changeset.change_attribute(:foo, 1)\n    |> Ash.Changeset.for_create(:create, ...opts)\n\nOnce a changeset has been validated by `for_create/4` (or `for_update/4`), it isn't validated again in the action.\nNew changes added are validated individually, though. This allows you to create a changeset according\nto a given action, and then add custom changes if necessary.","ref":"Ash.Changeset.html#for_create/4-customization"},{"type":"function","title":"What does this function do? - Ash.Changeset.for_create/4","doc":"The following steps are run when calling `Ash.Changeset.for_create/4`.\n\n- Cast input params | This is any arguments in addition to any accepted attributes\n- Set argument defaults\n- Require any missing arguments\n- Validate all provided attributes are accepted\n- Require any accepted attributes that are `allow_nil?` false\n- Set any default values for attributes\n- Run action changes & validations\n- Run validations, or add them in `before_action` hooks if using `d:Ash.Resource.Dsl.actions.create.validate|before_action?`. Any global validations are skipped if the action has `skip_global_validations?` set to `true`.","ref":"Ash.Changeset.html#for_create/4-what-does-this-function-do"},{"type":"function","title":"Examples - Ash.Changeset.for_create/4","doc":"# Basic create changeset with attributes\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create, %{title: \"Hello World\", content: \"This is my first post\"})\n    iex> changeset.valid?\n    true\n\n    # Create changeset with custom actor and tenant\n    iex> changeset = Ash.Changeset.for_create(MyApp.Comment, :create,\n    ...>   %{body: \"Great post!\"},\n    ...>   actor: current_user,\n    ...>   tenant: \"org_123\"\n    ...> )\n    iex> changeset.tenant\n    \"org_123\"","ref":"Ash.Changeset.html#for_create/4-examples"},{"type":"function","title":"See also - Ash.Changeset.for_create/4","doc":"- `for_action/4` for a generic action constructor\n- `for_update/4` for updating existing records\n- `for_destroy/4` for destroying records\n- `Ash.create/2` to execute the changeset\n- `d:Ash.Resource.Dsl.actions.create` for defining create actions\n- [Create Actions Guide](/documentation/topics/actions/create-actions.md) for understanding create operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.Changeset.html#for_create/4-see-also"},{"type":"function","title":"Ash.Changeset.for_destroy/4","doc":"Constructs a changeset for a given destroy action, and validates it.","ref":"Ash.Changeset.html#for_destroy/4"},{"type":"function","title":"Opts - Ash.Changeset.for_destroy/4","doc":"* `:actor` - set the actor, which can be used in any `Ash.Resource.Change`s configured on the action. (in the `context` argument)\n* `:tenant` - set the tenant on the changeset\n* `:private_arguments` - set private arguments on the changeset before validations and changes are run\n\nAnything that is modified prior to `for_destroy/4` is validated against the rules of the action, while *anything after it is not*.\n\nOnce a changeset has been validated by `for_destroy/4`, it isn't validated again in the action.\nNew changes added are validated individually, though. This allows you to create a changeset according\nto a given action, and then add custom changes if necessary.","ref":"Ash.Changeset.html#for_destroy/4-opts"},{"type":"function","title":"What does this function do? - Ash.Changeset.for_destroy/4","doc":"The following steps are run when calling `Ash.Changeset.for_destroy/4`.\n\n- Cast input params | This is any arguments in addition to any accepted attributes\n- Set argument defaults\n- Require any missing arguments\n- Validate all provided attributes are accepted\n- Require any accepted attributes that are `allow_nil?` false\n- Set any default values for attributes\n- Run action changes & validations\n- Run validations, or add them in `before_action` hooks if using `d:Ash.Resource.Dsl.actions.destroy.validate|before_action?`. Any global validations are skipped if the action has `skip_global_validations?` set to `true`.","ref":"Ash.Changeset.html#for_destroy/4-what-does-this-function-do"},{"type":"function","title":"Examples - Ash.Changeset.for_destroy/4","doc":"# Basic destroy changeset\n    iex> post = %MyApp.Post{id: 1, title: \"My Post\", content: \"Content\"}\n    iex> changeset = Ash.Changeset.for_destroy(post, :destroy)\n    iex> changeset.valid?\n    true\n\n    # Destroy changeset with inputs for the action\n    iex> user = %MyApp.User{id: 1, name: \"John\", email: \"john@example.com\"}\n    iex> changeset = Ash.Changeset.for_destroy(user, :archive, %{reason: \"User requested deletion\"}, actor: current_user)\n\n    # Soft delete changeset (if action is configured as soft)\n    iex> comment = %MyApp.Comment{id: 1, body: \"My comment\", post_id: 1}\n    iex> changeset = Ash.Changeset.for_destroy(comment, :soft_delete, %{}, actor: current_user, tenant: \"org_123\")\n    iex> changeset.tenant\n    \"org_123\"","ref":"Ash.Changeset.html#for_destroy/4-examples"},{"type":"function","title":"See also - Ash.Changeset.for_destroy/4","doc":"- `for_action/4` for a generic action constructor\n- `for_create/4` for creating new records\n- `for_update/4` for updating records\n- `Ash.destroy/2` to execute the changeset\n- `d:Ash.Resource.Dsl.actions.destroy` for defining destroy actions\n- [Destroy Actions Guide](/documentation/topics/actions/destroy-actions.md) for understanding destroy operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.Changeset.html#for_destroy/4-see-also"},{"type":"function","title":"Ash.Changeset.for_update/4","doc":"Constructs a changeset for a given update action, and validates it.\n\nAnything that is modified prior to `for_update/4` is validated against the rules of the action, while *anything after it is not*.","ref":"Ash.Changeset.html#for_update/4"},{"type":"function","title":"What does this function do? - Ash.Changeset.for_update/4","doc":"The following steps are run when calling `Ash.Changeset.for_update/4`.\n\n- Cast input params | This is any arguments in addition to any accepted attributes\n- Set argument defaults\n- Require any missing arguments\n- Validate all provided attributes are accepted\n- Require any accepted attributes that are `allow_nil?` false\n- Set any default values for attributes\n- Run action changes & validations\n- Run validations, or add them in `before_action` hooks if using `d:Ash.Resource.Dsl.actions.update.validate|before_action?`. Any global validations are skipped if the action has `skip_global_validations?` set to `true`.","ref":"Ash.Changeset.html#for_update/4-what-does-this-function-do"},{"type":"function","title":"Examples - Ash.Changeset.for_update/4","doc":"# Basic update changeset with attributes\n    iex> post = %MyApp.Post{id: 1, title: \"Original Title\", content: \"Original content\"}\n    iex> changeset = Ash.Changeset.for_update(post, :update, %{title: \"Updated Title\"})\n    iex> changeset.valid?\n    true\n\n    # Update changeset with custom actor and load related data\n    iex> comment = %MyApp.Comment{id: 1, body: \"Original comment\", post_id: 1}\n    iex> changeset = Ash.Changeset.for_update(comment, :update,\n    ...>   %{body: \"Updated comment\"},\n    ...>   actor: current_user,\n    ...>   load: [:post, :author]\n    ...> )\n    iex> changeset.load\n    [:post, :author]","ref":"Ash.Changeset.html#for_update/4-examples"},{"type":"function","title":"See also - Ash.Changeset.for_update/4","doc":"- `for_action/4` for a generic action constructor\n- `for_create/4` for creating new records\n- `for_destroy/4` for destroying records\n- `Ash.update/2` to execute the changeset\n- `d:Ash.Resource.Dsl.actions.update` for defining update actions\n- [Update Actions Guide](/documentation/topics/actions/update-actions.md) for understanding update operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.Changeset.html#for_update/4-see-also"},{"type":"function","title":"Ash.Changeset.force_change_attribute/3","doc":"Changes an attribute even if it isn't writable","ref":"Ash.Changeset.html#force_change_attribute/3"},{"type":"function","title":"Ash.Changeset.force_change_attributes/2","doc":"Calls `force_change_attribute/3` for each key/value pair provided.","ref":"Ash.Changeset.html#force_change_attributes/2"},{"type":"function","title":"Ash.Changeset.force_change_new_attribute/3","doc":"Force change an attribute if it is not currently being changed.\n\nSee `change_new_attribute/3` for more.","ref":"Ash.Changeset.html#force_change_new_attribute/3"},{"type":"function","title":"Ash.Changeset.force_change_new_attribute_lazy/3","doc":"Force change an attribute if it is not currently being changed, by calling the provided function.\n\nSee `change_new_attribute_lazy/3` for more.","ref":"Ash.Changeset.html#force_change_new_attribute_lazy/3"},{"type":"function","title":"Ash.Changeset.force_delete_argument/2","doc":"Remove an argument from the changeset, not warning if the changeset has already been validated.","ref":"Ash.Changeset.html#force_delete_argument/2"},{"type":"function","title":"Ash.Changeset.force_set_argument/3","doc":"Add an argument to the changeset, which will be provided to the action.\n\nDoes not show a warning when used in before/after action hooks.","ref":"Ash.Changeset.html#force_set_argument/3"},{"type":"function","title":"Ash.Changeset.force_set_arguments/2","doc":"Merge a map of arguments to the arguments list.\n\nDoes not show a warning when used in before/after action hooks.","ref":"Ash.Changeset.html#force_set_arguments/2"},{"type":"function","title":"Ash.Changeset.fully_atomic_changeset/4","doc":"","ref":"Ash.Changeset.html#fully_atomic_changeset/4"},{"type":"function","title":"Ash.Changeset.get_argument/2","doc":"Gets the value of an argument provided to the changeset.","ref":"Ash.Changeset.html#get_argument/2"},{"type":"function","title":"Examples - Ash.Changeset.get_argument/2","doc":"# Get an argument that was set\n    iex> changeset = MyApp.Post\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:auto_publish, true)\n    iex> Ash.Changeset.get_argument(changeset, :auto_publish)\n    true\n\n    # Get an argument that doesn't exist (returns nil)\n    iex> changeset = Ash.Changeset.new(MyApp.Post)\n    iex> Ash.Changeset.get_argument(changeset, :nonexistent)\n    nil\n\n    # Get argument with string key\n    iex> changeset = MyApp.User\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:send_email, false)\n    iex> Ash.Changeset.get_argument(changeset, \"send_email\")\n    false","ref":"Ash.Changeset.html#get_argument/2-examples"},{"type":"function","title":"See also - Ash.Changeset.get_argument/2","doc":"- `set_argument/3` for setting argument values\n- `fetch_argument/2` for getting arguments with error handling\n- `get_argument_or_attribute/2` for fallback to attributes","ref":"Ash.Changeset.html#get_argument/2-see-also"},{"type":"function","title":"Ash.Changeset.get_argument_or_attribute/2","doc":"Gets the value of an argument provided to the changeset, falling back to `Ash.Changeset.get_attribute/2` if nothing was provided.","ref":"Ash.Changeset.html#get_argument_or_attribute/2"},{"type":"function","title":"Ash.Changeset.get_attribute/2","doc":"Gets the changing value or the original value of an attribute.","ref":"Ash.Changeset.html#get_attribute/2"},{"type":"function","title":"Ash.Changeset.get_data/2","doc":"Gets the original value for an attribute","ref":"Ash.Changeset.html#get_data/2"},{"type":"function","title":"Ash.Changeset.handle_errors/2","doc":"Sets a custom error handler on the changeset.\n\nThe error handler should be a two argument function or an mfa, in which case the first two arguments will be set\nto the changeset and the error, w/ the supplied arguments following those. The changeset will be marked\nas invalid regardless of the outcome of this callback.\n\nAny errors generated are passed to `handle_errors`, which can return any of the following:\n\n* `:ignore` - the error is discarded.\n* `changeset` - a new (or the same) changeset. The error is not added.\n* `{changeset, error}` - a new (or the same) error and changeset. The error is added to the changeset.\n* `anything_else` - is treated as a new, transformed version of the error. The result is added as an error to the changeset.","ref":"Ash.Changeset.html#handle_errors/2"},{"type":"macro","title":"Ash.Changeset.is_valid/1","doc":"A guard which checks if the Changeset is valid.","ref":"Ash.Changeset.html#is_valid/1"},{"type":"function","title":"Ash.Changeset.load/2","doc":"Calls the provided load statement on the result of the action at the very end of the action.","ref":"Ash.Changeset.html#load/2"},{"type":"function","title":"Examples - Ash.Changeset.load/2","doc":"# Load a single relationship\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create, %{title: \"My Post\"})\n    iex> changeset = Ash.Changeset.load(changeset, :comments)\n    iex> changeset.load\n    [:comments]\n\n    # Load multiple relationships\n    iex> changeset = Ash.Changeset.for_update(user, :update, %{name: \"John Doe\"})\n    iex> changeset = Ash.Changeset.load(changeset, [:posts, :profile])\n    iex> changeset.load\n    [:posts, :profile]\n\n    # Load nested relationships\n    iex> changeset = Ash.Changeset.for_create(MyApp.Comment, :create, %{body: \"Great post!\"})\n    iex> changeset = Ash.Changeset.load(changeset, [post: [:author, :comments]])\n    iex> changeset.load\n    [[post: [:author, :comments]]]\n\n    # Chain multiple load calls (they accumulate)\n    iex> changeset = Ash.Changeset.for_update(post, :update, %{title: \"Updated Title\"})\n    iex> changeset = changeset\n    ...> |> Ash.Changeset.load(:author)\n    ...> |> Ash.Changeset.load(:comments)\n    iex> changeset.load\n    [:author, :comments]","ref":"Ash.Changeset.html#load/2-examples"},{"type":"function","title":"See also - Ash.Changeset.load/2","doc":"- `select/3` for controlling which attributes are returned\n- `loading?/2` for checking if something is being loaded\n- `Ash.Query.load/2` for loading in queries","ref":"Ash.Changeset.html#load/2-see-also"},{"type":"function","title":"Ash.Changeset.loading?/2","doc":"Returns true if the field/relationship or path to field/relationship is being loaded.\n\nIt accepts an atom or a list of atoms, which is treated for as a \"path\", i.e:\n\n    Resource |> Ash.Changeset.load(friends: [enemies: [:score]]) |> Ash.Changeset.loading?([:friends, :enemies, :score])\n    iex> true\n\n    Resource |> Ash.Changeset.load(friends: [enemies: [:score]]) |> Ash.Changeset.loading?([:friends, :score])\n    iex> false\n\n    Resource |> Ash.Changeset.load(friends: [enemies: [:score]]) |> Ash.Changeset.loading?(:friends)\n    iex> true","ref":"Ash.Changeset.html#loading?/2"},{"type":"function","title":"Ash.Changeset.manage_relationship/4","doc":"Manages the related records by creating, updating, or destroying them as necessary.\n\nKeep in mind that the default values for all `on_*` are `:ignore`, meaning nothing will happen\nunless you provide instructions.\n\nThe input provided to `manage_relationship` should be a map, in the case of to_one relationships, or a list of maps\nin the case of to_many relationships. The following steps are followed for each input provided:\n\n- The input is checked against the currently related records to find any matches. The primary key and unique identities are used to find matches.\n- For any input that had a match in the current relationship, the `:on_match` behavior is triggered\n- For any input that does not have a match:\n  - if there is `on_lookup` behavior:\n    - we try to find the record in the data layer.\n    - if the record is found, the on_lookup behavior is triggered\n    - if the record is not found, the `on_no_match` behavior is triggered\n  - if there is no `on_lookup` behavior:\n    - the `on_no_match` behavior is triggered\n- finally, for any records present in the *current relationship* that had no match *in the input*, the `on_missing` behavior is triggered","ref":"Ash.Changeset.html#manage_relationship/4"},{"type":"function","title":"Options - Ash.Changeset.manage_relationship/4","doc":"* `:type` - If the `type` is specified, the default values of each option is modified to match that `type` of operation.  \n  This allows for specifying certain operations much more succinctly. The defaults that are modified are listed below:  \n  - `:append_and_remove`  \n        [\n          on_lookup: :relate,\n          on_no_match: :error,\n          on_match: :ignore,\n          on_missing: :unrelate\n        ]  \n  - `:append`  \n        [\n          on_lookup: :relate,\n          on_no_match: :error,\n          on_match: :ignore,\n          on_missing: :ignore\n        ]  \n  - `:remove`  \n        [\n          on_no_match: :error,\n          on_match: :unrelate,\n          on_missing: :ignore\n        ]  \n  - `:direct_control`  \n        [\n          on_lookup: :ignore,\n          on_no_match: :create,\n          on_match: :update,\n          on_missing: :destroy\n        ]  \n  - `:create`  \n        [\n          on_no_match: :create,\n          on_match: :ignore\n        ]\n   Valid values are :append_and_remove, :append, :remove, :direct_control, :create\n\n* `:authorize?` (`t:boolean/0`) - Authorize reads and changes to the destination records, if the primary change is being authorized as well. The default value is `true`.\n\n* `:eager_validate_with` (`t:atom/0`) - Validates that any referenced entities exist *before* the action is being performed, using the provided domain for the read. The default value is `false`.\n\n* `:on_no_match` (`t:term/0`) - Instructions for handling records where no matching record existed in the relationship.  \n  * `:ignore` (default) - those inputs are ignored\n  * `:match` - For `has_one` and `belongs_to` only, any input is treated as a match for an existing value. For `has_many` and `many_to_many`, this is the same as `:ignore`.\n  * `:create` - the records are created using the destination's primary create action\n  * `{:create, :action_name}` - the records are created using the specified action on the destination resource\n  * `{:create, :action_name, :join_table_action_name, [:list, :of, :join_table, :params]}` - Same as `{:create, :action_name}` but takes\n      the list of params specified out and applies them when creating the join record, with the provided join_table_action_name.\n  * `:error`  - an error is returned indicating that a record would have been created\n    *  If `on_lookup` is set, and the data contained a primary key or identity, then the error is a `NotFound` error\n    * Otherwise, an `InvalidRelationship` error is returned The default value is `:ignore`.\n\n* `:value_is_key` (`t:atom/0`) - Configures what key to use when a single value is provided.  \n  This is useful when you use things like a list of strings i.e `friend_emails` to manage the relationship, instead of a list of maps.  \n  By default, we assume it is the primary key of the destination resource, unless it is a composite primary key.\n\n* `:order_is_key` (`t:atom/0`) - If set, the order that each input appears in the list will be added to the input as this key.  \n  This is useful when you want to accept an ordered list of related records and write that order to the entity.\n  This should only currently be used with `type: :direct_control` or `type: :create` when there are no currently\n  existing related records (like when creating the source record).  \n  If you have an identity on the field and relationship id on the destination, and you are using\n  AshPostgres, you will want to use the `deferrable` option to ensure that conflicting orders are temporarily\n  allowed within a single transaction.\n\n* `:identity_priority` (list of `t:atom/0`) - The list, in priority order, of identities to use when looking up records for `on_lookup`, and matching records with `on_match`.  \n  Use `:_primary_key` to prioritize checking a match with the primary key.\n  All identities, along with `:_primary_key` are checked regardless, this only allows ensuring that some are checked first.\n  Defaults to the list provided by `use_identities`, so you typically won't need this option.\n\n* `:use_identities` (list of `t:atom/0`) - A list of identities that may be used to look up and compare records. Use `:_primary_key` to include the primary key. By default, only `[:_primary_key]` is used.\n\n* `:on_lookup` (`t:term/0`) - Before creating a record (because no match was found in the relationship), the record can be looked up and related.  \n  * `:ignore` (default) - Does not look for existing entries (matches in the current relationship are still considered updates)\n  * `:relate` - Same as calling `{:relate, primary_action_name}`\n  * `{:relate, :action_name}` - the records are looked up by primary key/the first identity that is found (using the primary read action), and related. The action should be:\n      * `many_to_many` - a create action on the join resource\n      * `has_many` - an update action on the destination resource\n      * `has_one` - an update action on the destination resource\n      * `belongs_to` - an update action on the source resource\n  * `{:relate, :action_name, :read_action_name}` - Same as the above, but customizes the read action called to search for matches.\n  * `:relate_and_update` - Same as `:relate`, but the remaining parameters from the lookup are passed into the action that is used to change the relationship key\n  * `{:relate_and_update, :action_name}` - Same as the above, but customizes the action used. The action should be:\n      * `many_to_many` - a create action on the join resource\n      * `has_many` - an update action on the destination resource\n      * `has_one` - an update action on the destination resource\n      * `belongs_to` - an update action on the source resource\n  * `{:relate_and_update, :action_name, :read_action_name}` - Same as the above, but customizes the read action called to search for matches.\n  * `{:relate_and_update, :action_name, :read_action_name, [:list, :of, :join_table, :params]}` - Same as the above, but uses the provided list of parameters when creating\n      the join row (only relevant for many to many relationships). Use `:*` to *only* update the join record, and pass all parameters to its action The default value is `:ignore`.\n\n* `:on_match` (`t:term/0`) - Instructions for handling records where a matching record existed in the relationship already.  \n  * `:ignore` (default) - those inputs are ignored\n  * `:update` - the record is updated using the destination's primary update action\n  * `{:update, :action_name}` - the record is updated using the specified action on the destination resource\n  * `{:update, :action_name, :join_table_action_name, [:list, :of, :params]}` - Same as `{:update, :action_name}` but takes\n      the list of params specified out and applies them as an update to the join record (only valid for many to many)\n  * `:update_join` - update only the join record (only valid for many to many)\n  * `{:update_join, :join_table_action_name}` - use the specified update action on a join resource\n  * `{:update_join, :join_table_action_name, [:list, :of, :params]}` - pass specified params from input into a join resource update action\n  * `{:destroy, :action_name}` - the record is destroyed using the specified action on the destination resource. The action should be:\n    * `many_to_many` - a destroy action on the join record\n    * `has_many` - a destroy action on the destination resource\n    * `has_one` - a destroy action on the destination resource\n    * `belongs_to` - a destroy action on the destination resource\n  * `:error`  - an error is returned indicating that a record would have been updated\n  * `:no_match` - follows the `on_no_match` instructions with these records\n  * `:missing` - follows the `on_missing` instructions with these records\n  * `:unrelate` - the related item is not destroyed, but the data is \"unrelated\", making this behave like `remove_from_relationship/3`. The action should be:\n    * `many_to_many` - the join resource row is destroyed\n    * `has_many` - the `destination_attribute` (on the related record) is set to `nil`\n    * `has_one` - the `destination_attribute` (on the related record) is set to `nil`\n    * `belongs_to` - the `source_attribute` (on this record) is set to `nil`\n  * `{:unrelate, :action_name}` - the record is unrelated using the provided update action. The action should be:\n    * `many_to_many` - a destroy action on the join resource\n    * `has_many` - an update action on the destination resource\n    * `has_one` - an update action on the destination resource\n    * `belongs_to` - an update action on the source resource The default value is `:ignore`.\n\n* `:on_missing` (`t:term/0`) - Instructions for handling records that existed in the current relationship but not in the input.  \n  * `:ignore` (default) - those inputs are ignored\n  * `:destroy` - the record is destroyed using the destination's primary destroy action\n  * `{:destroy, :action_name}` - the record is destroyed using the specified action on the destination resource\n  * `{:destroy, :action_name, :join_resource_action_name}` - the record is destroyed using the specified action on the destination resource,\n    but first the join resource is destroyed with its specified action\n  * `:error`  - an error is returned indicating that a record would have been updated\n  * `:unrelate` - the related item is not destroyed, but the data is \"unrelated\", making this behave like `remove_from_relationship/3`. The action should be:\n    * `many_to_many` - the join resource row is destroyed\n    * `has_many` - the `destination_attribute` (on the related record) is set to `nil`\n    * `has_one` - the `destination_attribute` (on the related record) is set to `nil`\n    * `belongs_to` - the `source_attribute` (on this record) is set to `nil`\n  * `{:unrelate, :action_name}` - the record is unrelated using the provided update action. The action should be:\n    * `many_to_many` - a destroy action on the join resource\n    * `has_many` - an update action on the destination resource\n    * `has_one` - an update action on the destination resource\n    * `belongs_to` - an update action on the source resource The default value is `:ignore`.\n\n* `:error_path` (`t:term/0`) - By default, errors added to the changeset will use the path `[:relationship_name]`, or `[:relationship_name,  ]`.\n  If you want to modify this path, you can specify `error_path`, e.g if had a `change` on an action that takes an argument\n  and uses that argument data to call `manage_relationship`, you may want any generated errors to appear under the name of that\n  argument, so you could specify `error_path: :argument_name` when calling `manage_relationship`.\n\n* `:join_keys` (list of `t:atom/0`) - For many to many relationships specifies the parameters to pick from the input and pass into a join resource action.\n  Applicable in cases like `on_no_match: :create`, `on_match: :update` and `on_lookup: :relate`.\n  Can be overwritten by a full form instruction tuple which contains join parameters at the end.\n\n* `:meta` (`t:term/0`) - Freeform data that will be retained along with the options, which can be used to track/manage the changes\n  that are added to the `relationships` key. Use the `meta[:order]` option to specify the order in which multiple\n  calls to `manage_relationship` should be executed.\n\n* `:ignore?` (`t:term/0`) - This tells Ash to ignore the provided inputs when actually running the action. This can be useful for\n  building up a set of instructions that you intend to handle manually. The default value is `false`.\n\n* `:debug?` (`t:boolean/0`) - Logs queries executed by relationship. The default value is `false`.\n\n\n\nEach call to this function adds new records that will be handled according to their options. For example,\nif you tracked \"tags to add\" and \"tags to remove\" in separate fields, you could input them like so:\n\n```elixir\nchangeset\n|> Ash.Changeset.manage_relationship(\n  :tags,\n  [%{name: \"backend\"}],\n  on_lookup: :relate, #relate that tag if it exists in the database\n  on_no_match: :error # error if a tag with that name doesn't exist\n)\n|> Ash.Changeset.manage_relationship(\n  :tags,\n  [%{name: \"frontend\"}],\n  on_no_match: :error, # error if a tag with that name doesn't exist in the relationship\n  on_match: :unrelate # if a tag with that name is related, unrelate it\n)\n```\n\nWhen calling this multiple times with the `on_missing` option set, the list of records that are considered missing are checked\nafter each set of inputs is processed. For example, if you manage the relationship once with `on_missing: :unrelate`, the records\nmissing from your input will be removed, and *then* your next call to `manage_relationship` will be resolved (with those records unrelated).\nFor this reason, it is suggested that you don't call this function multiple times with an `on_missing` instruction, as you may be\nsurprised by the result.\n\nIf you want the input to update existing entities, you need to ensure that the primary key (or unique identity) is provided as\npart of the input. See the example below:\n\n    changeset\n    |> Ash.Changeset.manage_relationship(\n      :comments,\n      [%{rating: 10, contents: \"foo\"}],\n      on_no_match: {:create, :create_action},\n      on_missing: :ignore\n    )\n    |> Ash.Changeset.manage_relationship(\n      :comments,\n      [%{id: 10, rating: 10, contents: \"foo\"}],\n      on_match: {:update, :update_action},\n      on_no_match: {:create, :create_action})\n\nThis is a simple way to manage a relationship. If you need custom behavior, you can customize the action that is\ncalled, which allows you to add arguments/changes. However, at some point you may want to forego this function\nand make the changes yourself. For example:\n\n    input = [%{id: 10, rating: 10, contents: \"foo\"}]\n\n    changeset\n    |> Ash.Changeset.after_action(fn _changeset, result ->\n      # An example of updating comments based on a result of other changes\n      for comment <- input do\n        comment = Ash.get(Comment, comment.id)\n\n        comment\n        |> Map.update(:rating, 0, &(&1 * result.rating_weight))\n        |> Ash.update!()\n      end\n\n      {:ok, result}\n    end)","ref":"Ash.Changeset.html#manage_relationship/4-options"},{"type":"function","title":"Using records as input - Ash.Changeset.manage_relationship/4","doc":"Records can be supplied as the input values. If you do:\n\n* if it would be looked up due to `on_lookup`, the record is used as-is\n* if it would be created due to `on_no_match`, the record is used as-is\n* Instead of specifying `join_keys`, those keys must go in `__metadata__.join_keys`. If `join_keys` is specified in the options, it is ignored.\n\nFor example:\n\n```elixir\npost1 =\n  changeset\n  |> Ash.create!()\n  |> Ash.Resource.put_metadata(:join_keys, %{type: \"a\"})\n\npost2 =\n  changeset2\n  |> Ash.create!()\n  |> Ash.Resource.put_metadata(:join_keys, %{type: \"b\"})\n\nauthor = Ash.create!(author_changeset)\n\nAsh.Changeset.manage_relationship(\n  author,\n  :posts,\n  [post1, post2],\n  on_lookup: :relate\n)\n```","ref":"Ash.Changeset.html#manage_relationship/4-using-records-as-input"},{"type":"type","title":"Ash.Changeset.manage_relationship_type/0","doc":"The type of relationship management strategy to use.\n\nDefines how related records should be handled when managing relationships.","ref":"Ash.Changeset.html#t:manage_relationship_type/0"},{"type":"function","title":"Ash.Changeset.new/1","doc":"Returns a new changeset over a resource.\n\n*Warning*: You almost always want to use `for_action` or `for_create`, etc. over this function if possible.\n\nYou can use this to start a changeset and make changes prior to calling `for_action`. This is not typically\nnecessary, but can be useful as an escape hatch.","ref":"Ash.Changeset.html#new/1"},{"type":"function","title":"Examples - Ash.Changeset.new/1","doc":"# Create a changeset for a new record\n    iex> changeset = Ash.Changeset.new(MyApp.Post)\n    iex> changeset.action_type\n    :create\n\n    # Create a changeset for updating an existing record\n    iex> post = %MyApp.Post{id: 1, title: \"Original Title\"}\n    iex> changeset = Ash.Changeset.new(post)\n    iex> changeset.action_type\n    :update\n\n    # Use as an escape hatch before calling for_action\n    iex> MyApp.Post\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.change_attribute(:title, \"Draft Title\")\n    ...> |> Ash.Changeset.for_action(:create, %{content: \"Post content\"})","ref":"Ash.Changeset.html#new/1-examples"},{"type":"function","title":"See also - Ash.Changeset.new/1","doc":"- `for_action/4` for the recommended way to create changesets\n- `for_create/4` for creating new records\n- `for_update/4` for updating existing records\n- `for_destroy/4` for destroying records","ref":"Ash.Changeset.html#new/1-see-also"},{"type":"type","title":"Ash.Changeset.phase/0","doc":"The current phase of changeset processing.\n\nRepresents where the changeset is in its lifecycle, from pending through various hook phases.","ref":"Ash.Changeset.html#t:phase/0"},{"type":"function","title":"Ash.Changeset.prepare_changeset_for_action/3","doc":"","ref":"Ash.Changeset.html#prepare_changeset_for_action/3"},{"type":"function","title":"Ash.Changeset.present?/2","doc":"Checks if an argument is not nil or an attribute is not nil, either in the original data, or that it is not being changed to a `nil` value if it is changing.\n\nThis also accounts for the `accessing_from` context that is set when using `manage_relationship`, so it is aware that a particular value\n*will* be set by `manage_relationship` even if it isn't currently being set.","ref":"Ash.Changeset.html#present?/2"},{"type":"function","title":"Examples - Ash.Changeset.present?/2","doc":"# Check if attribute is present in original data\n    iex> post = %MyApp.Post{id: 1, title: \"My Post\", content: nil}\n    iex> changeset = Ash.Changeset.for_update(post, :update)\n    iex> Ash.Changeset.present?(changeset, :title)\n    true\n    iex> Ash.Changeset.present?(changeset, :content)\n    false\n\n    # Check if attribute is being changed to a non-nil value\n    iex> post = %MyApp.Post{id: 1, title: nil, content: \"Content\"}\n    iex> changeset = Ash.Changeset.for_update(post, :update, %{title: \"New Title\"})\n    iex> Ash.Changeset.present?(changeset, :title)\n    true\n\n    # Check if argument is present\n    iex> changeset = MyApp.User\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:send_email, true)\n    iex> Ash.Changeset.present?(changeset, :send_email)\n    true\n    iex> Ash.Changeset.present?(changeset, :other_arg)\n    false","ref":"Ash.Changeset.html#present?/2-examples"},{"type":"function","title":"See also - Ash.Changeset.present?/2","doc":"- `attribute_present?/2` for checking only attributes (not arguments)\n- `changing_attribute?/2` for checking if an attribute is being changed\n- `get_argument/2` and `get_attribute/2` for retrieving values","ref":"Ash.Changeset.html#present?/2-see-also"},{"type":"function","title":"Ash.Changeset.put_context/3","doc":"Puts a key/value in the changeset context that can be used later.\n\nDo not use the `private` key in your custom context, as that is reserved for internal use.","ref":"Ash.Changeset.html#put_context/3"},{"type":"function","title":"Ash.Changeset.run_before_transaction_hooks/1","doc":"","ref":"Ash.Changeset.html#run_before_transaction_hooks/1"},{"type":"function","title":"Ash.Changeset.select/3","doc":"Ensure that only the specified attributes are present in the results.\n\nThe first call to `select/2` will replace the default behavior of selecting\nall attributes. Subsequent calls to `select/2` will combine the provided\nfields unless the `replace?` option is provided with a value of `true`.\n\nIf a field has been deselected, selecting it again will override that (because a single list of fields is tracked for selection)\n\nPrimary key attributes always selected and cannot be deselected.\n\nWhen attempting to load a relationship (or manage it with `Ash.Changeset.manage_relationship/3`),\nif the source field is not selected on the query/provided data an error will be produced. If loading\na relationship with a query, an error is produced if the query does not select the destination field\nof the relationship.\n\nDatalayers currently are not notified of the `select` for a changeset(unlike queries), and creates/updates select all fields when they are performed.\nA select provided on a changeset sets the unselected fields to `nil` before returning the result.\n\nUse `ensure_selected/2` if you wish to make sure a field has been selected, without deselecting any other fields.","ref":"Ash.Changeset.html#select/3"},{"type":"function","title":"Examples - Ash.Changeset.select/3","doc":"# Select specific fields\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = Ash.Changeset.select(changeset, [:title, :content])\n    iex> changeset.select\n    [:title, :content]\n\n    # Combine multiple select calls (default behavior)\n    iex> changeset = Ash.Changeset.for_create(MyApp.User, :create)\n    iex> changeset = changeset\n    ...> |> Ash.Changeset.select([:name, :email])\n    ...> |> Ash.Changeset.select([:created_at])\n    iex> changeset.select\n    [:name, :email, :created_at]\n\n    # Replace previous selection\n    iex> changeset = Ash.Changeset.for_create(MyApp.Post, :create)\n    iex> changeset = changeset\n    ...> |> Ash.Changeset.select([:title, :content])\n    ...> |> Ash.Changeset.select([:title, :published_at], replace?: true)\n    iex> changeset.select\n    [:title, :published_at]","ref":"Ash.Changeset.html#select/3-examples"},{"type":"function","title":"See also - Ash.Changeset.select/3","doc":"- `ensure_selected/2` for adding fields without replacing existing selection\n- `deselect/2` for removing fields from selection\n- `selecting?/2` for checking if a field is selected\n- `load/2` for loading relationships and calculations","ref":"Ash.Changeset.html#select/3-see-also"},{"type":"function","title":"Ash.Changeset.selecting?/2","doc":"","ref":"Ash.Changeset.html#selecting?/2"},{"type":"function","title":"Ash.Changeset.set_argument/3","doc":"Add an argument to the changeset, which will be provided to the action.\n\nArguments can be set on a changeset before calling `for_action/4` (or the specific `for_*` functions).\nWhen `for_action/4` is called, arguments are validated and cast according to the action's argument\ndefinitions. Use this function to set arguments that will be available during the action's execution.","ref":"Ash.Changeset.html#set_argument/3"},{"type":"function","title":"Examples - Ash.Changeset.set_argument/3","doc":"# Set arguments before calling for_action\n    iex> changeset = MyApp.Post\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:auto_publish, true)\n    ...> |> Ash.Changeset.set_argument(:slug_prefix, \"blog\")\n    iex> changeset.arguments\n    %{auto_publish: true, slug_prefix: \"blog\"}\n\n    # Arguments are validated and cast when for_action is called\n    iex> changeset = MyApp.User\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:age, \"25\")  # String will be cast to integer\n    ...> |> Ash.Changeset.for_action(:create, %{name: \"John\"})\n    iex> changeset.arguments\n    %{age: 25}\n\n    # Chain argument setting with attribute changes\n    iex> changeset = MyApp.Post\n    ...> |> Ash.Changeset.new()\n    ...> |> Ash.Changeset.set_argument(:generate_excerpt, true)\n    ...> |> Ash.Changeset.change_attribute(:title, \"My Post\")\n    ...> |> Ash.Changeset.for_action(:create, %{content: \"Post content\"})","ref":"Ash.Changeset.html#set_argument/3-examples"},{"type":"function","title":"See also - Ash.Changeset.set_argument/3","doc":"- `set_arguments/2` for setting multiple arguments at once\n- `get_argument/2` for retrieving argument values\n- `force_set_argument/3` for setting arguments in hooks without warnings\n- `delete_argument/2` for removing arguments","ref":"Ash.Changeset.html#set_argument/3-see-also"},{"type":"function","title":"Ash.Changeset.set_arguments/2","doc":"Merge a map of arguments to the arguments list.","ref":"Ash.Changeset.html#set_arguments/2"},{"type":"function","title":"Ash.Changeset.set_context/2","doc":"Deep merges the provided map into the changeset context that can be used later.\n\nDo not use the `private` key in your custom context, as that is reserved for internal use.","ref":"Ash.Changeset.html#set_context/2"},{"type":"function","title":"Ash.Changeset.set_private_argument/3","doc":"Add a private argument to the changeset, which will be provided to the action.","ref":"Ash.Changeset.html#set_private_argument/3"},{"type":"function","title":"Ash.Changeset.set_result/2","doc":"Set the result of the action. This will prevent running the underlying datalayer behavior","ref":"Ash.Changeset.html#set_result/2"},{"type":"function","title":"Ash.Changeset.set_tenant/2","doc":"","ref":"Ash.Changeset.html#set_tenant/2"},{"type":"type","title":"Ash.Changeset.t/0","doc":"The changeset struct containing all the information about a pending action.\n\nThis struct tracks changes to attributes, arguments for the action, validation state,\nhooks to run at various points, and other metadata needed to execute an action.","ref":"Ash.Changeset.html#t:t/0"},{"type":"function","title":"Ash.Changeset.timeout/3","doc":"","ref":"Ash.Changeset.html#timeout/3"},{"type":"function","title":"Ash.Changeset.update_change/3","doc":"Updates an existing attribute change by applying a function to it.\n\nThis is useful for applying some kind of normalization to the attribute.\n\n```elixir\nAsh.Changeset.update_change(changeset, :serial, &String.downcase/1)\n```\n\nThe update function gets called with the value already cast to the correct type.\n\n```elixir\nchangeset\n|> Ash.Changeset.change_attribute(:integer_attribute, \"3\")\n|> Ash.Changeset.update_change(:integer_attribute, fn x -> x + 1 end)\n```","ref":"Ash.Changeset.html#update_change/3"},{"type":"function","title":"Invalid value handling - Ash.Changeset.update_change/3","doc":"If `update_change` is called with a changeset that has not been validated yet, the update\nfunction must handle potentially invalid and `nil` values.\n\nTo only deal with valid values, you can call `update_change` in a `before_action` hook.","ref":"Ash.Changeset.html#update_change/3-invalid-value-handling"},{"type":"function","title":"Ash.Changeset.with_hooks/3","doc":"Wraps a function in the before/after action hooks of a changeset.\n\nThe function takes a changeset and if it returns\n`{:ok, result}`, the result will be passed through the after\naction hooks.","ref":"Ash.Changeset.html#with_hooks/3"},{"type":"module","title":"Ash.Query","doc":"A data structure for reading data from a resource.\n\nQueries are run by calling `Ash.read/2`.\n\nExamples:\n\n```elixir\nrequire Ash.Query\n\nMyApp.Post\n|> Ash.Query.filter(likes > 10)\n|> Ash.Query.sort([:title])\n|> Ash.read!()\n\nMyApp.Author\n|> Ash.Query.aggregate(:published_post_count, :posts, query: [filter: [published: true]])\n|> Ash.Query.sort(published_post_count: :desc)\n|> Ash.Query.limit(10)\n|> Ash.read!()\n\nMyApp.Author\n|> Ash.Query.load([:post_count, :comment_count])\n|> Ash.Query.load(posts: [:comments])\n|> Ash.read!()\n```\n\nTo see more examples of what you can do with `Ash.Query` and read actions in general,\nsee the [writing queries how-to guide](/documentation/how-to/write-queries.livemd).","ref":"Ash.Query.html"},{"type":"module","title":"Capabilities & Limitations - Ash.Query","doc":"Ash Framework provides a comprehensive suite of querying tools designed to address common application development needs. While powerful and flexible, these tools are focused on domain-driven design rather than serving as a general-purpose ORM.\n\nAsh's query tools support:\n\n- Filtering records based on complex conditions\n- Sorting results using single or multiple criteria\n- Setting result limits and offsets\n- Pagination, with offset/limit and keysets\n- Selecting distinct records to eliminate duplicates\n- Computing dynamic properties at query time\n- Aggregating data from related resources\n\nWhile Ash's query tools often eliminate the need for direct database queries, Ash is not itself designed to be a comprehensive ORM or database query builder.\n\nFor specialized querying needs that fall outside Ash's standard capabilities, the framework provides escape hatches. These mechanisms allow developers to implement custom query logic when necessary.","ref":"Ash.Query.html#module-capabilities-limitations"},{"type":"module","title":"Important Considerations - Ash.Query","doc":"1. Ash is primarily a domain modeling framework, not a database abstraction layer\n2. While comprehensive, the tooling is intentionally constrained to resource-oriented access\n3. Escape hatches exist for cases that require custom query logic\n\nFor complex queries that fall outside these tools, consider whether they represent domain concepts that could be modeled differently, or if they truly require custom implementation through escape hatches.","ref":"Ash.Query.html#module-important-considerations"},{"type":"module","title":"Escape Hatches - Ash.Query","doc":"Many of the tools in `Ash.Query` are surprisingly deep and capable, covering everything you\nneed to build your domain logic. With that said, these tools are *not*\ndesigned to encompass *every kind of query* that you could possibly want to\nwrite over your data. `Ash` is *not* an ORM or a database query tool, despite\nthe fact that its query building tools often make those kinds of tools\nunnecessary in all but the rarest of cases. Not every kind of query that you\ncould ever wish to write can be expressed with Ash.Query. Elixir has a\nbest-in-class library for working directly with databases, called\n[Ecto](https://hexdocs.pm/ecto/Ecto.html), and if you end up building a\ncertain type of feature like analytics or reporting dashboards, you may find\nyourself working directly with Ecto. Data layers like AshPostgres are built\non top of Ecto. In fact, every `Ash.Resource` is an `Ecto.Schema`!\n\n>","ref":"Ash.Query.html#module-escape-hatches"},{"type":"module","title":"Choose escape hatches wisely - Ash.Query","doc":">\n> You should choose to use Ash builtin functionality wherever possible.\n> Barring that, you should choose the *least powerful* escape hatch that\n> can solve your problem. The options below are presented in the order\n> that you should prefer them, but you should only use *any of them*\n> if no builtin tooling will suffice.","ref":"Ash.Query.html#module-choose-escape-hatches-wisely"},{"type":"module","title":"Fragments - Ash.Query","doc":"Fragments only barely count as an escape hatch. You will often find yourself\nwanting to use a function or operator specific to your data layer, and fragments\nare purpose built to this end. You can use data-layer-specific expressions in your\nexpressions for filters, calculations, etc. For example:\n\n```elixir\nResource\n|> Ash.Query.filter(expr(fragment(\"lower(?)\", name) == \"fred\"))\n|> Ash.Query.filter(expr(fragment(\"? @> ?\", tags, [\"important\"])))\n```","ref":"Ash.Query.html#module-fragments"},{"type":"module","title":"Manual Read Actions - Ash.Query","doc":"See [the manual read actions guide](/documentation/topics/actions/manual-actions.md).\n\n### `d:Ash.Resource.Dsl|actions.read.modify_query`\n\nWhen running read actions, you can modify the underlying data layer query directly,\nwhich can solve for cases when you cannot express your query using the standard Ash query interface.\n\n```elixir\nactions do\n  read :complex_search do\n    argument\n    modify_query {SearchMod, :modify, []}\n  end\nend\n```\n\n```elixir\ndefmodule SearchMod do\n  def modify(ash_query, data_layer_query) do\n    # Here you can modify the underlying data layer query directly\n    # For example, with AshPostgres you get access to the Ecto query\n    {:ok, Ecto.Query.where(data_layer_query, [p], fragment(\"? @@ plainto_tsquery(?)\", p.search_vector, ^ash_query.arguments.search_text))}\n  end\nend\n```","ref":"Ash.Query.html#module-manual-read-actions"},{"type":"module","title":"Using Ecto directly - Ash.Query","doc":"For data layers like `AshPostgres`, you can interact directly with `Ecto`. You can do this\nby using the `Ash.Resource` as its corresponding `Ecto.Schema`, like so:\n\n```elixir\nimport Ecto.Query\n\nquery =\n  from p in MyApp.Post,\n    where: p.likes > 100,\n    select: p\n\n MyApp.Repo.all(query)\n```\n\nOr you can build an `Ash.Query`, and get the corresponding ecto query:\n\n```elixir\nMyApp.Post\n|> Ash.Query.for_read(:read)\n|> Ash.data_layer_query()\n|> case do\n  {:ok, %{query: ecto_query}} ->\n    ecto_query\n    |> Ecto.Query.where([p], p.likes > 100)\n    |> MyApp.Repo.all()\n\n  {:error, error} ->\n    {:error, error}\nend\n```","ref":"Ash.Query.html#module-using-ecto-directly"},{"type":"function","title":"Ash.Query.accessing/3","doc":"Returns a list of attributes, aggregates, relationships, and calculations that are being loaded\n\nProvide a list of field types to narrow down the returned results.","ref":"Ash.Query.html#accessing/3"},{"type":"function","title":"Ash.Query.add_error/3","doc":"Add an error to the errors list and mark the query as invalid.\n\nSee `Ash.Error.to_ash_error/3` for more on supported values for `error`","ref":"Ash.Query.html#add_error/3"},{"type":"function","title":"Inconsistencies - Ash.Query.add_error/3","doc":"The `path` argument is the second argument here, but the third argument\nin `Ash.ActionInput.add_error/2` and `Ash.Changeset.add_error/2`.\nThis will be fixed in 4.0.","ref":"Ash.Query.html#add_error/3-inconsistencies"},{"type":"function","title":"Ash.Query.after_action/2","doc":"Adds an after_action hook to the query.\n\nAfter action hooks are called with the query and the list of records returned\nfrom the action. They can modify the records, perform side effects, or return\nerrors to halt processing. The hook can return notifications alongside the records.","ref":"Ash.Query.html#after_action/2"},{"type":"function","title":"Examples - Ash.Query.after_action/2","doc":"# Transform records after loading\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.after_action(fn query, records ->\n    ...>   enriched_records = Enum.map(records, &add_computed_field/1)\n    ...>   {:ok, enriched_records}\n    ...> end)\n\n    # Log successful reads\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.after_action(fn query, records ->\n    ...>   IO.puts(\"Successfully loaded #{length(records)} posts\")\n    ...>   {:ok, records}\n    ...> end)\n\n    # Add notifications after the action\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.after_action(fn query, records ->\n    ...>   notifications = create_read_notifications(records)\n    ...>   {:ok, records, notifications}\n    ...> end)\n\n    # Validate results and potentially error\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.after_action(fn query, records ->\n    ...>   if Enum.any?(records, &restricted?/1) do\n    ...>     {:error, \"Access denied to restricted posts\"}\n    ...>   else\n    ...>     {:ok, records}\n    ...>   end\n    ...> end)","ref":"Ash.Query.html#after_action/2-examples"},{"type":"function","title":"See also - Ash.Query.after_action/2","doc":"- `before_action/3` for hooks that run before the action executes\n- `around_transaction/2` for hooks that wrap the entire transaction\n- `Ash.read/2` for executing queries with hooks","ref":"Ash.Query.html#after_action/2-see-also"},{"type":"function","title":"Ash.Query.aggregate/4","doc":"Adds an aggregation to the query.\n\nAggregations are made available on the `aggregates` field of the records returned.\nThey allow you to compute values from related data without loading entire relationships,\nmaking them very efficient for statistical operations.","ref":"Ash.Query.html#aggregate/4"},{"type":"function","title":"Examples - Ash.Query.aggregate/4","doc":"# Count related records\n    iex> Ash.Query.aggregate(MyApp.Author, :post_count, :count, :posts)\n    %Ash.Query{aggregates: %{post_count: %Ash.Query.Aggregate{...}}, ...}\n\n    # Sum values from related records\n    iex> Ash.Query.aggregate(MyApp.Author, :total_likes, :sum, :posts, field: :like_count)\n    %Ash.Query{aggregates: %{total_likes: %Ash.Query.Aggregate{...}}, ...}\n\n    # Average with filtered aggregation\n    iex> published_query = Ash.Query.filter(MyApp.Post, published: true)\n    iex> Ash.Query.aggregate(MyApp.Author, :avg_published_likes, :avg, :posts,\n    ...>   field: :like_count, query: published_query)\n    %Ash.Query{aggregates: %{avg_published_likes: %Ash.Query.Aggregate{...}}, ...}\n\n    # Count with default value\n    iex> Ash.Query.aggregate(MyApp.Author, :post_count, :count, :posts, default: 0)\n    %Ash.Query{aggregates: %{post_count: %Ash.Query.Aggregate{...}}, ...}","ref":"Ash.Query.html#aggregate/4-examples"},{"type":"function","title":"Options - Ash.Query.aggregate/4","doc":"* `query` - The query over the destination resource to use as a base for aggregation\n  * `field` - The field to use for the aggregate. Not necessary for all aggregate types\n  * `default` - The default value to use if the aggregate returns nil\n  * `filterable?` - Whether or not this aggregate may be referenced in filters\n  * `type` - The type of the aggregate\n  * `constraints` - Type constraints for the aggregate's type\n  * `implementation` - An implementation used when the aggregate kind is custom\n  * `read_action` - The read action to use on the destination resource\n  * `authorize?` - Whether or not to authorize access to this aggregate\n  * `join_filters` - A map of relationship paths to filter expressions","ref":"Ash.Query.html#aggregate/4-options"},{"type":"function","title":"See also - Ash.Query.aggregate/4","doc":"- [Resource DSL aggregates documentation](dsl-ash-resource.html#aggregates) for more information\n- `load/3` for loading relationships instead of aggregating\n- `calculate/8` for custom calculations\n- `Ash.read/2` for executing queries with aggregates","ref":"Ash.Query.html#aggregate/4-see-also"},{"type":"function","title":"Ash.Query.aggregate/5","doc":"","ref":"Ash.Query.html#aggregate/5"},{"type":"function","title":"Ash.Query.apply_to/3","doc":"Applies a query to a list of records in memory.\n\nThis function takes a query and applies its filters, sorting, pagination,\nand loading operations to an existing list of records in memory rather than\nquerying the data layer. Useful for post-processing records or applying\nquery logic to data from multiple sources.","ref":"Ash.Query.html#apply_to/3"},{"type":"function","title":"Examples - Ash.Query.apply_to/3","doc":"# Apply filtering to records in memory\n    iex> records = [%MyApp.Post{title: \"A\", published: true}, %MyApp.Post{title: \"B\", published: false}]\n    iex> query = MyApp.Post |> Ash.Query.filter(published: true)\n    iex> Ash.Query.apply_to(query, records)\n    {:ok, [%MyApp.Post{title: \"A\", published: true}]}\n\n    # Apply sorting and limiting\n    iex> records = [%MyApp.Post{title: \"C\", likes: 5}, %MyApp.Post{title: \"A\", likes: 10}]\n    iex> query = MyApp.Post |> Ash.Query.sort(likes: :desc) |> Ash.Query.limit(1)\n    iex> Ash.Query.apply_to(query, records)\n    {:ok, [%MyApp.Post{title: \"A\", likes: 10}]}\n\n    # Apply with loading relationships\n    iex> records = [%MyApp.Post{id: 1}, %MyApp.Post{id: 2}]\n    iex> query = MyApp.Post |> Ash.Query.load(:author)\n    iex> Ash.Query.apply_to(query, records, domain: MyApp.Blog)\n    {:ok, [%MyApp.Post{id: 1, author: %MyApp.User{...}}, ...]}","ref":"Ash.Query.html#apply_to/3-examples"},{"type":"function","title":"Options - Ash.Query.apply_to/3","doc":"* `domain` - The domain to use for loading relationships\n  * `actor` - The actor for authorization during loading\n  * `tenant` - The tenant for multitenant operations\n  * `parent` - Parent context for nested operations","ref":"Ash.Query.html#apply_to/3-options"},{"type":"function","title":"See also - Ash.Query.apply_to/3","doc":"- `Ash.read/2` for querying the data layer directly\n- `load/3` for configuring relationship loading\n- `filter/2` for adding filter conditions","ref":"Ash.Query.html#apply_to/3-see-also"},{"type":"type","title":"Ash.Query.around_action_fun/0","doc":"Function type for around_action hooks that modify query execution flow.","ref":"Ash.Query.html#t:around_action_fun/0"},{"type":"type","title":"Ash.Query.around_callback/0","doc":"Callback function type that takes a query and returns an around_result.","ref":"Ash.Query.html#t:around_callback/0"},{"type":"type","title":"Ash.Query.around_result/0","doc":"Result type for around_transaction hooks, containing either successful records or an error.","ref":"Ash.Query.html#t:around_result/0"},{"type":"function","title":"Ash.Query.around_transaction/2","doc":"Adds an around_transaction hook to the query.\n\nYour function will get the query, and a callback that must be called with a query (that may be modified).\nThe callback will return `{:ok, results}` or `{:error, error}`. You can modify these values, but the return value\nmust be one of those types.\n\nThe around_transaction calls happen first, and then (after they each resolve their callbacks) the `before_action`\nhooks are called, followed by the `after_action` hooks being run. Then, the code that appeared *after* the callbacks were called is then run.","ref":"Ash.Query.html#around_transaction/2"},{"type":"function","title":"Examples - Ash.Query.around_transaction/2","doc":"# Add logging around the transaction\n    iex> query = MyApp.Post |> Ash.Query.around_transaction(fn query, callback ->\n    ...>   IO.puts(\"Starting transaction for #{inspect(query.resource)}\")\n    ...>   result = callback.(query)\n    ...>   IO.puts(\"Transaction completed: #{inspect(result)}\")\n    ...>   result\n    ...> end)\n\n    # Add error handling and retry logic\n    iex> query = MyApp.Post |> Ash.Query.around_transaction(fn query, callback ->\n    ...>   case callback.(query) do\n    ...>     {:ok, results} = success -> success\n    ...>     {:error, %{retryable?: true}} ->\n    ...>       callback.(query)  # Retry once\n    ...>     error -> error\n    ...>   end\n    ...> end)","ref":"Ash.Query.html#around_transaction/2-examples"},{"type":"function","title":"Warning - Ash.Query.around_transaction/2","doc":"Using this without understanding how it works can cause big problems.\nYou *must* call the callback function that is provided to your hook, and the return value must\ncontain the same structure that was given to you, i.e `{:ok, result_of_action}`.","ref":"Ash.Query.html#around_transaction/2-warning"},{"type":"function","title":"See also - Ash.Query.around_transaction/2","doc":"- `before_action/3` for hooks that run before the action executes\n- `after_action/2` for hooks that run after the action completes\n- `Ash.read/2` for executing queries with hooks","ref":"Ash.Query.html#around_transaction/2-see-also"},{"type":"type","title":"Ash.Query.around_transaction_fun/0","doc":"Function type for around_transaction hooks that wrap query execution in a transaction.","ref":"Ash.Query.html#t:around_transaction_fun/0"},{"type":"function","title":"Ash.Query.before_action/3","doc":"Adds a before_action hook to the query.\n\nBefore action hooks are called after preparations but before the actual\ndata layer query is executed. They receive the prepared query and can\nmodify it or perform side effects before the action runs.","ref":"Ash.Query.html#before_action/3"},{"type":"function","title":"Examples - Ash.Query.before_action/3","doc":"# Add validation before the query runs\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.before_action(fn query ->\n    ...>   if Enum.empty?(query.sort) do\n    ...>     Ash.Query.sort(query, :created_at)\n    ...>   else\n    ...>     query\n    ...>   end\n    ...> end)\n\n    # Add logging before the action\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.before_action(fn query ->\n    ...>   IO.puts(\"Executing query for #{length(query.filter || [])} filters\")\n    ...>   query\n    ...> end)\n\n    # Prepend a hook to run first\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.before_action(&setup_query/1)\n    ...> |> Ash.Query.before_action(&early_validation/1, prepend?: true)","ref":"Ash.Query.html#before_action/3-examples"},{"type":"function","title":"Options - Ash.Query.before_action/3","doc":"- `prepend?` - when `true`, places the hook before all other hooks instead of after","ref":"Ash.Query.html#before_action/3-options"},{"type":"function","title":"See also - Ash.Query.before_action/3","doc":"- `after_action/2` for hooks that run after the action completes\n- `around_transaction/2` for hooks that wrap the entire transaction\n- `Ash.read/2` for executing queries with hooks","ref":"Ash.Query.html#before_action/3-see-also"},{"type":"function","title":"Ash.Query.build/3","doc":"Builds a query from a keyword list.\n\nThis is used by certain query constructs like aggregates. It can also be used to manipulate a data structure\nbefore passing it to an ash query. It allows for building an entire query struct using only a keyword list.\n\nFor example:\n\n```elixir\nAsh.Query.build(MyResource, filter: [name: \"fred\"], sort: [name: :asc], load: [:foo, :bar], offset: 10)\n```\n\nIf you want to use the expression style filters, you can use `expr/1`.\n\nFor example:\n\n```elixir\nimport Ash.Expr, only: [expr: 1]\n\nAsh.Query.build(Myresource, filter: expr(name == \"marge\"))\n```","ref":"Ash.Query.html#build/3"},{"type":"function","title":"Options - Ash.Query.build/3","doc":"* `:filter` (`t:term/0`) - A filter keyword, map or expression\n\n* `:filter_input` (`t:term/0`) - A filter keyword or map, provided as input from an external source\n\n* `:sort` (`t:term/0`) - A sort list or keyword\n\n* `:sort_input` (`t:term/0`) - A sort list or keyword, provided as input from an external source\n\n* `:default_sort` (`t:term/0`) - A sort list or keyword to apply only if no other sort is specified, So if you apply any `sort`, this will be ignored.\n\n* `:distinct_sort` (`t:term/0`) - A distinct_sort list or keyword\n\n* `:limit` (`t:integer/0`) - A limit to apply\n\n* `:offset` (`t:integer/0`) - An offset to apply\n\n* `:load` (`t:term/0`) - A load statement to add to the query\n\n* `:strict_load` (`t:term/0`) - A load statement to add to the query with the `strict?` option set to `true`\n\n* `:select` (`t:term/0`) - A select statement to add to the query\n\n* `:ensure_selected` (`t:term/0`) - An ensure_selected statement to add to the query\n\n* `:aggregate` (`t:term/0`) - A custom aggregate to add to the query. Can be `{name, type, relationship}` or `{name, type, relationship, build_opts}`\n\n* `:calculate` (`t:term/0`) - A custom calculation to add to the query. Can be `{name, module_and_opts}` or `{name, module_and_opts, context}`\n\n* `:distinct` (list of `t:atom/0`) - A distinct clause to add to the query\n\n* `:context` (`t:map/0`) - A map to merge into the query context","ref":"Ash.Query.html#build/3-options"},{"type":"function","title":"Ash.Query.calculate/8","doc":"Adds a calculation to the query.\n\nCalculations are made available on the `calculations` field of the records returned.\nThey allow you to compute dynamic values based on record data, other fields, or\nexternal information at query time.\n\nThe `module_and_opts` argument accepts either a `module` or a `{module, opts}`. For more information\non what that module should look like, see `Ash.Resource.Calculation`.","ref":"Ash.Query.html#calculate/8"},{"type":"function","title":"Examples - Ash.Query.calculate/8","doc":"# Add a simple calculation\n    iex> Ash.Query.calculate(MyApp.User, :display_name, :string,\n    ...>   {MyApp.Calculations.DisplayName, []})\n    %Ash.Query{calculations: %{display_name: %{...}}, ...}\n\n    # Add calculation with arguments\n    iex> Ash.Query.calculate(MyApp.Post, :word_count, :integer,\n    ...>   {MyApp.Calculations.WordCount, []}, %{field: :content})\n    %Ash.Query{calculations: %{word_count: %{...}}, ...}\n\n    # Add calculation with constraints and context\n    iex> Ash.Query.calculate(MyApp.Product, :discounted_price, :decimal,\n    ...>   {MyApp.Calculations.Discount, []}, %{rate: 0.1},\n    ...>   [precision: 2, scale: 2], %{currency: \"USD\"})\n    %Ash.Query{calculations: %{discounted_price: %{...}}, ...}","ref":"Ash.Query.html#calculate/8-examples"},{"type":"function","title":"See also - Ash.Query.calculate/8","doc":"- `Ash.Resource.Calculation` for implementing custom calculations\n- `aggregate/5` for computing values from related records\n- `load/3` for loading predefined calculations from the resource\n- `select/3` for controlling which fields are returned alongside calculations","ref":"Ash.Query.html#calculate/8-see-also"},{"type":"function","title":"Ash.Query.clear_result/1","doc":"Removes a result set previously with `set_result/2`","ref":"Ash.Query.html#clear_result/1"},{"type":"function","title":"Ash.Query.combination_of/2","doc":"Produces a query that is the combination of multiple queries.\n\nAll aspects of the parent query are applied to the combination in total.\n\nSee `Ash.Query.Combination` for more on creating combination queries.","ref":"Ash.Query.html#combination_of/2"},{"type":"function","title":"Example - Ash.Query.combination_of/2","doc":"```elixir\n# Top ten users not on a losing streak and top ten users who are not on a winning streak\nUser\n|> Ash.Query.filter(active == true)\n|> Ash.Query.combination_of([\n  # must always begin with a base combination\n  Ash.Query.Combination.base(\n    sort: [score: :desc],\n    filter: expr(not(on_a_losing_streak)),\n    limit: 10\n  ),\n  Ash.Query.Combination.union(\n    sort: [score: :asc],\n    filter: expr(not(on_a_winning_streak)),\n    limit: 10\n  )\n])\n|> Ash.read!()\n```","ref":"Ash.Query.html#combination_of/2-example"},{"type":"function","title":"Select and calculations - Ash.Query.combination_of/2","doc":"There is no `select` available for combinations, instead the select of the outer query\nis used for each combination. However, you can use the `calculations` field in\n`Ash.Query.Combination` to add expression calculations. Those calculations can \"overwrite\"\na selected attribute, or can introduce a new field. Note that, for SQL data layers, all\ncombinations will be required to have the same number of fields in their SELECT statement,\nwhich means that if one combination adds a calculation, all of the others must also add\nthat calculation.\n\nIn this example, we compute separate match scores\n\n```elixir\nquery = \"fred\"\n\nUser\n|> Ash.Query.filter(active == true)\n|> Ash.Query.combination_of([\n  # must always begin with a base combination\n  Ash.Query.Combination.base(\n    filter: expr(trigram_similarity(user_name, ^query) >= 0.5),\n    calculate: %{\n      match_score: trigram_similarity(user_name, ^query)\n    },\n    sort: [\n      calc(trigram_similarity(user_name, ^query), :desc)\n    ],\n    limit: 10\n  ),\n  Ash.Query.Combination.union(\n    filter: expr(trigram_similarity(email, ^query) >= 0.5),\n    calculate: %{\n      match_score: trigram_similarity(email, ^query)\n    },\n    sort: [\n      calc(trigram_similarity(email, ^query), :desc)\n    ],\n    limit: 10\n  )\n])\n|> Ash.read!()\n```","ref":"Ash.Query.html#combination_of/2-select-and-calculations"},{"type":"function","title":"Ash.Query.data_layer_query/2","doc":"Return the underlying data layer query for an ash query","ref":"Ash.Query.html#data_layer_query/2"},{"type":"function","title":"Ash.Query.default_sort/3","doc":"Apply a sort only if no sort has been specified yet.\n\nThis is useful for providing default sorts that can be overridden.","ref":"Ash.Query.html#default_sort/3"},{"type":"function","title":"Examples - Ash.Query.default_sort/3","doc":"```elixir\n# This will sort by name if no sort has been specified\nAsh.Query.default_sort(query, :name)\n\n# This will sort by name descending if no sort has been specified\nAsh.Query.default_sort(query, name: :desc)\n```","ref":"Ash.Query.html#default_sort/3-examples"},{"type":"function","title":"Ash.Query.delete_argument/2","doc":"Remove an argument from the query","ref":"Ash.Query.html#delete_argument/2"},{"type":"function","title":"Ash.Query.deselect/2","doc":"Ensures that the specified attributes are `nil` in the query results.\n\nThis function removes specified fields from the selection, causing them to be\nexcluded from the query results. If no fields are currently selected (meaning\nall fields would be returned by default), this will first select all default\nfields and then remove the specified ones.","ref":"Ash.Query.html#deselect/2"},{"type":"function","title":"Examples - Ash.Query.deselect/2","doc":"# Remove specific fields from results\n    iex> MyApp.Post |> Ash.Query.deselect([:content])\n    %Ash.Query{select: [:id, :title, :created_at, ...], ...}\n\n    # Remove multiple fields\n    iex> MyApp.Post |> Ash.Query.deselect([:content, :metadata])\n    %Ash.Query{select: [:id, :title, :created_at, ...], ...}\n\n    # Deselect from existing selection\n    iex> MyApp.Post\n    ...> |> Ash.Query.select([:title, :content, :author_id])\n    ...> |> Ash.Query.deselect([:content])\n    %Ash.Query{select: [:id, :title, :author_id], ...}\n\n    # Deselect empty list (no-op)\n    iex> MyApp.Post |> Ash.Query.deselect([])\n    %Ash.Query{...}","ref":"Ash.Query.html#deselect/2-examples"},{"type":"function","title":"See also - Ash.Query.deselect/2","doc":"- `select/3` for explicitly controlling field selection\n- `ensure_selected/2` for adding fields without removing others\n- Primary key fields cannot be deselected and will always be included","ref":"Ash.Query.html#deselect/2-see-also"},{"type":"function","title":"Ash.Query.distinct/2","doc":"Get results distinct on the provided fields.\n\nTakes a list of fields to distinct on. Each call is additive, so to remove the `distinct` use\n`unset/2`.\n\nExamples:\n\n```\nAsh.Query.distinct(query, [:first_name, :last_name])\n\nAsh.Query.distinct(query, :email)\n```","ref":"Ash.Query.html#distinct/2"},{"type":"function","title":"Ash.Query.distinct_sort/3","doc":"Set a sort to determine how distinct records are selected.\n\nIf none is set, any sort applied to the query will be used.\n\nThis is useful if you want to control how the `distinct` records\nare selected without affecting (necessarily, it may affect it if\nthere is no sort applied) the overall sort of the query","ref":"Ash.Query.html#distinct_sort/3"},{"type":"function","title":"Ash.Query.ensure_selected/2","doc":"Ensures that the given attributes are selected.\n\nThe first call to `select/2` will *limit* the fields to only the provided fields.\nUse `ensure_selected/2` to say \"select this field (or these fields) without deselecting anything else\".\nThis function is additive - it will not remove any fields that are already selected.","ref":"Ash.Query.html#ensure_selected/2"},{"type":"function","title":"Examples - Ash.Query.ensure_selected/2","doc":"# Ensure specific fields are selected (additive)\n    iex> MyApp.Post |> Ash.Query.ensure_selected([:title])\n    %Ash.Query{select: [:id, :title, :content, :created_at], ...}\n\n    # Add to existing selection\n    iex> MyApp.Post\n    ...> |> Ash.Query.select([:title])\n    ...> |> Ash.Query.ensure_selected([:content, :author_id])\n    %Ash.Query{select: [:id, :title, :content, :author_id], ...}\n\n    # Ensure fields for relationship loading\n    iex> MyApp.Post\n    ...> |> Ash.Query.ensure_selected([:author_id])\n    ...> |> Ash.Query.load(:author)\n    %Ash.Query{select: [..., :author_id], load: [author: []], ...}","ref":"Ash.Query.html#ensure_selected/2-examples"},{"type":"function","title":"See also - Ash.Query.ensure_selected/2","doc":"- `select/3` for explicitly controlling field selection\n- `deselect/2` for removing specific fields from selection\n- `load/3` for loading relationships that may require specific fields","ref":"Ash.Query.html#ensure_selected/2-see-also"},{"type":"macro","title":"Ash.Query.equivalent_to/2","doc":"Determines if the filter statement of a query is equivalent to the provided expression.\n\nThis uses the satisfiability solver that is used when solving for policy authorizations. In complex scenarios, or when using\ncustom database expressions, (like fragments in ash_postgres), this function may return `:maybe`. Use `supserset_of?` to always return\na boolean.","ref":"Ash.Query.html#equivalent_to/2"},{"type":"macro","title":"Ash.Query.equivalent_to?/2","doc":"Same as `equivalent_to/2` but always returns a boolean. `:maybe` returns `false`.","ref":"Ash.Query.html#equivalent_to?/2"},{"type":"function","title":"Ash.Query.fetch_argument/2","doc":"Fetches the value of an argument provided to the query.\n\nReturns `{:ok, value}` if the argument exists, or `:error` if not found.\nThis is the safer alternative to `get_argument/2` when you need to distinguish\nbetween a `nil` value and a missing argument.","ref":"Ash.Query.html#fetch_argument/2"},{"type":"function","title":"Examples - Ash.Query.fetch_argument/2","doc":"# Fetch an argument that exists\n    iex> query = Ash.Query.for_read(MyApp.Post, :published, %{since: ~D[2023-01-01]})\n    iex> Ash.Query.fetch_argument(query, :since)\n    {:ok, ~D[2023-01-01]}\n\n    # Fetch an argument that doesn't exist\n    iex> query = Ash.Query.for_read(MyApp.Post, :published, %{})\n    iex> Ash.Query.fetch_argument(query, :since)\n    :error\n\n    # Distinguish between nil and missing arguments\n    iex> query = Ash.Query.for_read(MyApp.Post, :search, %{query: nil})\n    iex> Ash.Query.fetch_argument(query, :query)\n    {:ok, nil}","ref":"Ash.Query.html#fetch_argument/2-examples"},{"type":"function","title":"See also - Ash.Query.fetch_argument/2","doc":"- `get_argument/2` for simpler argument access\n- `set_argument/3` for adding arguments to queries\n- `for_read/4` for creating queries with arguments","ref":"Ash.Query.html#fetch_argument/2-see-also"},{"type":"macro","title":"Ash.Query.filter/2","doc":"Attach a filter statement to the query.\n\nThe filter is applied as an \"and\" to any filters currently on the query.\nFilters allow you to specify conditions that records must meet to be included\nin the query results. Multiple filters on the same query are combined with \"and\" logic.","ref":"Ash.Query.html#filter/2"},{"type":"macro","title":"Examples - Ash.Query.filter/2","doc":"# Filter with simple equality\n    MyApp.Post\n    |> Ash.Query.filter(published: true)\n\n    # Filter with comparison operators\n    MyApp.Post\n    |> Ash.Query.filter(view_count > 100)\n\n    # Filter with complex expressions using do block\n    MyApp.Post\n    |> Ash.Query.filter do\n      published == true and view_count > 100\n    end","ref":"Ash.Query.html#filter/2-examples"},{"type":"macro","title":"See also - Ash.Query.filter/2","doc":"- `Ash.Filter` for comprehensive filter documentation\n- `sort/3` for ordering query results\n- `Ash.read/2` for executing filtered queries","ref":"Ash.Query.html#filter/2-see-also"},{"type":"function","title":"Ash.Query.filter_input/2","doc":"Attach a filter statement to the query labelled as user input.\n\nFilters added as user input (or filters constructed with `Ash.Filter.parse_input`)\nwill honor any field policies on resources by replacing any references to the field\nwith `nil` in cases where the actor should not be able to see the given field.\n\nThis function does not expect the expression style filter (because an external source\ncould never reasonably provide that). Instead, use the keyword/map style syntax. For\nexample:\n\n`expr(name == \"fred\")`\n\ncould be any of\n\n- map syntax: `%{\"name\" => %{\"eq\" => \"fred\"}}`\n- keyword syntax: `[name: [eq: \"fred\"]]`\n\nSee `Ash.Filter` for more.","ref":"Ash.Query.html#filter_input/2"},{"type":"function","title":"Ash.Query.for_read/4","doc":"Creates a query for a given read action and prepares it.\n\nThis function configures the query to use a specific read action with the provided\narguments and options. The query will be validated and prepared according to the\naction's configuration, including applying preparations and action filters.\n\nMultitenancy is *not* validated until an action is called. This allows you to avoid\nspecifying a tenant until just before calling the domain action.","ref":"Ash.Query.html#for_read/4"},{"type":"function","title":"Examples - Ash.Query.for_read/4","doc":"# Create a query for a simple read action\n    iex> Ash.Query.for_read(MyApp.Post, :read)\n    %Ash.Query{action: %{name: :read}, ...}\n\n    # Create a query with arguments for a parameterized action\n    iex> Ash.Query.for_read(MyApp.Post, :published, %{since: ~D[2023-01-01]})\n    %Ash.Query{action: %{name: :published}, arguments: %{since: ~D[2023-01-01]}, ...}\n\n    # Create a query with options\n    iex> Ash.Query.for_read(MyApp.Post, :read, %{}, actor: current_user, authorize?: true)\n    %Ash.Query{action: %{name: :read}, ...}","ref":"Ash.Query.html#for_read/4-examples"},{"type":"function","title":"Options - Ash.Query.for_read/4","doc":"* `:actor` (`t:term/0`) - set the actor, which can be used in any `Ash.Resource.Change`s configured on the action. (in the `context` argument)\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:authorize?` (`t:boolean/0`) - set authorize?, which can be used in any `Ash.Resource.Change`s configured on the action. (in the `context` argument)\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer to use. Will be carried over to the action. For more information see `Ash.Tracer`.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - set the tenant on the query\n\n* `:load` (`t:term/0`) - A load statement to apply to the query\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:context` (`t:map/0`) - A map of context to set on the query. This will be merged with any context set on the query itself.","ref":"Ash.Query.html#for_read/4-options"},{"type":"function","title":"See also - Ash.Query.for_read/4","doc":"- `Ash.read/2` for executing the prepared query\n- `new/2` for creating basic queries without specific actions\n- `load/3` for adding relationship loading to queries\n- `d:Ash.Resource.Dsl.actions.read` for defining read actions\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.Query.html#for_read/4-see-also"},{"type":"function","title":"Ash.Query.get_argument/2","doc":"Gets the value of an argument provided to the query.\n\nReturns the argument value if found, or `nil` if not found. Arguments can be\nprovided when creating queries with `for_read/4` and are used by action logic\nsuch as preparations and filters.","ref":"Ash.Query.html#get_argument/2"},{"type":"function","title":"Examples - Ash.Query.get_argument/2","doc":"# Get an argument that exists\n    iex> query = Ash.Query.for_read(MyApp.Post, :published, %{since: ~D[2023-01-01]})\n    iex> Ash.Query.get_argument(query, :since)\n    ~D[2023-01-01]\n\n    # Get an argument that doesn't exist\n    iex> query = Ash.Query.for_read(MyApp.Post, :published, %{})\n    iex> Ash.Query.get_argument(query, :since)\n    nil\n\n    # Arguments can be accessed by string or atom key\n    iex> query = Ash.Query.for_read(MyApp.Post, :search, %{\"query\" => \"elixir\"})\n    iex> Ash.Query.get_argument(query, :query)\n    \"elixir\"","ref":"Ash.Query.html#get_argument/2-examples"},{"type":"function","title":"See also - Ash.Query.get_argument/2","doc":"- `fetch_argument/2` for safer argument access with explicit error handling\n- `set_argument/3` for adding arguments to queries\n- `for_read/4` for creating queries with arguments","ref":"Ash.Query.html#get_argument/2-see-also"},{"type":"function","title":"Ash.Query.limit/2","doc":"Limits the number of results returned from the query.\n\nThis function sets the maximum number of records that will be returned\nwhen the query is executed. Useful for pagination and preventing\nlarge result sets from consuming too much memory.","ref":"Ash.Query.html#limit/2"},{"type":"function","title":"Examples - Ash.Query.limit/2","doc":"# Limit to 10 results\n    iex> MyApp.Post |> Ash.Query.limit(10)\n    %Ash.Query{limit: 10, ...}\n\n    # Remove existing limit\n    iex> query |> Ash.Query.limit(nil)\n    %Ash.Query{limit: nil, ...}\n\n    # Use with other query functions\n    iex> MyApp.Post\n    ...> |> Ash.Query.filter(published: true)\n    ...> |> Ash.Query.sort(:created_at)\n    ...> |> Ash.Query.limit(5)\n    %Ash.Query{limit: 5, ...}","ref":"Ash.Query.html#limit/2-examples"},{"type":"function","title":"See also - Ash.Query.limit/2","doc":"- `offset/2` for skipping records (pagination)\n- `page/2` for keyset pagination\n- `sort/3` for ordering results before limiting","ref":"Ash.Query.html#limit/2-see-also"},{"type":"function","title":"Ash.Query.load/3","doc":"Loads relationships, calculations, or aggregates on the resource.\n\nBy default, loading attributes has no effect, as all attributes are returned.\nSee the section below on \"Strict Loading\" for more.","ref":"Ash.Query.html#load/3"},{"type":"function","title":"Examples - Ash.Query.load/3","doc":"# Load simple relationships\n    iex> Ash.Query.load(MyApp.Post, :author)\n    %Ash.Query{load: [author: []], ...}\n\n    # Load nested relationships\n    iex> Ash.Query.load(MyApp.Post, [comments: [:author, :ratings]])\n    %Ash.Query{load: [comments: [author: [], ratings: []]], ...}\n\n    # Load relationships with custom queries\n    iex> author_query = Ash.Query.filter(MyApp.User, active: true)\n    iex> Ash.Query.load(MyApp.Post, [comments: [author: author_query]])\n    %Ash.Query{load: [comments: [author: %Ash.Query{...}]], ...}\n\n    # Load calculations with arguments\n    iex> Ash.Query.load(MyApp.User, full_name: %{format: :last_first})\n    %Ash.Query{calculations: %{full_name: %Ash.Query.Calculation{...}}, ...}","ref":"Ash.Query.html#load/3-examples"},{"type":"function","title":"Strict Loading - Ash.Query.load/3","doc":"By passing `strict?: true`, only specified attributes will be loaded when passing\na list of fields to fetch on a relationship, which allows for more optimized data-fetching.\n\n    # Only load specific fields on relationships\n    iex> Ash.Query.load(MyApp.Category, [:name, posts: [:title, :published_at]], strict?: true)\n    %Ash.Query{load: [posts: [:title, :published_at]], ...}\n\nWhen using `strict?: true` and loading nested relationships, you must specify all the\nattributes you want to load alongside the nested relationships:\n\n    # Must include needed attributes when loading nested relationships strictly\n    iex> Ash.Query.load(MyApp.Post, [:title, :published_at, category: [:name]], strict?: true)\n    %Ash.Query{...}","ref":"Ash.Query.html#load/3-strict-loading"},{"type":"function","title":"See also - Ash.Query.load/3","doc":"- `select/3` for controlling which attributes are returned\n- `ensure_selected/2` for ensuring specific fields are selected\n- `Ash.read/2` for executing queries with loaded data\n- [Relationships Guide](/documentation/topics/resources/relationships.md) for understanding relationships\n- [Calculations Guide](/documentation/topics/resources/calculations.md) for understanding calculations","ref":"Ash.Query.html#load/3-see-also"},{"type":"function","title":"Ash.Query.load_calculation_as/5","doc":"Adds a resource calculation to the query as a custom calculation with the provided name.\n\nExample:\n\n    Ash.Query.load_calculation_as(query, :calculation, :some_name, args: %{}, load_through: [:foo])","ref":"Ash.Query.html#load_calculation_as/5"},{"type":"function","title":"Ash.Query.load_through/4","doc":"Adds a load statement to the result of an attribute or calculation.\n\nUses `Ash.Type.load/5` to request that the type load nested data.","ref":"Ash.Query.html#load_through/4"},{"type":"function","title":"Ash.Query.loading?/2","doc":"Returns true if the field/relationship or path to field/relationship is being loaded.\n\nIt accepts an atom or a list of atoms, which is treated for as a \"path\", i.e:\n\n    Resource |> Ash.Query.load(friends: [enemies: [:score]]) |> Ash.Query.loading?([:friends, :enemies, :score])\n    iex> true\n\n    Resource |> Ash.Query.load(friends: [enemies: [:score]]) |> Ash.Query.loading?([:friends, :score])\n    iex> false\n\n    Resource |> Ash.Query.load(friends: [enemies: [:score]]) |> Ash.Query.loading?(:friends)\n    iex> true","ref":"Ash.Query.html#loading?/2"},{"type":"function","title":"Ash.Query.lock/2","doc":"Lock the query results.\n\nThis must be run while in a transaction, and is not supported by all data layers.","ref":"Ash.Query.html#lock/2"},{"type":"function","title":"Ash.Query.merge_query_load/3","doc":"Merges two query's load statements, for the purpose of handling calculation requirements.\n\nThis should only be used if you are writing a custom type that is loadable.\nSee the callback documentation for `c:Ash.Type.merge_load/4` for more.","ref":"Ash.Query.html#merge_query_load/3"},{"type":"function","title":"Ash.Query.new/2","doc":"Creates a new query for the given resource.\n\nThis is the starting point for building queries.  The query will automatically include the resource's base filter\nand default context.","ref":"Ash.Query.html#new/2"},{"type":"function","title":"Examples - Ash.Query.new/2","doc":"# Create a new query for a resource\n    iex> Ash.Query.new(MyApp.Post)\n    %Ash.Query{resource: MyApp.Post, ...}\n\n    # Create a query with options\n    iex> Ash.Query.new(MyApp.Post, domain: MyApp.Blog)\n    %Ash.Query{resource: MyApp.Post, domain: MyApp.Blog, ...}\n\n    # Pass an existing query (returns the query unchanged)\n    iex> query = Ash.Query.new(MyApp.Post)\n    iex> Ash.Query.new(query)\n    %Ash.Query{resource: MyApp.Post, ...}","ref":"Ash.Query.html#new/2-examples"},{"type":"function","title":"See also - Ash.Query.new/2","doc":"- `for_read/4` for creating queries bound to specific read actions\n- `filter/2` for adding filter conditions\n- `sort/3` for adding sort criteria\n- [Read Actions Guide](/documentation/topics/actions/read-actions.md) for understanding read operations\n- [Actions Guide](/documentation/topics/actions/actions.md) for general action concepts","ref":"Ash.Query.html#new/2-see-also"},{"type":"function","title":"Ash.Query.offset/2","doc":"Skips the first n records in the query results.\n\nThis function is often used for offset-based pagination, allowing you\nto skip a specified number of records from the beginning of the result set.\nOften used together with `limit/2` to implement pagination.","ref":"Ash.Query.html#offset/2"},{"type":"function","title":"Examples - Ash.Query.offset/2","doc":"# Skip the first 20 records\n    iex> MyApp.Post |> Ash.Query.offset(20)\n    %Ash.Query{offset: 20, ...}\n\n    # Remove existing offset\n    iex> query |> Ash.Query.offset(nil)\n    %Ash.Query{offset: 0, ...}\n\n    # Pagination example: page 3 with 10 items per page\n    iex> MyApp.Post\n    ...> |> Ash.Query.sort(:created_at)\n    ...> |> Ash.Query.offset(20)  # Skip first 20 (pages 1-2)\n    ...> |> Ash.Query.limit(10)   # Take next 10 (page 3)\n    %Ash.Query{offset: 20, limit: 10, ...}","ref":"Ash.Query.html#offset/2-examples"},{"type":"function","title":"See also - Ash.Query.offset/2","doc":"- `limit/2` for limiting the number of results\n- `page/2` for keyset pagination (more efficient for large datasets)\n- `sort/3` for ordering results before offsetting","ref":"Ash.Query.html#offset/2-see-also"},{"type":"function","title":"Ash.Query.page/2","doc":"Sets the pagination options of the query.\n\nThis function configures how results should be paginated when the query is executed.\nAsh supports both offset-based pagination (limit/offset) and keyset-based pagination\n(cursor-based), with keyset being more efficient for large datasets.","ref":"Ash.Query.html#page/2"},{"type":"function","title":"Examples - Ash.Query.page/2","doc":"# Offset-based pagination (page 2, 10 items per page)\n    iex> MyApp.Post\n    ...> |> Ash.Query.page(limit: 10, offset: 10)\n    %Ash.Query{page: [limit: 10, offset: 10], ...}\n\n    # Keyset pagination with before/after cursors\n    iex> MyApp.Post\n    ...> |> Ash.Query.sort(:created_at)\n    ...> |> Ash.Query.page(limit: 20, after: \"cursor_string\")\n    %Ash.Query{page: [limit: 20, after: \"cursor_string\"], ...}\n\n    # Disable pagination (return all results)\n    iex> MyApp.Post |> Ash.Query.page(nil)\n    %Ash.Query{page: nil, ...}\n\n    # Pagination with counting\n    iex> MyApp.Post |> Ash.Query.page(limit: 10, count: true)\n    %Ash.Query{page: [limit: 10, count: true], ...}","ref":"Ash.Query.html#page/2-examples"},{"type":"function","title":"Pagination Types - Ash.Query.page/2","doc":"","ref":"Ash.Query.html#page/2-pagination-types"},{"type":"function","title":"Limit/offset pagination - Ash.Query.page/2","doc":"* `:offset` (`t:non_neg_integer/0`) - The number of records to skip from the beginning of the query\n\n* `:limit` (`t:pos_integer/0`) - The number of records to include in the page\n\n* `:filter` (`t:term/0`) - A filter to apply for pagination purposes, that should not be considered in the full count.  \n  This is used by the liveview paginator to only fetch the records that were *already* on the\n  page when refreshing data, to avoid pages jittering.\n\n* `:count` (`t:boolean/0`) - Whether or not to return the page with a full count of all records","ref":"Ash.Query.html#page/2-limit-offset-pagination"},{"type":"function","title":"Keyset pagination - Ash.Query.page/2","doc":"* `:before` (`t:String.t/0`) - Get records that appear before the provided keyset (mutually exclusive with `after`)\n\n* `:after` (`t:String.t/0`) - Get records that appear after the provided keyset (mutually exclusive with `before`)\n\n* `:limit` (`t:pos_integer/0`) - How many records to include in the page\n\n* `:filter` (`t:term/0`) - See the `filter` option for offset pagination, this behaves the same.\n\n* `:count` (`t:boolean/0`) - Whether or not to return the page with a full count of all records","ref":"Ash.Query.html#page/2-keyset-pagination"},{"type":"function","title":"See also - Ash.Query.page/2","doc":"- `limit/2` and `offset/2` for simple pagination without page metadata\n- `sort/3` for ordering results (required for keyset pagination)\n- `Ash.read/2` for executing paginated queries","ref":"Ash.Query.html#page/2-see-also"},{"type":"function","title":"Ash.Query.put_context/3","doc":"Sets a specific context key to a specific value.\n\nContext is used to pass additional information through the query pipeline\nthat can be accessed by preparations, calculations, and other query logic.\nThis function adds or updates a single key in the query's context map.","ref":"Ash.Query.html#put_context/3"},{"type":"function","title":"Examples - Ash.Query.put_context/3","doc":"# Add actor information to context\n    iex> query = MyApp.Post |> Ash.Query.put_context(:actor, current_user)\n    %Ash.Query{context: %{actor: %User{...}}, ...}\n\n    # Add custom metadata for preparations\n    iex> query = MyApp.Post |> Ash.Query.put_context(:source, \"api\")\n    %Ash.Query{context: %{source: \"api\"}, ...}\n\n    # Chain multiple context additions\n    iex> MyApp.Post\n    ...> |> Ash.Query.put_context(:tenant, \"org_123\")\n    ...> |> Ash.Query.put_context(:locale, \"en_US\")\n    %Ash.Query{context: %{tenant: \"org_123\", locale: \"en_US\"}, ...}","ref":"Ash.Query.html#put_context/3-examples"},{"type":"function","title":"See also - Ash.Query.put_context/3","doc":"- `set_context/2` for setting the entire context map\n- `for_read/4` for passing context when creating queries\n- Preparations and calculations can access context for custom logic","ref":"Ash.Query.html#put_context/3-see-also"},{"type":"function","title":"Ash.Query.select/3","doc":"Ensure that only the specified *attributes* are present in the results.\n\nThe first call to `select/2` will replace the default behavior of selecting\nall attributes. Subsequent calls to `select/2` will combine the provided\nfields unless the `replace?` option is provided with a value of `true`.\n\nIf a field has been deselected, selecting it again will override that (because a single list of fields is tracked for selection)\n\nPrimary key attributes are always selected and cannot be deselected.\n\nWhen attempting to load a relationship (or manage it with `Ash.Changeset.manage_relationship/3`),\nif the source field is not selected on the query/provided data an error will be produced. If loading\na relationship with a query, an error is produced if the query does not select the destination field\nof the relationship.\n\nUse `ensure_selected/2` if you wish to make sure a field has been selected, without deselecting any other fields.","ref":"Ash.Query.html#select/3"},{"type":"function","title":"Examples - Ash.Query.select/3","doc":"# Select specific attributes\n    iex> MyApp.Post |> Ash.Query.select([:title, :content])\n    %Ash.Query{select: [:id, :title, :content], ...}\n\n    # Select additional attributes (combines with existing selection)\n    iex> MyApp.Post\n    ...> |> Ash.Query.select([:title])\n    ...> |> Ash.Query.select([:content])\n    %Ash.Query{select: [:id, :title, :content], ...}\n\n    # Replace existing selection\n    iex> MyApp.Post\n    ...> |> Ash.Query.select([:title])\n    ...> |> Ash.Query.select([:content], replace?: true)\n    %Ash.Query{select: [:id, :content], ...}","ref":"Ash.Query.html#select/3-examples"},{"type":"function","title":"See also - Ash.Query.select/3","doc":"- `ensure_selected/2` for adding fields without deselecting others\n- `deselect/2` for removing specific fields from selection\n- `load/3` for loading relationships and calculations","ref":"Ash.Query.html#select/3-see-also"},{"type":"function","title":"Ash.Query.selecting?/2","doc":"Checks if a specific field is currently selected in the query.\n\nReturns `true` if the field will be included in the query results, either\nbecause it's explicitly selected, it's selected by default, or it's a\nprimary key field (which are always selected).","ref":"Ash.Query.html#selecting?/2"},{"type":"function","title":"Examples - Ash.Query.selecting?/2","doc":"# Check selection when no explicit select is set (uses defaults)\n    iex> query = MyApp.Post |> Ash.Query.new()\n    iex> Ash.Query.selecting?(query, :title)\n    true\n\n    # Check selection with explicit select\n    iex> query = MyApp.Post |> Ash.Query.select([:title, :content])\n    iex> Ash.Query.selecting?(query, :title)\n    true\n    iex> Ash.Query.selecting?(query, :metadata)\n    false\n\n    # Primary key fields are always selected\n    iex> query = MyApp.Post |> Ash.Query.select([:title])\n    iex> Ash.Query.selecting?(query, :id)  # assuming :id is primary key\n    true","ref":"Ash.Query.html#selecting?/2-examples"},{"type":"function","title":"See also - Ash.Query.selecting?/2","doc":"- `select/3` for controlling field selection\n- `ensure_selected/2` for adding fields to selection\n- `load/3` for loading relationships that may require specific fields","ref":"Ash.Query.html#selecting?/2-see-also"},{"type":"function","title":"Ash.Query.set_argument/3","doc":"Adds an argument to the query.\n\nArguments are used by action logic such as preparations, filters, and other\nquery modifications. They become available in filter templates and can be\nreferenced in action configurations. Setting an argument after a query has\nbeen validated for an action will result in an error.","ref":"Ash.Query.html#set_argument/3"},{"type":"function","title":"Examples - Ash.Query.set_argument/3","doc":"# Set an argument for use in action filters\n    iex> query = Ash.Query.new(MyApp.Post)\n    iex> Ash.Query.set_argument(query, :author_id, 123)\n    %Ash.Query{arguments: %{author_id: 123}, ...}\n\n    # Set multiple arguments by chaining\n    iex> MyApp.Post\n    ...> |> Ash.Query.set_argument(:category, \"tech\")\n    ...> |> Ash.Query.set_argument(:published, true)\n    %Ash.Query{arguments: %{category: \"tech\", published: true}, ...}\n\n    # Arguments are used in action preparations and filters\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.for_read(:by_author, %{author_id: 123})\n    ...> |> Ash.Query.set_argument(:include_drafts, false)\n    %Ash.Query{arguments: %{author_id: 123, include_drafts: false}, ...}","ref":"Ash.Query.html#set_argument/3-examples"},{"type":"function","title":"See also - Ash.Query.set_argument/3","doc":"- `get_argument/2` for retrieving argument values\n- `fetch_argument/2` for safe argument retrieval\n- `for_read/4` for creating queries with initial arguments","ref":"Ash.Query.html#set_argument/3-see-also"},{"type":"function","title":"Ash.Query.set_arguments/2","doc":"Merge a map of arguments to the arguments list","ref":"Ash.Query.html#set_arguments/2"},{"type":"function","title":"Ash.Query.set_context/2","doc":"Merge a map of values into the query context","ref":"Ash.Query.html#set_context/2"},{"type":"function","title":"Ash.Query.set_domain/2","doc":"Set the query's domain, and any loaded query's domain","ref":"Ash.Query.html#set_domain/2"},{"type":"function","title":"Ash.Query.set_result/2","doc":"Set the result of the action. This will prevent running the underlying datalayer behavior","ref":"Ash.Query.html#set_result/2"},{"type":"function","title":"Ash.Query.set_tenant/2","doc":"Sets the tenant for the query.\n\nIn multitenant applications, this function configures which tenant's data\nthe query should operate on. The tenant value is used to filter data\nand ensure proper data isolation between tenants.","ref":"Ash.Query.html#set_tenant/2"},{"type":"function","title":"Examples - Ash.Query.set_tenant/2","doc":"# Set tenant using a string identifier\n    iex> MyApp.Post |> Ash.Query.set_tenant(\"org_123\")\n    %Ash.Query{tenant: \"org_123\", ...}\n\n    # Set tenant using a struct that implements Ash.ToTenant\n    iex> org = %MyApp.Organization{id: 456}\n    iex> MyApp.Post |> Ash.Query.set_tenant(org)\n    %Ash.Query{tenant: %MyApp.Organization{id: 456}, ...}\n\n    # Use with other query functions\n    iex> MyApp.Post\n    ...> |> Ash.Query.set_tenant(\"org_123\")\n    ...> |> Ash.Query.filter(published: true)\n    %Ash.Query{tenant: \"org_123\", ...}","ref":"Ash.Query.html#set_tenant/2-examples"},{"type":"function","title":"See also - Ash.Query.set_tenant/2","doc":"- `for_read/4` for setting tenant when creating queries\n- `Ash.ToTenant` protocol for custom tenant conversion\n- `put_context/3` for adding tenant to query context","ref":"Ash.Query.html#set_tenant/2-see-also"},{"type":"function","title":"Ash.Query.sort/3","doc":"Sort the results based on attributes, aggregates or calculations.","ref":"Ash.Query.html#sort/3"},{"type":"function","title":"Format - Ash.Query.sort/3","doc":"Your sort can be an atom, list of atoms, a keyword list, or a string. When an order is not specified,\n`:asc` is the default. See Sort Orders below for more on the available orders.\n\n```elixir\n# sort by name ascending\nAsh.Query.sort(query, :name)\n\n# sort by name descending\nAsh.Query.sort(query, name: :desc)\n\n# sort by name descending with nils at the end\nAsh.Query.sort(query, name: :desc_nils_last)\n\n# sort by name descending, and title ascending\nAsh.Query.sort(query, name: :desc, title: :asc)\n\n# sort by name ascending\nAsh.Query.sort(query, \"name\")\n\n# sort by name descending, and title ascending\nAsh.Query.sort(query, \"-name,title\")\n\n# sort by name descending with nils at the end\nAsh.Query.sort(query, \"--name\")\n```","ref":"Ash.Query.html#sort/3-format"},{"type":"function","title":"Related Fields - Ash.Query.sort/3","doc":"You can refer to related fields using the shorthand of `\"rel1.rel2.field\"`. For example:\n\n```elixir\n# sort by the username of the comment's author.\nAsh.Query.sort(query, \"comment.author.username\")\n\n# Use as an atom for keyword lists\nAsh.Query.sort(query, \"comment.author.username\": :desc)\n```","ref":"Ash.Query.html#sort/3-related-fields"},{"type":"function","title":"Expression Sorts - Ash.Query.sort/3","doc":"You can use the `Ash.Expr.calc/2` macro to sort on expressions:\n\n```elixir\nimport Ash.Expr\n\n# Sort on an expression\nAsh.Query.sort(query, calc(count(friends), :desc))\n\n# Specify a type (required in some cases when we can't determine a type)\nAsh.Query.sort(query, [{calc(fragment(\"some_sql(?)\", field, type: :string), :desc}])\n```","ref":"Ash.Query.html#sort/3-expression-sorts"},{"type":"function","title":"Sort Strings - Ash.Query.sort/3","doc":"A comma separated list of fields to sort on, each with an optional prefix.\n\nThe prefixes are:\n\n* \"+\" - Same as no prefix. Sorts `:asc`.\n* \"++\" - Sorts `:asc_nils_first`\n* \"-\" - Sorts `:desc`\n* \"--\" - Sorts `:desc_nils_last`\n\nFor example\n\n    \"foo,-bar,++baz,--buz\"","ref":"Ash.Query.html#sort/3-sort-strings"},{"type":"function","title":"A list of sort strings - Ash.Query.sort/3","doc":"Same prefix rules as above, but provided as a list.\n\nFor example:\n\n    [\"foo\", \"-bar\", \"++baz\", \"--buz\"]","ref":"Ash.Query.html#sort/3-a-list-of-sort-strings"},{"type":"function","title":"Calculations - Ash.Query.sort/3","doc":"Calculation inputs can be provided by providing a map. To provide both inputs and an order,\nuse a tuple with the first element being the inputs, and the second element being the order.\n\n```elixir\nAsh.Query.sort(query, full_name: %{separator: \" \"})\n\nAsh.Query.sort(query, full_name: {%{separator: \" \"}, :asc})\n```","ref":"Ash.Query.html#sort/3-calculations"},{"type":"function","title":"Sort Orders - Ash.Query.sort/3","doc":"The available orders are:\n\n- `:asc` - Sort values ascending, with lowest first and highest last, and `nil` values at the end\n- `:desc` - Sort values descending, with highest first and lowest last, and `nil` values at the beginning\n- `:asc_nils_first` - Sort values ascending, with lowest first and highest last, and `nil` values at the beginning\n- `:desc_nils_last` - Sort values descending, with highest first and lowest last, and `nil` values at the end","ref":"Ash.Query.html#sort/3-sort-orders"},{"type":"function","title":"Examples - Ash.Query.sort/3","doc":"```elixir\nAsh.Query.sort(query, [:foo, :bar])\n\nAsh.Query.sort(query, [:foo, bar: :desc])\n\nAsh.Query.sort(query, [foo: :desc, bar: :asc])\n```\n\nSee the guide on calculations for more.","ref":"Ash.Query.html#sort/3-examples"},{"type":"function","title":"Options - Ash.Query.sort/3","doc":"- `prepend?` - set to `true` to put your sort at the front of the list of a sort is already specified","ref":"Ash.Query.html#sort/3-options"},{"type":"function","title":"Ash.Query.sort_input/3","doc":"Attach a sort statement to the query labelled as user input.\n\nSorts added as user input (or filters constructed with `Ash.Filter.parse_input`)\nwill honor any field policies on resources by replacing any references to the field\nwith `nil` in cases where the actor should not be able to see the given field.\n\nSee `Ash.Query.sort/3` for more information on accepted formats.","ref":"Ash.Query.html#sort_input/3"},{"type":"macro","title":"Ash.Query.subset_of/2","doc":"Determines if the provided expression would return data that is a suprset of the data returned by the filter on the query.\n\nThis uses the satisfiability solver that is used when solving for policy authorizations. In complex scenarios, or when using\ncustom database expressions, (like fragments in ash_postgres), this function may return `:maybe`. Use `subset_of?` to always return\na boolean.","ref":"Ash.Query.html#subset_of/2"},{"type":"macro","title":"Ash.Query.subset_of?/2","doc":"Same as `subset_of/2` but always returns a boolean. `:maybe` returns `false`.","ref":"Ash.Query.html#subset_of?/2"},{"type":"macro","title":"Ash.Query.superset_of/2","doc":"Determines if the provided expression would return data that is a subset of the data returned by the filter on the query.\n\nThis uses the satisfiability solver that is used when solving for policy authorizations. In complex scenarios, or when using\ncustom database expressions, (like fragments in ash_postgres), this function may return `:maybe`. Use `supserset_of?` to always return\na boolean.","ref":"Ash.Query.html#superset_of/2"},{"type":"macro","title":"Ash.Query.superset_of?/2","doc":"Same as `superset_of/2` but always returns a boolean. `:maybe` returns `false`.","ref":"Ash.Query.html#superset_of?/2"},{"type":"type","title":"Ash.Query.t/0","doc":"A query struct for reading data from a resource.\n\nContains all the configuration needed to read data including filters, sorting,\npagination, field selection, and relationship loading. Built incrementally\nthrough functions like `filter/2`, `sort/2`, `load/2`, etc.","ref":"Ash.Query.html#t:t/0"},{"type":"function","title":"Ash.Query.timeout/2","doc":"Set a timeout for the query.\n\nFor more information, see the [timeouts guide](/documentation/topics/advanced/timeouts.md)","ref":"Ash.Query.html#timeout/2"},{"type":"function","title":"Ash.Query.unload/2","doc":"Removes a field from the list of fields to load","ref":"Ash.Query.html#unload/2"},{"type":"function","title":"Ash.Query.unset/2","doc":"Removes specified keys from the query, resetting them to their default values.\n\nThis function allows you to \"unset\" or reset parts of a query back to their\ninitial state. Useful when you want to remove filters, sorts, loads, or other\nquery modifications while keeping the rest of the query intact.","ref":"Ash.Query.html#unset/2"},{"type":"function","title":"Examples - Ash.Query.unset/2","doc":"# Remove multiple query aspects at once\n    iex> query = MyApp.Post\n    ...> |> Ash.Query.filter(published: true)\n    ...> |> Ash.Query.sort(:created_at)\n    ...> |> Ash.Query.limit(10)\n    iex> Ash.Query.unset(query, [:filter, :sort, :limit])\n    %Ash.Query{filter: nil, sort: [], limit: nil, ...}\n\n    # Remove just the sort from a query\n    iex> query = MyApp.Post |> Ash.Query.sort([:title, :created_at])\n    iex> Ash.Query.unset(query, :sort)\n    %Ash.Query{sort: [], ...}\n\n    # Remove load statements\n    iex> query = MyApp.Post |> Ash.Query.load([:author, :comments])\n    iex> Ash.Query.unset(query, :load)\n    %Ash.Query{load: [], ...}\n\n    # Reset pagination settings\n    iex> query = MyApp.Post |> Ash.Query.limit(20) |> Ash.Query.offset(10)\n    iex> Ash.Query.unset(query, [:limit, :offset])\n    %Ash.Query{limit: nil, offset: 0, ...}","ref":"Ash.Query.html#unset/2-examples"},{"type":"function","title":"See also - Ash.Query.unset/2","doc":"- `new/2` for creating fresh queries\n- `select/3`, `filter/2`, `sort/3` for building queries","ref":"Ash.Query.html#unset/2-see-also"},{"type":"module","title":"Ash.CodeInterface","doc":"Used to define the functions of a code interface for a resource.","ref":"Ash.CodeInterface.html"},{"type":"macro","title":"Ash.CodeInterface.define_interface/3","doc":"Defines the code interface for a given resource + domain combination in the current module. For example:\n\n```elixir\ndefmodule MyApp.Accounting do\n  require Ash.CodeInterface\n\n  Ash.CodeInterface.define_interface(MyApp.Accounting, MyApp.Accounting.Transaction)\n  Ash.CodeInterface.define_interface(MyApp.Accounting, MyApp.Accounting.Account)\n  Ash.CodeInterface.define_interface(MyApp.Accounting, MyApp.Accounting.Invoice)\nend\n```","ref":"Ash.CodeInterface.html#define_interface/3"},{"type":"function","title":"Ash.CodeInterface.params_and_opts/3","doc":"See `params_and_opts/2`.\n\nAdds a post process function that can takes the opts and can further process,\nvalidate, or transform them.","ref":"Ash.CodeInterface.html#params_and_opts/3"},{"type":"behaviour","title":"Ash.Domain","doc":"A domain allows you to interact with your resources, and holds domain-wide configuration.\n\nFor example, the json domain extension adds a domain extension that lets you toggle authorization on/off\nfor all resources in a given domain. You include resources in your domain like so:\n\n```elixir\ndefmodule MyApp.MyDomain do\n  use Ash.Domain\n\n  resources do\n    resource OneResource\n    resource SecondResource\n  end\nend\n```","ref":"Ash.Domain.html"},{"type":"behaviour","title":"Options - Ash.Domain","doc":"* `:validate_config_inclusion?` (`t:boolean/0`) - Whether or not to validate that this domain is included in the configuration. The default value is `true`.\n\n* `:backwards_compatible_interface?` (`t:boolean/0`) - Whether or not to include the 2.0 backwards compatible interface, which includes all of the interaction functions which are now defined on the `Ash` module The default value is `true`.\n\n* `:extensions` (list of module that adopts `Spark.Dsl.Extension`) - A list of DSL extensions to add to the `Spark.Dsl`\n\n* `:authorizers` (one or a list of module that adopts `Ash.Authorizer`) - authorizers extensions to add to the `Spark.Dsl` The default value is `[]`.\n\n* `:otp_app` (`t:atom/0`) - The otp_app to use for any application configurable options\n\n* `:fragments` (list of `t:module/0`) - Fragments to include in the `Spark.Dsl`. See the fragments guide for more.","ref":"Ash.Domain.html#module-options"},{"type":"type","title":"Ash.Domain.t/0","doc":"","ref":"Ash.Domain.html#t:t/0"},{"type":"behaviour","title":"Ash.Notifier","doc":"A notifier is an extension that receives various events","ref":"Ash.Notifier.html"},{"type":"function","title":"Ash.Notifier.notify/1","doc":"Sends any notifications that can be sent, and returns the rest.\n\nA notification can only be sent if you are not currently in a transaction\nfor the resource in question.","ref":"Ash.Notifier.html#notify/1"},{"type":"callback","title":"Ash.Notifier.notify/1","doc":"","ref":"Ash.Notifier.html#c:notify/1"},{"type":"callback","title":"Ash.Notifier.requires_original_data?/2","doc":"","ref":"Ash.Notifier.html#c:requires_original_data?/2"},{"type":"module","title":"Ash.Notifier.Notification","doc":"Represents a notification that will be handled by a resource's notifiers\n\nSet the `for` key to a notifier or a list of notifiers to route the notification\nto them. This allows you to produce notifications inside of a `change` module\nand target specific notifiers with them.\n\n`metadata` is freeform data to be set however you want. `resource`, `action`, `data`,\n`changeset` and `actor` are all set by default based on the details of the action, so\nthey can be omitted.\n\nWhen creating a notification, a resource is required to ensure that the notification isn't\nsent until the current transaction for that resource is closed. If you don't need this\nbehavior you can explicitly supply `nil` for the resource. If you supply `nil` for the resource,\nhowever, you must manually set the `for` option, e.g: `for: Notifier` or `for: [Notifier1, Notifier2]`","ref":"Ash.Notifier.Notification.html"},{"type":"function","title":"Ash.Notifier.Notification.new/2","doc":"","ref":"Ash.Notifier.Notification.html#new/2"},{"type":"type","title":"Ash.Notifier.Notification.t/0","doc":"","ref":"Ash.Notifier.Notification.html#t:t/0"},{"type":"behaviour","title":"Ash.Resource.Calculation","doc":"The behaviour for defining a module calculation, and the struct for storing a defined calculation.","ref":"Ash.Resource.Calculation.html"},{"type":"callback","title":"Ash.Resource.Calculation.calculate/3","doc":"","ref":"Ash.Resource.Calculation.html#c:calculate/3"},{"type":"callback","title":"Ash.Resource.Calculation.describe/1","doc":"","ref":"Ash.Resource.Calculation.html#c:describe/1"},{"type":"callback","title":"Ash.Resource.Calculation.expression/2","doc":"","ref":"Ash.Resource.Calculation.html#c:expression/2"},{"type":"callback","title":"Ash.Resource.Calculation.has_expression?/0","doc":"","ref":"Ash.Resource.Calculation.html#c:has_expression?/0"},{"type":"callback","title":"Ash.Resource.Calculation.init/1","doc":"","ref":"Ash.Resource.Calculation.html#c:init/1"},{"type":"function","title":"Ash.Resource.Calculation.init/2","doc":"","ref":"Ash.Resource.Calculation.html#init/2"},{"type":"callback","title":"Ash.Resource.Calculation.load/3","doc":"","ref":"Ash.Resource.Calculation.html#c:load/3"},{"type":"type","title":"Ash.Resource.Calculation.opts/0","doc":"","ref":"Ash.Resource.Calculation.html#t:opts/0"},{"type":"type","title":"Ash.Resource.Calculation.ref/0","doc":"","ref":"Ash.Resource.Calculation.html#t:ref/0"},{"type":"function","title":"Ash.Resource.Calculation.schema/0","doc":"","ref":"Ash.Resource.Calculation.html#schema/0"},{"type":"callback","title":"Ash.Resource.Calculation.strict_loads?/0","doc":"","ref":"Ash.Resource.Calculation.html#c:strict_loads?/0"},{"type":"type","title":"Ash.Resource.Calculation.t/0","doc":"","ref":"Ash.Resource.Calculation.html#t:t/0"},{"type":"module","title":"Ash.Resource.Calculation.Builtins","doc":"Built in calculations that are automatically imported in the calculations section","ref":"Ash.Resource.Calculation.Builtins.html"},{"type":"function","title":"Ash.Resource.Calculation.Builtins.concat/2","doc":"An example concatenation calculation, that accepts the delimiter as an argument","ref":"Ash.Resource.Calculation.Builtins.html#concat/2"},{"type":"function","title":"Examples - Ash.Resource.Calculation.Builtins.concat/2","doc":"calculate :full_name, :string, concat([:first_name, :last_name], \" \")","ref":"Ash.Resource.Calculation.Builtins.html#concat/2-examples"},{"type":"behaviour","title":"Ash.Resource.ManualCreate","doc":"A module to implement manual create actions.","ref":"Ash.Resource.ManualCreate.html"},{"type":"callback","title":"Ash.Resource.ManualCreate.bulk_create/3","doc":"","ref":"Ash.Resource.ManualCreate.html#c:bulk_create/3"},{"type":"callback","title":"Ash.Resource.ManualCreate.create/3","doc":"","ref":"Ash.Resource.ManualCreate.html#c:create/3"},{"type":"behaviour","title":"Ash.Resource.ManualDestroy","doc":"A module to implement manual destroy actions.\n\nNote that in the returns of these functions you must return the destroyed record or records.","ref":"Ash.Resource.ManualDestroy.html"},{"type":"callback","title":"Ash.Resource.ManualDestroy.bulk_destroy/3","doc":"","ref":"Ash.Resource.ManualDestroy.html#c:bulk_destroy/3"},{"type":"callback","title":"Ash.Resource.ManualDestroy.destroy/3","doc":"","ref":"Ash.Resource.ManualDestroy.html#c:destroy/3"},{"type":"behaviour","title":"Ash.Resource.ManualRead","doc":"A module to implement manual read actions.","ref":"Ash.Resource.ManualRead.html"},{"type":"type","title":"Ash.Resource.ManualRead.context/0","doc":"","ref":"Ash.Resource.ManualRead.html#t:context/0"},{"type":"type","title":"Ash.Resource.ManualRead.extra_info/0","doc":"","ref":"Ash.Resource.ManualRead.html#t:extra_info/0"},{"type":"callback","title":"Ash.Resource.ManualRead.load_relationships/5","doc":"","ref":"Ash.Resource.ManualRead.html#c:load_relationships/5"},{"type":"callback","title":"Ash.Resource.ManualRead.read/4","doc":"","ref":"Ash.Resource.ManualRead.html#c:read/4"},{"type":"behaviour","title":"Ash.Resource.ManualRelationship","doc":"A module to implement manual relationships.","ref":"Ash.Resource.ManualRelationship.html"},{"type":"callback","title":"Ash.Resource.ManualRelationship.load/3","doc":"","ref":"Ash.Resource.ManualRelationship.html#c:load/3"},{"type":"callback","title":"Ash.Resource.ManualRelationship.select/1","doc":"","ref":"Ash.Resource.ManualRelationship.html#c:select/1"},{"type":"behaviour","title":"Ash.Resource.ManualUpdate","doc":"A module to implement manual update actions.","ref":"Ash.Resource.ManualUpdate.html"},{"type":"callback","title":"Ash.Resource.ManualUpdate.bulk_update/3","doc":"","ref":"Ash.Resource.ManualUpdate.html#c:bulk_update/3"},{"type":"callback","title":"Ash.Resource.ManualUpdate.update/3","doc":"","ref":"Ash.Resource.ManualUpdate.html#c:update/3"},{"type":"module","title":"Ash.Query.Aggregate","doc":"Represents an aggregated association value","ref":"Ash.Query.Aggregate.html"},{"type":"function","title":"Ash.Query.Aggregate.default_value/1","doc":"","ref":"Ash.Query.Aggregate.html#default_value/1"},{"type":"type","title":"Ash.Query.Aggregate.kind/0","doc":"","ref":"Ash.Query.Aggregate.html#t:kind/0"},{"type":"function","title":"Ash.Query.Aggregate.new/4","doc":"Create a new aggregate, used with `Query.aggregate` or `Ash.aggregate`","ref":"Ash.Query.Aggregate.html#new/4"},{"type":"function","title":"Options - Ash.Query.Aggregate.new/4","doc":"* `:path` (list of `t:atom/0`) - The relationship path to aggregate over. Only used when adding aggregates to a query. The default value is `[]`.\n\n* `:query` (`t:term/0`) - A base query to use for the aggregate, or a keyword list to be passed to `Ash.Query.build/2`\n\n* `:field` - The field to use for the aggregate. Not necessary for all aggregate types.\n\n* `:expr` (`t:term/0`) - An expression to aggregate, cannot be used with `field`.\n\n* `:expr_type` - The type of the expression, required if `expr` is used.\n\n* `:arguments` (`t:map/0`) - Arguments to pass to the field, if field is a calculation.\n\n* `:default` (`t:term/0`) - A default value to use for the aggregate if it returns `nil`.\n\n* `:filterable?` (`t:boolean/0`) - Whether or not this aggregate may be used in filters. The default value is `true`.\n\n* `:sortable?` (`t:boolean/0`) - Whether or not this aggregate may be used in sorts. The default value is `true`.\n\n* `:type` (`t:term/0`) - A type to use for the aggregate.\n\n* `:constraints` (`t:term/0`) - Type constraints to use for the aggregate. The default value is `[]`.\n\n* `:implementation` (`t:term/0`) - The implementation for any custom aggregates.\n\n* `:read_action` (`t:atom/0`) - The read action to use for the aggregate, defaults to the primary read action.\n\n* `:uniq?` (`t:boolean/0`) - Whether or not to only consider unique values. Only relevant for `count` and `list` aggregates. The default value is `false`.\n\n* `:include_nil?` (`t:boolean/0`) - Whether or not to include `nil` values in the aggregate. Only relevant for `list` and `first` aggregates. The default value is `false`.\n\n* `:join_filters` (map of one or a list of `t:atom/0` keys and `t:term/0` values) - A map of relationship paths (an atom or list of atoms), to an expression to apply when fetching the aggregate data. See the aggregates guide for more. The default value is `%{}`.\n\n* `:sensitive?` (`t:boolean/0`) - Whether or not references to this aggregate will be considered sensitive The default value is `false`.\n\n* `:tenant` (`t:term/0`) - The tenant to use for the aggregate, if applicable. The default value is `nil`.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the aggregate query should authorize based on the target action.  \n  See `d:Ash.Resource.Dsl.aggregates|count` for more information. The default value is `true`.","ref":"Ash.Query.Aggregate.html#new/4-options"},{"type":"function","title":"Ash.Query.Aggregate.new!/4","doc":"Create a new aggregate, used with `Query.aggregate` or `Ash.aggregate`, raising on errors.\n\nSee `new/4` for more information.","ref":"Ash.Query.Aggregate.html#new!/4"},{"type":"function","title":"Options - Ash.Query.Aggregate.new!/4","doc":"* `:path` (list of `t:atom/0`) - The relationship path to aggregate over. Only used when adding aggregates to a query. The default value is `[]`.\n\n* `:query` (`t:term/0`) - A base query to use for the aggregate, or a keyword list to be passed to `Ash.Query.build/2`\n\n* `:field` - The field to use for the aggregate. Not necessary for all aggregate types.\n\n* `:expr` (`t:term/0`) - An expression to aggregate, cannot be used with `field`.\n\n* `:expr_type` - The type of the expression, required if `expr` is used.\n\n* `:arguments` (`t:map/0`) - Arguments to pass to the field, if field is a calculation.\n\n* `:default` (`t:term/0`) - A default value to use for the aggregate if it returns `nil`.\n\n* `:filterable?` (`t:boolean/0`) - Whether or not this aggregate may be used in filters. The default value is `true`.\n\n* `:sortable?` (`t:boolean/0`) - Whether or not this aggregate may be used in sorts. The default value is `true`.\n\n* `:type` (`t:term/0`) - A type to use for the aggregate.\n\n* `:constraints` (`t:term/0`) - Type constraints to use for the aggregate. The default value is `[]`.\n\n* `:implementation` (`t:term/0`) - The implementation for any custom aggregates.\n\n* `:read_action` (`t:atom/0`) - The read action to use for the aggregate, defaults to the primary read action.\n\n* `:uniq?` (`t:boolean/0`) - Whether or not to only consider unique values. Only relevant for `count` and `list` aggregates. The default value is `false`.\n\n* `:include_nil?` (`t:boolean/0`) - Whether or not to include `nil` values in the aggregate. Only relevant for `list` and `first` aggregates. The default value is `false`.\n\n* `:join_filters` (map of one or a list of `t:atom/0` keys and `t:term/0` values) - A map of relationship paths (an atom or list of atoms), to an expression to apply when fetching the aggregate data. See the aggregates guide for more. The default value is `%{}`.\n\n* `:sensitive?` (`t:boolean/0`) - Whether or not references to this aggregate will be considered sensitive The default value is `false`.\n\n* `:tenant` (`t:term/0`) - The tenant to use for the aggregate, if applicable. The default value is `nil`.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the aggregate query should authorize based on the target action.  \n  See `d:Ash.Resource.Dsl.aggregates|count` for more information. The default value is `true`.","ref":"Ash.Query.Aggregate.html#new!/4-options"},{"type":"type","title":"Ash.Query.Aggregate.t/0","doc":"","ref":"Ash.Query.Aggregate.html#t:t/0"},{"type":"module","title":"Ash.Query.Calculation","doc":"Represents a calculated attribute requested on a query","ref":"Ash.Query.Calculation.html"},{"type":"function","title":"Ash.Query.Calculation.from_resource_calculation/3","doc":"Creates a new query calculation from a resource calculation.","ref":"Ash.Query.Calculation.html#from_resource_calculation/3"},{"type":"function","title":"Options - Ash.Query.Calculation.from_resource_calculation/3","doc":"* `:args` (`t:map/0`) - Arguments to pass to the calculation The default value is `%{}`.\n\n* `:source_context` (`t:map/0`) - Context from the source query or changeset. The default value is `%{}`.","ref":"Ash.Query.Calculation.html#from_resource_calculation/3-options"},{"type":"function","title":"Ash.Query.Calculation.from_resource_calculation!/3","doc":"Creates a new query calculation from a resource calculation, raising any errors.\n\nSee `from_resource_calculation/3` for more.","ref":"Ash.Query.Calculation.html#from_resource_calculation!/3"},{"type":"function","title":"Ash.Query.Calculation.new/6","doc":"Creates a new query calculation.","ref":"Ash.Query.Calculation.html#new/6"},{"type":"function","title":"Options - Ash.Query.Calculation.new/6","doc":"* `:arguments` (`t:map/0`) - Arguments to pass to the calculation The default value is `%{}`.\n\n* `:async?` (`t:boolean/0`) - Whether or not this calculation should be run asynchronously The default value is `false`.\n\n* `:filterable?` (`t:boolean/0`) - Whether or not this calculation can be filtered on The default value is `true`.\n\n* `:sortable?` (`t:boolean/0`) - Whether or not this calculation can be sorted on The default value is `true`.\n\n* `:sensitive?` (`t:boolean/0`) - Whether or not references to this calculation will be considered sensitive The default value is `false`.\n\n* `:load` (`t:term/0`) - Loads that are required for the calculation.\n\n* `:actor` (`t:term/0`) - The actor performing the calculation.\n\n* `:tenant` (`t:term/0`) - The tenant performing the calculation.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not authorization is being performed\n\n* `:tracer` (`t:term/0`) - The tracer or tracers used in the calculation.\n\n* `:source_context` (`t:map/0`) - Context from the source query or changeset. The default value is `%{}`.","ref":"Ash.Query.Calculation.html#new/6-options"},{"type":"type","title":"Ash.Query.Calculation.t/0","doc":"","ref":"Ash.Query.Calculation.html#t:t/0"},{"type":"behaviour","title":"Ash.Resource.Preparation","doc":"The behaviour for an action-specific query preparation.\n\n`c:init/1` is defined automatically by `use Ash.Resource.Preparation`, but can be implemented if you want to validate/transform any\noptions passed to the module.\n\nThe main function is `c:prepare/3`. It takes the query, any options that were provided\nwhen this preparation was configured on a resource, and the context, which currently only has\nthe actor.\n\nTo access any query arguments from within a preparation, make sure you are using `Ash.Query.get_argument/2`\nas the argument keys may be strings or atoms.","ref":"Ash.Resource.Preparation.html"},{"type":"callback","title":"Ash.Resource.Preparation.init/1","doc":"","ref":"Ash.Resource.Preparation.html#c:init/1"},{"type":"callback","title":"Ash.Resource.Preparation.prepare/3","doc":"","ref":"Ash.Resource.Preparation.html#c:prepare/3"},{"type":"type","title":"Ash.Resource.Preparation.ref/0","doc":"","ref":"Ash.Resource.Preparation.html#t:ref/0"},{"type":"type","title":"Ash.Resource.Preparation.t/0","doc":"","ref":"Ash.Resource.Preparation.html#t:t/0"},{"type":"module","title":"Ash.Resource.Preparation.Builtins","doc":"Builtin query preparations","ref":"Ash.Resource.Preparation.Builtins.html"},{"type":"macro","title":"Ash.Resource.Preparation.Builtins.after_action/1","doc":"Directly attach an `after_action` function to the query.\n\nThis function will be called by `Ash.Query.after_action/2`,\nwith an additional `context` argument.","ref":"Ash.Resource.Preparation.Builtins.html#after_action/1"},{"type":"macro","title":"Example - Ash.Resource.Preparation.Builtins.after_action/1","doc":"prepare after_action(fn query, records, _context ->\n      Logger.debug(\"Query for #{query.action.name} on resource #{inspect(query.resource)} returned #{length(records)} records\")\n\n      {:ok, records}\n    end)","ref":"Ash.Resource.Preparation.Builtins.html#after_action/1-example"},{"type":"macro","title":"Ash.Resource.Preparation.Builtins.before_action/1","doc":"Directly attach a `before_action` function to the query.\n\nThis function will be called by `Ash.Query.before_action/2`,\nwith an additional `context` argument.","ref":"Ash.Resource.Preparation.Builtins.html#before_action/1"},{"type":"macro","title":"Example - Ash.Resource.Preparation.Builtins.before_action/1","doc":"prepare before_action(fn query, _context ->\n      Logger.debug(\"About to execute query for #{query.action.name} on #{inspect(query.resource)}\")\n\n      query\n    end)","ref":"Ash.Resource.Preparation.Builtins.html#before_action/1-example"},{"type":"function","title":"Ash.Resource.Preparation.Builtins.build/1","doc":"Passes the given keyword list to `Ash.Query.build/2` with the query being prepared.\n\nThis allows declaring simple query modifications in-line.\n\nTo see the available options, see `Ash.Query.build/2`","ref":"Ash.Resource.Preparation.Builtins.html#build/1"},{"type":"function","title":"Examples - Ash.Resource.Preparation.Builtins.build/1","doc":"prepare build(sort: [song_rank: :desc], limit: 10)\n    prepare build(load: [:friends])","ref":"Ash.Resource.Preparation.Builtins.html#build/1-examples"},{"type":"function","title":"Ash.Resource.Preparation.Builtins.set_context/1","doc":"Merges the given query context.\n\nIf an MFA is provided, it will be called with the changeset.\nThe MFA should return `{:ok, context_to_be_merged}` or `{:error, term}`","ref":"Ash.Resource.Preparation.Builtins.html#set_context/1"},{"type":"function","title":"Examples - Ash.Resource.Preparation.Builtins.set_context/1","doc":"change set_context(%{something_used_internally: true})\n    change set_context({MyApp.Context, :set_context, []})","ref":"Ash.Resource.Preparation.Builtins.html#set_context/1-examples"},{"type":"behaviour","title":"Ash.Resource.Change","doc":"The behaviour for an action-specific resource change.\n\n`c:init/1` is defined automatically by `use Ash.Resource.Change`, but can be implemented if you want to validate/transform any\noptions passed to the module.\n\nThe main function is `c:change/3`. It takes the changeset, any options that were provided\nwhen this change was configured on a resource, and the context, which currently only has\nthe actor.","ref":"Ash.Resource.Change.html"},{"type":"callback","title":"Ash.Resource.Change.after_batch/3","doc":"Runs on each batch result after it is dispatched to the data layer.","ref":"Ash.Resource.Change.html#c:after_batch/3"},{"type":"callback","title":"Ash.Resource.Change.atomic/3","doc":"","ref":"Ash.Resource.Change.html#c:atomic/3"},{"type":"callback","title":"Ash.Resource.Change.atomic?/0","doc":"","ref":"Ash.Resource.Change.html#c:atomic?/0"},{"type":"callback","title":"Ash.Resource.Change.batch_callbacks?/3","doc":"Whether or not batch callbacks should be run (if they are defined). Defaults to `true`.","ref":"Ash.Resource.Change.html#c:batch_callbacks?/3"},{"type":"callback","title":"Ash.Resource.Change.batch_change/3","doc":"Replaces `change/3` for batch actions, allowing to optimize changes for bulk actions.\n\nYou can define only `batch_change/3`, and it will be used for both single and batch actions.\nIt cannot, however, be used in place of the `atomic/3` callback.","ref":"Ash.Resource.Change.html#c:batch_change/3"},{"type":"callback","title":"Ash.Resource.Change.before_batch/3","doc":"Runs on each batch before it is dispatched to the data layer.","ref":"Ash.Resource.Change.html#c:before_batch/3"},{"type":"callback","title":"Ash.Resource.Change.change/3","doc":"","ref":"Ash.Resource.Change.html#c:change/3"},{"type":"type","title":"Ash.Resource.Change.context/0","doc":"","ref":"Ash.Resource.Change.html#t:context/0"},{"type":"callback","title":"Ash.Resource.Change.has_after_batch?/0","doc":"","ref":"Ash.Resource.Change.html#c:has_after_batch?/0"},{"type":"callback","title":"Ash.Resource.Change.has_batch_change?/0","doc":"","ref":"Ash.Resource.Change.html#c:has_batch_change?/0"},{"type":"callback","title":"Ash.Resource.Change.has_before_batch?/0","doc":"","ref":"Ash.Resource.Change.html#c:has_before_batch?/0"},{"type":"callback","title":"Ash.Resource.Change.has_change?/0","doc":"","ref":"Ash.Resource.Change.html#c:has_change?/0"},{"type":"callback","title":"Ash.Resource.Change.init/1","doc":"","ref":"Ash.Resource.Change.html#c:init/1"},{"type":"type","title":"Ash.Resource.Change.ref/0","doc":"","ref":"Ash.Resource.Change.html#t:ref/0"},{"type":"type","title":"Ash.Resource.Change.t/0","doc":"","ref":"Ash.Resource.Change.html#t:t/0"},{"type":"module","title":"Ash.Resource.Change.Builtins","doc":"Built in changes that are available to all resources\n\nThe functions in this module are imported by default in the actions section.","ref":"Ash.Resource.Change.Builtins.html"},{"type":"macro","title":"Ash.Resource.Change.Builtins.after_action/2","doc":"Directly attach an `after_action` function to the current change.\n\nSee `Ash.Changeset.after_action/3` for more information.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Resource.Change.Builtins.html#after_action/2"},{"type":"macro","title":"Example - Ash.Resource.Change.Builtins.after_action/2","doc":"change after_action(fn changeset, record, _context ->\n      Logger.debug(\"Successfully executed action #{changeset.action.name} on #{inspect(changeset.resource)}\")\n      {:ok, record}\n    end)","ref":"Ash.Resource.Change.Builtins.html#after_action/2-example"},{"type":"macro","title":"Ash.Resource.Change.Builtins.after_transaction/2","doc":"Directly attach an `after_transaction` function to the current change.\n\nSee `Ash.Changeset.after_transaction/3` for more information.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Resource.Change.Builtins.html#after_transaction/2"},{"type":"macro","title":"Example - Ash.Resource.Change.Builtins.after_transaction/2","doc":"change after_transaction(fn\n      changeset, {:ok, record}, _context ->\n        Logger.debug(\"Successfully executed transaction for action #{changeset.action.name} on #{inspect(changeset.resource)}\")\n        {:ok, record}\n      changeset, {:error, reason}, _context ->\n        Logger.debug(\"Failed to execute transaction for action #{changeset.action.name} on #{inspect(changeset.resource)}, reason: #{inspect(reason)}\")\n        {:error, reason}\n    end)","ref":"Ash.Resource.Change.Builtins.html#after_transaction/2-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.atomic_update/3","doc":"Updates an attribute using an expression. See `Ash.Changeset.atomic_update/3` for more.\n\nOptions:\n\n* `:cast_atomic?` - set to `false` to ignore atomic type casting logic. Defaults to `true`.","ref":"Ash.Resource.Change.Builtins.html#atomic_update/3"},{"type":"macro","title":"Ash.Resource.Change.Builtins.before_action/2","doc":"Directly attach a `before_action` function to the current change.\n\nSee `Ash.Changeset.before_action/3` for more information.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Resource.Change.Builtins.html#before_action/2"},{"type":"macro","title":"Example - Ash.Resource.Change.Builtins.before_action/2","doc":"change before_action(fn changeset, _context ->\n      Logger.debug(\"About to execute #{changeset.action.name} on #{inspect(changeset.resource)}\")\n\n      changeset\n    end)","ref":"Ash.Resource.Change.Builtins.html#before_action/2-example"},{"type":"macro","title":"Ash.Resource.Change.Builtins.before_transaction/2","doc":"Directly attach a `before_transaction` function to the current change.\n\nSee `Ash.Changeset.before_transaction/3` for more information.\n\nProvide the option `prepend?: true` to place the hook before all other hooks instead of after.","ref":"Ash.Resource.Change.Builtins.html#before_transaction/2"},{"type":"macro","title":"Example - Ash.Resource.Change.Builtins.before_transaction/2","doc":"change before_transaction(fn changeset, _context ->\n      Logger.debug(\"About to execute transaction for #{changeset.action.name} on #{inspect(changeset.resource)}\")\n\n      changeset\n    end)","ref":"Ash.Resource.Change.Builtins.html#before_transaction/2-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.cascade_destroy/2","doc":"Cascade this resource's destroy action to a related resource's destroy action.\n\nAdds an after-action hook that explicitly calls destroy on any records related\nvia the named relationship.  It will optimise for bulk destroys where\npossible.\n\n> #### Beware database constraints {: .warning}\n>\n> Think carefully before using this change with data layers which enforce\n> referential integrity (ie PostgreSQL and SQLite) and you may need to defer\n> constraints for the relationship in question.\n>\n> See also:\n>   1. [`postgres.references.reference.deferrable` DSL](https://hexdocs.pm/ash_postgres/dsl-ashpostgres-datalayer.html#postgres-references-reference-deferrable)\n>   2. [`sqlite.references.reference.deferrable` DSL](https://hexdocs.pm/ash_sqlite/dsl-ashsqlite-datalayer.html#sqlite-references-reference-deferrable)\n>   3. [PostgreSQL's `SET CONSTRAINTS` documentation](https://www.postgresql.org/docs/current/sql-set-constraints.html)\n>   4. [SQLite's `PRAGMA defer_foreign_keys` documentation](https://www.sqlite.org/pragma.html#pragma_defer_foreign_keys)\n\n> #### Cascading notifications {: .tip}\n>\n> By default notifications are disabled for the related destroy. This is to avoid potentially sending a **lot** of notifications for high-cardinality relationships.","ref":"Ash.Resource.Change.Builtins.html#cascade_destroy/2"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.cascade_destroy/2","doc":"* `:relationship` (`t:atom/0`) - Required. The name of the relationship to work on\n\n* `:action` (`t:atom/0`) - The name of the destroy action to call on the related resource. Uses the primary destroy by default.\n\n* `:read_action` (`t:atom/0`) - The name of the read action to call on the related resource to find results to be destroyed\n\n* `:return_notifications?` (`t:boolean/0`) - Return notifications for all destroyed records? The default value is `false`.\n\n* `:after_action?` (`t:boolean/0`) - If true all the cascade destroys are done in after_action hooks. This makes it safe to use in atomic actions The default value is `true`.\n\n* `:domain` (`Ash.Domain`) -","ref":"Ash.Resource.Change.Builtins.html#cascade_destroy/2-options"},{"type":"function","title":"Example - Ash.Resource.Change.Builtins.cascade_destroy/2","doc":"change cascade_destroy(:relationship)","ref":"Ash.Resource.Change.Builtins.html#cascade_destroy/2-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.cascade_update/2","doc":"Cascade a resource's update action to a related resource's update action.\n\n  Adds an after-action hook that explicitly calls update on any records related\n  via the named relationship.  It will optimise for bulk updates where\n  possible.\n\n  Allows you to copy fields from the arguments or changes to the destination,\n  this way you can cascade a bunch of changes downstream.\n\n> #### Beware database constraints {: .warning}\n>\n> Think carefully before using this change with data layers which enforce\n> referential integrity (ie PostgreSQL and SQLite) and you may need to defer\n> constraints for the relationship in question.\n>\n> See also:\n>   1. [`postgres.references.reference.deferrable` DSL](https://hexdocs.pm/ash_postgres/dsl-ashpostgres-datalayer.html#postgres-references-reference-deferrable)\n>   2. [`sqlite.references.reference.deferrable` DSL](https://hexdocs.pm/ash_sqlite/dsl-ashsqlite-datalayer.html#sqlite-references-reference-deferrable)\n>   3. [PostgreSQL's `SET CONSTRAINTS` documentation](https://www.postgresql.org/docs/current/sql-set-constraints.html)\n>   4. [SQLite's `PRAGMA defer_foreign_keys` documentation](https://www.sqlite.org/pragma.html#pragma_defer_foreign_keys)\n\n> #### Cascading notifications {: .tip}\n>\n> By default notifications are disabled for the related updates. This is to avoid potentially sending a **lot** of notifications for high-cardinality relationships.","ref":"Ash.Resource.Change.Builtins.html#cascade_update/2"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.cascade_update/2","doc":"* `:relationship` (`t:atom/0`) - Required. The name of the relationship to work on\n\n* `:action` (`t:atom/0`) - The name of the update action to call on the related resource. Uses the primary update by default.\n\n* `:copy_inputs` (list of `t:atom/0`) - A list of fields to copy & pass on to the downstream update. The source action cannot be atomic. The default value is `[]`.\n\n* `:read_action` (`t:atom/0`) - The name of the read action to call on the related resource to find results to be updated\n\n* `:return_notifications?` (`t:boolean/0`) - Return notifications for all updated records? The default value is `false`.\n\n* `:domain` (`Ash.Domain`) -","ref":"Ash.Resource.Change.Builtins.html#cascade_update/2-options"},{"type":"function","title":"Example - Ash.Resource.Change.Builtins.cascade_update/2","doc":"change cascade_update(:relationship1)\n    change cascade_update(:relationship2, copy_inputs: [:field1, :field2])","ref":"Ash.Resource.Change.Builtins.html#cascade_update/2-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.debug_log/1","doc":"","ref":"Ash.Resource.Change.Builtins.html#debug_log/1"},{"type":"function","title":"Ash.Resource.Change.Builtins.ensure_selected/1","doc":"Passes the provided value into `Ash.Changeset.ensure_selected/2`\n\nIf the value is not already selected, this makes sure it is. Does not deselect anything else.","ref":"Ash.Resource.Change.Builtins.html#ensure_selected/1"},{"type":"function","title":"Example - Ash.Resource.Change.Builtins.ensure_selected/1","doc":"change ensure_selected([:necessary_field])","ref":"Ash.Resource.Change.Builtins.html#ensure_selected/1-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.filter/1","doc":"Applies a filter to the changeset. Has no effect for create actions.\n\nThis ensures that only things matching the provided filter are updated or destroyed.","ref":"Ash.Resource.Change.Builtins.html#filter/1"},{"type":"function","title":"Ash.Resource.Change.Builtins.get_and_lock/2","doc":"Re-fetches the record being updated and locks it with the given type.\n\nThis happens in a `before_action` hook (so that it is done as part of the transaction).\n\nIf your resource has global validations (in the top level `validations` block), you may\nwant to add `delay_global_validations? true` to your action to ensure they happen on the\nlocked record.","ref":"Ash.Resource.Change.Builtins.html#get_and_lock/2"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.get_and_lock/2","doc":"- `:skip_atomic?` - set to `true` to skip in the case that the update is done atomically. Defaults to `false`.","ref":"Ash.Resource.Change.Builtins.html#get_and_lock/2-options"},{"type":"function","title":"Ash.Resource.Change.Builtins.get_and_lock_for_update/1","doc":"Re-fetches the record being updated and locks it for update.\n\nOnly usable with data layers that support locking `:for_update`.\n\nThis happens in a `before_action` hook (so that it is done as part of the transaction).\n\nIf your resource has global validations (in the top level `validations` block), you may\nwant to add `delay_global_validations? true` to your action to ensure they happen on the\nlocked record.","ref":"Ash.Resource.Change.Builtins.html#get_and_lock_for_update/1"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.get_and_lock_for_update/1","doc":"- `:skip_atomic?` - set to `true` to skip in the case that the update is done atomically. Defaults to `false`.","ref":"Ash.Resource.Change.Builtins.html#get_and_lock_for_update/1-options"},{"type":"function","title":"Ash.Resource.Change.Builtins.increment/2","doc":"Increments an attribute's value by the amount specified, which defaults to 1.\n\nOptions:\n\n* `:amount` - Defaults to 1\n* `:overflow_limit` - Defaults to `nil`. If the value is over the overflow limit it will roll-over to the amount being incremented by (for common database limit support)","ref":"Ash.Resource.Change.Builtins.html#increment/2"},{"type":"function","title":"Ash.Resource.Change.Builtins.load/1","doc":"Passes the provided value into `Ash.load` after the action has completed.","ref":"Ash.Resource.Change.Builtins.html#load/1"},{"type":"function","title":"Example - Ash.Resource.Change.Builtins.load/1","doc":"change load(:comments)\n    change load([:friend_count, :friends])","ref":"Ash.Resource.Change.Builtins.html#load/1-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.manage_relationship/3","doc":"Calls `Ash.Changeset.manage_relationship/4` with the changeset and relationship provided, using the value provided for the named argument.\n\nIf relationship_name is not specified, it is assumed to be the same as the argument.\n\nFor information on the available options, see `Ash.Changeset.manage_relationship/4`.","ref":"Ash.Resource.Change.Builtins.html#manage_relationship/3"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.manage_relationship/3","doc":"change manage_relationship(:comments, type: :append)\n    change manage_relationship(:remove_comments, :comments, type: :remove)","ref":"Ash.Resource.Change.Builtins.html#manage_relationship/3-examples"},{"type":"function","title":"Ash.Resource.Change.Builtins.optimistic_lock/1","doc":"Apply an \"optimistic lock\" on a record being updated or destroyed.\n\nSee `Ash.Resource.Change.OptimisticLock` for more.","ref":"Ash.Resource.Change.Builtins.html#optimistic_lock/1"},{"type":"function","title":"Ash.Resource.Change.Builtins.prevent_change/1","doc":"Clears a change off of the changeset before the action runs.\n\nDoes not fail if it is being changed, but ensures it is cleared just before the action.\n\nCan be useful if a change is only used in validations but shouldn't ultimately be written to the data layer.","ref":"Ash.Resource.Change.Builtins.html#prevent_change/1"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.prevent_change/1","doc":"change prevent_change(:email)","ref":"Ash.Resource.Change.Builtins.html#prevent_change/1-examples"},{"type":"function","title":"Ash.Resource.Change.Builtins.relate_actor/2","doc":"Relates the actor to the data being changed, as the provided relationship.","ref":"Ash.Resource.Change.Builtins.html#relate_actor/2"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.relate_actor/2","doc":"* `:allow_nil?` (`t:boolean/0`) - Whether or not to allow the actor to be nil, in which case nothing will happen. The default value is `false`.\n\n* `:field` (`t:atom/0`) - The field of the actor to set the relationship to","ref":"Ash.Resource.Change.Builtins.html#relate_actor/2-options"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.relate_actor/2","doc":"change relate_actor(:owner, allow_nil?: true)","ref":"Ash.Resource.Change.Builtins.html#relate_actor/2-examples"},{"type":"function","title":"Ash.Resource.Change.Builtins.select/1","doc":"Passes the provided value into `Ash.Changeset.select/3`\n\nKeep in mind, this will *limit* the fields that are selected. You may want `ensure_selected/1` if you\nwant to make sure that something is selected, without deselecting anything else.\n\nSelecting in changesets does not actually do a select in the data layer. It nils out any\nfields that were not selected after completing the action. This can be useful if you are writing\npolicies that have to do with specific fields being selected.","ref":"Ash.Resource.Change.Builtins.html#select/1"},{"type":"function","title":"Example - Ash.Resource.Change.Builtins.select/1","doc":"change select([:name])","ref":"Ash.Resource.Change.Builtins.html#select/1-example"},{"type":"function","title":"Ash.Resource.Change.Builtins.set_attribute/3","doc":"Sets the attribute to the value provided.\n\nIf a zero argument function is provided, it is called to determine the value.\n\nUse `arg(:argument_name)` to use the value of the given argument. If the argument is not supplied then nothing happens.","ref":"Ash.Resource.Change.Builtins.html#set_attribute/3"},{"type":"function","title":"Options - Ash.Resource.Change.Builtins.set_attribute/3","doc":"* `:set_when_nil?` (`t:boolean/0`) - When false, decline setting the attribute if it is nil. The default value is `true`.\n\n* `:new?` (`t:boolean/0`) - When true, sets the attribute to the value provided if the attribute is not already being changed. The default value is `false`.","ref":"Ash.Resource.Change.Builtins.html#set_attribute/3-options"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.set_attribute/3","doc":"change set_attribute(:active, false)\n    change set_attribute(:opened_at, &DateTime.utc_now/0)\n    change set_attribute(:status, arg(:status))\n    change set_attribute(:encrypted_data, arg(:data), set_when_nil?: false)","ref":"Ash.Resource.Change.Builtins.html#set_attribute/3-examples"},{"type":"function","title":"Ash.Resource.Change.Builtins.set_context/1","doc":"Merges the given query context.\n\nIf an MFA is provided, it will be called with the changeset.\nThe MFA should return `{:ok, context_to_be_merged}` or `{:error, term}`","ref":"Ash.Resource.Change.Builtins.html#set_context/1"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.set_context/1","doc":"change set_context(%{something_used_internally: true})\n    change set_context({MyApp.Context, :set_context, []})","ref":"Ash.Resource.Change.Builtins.html#set_context/1-examples"},{"type":"function","title":"Ash.Resource.Change.Builtins.set_new_attribute/2","doc":"Sets the attribute to the value provided if the attribute is not already being changed.\n\nIf a zero argument function is provided, it is called to determine the value.\n\nUse `arg(:argument_name)` to use the value of the given argument. If the argument is not supplied then nothing happens.","ref":"Ash.Resource.Change.Builtins.html#set_new_attribute/2"},{"type":"function","title":"Examples - Ash.Resource.Change.Builtins.set_new_attribute/2","doc":"change set_new_attribute(:active, false)\n    change set_new_attribute(:opened_at, &DateTime.utc_now/0)\n    change set_new_attribute(:status, arg(:status))","ref":"Ash.Resource.Change.Builtins.html#set_new_attribute/2-examples"},{"type":"macro","title":"Ash.Resource.Change.Builtins.update_change/2","doc":"Updates an existing attribute change by applying a function to it.\n\nThe update function gets called with the value already cast to the correct type, and only gets called\non valid changesets, so the value is guaranteed to have passed validations and constraints.","ref":"Ash.Resource.Change.Builtins.html#update_change/2"},{"type":"behaviour","title":"Ash.Resource.Validation","doc":"Represents a validation in Ash.\n\nSee `Ash.Resource.Validation.Builtins` for a list of builtin validations.\n\nTo write your own validation, define a module that implements the `c:init/1` callback\nto validate options at compile time, and `c:validate/3` callback to do the validation.\n\nThen, in a resource, you can say:\n\n```\nvalidations do\n  validate {MyValidation, [foo: :bar]}\nend\n```","ref":"Ash.Resource.Validation.html"},{"type":"function","title":"Ash.Resource.Validation.action_schema/0","doc":"","ref":"Ash.Resource.Validation.html#action_schema/0"},{"type":"callback","title":"Ash.Resource.Validation.atomic/3","doc":"","ref":"Ash.Resource.Validation.html#c:atomic/3"},{"type":"callback","title":"Ash.Resource.Validation.atomic?/0","doc":"","ref":"Ash.Resource.Validation.html#c:atomic?/0"},{"type":"callback","title":"Ash.Resource.Validation.describe/1","doc":"","ref":"Ash.Resource.Validation.html#c:describe/1"},{"type":"callback","title":"Ash.Resource.Validation.has_validate?/0","doc":"","ref":"Ash.Resource.Validation.html#c:has_validate?/0"},{"type":"callback","title":"Ash.Resource.Validation.init/1","doc":"","ref":"Ash.Resource.Validation.html#c:init/1"},{"type":"function","title":"Ash.Resource.Validation.opt_schema/0","doc":"","ref":"Ash.Resource.Validation.html#opt_schema/0"},{"type":"type","title":"Ash.Resource.Validation.path/0","doc":"","ref":"Ash.Resource.Validation.html#t:path/0"},{"type":"type","title":"Ash.Resource.Validation.ref/0","doc":"","ref":"Ash.Resource.Validation.html#t:ref/0"},{"type":"type","title":"Ash.Resource.Validation.t/0","doc":"","ref":"Ash.Resource.Validation.html#t:t/0"},{"type":"callback","title":"Ash.Resource.Validation.validate/3","doc":"","ref":"Ash.Resource.Validation.html#c:validate/3"},{"type":"function","title":"Ash.Resource.Validation.validation_type/0","doc":"","ref":"Ash.Resource.Validation.html#validation_type/0"},{"type":"module","title":"Ash.Resource.Validation.Builtins","doc":"Built in validations that are available to all resources\n\nThe functions in this module are imported by default in the validations section.","ref":"Ash.Resource.Validation.Builtins.html"},{"type":"function","title":"Ash.Resource.Validation.Builtins.absent/2","doc":"Validates that the given attribute or argument or list of attributes or arguments are `nil`.\n\nThis is the inverse of `present/2`.\n\nUse options to specify how many must be `nil`. If no options are provided, validates that they are all absent.\n\nKeep in mind that some types cast certain values to `nil`, and validations are\napplied after all inputs have been cast. For example, a `:string` type attribute\nwith the default constraints will cast `\"\"` as `nil`, meaning an input of\n`\"\"` would pass the `absent` validation.","ref":"Ash.Resource.Validation.Builtins.html#absent/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.absent/2","doc":"* `:at_least` (`t:non_neg_integer/0`) - At least this many must be absent. Defaults to the number of attributes provided\n\n* `:at_most` (`t:non_neg_integer/0`) - At most this many must be absent. Defaults to the number of attributes provided\n\n* `:exactly` (`t:non_neg_integer/0`) - Exactly this many must be absent","ref":"Ash.Resource.Validation.Builtins.html#absent/2-options"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.absent/2","doc":"validate absent(:unsettable_option)\n\n    validate absent([:first_name, :last_name]), where: [present(:full_name)]\n\n    validate absent([:is_admin, :is_normal_user], at_least: 1)","ref":"Ash.Resource.Validation.Builtins.html#absent/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.action_is/1","doc":"Validates that the action name matches the provided action name or names. Primarily meant for use in `where`.","ref":"Ash.Resource.Validation.Builtins.html#action_is/1"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.action_is/1","doc":"validate present(:foo), where: [action_is(:bar)]\n\n    validate present(:foo), where: action_is([:bar, :baz])","ref":"Ash.Resource.Validation.Builtins.html#action_is/1-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.argument_does_not_equal/2","doc":"Validates that an argument is not being changed to a specific value, or does not equal the given value if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#argument_does_not_equal/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.argument_does_not_equal/2","doc":"validate argument_does_not_equal(:admin, true)\n\n    # Or to only check for changing to a given value\n    validate argument_does_not_equal(:admin, true), where: [changing(:admin)]","ref":"Ash.Resource.Validation.Builtins.html#argument_does_not_equal/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.argument_equals/2","doc":"Validates that an argument is being changed to a specific value, or equals the given value if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#argument_equals/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.argument_equals/2","doc":"validate argument_equals(:admin, true)\n\n    # Or to only check for changing to a given value\n    validate argument_equals(:admin, true), where: [changing(:admin)]","ref":"Ash.Resource.Validation.Builtins.html#argument_equals/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.argument_in/2","doc":"Validates that an argument is being changed to one of a set of specific values, or is in the the given list if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#argument_in/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.argument_in/2","doc":"validate argument_in(:state, [1, 2, 3])\n\n    # Or to only check for changing to a something in a given list\n    validate argument_in(:state, [1, 2, 3]), where: [changing(:state)]","ref":"Ash.Resource.Validation.Builtins.html#argument_in/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.attribute_does_not_equal/2","doc":"Validates that an attribute is not being changed to a specific value, or does not equal the given value if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#attribute_does_not_equal/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.attribute_does_not_equal/2","doc":"validate attribute_does_not_equal(:admin, true)\n\n    # Or to only check for changing to a given value\n    validate attribute_does_not_equal(:admin, true), where: [changing(:admin)]","ref":"Ash.Resource.Validation.Builtins.html#attribute_does_not_equal/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.attribute_equals/2","doc":"Validates that an attribute is being changed to a specific value, or equals the given value if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#attribute_equals/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.attribute_equals/2","doc":"validate attribute_equals(:admin, true)\n\n    # Or to only check for changing to a given value\n    validate attribute_equals(:admin, true), where: [changing(:admin)]","ref":"Ash.Resource.Validation.Builtins.html#attribute_equals/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.attribute_in/2","doc":"Validates that an attribute is being changed to one of a set of specific values, or is in the the given list if it is not being changed.","ref":"Ash.Resource.Validation.Builtins.html#attribute_in/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.attribute_in/2","doc":"validate attribute_in(:state, [1, 2, 3])\n\n    # Or to only check for changing to a something in a given list\n    validate attribute_in(:state, [1, 2, 3]), where: [changing(:state)]","ref":"Ash.Resource.Validation.Builtins.html#attribute_in/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.attributes_absent/2","doc":"Validates that the attribute or list of attributes are `nil`. See `absent/2` for more information.","ref":"Ash.Resource.Validation.Builtins.html#attributes_absent/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.attributes_absent/2","doc":"* `:at_least` (`t:non_neg_integer/0`) - At least this many must be absent. Defaults to the number of attributes provided\n\n* `:at_most` (`t:non_neg_integer/0`) - At most this many must be absent. Defaults to the number of attributes provided\n\n* `:exactly` (`t:non_neg_integer/0`) - Exactly this many must be absent","ref":"Ash.Resource.Validation.Builtins.html#attributes_absent/2-options"},{"type":"function","title":"Ash.Resource.Validation.Builtins.attributes_present/2","doc":"Validates that the attribute or list of attributes are not `nil`. See `present/2` for more information.","ref":"Ash.Resource.Validation.Builtins.html#attributes_present/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.attributes_present/2","doc":"* `:at_least` (`t:non_neg_integer/0`) - At least this many must be present. Defaults to the number of attributes provided\n\n* `:at_most` (`t:non_neg_integer/0`) - At most this many must be present. Defaults to the number of attributes provided\n\n* `:exactly` (`t:non_neg_integer/0`) - Exactly this many must be present","ref":"Ash.Resource.Validation.Builtins.html#attributes_present/2-options"},{"type":"function","title":"Ash.Resource.Validation.Builtins.changing/2","doc":"Validates that an attribute or relationship is being changed","ref":"Ash.Resource.Validation.Builtins.html#changing/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.changing/2","doc":"validate changing(:first_name)\n    validate changing(:comments)\n    validate changing(:comments, touching?: true)","ref":"Ash.Resource.Validation.Builtins.html#changing/2-examples"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.changing/2","doc":"* `:field` (`t:atom/0`) - Required. The attribute or relationship to check for changes. Using a relationship does not compare old and new value, returning `true` if the value is being touched)\n\n* `:touching?` (`t:atom/0`) - Whether to consider a field as changing if it is just being touched (i.e consider it changed even if it is being changed to its current value) The default value is `false`.","ref":"Ash.Resource.Validation.Builtins.html#changing/2-options"},{"type":"function","title":"Ash.Resource.Validation.Builtins.compare/2","doc":"Validates that an attribute or argument meets the given comparison criteria.\n\nThe values provided for each option may be a literal value, attribute, argument, or a zero argument function.","ref":"Ash.Resource.Validation.Builtins.html#compare/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.compare/2","doc":"* `:greater_than` - The value that the attribute should be greater than.\n\n* `:greater_than_or_equal_to` - The value that the attribute should be greater than or equal to\n\n* `:less_than` - The value that the attribute should be less than\n\n* `:less_than_or_equal_to` - The value that the attribute should be less than or equal to\n\n* `:is_equal` - The value that the attribute should be equal to\n\n* `:is_not_equal` - The value that the attribute should not be equal to\n\n* `:is_nil` (`t:boolean/0`) - Whether the attribute should be nil (true) or not nil (false)","ref":"Ash.Resource.Validation.Builtins.html#compare/2-options"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.compare/2","doc":"validate compare(:age, greater_than_or_equal_to: 18),\n      where: [attribute_equals(:show_adult_content, true)],\n      message: \"must be over %{greater_than_or_equal_to} to enable adult content.\"\n\n    validate compare(:points, greater_than: 0, less_than_or_equal_to: 100)","ref":"Ash.Resource.Validation.Builtins.html#compare/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.confirm/2","doc":"Validates that a field or argument matches another field or argument","ref":"Ash.Resource.Validation.Builtins.html#confirm/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.confirm/2","doc":"validate confirm(:password, :password_confirmation)\n    validate confirm(:email, :email_confirmation)","ref":"Ash.Resource.Validation.Builtins.html#confirm/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.match/2","doc":"Validates that an attribute's value matches a given regex.\n\n`String.match?/2` is used to determine if the value matches.","ref":"Ash.Resource.Validation.Builtins.html#match/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.match/2","doc":"validate match(:slug, \"^[0-9a-z-_]+$\")","ref":"Ash.Resource.Validation.Builtins.html#match/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.negate/1","doc":"Validates that other validation does not pass","ref":"Ash.Resource.Validation.Builtins.html#negate/1"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.negate/1","doc":"validate negate(one_of(:status, [:closed, :finished]))","ref":"Ash.Resource.Validation.Builtins.html#negate/1-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.numericality/2","doc":"Validates that an attribute or argument meets the given comparison criteria.\n\nThe values provided for each option may be a literal value, attribute, argument, or a zero argument function.","ref":"Ash.Resource.Validation.Builtins.html#numericality/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.numericality/2","doc":"* `:greater_than` - The value that the attribute should be greater than.\n\n* `:greater_than_or_equal_to` - The value that the attribute should be greater than or equal to\n\n* `:less_than` - The value that the attribute should be less than\n\n* `:less_than_or_equal_to` - The value that the attribute should be less than or equal to\n\n* `:is_equal` - The value that the attribute should be equal to\n\n* `:is_not_equal` - The value that the attribute should not be equal to\n\n* `:is_nil` (`t:boolean/0`) - Whether the attribute should be nil (true) or not nil (false)","ref":"Ash.Resource.Validation.Builtins.html#numericality/2-options"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.numericality/2","doc":"validate numericality(:age, greater_than_or_equal_to: 18),\n      where: [attribute_equals(:show_adult_content, true)],\n      message: \"must be over %{greater_than_or_equal_to} to enable adult content.\"\n\n    validate numericality(:points, greater_than: 0, less_than_or_equal_to: 100)","ref":"Ash.Resource.Validation.Builtins.html#numericality/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.one_of/2","doc":"Validates that an attribute's value is in a given list","ref":"Ash.Resource.Validation.Builtins.html#one_of/2"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.one_of/2","doc":"validate one_of(:status, [:closed_won, :closed_lost])","ref":"Ash.Resource.Validation.Builtins.html#one_of/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.present/2","doc":"Validates that the given attribute or argument or list of attributes or arguments are not `nil`.\n\nUse options to specify how many must not be `nil`. If no options are provided, validates that they are all present.\n\nKeep in mind that some types cast certain values to `nil`, and validations are\napplied after all inputs have been cast. For example, a `:string` type attribute\nwith the default constraints will cast `\"\"` as `nil`, meaning an input of\n`\"\"` would fail the `present` validation.","ref":"Ash.Resource.Validation.Builtins.html#present/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.present/2","doc":"* `:at_least` (`t:non_neg_integer/0`) - At least this many must be present. Defaults to the number of attributes provided\n\n* `:at_most` (`t:non_neg_integer/0`) - At most this many must be present. Defaults to the number of attributes provided\n\n* `:exactly` (`t:non_neg_integer/0`) - Exactly this many must be present","ref":"Ash.Resource.Validation.Builtins.html#present/2-options"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.present/2","doc":"validate present(:name)\n\n    validate present([:first_name, :last_name]), where: [absent(:full_name)]\n\n    validate present([:is_admin, :is_normal_user], at_most: 1)","ref":"Ash.Resource.Validation.Builtins.html#present/2-examples"},{"type":"function","title":"Ash.Resource.Validation.Builtins.string_length/2","doc":"Validates that an attribute on the original record meets the given length criteria","ref":"Ash.Resource.Validation.Builtins.html#string_length/2"},{"type":"function","title":"Options - Ash.Resource.Validation.Builtins.string_length/2","doc":"* `:min` (`t:non_neg_integer/0`) - String must be this length at least\n\n* `:max` (`t:non_neg_integer/0`) - String must be this length at most\n\n* `:exact` (`t:non_neg_integer/0`) - String must be this length exactly","ref":"Ash.Resource.Validation.Builtins.html#string_length/2-options"},{"type":"function","title":"Examples - Ash.Resource.Validation.Builtins.string_length/2","doc":"validate string_length(:slug, exact: 8)\n    validate string_length(:password, min: 6)\n    validate string_length(:secret, min: 4, max: 12)","ref":"Ash.Resource.Validation.Builtins.html#string_length/2-examples"},{"type":"behaviour","title":"Ash.Authorizer","doc":"The interface for an ash authorizer\n\nThese will typically be implemented by an extension, but a custom\none can be implemented by defining an extension that also adopts this behaviour.\n\nThen you can extend a resource with `authorizers: [YourAuthorizer]`","ref":"Ash.Authorizer.html"},{"type":"callback","title":"Ash.Authorizer.add_calculations/3","doc":"","ref":"Ash.Authorizer.html#c:add_calculations/3"},{"type":"function","title":"Ash.Authorizer.add_calculations/4","doc":"","ref":"Ash.Authorizer.html#add_calculations/4"},{"type":"callback","title":"Ash.Authorizer.alter_filter/3","doc":"","ref":"Ash.Authorizer.html#c:alter_filter/3"},{"type":"function","title":"Ash.Authorizer.alter_filter/4","doc":"","ref":"Ash.Authorizer.html#alter_filter/4"},{"type":"callback","title":"Ash.Authorizer.alter_results/3","doc":"","ref":"Ash.Authorizer.html#c:alter_results/3"},{"type":"function","title":"Ash.Authorizer.alter_results/4","doc":"","ref":"Ash.Authorizer.html#alter_results/4"},{"type":"function","title":"Ash.Authorizer.alter_sort/4","doc":"","ref":"Ash.Authorizer.html#alter_sort/4"},{"type":"callback","title":"Ash.Authorizer.check/2","doc":"","ref":"Ash.Authorizer.html#c:check/2"},{"type":"function","title":"Ash.Authorizer.check/3","doc":"","ref":"Ash.Authorizer.html#check/3"},{"type":"callback","title":"Ash.Authorizer.check_context/1","doc":"","ref":"Ash.Authorizer.html#c:check_context/1"},{"type":"function","title":"Ash.Authorizer.check_context/2","doc":"","ref":"Ash.Authorizer.html#check_context/2"},{"type":"type","title":"Ash.Authorizer.context/0","doc":"","ref":"Ash.Authorizer.html#t:context/0"},{"type":"callback","title":"Ash.Authorizer.exception/2","doc":"","ref":"Ash.Authorizer.html#c:exception/2"},{"type":"function","title":"Ash.Authorizer.exception/3","doc":"","ref":"Ash.Authorizer.html#exception/3"},{"type":"callback","title":"Ash.Authorizer.initial_state/4","doc":"","ref":"Ash.Authorizer.html#c:initial_state/4"},{"type":"function","title":"Ash.Authorizer.initial_state/5","doc":"","ref":"Ash.Authorizer.html#initial_state/5"},{"type":"type","title":"Ash.Authorizer.state/0","doc":"","ref":"Ash.Authorizer.html#t:state/0"},{"type":"callback","title":"Ash.Authorizer.strict_check/2","doc":"","ref":"Ash.Authorizer.html#c:strict_check/2"},{"type":"function","title":"Ash.Authorizer.strict_check/3","doc":"","ref":"Ash.Authorizer.html#strict_check/3"},{"type":"callback","title":"Ash.Authorizer.strict_check_context/1","doc":"","ref":"Ash.Authorizer.html#c:strict_check_context/1"},{"type":"function","title":"Ash.Authorizer.strict_check_context/2","doc":"","ref":"Ash.Authorizer.html#strict_check_context/2"},{"type":"behaviour","title":"Ash.Policy.Check","doc":"A behaviour for declaring checks, which can be used to easily construct\nauthorization rules.\n\nIf a check can be expressed simply, i.e as a function of the actor, or the context of the request,\nsee `Ash.Policy.SimpleCheck` for an easy way to write that check.\nIf a check can be expressed with a filter statement, see `Ash.Policy.FilterCheck`\nfor an easy way to write that check.","ref":"Ash.Policy.Check.html"},{"type":"type","title":"Ash.Policy.Check.actor/0","doc":"","ref":"Ash.Policy.Check.html#t:actor/0"},{"type":"type","title":"Ash.Policy.Check.authorizer/0","doc":"","ref":"Ash.Policy.Check.html#t:authorizer/0"},{"type":"callback","title":"Ash.Policy.Check.auto_filter/3","doc":"An optional callback, that allows the check to work with policies set to `access_type :filter`\n\nReturn a keyword list filter that will be applied to the query being made, and will scope the results to match the rule","ref":"Ash.Policy.Check.html#c:auto_filter/3"},{"type":"callback","title":"Ash.Policy.Check.check/4","doc":"An optional callback, that allows the check to work with policies set to `access_type :runtime`\n\nTakes a list of records, and returns the subset of authorized records.","ref":"Ash.Policy.Check.html#c:check/4"},{"type":"type","title":"Ash.Policy.Check.check_type/0","doc":"","ref":"Ash.Policy.Check.html#t:check_type/0"},{"type":"function","title":"Ash.Policy.Check.defines_auto_filter?/1","doc":"","ref":"Ash.Policy.Check.html#defines_auto_filter?/1"},{"type":"function","title":"Ash.Policy.Check.defines_check?/1","doc":"","ref":"Ash.Policy.Check.html#defines_check?/1"},{"type":"callback","title":"Ash.Policy.Check.describe/1","doc":"Describe the check in human readable format, given the options","ref":"Ash.Policy.Check.html#c:describe/1"},{"type":"callback","title":"Ash.Policy.Check.eager_evaluate?/0","doc":"","ref":"Ash.Policy.Check.html#c:eager_evaluate?/0"},{"type":"callback","title":"Ash.Policy.Check.expand_description/3","doc":"Expands the description of the check, given the actor and subject","ref":"Ash.Policy.Check.html#c:expand_description/3"},{"type":"type","title":"Ash.Policy.Check.options/0","doc":"","ref":"Ash.Policy.Check.html#t:options/0"},{"type":"callback","title":"Ash.Policy.Check.prefer_expanded_description?/0","doc":"Whether or not the expanded description should replace the basic description in breakdowns","ref":"Ash.Policy.Check.html#c:prefer_expanded_description?/0"},{"type":"type","title":"Ash.Policy.Check.ref/0","doc":"","ref":"Ash.Policy.Check.html#t:ref/0"},{"type":"callback","title":"Ash.Policy.Check.requires_original_data?/2","doc":"Whether or not your check requires the original data of a changeset (if applicable)","ref":"Ash.Policy.Check.html#c:requires_original_data?/2"},{"type":"callback","title":"Ash.Policy.Check.strict_check/3","doc":"Strict checks should be cheap, and should never result in external calls (like database or domain)\n\nIt should return `{:ok, true}` if it can tell that the request is authorized, and `{:ok, false}` if\nit can tell that it is not. If unsure, it should return `{:ok, :unknown}`","ref":"Ash.Policy.Check.html#c:strict_check/3"},{"type":"type","title":"Ash.Policy.Check.t/0","doc":"","ref":"Ash.Policy.Check.html#t:t/0"},{"type":"callback","title":"Ash.Policy.Check.type/0","doc":"The type of the check\n\n`:manual` checks must be written by hand as standard check modules\n`:filter` checks can use `Ash.Policy.FilterCheck` for simplicity\n`:simple` checks can use `Ash.Policy.SimpleCheck` for simplicity","ref":"Ash.Policy.Check.html#c:type/0"},{"type":"behaviour","title":"Ash.Policy.FilterCheck","doc":"A type of check that is represented by a filter statement\n\nYou can customize what the \"negative\" filter looks like by defining `c:reject/3`. This is important for\nfilters over related data. For example, given an `owner` relationship and a data layer like `ash_postgres`\nwhere `column != NULL` does *not* evaluate to true (see postgres docs on NULL for more):\n\n    # The opposite of\n    `owner.id == 1`\n    # in most cases is not\n    `not(owner.id == 1)`\n    # because in postgres that would be `NOT (owner.id = NULL)` in cases where there was no owner\n    # A better opposite would be\n    `owner.id != 1 or is_nil(owner.id)`\n    # alternatively\n    `not(owner.id == 1) or is_nil(owner.id)`\n\nBy being able to customize the `reject` filter, you can use related filters in your policies. Without it,\nthey will likely have undesired effects.","ref":"Ash.Policy.FilterCheck.html"},{"type":"type","title":"Ash.Policy.FilterCheck.context/0","doc":"","ref":"Ash.Policy.FilterCheck.html#t:context/0"},{"type":"callback","title":"Ash.Policy.FilterCheck.filter/3","doc":"","ref":"Ash.Policy.FilterCheck.html#c:filter/3"},{"type":"function","title":"Ash.Policy.FilterCheck.is_filter_check?/1","doc":"","ref":"Ash.Policy.FilterCheck.html#is_filter_check?/1"},{"type":"type","title":"Ash.Policy.FilterCheck.options/0","doc":"","ref":"Ash.Policy.FilterCheck.html#t:options/0"},{"type":"callback","title":"Ash.Policy.FilterCheck.reject/3","doc":"","ref":"Ash.Policy.FilterCheck.html#c:reject/3"},{"type":"behaviour","title":"Ash.Policy.SimpleCheck","doc":"A type of check that operates only on request context, never on the data\n\nDefine `c:match?/3`, which gets the actor, request context, and opts, and returns true or false","ref":"Ash.Policy.SimpleCheck.html"},{"type":"behaviour","title":"Example - Ash.Policy.SimpleCheck","doc":"This is a simple check that checks if the user is changing anything other than the\nprovided list.\n\n```elixir\ndefmodule ChangingNothingExcept do\n  use Ash.Policy.SimpleCheck\n\n  def match?(_actor, %{subject: %Ash.Changeset{} = changeset}, opts) do\n    allowed = opts[:attributes]\n    {:ok, Enum.all?(Map.keys(changeset.attributes), &(&1 in allowed))}\n  end\n\n  def match?(_, _, _), do: {:ok, true}\nend\n```\n\nYou could then use this like\n\n```elixir\npolicy actor_attribute_equals(:role, :foobar) do\n  authorize_if {ChangingNothingExcept, attributes: [:foo, :bar]}\nend\n```","ref":"Ash.Policy.SimpleCheck.html#module-example"},{"type":"type","title":"Ash.Policy.SimpleCheck.actor/0","doc":"","ref":"Ash.Policy.SimpleCheck.html#t:actor/0"},{"type":"type","title":"Ash.Policy.SimpleCheck.context/0","doc":"","ref":"Ash.Policy.SimpleCheck.html#t:context/0"},{"type":"callback","title":"Ash.Policy.SimpleCheck.match?/3","doc":"Whether or not the request matches the check","ref":"Ash.Policy.SimpleCheck.html#c:match?/3"},{"type":"type","title":"Ash.Policy.SimpleCheck.options/0","doc":"","ref":"Ash.Policy.SimpleCheck.html#t:options/0"},{"type":"module","title":"Ash.Policy.Check.Builtins","doc":"The global authorization checks built into ash","ref":"Ash.Policy.Check.Builtins.html"},{"type":"function","title":"Ash.Policy.Check.Builtins.accessing_from/2","doc":"This check is true when the current action is being run \"through\" a relationship.\n\nCases where this happens:\n\n1. Loading related data\n2. Managing relationships\n3. Aggregating data\n4. Filtering on relationships","ref":"Ash.Policy.Check.Builtins.html#accessing_from/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.action/1","doc":"This check is true when the action name matches the provided action name or names.\n\nThis is a very common pattern, allowing action-specific policies.","ref":"Ash.Policy.Check.Builtins.html#action/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.action_type/1","doc":"This check is true when the action type matches the provided type\n\nThis is useful for writing policies that apply to all actions of a given type.\n\nFor example:\n\n```elixir\npolicy action_type(:read) do\n  authorize_if relates_to_actor_via(:owner)\nend\n```","ref":"Ash.Policy.Check.Builtins.html#action_type/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.actor_absent/0","doc":"This check is false when there is an actor specified, and true when the actor is `nil`.","ref":"Ash.Policy.Check.Builtins.html#actor_absent/0"},{"type":"function","title":"Ash.Policy.Check.Builtins.actor_attribute_equals/2","doc":"This check is true when the value of the specified attribute of the actor equals the specified value.\n\nThis check will *never* pass if the actor does not have the specified key. For example,\n`actor_attribute_equals(:missing_key, nil)`","ref":"Ash.Policy.Check.Builtins.html#actor_attribute_equals/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.actor_present/0","doc":"This check is true when there is an actor specified, and false when the actor is `nil`.","ref":"Ash.Policy.Check.Builtins.html#actor_present/0"},{"type":"function","title":"Ash.Policy.Check.Builtins.always/0","doc":"This check always passes.\n\nCan be useful for \"deny-list\" style authorization. For example:\n\n```elixir\npolicy action_type(:read) do\n  forbid_if actor_attribute_equals(:disabled, true)\n  forbid_if actor_attribute_equals(:active, false)\n  authorize_if always()\nend\n```\n\nWithout that last clause, the policy would never pass.","ref":"Ash.Policy.Check.Builtins.html#always/0"},{"type":"function","title":"Ash.Policy.Check.Builtins.changing_attributes/1","doc":"This check is true when attribute changes correspond to the provided options.\n\nProvide a keyword list of options or just an atom representing the attribute.\n\nFor example:\n\n```elixir\n# if you are changing both first name and last name\nchanging_attributes([:first_name, :last_name])\n\n# if you are changing first name to fred\nchanging_attributes(first_name: [to: \"fred\"])\n\n# if you are changing last name from bob\nchanging_attributes(last_name: [from: \"bob\"])\n\n# if you are changing :first_name at all, last_name from \"bob\" and middle name from \"tom\" to \"george\"\nchanging_attributes([:first_name, last_name: [from: \"bob\"], middle_name: [from: \"tom\", to: \"george]])\n```","ref":"Ash.Policy.Check.Builtins.html#changing_attributes/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.changing_relationship/1","doc":"This check is true when the specified relationship is changing","ref":"Ash.Policy.Check.Builtins.html#changing_relationship/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.changing_relationships/1","doc":"This check is true when the specified relationships are changing","ref":"Ash.Policy.Check.Builtins.html#changing_relationships/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.context_equals/2","doc":"This check is true when the value of the specified key or path in the changeset or query context equals the specified value.\n\nNote that the context is not shared with other queries (e.g. loads).\n\nFor example:\n```elixir\n# Given this check on Profile\nauthorize_if context_equals(:allow_this?, true)\n\n# This load will not have the context and will not be authorized\nAsh.load!(user, :profile, context: %{allow_this?: true})\n\n# But this will have the context and will be authorized\nAsh.load!(user, [profile: Ash.Query.set_context(Profile, %{allow_this?: true})])\n```","ref":"Ash.Policy.Check.Builtins.html#context_equals/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.filtering_on/2","doc":"This check is true when the field provided is being referenced anywhere in a filter statement.\n\nThis applies to related filters as well. For example:\n\n```elixir\npolicy actor_attribute_equals(:is_admin, false) do\n  forbid_if filtering_on(:email)\n  # a path can be provided as well\n  forbid_if filtering_on([:owner], :email)\nend\n```\n\nThe first will return true in situations like:\n\n```elixir\nAsh.Query.filter(User, email == \"blah\")\nAsh.Query.filter(Tweet, author.email == \"blah\")\n```\n\nThe second will return true on queries like:\n\n```elixir\nAsh.Query.filter(Post, owner.email == \"blah\")\nAsh.Query.filter(Comment, post.owner.email == \"blah\")\n```","ref":"Ash.Policy.Check.Builtins.html#filtering_on/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.just_created_with_action/1","doc":"","ref":"Ash.Policy.Check.Builtins.html#just_created_with_action/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.loading/1","doc":"This check is true when the field or relationship, or path to field, is being loaded and false when it is not.\n\nThis is always false for `create`/`update`/`destroy` actions, because you cannot load fields on those action types.","ref":"Ash.Policy.Check.Builtins.html#loading/1"},{"type":"macro","title":"Ash.Policy.Check.Builtins.matches/2","doc":"This check is true when the specified function returns true","ref":"Ash.Policy.Check.Builtins.html#matches/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.never/0","doc":"This check never passes.\n\nThere is, generally speaking, no reason to use this, but it exists for\ncompleteness sake.","ref":"Ash.Policy.Check.Builtins.html#never/0"},{"type":"function","title":"Ash.Policy.Check.Builtins.relates_to_actor_via/2","doc":"This check passes if the data relates to the actor via the specified relationship or path of relationships.\n\nFor `update` & `destroy` actions, this check will apply to *the original data* before the changes are applied.\n\nFor `create` actions this check is very unlikely to pass. This is because relationships are modified *after* authorization\nhappens, not before.\n\nFor example:\n\n```elixir\npolicy action_type(:read) do\n  authorize_if relates_to_actor_via(:owner)\n\n  # Path of relationships:\n  authorize_if relates_to_actor_via([:account, :user])\n\n  # When the resource relates to a field of the actor:\n  authorize_if relates_to_actor_via(:roles, field: :role)\nend\n```","ref":"Ash.Policy.Check.Builtins.html#relates_to_actor_via/2"},{"type":"function","title":"Ash.Policy.Check.Builtins.relating_to_actor/1","doc":"This check is true when the specified relationship is being changed to the current actor.\n\nThis only supports `belongs_to` relationships at the moment, and will detect two cases:\n\n1. the `source_attribute` is being changed directly\n2. the relationship is being changed with `on_lookup?: :relate`, and a single input is being provided.","ref":"Ash.Policy.Check.Builtins.html#relating_to_actor/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.resource/1","doc":"This check is true when the resource name matches the provided resource name or names.","ref":"Ash.Policy.Check.Builtins.html#resource/1"},{"type":"function","title":"Ash.Policy.Check.Builtins.selecting/1","doc":"This check is true when the field is being selected and false when it is not.\n\nThis won't affect filters placed on this resource, so you may also want to either:\n\n- Mark the given field as `filterable? false`\n- Add another check for `filtering_on(:field)`\n\nFor example:\n\n```elixir\npolicy action_type(:read) do\n  # The actor can read and filter on their own email\n  authorize_if expr(id == ^actor(:id))\n\n  # No one else can select or filter on their email\n  forbid_if selecting(:email)\n  forbid_if filtering_on(:email)\n\n  # Otherwise, the policy passes\n  authorize_if always()\nend\n```","ref":"Ash.Policy.Check.Builtins.html#selecting/1"},{"type":"module","title":"Ash.DataLayer.Ets","doc":"An ETS (Erlang Term Storage) backed Ash Datalayer, for testing and lightweight usage.\n\nRemember, this does not have support for transactions! This is not recommended for production\nuse, especially in multi-user applications. It can, however, be great for prototyping.","ref":"Ash.DataLayer.Ets.html"},{"type":"function","title":"Ash.DataLayer.Ets.do_add_calculations/4","doc":"","ref":"Ash.DataLayer.Ets.html#do_add_calculations/4"},{"type":"macro","title":"Ash.DataLayer.Ets.ets/1","doc":"","ref":"Ash.DataLayer.Ets.html#ets/1"},{"type":"function","title":"Ash.DataLayer.Ets.stop/2","doc":"Stops the storage for a given resource/tenant (deleting all of the data)","ref":"Ash.DataLayer.Ets.html#stop/2"},{"type":"module","title":"Ash.DataLayer.Mnesia","doc":"An Mnesia backed Ash Datalayer.\n\nIn your application initialization, you will need to call `Mnesia.create_schema([node()])`.\n\nAdditionally, you will want to create your mnesia tables there.\n\nThis data layer is *unoptimized*, fetching all records from a table and filtering them\nin memory. For that reason, it is not recommended to use it with large amounts of data. It can be\ngreat for prototyping or light usage, though.","ref":"Ash.DataLayer.Mnesia.html"},{"type":"macro","title":"Ash.DataLayer.Mnesia.mnesia/1","doc":"","ref":"Ash.DataLayer.Mnesia.html#mnesia/1"},{"type":"function","title":"Ash.DataLayer.Mnesia.start/2","doc":"Creates the table for each mnesia resource in a domain","ref":"Ash.DataLayer.Mnesia.html#start/2"},{"type":"module","title":"Ash.DataLayer.Simple","doc":"A data layer that returns structs.\n\nThis is the data layer that is used under the hood\nby embedded resources, and resources without data layers.","ref":"Ash.DataLayer.Simple.html"},{"type":"function","title":"Ash.DataLayer.Simple.bulk_create/3","doc":"Callback implementation for  Ash.DataLayer.bulk_create/3 .","ref":"Ash.DataLayer.Simple.html#bulk_create/3"},{"type":"function","title":"Ash.DataLayer.Simple.set_data/2","doc":"Sets the data for a query against a data-layer-less resource","ref":"Ash.DataLayer.Simple.html#set_data/2"},{"type":"module","title":"Ash.Notifier.PubSub","doc":"A builtin notifier to help you publish events over any kind of pub-sub tooling.\n\nThis is plug and play with `Phoenix.PubSub`, but could be used with any pubsub system.\n\nYou configure a module that defines a `broadcast/3` function, and then add some \"publications\"\nwhich configure under what conditions an event should be sent and what the topic should be.","ref":"Ash.Notifier.PubSub.html"},{"type":"module","title":"Example - Ash.Notifier.PubSub","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    # ...\n    notifiers: [Ash.Notifier.PubSub]\n\n  # ...\n\n  pub_sub do\n    module MyAppWeb.Endpoint\n\n    prefix \"user\"\n    publish :update, [\"updated\", :_pkey]\n  end\nend\n```","ref":"Ash.Notifier.PubSub.html#module-example"},{"type":"module","title":"Debugging PubSub - Ash.Notifier.PubSub","doc":"It can be quite frustrating when setting up pub_sub when everything appears to be set up properly, but\nyou aren't receiving events. This usually means some kind of mismatch between the event names produced\nby the resource/config of your publications, and you can use the following flag to display debug\ninformation about all pub sub events.\n\n```elixir\nconfig :ash, :pub_sub, debug?: true\n```","ref":"Ash.Notifier.PubSub.html#module-debugging-pubsub"},{"type":"module","title":"Topic Templates - Ash.Notifier.PubSub","doc":"Often you want to include some piece of data in the thing being changed, like the `:id` attribute. This\nis done by providing a list as the topic, and using atoms which will be replaced by their corresponding\nvalues. They will ultimately be joined with `:`.\n\nFor example:\n\n```elixir\nprefix \"user\"\n\npublish :create, [\"created\", :user_id]\n```\n\nThis might publish a message to \"user:created:1\" for example.\n\nFor updates, if the field in the template is being changed, a message is sent\nto *both* values. So if you change `user 1` to `user 2`, the same message would\nbe published to `user:updated:1` and `user:updated:2`. If there are multiple\nattributes in the template, and they are all being changed, a message is sent for\nevery combination of substitutions.","ref":"Ash.Notifier.PubSub.html#module-topic-templates"},{"type":"module","title":"Important - Ash.Notifier.PubSub","doc":"If the previous value was `nil` or the field was not selected on the data passed into the action, then a\nnotification is not sent for the previous value.\n\nIf the new value is `nil` then a notification is not sent for the new value.","ref":"Ash.Notifier.PubSub.html#module-important"},{"type":"module","title":"Template parts - Ash.Notifier.PubSub","doc":"Templates may contain lists, in which case all combinations of values in the list will be used. Add\n`nil` to the list if you want to produce a pattern where that entry is omitted.\n\nThe atom `:_tenant` may be used. If the changeset has a tenant set on it, that\nvalue will be used, otherwise that combination of values is ignored.\n\nThe atom `:_pkey` may be used. It will be a stringified, concatenation of the primary key fields,\nor just the primary key if there is only one primary key field.\n\nThe atom `nil` may be used. It only makes sense to use it in the context of a list of alternatives,\nand adds a pattern where that part is skipped.\n\n```elixir\npublish :updated, [[:team_id, :_tenant], \"updated\", [:id, nil]]\n```\n\nWould produce the following messages, given a `team_id` of 1, a `tenant` of `org_1`, and an `id` of `50`:\n\n```elixir\n\"1:updated:50\"\n\"1:updated\"\n\"org_1:updated:50\"\n\"org_1:updated\"\n```","ref":"Ash.Notifier.PubSub.html#module-template-parts"},{"type":"module","title":"Custom Delimiters - Ash.Notifier.PubSub","doc":"It's possible to change the default delimiter used when generating topics. This is useful when working with message brokers\nlike RabbitMQ, which rely on a different set of delimiters for routing.\n\n\n```elixir\npub_sub do\n  delimiter \".\"\nend\n```","ref":"Ash.Notifier.PubSub.html#module-custom-delimiters"},{"type":"module","title":"Named Pubsub modules - Ash.Notifier.PubSub","doc":"If you are using a phoenix `Endpoint` module for pubsub then this is unnecessary. If you want to use a custom pub sub started\nwith something like `{Phoenix.PubSub, name: MyName}`, then you can provide `MyName` to here.","ref":"Ash.Notifier.PubSub.html#module-named-pubsub-modules"},{"type":"module","title":"Broadcast Types - Ash.Notifier.PubSub","doc":"Configured with `broadcast_type`.\n\n- `:notification` just sends the notification\n- `:phoenix_broadcast` sends a `%Phoenix.Socket.Broadcast{}` (see above)\n- `:broadcast` sends `%{topic: (topic), event: (event), payload: (notification)}`","ref":"Ash.Notifier.PubSub.html#module-broadcast-types"},{"type":"macro","title":"Ash.Notifier.PubSub.pub_sub/1","doc":"","ref":"Ash.Notifier.PubSub.html#pub_sub/1"},{"type":"function","title":"Ash.Notifier.PubSub.to_payload/4","doc":"","ref":"Ash.Notifier.PubSub.html#to_payload/4"},{"type":"module","title":"Ash.Policy.Authorizer","doc":"An authorization extension for ash resources.\n\nTo add this extension to a resource, add it to the list of `authorizers` like so:\n\n```elixir\nuse Ash.Resource,\n  ...,\n  authorizers: [\n    Ash.Policy.Authorizer\n  ]\n```\n\nA resource can be given a set of policies, which are enforced on each call to a resource action.\n\nFor reads, policies can be configured to filter out data that the actor shouldn't see, as opposed to\nresulting in a forbidden error.\n\nSee the [policies guide](/documentation/topics/security/policies.md) for practical examples.\n\nPolicies are solved/managed via a boolean satisfiability solver. To read more about boolean satisfiability,\nsee this page: https://en.wikipedia.org/wiki/Boolean_satisfiability_problem. At the end of\nthe day, however, it is not necessary to understand exactly how Ash takes your\nauthorization requirements and determines if a request is allowed. The\nimportant thing to understand is that Ash may or may not run any/all of your\nauthorization rules as they may be deemed unnecessary. As such, authorization\nchecks should have no side effects. Ideally, the checks built-in to ash should\ncover the bulk of your needs.","ref":"Ash.Policy.Authorizer.html"},{"type":"function","title":"Ash.Policy.Authorizer.alter_sort/3","doc":"","ref":"Ash.Policy.Authorizer.html#alter_sort/3"},{"type":"function","title":"Ash.Policy.Authorizer.expr_check/1","doc":"","ref":"Ash.Policy.Authorizer.html#expr_check/1"},{"type":"macro","title":"Ash.Policy.Authorizer.field_policies/1","doc":"","ref":"Ash.Policy.Authorizer.html#field_policies/1"},{"type":"function","title":"Ash.Policy.Authorizer.install/5","doc":"","ref":"Ash.Policy.Authorizer.html#install/5"},{"type":"macro","title":"Ash.Policy.Authorizer.policies/1","doc":"","ref":"Ash.Policy.Authorizer.html#policies/1"},{"type":"function","title":"Ash.Policy.Authorizer.print_tuple_boolean/1","doc":"","ref":"Ash.Policy.Authorizer.html#print_tuple_boolean/1"},{"type":"type","title":"Ash.Policy.Authorizer.t/0","doc":"","ref":"Ash.Policy.Authorizer.html#t:t/0"},{"type":"function","title":"Ash.Policy.Authorizer.template_var/1","doc":"","ref":"Ash.Policy.Authorizer.html#template_var/1"},{"type":"module","title":"Ash.Reactor","doc":"`Ash.Reactor` is a [`Reactor`](https://hex.pm/packages/reactor) extension\nwhich provides steps for working with Ash resources and actions.\n\nSee the [Ash Reactor Guide](https://hexdocs.pm/ash/reactor.html) for more\ninformation.","ref":"Ash.Reactor.html"},{"type":"type","title":"Ash.Reactor.action/0","doc":"","ref":"Ash.Reactor.html#t:action/0"},{"type":"macro","title":"Ash.Reactor.ash/1","doc":"","ref":"Ash.Reactor.html#ash/1"},{"type":"module","title":"Ash.Resource","doc":"A resource is a static definition of an entity in your system.\n\n[Resource DSL documentation](dsl-ash-resource.html)","ref":"Ash.Resource.html"},{"type":"module","title":"Options - Ash.Resource","doc":"* `:simple_notifiers` (list of module that adopts `Ash.Notifier`) - Notifiers with no DSL.\n\n* `:validate_domain_inclusion?` (`t:boolean/0`) - Whether or not to validate that this resource is included in a domain. The default value is `true`.\n\n* `:primary_read_warning?` (`t:boolean/0`) - Set to `false` to silence warnings about arguments, preparations and filters on the primary read action. The default value is `true`.\n\n* `:domain` (module that adopts `Ash.Domain`) - The domain to use when interacting with this resource. Also sets defaults for various options that ask for a domain.\n\n* `:embed_nil_values?` (`t:boolean/0`) - Whether or not to include keys with `nil` values in an embedded representation. Has no effect unless resource is an embedded resource. The default value is `true`.\n\n* `:extensions` (list of module that adopts `Spark.Dsl.Extension`) - A list of DSL extensions to add to the `Spark.Dsl`\n\n* `:data_layer` (module that adopts `Ash.DataLayer`) - data_layer extensions to add to the `Spark.Dsl` The default value is `Ash.DataLayer.Simple`.\n\n* `:authorizers` (one or a list of module that adopts `Ash.Authorizer`) - authorizers extensions to add to the `Spark.Dsl` The default value is `[]`.\n\n* `:notifiers` (one or a list of module that adopts `Ash.Notifier`) - notifiers extensions to add to the `Spark.Dsl` The default value is `[]`.\n\n* `:otp_app` (`t:atom/0`) - The otp_app to use for any application configurable options\n\n* `:fragments` (list of `t:module/0`) - Fragments to include in the `Spark.Dsl`. See the fragments guide for more.","ref":"Ash.Resource.html#module-options"},{"type":"function","title":"Ash.Resource.get_metadata/2","doc":"","ref":"Ash.Resource.html#get_metadata/2"},{"type":"function","title":"Ash.Resource.loaded?/3","doc":"Returns true if the load or path to load has been loaded","ref":"Ash.Resource.html#loaded?/3"},{"type":"function","title":"Options - Ash.Resource.loaded?/3","doc":"- `lists`: set to `:any` to have this return true if any record in a list that appears has the value loaded. Default is `:all`.\n- `unknown`: set to `true` to have unknown paths (like nil values or non-resources) return true. Defaults to `false`\n- `strict?`: set to `true` to return false if a calculation with arguments is being checked","ref":"Ash.Resource.html#loaded?/3-options"},{"type":"function","title":"Ash.Resource.put_metadata/3","doc":"","ref":"Ash.Resource.html#put_metadata/3"},{"type":"type","title":"Ash.Resource.record/0","doc":"","ref":"Ash.Resource.html#t:record/0"},{"type":"function","title":"Ash.Resource.selected?/3","doc":"Returns true if the given field has been selected on a record","ref":"Ash.Resource.html#selected?/3"},{"type":"function","title":"Options - Ash.Resource.selected?/3","doc":"- `forbidden_means_selected?`: set to `true` to return `true` if the field is marked as forbidden","ref":"Ash.Resource.html#selected?/3-options"},{"type":"function","title":"Ash.Resource.set_metadata/2","doc":"","ref":"Ash.Resource.html#set_metadata/2"},{"type":"type","title":"Ash.Resource.t/0","doc":"","ref":"Ash.Resource.html#t:t/0"},{"type":"macro","title":"Ash.Resource.timestamps/1","doc":"Defines create and update timestamp attributes.\n\nShorthand for `d:Ash.Resource.Dsl.attributes.create_timestamp` and\n`d:Ash.Resource.Dsl.attributes.update_timestamp` with the attribute names\n`:inserted_at` and `:updated_at` respectively.  Any options passed to this\nhelper are passed to both timestamp macros.","ref":"Ash.Resource.html#timestamps/1"},{"type":"function","title":"Ash.Resource.unload/2","doc":"Sets a loaded key or path to a key back to its original unloaded stated","ref":"Ash.Resource.html#unload/2"},{"type":"function","title":"Ash.Resource.unload_many/2","doc":"Sets a list of loaded key or paths to a key back to their original unloaded stated","ref":"Ash.Resource.html#unload_many/2"},{"type":"module","title":"Ash.DataLayer.Ets.Info","doc":"Introspection helpers for the Ets data layer","ref":"Ash.DataLayer.Ets.Info.html"},{"type":"function","title":"Ash.DataLayer.Ets.Info.private?/1","doc":"Whether or not the ets table for the resource should be private","ref":"Ash.DataLayer.Ets.Info.html#private?/1"},{"type":"function","title":"Ash.DataLayer.Ets.Info.table/1","doc":"The ets table name for a resource","ref":"Ash.DataLayer.Ets.Info.html#table/1"},{"type":"module","title":"Ash.DataLayer.Mnesia.Info","doc":"Introspection helpers for Ash.DataLayer.Mnesia","ref":"Ash.DataLayer.Mnesia.Info.html"},{"type":"function","title":"Ash.DataLayer.Mnesia.Info.table/1","doc":"The mnesia table for a resource","ref":"Ash.DataLayer.Mnesia.Info.html#table/1"},{"type":"module","title":"Ash.Domain.Info","doc":"Introspection tools for Ash.Domain","ref":"Ash.Domain.Info.html"},{"type":"function","title":"Ash.Domain.Info.allow/1","doc":"The allow MFA for a domain","ref":"Ash.Domain.Info.html#allow/1"},{"type":"function","title":"Ash.Domain.Info.allow_unregistered?/1","doc":"Whether or not the domain allows unregistered resources to be used with it","ref":"Ash.Domain.Info.html#allow_unregistered?/1"},{"type":"function","title":"Ash.Domain.Info.authorize/1","doc":"When authorization should happen for a given domain","ref":"Ash.Domain.Info.html#authorize/1"},{"type":"macro","title":"Ash.Domain.Info.depend_on_resources/1","doc":"Gets the resources of a domain module. Can be used at compile time.\n\nLiberal use of this can greatly increase compile times, or even cause compiler deadlocks.\nUse with care.","ref":"Ash.Domain.Info.html#depend_on_resources/1"},{"type":"function","title":"Ash.Domain.Info.description/1","doc":"The description of the domain","ref":"Ash.Domain.Info.html#description/1"},{"type":"function","title":"Ash.Domain.Info.find_manage_relationships_with_identity_not_configured/1","doc":"","ref":"Ash.Domain.Info.html#find_manage_relationships_with_identity_not_configured/1"},{"type":"function","title":"Ash.Domain.Info.related_domain/3","doc":"Determine what domain to use when interacting with a related resource.\n\nWe choose the first domain found in the following order:\n\n* `relationship.domain`, i.e an explicitly configured domain for a relationship\n* `resource.domain`, i.e. the domain the resource declares\n* `subject.domain`, i.e. the domain of the query, changeset or action input (if it has one)\n* `default`, the default domain provided as the third argument","ref":"Ash.Domain.Info.html#related_domain/3"},{"type":"function","title":"Ash.Domain.Info.require_actor?/1","doc":"Whether or not the actor is always required for a domain","ref":"Ash.Domain.Info.html#require_actor?/1"},{"type":"function","title":"Ash.Domain.Info.resource/2","doc":"Returns `{:ok, resource}` if the resource can be used by the domain, or `{:error, error}`.","ref":"Ash.Domain.Info.html#resource/2"},{"type":"function","title":"Ash.Domain.Info.resource_references/1","doc":"Gets the resource references of a domain module. DO NOT USE AT COMPILE TIME.\n\nIf you need the resource list at compile time, use `depend_on_resources/1`","ref":"Ash.Domain.Info.html#resource_references/1"},{"type":"function","title":"Ash.Domain.Info.resources/1","doc":"Gets the resources of a domain module.","ref":"Ash.Domain.Info.html#resources/1"},{"type":"function","title":"Ash.Domain.Info.short_name/1","doc":"The short name for a domain","ref":"Ash.Domain.Info.html#short_name/1"},{"type":"function","title":"Ash.Domain.Info.span_name/3","doc":"The span_name for a domain and resource combination","ref":"Ash.Domain.Info.html#span_name/3"},{"type":"function","title":"Ash.Domain.Info.telemetry_event_name/2","doc":"Names a telemetry event for a given domain/resource combo","ref":"Ash.Domain.Info.html#telemetry_event_name/2"},{"type":"function","title":"Ash.Domain.Info.timeout/1","doc":"The execution timeout for a domain","ref":"Ash.Domain.Info.html#timeout/1"},{"type":"function","title":"Ash.Domain.Info.trace_name/1","doc":"The trace name for a domain","ref":"Ash.Domain.Info.html#trace_name/1"},{"type":"module","title":"Ash.Notifier.PubSub.Info","doc":"Introspection helpers for Ash.Notifier.PubSub","ref":"Ash.Notifier.PubSub.Info.html"},{"type":"function","title":"Ash.Notifier.PubSub.Info.broadcast_type/1","doc":"The broadcast type for a resource","ref":"Ash.Notifier.PubSub.Info.html#broadcast_type/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.delimiter/1","doc":"The delimiter to use when generating message topics","ref":"Ash.Notifier.PubSub.Info.html#delimiter/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.filter/1","doc":"The pubsub filter for a resource","ref":"Ash.Notifier.PubSub.Info.html#filter/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.module/1","doc":"The pubsub module for a resource","ref":"Ash.Notifier.PubSub.Info.html#module/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.name/1","doc":"The pubsub name for a resource","ref":"Ash.Notifier.PubSub.Info.html#name/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.prefix/1","doc":"The topic prefix for a resource","ref":"Ash.Notifier.PubSub.Info.html#prefix/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.publications/1","doc":"The list of publications for a resource","ref":"Ash.Notifier.PubSub.Info.html#publications/1"},{"type":"function","title":"Ash.Notifier.PubSub.Info.transform/1","doc":"The transform for a resource","ref":"Ash.Notifier.PubSub.Info.html#transform/1"},{"type":"module","title":"Ash.Policy.Info","doc":"An authorization extension for ash resources.\n\nFor more information, see `Ash.Policy.Authorizer`","ref":"Ash.Policy.Info.html"},{"type":"function","title":"Ash.Policy.Info.default_access_type/1","doc":"","ref":"Ash.Policy.Info.html#default_access_type/1"},{"type":"function","title":"Ash.Policy.Info.describe_resource/2","doc":"","ref":"Ash.Policy.Info.html#describe_resource/2"},{"type":"function","title":"Ash.Policy.Info.field_policies/1","doc":"","ref":"Ash.Policy.Info.html#field_policies/1"},{"type":"function","title":"Ash.Policy.Info.field_policies_for_field/2","doc":"Gets the field policies relevant to a given field","ref":"Ash.Policy.Info.html#field_policies_for_field/2"},{"type":"function","title":"Ash.Policy.Info.log_policy_breakdowns/0","doc":"The log level at which Ash policy authorizer logs policy breakdowns. Defaults to none.","ref":"Ash.Policy.Info.html#log_policy_breakdowns/0"},{"type":"function","title":"Ash.Policy.Info.log_successful_policy_breakdowns/0","doc":"The log level at which Ash policy authorizer logs successful policy breakdowns. Defaults to none.","ref":"Ash.Policy.Info.html#log_successful_policy_breakdowns/0"},{"type":"function","title":"Ash.Policy.Info.policies/2","doc":"","ref":"Ash.Policy.Info.html#policies/2"},{"type":"function","title":"Ash.Policy.Info.private_fields_policy/1","doc":"","ref":"Ash.Policy.Info.html#private_fields_policy/1"},{"type":"function","title":"Ash.Policy.Info.show_policy_breakdowns?/0","doc":"Whether or not Ash policy authorizer is configured to show policy breakdowns in error messages","ref":"Ash.Policy.Info.html#show_policy_breakdowns?/0"},{"type":"function","title":"Ash.Policy.Info.strict_check/3","doc":"A utility to determine if a given query/changeset would pass authorization.\n\n*This is still experimental.*","ref":"Ash.Policy.Info.html#strict_check/3"},{"type":"module","title":"Ash.Resource.Info","doc":"Introspection for resources","ref":"Ash.Resource.Info.html"},{"type":"function","title":"Ash.Resource.Info.action/3","doc":"Returns the action with the matching name and type on the resource","ref":"Ash.Resource.Info.html#action/3"},{"type":"function","title":"Ash.Resource.Info.action_input?/3","doc":"Returns true or false if the input is accepted by the action, as an argument or an attribute","ref":"Ash.Resource.Info.html#action_input?/3"},{"type":"function","title":"Ash.Resource.Info.action_inputs/2","doc":"Returns the list of possible accepted keys by an action","ref":"Ash.Resource.Info.html#action_inputs/2"},{"type":"function","title":"Ash.Resource.Info.action_select/2","doc":"Returns the list of attributes that must be selected for an action invocation","ref":"Ash.Resource.Info.html#action_select/2"},{"type":"function","title":"Ash.Resource.Info.actions/1","doc":"Returns all actions of a resource","ref":"Ash.Resource.Info.html#actions/1"},{"type":"function","title":"Ash.Resource.Info.aggregate/2","doc":"Get an aggregate by name","ref":"Ash.Resource.Info.html#aggregate/2"},{"type":"function","title":"Ash.Resource.Info.aggregate_type/2","doc":"Gets the type of an aggregate for a given resource.","ref":"Ash.Resource.Info.html#aggregate_type/2"},{"type":"function","title":"Ash.Resource.Info.aggregates/1","doc":"Returns all aggregates of a resource","ref":"Ash.Resource.Info.html#aggregates/1"},{"type":"function","title":"Ash.Resource.Info.always_selected_attribute_names/1","doc":"","ref":"Ash.Resource.Info.html#always_selected_attribute_names/1"},{"type":"function","title":"Ash.Resource.Info.attribute/2","doc":"Get an attribute name from the resource","ref":"Ash.Resource.Info.html#attribute/2"},{"type":"function","title":"Ash.Resource.Info.attribute_names/1","doc":"","ref":"Ash.Resource.Info.html#attribute_names/1"},{"type":"function","title":"Ash.Resource.Info.attributes/1","doc":"Returns all attributes of a resource","ref":"Ash.Resource.Info.html#attributes/1"},{"type":"function","title":"Ash.Resource.Info.attributes_to_require/1","doc":"","ref":"Ash.Resource.Info.html#attributes_to_require/1"},{"type":"function","title":"Ash.Resource.Info.attributes_to_require/2","doc":"","ref":"Ash.Resource.Info.html#attributes_to_require/2"},{"type":"function","title":"Ash.Resource.Info.authorizers/1","doc":"A list of authorizers to be used when accessing","ref":"Ash.Resource.Info.html#authorizers/1"},{"type":"function","title":"Ash.Resource.Info.base_filter/1","doc":"The base filter of the resource","ref":"Ash.Resource.Info.html#base_filter/1"},{"type":"function","title":"Ash.Resource.Info.calculation/2","doc":"Get a calculation by name","ref":"Ash.Resource.Info.html#calculation/2"},{"type":"function","title":"Ash.Resource.Info.calculation_interface/2","doc":"Get an calculation interface by name from the resource","ref":"Ash.Resource.Info.html#calculation_interface/2"},{"type":"function","title":"Ash.Resource.Info.calculation_interfaces/1","doc":"The list of code interface calculation definitions.","ref":"Ash.Resource.Info.html#calculation_interfaces/1"},{"type":"function","title":"Ash.Resource.Info.calculations/1","doc":"Returns all calculations of a resource","ref":"Ash.Resource.Info.html#calculations/1"},{"type":"function","title":"Ash.Resource.Info.changes/1","doc":"A list of all changes for the resource","ref":"Ash.Resource.Info.html#changes/1"},{"type":"function","title":"Ash.Resource.Info.changes/2","doc":"A list of all changes for the resource for a given action type","ref":"Ash.Resource.Info.html#changes/2"},{"type":"function","title":"Ash.Resource.Info.code_interface_domain/1","doc":"The domain to define the interface for, when defining it in the resource","ref":"Ash.Resource.Info.html#code_interface_domain/1"},{"type":"function","title":"Ash.Resource.Info.data_layer/1","doc":"The data layer of the resource, or nil if it does not have one","ref":"Ash.Resource.Info.html#data_layer/1"},{"type":"function","title":"Ash.Resource.Info.default_actions/1","doc":"Returns the configured default actions","ref":"Ash.Resource.Info.html#default_actions/1"},{"type":"function","title":"Ash.Resource.Info.default_context/1","doc":"The default context of the resource","ref":"Ash.Resource.Info.html#default_context/1"},{"type":"function","title":"Ash.Resource.Info.define_interface?/1","doc":"Whether or not to define the interface on the resource","ref":"Ash.Resource.Info.html#define_interface?/1"},{"type":"function","title":"Ash.Resource.Info.description/1","doc":"The description of the resource","ref":"Ash.Resource.Info.html#description/1"},{"type":"function","title":"Ash.Resource.Info.domain/1","doc":"Returns the statically configured domain for the resource.","ref":"Ash.Resource.Info.html#domain/1"},{"type":"function","title":"Ash.Resource.Info.embedded?/1","doc":"Whether or not the resource is an embedded resource","ref":"Ash.Resource.Info.html#embedded?/1"},{"type":"function","title":"Ash.Resource.Info.field/2","doc":"Get a field from a resource by name","ref":"Ash.Resource.Info.html#field/2"},{"type":"function","title":"Ash.Resource.Info.fields/2","doc":"Returns all attributes, aggregates, calculations and relationships of a resource","ref":"Ash.Resource.Info.html#fields/2"},{"type":"function","title":"Ash.Resource.Info.hide_inspect_fields/1","doc":"A deny-list of fields to hide from the inspect output.\nTakes precedence over `show_inspect_fields`.","ref":"Ash.Resource.Info.html#hide_inspect_fields/1"},{"type":"function","title":"Ash.Resource.Info.identities/1","doc":"A list of identities for the resource","ref":"Ash.Resource.Info.html#identities/1"},{"type":"function","title":"Ash.Resource.Info.identity/2","doc":"Get an identity by name from the resource","ref":"Ash.Resource.Info.html#identity/2"},{"type":"function","title":"Ash.Resource.Info.inspect_private_fields?/1","doc":"Whether to include private fields in the inspect output.\n`show_inspect_fields` takes precedence over this option.","ref":"Ash.Resource.Info.html#inspect_private_fields?/1"},{"type":"function","title":"Ash.Resource.Info.interface/2","doc":"Get an interface by name from the resource","ref":"Ash.Resource.Info.html#interface/2"},{"type":"function","title":"Ash.Resource.Info.interfaces/1","doc":"The list of code interface definitions.","ref":"Ash.Resource.Info.html#interfaces/1"},{"type":"function","title":"Ash.Resource.Info.lazy_matching_default_attributes/2","doc":"Returns all attributes of a resource with lazy matching defaults","ref":"Ash.Resource.Info.html#lazy_matching_default_attributes/2"},{"type":"function","title":"Ash.Resource.Info.lazy_non_matching_default_attributes/2","doc":"Returns all attributes of a resource with lazy non-matching-defaults","ref":"Ash.Resource.Info.html#lazy_non_matching_default_attributes/2"},{"type":"function","title":"Ash.Resource.Info.multitenancy_attribute/1","doc":"The multitenancy attribute for a resource","ref":"Ash.Resource.Info.html#multitenancy_attribute/1"},{"type":"function","title":"Ash.Resource.Info.multitenancy_global?/1","doc":"The MFA to parse the tenant from the attribute","ref":"Ash.Resource.Info.html#multitenancy_global?/1"},{"type":"function","title":"Ash.Resource.Info.multitenancy_parse_attribute/1","doc":"The function to parse the tenant from the attribute","ref":"Ash.Resource.Info.html#multitenancy_parse_attribute/1"},{"type":"function","title":"Ash.Resource.Info.multitenancy_strategy/1","doc":"The multitenancy strategy for a resource","ref":"Ash.Resource.Info.html#multitenancy_strategy/1"},{"type":"function","title":"Ash.Resource.Info.multitenancy_template/1","doc":"The template for creating the tenant name","ref":"Ash.Resource.Info.html#multitenancy_template/1"},{"type":"function","title":"Ash.Resource.Info.notifiers/1","doc":"A list of notifiers to be used when accessing","ref":"Ash.Resource.Info.html#notifiers/1"},{"type":"function","title":"Ash.Resource.Info.plural_name/1","doc":"The plural_name of the resource","ref":"Ash.Resource.Info.html#plural_name/1"},{"type":"function","title":"Ash.Resource.Info.preparations/1","doc":"","ref":"Ash.Resource.Info.html#preparations/1"},{"type":"function","title":"Ash.Resource.Info.primary_action/2","doc":"Returns the primary action of a given type","ref":"Ash.Resource.Info.html#primary_action/2"},{"type":"function","title":"Ash.Resource.Info.primary_action!/2","doc":"Returns the primary action of the given type","ref":"Ash.Resource.Info.html#primary_action!/2"},{"type":"function","title":"Ash.Resource.Info.primary_key/1","doc":"A list of field names corresponding to the primary key","ref":"Ash.Resource.Info.html#primary_key/1"},{"type":"function","title":"Ash.Resource.Info.primary_key_simple_equality?/1","doc":"Whether or not all primary key attributes can be compared with simple_equality","ref":"Ash.Resource.Info.html#primary_key_simple_equality?/1"},{"type":"function","title":"Ash.Resource.Info.public_aggregate/2","doc":"Get an aggregate by name","ref":"Ash.Resource.Info.html#public_aggregate/2"},{"type":"function","title":"Ash.Resource.Info.public_aggregates/1","doc":"Returns all public aggregates of a resource","ref":"Ash.Resource.Info.html#public_aggregates/1"},{"type":"function","title":"Ash.Resource.Info.public_attribute/2","doc":"Get a public attribute name from the resource","ref":"Ash.Resource.Info.html#public_attribute/2"},{"type":"function","title":"Ash.Resource.Info.public_attributes/1","doc":"Returns all public attributes of a resource","ref":"Ash.Resource.Info.html#public_attributes/1"},{"type":"function","title":"Ash.Resource.Info.public_calculation/2","doc":"Get a public calculation by name","ref":"Ash.Resource.Info.html#public_calculation/2"},{"type":"function","title":"Ash.Resource.Info.public_calculations/1","doc":"Returns all public calculations of a resource","ref":"Ash.Resource.Info.html#public_calculations/1"},{"type":"function","title":"Ash.Resource.Info.public_field/2","doc":"Get a public field from a resource by name","ref":"Ash.Resource.Info.html#public_field/2"},{"type":"function","title":"Ash.Resource.Info.public_fields/1","doc":"Returns all public attributes, aggregates, calculations and relationships of a resource","ref":"Ash.Resource.Info.html#public_fields/1"},{"type":"function","title":"Ash.Resource.Info.public_relationship/2","doc":"Get a public relationship by name or path","ref":"Ash.Resource.Info.html#public_relationship/2"},{"type":"function","title":"Ash.Resource.Info.public_relationships/1","doc":"Returns all public relationships of a resource","ref":"Ash.Resource.Info.html#public_relationships/1"},{"type":"function","title":"Ash.Resource.Info.related/2","doc":"","ref":"Ash.Resource.Info.html#related/2"},{"type":"function","title":"Ash.Resource.Info.relationship/2","doc":"Get a relationship by name or path","ref":"Ash.Resource.Info.html#relationship/2"},{"type":"function","title":"Ash.Resource.Info.relationships/1","doc":"Returns all relationships of a resource","ref":"Ash.Resource.Info.html#relationships/1"},{"type":"function","title":"Ash.Resource.Info.required_belongs_to_relationships/1","doc":"The required belongs_to relationships","ref":"Ash.Resource.Info.html#required_belongs_to_relationships/1"},{"type":"function","title":"Ash.Resource.Info.resource?/1","doc":"Whether or not a given module is a resource module","ref":"Ash.Resource.Info.html#resource?/1"},{"type":"function","title":"Ash.Resource.Info.reverse_relationship/3","doc":"Retrieves a relationship path from the resource related by path, to the provided resource.","ref":"Ash.Resource.Info.html#reverse_relationship/3"},{"type":"function","title":"Ash.Resource.Info.selected_by_default_attribute_names/1","doc":"","ref":"Ash.Resource.Info.html#selected_by_default_attribute_names/1"},{"type":"function","title":"Ash.Resource.Info.short_name/1","doc":"The short_name of the resource","ref":"Ash.Resource.Info.html#short_name/1"},{"type":"function","title":"Ash.Resource.Info.show_inspect_fields/1","doc":"An allow-list of fields to show in the inspect output.","ref":"Ash.Resource.Info.html#show_inspect_fields/1"},{"type":"function","title":"Ash.Resource.Info.simple_notifiers/1","doc":"A list of simple notifiers (require no DSL, used to avoid compile time dependencies)","ref":"Ash.Resource.Info.html#simple_notifiers/1"},{"type":"function","title":"Ash.Resource.Info.sortable?/3","doc":"Determine if a field is sortable by name","ref":"Ash.Resource.Info.html#sortable?/3"},{"type":"function","title":"Ash.Resource.Info.static_default_attributes/2","doc":"Returns all attributes of a resource with static defaults","ref":"Ash.Resource.Info.html#static_default_attributes/2"},{"type":"function","title":"Ash.Resource.Info.trace_name/1","doc":"The trace_name of the resource","ref":"Ash.Resource.Info.html#trace_name/1"},{"type":"function","title":"Ash.Resource.Info.unique_keys/1","doc":"A list of unique keys and information for a resource","ref":"Ash.Resource.Info.html#unique_keys/1"},{"type":"function","title":"Ash.Resource.Info.validations/1","doc":"A list of all validations for the resource","ref":"Ash.Resource.Info.html#validations/1"},{"type":"function","title":"Ash.Resource.Info.validations/2","doc":"A list of all validations for the resource for a given action type","ref":"Ash.Resource.Info.html#validations/2"},{"type":"module","title":"Ash.Domain.Info.Diagram","doc":"Generate Mermaid diagrams from a specified domain.","ref":"Ash.Domain.Info.Diagram.html"},{"type":"module","title":"Limitations - Ash.Domain.Info.Diagram","doc":"We can't easily model Ash relationships with Mermaid diagrams\nbecause they are unidirectional and could be asymmetric.\nMermaid assumes symmetrical, bidirectional relationships.\nIf we try to model all unidirectional relationships as separate\nlines in the diagram it gets very hard to read very quickly.","ref":"Ash.Domain.Info.Diagram.html#module-limitations"},{"type":"function","title":"Ash.Domain.Info.Diagram.mermaid_class_diagram/2","doc":"Generates a Mermaid Class Diagram for a given domain.\n\nShows only public attributes, calculations, aggregates and actions.\nShows a connecting line for relationships with the type of relationship\nindicated in the attribute list.","ref":"Ash.Domain.Info.Diagram.html#mermaid_class_diagram/2"},{"type":"function","title":"Ash.Domain.Info.Diagram.mermaid_er_diagram/2","doc":"Generates a Mermaid Entity Relationship Diagram for a given domain or list of domains.\n\nShows only public attributes, calculations, aggregates and actions.\nShows a one-to-one line for relationships as enumerating all unidirectional\nrelationships is far too noisy.","ref":"Ash.Domain.Info.Diagram.html#mermaid_er_diagram/2"},{"type":"module","title":"Ash.Domain.Info.Livebook","doc":"Generate a Livebook from a specified domain.","ref":"Ash.Domain.Info.Livebook.html"},{"type":"function","title":"Ash.Domain.Info.Livebook.action_header/0","doc":"","ref":"Ash.Domain.Info.Livebook.html#action_header/0"},{"type":"function","title":"Ash.Domain.Info.Livebook.action_input_section/2","doc":"","ref":"Ash.Domain.Info.Livebook.html#action_input_section/2"},{"type":"function","title":"Ash.Domain.Info.Livebook.action_section/2","doc":"","ref":"Ash.Domain.Info.Livebook.html#action_section/2"},{"type":"function","title":"Ash.Domain.Info.Livebook.attr_header/0","doc":"","ref":"Ash.Domain.Info.Livebook.html#attr_header/0"},{"type":"function","title":"Ash.Domain.Info.Livebook.attr_section/1","doc":"","ref":"Ash.Domain.Info.Livebook.html#attr_section/1"},{"type":"function","title":"Ash.Domain.Info.Livebook.domain_section/1","doc":"","ref":"Ash.Domain.Info.Livebook.html#domain_section/1"},{"type":"function","title":"Ash.Domain.Info.Livebook.overview/1","doc":"","ref":"Ash.Domain.Info.Livebook.html#overview/1"},{"type":"function","title":"Ash.Domain.Info.Livebook.resource_section/1","doc":"","ref":"Ash.Domain.Info.Livebook.html#resource_section/1"},{"type":"module","title":"Ash.Policy.Chart.Mermaid","doc":"Generates policy mermaid charts","ref":"Ash.Policy.Chart.Mermaid.html"},{"type":"function","title":"Ash.Policy.Chart.Mermaid.chart/1","doc":"","ref":"Ash.Policy.Chart.Mermaid.html#chart/1"},{"type":"module","title":"Ash.Generator","doc":"Tools for generating input to Ash resource actions and for generating seed data.","ref":"Ash.Generator.html"},{"type":"module","title":"Using Ash.Generator - Ash.Generator","doc":"To define generators for your tests, `use Ash.Generator`, and define\nfunctions that use `changeset_generator/3` and/or `seed_generator/2`.\n\n```elixir\ndefmodule YourApp.Generator do\n  use Ash.Generator\n\n  # using `seed_generator`, bypasses the action and saves directly to the data layer\n  def blog_post(opts \\\\ []) do\n    seed_generator(\n      %MyApp.Blog.Post{\n        name: sequence(:title, &\"My Blog Post #{&1}\")\n        text: StreamData.repeatedly(fn -> Faker.Lorem.paragraph() end)\n      },\n      overrides: opts\n    )\n  end\n\n  # using `changeset_generator`, calls the action when passed to `generate`\n  def blog_post_comment(opts \\\\ []) do\n    blog_post_id = opts[:blog_post_id] || once(:default_blog_post_id, fn -> generate(blog_post()).id end)\n\n    changeset_generator(\n      MyApp.Blog.Comment,\n      :create,\n      defaults: [\n        blog_post_id: blog_post_id\n      ],\n      overrides: opts\n    )\n  end\nend\n```\n\nThen, in your tests, you can `import YourApp.Generator`, and use `generate/1` and `generate_many/1` to generate data.\nFor example:\n\n```elixir\nimport YourApp.Generator\n\ntest \"`comment_count` on blog_post shows the count of comments\" do\n  blog_post = generate(blog_post())\n  assert Ash.load!(blog_post, :comment_count).comment_count == 0\n\n  generate_many(blog_post_comment(blog_post_id: blog_post.id), 10)\n\n  assert Ash.load!(blog_post, :comment_count).comment_count == 10\nend\n```","ref":"Ash.Generator.html#module-using-ash-generator"},{"type":"module","title":"About Generators - Ash.Generator","doc":"These generators are backed by `StreamData`, and are ready for use with property testing via `ExUnitProperties`\n\nMany functions in this module support \"overrides\", which allow passing down either constant values\nor your own `StreamData` generators.\n\nFor example:\n\n```elixir\n# All generated posts will have text as `\"text\"`. Equivalent to providing `StreamData.constant(\"text\")`.\nAsh.Generator.seed_input(Post, %{text: \"text\"})\n```","ref":"Ash.Generator.html#module-about-generators"},{"type":"function","title":"Ash.Generator.action_input/3","doc":"Generate input meant to be passed into a resource action.\n\nArguments that are passed to a `manage_relationship` are not generated by default, and you will\nhave to generate them yourself by passing your own generators/values down. See the module documentation for more.","ref":"Ash.Generator.html#action_input/3"},{"type":"function","title":"Ash.Generator.changeset/4","doc":"Creates the input for the provided action with `action_input/3`, and creates a changeset for that action with that input.\n\nSee `action_input/3` and the module documentation for more.","ref":"Ash.Generator.html#changeset/4"},{"type":"function","title":"Ash.Generator.changeset_generator/3","doc":"A generator of changesets which call their specific actions when passed to `generate/1` or `generate_many/2`.\n\nSee `seed_generator/2` for the equivalent construct for cases when you want to seed directly to the data layer as opposed to calling resource\nactions.","ref":"Ash.Generator.html#changeset_generator/3"},{"type":"function","title":"Examples - Ash.Generator.changeset_generator/3","doc":"```elixir\niex> changeset_generator(MyApp.Blog.Post, :create, defaults: [title: sequence(:blog_post_title, &\"My Blog Post #{&1}\")]) |> generate()\n%Ash.Changeset{...}\n```","ref":"Ash.Generator.html#changeset_generator/3-examples"},{"type":"function","title":"Usage in tests - Ash.Generator.changeset_generator/3","doc":"This can be used to define generators in tests. A useful pattern is defining a function like so:\n\n```elixir\ndef blog_post(opts \\ []) do\n  changeset_generator(\n    MyApp.Blog.Post,\n    :create,\n    defaults: [\n      name: sequence(:blog_post_title, &\"My Blog Post #{&1}\")\n      text: StreamData.repeatedly(fn -> Faker.Lorem.paragraph() end)\n    ],\n    overrides: opts\n  )\nend\n```\n\nWhen you only allow child resource to be created through a managed relationship, e.g. an update action on a parent resource,\nthis pattern could be expanded, yielding a resource with a new child resource:\n\n```elixir\ndef post_for(author, opts \\ []) do\n  changeset_generator(\n    author,\n    :new_post,\n    uses: [\n      post_input:\n        action_input(\n          MyApp.Blog.Post,\n          :create,\n          title: sequence(:title, &\"Post #{&1}\")\n        )\n    ],\n    defaults: fn inputs ->\n      [posts: [inputs.post_input]]\n    end,\n    overrides: opts\n  )\nend\n```\n\nSee the `Ash.Generator` moduledocs for more information.","ref":"Ash.Generator.html#changeset_generator/3-usage-in-tests"},{"type":"function","title":"Options - Ash.Generator.changeset_generator/3","doc":"* `:defaults` - A keyword list of values or generators, used as inputs. Can also be a function\n  when using the `:uses` option.\n* `:overrides` - A keyword list or map of `t:overrides()`\n* `:actor` - Passed through to the changeset\n* `:tenant` - Passed through to the changeset\n* `:uses` - A map of generators that are passed into your `defaults`. `defaults` must be a\n  function. This is useful when multiple things in your `defaults` need to use the same generated\n  value.\n* `:authorize?` - Passed through to the changeset\n* `:context` - Passed through to the changeset\n* `:after_action` - A one argument function that takes the result and returns\n  a new result to run after the record is created.\n* `:private_arguments` - A map of private arguments, whos values can also be generators. Can also\n  be a function when using the `:uses` option.","ref":"Ash.Generator.html#changeset_generator/3-options"},{"type":"function","title":"The uses option - Ash.Generator.changeset_generator/3","doc":"```elixir\ndef blog_post(opts \\ []) do\n  changeset_generator(\n    MyApp.Blog.Post,\n    :create,\n    uses: [\n      author: author() # A function using `changeset_generator` just like this one.\n    ],\n    defaults: fn %{author: author} ->\n      author = generate(author)\n\n      [\n        name: sequence(:blog_post_title, &\"My Blog Post #{&1}\")\n        author_name: author.name,\n        text: StreamData.repeatedly(fn -> Faker.Lorem.paragraph() end)\n      ]\n    end\n    overrides: opts\n  )\nend\n```","ref":"Ash.Generator.html#changeset_generator/3-the-uses-option"},{"type":"function","title":"Ash.Generator.generate/1","doc":"Takes one value from a changeset or seed generator and calls `Ash.create!` or `Ash.update!` on it.\n\nPasses through resource structs without doing anything.\nCreates a changeset if given","ref":"Ash.Generator.html#generate/1"},{"type":"function","title":"Ash.Generator.generate_many/2","doc":"Takes `count` values from a changeset or seed generator and passes their inputs into `Ash.bulk_create!` or `Ash.Seed.seed!` respectively.","ref":"Ash.Generator.html#generate_many/2"},{"type":"function","title":"Ash.Generator.initialize_once/1","doc":"Starts and links an agent for a `once/2`, or returns the existing agent pid if it already exists.\n\nSee `once/2` for more.","ref":"Ash.Generator.html#initialize_once/1"},{"type":"function","title":"Ash.Generator.initialize_sequence/1","doc":"Starts and links an agent for a sequence, or returns the existing agent pid if it already exists.\n\nSee `sequence/3` for more.","ref":"Ash.Generator.html#initialize_sequence/1"},{"type":"function","title":"Ash.Generator.many_changesets/5","doc":"Generate `count` changesets and return them as a list.","ref":"Ash.Generator.html#many_changesets/5"},{"type":"function","title":"Ash.Generator.many_queries/5","doc":"Generate `count` queries and return them as a list.","ref":"Ash.Generator.html#many_queries/5"},{"type":"function","title":"Ash.Generator.mixed_map/2","doc":"Creates a generator of maps where all keys are required except the list provided","ref":"Ash.Generator.html#mixed_map/2"},{"type":"function","title":"Example - Ash.Generator.mixed_map/2","doc":"```elixir\niex> mixed_map(%{a: StreamData.constant(1), b: StreamData.constant(2)}, [:b]) |> Enum.take(2)\n[%{a: 1}, %{a: 1, b: 2}]\n```","ref":"Ash.Generator.html#mixed_map/2-example"},{"type":"function","title":"Ash.Generator.next_in_sequence/3","doc":"Gets the next value for a given sequence identifier.\n\nSee `sequence/3` for more.\n\nThis is equivalent to `identifier |> Ash.Generator.sequence(fun, sequencer) |> Enum.at(0)`","ref":"Ash.Generator.html#next_in_sequence/3"},{"type":"function","title":"Ash.Generator.once/2","doc":"Run the provided function or enumerable (i.e generator) only once.\n\nThis is useful for ensuring that some piece of data is generated a single time during a test.\n\nThe lifecycle of this generator is tied to the process that initially starts it. In general,\nthat will be the test. In the rare case where you are running async processes that need to share a sequence\nthat is not created in the test process, you can initialize a sequence in the test using `initialize_once/1`.\n\nExample:\n\n    iex> Ash.Generator.once(:user, fn ->\n           register_user(...)\n         end) |> Enum.at(0)\n    %User{id: 1} # created the user\n\n    iex> Ash.Generator.once(:user, fn ->\n           register_user(...)\n         end) |> Enum.at(0)\n    %User{id: 1} # reused the last user","ref":"Ash.Generator.html#once/2"},{"type":"type","title":"Ash.Generator.overrides/0","doc":"A map or keyword of data generators or constant values to use in place of defaults.\n\nMany functions in `Ash.Generator` support `overrides`, allowing to customize the default\ngenerated values.","ref":"Ash.Generator.html#t:overrides/0"},{"type":"function","title":"Ash.Generator.query/4","doc":"Creates the input for the provided action with `action_input/3`, and returns a query for that action with that input.\n\nSee `action_input/3` and the module documentation for more.","ref":"Ash.Generator.html#query/4"},{"type":"function","title":"Ash.Generator.seed!/2","doc":"Gets input using `seed_input/2` and passes it to `Ash.Seed.seed!/2`, returning the result","ref":"Ash.Generator.html#seed!/2"},{"type":"function","title":"Ash.Generator.seed_generator/2","doc":"A generator of seedable records, to be passed to `generate/1` or `generate_many/1`\n\nSee `changeset_generator/3` for the equivalent construct for cases when you want to call resource\nactions as opposed to seed directly to the data layer.\n\nWhen a struct is given, only exactly the given values/generators will be used. If you\npass a tuple, i.e `{Resource, %{field: :value}}`, all values not provided will be generated\nautomatically.","ref":"Ash.Generator.html#seed_generator/2"},{"type":"function","title":"Examples - Ash.Generator.seed_generator/2","doc":"```elixir\niex> seed_generator(%MyApp.Blog.Post{name: sequence(:blog_post_title, &\"My Blog Post #{&1}\")}) |> generate()\n%Tunez.Music.Artist{name: \"Artist 1\"}\n\niex> seed_generator({MyApp.Blog.Post, %{}}) |> generate()\n%Tunez.Music.Artist{name: \"A random name\"}\n```","ref":"Ash.Generator.html#seed_generator/2-examples"},{"type":"function","title":"Usage in tests - Ash.Generator.seed_generator/2","doc":"This can be used to define seed generators in tests. A useful pattern is defining a function like so:\n\n```elixir\ndef blog_post(opts \\ []) do\n  seed_generator(\n    %MyApp.Blog.Post{\n      name: sequence(:blog_post_title, &\"My Blog Post #{&1}\")\n      text: StreamData.repeatedly(fn -> Faker.Lorem.paragraph() end)\n    },\n    overrides: opts\n  )\nend\n```\n\nSee the `Ash.Generator` moduledocs for more information.","ref":"Ash.Generator.html#seed_generator/2-usage-in-tests"},{"type":"function","title":"Options - Ash.Generator.seed_generator/2","doc":"* `:overrides` - A keyword list or map of `t:overrides()`\n* `:actor` - Passed through to the changeset\n* `:tenant` - Passed through to the changeset\n* `:uses` - A map of generators that are passed into the first argument, if it is a function.\n* `:authorize?` - Passed through to the changeset\n* `:context` - Passed through to the changeset\n* `:after_action` - A one argument function that takes the result and returns\n  a new result to run after the record is created.","ref":"Ash.Generator.html#seed_generator/2-options"},{"type":"function","title":"Ash.Generator.seed_input/2","doc":"Generate input meant to be passed into `Ash.Seed.seed!/2`.\n\nA map of custom `StreamData` generators can be provided to add to or overwrite the generated input,\nfor example: `Ash.Generator.seed_input(Post, %{text: StreamData.constant(\"Post\")})`","ref":"Ash.Generator.html#seed_input/2"},{"type":"function","title":"Ash.Generator.seed_many!/3","doc":"Generates an input `n` times, and passes them all to seed, returning the list of seeded items.","ref":"Ash.Generator.html#seed_many!/3"},{"type":"function","title":"Ash.Generator.sequence/3","doc":"Generate globally unique values.\n\nThis is useful for generating values that are unique within a given test or processes that it spawns, such as email addresses,\nor for generating values that are unique across a single resource, such as identifiers. The values will be unique\nfor anything using the same sequence name, **within the same test**.\n\n>","ref":"Ash.Generator.html#sequence/3"},{"type":"function","title":"Not Globally Unique - Ash.Generator.sequence/3","doc":"> The lifecycle of this generator is tied to the process that initially starts it. In general,\n> that will be the test. In the rare case where you are running async processes that need to share a sequence\n> that is not created in the test process, you can initialize a sequence in the test using `initialize_sequence/1`.\n>\n> If you need a globally unique value, use a value like `System.unique_integer([:positive])` in your values instead.\n>\n> For example:\n>\n> ```elixir\n> StreamData.repeatedly(fn -> \"email#{System.unique_integer([:positive])}@example.com\" end)\n> ```\n\nExample:\n\n    Ash.Generator.sequence(:unique_email, fn i -> \"user#{i}@example.com\" end) |> Enum.take(3)\n    iex> [\"user0@example.com\", \"user1@example.com\", \"user2@example.com\"]","ref":"Ash.Generator.html#sequence/3-not-globally-unique"},{"type":"function","title":"Using a different sequencer - Ash.Generator.sequence/3","doc":"By default we use an incrementing integer starting at 0. However, if you want to use something else, you can provide\nyour own sequencer. The initial value will be `nil`, which you can use to detect that you are the start of the sequence.\n\nExample:\n\n    Ash.Generator.sequence(:unique_email, fn i -> \"user#{i}@example.com\" end, fn num -> (num || 1) - 1 end) |> Enum.take(3)\n    iex> [\"user0@example.com\", \"user-1@example.com\", \"user-2@example.com\"]","ref":"Ash.Generator.html#sequence/3-using-a-different-sequencer"},{"type":"function","title":"Ash.Generator.stop_once/1","doc":"Stops the agent for a `once/2`.\n\nSee `once/2` for more.","ref":"Ash.Generator.html#stop_once/1"},{"type":"function","title":"Ash.Generator.stop_sequence/1","doc":"Stops the agent for a sequence.\n\nSee `sequence/3` for more.","ref":"Ash.Generator.html#stop_sequence/1"},{"type":"type","title":"Ash.Generator.stream_data/0","doc":"An instance of `StreamData`, gotten from one of the functions in that module.","ref":"Ash.Generator.html#t:stream_data/0"},{"type":"module","title":"Ash.Seed","doc":"Helpers for seeding data, useful for quickly creating lots of data either for database seeding or testing.\n\nImportant: this bypasses resource actions, and goes straight to the data layer. No action changes or validations are run.\nThe only thing that it does at the moment is ensure that default values for attributes are set, it does not validate\nthat required attributes are set (although the data layer may do that for you, e.g with ash_postgres).","ref":"Ash.Seed.html"},{"type":"function","title":"Ash.Seed.keep_nil/0","doc":"Returns `:__keep_nil__`, allowing to ensure a default value is not used when you want the value to be `nil`.","ref":"Ash.Seed.html#keep_nil/0"},{"type":"function","title":"Ash.Seed.seed!/1","doc":"Seed using a record (instance of a resource) as input.\n\nIf the passed in struct was retrieved from the data layer already (i.e already seeded),\nthen it is returned and nothing is done. Otherwise, the attributes and relationships are\nused as input to `seed/2`, after having any `%Ash.NotLoaded{}` values stripped out.\n\nAny `nil` values will be overwritten with their default values. To avoid this, either use `seed/2`\nin which providing the key will have it not set the default values.\nIf you want to force `nil` to be accepted and prevent the default value from being set, use the\n`keep_nil/0` function provided here, which returns `:__keep_nil__`. Alternatively, use\n`seed!(Post, %{text: nil})`.\n\nSee `seed!/2` for more information.","ref":"Ash.Seed.html#seed!/1"},{"type":"function","title":"Ash.Seed.seed!/3","doc":"Performs a direct call to the data layer of a resource with the provided input.\n\nIf a list is provided as input, then you will get back that many results.\n\nTo set a tenant, use the tenant option.","ref":"Ash.Seed.html#seed!/3"},{"type":"function","title":"Ash.Seed.skip/0","doc":"Returns `:__skip__`, allowing to ensure no value is generated for a given field when used with generators.","ref":"Ash.Seed.html#skip/0"},{"type":"function","title":"Ash.Seed.update!/3","doc":"Usage is the same as `seed!/2`, but it will update an existing record.\n\nFor multitenant resources, tenant will be extracted from the record if\nnot provided in opts.","ref":"Ash.Seed.html#update!/3"},{"type":"function","title":"Ash.Seed.upsert!/2","doc":"Performs an upsert operation on the data layer of a resource with the provided input and identities.\nThe usage is the same as `seed!/1`, but it will update the record if it already exists.\n\n```elixir\nAsh.Seed.upsert!(%User{email: 'test@gmail.com', name: 'Test'}, identity: :email)\n```","ref":"Ash.Seed.html#upsert!/2"},{"type":"function","title":"Ash.Seed.upsert!/3","doc":"Usage is the same as `seed!/2`, but it will update the record if it already exists based on the identities.","ref":"Ash.Seed.html#upsert!/3"},{"type":"module","title":"Ash.Test","doc":"Testing helpers for Ash.","ref":"Ash.Test.html"},{"type":"function","title":"Ash.Test.assert_has_error/4","doc":"Assert that the given changeset, query, or action input has a matching error.\n\nUse the optional second argument to assert that the errors (all together) are of a specific class.","ref":"Ash.Test.html#assert_has_error/4"},{"type":"macro","title":"Ash.Test.assert_stripped/1","doc":"A macro for comparing Ash resources while ignoring metadata differences.","ref":"Ash.Test.html#assert_stripped/1"},{"type":"macro","title":"Overview - Ash.Test.assert_stripped/1","doc":"Ash resources contain metadata fields (`__metadata__` and `__meta__`) that track internal state like\nloaded relationships and action history. These fields can cause equality comparisons to fail even\nwhen the actual data is identical. This macro uses `strip_metadata/1` to remove these fields before\ncomparison.","ref":"Ash.Test.html#assert_stripped/1-overview"},{"type":"macro","title":"Usage - Ash.Test.assert_stripped/1","doc":"Use when comparing resources that have gone through different processing paths, such as comparing\nseeded data with data that has been processed through resource actions.","ref":"Ash.Test.html#assert_stripped/1-usage"},{"type":"macro","title":"Supported Operators - Ash.Test.assert_stripped/1","doc":"* `==` and `===` - Equality comparison\n* `!=` and `!==` - Inequality comparison\n* `in` and `not in` - Membership testing","ref":"Ash.Test.html#assert_stripped/1-supported-operators"},{"type":"macro","title":"Examples - Ash.Test.assert_stripped/1","doc":"# Compare resources\n    assert_stripped user1 == user2\n    assert_stripped [user1, user2] === [user3, user4]\n    assert_stripped user1 in [user2, user3, user4]\n    assert_stripped user1 not in [user2, user3, user4]","ref":"Ash.Test.html#assert_stripped/1-examples"},{"type":"function","title":"Ash.Test.refute_has_error/4","doc":"Refute that the given changeset, query, or action input has a matching error.\n\nThe `error_class` argument has been deprecated and should not be used.","ref":"Ash.Test.html#refute_has_error/4"},{"type":"function","title":"Ash.Test.strip_metadata/1","doc":"Clears the `__metadata__` field and the underlying ecto `__meta__` field\n\nThis allows for easier comparison of changeset/query results","ref":"Ash.Test.html#strip_metadata/1"},{"type":"module","title":"Ash.Policy.Check.AccessingFrom","doc":"This check is true when the current action is being run \"through\" a relationship.","ref":"Ash.Policy.Check.AccessingFrom.html"},{"type":"function","title":"Ash.Policy.Check.AccessingFrom.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.AccessingFrom.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.AccessingFrom.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.AccessingFrom.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.AccessingFrom.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.AccessingFrom.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.AccessingFrom.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.AccessingFrom.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.AccessingFrom.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.AccessingFrom.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Action","doc":"This check is true when the action name matches the provided action name.","ref":"Ash.Policy.Check.Action.html"},{"type":"function","title":"Ash.Policy.Check.Action.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Action.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Action.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Action.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Action.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Action.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Action.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Action.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Action.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Action.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ActionType","doc":"This check is true when the action type matches the provided type","ref":"Ash.Policy.Check.ActionType.html"},{"type":"function","title":"Ash.Policy.Check.ActionType.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ActionType.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ActionType.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ActionType.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ActionType.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ActionType.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ActionType.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ActionType.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ActionType.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ActionType.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ActorAbsent","doc":"This check is true when the actor is `nil`, and false when the actor is specified.","ref":"Ash.Policy.Check.ActorAbsent.html"},{"type":"function","title":"Ash.Policy.Check.ActorAbsent.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ActorAbsent.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ActorAbsent.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ActorAbsent.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ActorAbsent.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ActorAbsent.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ActorAbsent.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ActorAbsent.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ActorAbsent.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ActorAbsent.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ActorAttributeEquals","doc":"This check is true when the value of the specified attribute of the actor equals the specified value.","ref":"Ash.Policy.Check.ActorAttributeEquals.html"},{"type":"function","title":"Ash.Policy.Check.ActorAttributeEquals.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ActorAttributeEquals.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ActorAttributeEquals.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ActorAttributeEquals.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ActorAttributeEquals.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ActorAttributeEquals.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ActorAttributeEquals.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ActorAttributeEquals.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ActorAttributeEquals.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ActorAttributeEquals.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ActorPresent","doc":"This check is true when there is an actor specified, and false when the actor is `nil`.","ref":"Ash.Policy.Check.ActorPresent.html"},{"type":"function","title":"Ash.Policy.Check.ActorPresent.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ActorPresent.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ActorPresent.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ActorPresent.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ActorPresent.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ActorPresent.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ActorPresent.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ActorPresent.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ActorPresent.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ActorPresent.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ChangingAttributes","doc":"This check is true when attribute changes correspond to the provided options.","ref":"Ash.Policy.Check.ChangingAttributes.html"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.auto_filter/3","doc":"Callback implementation for  Ash.Policy.Check.auto_filter/3 .","ref":"Ash.Policy.Check.ChangingAttributes.html#auto_filter/3"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.auto_filter_not/3","doc":"","ref":"Ash.Policy.Check.ChangingAttributes.html#auto_filter_not/3"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.check/4","doc":"Callback implementation for  Ash.Policy.Check.check/4 .","ref":"Ash.Policy.Check.ChangingAttributes.html#check/4"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ChangingAttributes.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.expand_description/3","doc":"Callback implementation for  Ash.Policy.Check.expand_description/3 .","ref":"Ash.Policy.Check.ChangingAttributes.html#expand_description/3"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ChangingAttributes.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.reject/3","doc":"Callback implementation for  Ash.Policy.FilterCheck.reject/3 .","ref":"Ash.Policy.Check.ChangingAttributes.html#reject/3"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ChangingAttributes.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ChangingAttributes.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.strict_check_context/1","doc":"","ref":"Ash.Policy.Check.ChangingAttributes.html#strict_check_context/1"},{"type":"function","title":"Ash.Policy.Check.ChangingAttributes.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ChangingAttributes.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ChangingRelationships","doc":"This check is true when the specified relationship is changing","ref":"Ash.Policy.Check.ChangingRelationships.html"},{"type":"function","title":"Ash.Policy.Check.ChangingRelationships.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ChangingRelationships.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ChangingRelationships.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ChangingRelationships.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ChangingRelationships.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ChangingRelationships.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ChangingRelationships.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ChangingRelationships.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ChangingRelationships.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ChangingRelationships.html#type/0"},{"type":"module","title":"Ash.Policy.Check.ContextEquals","doc":"This check is true when the value of the specified key or path in the changeset or query context equals the specified value.","ref":"Ash.Policy.Check.ContextEquals.html"},{"type":"function","title":"Ash.Policy.Check.ContextEquals.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.ContextEquals.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.ContextEquals.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.ContextEquals.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.ContextEquals.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.ContextEquals.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.ContextEquals.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.ContextEquals.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.ContextEquals.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.ContextEquals.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Expression","doc":"The check module used for `expr`s in policies","ref":"Ash.Policy.Check.Expression.html"},{"type":"function","title":"Ash.Policy.Check.Expression.auto_filter/3","doc":"Callback implementation for  Ash.Policy.Check.auto_filter/3 .","ref":"Ash.Policy.Check.Expression.html#auto_filter/3"},{"type":"function","title":"Ash.Policy.Check.Expression.auto_filter_not/3","doc":"","ref":"Ash.Policy.Check.Expression.html#auto_filter_not/3"},{"type":"function","title":"Ash.Policy.Check.Expression.check/4","doc":"Callback implementation for  Ash.Policy.Check.check/4 .","ref":"Ash.Policy.Check.Expression.html#check/4"},{"type":"function","title":"Ash.Policy.Check.Expression.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Expression.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Expression.expand_description/3","doc":"Callback implementation for  Ash.Policy.Check.expand_description/3 .","ref":"Ash.Policy.Check.Expression.html#expand_description/3"},{"type":"function","title":"Ash.Policy.Check.Expression.reject/3","doc":"Callback implementation for  Ash.Policy.FilterCheck.reject/3 .","ref":"Ash.Policy.Check.Expression.html#reject/3"},{"type":"function","title":"Ash.Policy.Check.Expression.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Expression.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Expression.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Expression.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Expression.strict_check_context/1","doc":"","ref":"Ash.Policy.Check.Expression.html#strict_check_context/1"},{"type":"function","title":"Ash.Policy.Check.Expression.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Expression.html#type/0"},{"type":"module","title":"Ash.Policy.Check.FilteringOn","doc":"This check is true when the field provided is being referenced anywhere in a filter statement.","ref":"Ash.Policy.Check.FilteringOn.html"},{"type":"function","title":"Ash.Policy.Check.FilteringOn.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.FilteringOn.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.FilteringOn.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.FilteringOn.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.FilteringOn.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.FilteringOn.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.FilteringOn.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.FilteringOn.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.FilteringOn.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.FilteringOn.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Loading","doc":"This check is true when the field or relationship, or path to field, is being loaded and false when it is not.","ref":"Ash.Policy.Check.Loading.html"},{"type":"function","title":"Ash.Policy.Check.Loading.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Loading.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Loading.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Loading.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Loading.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Loading.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Loading.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Loading.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Loading.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Loading.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Matches","doc":"This check is true when the specified function returns true","ref":"Ash.Policy.Check.Matches.html"},{"type":"function","title":"Ash.Policy.Check.Matches.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Matches.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Matches.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Matches.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Matches.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Matches.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Matches.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Matches.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Matches.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Matches.html#type/0"},{"type":"module","title":"Ash.Policy.Check.RelatesToActorVia","doc":"This check passes if the data relates to the actor via the specified relationship or path of relationships.","ref":"Ash.Policy.Check.RelatesToActorVia.html"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.auto_filter/3","doc":"Callback implementation for  Ash.Policy.Check.auto_filter/3 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#auto_filter/3"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.auto_filter_not/3","doc":"","ref":"Ash.Policy.Check.RelatesToActorVia.html#auto_filter_not/3"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.check/4","doc":"Callback implementation for  Ash.Policy.Check.check/4 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#check/4"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.expand_description/3","doc":"Callback implementation for  Ash.Policy.Check.expand_description/3 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#expand_description/3"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.strict_check_context/1","doc":"","ref":"Ash.Policy.Check.RelatesToActorVia.html#strict_check_context/1"},{"type":"function","title":"Ash.Policy.Check.RelatesToActorVia.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.RelatesToActorVia.html#type/0"},{"type":"module","title":"Ash.Policy.Check.RelatingToActor","doc":"This check is true when the specified relationship is being changed to the current actor.","ref":"Ash.Policy.Check.RelatingToActor.html"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.auto_filter/3","doc":"Callback implementation for  Ash.Policy.Check.auto_filter/3 .","ref":"Ash.Policy.Check.RelatingToActor.html#auto_filter/3"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.auto_filter_not/3","doc":"","ref":"Ash.Policy.Check.RelatingToActor.html#auto_filter_not/3"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.check/4","doc":"Callback implementation for  Ash.Policy.Check.check/4 .","ref":"Ash.Policy.Check.RelatingToActor.html#check/4"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.RelatingToActor.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.expand_description/3","doc":"Callback implementation for  Ash.Policy.Check.expand_description/3 .","ref":"Ash.Policy.Check.RelatingToActor.html#expand_description/3"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.RelatingToActor.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.reject/3","doc":"Callback implementation for  Ash.Policy.FilterCheck.reject/3 .","ref":"Ash.Policy.Check.RelatingToActor.html#reject/3"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.RelatingToActor.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.RelatingToActor.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.strict_check_context/1","doc":"","ref":"Ash.Policy.Check.RelatingToActor.html#strict_check_context/1"},{"type":"function","title":"Ash.Policy.Check.RelatingToActor.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.RelatingToActor.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Resource","doc":"This check is true when the resource matches the provided resource name or names.","ref":"Ash.Policy.Check.Resource.html"},{"type":"function","title":"Ash.Policy.Check.Resource.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Resource.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Resource.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Resource.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Resource.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Resource.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Resource.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Resource.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Resource.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Resource.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Selecting","doc":"This check is true when the field is being selected and false when it is not.","ref":"Ash.Policy.Check.Selecting.html"},{"type":"function","title":"Ash.Policy.Check.Selecting.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Selecting.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Selecting.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Selecting.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Selecting.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Selecting.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Selecting.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Selecting.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Selecting.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Selecting.html#type/0"},{"type":"module","title":"Ash.Policy.Check.Static","doc":"This check is always the result provided","ref":"Ash.Policy.Check.Static.html"},{"type":"function","title":"Ash.Policy.Check.Static.eager_evaluate?/0","doc":"Callback implementation for  Ash.Policy.Check.eager_evaluate?/0 .","ref":"Ash.Policy.Check.Static.html#eager_evaluate?/0"},{"type":"function","title":"Ash.Policy.Check.Static.prefer_expanded_description?/0","doc":"Callback implementation for  Ash.Policy.Check.prefer_expanded_description?/0 .","ref":"Ash.Policy.Check.Static.html#prefer_expanded_description?/0"},{"type":"function","title":"Ash.Policy.Check.Static.requires_original_data?/2","doc":"Callback implementation for  Ash.Policy.Check.requires_original_data?/2 .","ref":"Ash.Policy.Check.Static.html#requires_original_data?/2"},{"type":"function","title":"Ash.Policy.Check.Static.strict_check/3","doc":"Callback implementation for  Ash.Policy.Check.strict_check/3 .","ref":"Ash.Policy.Check.Static.html#strict_check/3"},{"type":"function","title":"Ash.Policy.Check.Static.type/0","doc":"Callback implementation for  Ash.Policy.Check.type/0 .","ref":"Ash.Policy.Check.Static.html#type/0"},{"type":"module","title":"Ash.Resource.Change.CascadeDestroy","doc":"Cascade a resource's destroy action to a related resource's destroy action.\n\nIf after_action? is true this change adds an after-action hook that explicitly\ncalls destroy on any records related via the named relationship.  It will optimise\nfor bulk destroys where possible. This makes it safe to use in atomic actions, but\nmight not be possible depending on the data layer setup (see warning below).\n\nIf after_action? is false this change will add a before-action hook for relationships\nwhere the child record points to the parent (has_many, has_one, many_to_many).\n\n> #### Beware database constraints {: .warning}\n>\n> Think carefully before using this change with data layers which enforce\n> referential integrity (ie PostgreSQL and SQLite) and you may need to defer\n> constraints for the relationship in question.\n>\n> See also:\n>   1. [`postgres.references.reference.deferrable` DSL](https://hexdocs.pm/ash_postgres/dsl-ashpostgres-datalayer.html#postgres-references-reference-deferrable)\n>   2. [`sqlite.references.reference.deferrable` DSL](https://hexdocs.pm/ash_sqlite/dsl-ashsqlite-datalayer.html#sqlite-references-reference-deferrable)\n>   3. [PostgreSQL's `SET CONSTRAINTS` documentation](https://www.postgresql.org/docs/current/sql-set-constraints.html)\n>   4. [SQLite's `PRAGMA defer_foreign_keys` documentation](https://www.sqlite.org/pragma.html#pragma_defer_foreign_keys)\n\n> #### Cascading notifications {: .tip}\n>\n> By default notifications are disabled for the related destroy. This is to avoid potentially sending a **lot** of notifications for high-cardinality relationships.","ref":"Ash.Resource.Change.CascadeDestroy.html"},{"type":"module","title":"Options - Ash.Resource.Change.CascadeDestroy","doc":"* `:relationship` (`t:atom/0`) - Required. The name of the relationship to work on\n\n* `:action` (`t:atom/0`) - The name of the destroy action to call on the related resource. Uses the primary destroy by default.\n\n* `:read_action` (`t:atom/0`) - The name of the read action to call on the related resource to find results to be destroyed\n\n* `:return_notifications?` (`t:boolean/0`) - Return notifications for all destroyed records? The default value is `false`.\n\n* `:after_action?` (`t:boolean/0`) - If true all the cascade destroys are done in after_action hooks. This makes it safe to use in atomic actions The default value is `true`.","ref":"Ash.Resource.Change.CascadeDestroy.html#module-options"},{"type":"module","title":"Example - Ash.Resource.Change.CascadeDestroy","doc":"change {Ash.Resource.Change.CascadeDestroy, relationship: :comments, action: :destroy}\n\n  or, equivalently using `Ash.Resource.Change.Builtins.cascade_destroy/2`:\n\n    change cascade_destroy(:comments, action: :destroy)","ref":"Ash.Resource.Change.CascadeDestroy.html#module-example"},{"type":"module","title":"Ash.Resource.Change.CascadeUpdate","doc":"Cascade a resource's update action to a related resource's update action.\n\nAdds an after-action hook that explicitly calls update on any records related\nvia the named relationship.  It will optimise for bulk updates where\npossible.\n\nAllows you to copy fields from the arguments or changes to the destination,\nthis way you can cascade a bunch of changes downstream.\n\n\n> #### Beware database constraints {: .warning}\n>\n> Think carefully before using this change with data layers which enforce\n> referential integrity (ie PostgreSQL and SQLite) and you may need to defer\n> constraints for the relationship in question.\n>\n> See also:\n>   1. [`postgres.references.reference.deferrable` DSL](https://hexdocs.pm/ash_postgres/dsl-ashpostgres-datalayer.html#postgres-references-reference-deferrable)\n>   2. [`sqlite.references.reference.deferrable` DSL](https://hexdocs.pm/ash_sqlite/dsl-ashsqlite-datalayer.html#sqlite-references-reference-deferrable)\n>   3. [PostgreSQL's `SET CONSTRAINTS` documentation](https://www.postgresql.org/docs/current/sql-set-constraints.html)\n>   4. [SQLite's `PRAGMA defer_foreign_keys` documentation](https://www.sqlite.org/pragma.html#pragma_defer_foreign_keys)\n\n> #### Cascading notifications {: .tip}\n>\n> By default notifications are disabled for the related updates. This is to avoid potentially sending a **lot** of notifications for high-cardinality relationships.","ref":"Ash.Resource.Change.CascadeUpdate.html"},{"type":"module","title":"Options - Ash.Resource.Change.CascadeUpdate","doc":"* `:relationship` (`t:atom/0`) - Required. The name of the relationship to work on\n\n* `:action` (`t:atom/0`) - The name of the update action to call on the related resource. Uses the primary update by default.\n\n* `:copy_inputs` (list of `t:atom/0`) - A list of fields to copy & pass on to the downstream update. The source action cannot be atomic. The default value is `[]`.\n\n* `:read_action` (`t:atom/0`) - The name of the read action to call on the related resource to find results to be updated\n\n* `:return_notifications?` (`t:boolean/0`) - Return notifications for all updated records? The default value is `false`.","ref":"Ash.Resource.Change.CascadeUpdate.html#module-options"},{"type":"module","title":"Example - Ash.Resource.Change.CascadeUpdate","doc":"change {Ash.Resource.Change.CascadeUpdate, relationship: :comments, action: :update_all, copy_inputs: [:name]}\n\n    or, equivalently using `Ash.Resource.Change.Builtins.cascade_update/2`:\n\n    change cascade_update(:comments, action: :update_all, copy_inputs: [:name])","ref":"Ash.Resource.Change.CascadeUpdate.html#module-example"},{"type":"module","title":"Ash.Resource.Change.Context","doc":"The context for a change.\n\nThis is passed into various callbacks for `Ash.Resource.Change`.","ref":"Ash.Resource.Change.Context.html"},{"type":"type","title":"Ash.Resource.Change.Context.t/0","doc":"","ref":"Ash.Resource.Change.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.Change.GetAndLock","doc":"Refetches the record being updated or destroyed, and locks it with the given type.","ref":"Ash.Resource.Change.GetAndLock.html"},{"type":"module","title":"Ash.Resource.Change.GetAndLockForUpdate","doc":"Refetches the record being updated or destroyed, and locks it for update.","ref":"Ash.Resource.Change.GetAndLockForUpdate.html"},{"type":"module","title":"Ash.Resource.Change.Increment","doc":"Increments an attribute's value by the amount specified, which defaults to 1.","ref":"Ash.Resource.Change.Increment.html"},{"type":"module","title":"Ash.Resource.Change.OptimisticLock","doc":"Apply an \"optimistic lock\" on a record being updated or destroyed.","ref":"Ash.Resource.Change.OptimisticLock.html"},{"type":"module","title":"What is Optimistic Locking? - Ash.Resource.Change.OptimisticLock","doc":"Optimistic Locking is the process of only allowing an update to occur if the version of a record that you have in memory is the same as the version in the database.\nOtherwise, an error is returned. On success, it increments the version while performing the action.\n\nOptimistic locking may used for two primary purposes:","ref":"Ash.Resource.Change.OptimisticLock.html#module-what-is-optimistic-locking"},{"type":"module","title":"User Experience - Ash.Resource.Change.OptimisticLock","doc":"For example, if a user is editing a form that contains `State` and `County` fields, and they change the `County`, while another user has used the form to change the `State`, you could end up with a mismatch between `State` and `County`.\n\nWith optimistic locking, the user will instead get an error message that the record has been changed since they last looked.","ref":"Ash.Resource.Change.OptimisticLock.html#module-user-experience"},{"type":"module","title":"Concurrency Safety - Ash.Resource.Change.OptimisticLock","doc":"Optimistic locking can make actions safe to run concurrently even if they can't be performed in a single query (atomically), by returning an error if the resource in the data layer does not have the same version as the one being edited.\n\nThis tells the user that they need to reload and try again.","ref":"Ash.Resource.Change.OptimisticLock.html#module-concurrency-safety"},{"type":"module","title":"Ash.Resource.Validation.ActionIs","doc":"Validates that the action is the specified action.","ref":"Ash.Resource.Validation.ActionIs.html"},{"type":"module","title":"Ash.Resource.Validation.Context","doc":"Context for a validation.","ref":"Ash.Resource.Validation.Context.html"},{"type":"type","title":"Ash.Resource.Validation.Context.t/0","doc":"","ref":"Ash.Resource.Validation.Context.html#t:t/0"},{"type":"behaviour","title":"Ash.Tracer","doc":"A behaviour for implementing tracing for an Ash application.","ref":"Ash.Tracer.html"},{"type":"callback","title":"Ash.Tracer.get_span_context/0","doc":"","ref":"Ash.Tracer.html#c:get_span_context/0"},{"type":"function","title":"Ash.Tracer.get_span_context/1","doc":"","ref":"Ash.Tracer.html#get_span_context/1"},{"type":"type","title":"Ash.Tracer.metadata/0","doc":"","ref":"Ash.Tracer.html#t:metadata/0"},{"type":"callback","title":"Ash.Tracer.set_error/1","doc":"","ref":"Ash.Tracer.html#c:set_error/1"},{"type":"function","title":"Ash.Tracer.set_error/2","doc":"","ref":"Ash.Tracer.html#set_error/2"},{"type":"callback","title":"Ash.Tracer.set_error/2","doc":"","ref":"Ash.Tracer.html#c:set_error/2"},{"type":"function","title":"Ash.Tracer.set_error/3","doc":"","ref":"Ash.Tracer.html#set_error/3"},{"type":"callback","title":"Ash.Tracer.set_handled_error/2","doc":"","ref":"Ash.Tracer.html#c:set_handled_error/2"},{"type":"function","title":"Ash.Tracer.set_handled_error/3","doc":"","ref":"Ash.Tracer.html#set_handled_error/3"},{"type":"callback","title":"Ash.Tracer.set_metadata/2","doc":"Set metadata for the current span.\n\nThis may be called multiple times per span, and should ideally merge with previous metadata.","ref":"Ash.Tracer.html#c:set_metadata/2"},{"type":"function","title":"Ash.Tracer.set_metadata/3","doc":"","ref":"Ash.Tracer.html#set_metadata/3"},{"type":"callback","title":"Ash.Tracer.set_span_context/1","doc":"","ref":"Ash.Tracer.html#c:set_span_context/1"},{"type":"function","title":"Ash.Tracer.set_span_context/2","doc":"","ref":"Ash.Tracer.html#set_span_context/2"},{"type":"macro","title":"Ash.Tracer.span/4","doc":"","ref":"Ash.Tracer.html#span/4"},{"type":"type","title":"Ash.Tracer.span_type/0","doc":"","ref":"Ash.Tracer.html#t:span_type/0"},{"type":"callback","title":"Ash.Tracer.start_span/2","doc":"","ref":"Ash.Tracer.html#c:start_span/2"},{"type":"function","title":"Ash.Tracer.start_span/3","doc":"","ref":"Ash.Tracer.html#start_span/3"},{"type":"callback","title":"Ash.Tracer.stop_span/0","doc":"","ref":"Ash.Tracer.html#c:stop_span/0"},{"type":"function","title":"Ash.Tracer.stop_span/1","doc":"","ref":"Ash.Tracer.html#stop_span/1"},{"type":"type","title":"Ash.Tracer.t/0","doc":"","ref":"Ash.Tracer.html#t:t/0"},{"type":"macro","title":"Ash.Tracer.telemetry_span/4","doc":"","ref":"Ash.Tracer.html#telemetry_span/4"},{"type":"callback","title":"Ash.Tracer.trace_type?/1","doc":"","ref":"Ash.Tracer.html#c:trace_type?/1"},{"type":"function","title":"Ash.Tracer.trace_type?/2","doc":"","ref":"Ash.Tracer.html#trace_type?/2"},{"type":"module","title":"Ash.Tracer.Simple","doc":"A simple tracer that can send traces to the current process or call a module with the trace.","ref":"Ash.Tracer.Simple.html"},{"type":"function","title":"Ash.Tracer.Simple.gather_spans/0","doc":"","ref":"Ash.Tracer.Simple.html#gather_spans/0"},{"type":"module","title":"Ash.Tracer.Simple.Span","doc":"A span produced by `Ash.Tracer.Simple`","ref":"Ash.Tracer.Simple.Span.html"},{"type":"module","title":"Ash.BulkResult","doc":"The return value for bulk actions.","ref":"Ash.BulkResult.html"},{"type":"type","title":"Ash.BulkResult.t/0","doc":"","ref":"Ash.BulkResult.html#t:t/0"},{"type":"module","title":"Ash.Changeset.ManagedRelationshipHelpers","doc":"Tools for introspecting managed relationships.\n\nExtensions can use this to look at an argument that will be passed\nto a `manage_relationship` change and determine what their behavior\nshould be. For example, AshAdmin uses these to find out what kind of\nnested form it should offer for each argument that manages a relationship.","ref":"Ash.Changeset.ManagedRelationshipHelpers.html"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.could_create?/1","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#could_create?/1"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.could_handle_missing?/1","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#could_handle_missing?/1"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.could_lookup?/1","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#could_lookup?/1"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.could_update?/1","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#could_update?/1"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.must_load?/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#must_load?/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.on_lookup_read_action/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#on_lookup_read_action/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.on_lookup_update_action/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#on_lookup_update_action/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.on_match_destination_actions/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#on_match_destination_actions/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.on_missing_destination_actions/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#on_missing_destination_actions/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.on_no_match_destination_actions/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#on_no_match_destination_actions/2"},{"type":"function","title":"Ash.Changeset.ManagedRelationshipHelpers.sanitize_opts/2","doc":"","ref":"Ash.Changeset.ManagedRelationshipHelpers.html#sanitize_opts/2"},{"type":"module","title":"Ash.CiString","doc":"Represents a case insensitive string\n\nWhile some data layers are aware of case insensitive string types, in order for values\nof this type to be used in other parts of Ash Framework, it has to be embedded in a module\nthis allows us to implement the `Comparable` protocol for it.\n\nFor the type implementation, see `Ash.Type.CiString`","ref":"Ash.CiString.html"},{"type":"type","title":"Ash.CiString.casing/0","doc":"","ref":"Ash.CiString.html#t:casing/0"},{"type":"function","title":"Ash.CiString.compare/2","doc":"Compares an Elixir String or Ash.CiString. It will return `:eq` if equal, `:lt`, if\nthe string is ordered alphabetically before the second string, and `:gt` if after.","ref":"Ash.CiString.html#compare/2"},{"type":"function","title":"Ash.CiString.new/2","doc":"Returns a Ash.CiString from a String, or `nil` if the value is `nil`.","ref":"Ash.CiString.html#new/2"},{"type":"function","title":"Ash.CiString.sigil_i/2","doc":"Creates a case insensitive string","ref":"Ash.CiString.html#sigil_i/2"},{"type":"type","title":"Ash.CiString.string_type/0","doc":"","ref":"Ash.CiString.html#t:string_type/0"},{"type":"type","title":"Ash.CiString.t/0","doc":"","ref":"Ash.CiString.html#t:t/0"},{"type":"function","title":"Ash.CiString.to_comparable_string/1","doc":"Returns the downcased value, only downcasing if it hasn't already been done","ref":"Ash.CiString.html#to_comparable_string/1"},{"type":"function","title":"Ash.CiString.value/1","doc":"Converts a `Ash.CiString` into a normal Elixir String.","ref":"Ash.CiString.html#value/1"},{"type":"module","title":"Ash.Expr","doc":"Tools to build Ash expressions","ref":"Ash.Expr.html"},{"type":"function","title":"Ash.Expr.actor/1","doc":"A template helper for using actor values in filter templates","ref":"Ash.Expr.html#actor/1"},{"type":"function","title":"Ash.Expr.arg/1","doc":"A template helper for using action arguments in filter templates","ref":"Ash.Expr.html#arg/1"},{"type":"function","title":"Ash.Expr.atomic_ref/1","doc":"A template helper for referring to the most recent atomic expression applied to an update field","ref":"Ash.Expr.html#atomic_ref/1"},{"type":"macro","title":"Ash.Expr.calc/2","doc":"Creates an expression calculation for use in sort and distinct statements.","ref":"Ash.Expr.html#calc/2"},{"type":"macro","title":"Examples - Ash.Expr.calc/2","doc":"```elixir\nAsh.Query.sort(query, [\n  {calc(string_upcase(name), :asc},\n  {calc(count_nils([field1, field2]), type: :integer), :desc})\n])\n```","ref":"Ash.Expr.html#calc/2-examples"},{"type":"function","title":"Ash.Expr.can_return_nil?/1","doc":"","ref":"Ash.Expr.html#can_return_nil?/1"},{"type":"function","title":"Ash.Expr.combinations/1","doc":"A template helper for creating a reference","ref":"Ash.Expr.html#combinations/1"},{"type":"function","title":"Ash.Expr.context/1","doc":"A template helper for using query context in filter templates\n\nAn atom will get the value for a key, and a list will be accessed via `get_in`.","ref":"Ash.Expr.html#context/1"},{"type":"function","title":"Ash.Expr.determine_type/1","doc":"","ref":"Ash.Expr.html#determine_type/1"},{"type":"function","title":"Ash.Expr.determine_types/4","doc":"","ref":"Ash.Expr.html#determine_types/4"},{"type":"function","title":"Ash.Expr.eval/2","doc":"Evaluate an expression. This function only works if you have no references, or if you provide the `record` option.","ref":"Ash.Expr.html#eval/2"},{"type":"function","title":"Ash.Expr.eval!/2","doc":"Evaluate an expression. See `eval/2` for more.","ref":"Ash.Expr.html#eval!/2"},{"type":"macro","title":"Ash.Expr.expr/1","doc":"Creates an expression. See the [Expressions guide](/documentation/topics/reference/expressions.md) for more.","ref":"Ash.Expr.html#expr/1"},{"type":"function","title":"Ash.Expr.expr?/1","doc":"Returns true if the value is or contains an expression","ref":"Ash.Expr.html#expr?/1"},{"type":"macro","title":"Ash.Expr.or_where/2","doc":"","ref":"Ash.Expr.html#or_where/2"},{"type":"function","title":"Ash.Expr.parent/1","doc":"A template helper for creating a parent reference","ref":"Ash.Expr.html#parent/1"},{"type":"function","title":"Ash.Expr.ref/1","doc":"A template helper for creating a reference","ref":"Ash.Expr.html#ref/1"},{"type":"function","title":"Ash.Expr.ref/2","doc":"A template helper for creating a reference to a related path","ref":"Ash.Expr.html#ref/2"},{"type":"type","title":"Ash.Expr.t/0","doc":"","ref":"Ash.Expr.html#t:t/0"},{"type":"function","title":"Ash.Expr.template_references?/2","doc":"Whether or not a given template contains an actor reference","ref":"Ash.Expr.html#template_references?/2"},{"type":"function","title":"Ash.Expr.template_references_argument?/1","doc":"","ref":"Ash.Expr.html#template_references_argument?/1"},{"type":"function","title":"Ash.Expr.template_references_context?/1","doc":"","ref":"Ash.Expr.html#template_references_context?/1"},{"type":"function","title":"Ash.Expr.tenant/0","doc":"A template helper for using the tenant in filter templates","ref":"Ash.Expr.html#tenant/0"},{"type":"macro","title":"Ash.Expr.where/2","doc":"","ref":"Ash.Expr.html#where/2"},{"type":"module","title":"Ash.Filter","doc":"The representation of a filter in Ash.","ref":"Ash.Filter.html"},{"type":"module","title":"Security Concerns - Ash.Filter","doc":"Do not pass user input directly to `Ash.Query.filter/2`, it will not be sanitised. Instead use\n`Ash.Filter.parse_input/2` or `Ash.Query.filter_input/2`.\n\nRefer to those functions for more information on how to safely work with user input.","ref":"Ash.Filter.html#module-security-concerns"},{"type":"module","title":"Writing a filter - Ash.Filter","doc":"","ref":"Ash.Filter.html#module-writing-a-filter"},{"type":"module","title":"Built In Predicates - Ash.Filter","doc":"* `is_nil`\n* `==`\n* `!=`\n* `in`\n* `<`\n* `>`\n* `<=`\n* `>=`\n* `&&`\n* `||`\n* `<>`\n* `/`\n* `-`\n* `*`\n* `+`\n* `equals` (alias\n  for `==`)\n* `not_equals` (alias\n  for `!=`)\n* `gt` (alias\n  for `>`)\n* `lt` (alias\n  for `<`)\n* `gte` (alias\n  for `>=`)\n* `lte` (alias\n  for `<=`)\n* `eq` (alias\n  for `==`)\n* `not_eq` (alias\n  for `!=`)\n* `less_than` (alias\n  for `<`)\n* `greater_than` (alias\n  for `>`)\n* `less_than_or_equal` (alias\n  for `<=`)\n* `greater_than_or_equal` (alias\n  for `>=`)\n* `and` (alias\n  for `&&`)\n* `or` (alias\n  for `||`)\n* `concat` (alias\n  for `<>`)\n* `div` (alias\n  for `/`)\n* `minus` (alias\n  for `-`)\n* `times` (alias\n  for `*`)\n* `plus` (alias\n  for `+`)","ref":"Ash.Filter.html#module-built-in-predicates"},{"type":"module","title":"BooleanExpression syntax - Ash.Filter","doc":"The expression syntax ultimately just builds the keyword list style filter,\nbut with lots of conveniences that would be very annoying to do manually.\n\nExamples\n\n```elixir\nAsh.Query.filter(resource, name == \"Zardoz\")\nAsh.Query.filter(resource, first_name == \"Zar\" and last_name == \"Doz\")\nAsh.Query.filter(resource, first_name == \"Zar\" and last_name in [\"Doz\", \"Daz\"] and high_score > 10)\nAsh.Query.filter(resource, first_name == \"Zar\" or last_name == \"Doz\" or (high_score > 10 and high_score < -10))\n```","ref":"Ash.Filter.html#module-booleanexpression-syntax"},{"type":"module","title":"Expressions - Ash.Filter","doc":"More complex filters can be built using Ash Expressions.\n\nExamples\n\n```elixir\n# Filter based on the contents of a string attribute\nAsh.Query.filter(Helpdesk.Support.Ticket, contains(subject, \"2\"))\n# Filter based on the attribute of a joined relationship:\nAsh.Query.filter(Helpdesk.Support.Ticket, representative.name == ^name)\n```\n\nSee the [Expressions guide](/documentation/topics/reference/expressions.md)\nguide for more information.","ref":"Ash.Filter.html#module-expressions"},{"type":"module","title":"Keyword list syntax - Ash.Filter","doc":"A filter is a nested keyword list (with some exceptions, like `true` for\neverything and `false` for nothing).\n\nThe key is the \"predicate\" (or \"condition\") and the value is the parameter.\nYou can use `and` and `or` to create nested filters. Data layers can expose\ncustom predicates. Eventually, you will be able to define your own custom\npredicates, which will be a mechanism for you to attach complex filters\nsupported by the data layer to your queries.\n\n** Important ** In a given keyword list, all predicates are considered to be\n\"ands\". So `[or: [first_name: \"Tom\", last_name: \"Bombadil\"]]` doesn't mean\n'First name == \"tom\" or last_name == \"bombadil\"'. To say that, you want to\nprovide a list of filters, like so: `[or: [[first_name: \"Tom\"], [last_name:\n\"Bombadil\"]]]`\n\nSome example filters:\n\n```elixir\nAsh.Query.filter(resource, [name: \"Zardoz\"])\nAsh.Query.filter(resource, [first_name: \"Zar\", last_name: \"Doz\"])\nAsh.Query.filter(resource, [first_name: \"Zar\", last_name: [in: [\"Doz\", \"Daz\"]], high_score: [greater_than: 10]])\nAsh.Query.filter(resource, [or: [\n  [first_name: \"Zar\"],\n  [last_name: \"Doz\"],\n  [or: [\n    [high_score: [greater_than: 10]]],\n    [high_score: [less_than: -10]]\n  ]\n]])\n```","ref":"Ash.Filter.html#module-keyword-list-syntax"},{"type":"module","title":"Other formats - Ash.Filter","doc":"Maps are also accepted, as are maps with string keys. Technically, a list of\n`[{\"string_key\", value}]` would also work.","ref":"Ash.Filter.html#module-other-formats"},{"type":"function","title":"Ash.Filter.add_to_filter/6","doc":"","ref":"Ash.Filter.html#add_to_filter/6"},{"type":"function","title":"Ash.Filter.add_to_filter!/6","doc":"","ref":"Ash.Filter.html#add_to_filter!/6"},{"type":"function","title":"Ash.Filter.builtin_functions/0","doc":"","ref":"Ash.Filter.html#builtin_functions/0"},{"type":"function","title":"Ash.Filter.builtin_operators/0","doc":"","ref":"Ash.Filter.html#builtin_operators/0"},{"type":"function","title":"Ash.Filter.builtin_predicate_operators/0","doc":"","ref":"Ash.Filter.html#builtin_predicate_operators/0"},{"type":"function","title":"Ash.Filter.builtins/0","doc":"","ref":"Ash.Filter.html#builtins/0"},{"type":"function","title":"Ash.Filter.custom_expression/2","doc":"","ref":"Ash.Filter.html#custom_expression/2"},{"type":"function","title":"Ash.Filter.do_hydrate_refs/2","doc":"","ref":"Ash.Filter.html#do_hydrate_refs/2"},{"type":"function","title":"Ash.Filter.fetch_simple_equality_predicate/2","doc":"Can be used to find a simple equality predicate on an attribute\n\nUse this when your attribute is configured with `filterable? :simple_equality`, and you want to\nto find the value that it is being filtered on with (if any).","ref":"Ash.Filter.html#fetch_simple_equality_predicate/2"},{"type":"function","title":"Ash.Filter.find/5","doc":"Find an expression inside of a filter that matches the provided predicate","ref":"Ash.Filter.html#find/5"},{"type":"function","title":"Ash.Filter.find_simple_equality_predicate/2","doc":"Can be used to find a simple equality predicate on an attribute\n\nPrefer `fetch_simple_equality_predicate/2`.","ref":"Ash.Filter.html#find_simple_equality_predicate/2"},{"type":"function","title":"Ash.Filter.find_value/2","doc":"","ref":"Ash.Filter.html#find_value/2"},{"type":"function","title":"Ash.Filter.flat_map/2","doc":"","ref":"Ash.Filter.html#flat_map/2"},{"type":"function","title":"Ash.Filter.get_filter/2","doc":"Returns a filter statement that would find a single record based on the input.\n\nFor example:\n\n    iex> get_filter(MyApp.Post, 1)\n    {:ok, %{id: 1}} #using primary key\n    iex> get_filter(MyApp.Post, id: 1)\n    {:ok, %{id: 1}} #using primary key\n    iex> get_filter(MyApp.Post, author_id: 1, publication_id: 2, first_name: \"fred\")\n    {:ok, %{author_id: 1, publication_id: 1}} # using a unique identity\n    iex> get_filter(MyApp.Post, first_name: \"fred\")\n    :error # not enough information","ref":"Ash.Filter.html#get_filter/2"},{"type":"function","title":"Ash.Filter.get_function/3","doc":"","ref":"Ash.Filter.html#get_function/3"},{"type":"function","title":"Ash.Filter.get_operator/1","doc":"","ref":"Ash.Filter.html#get_operator/1"},{"type":"function","title":"Ash.Filter.get_predicate_function/3","doc":"","ref":"Ash.Filter.html#get_predicate_function/3"},{"type":"function","title":"Ash.Filter.hydrate_refs/2","doc":"","ref":"Ash.Filter.html#hydrate_refs/2"},{"type":"function","title":"Ash.Filter.list_predicates/1","doc":"","ref":"Ash.Filter.html#list_predicates/1"},{"type":"function","title":"Ash.Filter.list_refs/5","doc":"","ref":"Ash.Filter.html#list_refs/5"},{"type":"function","title":"Ash.Filter.map/2","doc":"","ref":"Ash.Filter.html#map/2"},{"type":"function","title":"Ash.Filter.move_exprs_to_relationship_path/2","doc":"","ref":"Ash.Filter.html#move_exprs_to_relationship_path/2"},{"type":"function","title":"Ash.Filter.move_to_relationship_path/2","doc":"","ref":"Ash.Filter.html#move_to_relationship_path/2"},{"type":"function","title":"Ash.Filter.parse/3","doc":"Parses a filter statement\n\nSee the module documentation for more information on the supported formats for filter\nstatements.","ref":"Ash.Filter.html#parse/3"},{"type":"function","title":"Important - Ash.Filter.parse/3","doc":"If you are trying to validate a filter supplied from an external/untrusted source,\nbe sure to use `parse_input/2` instead!","ref":"Ash.Filter.html#parse/3-important"},{"type":"function","title":"Ash.Filter.parse!/3","doc":"Parses a filter statement\n\nSee `parse/2` for more","ref":"Ash.Filter.html#parse!/3"},{"type":"function","title":"Ash.Filter.parse_input/2","doc":"Parses a filter statement, accepting *only public attributes/relationships* and\nhonoring field policies & related resource policies, raising on errors.\n\nSee `parse/2` for more on filter parsing, and the [policies guide](/documentation/topics/security/policies.md)\nfor more on input references.","ref":"Ash.Filter.html#parse_input/2"},{"type":"function","title":"Ash.Filter.parse_input!/2","doc":"Parses a filter statement, accepting *only public attributes/relationships* and\nhonoring field policies & related resource policies, raising on errors.\n\nSee `parse_input/2` for more","ref":"Ash.Filter.html#parse_input!/2"},{"type":"function","title":"Ash.Filter.prefix_refs/2","doc":"","ref":"Ash.Filter.html#prefix_refs/2"},{"type":"function","title":"Ash.Filter.put_at_path/2","doc":"","ref":"Ash.Filter.html#put_at_path/2"},{"type":"function","title":"Ash.Filter.relationship_paths/4","doc":"","ref":"Ash.Filter.html#relationship_paths/4"},{"type":"function","title":"Ash.Filter.run_other_data_layer_filters/4","doc":"","ref":"Ash.Filter.html#run_other_data_layer_filters/4"},{"type":"function","title":"Ash.Filter.strict_subset_of/2","doc":"Returns true if the second argument is a strict subset (always returns the same or less data) of the first","ref":"Ash.Filter.html#strict_subset_of/2"},{"type":"function","title":"Ash.Filter.strict_subset_of?/2","doc":"","ref":"Ash.Filter.html#strict_subset_of?/2"},{"type":"type","title":"Ash.Filter.t/0","doc":"","ref":"Ash.Filter.html#t:t/0"},{"type":"function","title":"Ash.Filter.to_simple_filter/2","doc":"Transform an expression based filter to a simple filter, which is just a list of predicates\n\nOptions:\n\n* `:skip_invalid?` (`t:boolean/0`) - If an invalid filter expression is reached that can't be used with a simple filter (like an `or` statement, or a non-predicate expression), it will be ignored instead of raising an error. The default value is `false`.","ref":"Ash.Filter.html#to_simple_filter/2"},{"type":"function","title":"Ash.Filter.used_aggregates/3","doc":"","ref":"Ash.Filter.html#used_aggregates/3"},{"type":"function","title":"Ash.Filter.used_calculations/6","doc":"","ref":"Ash.Filter.html#used_calculations/6"},{"type":"module","title":"Ash.Filter.Runtime","doc":"Tools to checks a record to see if it matches a filter statement, or to\nevalute expressions against records.","ref":"Ash.Filter.Runtime.html"},{"type":"function","title":"Ash.Filter.Runtime.filter_matches/4","doc":"Removes any records that don't match the filter. Automatically loads\nif necessary. If there are any ambiguous terms in the filter (e.g things\nthat could only be determined by data layer), it is assumed that they\nare not matches.","ref":"Ash.Filter.Runtime.html#filter_matches/4"},{"type":"function","title":"Ash.Filter.Runtime.old_get_related/6","doc":"","ref":"Ash.Filter.Runtime.html#old_get_related/6"},{"type":"module","title":"Ash.Filter.Simple","doc":"Represents a simplified filter, with a simple list of predicates","ref":"Ash.Filter.Simple.html"},{"type":"module","title":"Ash.Filter.Simple.Not","doc":"A negated predicate","ref":"Ash.Filter.Simple.Not.html"},{"type":"module","title":"Ash.ForbiddenField","doc":"Represents a field that was hidden due to authorization rules.","ref":"Ash.ForbiddenField.html"},{"type":"type","title":"Ash.ForbiddenField.field_type/0","doc":"","ref":"Ash.ForbiddenField.html#t:field_type/0"},{"type":"type","title":"Ash.ForbiddenField.t/0","doc":"","ref":"Ash.ForbiddenField.html#t:t/0"},{"type":"module","title":"Ash.Mix.Tasks.Helpers","doc":"Helpers for Ash Mix tasks.","ref":"Ash.Mix.Tasks.Helpers.html"},{"type":"function","title":"Ash.Mix.Tasks.Helpers.domains!/1","doc":"Get all domains for the current project and ensure they are compiled.","ref":"Ash.Mix.Tasks.Helpers.html#domains!/1"},{"type":"function","title":"Ash.Mix.Tasks.Helpers.extensions!/2","doc":"Gets all extensions in use by the current project's domains and resources","ref":"Ash.Mix.Tasks.Helpers.html#extensions!/2"},{"type":"module","title":"Ash.NotLoaded","doc":"Used when a field hasn't been loaded or selected.","ref":"Ash.NotLoaded.html"},{"type":"type","title":"Ash.NotLoaded.t/0","doc":"","ref":"Ash.NotLoaded.html#t:t/0"},{"type":"module","title":"Ash.OptionsHelpers","doc":"Helpers for working with nimble options","ref":"Ash.OptionsHelpers.html"},{"type":"function","title":"Ash.OptionsHelpers.ash_resource/0","doc":"Used to configure Spark to know that the type referenced is an Ash.Resource\nalso using Spark.","ref":"Ash.OptionsHelpers.html#ash_resource/0"},{"type":"function","title":"Ash.OptionsHelpers.ash_type/0","doc":"Specifies Ash types that can be allowed when declaring a type for an\nattribute, calculation, aggregate, etc.","ref":"Ash.OptionsHelpers.html#ash_type/0"},{"type":"function","title":"Ash.OptionsHelpers.calculation_type/0","doc":"Defines the calculation type used in DSL schema definitions, allowing either\na calculation module/function or an `expr(...)` expression.","ref":"Ash.OptionsHelpers.html#calculation_type/0"},{"type":"function","title":"Ash.OptionsHelpers.hide_all_except/2","doc":"Used for marking an option as hidden so it doesn't show up as an option\nwith Spark.","ref":"Ash.OptionsHelpers.html#hide_all_except/2"},{"type":"module","title":"Ash.Page","doc":"Types for Ash pages","ref":"Ash.Page.html"},{"type":"type","title":"Ash.Page.page/0","doc":"","ref":"Ash.Page.html#t:page/0"},{"type":"type","title":"Ash.Page.type/0","doc":"","ref":"Ash.Page.html#t:type/0"},{"type":"module","title":"Ash.Page.Keyset","doc":"A page of results from `keyset` based pagination.\n\nThe results are generated with a `keyset` metadata,\nwhich can be used to fetch the next/previous pages.","ref":"Ash.Page.Keyset.html"},{"type":"function","title":"Ash.Page.Keyset.data_with_keyset/3","doc":"Appends keyset info to results.","ref":"Ash.Page.Keyset.html#data_with_keyset/3"},{"type":"function","title":"Ash.Page.Keyset.filter/4","doc":"Creates filters on the query using the query for the Keyset.","ref":"Ash.Page.Keyset.html#filter/4"},{"type":"function","title":"Ash.Page.Keyset.new/6","doc":"Creates a new `Ash.Page.Keyset.t`.","ref":"Ash.Page.Keyset.html#new/6"},{"type":"function","title":"Ash.Page.Keyset.non_executable_binary_to_term/2","doc":"A restricted version of `:erlang.binary_to_term/2` that forbids\n*executable* terms, such as anonymous functions.\nThe `opts` are given to the underlying `:erlang.binary_to_term/2`\ncall, with an empty list as a default.\nBy default this function does not restrict atoms, as an atom\ninterned in one node may not yet have been interned on another\n(except for releases, which preload all code).\nIf you want to avoid atoms from being created, then you can pass\n`[:safe]` as options, as that will also enable the safety mechanisms\nfrom `:erlang.binary_to_term/2` itself.\nRipped from https://github.com/elixir-plug/plug_crypto/blob/v1.2.0/lib/plug/crypto.ex","ref":"Ash.Page.Keyset.html#non_executable_binary_to_term/2"},{"type":"type","title":"Ash.Page.Keyset.page_opts/0","doc":"","ref":"Ash.Page.Keyset.html#t:page_opts/0"},{"type":"type","title":"Ash.Page.Keyset.page_opts_opts/0","doc":"","ref":"Ash.Page.Keyset.html#t:page_opts_opts/0"},{"type":"type","title":"Ash.Page.Keyset.page_opts_type/0","doc":"","ref":"Ash.Page.Keyset.html#t:page_opts_type/0"},{"type":"type","title":"Ash.Page.Keyset.t/0","doc":"","ref":"Ash.Page.Keyset.html#t:t/0"},{"type":"module","title":"Ash.Page.Offset","doc":"A page of results from `offset` based pagination.\n\nIf a resource supports `keyset` pagination as well,\nit will also have the `keyset` metadata.","ref":"Ash.Page.Offset.html"},{"type":"function","title":"Ash.Page.Offset.new/5","doc":"Creates a new `Ash.Page.Offset.t()`.","ref":"Ash.Page.Offset.html#new/5"},{"type":"type","title":"Ash.Page.Offset.page_opts/0","doc":"","ref":"Ash.Page.Offset.html#t:page_opts/0"},{"type":"type","title":"Ash.Page.Offset.page_opts_opts/0","doc":"","ref":"Ash.Page.Offset.html#t:page_opts_opts/0"},{"type":"type","title":"Ash.Page.Offset.page_opts_type/0","doc":"","ref":"Ash.Page.Offset.html#t:page_opts_type/0"},{"type":"type","title":"Ash.Page.Offset.t/0","doc":"","ref":"Ash.Page.Offset.html#t:t/0"},{"type":"module","title":"Ash.PlugHelpers","doc":"Helpers for working with the Plug connection.","ref":"Ash.PlugHelpers.html"},{"type":"function","title":"Ash.PlugHelpers.get_actor/1","doc":"Retrieves the actor from the Plug connection.\n\nThe actor is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#get_actor/1"},{"type":"function","title":"Deprecation warning - Ash.PlugHelpers.get_actor/1","doc":"This function checks to see if the actor is already set in the `@actor`\nassign, and if so will emit a deprecation warning.\n\nThis is to allow apps using the previous method a chance to update.\n\nRather than setting the actor in the assigns, please use the `set_actor/2`\nmethod.","ref":"Ash.PlugHelpers.html#get_actor/1-deprecation-warning"},{"type":"function","title":"Example - Ash.PlugHelpers.get_actor/1","doc":"iex> actor = build_actor(%{email: \"marty@1985.retro\"})\n    ...> conn = build_conn() |> put_private(:ash, %{actor: actor})\n    ...> actor = get_actor(conn)\n    %{email: \"marty@1985.retro\"} = actor\n\n    iex> actor = build_actor(%{email: \"marty@1985.retro\"})\n    ...> conn = build_conn() |> assign(:actor, actor)\n    ...> actor = get_actor(conn)\n    %{email: \"marty@1985.retro\"} = actor","ref":"Ash.PlugHelpers.html#get_actor/1-example"},{"type":"function","title":"Ash.PlugHelpers.get_context/1","doc":"Retrieves the context from the Plug connection.\n\nThe context is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#get_context/1"},{"type":"function","title":"Example - Ash.PlugHelpers.get_context/1","doc":"iex> context = %{fraud_score: 0.427}\n    ...> conn = build_conn() |> put_private(:ash, %{context: context})\n    ...> context = get_context(conn)\n    %{fraud_score: 0.427}","ref":"Ash.PlugHelpers.html#get_context/1-example"},{"type":"function","title":"Ash.PlugHelpers.get_tenant/1","doc":"Retrieves the tenant from the Plug connection.\n\nThe tenant is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#get_tenant/1"},{"type":"function","title":"Deprecation warning - Ash.PlugHelpers.get_tenant/1","doc":"This function checks to see if the tenant is already set in the `@tenant`\nassign, and if so will emit a deprecation warning.\n\nThis is to allow apps using the previous method a chance to update.\n\nRather than setting the tenant in the assigns, please use the `set_tenant/2`\nmethod.","ref":"Ash.PlugHelpers.html#get_tenant/1-deprecation-warning"},{"type":"function","title":"Example - Ash.PlugHelpers.get_tenant/1","doc":"iex> conn = build_conn() |> put_private(:ash, %{tenant: \"my-tenant\"})\n    ...> tenant = get_tenant(conn)\n    \"my-tenant\" = tenant\n\n    iex> conn = build_conn() |> assign(:tenant, \"my-tenant\")\n    ...> tenant = get_tenant(conn)\n    \"my-tenant\" = tenant","ref":"Ash.PlugHelpers.html#get_tenant/1-example"},{"type":"function","title":"Ash.PlugHelpers.set_actor/2","doc":"Sets the actor inside the Plug connection.\n\nThe actor is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#set_actor/2"},{"type":"function","title":"Example - Ash.PlugHelpers.set_actor/2","doc":"iex> actor = build_actor(%{email: \"marty@1985.retro\"})\n    ...> conn = build_conn() |> set_actor(actor)\n    %Plug.Conn{private: %{ash: %{actor: %{email: \"marty@1985.retro\"}}}} = conn","ref":"Ash.PlugHelpers.html#set_actor/2-example"},{"type":"function","title":"Ash.PlugHelpers.set_context/2","doc":"Sets the context inside the Plug connection.\n\nContext can be used to store arbitrary data about the user, connection, or\nanything else you like that doesn't belong as part of the actor or tenant.\n\nThe context is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#set_context/2"},{"type":"function","title":"Example - Ash.PlugHelpers.set_context/2","doc":"iex> context = %{fraud_score: 0.427}\n    ...> conn = build_conn() |> set_context(context)\n    %Plug.Conn{private: %{ash: %{context: %{fraud_score: 0.427}}}} = conn","ref":"Ash.PlugHelpers.html#set_context/2-example"},{"type":"function","title":"Ash.PlugHelpers.set_tenant/2","doc":"Sets the tenant inside the Plug connection.\n\nThe tenant is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#set_tenant/2"},{"type":"function","title":"Example - Ash.PlugHelpers.set_tenant/2","doc":"iex> conn = build_conn() |> set_tenant(\"my-tenant\")\n    %Plug.Conn{private: %{ash: %{tenant: \"my-tenant\"}}} = conn","ref":"Ash.PlugHelpers.html#set_tenant/2-example"},{"type":"function","title":"Ash.PlugHelpers.update_actor/2","doc":"Updates the actor inside the Plug connection.\n\nThe actor is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#update_actor/2"},{"type":"function","title":"Example - Ash.PlugHelpers.update_actor/2","doc":"iex> actor = build_actor(%{email: \"marty@1985.retro\"})\n    ...> conn = build_conn() |> put_private(:ash, %{actor: actor})\n    ...> actor = get_actor(conn)\n    %{email: \"marty@1985.retro\"} = actor\n    ...> conn = update_actor(conn, fn actor -> Map.put(actor, :name, \"Marty Retro\") end)\n    ...> actor = get_actor(conn)\n    %{email: \"marty@1985.retro\", name: \"Marty Retro\"} = actor\n    ...> conn = update_actor(conn, fn actor -> Map.delete(actor, :email) end)\n    ...> actor = get_actor(conn)\n    %{name: \"Marty Retro\"} = actor","ref":"Ash.PlugHelpers.html#update_actor/2-example"},{"type":"function","title":"Ash.PlugHelpers.update_context/2","doc":"Updates the context inside the Plug connection.\n\nThe context is stored inside the [connection's private\nfields](https://hexdocs.pm/plug/Plug.Conn.html#module-private-fields).","ref":"Ash.PlugHelpers.html#update_context/2"},{"type":"function","title":"Example - Ash.PlugHelpers.update_context/2","doc":"iex> context = %{species: \"Fythetropozoat\"}\n    ...> conn = build_conn() |> put_private(:ash, %{context: context})\n    ...> context = get_context(conn)\n    %{fraud_score: 0.427} = context\n    ...> conn = update_context(conn, fn context -> Map.put(context, :location, \"Barnard's Loop\") end)\n    ...> context = get_context(conn)\n    %{species: \"Fythetropozoat\", location: \"Barnard's Loop\"} = context\n    ...> conn = update_context(conn, fn context -> Map.delete(context, :fraud_score) end)\n    ...> context = get_context(conn)\n    %{location: \"Barnard's Loop\"} = context","ref":"Ash.PlugHelpers.html#update_context/2-example"},{"type":"module","title":"Ash.ProcessHelpers","doc":"Helpers for working with processes and Ash actions.","ref":"Ash.ProcessHelpers.html"},{"type":"function","title":"Ash.ProcessHelpers.async/2","doc":"Creates a task that will properly transfer the ash context to the new process","ref":"Ash.ProcessHelpers.html#async/2"},{"type":"function","title":"Ash.ProcessHelpers.get_context_for_transfer/1","doc":"Gets all of the ash context so it can be set into a new process.\n\nUse `transfer_context/1` in the new process to set the context.","ref":"Ash.ProcessHelpers.html#get_context_for_transfer/1"},{"type":"function","title":"Ash.ProcessHelpers.task_with_timeout/5","doc":"Creates a task that will properly transfer the ash context to the new process, and timeout if it takes longer than the given timeout","ref":"Ash.ProcessHelpers.html#task_with_timeout/5"},{"type":"function","title":"Ash.ProcessHelpers.transfer_context/2","doc":"","ref":"Ash.ProcessHelpers.html#transfer_context/2"},{"type":"module","title":"Ash.Resource.Builder","doc":"Tools for transforming resources in DSL Transformers.","ref":"Ash.Resource.Builder.html"},{"type":"function","title":"Ash.Resource.Builder.add_action/4","doc":"Builds and adds an action","ref":"Ash.Resource.Builder.html#add_action/4"},{"type":"function","title":"Ash.Resource.Builder.add_aggregate/5","doc":"Builds and adds an aggregate to a resource","ref":"Ash.Resource.Builder.html#add_aggregate/5"},{"type":"function","title":"Ash.Resource.Builder.add_attribute/4","doc":"Builds and adds an attribute to a resource","ref":"Ash.Resource.Builder.html#add_attribute/4"},{"type":"function","title":"Ash.Resource.Builder.add_calculation/5","doc":"Builds and adds a calculation to a resource","ref":"Ash.Resource.Builder.html#add_calculation/5"},{"type":"function","title":"Ash.Resource.Builder.add_calculation_interface/3","doc":"Builds and adds an calculation interface to a resource","ref":"Ash.Resource.Builder.html#add_calculation_interface/3"},{"type":"function","title":"Ash.Resource.Builder.add_change/3","doc":"Builds and adds a change","ref":"Ash.Resource.Builder.html#add_change/3"},{"type":"function","title":"Ash.Resource.Builder.add_create_timestamp/3","doc":"Builds and adds a create_timestamp to a resource","ref":"Ash.Resource.Builder.html#add_create_timestamp/3"},{"type":"function","title":"Ash.Resource.Builder.add_identity/4","doc":"Builds and adds an identity","ref":"Ash.Resource.Builder.html#add_identity/4"},{"type":"function","title":"Ash.Resource.Builder.add_interface/3","doc":"Builds and adds an interface to a resource","ref":"Ash.Resource.Builder.html#add_interface/3"},{"type":"function","title":"Ash.Resource.Builder.add_new_action/4","doc":"Builds and adds a new action unless an action with that name already exists","ref":"Ash.Resource.Builder.html#add_new_action/4"},{"type":"function","title":"Ash.Resource.Builder.add_new_aggregate/5","doc":"Builds and adds an aggregate unless an aggregate with that name already exists","ref":"Ash.Resource.Builder.html#add_new_aggregate/5"},{"type":"function","title":"Ash.Resource.Builder.add_new_attribute/4","doc":"Builds and adds an attribute unless an attribute with that name already exists","ref":"Ash.Resource.Builder.html#add_new_attribute/4"},{"type":"function","title":"Ash.Resource.Builder.add_new_calculation/5","doc":"Builds and adds a calculation unless a calculation with that name already exists","ref":"Ash.Resource.Builder.html#add_new_calculation/5"},{"type":"function","title":"Ash.Resource.Builder.add_new_calculation_interface/3","doc":"Builds and adds an calculation interface unless an calculation interface with that name already exists","ref":"Ash.Resource.Builder.html#add_new_calculation_interface/3"},{"type":"function","title":"Ash.Resource.Builder.add_new_create_timestamp/3","doc":"Builds and adds a create_timestamp unless a create_timestamp with that name already exists","ref":"Ash.Resource.Builder.html#add_new_create_timestamp/3"},{"type":"function","title":"Ash.Resource.Builder.add_new_identity/4","doc":"Builds and adds a new identity unless an identity with that name already exists","ref":"Ash.Resource.Builder.html#add_new_identity/4"},{"type":"function","title":"Ash.Resource.Builder.add_new_interface/3","doc":"Builds and adds an interface unless an interface with that name already exists","ref":"Ash.Resource.Builder.html#add_new_interface/3"},{"type":"function","title":"Ash.Resource.Builder.add_new_relationship/5","doc":"Builds and adds a new relationship unless a relationship with that name already exists","ref":"Ash.Resource.Builder.html#add_new_relationship/5"},{"type":"function","title":"Ash.Resource.Builder.add_new_update_timestamp/3","doc":"Builds and adds an update_timestamp unless an update_timestamp with that name already exists","ref":"Ash.Resource.Builder.html#add_new_update_timestamp/3"},{"type":"function","title":"Ash.Resource.Builder.add_preparation/3","doc":"Builds and adds a preparation","ref":"Ash.Resource.Builder.html#add_preparation/3"},{"type":"function","title":"Ash.Resource.Builder.add_relationship/5","doc":"Builds and adds an action","ref":"Ash.Resource.Builder.html#add_relationship/5"},{"type":"function","title":"Ash.Resource.Builder.add_update_timestamp/3","doc":"Builds and adds an update_timestamp","ref":"Ash.Resource.Builder.html#add_update_timestamp/3"},{"type":"function","title":"Ash.Resource.Builder.build_action/3","doc":"Builds an action","ref":"Ash.Resource.Builder.html#build_action/3"},{"type":"function","title":"Ash.Resource.Builder.build_action_argument/3","doc":"Builds an action argument","ref":"Ash.Resource.Builder.html#build_action_argument/3"},{"type":"function","title":"Ash.Resource.Builder.build_action_change/2","doc":"Builds an action change","ref":"Ash.Resource.Builder.html#build_action_change/2"},{"type":"function","title":"Ash.Resource.Builder.build_action_metadata/3","doc":"Builds an action metadata","ref":"Ash.Resource.Builder.html#build_action_metadata/3"},{"type":"function","title":"Ash.Resource.Builder.build_action_validation/2","doc":"Builds an action validation","ref":"Ash.Resource.Builder.html#build_action_validation/2"},{"type":"function","title":"Ash.Resource.Builder.build_aggregate/4","doc":"Builds a calculation with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_aggregate/4"},{"type":"function","title":"Ash.Resource.Builder.build_attribute/3","doc":"Builds an attribute with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_attribute/3"},{"type":"function","title":"Ash.Resource.Builder.build_calculation/4","doc":"Builds a calculation with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_calculation/4"},{"type":"function","title":"Ash.Resource.Builder.build_calculation_argument/3","doc":"Builds a calculation argument","ref":"Ash.Resource.Builder.html#build_calculation_argument/3"},{"type":"function","title":"Ash.Resource.Builder.build_calculation_interface/2","doc":"Builds an calculation interface with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_calculation_interface/2"},{"type":"function","title":"Ash.Resource.Builder.build_change/2","doc":"Builds a change","ref":"Ash.Resource.Builder.html#build_change/2"},{"type":"function","title":"Ash.Resource.Builder.build_create_timestamp/2","doc":"Builds an create_timestamp with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_create_timestamp/2"},{"type":"function","title":"Ash.Resource.Builder.build_identity/3","doc":"Builds an action","ref":"Ash.Resource.Builder.html#build_identity/3"},{"type":"function","title":"Ash.Resource.Builder.build_interface/2","doc":"Builds an interface with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_interface/2"},{"type":"function","title":"Ash.Resource.Builder.build_pagination/1","doc":"Builds a pagination object","ref":"Ash.Resource.Builder.html#build_pagination/1"},{"type":"function","title":"Ash.Resource.Builder.build_preparation/2","doc":"Builds a preparation","ref":"Ash.Resource.Builder.html#build_preparation/2"},{"type":"function","title":"Ash.Resource.Builder.build_relationship/4","doc":"Builds a relationship","ref":"Ash.Resource.Builder.html#build_relationship/4"},{"type":"function","title":"Ash.Resource.Builder.build_update_timestamp/2","doc":"Builds an update_timestamp with the given name, type, and options","ref":"Ash.Resource.Builder.html#build_update_timestamp/2"},{"type":"function","title":"Ash.Resource.Builder.prepend_action/4","doc":"Builds and adds an action to the front of the actions list","ref":"Ash.Resource.Builder.html#prepend_action/4"},{"type":"module","title":"Ash.SatSolver","doc":"Tools for working with the satsolver that drives filter subset checking (for authorization)\n\nThis is public as a very low level toolkit for writing authorizers, but you almost certainly\ndo not need to look at this module.\n\nIf you are looking for information about how authorization works, see the [policy guide](/documentation/topics/security/policies.md)","ref":"Ash.SatSolver.html"},{"type":"macro","title":"Ash.SatSolver.b/1","doc":"Creates tuples of a boolean statement.\n\ni.e `b(1 and 2) #=> {:and, 1, 2}`","ref":"Ash.SatSolver.html#b/1"},{"type":"function","title":"Ash.SatSolver.left_excludes_right/2","doc":"Returns `b(not (left and right))`","ref":"Ash.SatSolver.html#left_excludes_right/2"},{"type":"function","title":"Ash.SatSolver.left_implies_right/2","doc":"Returns `b(not (left and not right))`","ref":"Ash.SatSolver.html#left_implies_right/2"},{"type":"function","title":"Ash.SatSolver.mutually_exclusive/2","doc":"Returns a statement expressing that the predicates are mutually exclusive.","ref":"Ash.SatSolver.html#mutually_exclusive/2"},{"type":"function","title":"Ash.SatSolver.mutually_exclusive_and_collectively_exhaustive/1","doc":"Returns a statement expressing that the predicates are mutually exclusive and collectively exhaustive.","ref":"Ash.SatSolver.html#mutually_exclusive_and_collectively_exhaustive/1"},{"type":"function","title":"Ash.SatSolver.mutually_inclusive/2","doc":"Returns a statement expressing that the predicates are mutually inclusive","ref":"Ash.SatSolver.html#mutually_inclusive/2"},{"type":"function","title":"Ash.SatSolver.right_excludes_left/2","doc":"Returns `b(not (right and left))`","ref":"Ash.SatSolver.html#right_excludes_left/2"},{"type":"function","title":"Ash.SatSolver.right_implies_left/2","doc":"Returns `b(not (right and not left))`","ref":"Ash.SatSolver.html#right_implies_left/2"},{"type":"function","title":"Ash.SatSolver.strict_filter_subset/2","doc":"Returns true if the candidate filter returns the same or less data than the filter","ref":"Ash.SatSolver.html#strict_filter_subset/2"},{"type":"function","title":"Ash.SatSolver.synonymous_relationship_paths?/4","doc":"Returns `true` if the relationship paths are synonymous from a data perspective","ref":"Ash.SatSolver.html#synonymous_relationship_paths?/4"},{"type":"function","title":"Ash.SatSolver.to_cnf/1","doc":"Transforms a statement to Conjunctive Normal Form(CNF), as lists of lists of integers.","ref":"Ash.SatSolver.html#to_cnf/1"},{"type":"function","title":"Ash.SatSolver.transform/2","doc":"Prepares a filter for comparison","ref":"Ash.SatSolver.html#transform/2"},{"type":"function","title":"Ash.SatSolver.transform_and_solve/2","doc":"Calls `transform/2` and solves the expression","ref":"Ash.SatSolver.html#transform_and_solve/2"},{"type":"module","title":"Ash.Sort","doc":"Utilities and types for sorting.","ref":"Ash.Sort.html"},{"type":"macro","title":"Ash.Sort.expr_sort/2","doc":"Builds an expression to be used in a sort statement. Prefer to use `Ash.Expr.calc/2` instead.\n\nFor example:\n\n```elixir\nAsh.Query.sort(query, Ash.Sort.expr_sort(author.full_name, :string))\n\nAsh.Query.sort(query, [{Ash.Sort.expr_sort(author.full_name, :string), :desc_nils_first}])\n```","ref":"Ash.Sort.html#expr_sort/2"},{"type":"function","title":"Ash.Sort.parse_input/3","doc":"A utility for parsing sorts provided from external input. Only allows sorting on public fields.\n\nSee `Ash.Query.sort/3` for supported formats.","ref":"Ash.Sort.html#parse_input/3"},{"type":"function","title":"Handling specific values - Ash.Sort.parse_input/3","doc":"A handler function may be provided that takes a string, and returns the relevant sort\nIt won't be given any prefixes, only the field. This allows for things like parsing the calculation values\nout of the sort, or setting calculation values if they are not included in the sort string.\n\nTo return calculation parameters, return `{:field, %{param: :value}}`. This will end up as something\nlike `{:field, {%{param: :value}, :desc}}`, with the corresponding sort order.\n\nThis handler function will only be called if you pass in a string or list of strings for the sort.\nAtoms will be assumed to have already been handled. The handler should return `nil` if it is not handling\nthe given field.","ref":"Ash.Sort.html#parse_input/3-handling-specific-values"},{"type":"function","title":"Ash.Sort.parse_input!/3","doc":"Same as `parse_input/2` except raises any errors\n\nSee `parse_input/2` for more.","ref":"Ash.Sort.html#parse_input!/3"},{"type":"function","title":"Ash.Sort.parse_sort/4","doc":"","ref":"Ash.Sort.html#parse_sort/4"},{"type":"function","title":"Ash.Sort.reverse/1","doc":"Reverses an Ash sort statement.","ref":"Ash.Sort.html#reverse/1"},{"type":"function","title":"Ash.Sort.runtime_sort/3","doc":"A utility for sorting a list of records at runtime.\n\nFor example:\n\n    Ash.Sort.runtime_sort([record1, record2, record3], name: :asc, type: :desc_nils_last)\n\nKeep in mind that it is unrealistic to expect this runtime sort to always\nbe exactly the same as a sort that may have been applied by your data layer.\nThis is especially true for strings. For example, `Postgres` strings have a\ncollation that affects their sorting, making it unpredictable from the perspective\nof a tool using the database: https://www.postgresql.org/docs/current/collation.html","ref":"Ash.Sort.html#runtime_sort/3"},{"type":"type","title":"Ash.Sort.sort_item/0","doc":"","ref":"Ash.Sort.html#t:sort_item/0"},{"type":"type","title":"Ash.Sort.sort_order/0","doc":"","ref":"Ash.Sort.html#t:sort_order/0"},{"type":"type","title":"Ash.Sort.t/0","doc":"","ref":"Ash.Sort.html#t:t/0"},{"type":"module","title":"Ash.UUID","doc":"Helpers for working with UUIDs","ref":"Ash.UUID.html"},{"type":"function","title":"Ash.UUID.generate/0","doc":"Generates a new uuid","ref":"Ash.UUID.html#generate/0"},{"type":"type","title":"Ash.UUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Ash.UUID.html#t:raw/0"},{"type":"type","title":"Ash.UUID.t/0","doc":"A hex-encoded UUID string.","ref":"Ash.UUID.html#t:t/0"},{"type":"module","title":"Ash.UUIDv7","doc":"Helpers for working with UUIDs version 7.\n\n[RFC 9562](https://www.rfc-editor.org/rfc/rfc9562#name-uuid-version-7)\n\nUsed for generating UUIDs version 7 with increased clock precision for better monotonicity,\nas described by method 3 of the [Section 6.2](https://www.rfc-editor.org/rfc/rfc9562#name-monotonicity-and-counters\n\nInspired by the work of [Ryan Winchester](https://github.com/ryanwinchester/) on [uuidv7](https://github.com/ryanwinchester/uuidv7)","ref":"Ash.UUIDv7.html"},{"type":"module","title":"Examples - Ash.UUIDv7","doc":"iex> UUIDv7.generate()\n    \"018e90d8-06e8-7f9f-bfd7-6730ba98a51b\"\n\n    iex> UUIDv7.bingenerate()\n    <<1, 142, 144, 216, 6, 232, 127, 159, 191, 215, 103, 48, 186, 152, 165, 27>>","ref":"Ash.UUIDv7.html#module-examples"},{"type":"function","title":"Ash.UUIDv7.bingenerate/0","doc":"Generates a version 7 UUID in the binary format.","ref":"Ash.UUIDv7.html#bingenerate/0"},{"type":"function","title":"Example - Ash.UUIDv7.bingenerate/0","doc":"iex> UUIDv7.bingenerate()\n    <<1, 142, 144, 216, 6, 232, 127, 159, 191, 215, 103, 48, 186, 152, 165, 27>>","ref":"Ash.UUIDv7.html#bingenerate/0-example"},{"type":"function","title":"Ash.UUIDv7.decode/1","doc":"Decode a string representation of a UUID to the raw binary version.","ref":"Ash.UUIDv7.html#decode/1"},{"type":"function","title":"Example - Ash.UUIDv7.decode/1","doc":"iex> UUIDv7.decode(\"018e90d8-06e8-7f9f-bfd7-6730ba98a51b\")\n    <<1, 142, 144, 216, 6, 232, 127, 159, 191, 215, 103, 48, 186, 152, 165, 27>>","ref":"Ash.UUIDv7.html#decode/1-example"},{"type":"function","title":"Ash.UUIDv7.encode/1","doc":"Encode a raw UUID to the string representation.","ref":"Ash.UUIDv7.html#encode/1"},{"type":"function","title":"Example - Ash.UUIDv7.encode/1","doc":"iex> UUIDv7.encode(<<1, 142, 144, 216, 6, 232, 127, 159, 191, 215, 103, 48, 186, 152, 165, 27>>)\n    \"018e90d8-06e8-7f9f-bfd7-6730ba98a51b\"","ref":"Ash.UUIDv7.html#encode/1-example"},{"type":"function","title":"Ash.UUIDv7.extract_timestamp/1","doc":"Extract the millisecond timestamp from the UUID.","ref":"Ash.UUIDv7.html#extract_timestamp/1"},{"type":"function","title":"Example - Ash.UUIDv7.extract_timestamp/1","doc":"iex> UUIDv7.extract_timestamp(\"018ecb40-c457-73e6-a400-000398daddd9\")\n    1712807003223","ref":"Ash.UUIDv7.html#extract_timestamp/1-example"},{"type":"function","title":"Ash.UUIDv7.generate/0","doc":"Generates a version 7 UUID using submilliseconds for increased clock precision.","ref":"Ash.UUIDv7.html#generate/0"},{"type":"function","title":"Example - Ash.UUIDv7.generate/0","doc":"iex> UUIDv7.generate()\n    \"018e90d8-06e8-7f9f-bfd7-6730ba98a51b\"","ref":"Ash.UUIDv7.html#generate/0-example"},{"type":"type","title":"Ash.UUIDv7.raw/0","doc":"A raw binary representation of a UUID.","ref":"Ash.UUIDv7.html#t:raw/0"},{"type":"type","title":"Ash.UUIDv7.t/0","doc":"A hex-encoded UUID string.","ref":"Ash.UUIDv7.html#t:t/0"},{"type":"module","title":"Ash.Union","doc":"A wrapper for values that are sourced from `Ash.Type.Union`.","ref":"Ash.Union.html"},{"type":"type","title":"Ash.Union.t/0","doc":"","ref":"Ash.Union.html#t:t/0"},{"type":"module","title":"Ash.Vector","doc":"A vector struct for Ash.\n\nImplementation based off of https://github.com/pgvector/pgvector-elixir/blob/v0.2.0/lib/pgvector.ex","ref":"Ash.Vector.html"},{"type":"function","title":"Ash.Vector.from_binary/1","doc":"Creates a new vector from its binary representation","ref":"Ash.Vector.html#from_binary/1"},{"type":"function","title":"Ash.Vector.new/1","doc":"Creates a new vector from a list or tensor","ref":"Ash.Vector.html#new/1"},{"type":"type","title":"Ash.Vector.t/0","doc":"","ref":"Ash.Vector.html#t:t/0"},{"type":"function","title":"Ash.Vector.to_binary/1","doc":"Converts the vector to its binary representation","ref":"Ash.Vector.html#to_binary/1"},{"type":"function","title":"Ash.Vector.to_list/1","doc":"Converts the vector to a list","ref":"Ash.Vector.html#to_list/1"},{"type":"behaviour","title":"Ash.Type","doc":"The `Ash.Type` behaviour is used to define a value type in Ash.","ref":"Ash.Type.html"},{"type":"behaviour","title":"Built in types - Ash.Type","doc":"* `:map` - `Ash.Type.Map`\n* `:keyword` - `Ash.Type.Keyword`\n* `:term` - `Ash.Type.Term`\n* `:atom` - `Ash.Type.Atom`\n* `:tuple` - `Ash.Type.Tuple`\n* `:string` - `Ash.Type.String`\n* `:integer` - `Ash.Type.Integer`\n* `:file` - `Ash.Type.File`\n* `:float` - `Ash.Type.Float`\n* `:duration_name` - `Ash.Type.DurationName`\n* `:function` - `Ash.Type.Function`\n* `:boolean` - `Ash.Type.Boolean`\n* `:struct` - `Ash.Type.Struct`\n* `:uuid` - `Ash.Type.UUID`\n* `:uuid_v7` - `Ash.Type.UUIDv7`\n* `:binary` - `Ash.Type.Binary`\n* `:date` - `Ash.Type.Date`\n* `:time` - `Ash.Type.Time`\n* `:time_usec` - `Ash.Type.TimeUsec`\n* `:decimal` - `Ash.Type.Decimal`\n* `:ci_string` - `Ash.Type.CiString`\n* `:naive_datetime` - `Ash.Type.NaiveDatetime`\n* `:utc_datetime` - `Ash.Type.UtcDatetime`\n* `:utc_datetime_usec` - `Ash.Type.UtcDatetimeUsec`\n* `:datetime` - `Ash.Type.DateTime`\n* `:duration` - `Ash.Type.Duration`\n* `:url_encoded_binary` - `Ash.Type.UrlEncodedBinary`\n* `:union` - `Ash.Type.Union`\n* `:module` - `Ash.Type.Module`\n* `:vector` - `Ash.Type.Vector`","ref":"Ash.Type.html#module-built-in-types"},{"type":"behaviour","title":"Lists/Arrays - Ash.Type","doc":"To specify a list of values, use `{:array, Type}`. Arrays are special, and have special constraints:\n\n* `:items` (`t:term/0`) - Constraints for the elements of the list. See the contained type's docs for more.\n\n* `:min_length` (`t:non_neg_integer/0`) - A minimum length for the items.\n\n* `:max_length` (`t:non_neg_integer/0`) - A maximum length for the items.\n\n* `:nil_items?` (`t:boolean/0`) - Whether or not the list can contain nil items. The default value is `false`.\n\n* `:remove_nil_items?` (`t:boolean/0`) - Whether or not to remove the nil items from the list instead of adding errors. The default value is `false`.\n\n* `:empty_values` (list of `t:term/0`) - A set of values that, if encountered, will be considered an empty list. The default value is `[\"\"]`.","ref":"Ash.Type.html#module-lists-arrays"},{"type":"behaviour","title":"Defining Custom Types - Ash.Type","doc":"Generally you add `use Ash.Type` to your module (it is possible to add `@behaviour\nAsh.Type` and define everything yourself, but this is more work and error-prone).\n\nAnother option is to use `Ash.Type.NewType`, which supports defining a new type that\nis the combination of an existing type and custom constraints.\nThis can be helpful when defining a custom attribute (e.g. struct) for a resource.\n\nSimple example of a float custom type\n\n```elixir\ndefmodule GenTracker.AshFloat do\n  use Ash.Type\n\n  @impl Ash.Type\n  def storage_type(_), do: :float\n\n  @impl Ash.Type\n  def cast_input(nil, _), do: {:ok, nil}\n  def cast_input(value, _) do\n    Ecto.Type.cast(:float, value)\n  end\n\n  @impl Ash.Type\n  def cast_stored(nil, _), do: {:ok, nil}\n  def cast_stored(value, _) do\n    Ecto.Type.load(:float, value)\n  end\n\n  @impl Ash.Type\n  def dump_to_native(nil, _), do: {:ok, nil}\n  def dump_to_native(value, _) do\n    Ecto.Type.dump(:float, value)\n  end\nend\n```","ref":"Ash.Type.html#module-defining-custom-types"},{"type":"behaviour","title":"Overriding the {:array, type} behaviour - Ash.Type","doc":"By defining the `*_array` versions of `cast_input`, `cast_stored`, `dump_to_native` and `apply_constraints`, you can\noverride how your type behaves as a collection. This is how the features of embedded\nresources are implemented. No need to implement them unless you wish to override the\ndefault behaviour. Your type is responsible for handling nil values in each callback as well.\n\nAll the Ash built-in types are implemented with `use Ash.Type` so they are good\nexamples to look at to create your own `Ash.Type`.","ref":"Ash.Type.html#module-overriding-the-array-type-behaviour"},{"type":"behaviour","title":"Short names - Ash.Type","doc":"You can define short `:atom_names` for your custom types by adding them to your Ash configuration:\n\n```elixir\nconfig :ash, :custom_types, [ash_float: GenTracker.AshFloat]\n```\n\nDoing this will require a recompilation of the `:ash` dependency which can be triggered by calling:\n\n```bash\n$ mix deps.compile ash --force\n```","ref":"Ash.Type.html#module-short-names"},{"type":"behaviour","title":"Composite Types - Ash.Type","doc":"Composite types are composite *in the data layer*. Many data layers do not support this, but some (like AshPostgres),\ndo. To define a composite type, the following things should be true:\n\n1. A casted value should be a map or struct, for example for a point: `%{x: 1, y: 2}`\n2. The data layer must support composite types, and the data layer representation will be a tuple, i.e `{1, 2}`\n3. Define `def composite?(_), do: true` in your composite type\n4. Define the type & constraints of each item in the tuple, and its name in the map\n   representation: `def composite_types(_), do: [{:x, :integer, []}, {:y, :integer, []}]`.\n   You can also define a storage key for each item in the tuple, if the underlying type implementation\n   has a different reference for an item, i.e `def composite_types(_), do: [{:x, :x_coord, :integer, []}, {:y, :y_coord, :integer, []}]`\n\nWith the above implemented, your composite type can be used in expressions, for example:\n\n```elixir\nAsh.Query.filter(expr(coordinates[:x] == 1))\n```\n\nAnd you can also *construct* composite types in expressions, for example:\n\n```elixir\ncalculate :coordinates, :composite_point, expr(\n  composite_type(%{x: some_value, y: some_other_value}, Point)\n)\n```","ref":"Ash.Type.html#module-composite-types"},{"type":"behaviour","title":"Constraints - Ash.Type","doc":"Constraints are a way of validating an input type. This validation can be used in both attributes and arguments. The kinds of constraints you can apply depends on the type of data. You can find all types in `Ash.Type` . Each type has its own page on which the available constraints are listed. For example in `Ash.Type.String` you can find 5 constraints:\n\n- `:max_length`\n- `:min_length`\n- `:match`\n- `:trim?`\n- `:allow_empty?`\n\nYou can also discover these constraints from iex:\n\n```bash\n$ iex -S mix\niex(1)> Ash.Type.String.constraints\n[\n  max_length: [\n    type: :non_neg_integer,\n    doc: \"Enforces a maximum length on the value\"\n  ],\n  min_length: [\n    type: :non_neg_integer,\n    doc: \"Enforces a minimum length on the value\"\n  ],\n  match: [\n    type: :regex_as_mfa,\n    doc: \"Enforces that the string matches a passed in regex\"\n  ],\n  trim?: [type: :boolean, doc: \"Trims the value.\", default: true],\n  allow_empty?: [\n    type: :boolean,\n    doc: \"If false, the value is set to `nil` if it's empty.\",\n    default: false\n  ]\n]\n```","ref":"Ash.Type.html#module-constraints"},{"type":"behaviour","title":"Attribute example - Ash.Type","doc":"To show how constraints can be used in a attribute, here is an example attribute describing a username:\n\n```elixir\ndefmodule MyProject.MyDomain.Account do\n  # ...\n\n  code_interface do\n    define :create, action: :create\n  end\n\n  actions do\n    default [:create, :read, :update, :destroy]\n  end\n\n  attributes do\n    uuid_primary_key :id\n\n    attribute :username, :string do\n      constraints [\n        max_length: 20,\n        min_length: 3,\n        match: \"^[a-z_-]*$\",\n        trim?: true,\n        allow_empty?: false\n      ]\n    end\n  end\n\n  # ...\nend\n```\n\nIf, when creating or updating this attribute, one of the constraints are not met, an error will be given telling you which constraint was broken. See below:\n\n```elixir\niex(1)> MyProject.MyDomain.Account.create!(%{username: \"hi\"})\n\n** (Ash.Error.Invalid) Invalid Error\n\n* Invalid value provided for username: length must be greater than or equal to 3.\n\n\"hi\"\n\niex(2)> MyProject.MyDomain.Account.create!(%{username: \"Hello there this is a long string\"})\n\n** (Ash.Error.Invalid) Invalid Error\n\n* Invalid value provided for username: length must be less than or equal to 20.\n\n\"Hello there this is a long string\"\n\niex(3)> MyProject.MyDomain.Account.create!(%{username: \"hello there\"})\n** (Ash.Error.Invalid) Invalid Error\n\n* Invalid value provided for username: must match the pattern ~r/^[a-z_-]*$/.\n\n\"hello there\"\n\niex(4)> MyProject.MyDomain.Account.create!(%{username: \"\"})\n** (Ash.Error.Invalid) Invalid Error\n\n* attribute title is required\n```\n\nIt will give you the resource as usual on successful requests:\n\n```elixir\niex(5)> MyProject.MyDomain.Account.create!(%{username: \"hello\"})\n#MyProject.MyDomain.Account<\n  __meta__: #Ecto.Schema.Metadata<:loaded, \"account\">,\n  id: \"7ba467dd-277c-4916-88ae-f62c93fee7a3\",\n  username: \"hello\",\n  ...\n>\n```","ref":"Ash.Type.html#module-attribute-example"},{"type":"callback","title":"Ash.Type.apply_atomic_constraints/2","doc":"Applies type constraints within an expression.","ref":"Ash.Type.html#c:apply_atomic_constraints/2"},{"type":"function","title":"Ash.Type.apply_atomic_constraints/3","doc":"Applies a types constraints to an expression.\n\nThis delegates to the underlying types implementation of `c:apply_atomic_constraints/2`.","ref":"Ash.Type.html#apply_atomic_constraints/3"},{"type":"callback","title":"Ash.Type.apply_atomic_constraints_array/2","doc":"Applies type constraints to a list of values within an expression. See `c:apply_atomic_constraints/2` for more.","ref":"Ash.Type.html#c:apply_atomic_constraints_array/2"},{"type":"callback","title":"Ash.Type.apply_constraints/2","doc":"Called after casting, to apply additional constraints to the value.","ref":"Ash.Type.html#c:apply_constraints/2"},{"type":"function","title":"Ash.Type.apply_constraints/3","doc":"Confirms if a casted value matches the provided constraints.","ref":"Ash.Type.html#apply_constraints/3"},{"type":"callback","title":"Ash.Type.apply_constraints_array/2","doc":"Called after casting a list of values, to apply additional constraints to the value.\n\nIf not defined, `c:apply_constraints/2` is called for each item.","ref":"Ash.Type.html#c:apply_constraints_array/2"},{"type":"callback","title":"Ash.Type.array_constraints/0","doc":"Returns a `Spark.Options` spec for the additional constraints supported when used in a list.","ref":"Ash.Type.html#c:array_constraints/0"},{"type":"function","title":"Ash.Type.array_constraints/1","doc":"Gets the array constraints for a type","ref":"Ash.Type.html#array_constraints/1"},{"type":"function","title":"Ash.Type.ash_type?/1","doc":"Returns true if the value is a builtin type or adopts the `Ash.Type` behaviour","ref":"Ash.Type.html#ash_type?/1"},{"type":"function","title":"Ash.Type.builtin?/1","doc":"Returns true if the type is an ash builtin type","ref":"Ash.Type.html#builtin?/1"},{"type":"callback","title":"Ash.Type.can_load?/1","doc":"Whether or not `c:load/4` can be used. Defined automatically","ref":"Ash.Type.html#c:can_load?/1"},{"type":"function","title":"Ash.Type.can_load?/2","doc":"Returns true if the type supports nested loads","ref":"Ash.Type.html#can_load?/2"},{"type":"callback","title":"Ash.Type.cast_atomic/2","doc":"Casts a value within an expression.\n\nFor instance, if you had a type like `:non_neg_integer`, you might do:\n\n```elixir\ndef cast_atomic(value, _constraints)  do\n  expr(\n    if ^value < 0 do\n      error(Ash.Error.Changes.InvalidChanges, %{message: \"must be positive\", value: ^value})\n    else\n      value\n    end\n  )\nend\n```","ref":"Ash.Type.html#c:cast_atomic/2"},{"type":"function","title":"Ash.Type.cast_atomic/3","doc":"Modifies an expression to apply a type's casting logic to the value it produces.\n\nThis delegates to the underlying types implementation of `c:cast_atomic/2`.","ref":"Ash.Type.html#cast_atomic/3"},{"type":"callback","title":"Ash.Type.cast_atomic_array/2","doc":"Casts a list of values within an expression. See `c:cast_atomic/2` for more.","ref":"Ash.Type.html#c:cast_atomic_array/2"},{"type":"callback","title":"Ash.Type.cast_in_query?/1","doc":"Whether or not data layers that build queries should attempt to type cast values of this type while doing so.","ref":"Ash.Type.html#c:cast_in_query?/1"},{"type":"function","title":"Ash.Type.cast_in_query?/2","doc":"Returns `true` if the type should be cast in underlying queries","ref":"Ash.Type.html#cast_in_query?/2"},{"type":"callback","title":"Ash.Type.cast_input/2","doc":"Attempt to cast unknown, potentially user-provided input, into a valid instance of the type.","ref":"Ash.Type.html#c:cast_input/2"},{"type":"function","title":"Ash.Type.cast_input/3","doc":"Casts input (e.g. unknown) data to an instance of the type, or errors\n\nMaps to `Ecto.Type.cast/2`","ref":"Ash.Type.html#cast_input/3"},{"type":"callback","title":"Ash.Type.cast_input_array/2","doc":"Attempt to cast a list of unknown, potentially user-provided inputs, into a list of valid instances of type.\n\nThis callback allows to define types that are \"collection-aware\", i.e an integer that is unique whenever\nit appears in a list.\n\nIf not defined, `c:cast_input/2` is called for each item.","ref":"Ash.Type.html#c:cast_input_array/2"},{"type":"callback","title":"Ash.Type.cast_stored/2","doc":"Attempt to load a stored value from the data layer into a valid instance of the type.","ref":"Ash.Type.html#c:cast_stored/2"},{"type":"function","title":"Ash.Type.cast_stored/3","doc":"Casts a value from the data store to an instance of the type, or errors\n\nMaps to `Ecto.Type.load/2`","ref":"Ash.Type.html#cast_stored/3"},{"type":"callback","title":"Ash.Type.cast_stored_array/2","doc":"Attempt to load a list of stored values from the data layer into a list of valid instances of the type.\n\nIf not defined, `c:cast_stored/2` is called for each item.","ref":"Ash.Type.html#c:cast_stored_array/2"},{"type":"callback","title":"Ash.Type.coerce/2","doc":"Attempt to coerce unknown, potentially user-provided input, into a valid instance of the type.","ref":"Ash.Type.html#c:coerce/2"},{"type":"callback","title":"Coercion vs Casting - Ash.Type.coerce/2","doc":"Coercion can be summed up as a more \"insistent\" form of casting. It means \"we really want to use\nthis value as this type, so please try to convert it to that type\". This is used in expressions as\nopposed to `cast_input`. For example, the value `10`, if passed into `Ash.Type.cast_input(:string, 10)`\nwould fail to cast. However, if used in the following expression: `expr(type(10, :string) <> \" minutes\")`\nthe `10` would be \"coerced\" (using `to_string/1`) into `\"10\"`.\n\nBy default, coercion uses `cast_input/2` unless","ref":"Ash.Type.html#c:coerce/2-coercion-vs-casting"},{"type":"function","title":"Ash.Type.coerce/3","doc":"Coerces input (e.g. unknown) data to an instance of the type, or errors.\n\nSee `c:Ash.Type.coerce/2`","ref":"Ash.Type.html#coerce/3"},{"type":"callback","title":"Ash.Type.composite?/1","doc":"Return true if the type is a composite type, meaning it is made up of one or more values. How this works is up to the data layer.\n\nFor example, `AshMoney` provides a type that is composite with a \"currency\" and an \"amount\".","ref":"Ash.Type.html#c:composite?/1"},{"type":"function","title":"Ash.Type.composite?/2","doc":"Returns true if the type is a composite type","ref":"Ash.Type.html#composite?/2"},{"type":"callback","title":"Ash.Type.composite_types/1","doc":"Information about each member of the composite type, if it is a composite type\n\nAn example given the `AshMoney` example listed above:\n\n`[{:currency, :string, []}, {:amount, :decimal, []}]`","ref":"Ash.Type.html#c:composite_types/1"},{"type":"function","title":"Ash.Type.composite_types/2","doc":"Returns the wrapped composite types","ref":"Ash.Type.html#composite_types/2"},{"type":"callback","title":"Ash.Type.constraints/0","doc":"Returns a `Spark.Options` spec for the constraints supported by the type.","ref":"Ash.Type.html#c:constraints/0"},{"type":"type","title":"Ash.Type.constraints/0","doc":"A keyword list of constraints for a type","ref":"Ash.Type.html#t:constraints/0"},{"type":"function","title":"Ash.Type.constraints/1","doc":"Returns the constraint schema for a type","ref":"Ash.Type.html#constraints/1"},{"type":"callback","title":"Ash.Type.custom_apply_constraints_array?/0","doc":"Whether or not an `c:apply_constraints_array/2` callback has been defined. This is defined automatically.","ref":"Ash.Type.html#c:custom_apply_constraints_array?/0"},{"type":"callback","title":"Ash.Type.describe/1","doc":"Describes a type given its constraints. Can be used to generate docs, for example.","ref":"Ash.Type.html#c:describe/1"},{"type":"function","title":"Ash.Type.describe/2","doc":"Calls the type's `describe` function with the given constraints","ref":"Ash.Type.html#describe/2"},{"type":"function","title":"Ash.Type.determine_types/2","doc":"Determine types for a given function or operator.","ref":"Ash.Type.html#determine_types/2"},{"type":"callback","title":"Ash.Type.dump_to_embedded/2","doc":"Transform a valid instance of the type into a format that can be JSON encoded.","ref":"Ash.Type.html#c:dump_to_embedded/2"},{"type":"function","title":"Ash.Type.dump_to_embedded/3","doc":"Casts a value from the Elixir type to a value that can be embedded in another data structure.\n\nEmbedded resources expect to be stored in JSON, so this allows things like UUIDs to be stored\nas strings in embedded resources instead of binary.","ref":"Ash.Type.html#dump_to_embedded/3"},{"type":"callback","title":"Ash.Type.dump_to_embedded_array/2","doc":"Transform a list of valid instances of the type into a format that can be JSON encoded.\n\nIf not defined, `c:dump_to_embedded/2` is called for each item.","ref":"Ash.Type.html#c:dump_to_embedded_array/2"},{"type":"callback","title":"Ash.Type.dump_to_native/2","doc":"Transform a valid instance of the type into a format that the data layer can store.","ref":"Ash.Type.html#c:dump_to_native/2"},{"type":"function","title":"Ash.Type.dump_to_native/3","doc":"Casts a value from the Elixir type to a value that the data store can persist\n\nMaps to `Ecto.Type.dump/2`","ref":"Ash.Type.html#dump_to_native/3"},{"type":"callback","title":"Ash.Type.dump_to_native_array/2","doc":"Transform a list of valid instance of the type into a format that the data layer can store.\n\nIf not defined, `c:dump_to_native/2` is called for each item.","ref":"Ash.Type.html#c:dump_to_native_array/2"},{"type":"callback","title":"Ash.Type.ecto_type/0","doc":"The underlying Ecto.Type.","ref":"Ash.Type.html#c:ecto_type/0"},{"type":"function","title":"Ash.Type.ecto_type/1","doc":"Returns the ecto compatible type for an Ash.Type.\n\nIf you `use Ash.Type`, this is created for you. For builtin types\nthis may return a corresponding ecto builtin type (atom)","ref":"Ash.Type.html#ecto_type/1"},{"type":"callback","title":"Ash.Type.embedded?/0","doc":"Whether or not the type is an embedded resource. This is defined by embedded resources, you should not define this.","ref":"Ash.Type.html#c:embedded?/0"},{"type":"function","title":"Ash.Type.embedded_type?/1","doc":"Returns true if the type is an embedded resource","ref":"Ash.Type.html#embedded_type?/1"},{"type":"callback","title":"Ash.Type.equal?/2","doc":"Determine if two valid instances of the type are equal.\n\n*Do not define this* if `==` is sufficient for your type. See `c:simple_equality?/0` for more.","ref":"Ash.Type.html#c:equal?/2"},{"type":"function","title":"Ash.Type.equal?/3","doc":"Determines if two values of a given type are equal.\n\nMaps to `Ecto.Type.equal?/3`","ref":"Ash.Type.html#equal?/3"},{"type":"type","title":"Ash.Type.error/0","doc":"An error value that can be returned from various callbacks","ref":"Ash.Type.html#t:error/0"},{"type":"callback","title":"Ash.Type.evaluate_operator/1","doc":"The implementation for any overloaded implementations.","ref":"Ash.Type.html#c:evaluate_operator/1"},{"type":"callback","title":"Ash.Type.generator/1","doc":"An Enumerable that produces valid instances of the type.\n\nThis can be used for property testing, or generating valid inputs for seeding.\nTypically you would use `StreamData` for this.","ref":"Ash.Type.html#c:generator/1"},{"type":"function","title":"Ash.Type.generator/2","doc":"Returns the StreamData generator for a given type","ref":"Ash.Type.html#generator/2"},{"type":"callback","title":"Ash.Type.get_rewrites/4","doc":"Gets any \"rewrites\" necessary to apply a given load statement.\n\nThis is a low level tool used when types can contain instances of resources. You generally\nshould not need to know how this works. See `Ash.Type.Union` and `Ash.Type.Struct` for examples\nif you are trying to write a similar type.","ref":"Ash.Type.html#c:get_rewrites/4"},{"type":"function","title":"Ash.Type.get_rewrites/5","doc":"Gets the load rewrites for a given type, load, calculation and path.\n\nThis is used for defining types that support a nested load statement.\nSee the embedded type and union type implementations for examples of how\nto use this.","ref":"Ash.Type.html#get_rewrites/5"},{"type":"function","title":"Ash.Type.get_type/1","doc":"Gets the type module for a given short name or module","ref":"Ash.Type.html#get_type/1"},{"type":"function","title":"Ash.Type.get_type!/1","doc":"Gets the type module for a given short name or module,\nensures that it is a valid `type`","ref":"Ash.Type.html#get_type!/1"},{"type":"function","title":"Raises - Ash.Type.get_type!/1","doc":"- `RuntimeError`: If the provided type module is not found or invalid.","ref":"Ash.Type.html#get_type!/1-raises"},{"type":"callback","title":"Ash.Type.handle_change/3","doc":"React to a changing value. This could be used, for example, to have a type like `:strictly_increasing_integer`.","ref":"Ash.Type.html#c:handle_change/3"},{"type":"function","title":"Ash.Type.handle_change/4","doc":"Process the old casted values alongside the new casted values.\n\nThis is leveraged by embedded types to know if something is being updated\nor destroyed. This is not called on creates.","ref":"Ash.Type.html#handle_change/4"},{"type":"callback","title":"Ash.Type.handle_change_array/3","doc":"React to a changing list of values. This could be used, for example, to have a type like `:unique_integer`, which when used in a list all items must be unique.\n\nIf not defined, `c:handle_change/3` is called for each item with a `nil` old value.","ref":"Ash.Type.html#c:handle_change_array/3"},{"type":"callback","title":"Ash.Type.handle_change_array?/0","doc":"Whether or not a custom `c:handle_change_array/3` has been defined by the type. Defined automatically.","ref":"Ash.Type.html#c:handle_change_array?/0"},{"type":"function","title":"Ash.Type.handle_change_array?/1","doc":"Handles the change of a given array of values for an attribute change. Runs after casting.","ref":"Ash.Type.html#handle_change_array?/1"},{"type":"callback","title":"Ash.Type.include_source/2","doc":"Add the source changeset to the constraints, in cases where it is needed for type casting logic","ref":"Ash.Type.html#c:include_source/2"},{"type":"function","title":"Ash.Type.include_source/3","doc":"Provides the changeset, action_input or query to the type, to potentially store in its constraints.\n\nThis is used for embedded types to allow accessing the parent changeset in certain cases.","ref":"Ash.Type.html#include_source/3"},{"type":"callback","title":"Ash.Type.init/1","doc":"Useful for typed data layers (like ash_postgres) to instruct them not to attempt to cast input values.\n\nYou generally won't need this, but it can be an escape hatch for certain cases.","ref":"Ash.Type.html#c:init/1"},{"type":"function","title":"Ash.Type.init/2","doc":"Initializes the constraints according to the underlying type","ref":"Ash.Type.html#init/2"},{"type":"callback","title":"Ash.Type.load/4","doc":"Applies a load statement through a list of values.\n\nThis allows types to support load statements, like `Ash.Type.Union`, embedded resources,\nor the `Ash.Type.Struct` when it is an `instance_of` a resource.","ref":"Ash.Type.html#c:load/4"},{"type":"function","title":"Ash.Type.load/5","doc":"Apply a load statement to a value.\n\nThis is used for types that can be \"loaded through\". For example, maps, unions and structs.\nIf they have keys that are embedded types, for example, we want to be able to apply a load\nstatements to their contents.","ref":"Ash.Type.html#load/5"},{"type":"type","title":"Ash.Type.load_context/0","doc":"The context that is provided to the `c:load/4` callback.","ref":"Ash.Type.html#t:load_context/0"},{"type":"callback","title":"Ash.Type.loaded?/4","doc":"Checks if the given path has been loaded on the type.","ref":"Ash.Type.html#c:loaded?/4"},{"type":"function","title":"Ash.Type.loaded?/5","doc":"Checks if a given path has been loaded on a type.\n\nThis is used to \"load through\" types. For more see `load/5`.","ref":"Ash.Type.html#loaded?/5"},{"type":"callback","title":"Ash.Type.matches_type?/2","doc":"Whether or not the value a valid instance of the type.","ref":"Ash.Type.html#c:matches_type?/2"},{"type":"function","title":"Ash.Type.matches_type?/3","doc":"Detects as a best effort if an arbitrary value matches the given type","ref":"Ash.Type.html#matches_type?/3"},{"type":"callback","title":"Ash.Type.may_support_atomic_update?/1","doc":"Whether or not a value with given constraints may support being cast atomic\n\nDefaults to checking if `cast_atomic/2` is defined on the type.","ref":"Ash.Type.html#c:may_support_atomic_update?/1"},{"type":"callback","title":"Ash.Type.merge_load/4","doc":"Merges a load statement with an existing load statement for the type.","ref":"Ash.Type.html#c:merge_load/4"},{"type":"function","title":"Ash.Type.merge_load/5","doc":"Merges two load statements for a given type.\n\nThis is used to \"load through\" types. For more see `load/5`.","ref":"Ash.Type.html#merge_load/5"},{"type":"type","title":"Ash.Type.merge_load_context/0","doc":"The context that is provided to the `c:merge_load/4` callback.","ref":"Ash.Type.html#t:merge_load_context/0"},{"type":"callback","title":"Ash.Type.operator_overloads/0","doc":"A map of operators with overloaded implementations.\n\nThese will only be honored if the type is placed in `config :ash, :known_types, [...Type]`\n\nA corresponding `evaluate_operator/1` clause should match.","ref":"Ash.Type.html#c:operator_overloads/0"},{"type":"callback","title":"Ash.Type.prepare_change/3","doc":"Prepare a change, given the old value and the new uncasted value.","ref":"Ash.Type.html#c:prepare_change/3"},{"type":"function","title":"Ash.Type.prepare_change/4","doc":"Process the old casted values alongside the new *un*casted values.\n\nThis is leveraged by embedded types to know if something is being updated\nor destroyed. This is not called on creates.","ref":"Ash.Type.html#prepare_change/4"},{"type":"callback","title":"Ash.Type.prepare_change_array/3","doc":"Prepare a changing list of values, given the old value and the new uncasted value.\n\nIf not defined, `c:prepare_change/3` is called for each item with a `nil` old value.","ref":"Ash.Type.html#c:prepare_change_array/3"},{"type":"callback","title":"Ash.Type.prepare_change_array?/0","doc":"Whether or not a custom `c:prepare_change_array/3` has been defined by the type. Defined automatically.","ref":"Ash.Type.html#c:prepare_change_array?/0"},{"type":"function","title":"Ash.Type.prepare_change_array?/1","doc":"Prepares a given array of values for an attribute change. Runs before casting.","ref":"Ash.Type.html#prepare_change_array?/1"},{"type":"callback","title":"Ash.Type.rewrite/3","doc":"Apply any \"rewrites\" necessary to provide the results of a load statement to calculations that depended on a given load.\n\nThis is a low level tool used when types can contain instances of resources. You generally\nshould not need to know how this works. See `Ash.Type.Union` and `Ash.Type.Struct` for examples\nif you are trying to write a similar type.","ref":"Ash.Type.html#c:rewrite/3"},{"type":"function","title":"Ash.Type.rewrite/4","doc":"Applies rewrites to a given value.\n\nThis is used for defining types that support a nested load statement.\nSee the embedded type and union type implementations for examples of how\nto use this.","ref":"Ash.Type.html#rewrite/4"},{"type":"function","title":"Ash.Type.short_names/0","doc":"Returns the list of available type short names","ref":"Ash.Type.html#short_names/0"},{"type":"callback","title":"Ash.Type.simple_equality?/0","doc":"Whether or not `==` can be used to compare instances of the type.\n\nThis is defined automatically to return `false` if `c:equal?/2` is defined.\n\nTypes that cannot be compared using `==` incur significant runtime costs when used in certain ways.\nFor example, if a resource's primary key cannot be compared with `==`, we cannot do things like key\na list of records by their primary key. Implementing `c:equal?/2` will cause various code paths to be considerably\nslower, so only do it when necessary.","ref":"Ash.Type.html#c:simple_equality?/0"},{"type":"function","title":"Ash.Type.simple_equality?/1","doc":"Determines if a type can be compared using the `==` operator.","ref":"Ash.Type.html#simple_equality?/1"},{"type":"callback","title":"Ash.Type.storage_type/0","doc":"The storage type, which should be known by a data layer supporting this type.\n\nUse `c:storage_type/1`, as this will be deprecated in the future.","ref":"Ash.Type.html#c:storage_type/0"},{"type":"callback","title":"Ash.Type.storage_type/1","doc":"The storage type, which should be known by a data layer supporting this type.","ref":"Ash.Type.html#c:storage_type/1"},{"type":"function","title":"Ash.Type.storage_type/2","doc":"Returns the *underlying* storage type (the underlying type of the *ecto type* of the *ash type*)","ref":"Ash.Type.html#storage_type/2"},{"type":"type","title":"Ash.Type.t/0","doc":"A valid Ash.Type","ref":"Ash.Type.html#t:t/0"},{"type":"module","title":"Ash.Type.Atom","doc":"Stores an atom as a string in the database\n\nA builtin type that can be referenced via `:atom`","ref":"Ash.Type.Atom.html"},{"type":"module","title":"Constraints - Ash.Type.Atom","doc":"* `:one_of` (`t:term/0`) - Allows constraining the value of an atom to a pre-defined list\n\n* `:unsafe_to_atom?` (`t:boolean/0`) - Allows casting to atoms that don't yet exist in the system. See https://security.erlef.org/secure_coding_and_deployment_hardening/atom_exhaustion.html for more. The default value is `false`.","ref":"Ash.Type.Atom.html#module-constraints"},{"type":"function","title":"Ash.Type.Atom.handle_change?/0","doc":"","ref":"Ash.Type.Atom.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Atom.prepare_change?/0","doc":"","ref":"Ash.Type.Atom.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Binary","doc":"Represents a binary.\n\nA builtin type that can be referenced via `:binary`","ref":"Ash.Type.Binary.html"},{"type":"function","title":"Ash.Type.Binary.handle_change?/0","doc":"","ref":"Ash.Type.Binary.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Binary.prepare_change?/0","doc":"","ref":"Ash.Type.Binary.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Boolean","doc":"Represents a boolean.\n\nA builtin type that can be referenced via `:boolean`","ref":"Ash.Type.Boolean.html"},{"type":"function","title":"Ash.Type.Boolean.handle_change?/0","doc":"","ref":"Ash.Type.Boolean.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Boolean.prepare_change?/0","doc":"","ref":"Ash.Type.Boolean.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.CiString","doc":"Stores a case insensitive string in the database\n\nSee `Ash.CiString` for more information.\n\nA builtin type that can be referenced via `:ci_string`","ref":"Ash.Type.CiString.html"},{"type":"module","title":"Constraints - Ash.Type.CiString","doc":"* `:max_length` (`t:non_neg_integer/0`) - Enforces a maximum length on the value\n\n* `:min_length` (`t:non_neg_integer/0`) - Enforces a minimum length on the value\n\n* `:match` (`t:Regex.t/0`) - Enforces that the string matches a passed in regex\n\n* `:trim?` (`t:boolean/0`) - Trims the value. The default value is `true`.\n\n* `:allow_empty?` (`t:boolean/0`) - Sets the value to `nil` if it's empty. The default value is `false`.\n\n* `:casing` - Lowercases or uppercases the value, fully discarding case information.  \n  For example, if you don't set this, a value of `FrEd` could be saved to the data layer.\n  `FrEd` and `fReD` would still compare as equal, but the original casing information  is retained.\n  In many cases, this is what you want. In some cases, however, you want to remove all case information.\n  For example, in an email, you may want to support a user inputting an upper case letter, but discard it\n  when saved.\n   Valid values are :upper, :lower, nil The default value is `nil`.","ref":"Ash.Type.CiString.html#module-constraints"},{"type":"function","title":"Ash.Type.CiString.handle_change?/0","doc":"","ref":"Ash.Type.CiString.html#handle_change?/0"},{"type":"function","title":"Ash.Type.CiString.prepare_change?/0","doc":"","ref":"Ash.Type.CiString.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Comparable","doc":"Helpers for working with `Comparable`","ref":"Ash.Type.Comparable.html"},{"type":"macro","title":"Ash.Type.Comparable.defcomparable/3","doc":"","ref":"Ash.Type.Comparable.html#defcomparable/3"},{"type":"module","title":"Ash.Type.Date","doc":"Represents a date in the database\n\nA builtin type that can be referenced via `:date`","ref":"Ash.Type.Date.html"},{"type":"function","title":"Ash.Type.Date.handle_change?/0","doc":"","ref":"Ash.Type.Date.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Date.prepare_change?/0","doc":"","ref":"Ash.Type.Date.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.DateTime","doc":"Represents a datetime, with configurable precision and timezone.\n\nA builtin type that can be referenced via `:datetime`","ref":"Ash.Type.DateTime.html"},{"type":"module","title":"Constraints - Ash.Type.DateTime","doc":"* `:precision` - Valid values are :microsecond, :second The default value is `:second`.\n\n* `:cast_dates_as` - Valid values are :start_of_day, :error The default value is `:start_of_day`.\n\n* `:timezone` - Valid values are :utc The default value is `:utc`.","ref":"Ash.Type.DateTime.html#module-constraints"},{"type":"function","title":"Ash.Type.DateTime.handle_change?/0","doc":"","ref":"Ash.Type.DateTime.html#handle_change?/0"},{"type":"function","title":"Ash.Type.DateTime.prepare_change?/0","doc":"","ref":"Ash.Type.DateTime.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Decimal","doc":"Represents a decimal.\n\nA builtin type that can be referenced via `:decimal`","ref":"Ash.Type.Decimal.html"},{"type":"module","title":"Constraints - Ash.Type.Decimal","doc":"* `:precision` - Enforces a maximum number of significant digits. Set to :arbitrary for no limit. The default value is `:arbitrary`.\n\n* `:scale` - Enforces a maximum number of decimal places. Set to :arbitrary for no limit. The default value is `:arbitrary`.\n\n* `:max` - Enforces a maximum on the value\n\n* `:min` - Enforces a minimum on the value\n\n* `:greater_than` - Enforces a minimum on the value (exclusive)\n\n* `:less_than` - Enforces a maximum on the value (exclusive)","ref":"Ash.Type.Decimal.html#module-constraints"},{"type":"function","title":"Ash.Type.Decimal.handle_change?/0","doc":"","ref":"Ash.Type.Decimal.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Decimal.prepare_change?/0","doc":"","ref":"Ash.Type.Decimal.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Duration","doc":"Represents a Duration\n\nA builtin type that can be referenced via `:duration`","ref":"Ash.Type.Duration.html"},{"type":"function","title":"Ash.Type.Duration.handle_change?/0","doc":"","ref":"Ash.Type.Duration.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Duration.prepare_change?/0","doc":"","ref":"Ash.Type.Duration.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.DurationName","doc":"An interval of time, primarily meant to be used in expression functions\n\nValid intervals are (as strings or atoms): [:year, :month, :week, :day, :hour, :minute, :second, :millisecond, :microsecond]","ref":"Ash.Type.DurationName.html"},{"type":"function","title":"Ash.Type.DurationName.handle_change?/0","doc":"","ref":"Ash.Type.DurationName.html#handle_change?/0"},{"type":"function","title":"Ash.Type.DurationName.prepare_change?/0","doc":"","ref":"Ash.Type.DurationName.html#prepare_change?/0"},{"type":"type","title":"Ash.Type.DurationName.t/0","doc":"","ref":"Ash.Type.DurationName.html#t:t/0"},{"type":"behaviour","title":"Ash.Type.Enum","doc":"A type for abstracting enums into a single type.\n\nFor example, your existing attribute might look like:\n```elixir\nattribute :status, :atom, constraints: [one_of: [:open, :closed]]\n```\n\nBut as that starts to spread around your system, you may find that you want\nto centralize that logic. To do that, use this module to define an Ash type\neasily:\n\n```elixir\ndefmodule MyApp.TicketStatus do\n  use Ash.Type.Enum, values: [:open, :closed]\nend\n```\n\nThen, you can rewrite your original attribute as follows:\n\n```elixir\nattribute :status, MyApp.TicketStatus\n```\n\nValid values are:\n\n* The atom itself, e.g `:open`\n* A string that matches the atom, e.g `\"open\"`\n* A string that matches the atom after being downcased, e.g `\"OPEN\"` or `\"oPeN\"`\n* A string that matches the stringified, downcased atom, after itself being downcased.\n  This allows for enum values like `:Open`, `:SomeState` and `:Some_State`","ref":"Ash.Type.Enum.html"},{"type":"behaviour","title":"Custom input values - Ash.Type.Enum","doc":"If you need to accept inputs beyond those described above while still mapping them to one\nof the enum values, you can override the `match/1` callback.\n\nFor example, if you want to map both the `:half_empty` and `:half_full` states to the same enum\nvalue, you could implement it as follows:\n\n```elixir\ndefmodule MyApp.GlassState do\n  use Ash.Type.Enum, values: [:empty, :half_full, :full]\n\n  def match(:half_empty), do: {:ok, :half_full}\n  def match(\"half_empty\"), do: {:ok, :half_full}\n  def match(value), do: super(value)\nend\n```\n\nIn the provided example, if no additional value is matched, `super(value)` is called, invoking\nthe default implementation of `match/1`. This approach is typically suitable if you only aim to\nextend default matching rather than completely reimplementing it.","ref":"Ash.Type.Enum.html#module-custom-input-values"},{"type":"behaviour","title":"Caveats - Ash.Type.Enum","doc":"Additional input values are not exposed in derived interfaces. For example, `HALF_EMPTY` will not\nbe present as a possible enum value when using `ash_graphql`.\n\nMoreover, only explicitly matched values are mapped to the enum value. For instance,\n`\"HaLf_emPty\"` would not be accepted by the code provided earlier. If case normalization is\nneeded for additional values, it must be explicitly implemented.","ref":"Ash.Type.Enum.html#module-caveats"},{"type":"behaviour","title":"Value labels and descriptions - Ash.Type.Enum","doc":"It's possible to associate a label and/or description for each value.\n\n```elixir\ndefmodule MyApp.TicketStatus do\n  use Ash.Type.Enum,\n    values: [\n      open: \"An open ticket\", # <- description only,\n      escalated: [description: gettext(\"An escalated ticket\")],\n      follow_up: [label: gettext(\"Follow up\")],\n      closed: [description: gettext(\"A closed ticket\"), label: gettext(\"Closed\")]\n    ]\nend\n```\n\nAdding labels and descriptions can be helpful when displaying the Enum values to users.\n\nThis can be used by extensions to provide detailed descriptions of enum values.\n\nThe description of a value can be retrieved with `description/1`:\n\n```elixir\nMyApp.TicketStatus.description(:open)\niex> \"An open ticket\"\n```\n\nThe label of a value can be retrieved with `label/1`:\n\n```elixir\nMyApp.TicketStatus.label(:closed)\niex> \"Closed\"\n```\n\nBoth the description and label can be retrieved with `details/1`\n\n```elixir\nMyApp.TicketStatus.details(:closed)\niex> %{description: \"A closed ticket\", label: \"Closed\"}\n```","ref":"Ash.Type.Enum.html#module-value-labels-and-descriptions"},{"type":"callback","title":"Ash.Type.Enum.description/1","doc":"The description of the value, if existing","ref":"Ash.Type.Enum.html#c:description/1"},{"type":"callback","title":"Ash.Type.Enum.details/1","doc":"The value detail map, if existing","ref":"Ash.Type.Enum.html#c:details/1"},{"type":"callback","title":"Ash.Type.Enum.label/1","doc":"The label of the value, if existing","ref":"Ash.Type.Enum.html#c:label/1"},{"type":"callback","title":"Ash.Type.Enum.match/1","doc":"finds the valid value that matches a given input term","ref":"Ash.Type.Enum.html#c:match/1"},{"type":"callback","title":"Ash.Type.Enum.match?/1","doc":"true if a given term matches a value","ref":"Ash.Type.Enum.html#c:match?/1"},{"type":"callback","title":"Ash.Type.Enum.values/0","doc":"The list of valid values (not all input types that match them)","ref":"Ash.Type.Enum.html#c:values/0"},{"type":"module","title":"Ash.Type.File","doc":"A type that represents a file on the filesystem.\n\n> #### Persistence {: .warning}\n>\n> This type does not support persisting via `Ash.DataLayer`.\n>\n> It is mainly intended to be used in\n> [arguments](dsl-ash-resource.html#actions-action-argument).","ref":"Ash.Type.File.html"},{"type":"module","title":"Valid values to cast - Ash.Type.File","doc":"This type can cast multiple types of values:\n\n* itself\n* `Plug.Upload`\n* Any value that implements the `Ash.Type.File.Source` protocol.","ref":"Ash.Type.File.html#module-valid-values-to-cast"},{"type":"function","title":"Ash.Type.File.from_io/1","doc":"Create a file from an `IO.device()`","ref":"Ash.Type.File.html#from_io/1"},{"type":"function","title":"Example - Ash.Type.File.from_io/1","doc":"iex> path = \"README.md\"\n    ...> {:ok, device} = File.open(path)\n    ...> Ash.Type.File.from_io(device)\n    %Ash.Type.File{source: device, implementation: Ash.Type.File.IO}","ref":"Ash.Type.File.html#from_io/1-example"},{"type":"function","title":"Ash.Type.File.from_path/1","doc":"Create a file from a path.","ref":"Ash.Type.File.html#from_path/1"},{"type":"function","title":"Example - Ash.Type.File.from_path/1","doc":"iex> path = \"README.md\"\n    ...> Ash.Type.File.from_path(path)\n    %Ash.Type.File{source: \"README.md\", implementation: Ash.Type.File.Path}","ref":"Ash.Type.File.html#from_path/1-example"},{"type":"function","title":"Ash.Type.File.handle_change?/0","doc":"","ref":"Ash.Type.File.html#handle_change?/0"},{"type":"function","title":"Ash.Type.File.open/2","doc":"Open the file with the given `modes`.\n\nThis function will delegate to the `open/2` function on the `implementation`.\n\nFor details on the `modes` argument, see the `File.open/2` documentation.","ref":"Ash.Type.File.html#open/2"},{"type":"function","title":"Example - Ash.Type.File.open/2","doc":"iex> path = \"README.md\"\n    ...> file = Ash.Type.File.from_path(path)\n    ...> Ash.Type.File.open(file, [:read])\n    ...> # => {:ok, #PID<0.109.0>}","ref":"Ash.Type.File.html#open/2-example"},{"type":"function","title":"Ash.Type.File.path/1","doc":"Returns the path to the file.\n\nNot every implementation will support this operation. If the implementation\ndoes not support this operation, then `{:error, :not_supported}` will be\nreturned. In this case, use the `open/2` function to access the file.","ref":"Ash.Type.File.html#path/1"},{"type":"function","title":"Example - Ash.Type.File.path/1","doc":"iex> path = \"README.md\"\n    ...> file = Ash.Type.File.from_path(path)\n    ...> Ash.Type.File.path(file)\n    {:ok, \"README.md\"}","ref":"Ash.Type.File.html#path/1-example"},{"type":"function","title":"Ash.Type.File.prepare_change?/0","doc":"","ref":"Ash.Type.File.html#prepare_change?/0"},{"type":"type","title":"Ash.Type.File.t/0","doc":"","ref":"Ash.Type.File.html#t:t/0"},{"type":"behaviour","title":"Ash.Type.File.Implementation","doc":"Behaviour for file implementations that are compatible with `Ash.Type.File`.","ref":"Ash.Type.File.Implementation.html"},{"type":"type","title":"Ash.Type.File.Implementation.error/0","doc":"Errors returned by the implementation.","ref":"Ash.Type.File.Implementation.html#t:error/0"},{"type":"callback","title":"Ash.Type.File.Implementation.open/2","doc":"Open `IO.device()` for the file.\n\nSee `Ash.Type.File.open/2`\n\nThe return pid must point to a process following the\n[Erlang I/O Protocol](https://www.erlang.org/doc/apps/stdlib/io_protocol.html)\nlike `StringIO` or `File`.","ref":"Ash.Type.File.Implementation.html#c:open/2"},{"type":"callback","title":"Ash.Type.File.Implementation.path/1","doc":"Return path of the file on disk.\n\nSee: `Ash.Type.File.path/1`\n\nThis callback is optional. If the file is not stored on disk, this callback\ncan be omitted.","ref":"Ash.Type.File.Implementation.html#c:path/1"},{"type":"type","title":"Ash.Type.File.Implementation.source/0","doc":"The source of the file the implementation operates on.","ref":"Ash.Type.File.Implementation.html#t:source/0"},{"type":"type","title":"Ash.Type.File.Implementation.t/0","doc":"Any `module()` implementing the `Ash.Type.File.Implementation` behaviour.","ref":"Ash.Type.File.Implementation.html#t:t/0"},{"type":"protocol","title":"Ash.Type.File.Source","doc":"Protocol for allowing the casting of something into an `Ash.Type.File`.","ref":"Ash.Type.File.Source.html"},{"type":"protocol","title":"Usage - Ash.Type.File.Source","doc":"```elixir\ndefmodule MyStruct do\n  defstruct [:path]\n\n  @behavior Ash.Type.File.Implementation\n\n  @impl Ash.Type.File.Implementation\n  def path(%__MODULE__{path: path}), do: {:ok, path}\n\n  @impl Ash.Type.File.Implementation\n  def open(%__MODULE__{path: path}, modes), do: File.open(path, modes)\n\n  defimpl Ash.Type.File.Source do\n    def implementation(%MyStruct{} = struct), do: {:ok, MyStruct}\n  end\nend\n```","ref":"Ash.Type.File.Source.html#module-usage"},{"type":"function","title":"Ash.Type.File.Source.implementation/1","doc":"Detect Implementation of the file.\n\nReturns an `:ok` tuple with the implementation module if the file is supported\nand `:error` otherwise.","ref":"Ash.Type.File.Source.html#implementation/1"},{"type":"type","title":"Ash.Type.File.Source.t/0","doc":"All the types that implement this protocol.","ref":"Ash.Type.File.Source.html#t:t/0"},{"type":"module","title":"Ash.Type.Float","doc":"Represents a float (floating point number)\n\nA builtin type that be referenced via `:float`","ref":"Ash.Type.Float.html"},{"type":"module","title":"Constraints - Ash.Type.Float","doc":"* `:max` - Enforces a maximum on the value\n\n* `:min` - Enforces a minimum on the value\n\n* `:greater_than` - Enforces a minimum on the value (exclusive)\n\n* `:less_than` - Enforces a maximum on the value (exclusive)","ref":"Ash.Type.Float.html#module-constraints"},{"type":"function","title":"Ash.Type.Float.handle_change?/0","doc":"","ref":"Ash.Type.Float.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Float.prepare_change?/0","doc":"","ref":"Ash.Type.Float.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Function","doc":"Represents a function.\n\nIf the type would be dumped to a native format, `:erlang.term_to_binary(term, [:safe])` is used.\n\nPlease keep in mind, this is *NOT SAFE* to use with external input.\n\nMore information available here: https://erlang.org/doc/man/erlang.html#binary_to_term-2","ref":"Ash.Type.Function.html"},{"type":"module","title":"Constraints - Ash.Type.Function","doc":"* `:arity` (`t:pos_integer/0`) - Enforces a specific arity on the provided function","ref":"Ash.Type.Function.html#module-constraints"},{"type":"function","title":"Ash.Type.Function.handle_change?/0","doc":"","ref":"Ash.Type.Function.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Function.prepare_change?/0","doc":"","ref":"Ash.Type.Function.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Integer","doc":"Represents a simple integer\n\nA builtin type that can be referenced via `:integer`","ref":"Ash.Type.Integer.html"},{"type":"module","title":"Constraints - Ash.Type.Integer","doc":"* `:max` - Enforces a maximum on the value\n\n* `:min` - Enforces a minimum on the value","ref":"Ash.Type.Integer.html#module-constraints"},{"type":"function","title":"Ash.Type.Integer.handle_change?/0","doc":"","ref":"Ash.Type.Integer.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Integer.prepare_change?/0","doc":"","ref":"Ash.Type.Integer.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Keyword","doc":"Represents a keyword list, stored as a `:map` in the database.\n\nA builtin type that can be referenced via `:keyword_list`\n\n* `:fields` (`t:keyword/0`) - Required. The types of the fields in the keyword, and their constraints.  \n  If constraints are specified, only those fields will be in the casted keyword.  \n  For example:  \n      fields:  [\n        amount: [\n          type: :integer,\n          description: \"The amount of the transaction\",\n          constraints: [\n            max: 10\n          ]\n        ],\n        currency: [\n          type: :string,\n          allow_nil?: false,\n          description: \"The currency code of the transaction\",\n          constraints: [\n            max_length: 3\n          ]\n        ]\n      ]  \n  allow_nil? is true by default\n\n  * `:type` (an `Ash.Type`) - Required.\n\n  * `:description` (`t:String.t/0`)\n\n  * `:allow_nil?` (`t:boolean/0`) - The default value is `true`.\n\n  * `:constraints` (`t:keyword/0`) - The default value is `[]`.","ref":"Ash.Type.Keyword.html"},{"type":"function","title":"Ash.Type.Keyword.handle_change?/0","doc":"","ref":"Ash.Type.Keyword.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Keyword.prepare_change?/0","doc":"","ref":"Ash.Type.Keyword.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Map","doc":"Represents a map stored in the database.\n\nIn postgres, for example, this represents binary encoded json\n\nA builtin type that can be referenced via `:map`\n\n* `:fields` (`t:keyword/0`) - The types of the fields in the map, and their constraints.  \n  If constraints are specified, only those fields will be in the casted map.  \n  For example:  \n      fields:  [\n        amount: [\n          type: :integer,\n          description: \"The amount of the transaction\",\n          constraints: [\n            max: 10\n          ]\n        ],\n        currency: [\n          type: :string,\n          allow_nil?: false,\n          description: \"The currency code of the transaction\",\n          constraints: [\n            max_length: 3\n          ]\n        ]\n      ]  \n  allow_nil? is true by default\n\n  * `:type` (an `Ash.Type`) - Required.\n\n  * `:allow_nil?` (`t:boolean/0`) - The default value is `true`.\n\n  * `:description` (`t:String.t/0`)\n\n  * `:constraints` (`t:keyword/0`) - The default value is `[]`.","ref":"Ash.Type.Map.html"},{"type":"function","title":"Ash.Type.Map.handle_change?/0","doc":"","ref":"Ash.Type.Map.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Map.prepare_change?/0","doc":"","ref":"Ash.Type.Map.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Module","doc":"Stores a module as a string in the database.\n\nA builtin type that can be referenced via `:module`.","ref":"Ash.Type.Module.html"},{"type":"module","title":"Constraints - Ash.Type.Module","doc":"* `:behaviour` (`t:atom/0`) - Allows constraining the module a one which implements a behaviour\n\n* `:protocol` (`t:atom/0`) - Allows constraining the module a one which implements a protocol","ref":"Ash.Type.Module.html#module-constraints"},{"type":"function","title":"Ash.Type.Module.handle_change?/0","doc":"","ref":"Ash.Type.Module.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Module.prepare_change?/0","doc":"","ref":"Ash.Type.Module.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.NaiveDatetime","doc":"Represents a Naive datetime\n\nA builtin type that can be referenced via `:naive_datetime`","ref":"Ash.Type.NaiveDatetime.html"},{"type":"function","title":"Ash.Type.NaiveDatetime.handle_change?/0","doc":"","ref":"Ash.Type.NaiveDatetime.html#handle_change?/0"},{"type":"function","title":"Ash.Type.NaiveDatetime.prepare_change?/0","doc":"","ref":"Ash.Type.NaiveDatetime.html#prepare_change?/0"},{"type":"behaviour","title":"Ash.Type.NewType","doc":"Allows defining a new type that is the combination of an existing type and custom constraints\n\nA subtle difference between this type and its supertype (one that will almost certainly not matter\nin any case) is that we use the `apply_constraints` logic of the underlying type in the same step\nas `cast_input`. We do this because new types like these are, generally speaking, considering the constraint\napplication as part of the core type. Other types, if you simply do `Ash.Type.cast_input/3` you will not be\nalso applying their constraints.","ref":"Ash.Type.NewType.html"},{"type":"behaviour","title":"Options - Ash.Type.NewType","doc":"- `:subtype_of` - The type that this new type is a subtype of.\n- `:constraints` - The constraints that this new type uses for the underlying type.\n- `:lazy_init?` - If true, the `init/1` function will be called at runtime instead of compile time.\n  Allows for recursive types.\n\nFor Example:\n\n```elixir\ndefmodule MyApp.Types.SSN do\n  use Ash.Type.NewType, subtype_of: :string, constraints: [match: ~r/regex for ssn/]\nend\n\ndefmodule MyApp.Types.Metadata do\n  use Ash.Type.NewType, subtype_of: :union, constraints: [types: [\n    foo: [...],\n    bar: [...]\n  ]]\nend\n```","ref":"Ash.Type.NewType.html#module-options"},{"type":"function","title":"Ash.Type.NewType.constraints/2","doc":"Returns the constraints schema.","ref":"Ash.Type.NewType.html#constraints/2"},{"type":"callback","title":"Ash.Type.NewType.lazy_init?/0","doc":"Whether or not the type is lazy initialized (so needs to be initialized when fetching constraints)","ref":"Ash.Type.NewType.html#c:lazy_init?/0"},{"type":"function","title":"Ash.Type.NewType.new_type?/1","doc":"Returns true if the corresponding type is an Ash.Type.NewType","ref":"Ash.Type.NewType.html#new_type?/1"},{"type":"callback","title":"Ash.Type.NewType.subtype_constraints/0","doc":"Returns the underlying subtype constraints","ref":"Ash.Type.NewType.html#c:subtype_constraints/0"},{"type":"callback","title":"Ash.Type.NewType.subtype_of/0","doc":"Returns the type that the NewType is a subtype of.","ref":"Ash.Type.NewType.html#c:subtype_of/0"},{"type":"function","title":"Ash.Type.NewType.subtype_of/1","doc":"Returns the type that the given newtype is a subtype of","ref":"Ash.Type.NewType.html#subtype_of/1"},{"type":"type","title":"Ash.Type.NewType.t/0","doc":"","ref":"Ash.Type.NewType.html#t:t/0"},{"type":"callback","title":"Ash.Type.NewType.type_constraints/2","doc":"Returns the modified NewType constraints","ref":"Ash.Type.NewType.html#c:type_constraints/2"},{"type":"module","title":"Ash.Type.String","doc":"Stores a string in the database.\n\nA built-in type that can be referenced via `:string`.\n\nBy default, values are trimmed and empty values are set to `nil`.\nYou can use the `allow_empty?` and `trim?` constraints to change these behaviors.","ref":"Ash.Type.String.html"},{"type":"module","title":"Constraints - Ash.Type.String","doc":"* `:max_length` (`t:non_neg_integer/0`) - Enforces a maximum length on the value\n\n* `:min_length` (`t:non_neg_integer/0`) - Enforces a minimum length on the value\n\n* `:match` (`t:Regex.t/0`) - Enforces that the string matches a passed in regex\n\n* `:trim?` (`t:boolean/0`) - Trims the value. The default value is `true`.\n\n* `:allow_empty?` (`t:boolean/0`) - If false, the value is set to `nil` if it's empty. The default value is `false`.","ref":"Ash.Type.String.html#module-constraints"},{"type":"function","title":"Ash.Type.String.handle_change?/0","doc":"","ref":"Ash.Type.String.html#handle_change?/0"},{"type":"function","title":"Ash.Type.String.prepare_change?/0","doc":"","ref":"Ash.Type.String.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Struct","doc":"Represents a struct.\n\nUse the `instance_of` constraint to specify that it must be an instance of a specific struct.\n\nThis cannot be loaded from a database unless the `instance_of` constraint is provided.\nIf not, it can only be used to cast input, i.e for arguments.","ref":"Ash.Type.Struct.html"},{"type":"module","title":"Constraints - Ash.Type.Struct","doc":"* `:instance_of` (`t:atom/0`) - The module the struct should be an instance of\n\n* `:preserve_nil_values?` (`t:boolean/0`) - If set to true, when storing, nil values will be kept. Otherwise, nil values will be omitted. The default value is `false`.\n\n* `:fields` (`t:keyword/0`) - The types of the fields in the struct, and their constraints.  \n  For example:  \n      fields:  [\n        amount: [\n          type: :integer,\n          description: \"The amount of the transaction\",\n          constraints: [\n            max: 10\n          ]\n        ],\n        currency: [\n          type: :string,\n          allow_nil?: false,\n          description: \"The currency code of the transaction\",\n          constraints: [\n            max_length: 3\n          ]\n        ]\n      ]  \n  allow_nil? is true by default\n\n  * `:type` (an `Ash.Type`) - Required.\n\n  * `:allow_nil?` (`t:boolean/0`) - The default value is `true`.\n\n  * `:description` (`t:String.t/0`)\n\n  * `:constraints` (`t:keyword/0`) - The default value is `[]`.","ref":"Ash.Type.Struct.html#module-constraints"},{"type":"function","title":"Ash.Type.Struct.handle_change?/0","doc":"","ref":"Ash.Type.Struct.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Struct.prepare_change?/0","doc":"","ref":"Ash.Type.Struct.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Term","doc":"Represents a raw elixir term in the database\n\nA builtin type that can be referenced via `:binary`","ref":"Ash.Type.Term.html"},{"type":"function","title":"Ash.Type.Term.handle_change?/0","doc":"","ref":"Ash.Type.Term.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Term.prepare_change?/0","doc":"","ref":"Ash.Type.Term.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Time","doc":"Represents a time in the database, with a 'second' precision\n\nA builtin type that can be referenced via `:time`","ref":"Ash.Type.Time.html"},{"type":"module","title":"Constraints - Ash.Type.Time","doc":"* `:precision` - Valid values are :microsecond, :second The default value is `:second`.","ref":"Ash.Type.Time.html#module-constraints"},{"type":"function","title":"Ash.Type.Time.handle_change?/0","doc":"","ref":"Ash.Type.Time.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Time.prepare_change?/0","doc":"","ref":"Ash.Type.Time.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.TimeUsec","doc":"Represents a time with `microsecond` precision.\n\nA builtin type that can be referenced via `:time_usec`","ref":"Ash.Type.TimeUsec.html"},{"type":"function","title":"Ash.Type.TimeUsec.handle_change?/0","doc":"","ref":"Ash.Type.TimeUsec.html#handle_change?/0"},{"type":"function","title":"Ash.Type.TimeUsec.prepare_change?/0","doc":"","ref":"Ash.Type.TimeUsec.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Tuple","doc":"Represents a tuple stored in the data layer as a map.\n\nA builtin type that can be referenced via `:tuple`\n\n* `:fields` (`t:keyword/0`) - Required. The types of the fields in the tuple, and their constraints.  \n  This type is stored as a map under the hood, keyed by the field name,\n  and is represented in memory as a tuple.  \n  Example constraints:  \n      constraints: fields:  [\n                    amount: [\n                      type: :integer,\n                      description: \"The amount of the transaction\",\n                      constraints: [\n                        max: 10\n                      ]\n                    ],\n                    currency: [\n                      type: :string,\n                      allow_nil?: false,\n                      description: \"The currency code of the transaction\",\n                      constraints: [\n                        max_length: 3\n                      ]\n                    ]\n                  ]  \n  `allow_nil?` is `true` by default.\n\n  * `:type` (an `Ash.Type`) - Required.\n\n  * `:allow_nil?` (`t:boolean/0`) - The default value is `true`.\n\n  * `:description` (`t:String.t/0`)\n\n  * `:constraints` (`t:keyword/0`) - The default value is `[]`.","ref":"Ash.Type.Tuple.html"},{"type":"function","title":"Ash.Type.Tuple.handle_change?/0","doc":"","ref":"Ash.Type.Tuple.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Tuple.prepare_change?/0","doc":"","ref":"Ash.Type.Tuple.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.UUID","doc":"Represents a UUID.\n\nA builtin type that can be referenced via `:uuid`","ref":"Ash.Type.UUID.html"},{"type":"function","title":"Ash.Type.UUID.handle_change?/0","doc":"","ref":"Ash.Type.UUID.html#handle_change?/0"},{"type":"function","title":"Ash.Type.UUID.prepare_change?/0","doc":"","ref":"Ash.Type.UUID.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.UUIDv7","doc":"Represents a UUID.\n\nA builtin type that can be referenced via `:uuid_v7`","ref":"Ash.Type.UUIDv7.html"},{"type":"function","title":"Ash.Type.UUIDv7.handle_change?/0","doc":"","ref":"Ash.Type.UUIDv7.html#handle_change?/0"},{"type":"function","title":"Ash.Type.UUIDv7.prepare_change?/0","doc":"","ref":"Ash.Type.UUIDv7.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Union","doc":"A union between multiple types, distinguished with a tag or by attempting to validate.\n\nUnion types allow you to define attributes that can hold values of different types. There are two\nmain strategies for distinguishing between types:\n\n1. **Tagged unions** - Uses a specific field (tag) and value (tag_value) to identify the type\n2. **Untagged unions** - Attempts to cast the value against each type in order until one succeeds","ref":"Ash.Type.Union.html"},{"type":"module","title":"Basic Usage - Ash.Type.Union","doc":"Define a union type in an attribute:\n\n    attribute :content, :union,\n      constraints: [\n        types: [\n          text: [type: :string],\n          number: [type: :integer],\n          flag: [type: :boolean]\n        ]\n      ]\n\nValues are wrapped in an `%Ash.Union{}` struct with `:type` and `:value` fields:\n\n    # Reading union values\n    %Ash.Union{type: :text, value: \"Hello\"}\n    %Ash.Union{type: :number, value: 42}","ref":"Ash.Type.Union.html#module-basic-usage"},{"type":"module","title":"Tagged Unions - Ash.Type.Union","doc":"Tagged unions use a discriminator field to identify the type. This is more reliable but requires\nthe data to include the tag field:\n\n    attribute :data, :union,\n      constraints: [\n        types: [\n          user: [\n            type: :map,\n            tag: :type,\n            tag_value: \"user\"\n          ],\n          admin: [\n            type: :map,\n            tag: :type,\n            tag_value: \"admin\"\n          ]\n        ]\n      ]\n\nInput data must include the tag field:\n\n    # Valid inputs\n    %{type: \"user\", name: \"John\", email: \"john@example.com\"}\n    %{type: \"admin\", name: \"Jane\", permissions: [\"read\", \"write\"]}","ref":"Ash.Type.Union.html#module-tagged-unions"},{"type":"module","title":"Tag Options - Ash.Type.Union","doc":"- `tag` - The field name to check (e.g., `:type`, `:kind`, `:__type__`)\n- `tag_value` - The expected value for this type (string, atom, or nil)\n- `cast_tag?` - Whether to include the tag in the final value (default: `true`)\n\nWhen `cast_tag?: false`, the tag field is removed from the final value.","ref":"Ash.Type.Union.html#module-tag-options"},{"type":"module","title":"Untagged Unions - Ash.Type.Union","doc":"Without tags, union types attempt to cast values against each type in order:\n\n    attribute :flexible, :union,\n      constraints: [\n        types: [\n          integer: [type: :integer],\n          string: [type: :string]\n        ]\n      ]\n\n**Order matters!** The first successful cast wins:\n\n    # \"42\" would be cast as :integer (42), not :string (\"42\")\n    # if integer comes first in the types list","ref":"Ash.Type.Union.html#module-untagged-unions"},{"type":"module","title":"Mixed Tagged and Untagged Types - Ash.Type.Union","doc":"You can mix tagged and untagged types within a single union. Tagged types are checked first\nby their tag values, and if no tagged type matches, untagged types are tried in order:\n\n    attribute :flexible_data, :union,\n      constraints: [\n        types: [\n          # Tagged types - checked first by tag\n          user: [\n            type: :map,\n            tag: :type,\n            tag_value: \"user\"\n          ],\n          admin: [\n            type: :map,\n            tag: :type,\n            tag_value: \"admin\"\n          ],\n          # Untagged types - tried in order if no tag matches\n          number: [type: :integer],\n          text: [type: :string]\n        ]\n      ]\n\nThis allows for both explicit type identification (via tags) and fallback casting:\n\n    # Tagged - uses :type field to determine it's a user\n    %{type: \"user\", name: \"John\"}\n\n    # Untagged - tries :integer first, then :string\n    42        # -> %Ash.Union{type: :number, value: 42}\n    \"hello\"   # -> %Ash.Union{type: :text, value: \"hello\"}","ref":"Ash.Type.Union.html#module-mixed-tagged-and-untagged-types"},{"type":"module","title":"Storage Modes - Ash.Type.Union","doc":"Union values can be stored in different formats:\n\n### `:type_and_value` (default)\n\nStores as a map with explicit type and value fields:\n\n    # Stored as: %{\"type\" => \"text\", \"value\" => \"Hello\"}\n\n### `:map_with_tag`\n\nStores the value directly (requires all types to have tags):\n\n    # Stored as: %{\"type\" => \"user\", \"name\" => \"John\", \"email\" => \"john@example.com\"}\n\n    constraints: [\n      storage: :map_with_tag,\n      types: [\n        user: [type: :map, tag: :type, tag_value: \"user\"],\n        admin: [type: :map, tag: :type, tag_value: \"admin\"]\n      ]\n    ]","ref":"Ash.Type.Union.html#module-storage-modes"},{"type":"module","title":"Embedded Resources - Ash.Type.Union","doc":"Union types work seamlessly with embedded resources:\n\n    attribute :contact_info, :union,\n      constraints: [\n        types: [\n          email: [\n            type: EmailContact,\n            tag: :type,\n            tag_value: \"email\"\n          ],\n          phone: [\n            type: PhoneContact,\n            tag: :type,\n            tag_value: \"phone\"\n          ]\n        ]\n      ]","ref":"Ash.Type.Union.html#module-embedded-resources"},{"type":"module","title":"Arrays of Unions - Ash.Type.Union","doc":"Union types support arrays using the standard `{:array, :union}` syntax:\n\n    attribute :mixed_data, {:array, :union},\n      constraints: [\n        items: [\n          types: [\n            text: [type: :string],\n            number: [type: :integer]\n          ]\n        ]\n      ]","ref":"Ash.Type.Union.html#module-arrays-of-unions"},{"type":"module","title":"Advanced Input Formats - Ash.Type.Union","doc":"Union types support multiple input formats for flexibility:","ref":"Ash.Type.Union.html#module-advanced-input-formats"},{"type":"module","title":"Direct Union Struct - Ash.Type.Union","doc":"%Ash.Union{type: :text, value: \"Hello\"}","ref":"Ash.Type.Union.html#module-direct-union-struct"},{"type":"module","title":"Tagged Map (when tags are configured) - Ash.Type.Union","doc":"%{type: \"text\", content: \"Hello\"}","ref":"Ash.Type.Union.html#module-tagged-map-when-tags-are-configured"},{"type":"module","title":"Explicit Union Format - Ash.Type.Union","doc":"%{\n      \"_union_type\" => \"text\",\n      \"_union_value\" => \"Hello\"\n    }\n\n    # Or with the value merged in\n    %{\n      \"_union_type\" => \"text\",\n      \"content\" => \"Hello\"\n    }","ref":"Ash.Type.Union.html#module-explicit-union-format"},{"type":"module","title":"Nested Unions - Ash.Type.Union","doc":"Unions can contain other union types. All type names must be unique across the entire\nnested structure:\n\n    types: [\n      simple: [type: :string],\n      complex: [\n        type: :union,\n        constraints: [\n          types: [\n            # Names must be unique - can't reuse 'simple'\n            nested_text: [type: :string],\n            nested_num: [type: :integer]\n          ]\n        ]\n      ]\n    ]","ref":"Ash.Type.Union.html#module-nested-unions"},{"type":"module","title":"Loading and Calculations - Ash.Type.Union","doc":"Union types support loading related data when member types are loadable (like embedded resources):\n\n    # Load through all union types\n    query |> Ash.Query.load(union_field: :*)\n\n    # Load through specific type\n    query |> Ash.Query.load(union_field: [user: [:profile, :preferences]])","ref":"Ash.Type.Union.html#module-loading-and-calculations"},{"type":"module","title":"Error Handling - Ash.Type.Union","doc":"Union casting provides detailed error information:\n\n- **Tagged unions**: Clear errors when tags don't match expected values\n- **Untagged unions**: Aggregated errors from all attempted type casts\n- **Array unions**: Errors include index and path information for debugging","ref":"Ash.Type.Union.html#module-error-handling"},{"type":"module","title":"NewType Integration - Ash.Type.Union","doc":"Create reusable union types with `Ash.Type.NewType`:\n\n    defmodule MyApp.Types.ContactMethod do\n      use Ash.Type.NewType,\n        subtype_of: :union,\n        constraints: [\n          types: [\n            email: [type: :string, constraints: [match: ~r/@/]],\n            phone: [type: :string, constraints: [match: ~r/^+$/]]\n          ]\n        ]\n    end","ref":"Ash.Type.Union.html#module-newtype-integration"},{"type":"module","title":"Performance Considerations - Ash.Type.Union","doc":"- **Tagged unions** are more efficient as they avoid trial-and-error casting\n- **Type order matters** in untagged unions - put more specific types first\n- **Use constraints** on member types to fail fast on invalid data","ref":"Ash.Type.Union.html#module-performance-considerations"},{"type":"module","title":"Constraints - Ash.Type.Union","doc":"* `:storage` - How the value will be stored when persisted.  \n  `:type_and_value` will store the type and value in a map like so `{type: :type_name, value: the_value}`\n  `:map_with_tag` will store the value directly. This only works if all types have a `tag` and `tag_value` configured.\n   Valid values are :type_and_value, :map_with_tag The default value is `:type_and_value`.\n\n* `:include_source?` (`t:boolean/0`) - Whether to include the source changeset in the context. Defaults to the value of `config :ash, :include_embedded_source_by_default`, or `true`. In 4.x, the default will be `false`. The default value is `true`.\n\n* `:types` - The types to be unioned, a map of an identifier for the enum value to its configuration.  \n  When using `tag` and `tag_value` we are referring to a map key that must equal a certain value\n  in order for the value to be considered an instance of that type.  \n  For example:  \n      types:  [\n        int: [\n          type: :integer,\n          constraints: [\n            max: 10\n          ]\n        ],\n        object: [\n          type: MyObjectType,\n          # The default value is `true`\n          # this passes the tag key/value to the nested type\n          # when casting input\n          cast_tag?: true,\n          tag: :type,\n          tag_value: \"my_object\"\n        ],\n        other_object: [\n          type: MyOtherObjectType,\n          cast_tag?: true,\n          tag: :type,\n          tag_value: \"my_other_object\"\n        ],\n        other_object_without_type: [\n          type: MyOtherObjectTypeWithoutType,\n          cast_tag?: false,\n          tag: :type,\n          tag_value: nil\n        ]\n      ]  \n  IMPORTANT:  \n  This is stored as a map under the hood. Filters over the data will need to take this into account.  \n  Additionally, if you are not using a tag, a value will be considered to be of the given type if it successfully casts.\n  This means that, for example, if you try to cast `\"10\"` as a union of a string and an integer, it will end up as `\"10\"` because\n  it is a string. If you put the integer type ahead of the string type, it will cast first and `10` will be the value.","ref":"Ash.Type.Union.html#module-constraints"},{"type":"function","title":"Ash.Type.Union.handle_change?/0","doc":"","ref":"Ash.Type.Union.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Union.prepare_change?/0","doc":"","ref":"Ash.Type.Union.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.UrlEncodedBinary","doc":"Represents a binary that attempts to decode input strings as a url encoded base64 string.\n\nA builtin type that can be referenced via `:url_encoded_binary`","ref":"Ash.Type.UrlEncodedBinary.html"},{"type":"function","title":"Ash.Type.UrlEncodedBinary.handle_change?/0","doc":"","ref":"Ash.Type.UrlEncodedBinary.html#handle_change?/0"},{"type":"function","title":"Ash.Type.UrlEncodedBinary.prepare_change?/0","doc":"","ref":"Ash.Type.UrlEncodedBinary.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.UtcDatetime","doc":"Represents a utc datetime, with 'second' precision. A wrapper around `:datetime` for backwards compatibility.\n\nA builtin type that can be referenced via `:utc_datetime`","ref":"Ash.Type.UtcDatetime.html"},{"type":"function","title":"Ash.Type.UtcDatetime.handle_change?/0","doc":"","ref":"Ash.Type.UtcDatetime.html#handle_change?/0"},{"type":"function","title":"Ash.Type.UtcDatetime.prepare_change?/0","doc":"","ref":"Ash.Type.UtcDatetime.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.UtcDatetimeUsec","doc":"Represents a utc datetime with `microsecond` precision. A wrapper around `:datetime` for backwards compatibility.\n\nA builtin type that can be referenced via `:utc_datetime_usec`","ref":"Ash.Type.UtcDatetimeUsec.html"},{"type":"function","title":"Ash.Type.UtcDatetimeUsec.handle_change?/0","doc":"","ref":"Ash.Type.UtcDatetimeUsec.html#handle_change?/0"},{"type":"function","title":"Ash.Type.UtcDatetimeUsec.prepare_change?/0","doc":"","ref":"Ash.Type.UtcDatetimeUsec.html#prepare_change?/0"},{"type":"module","title":"Ash.Type.Vector","doc":"Represents a vector.\n\nA builtin type that can be referenced via `:vector`.","ref":"Ash.Type.Vector.html"},{"type":"module","title":"Constraints - Ash.Type.Vector","doc":"* `:dimensions` (`t:integer/0`) - The dimensions of the vector","ref":"Ash.Type.Vector.html#module-constraints"},{"type":"function","title":"Ash.Type.Vector.handle_change?/0","doc":"","ref":"Ash.Type.Vector.html#handle_change?/0"},{"type":"function","title":"Ash.Type.Vector.prepare_change?/0","doc":"","ref":"Ash.Type.Vector.html#prepare_change?/0"},{"type":"module","title":"Ash.Error","doc":"Tools and utilities used by Ash to manage and conform errors","ref":"Ash.Error.html"},{"type":"type","title":"Ash.Error.ash_error/0","doc":"","ref":"Ash.Error.html#t:ash_error/0"},{"type":"function","title":"Ash.Error.ash_error?/1","doc":"Returns whether or not a term is an Ash.Error type.","ref":"Ash.Error.html#ash_error?/1"},{"type":"type","title":"Ash.Error.ash_error_subject/0","doc":"","ref":"Ash.Error.html#t:ash_error_subject/0"},{"type":"type","title":"Ash.Error.class/0","doc":"","ref":"Ash.Error.html#t:class/0"},{"type":"type","title":"Ash.Error.class_module/0","doc":"","ref":"Ash.Error.html#t:class_module/0"},{"type":"type","title":"Ash.Error.error_class/0","doc":"","ref":"Ash.Error.html#t:error_class/0"},{"type":"function","title":"Ash.Error.error_descriptions/1","doc":"Converts errors into a single `String.t`.","ref":"Ash.Error.html#error_descriptions/1"},{"type":"type","title":"Ash.Error.error_input/0","doc":"","ref":"Ash.Error.html#t:error_input/0"},{"type":"type","title":"Ash.Error.error_keyword/0","doc":"","ref":"Ash.Error.html#t:error_keyword/0"},{"type":"type","title":"Ash.Error.error_keyword_option/0","doc":"","ref":"Ash.Error.html#t:error_keyword_option/0"},{"type":"type","title":"Ash.Error.path/0","doc":"","ref":"Ash.Error.html#t:path/0"},{"type":"type","title":"Ash.Error.path_input/0","doc":"","ref":"Ash.Error.html#t:path_input/0"},{"type":"function","title":"Ash.Error.set_path/2","doc":"This function prepends the provided path to any existing path on the errors.","ref":"Ash.Error.html#set_path/2"},{"type":"function","title":"Ash.Error.splode_error?/2","doc":"","ref":"Ash.Error.html#splode_error?/2"},{"type":"type","title":"Ash.Error.t/0","doc":"","ref":"Ash.Error.html#t:t/0"},{"type":"function","title":"Ash.Error.to_ash_error/3","doc":"Converts a value to an Ash exception.\n\nThe supported inputs to this function can be provided to various places,\nlike `Ash.Query.add_error/2`, `Ash.Changeset.add_error/2` and `Ash.ActionInput.add_error/2`.\n\nAdditionally, any place that you can *return* an error you can return instead a valid\nerror input.\n\nSee [the error handling guide](/documentation/development/error-handling.md) for more.","ref":"Ash.Error.html#to_ash_error/3"},{"type":"function","title":"Ash.Error.to_error_class/2","doc":"Converts a value to an Ash.Error type.","ref":"Ash.Error.html#to_error_class/2"},{"type":"function","title":"Ash.Error.unwrap!/2","doc":"Raises an error if the result is an error, otherwise returns the result\n\nAlternatively, you can use the `defsplode` macro, which does this automatically.","ref":"Ash.Error.html#unwrap!/2"},{"type":"function","title":"Options - Ash.Error.unwrap!/2","doc":"- `:error_opts` - Options to pass to `to_error/2` when converting the returned error\n- `:unknown_error_opts` - Options to pass to the unknown error if the function returns only `:error`.\n  not necessary if your function always returns `{:error, error}`.","ref":"Ash.Error.html#unwrap!/2-options"},{"type":"function","title":"Examples - Ash.Error.unwrap!/2","doc":"def function(arg) do\n    case do_something(arg) do\n      :success -> :ok\n      {:success, result} -> {:ok, result}\n      {:error, error} -> {:error, error}\n    end\n  end\n\n  def function!(arg) do\n    YourErrors.unwrap!(function(arg))\n  end","ref":"Ash.Error.html#unwrap!/2-examples"},{"type":"exception","title":"Ash.Error.Action.InvalidArgument","doc":"Used when an invalid value is provided for an action argument","ref":"Ash.Error.Action.InvalidArgument.html"},{"type":"function","title":"Ash.Error.Action.InvalidArgument.exception/1","doc":"Create an `Elixir.Ash.Error.Action.InvalidArgument` without raising it.","ref":"Ash.Error.Action.InvalidArgument.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Action.InvalidArgument.exception/1","doc":"- :field\n- :message\n- :value","ref":"Ash.Error.Action.InvalidArgument.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.ActionRequiresActor","doc":"Used when an actor is referenced in a filter template, but no actor exists","ref":"Ash.Error.Changes.ActionRequiresActor.html"},{"type":"function","title":"Ash.Error.Changes.ActionRequiresActor.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.ActionRequiresActor` without raising it.","ref":"Ash.Error.Changes.ActionRequiresActor.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.ActionRequiresActor.exception/1","doc":"","ref":"Ash.Error.Changes.ActionRequiresActor.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.InvalidArgument","doc":"Used when an invalid value is provided for an action argument","ref":"Ash.Error.Changes.InvalidArgument.html"},{"type":"function","title":"Ash.Error.Changes.InvalidArgument.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.InvalidArgument` without raising it.","ref":"Ash.Error.Changes.InvalidArgument.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.InvalidArgument.exception/1","doc":"- :field\n- :message\n- :value","ref":"Ash.Error.Changes.InvalidArgument.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.InvalidAttribute","doc":"Used when an invalid value is provided for an attribute change","ref":"Ash.Error.Changes.InvalidAttribute.html"},{"type":"function","title":"Ash.Error.Changes.InvalidAttribute.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.InvalidAttribute` without raising it.","ref":"Ash.Error.Changes.InvalidAttribute.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.InvalidAttribute.exception/1","doc":"- :field\n- :message\n- :private_vars\n- :value\n- :has_value?","ref":"Ash.Error.Changes.InvalidAttribute.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.InvalidChanges","doc":"Used when a change is provided that covers multiple attributes/relationships","ref":"Ash.Error.Changes.InvalidChanges.html"},{"type":"function","title":"Ash.Error.Changes.InvalidChanges.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.InvalidChanges` without raising it.","ref":"Ash.Error.Changes.InvalidChanges.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.InvalidChanges.exception/1","doc":"- :fields\n- :message\n- :validation\n- :value","ref":"Ash.Error.Changes.InvalidChanges.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.InvalidRelationship","doc":"Used when an invalid value is provided for a relationship change","ref":"Ash.Error.Changes.InvalidRelationship.html"},{"type":"function","title":"Ash.Error.Changes.InvalidRelationship.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.InvalidRelationship` without raising it.","ref":"Ash.Error.Changes.InvalidRelationship.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.InvalidRelationship.exception/1","doc":"- :relationship\n- :message","ref":"Ash.Error.Changes.InvalidRelationship.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.NoSuchAttribute","doc":"Used when a change is provided for an attribute that does not exist","ref":"Ash.Error.Changes.NoSuchAttribute.html"},{"type":"function","title":"Ash.Error.Changes.NoSuchAttribute.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.NoSuchAttribute` without raising it.","ref":"Ash.Error.Changes.NoSuchAttribute.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.NoSuchAttribute.exception/1","doc":"- :resource\n- :attribute","ref":"Ash.Error.Changes.NoSuchAttribute.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.NoSuchRelationship","doc":"Used when a change is provided for an relationship that does not exist","ref":"Ash.Error.Changes.NoSuchRelationship.html"},{"type":"function","title":"Ash.Error.Changes.NoSuchRelationship.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.NoSuchRelationship` without raising it.","ref":"Ash.Error.Changes.NoSuchRelationship.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.NoSuchRelationship.exception/1","doc":"- :resource\n- :relationship","ref":"Ash.Error.Changes.NoSuchRelationship.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.Required","doc":"Used when an attribute or relationship is required","ref":"Ash.Error.Changes.Required.html"},{"type":"function","title":"Ash.Error.Changes.Required.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.Required` without raising it.","ref":"Ash.Error.Changes.Required.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.Required.exception/1","doc":"- :field\n- :type\n- :resource","ref":"Ash.Error.Changes.Required.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Changes.StaleRecord","doc":"Used when a stale record is attempted to be updated or deleted","ref":"Ash.Error.Changes.StaleRecord.html"},{"type":"function","title":"Ash.Error.Changes.StaleRecord.exception/1","doc":"Create an `Elixir.Ash.Error.Changes.StaleRecord` without raising it.","ref":"Ash.Error.Changes.StaleRecord.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Changes.StaleRecord.exception/1","doc":"- :resource\n- :filter\n- :field","ref":"Ash.Error.Changes.StaleRecord.html#exception/1-keys"},{"type":"module","title":"Ash.Error.Exception","doc":"Tooling for creating an Ash exception","ref":"Ash.Error.Exception.html"},{"type":"macro","title":"Ash.Error.Exception.def_ash_error/2","doc":"","ref":"Ash.Error.Exception.html#def_ash_error/2"},{"type":"exception","title":"Ash.Error.Forbidden","doc":"Used when authorization for an action fails","ref":"Ash.Error.Forbidden.html"},{"type":"function","title":"Ash.Error.Forbidden.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden` without raising it.","ref":"Ash.Error.Forbidden.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.exception/1","doc":"- :changeset\n- :query\n- :action_input\n- :errors","ref":"Ash.Error.Forbidden.html#exception/1-keys"},{"type":"function","title":"Ash.Error.Forbidden.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Ash.Error.Forbidden.html#message/1"},{"type":"type","title":"Ash.Error.Forbidden.t/0","doc":"","ref":"Ash.Error.Forbidden.html#t:t/0"},{"type":"exception","title":"Ash.Error.Forbidden.CannotFilterCreates","doc":"Used when a create action would be filtered","ref":"Ash.Error.Forbidden.CannotFilterCreates.html"},{"type":"function","title":"Ash.Error.Forbidden.CannotFilterCreates.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.CannotFilterCreates` without raising it.","ref":"Ash.Error.Forbidden.CannotFilterCreates.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.CannotFilterCreates.exception/1","doc":"- :filter","ref":"Ash.Error.Forbidden.CannotFilterCreates.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.DomainRequiresActor","doc":"Used when a domain that has `require_actor? true` is provided no actor","ref":"Ash.Error.Forbidden.DomainRequiresActor.html"},{"type":"function","title":"Ash.Error.Forbidden.DomainRequiresActor.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.DomainRequiresActor` without raising it.","ref":"Ash.Error.Forbidden.DomainRequiresActor.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.DomainRequiresActor.exception/1","doc":"- :domain","ref":"Ash.Error.Forbidden.DomainRequiresActor.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.DomainRequiresAuthorization","doc":"Used when a domain that has `authorize :always` is provided authorize?: false","ref":"Ash.Error.Forbidden.DomainRequiresAuthorization.html"},{"type":"function","title":"Ash.Error.Forbidden.DomainRequiresAuthorization.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.DomainRequiresAuthorization` without raising it.","ref":"Ash.Error.Forbidden.DomainRequiresAuthorization.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.DomainRequiresAuthorization.exception/1","doc":"- :domain","ref":"Ash.Error.Forbidden.DomainRequiresAuthorization.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.ForbiddenField","doc":"Raised in cases where access to a specific field was prevented","ref":"Ash.Error.Forbidden.ForbiddenField.html"},{"type":"function","title":"Ash.Error.Forbidden.ForbiddenField.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.ForbiddenField` without raising it.","ref":"Ash.Error.Forbidden.ForbiddenField.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.ForbiddenField.exception/1","doc":"- :resource\n- :field","ref":"Ash.Error.Forbidden.ForbiddenField.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.InitialDataRequired","doc":"Used when initial data was not supplied when it was required","ref":"Ash.Error.Forbidden.InitialDataRequired.html"},{"type":"function","title":"Ash.Error.Forbidden.InitialDataRequired.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.InitialDataRequired` without raising it.","ref":"Ash.Error.Forbidden.InitialDataRequired.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.InitialDataRequired.exception/1","doc":"- :source","ref":"Ash.Error.Forbidden.InitialDataRequired.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.MustPassStrictCheck","doc":"Used when unreachable code/conditions are reached in the framework","ref":"Ash.Error.Forbidden.MustPassStrictCheck.html"},{"type":"function","title":"Ash.Error.Forbidden.MustPassStrictCheck.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.MustPassStrictCheck` without raising it.","ref":"Ash.Error.Forbidden.MustPassStrictCheck.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.MustPassStrictCheck.exception/1","doc":"","ref":"Ash.Error.Forbidden.MustPassStrictCheck.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.Placeholder","doc":"A placeholder exception that the user should never see","ref":"Ash.Error.Forbidden.Placeholder.html"},{"type":"function","title":"Ash.Error.Forbidden.Placeholder.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.Placeholder` without raising it.","ref":"Ash.Error.Forbidden.Placeholder.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.Placeholder.exception/1","doc":"- :authorizer","ref":"Ash.Error.Forbidden.Placeholder.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Forbidden.Policy","doc":"Raised when policy authorization for an action fails","ref":"Ash.Error.Forbidden.Policy.html"},{"type":"function","title":"Ash.Error.Forbidden.Policy.exception/1","doc":"Create an `Elixir.Ash.Error.Forbidden.Policy` without raising it.","ref":"Ash.Error.Forbidden.Policy.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Forbidden.Policy.exception/1","doc":"- :scenarios\n- :facts\n- :filter\n- :actor\n- :policy_breakdown?\n- :must_pass_strict_check?\n- :for_fields\n- :subject\n- :context_description\n- :policies\n- :resource\n- :solver_statement\n- :domain\n- :action\n- :changeset_doesnt_match_filter","ref":"Ash.Error.Forbidden.Policy.html#exception/1-keys"},{"type":"function","title":"Ash.Error.Forbidden.Policy.get_breakdown/4","doc":"Print a report of an authorization failure from authorization information.\n\nOptions:\n\n- `:help_text?`: Defaults to true. Displays help text at the top of the policy breakdown.\n- `:success?`: Defaults to false. Changes the messaging/graphics around to indicate successful policy authorization.\n- `:must_pass_strict_check?`: Defaults to false. Adds a message about this authorization requiring passing strict check.","ref":"Ash.Error.Forbidden.Policy.html#get_breakdown/4"},{"type":"function","title":"Ash.Error.Forbidden.Policy.report/1","doc":"Print a report of an authorization failure from a forbidden error\n\nOptions:\n\n- `:help_text?`: Defaults to true. Displays help text at the top of the policy breakdown.","ref":"Ash.Error.Forbidden.Policy.html#report/1"},{"type":"function","title":"Ash.Error.Forbidden.Policy.report/2","doc":"","ref":"Ash.Error.Forbidden.Policy.html#report/2"},{"type":"exception","title":"Ash.Error.Framework","doc":"Used when an unknown/generic framework error occurs","ref":"Ash.Error.Framework.html"},{"type":"function","title":"Ash.Error.Framework.exception/1","doc":"Create an `Elixir.Ash.Error.Framework` without raising it.","ref":"Ash.Error.Framework.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.exception/1","doc":"- :changeset\n- :query\n- :action_input\n- :errors","ref":"Ash.Error.Framework.html#exception/1-keys"},{"type":"function","title":"Ash.Error.Framework.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Ash.Error.Framework.html#message/1"},{"type":"type","title":"Ash.Error.Framework.t/0","doc":"","ref":"Ash.Error.Framework.html#t:t/0"},{"type":"exception","title":"Ash.Error.Framework.AssumptionFailed","doc":"Used when unreachable code/conditions are reached in the framework","ref":"Ash.Error.Framework.AssumptionFailed.html"},{"type":"function","title":"Ash.Error.Framework.AssumptionFailed.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.AssumptionFailed` without raising it.","ref":"Ash.Error.Framework.AssumptionFailed.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.AssumptionFailed.exception/1","doc":"- :message","ref":"Ash.Error.Framework.AssumptionFailed.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.CanNotBeAtomic","doc":"Used when a change that is only atomic cannot be done atomically","ref":"Ash.Error.Framework.CanNotBeAtomic.html"},{"type":"function","title":"Ash.Error.Framework.CanNotBeAtomic.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.CanNotBeAtomic` without raising it.","ref":"Ash.Error.Framework.CanNotBeAtomic.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.CanNotBeAtomic.exception/1","doc":"- :resource\n- :change\n- :reason","ref":"Ash.Error.Framework.CanNotBeAtomic.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.FlagAssertionFailed","doc":"Used when unreachable code/conditions are reached in the framework","ref":"Ash.Error.Framework.FlagAssertionFailed.html"},{"type":"function","title":"Ash.Error.Framework.FlagAssertionFailed.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.FlagAssertionFailed` without raising it.","ref":"Ash.Error.Framework.FlagAssertionFailed.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.FlagAssertionFailed.exception/1","doc":"- :flag\n- :heading","ref":"Ash.Error.Framework.FlagAssertionFailed.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.InvalidReturnType","doc":"Used when a callback returns an invalid type","ref":"Ash.Error.Framework.InvalidReturnType.html"},{"type":"function","title":"Ash.Error.Framework.InvalidReturnType.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.InvalidReturnType` without raising it.","ref":"Ash.Error.Framework.InvalidReturnType.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.InvalidReturnType.exception/1","doc":"- :message","ref":"Ash.Error.Framework.InvalidReturnType.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.MustBeAtomic","doc":"Used when an action that must be atomic cannot be done atomically","ref":"Ash.Error.Framework.MustBeAtomic.html"},{"type":"function","title":"Ash.Error.Framework.MustBeAtomic.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.MustBeAtomic` without raising it.","ref":"Ash.Error.Framework.MustBeAtomic.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.MustBeAtomic.exception/1","doc":"- :resource\n- :action\n- :reason","ref":"Ash.Error.Framework.MustBeAtomic.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.PendingCodegen","doc":"Used when an extension has pending code generation and the --check flag is provided","ref":"Ash.Error.Framework.PendingCodegen.html"},{"type":"function","title":"Ash.Error.Framework.PendingCodegen.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.PendingCodegen` without raising it.","ref":"Ash.Error.Framework.PendingCodegen.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.PendingCodegen.exception/1","doc":"- :diff\n- :explain","ref":"Ash.Error.Framework.PendingCodegen.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Framework.SynchronousEngineStuck","doc":"Used when the sycnrhonous engine cannot proceed","ref":"Ash.Error.Framework.SynchronousEngineStuck.html"},{"type":"function","title":"Ash.Error.Framework.SynchronousEngineStuck.exception/1","doc":"Create an `Elixir.Ash.Error.Framework.SynchronousEngineStuck` without raising it.","ref":"Ash.Error.Framework.SynchronousEngineStuck.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Framework.SynchronousEngineStuck.exception/1","doc":"","ref":"Ash.Error.Framework.SynchronousEngineStuck.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid","doc":"The top level invalid error","ref":"Ash.Error.Invalid.html"},{"type":"function","title":"Ash.Error.Invalid.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid` without raising it.","ref":"Ash.Error.Invalid.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.exception/1","doc":"- :changeset\n- :query\n- :action_input\n- :errors","ref":"Ash.Error.Invalid.html#exception/1-keys"},{"type":"function","title":"Ash.Error.Invalid.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Ash.Error.Invalid.html#message/1"},{"type":"type","title":"Ash.Error.Invalid.t/0","doc":"","ref":"Ash.Error.Invalid.html#t:t/0"},{"type":"exception","title":"Ash.Error.Invalid.ActionRequiresPagination","doc":"Used when page option is passed but pagination is not enabled.","ref":"Ash.Error.Invalid.ActionRequiresPagination.html"},{"type":"function","title":"Ash.Error.Invalid.ActionRequiresPagination.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.ActionRequiresPagination` without raising it.","ref":"Ash.Error.Invalid.ActionRequiresPagination.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.ActionRequiresPagination.exception/1","doc":"- :resource\n- :action","ref":"Ash.Error.Invalid.ActionRequiresPagination.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.AtomicsNotSupported","doc":"Used when atomics for the given action type are not not supported by the data layer, but one is used.","ref":"Ash.Error.Invalid.AtomicsNotSupported.html"},{"type":"function","title":"Ash.Error.Invalid.AtomicsNotSupported.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.AtomicsNotSupported` without raising it.","ref":"Ash.Error.Invalid.AtomicsNotSupported.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.AtomicsNotSupported.exception/1","doc":"- :resource\n- :action_type","ref":"Ash.Error.Invalid.AtomicsNotSupported.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.InvalidActionType","doc":"Used when a callback returns an invalid type","ref":"Ash.Error.Invalid.InvalidActionType.html"},{"type":"function","title":"Ash.Error.Invalid.InvalidActionType.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.InvalidActionType` without raising it.","ref":"Ash.Error.Invalid.InvalidActionType.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.InvalidActionType.exception/1","doc":"- :message\n- :type\n- :expectation\n- :resource\n- :action","ref":"Ash.Error.Invalid.InvalidActionType.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.InvalidCustomInput","doc":"Used when an invalid value is provided for a code interface custom input","ref":"Ash.Error.Invalid.InvalidCustomInput.html"},{"type":"function","title":"Ash.Error.Invalid.InvalidCustomInput.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.InvalidCustomInput` without raising it.","ref":"Ash.Error.Invalid.InvalidCustomInput.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.InvalidCustomInput.exception/1","doc":"- :field\n- :message\n- :value","ref":"Ash.Error.Invalid.InvalidCustomInput.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.InvalidPrimaryKey","doc":"Used when an invalid primary key is given to `Ash.get/2`","ref":"Ash.Error.Invalid.InvalidPrimaryKey.html"},{"type":"function","title":"Ash.Error.Invalid.InvalidPrimaryKey.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.InvalidPrimaryKey` without raising it.","ref":"Ash.Error.Invalid.InvalidPrimaryKey.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.InvalidPrimaryKey.exception/1","doc":"- :resource\n- :value","ref":"Ash.Error.Invalid.InvalidPrimaryKey.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.LimitRequired","doc":"Used when no limit is provided, pagination is required, and no default page size is configured","ref":"Ash.Error.Invalid.LimitRequired.html"},{"type":"function","title":"Ash.Error.Invalid.LimitRequired.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.LimitRequired` without raising it.","ref":"Ash.Error.Invalid.LimitRequired.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.LimitRequired.exception/1","doc":"","ref":"Ash.Error.Invalid.LimitRequired.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.MultipleResults","doc":"Used when multiple results are returned in a case where only one result was expected","ref":"Ash.Error.Invalid.MultipleResults.html"},{"type":"function","title":"Ash.Error.Invalid.MultipleResults.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.MultipleResults` without raising it.","ref":"Ash.Error.Invalid.MultipleResults.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.MultipleResults.exception/1","doc":"- :count\n- :at_least?\n- :query","ref":"Ash.Error.Invalid.MultipleResults.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoIdentityFound","doc":"Used when an identity name is used that does not reference identity on the resource","ref":"Ash.Error.Invalid.NoIdentityFound.html"},{"type":"function","title":"Ash.Error.Invalid.NoIdentityFound.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoIdentityFound` without raising it.","ref":"Ash.Error.Invalid.NoIdentityFound.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoIdentityFound.exception/1","doc":"- :resource\n- :identity","ref":"Ash.Error.Invalid.NoIdentityFound.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoMatchingBulkStrategy","doc":"Used when an identity name is used that does not reference identity on the resource","ref":"Ash.Error.Invalid.NoMatchingBulkStrategy.html"},{"type":"function","title":"Ash.Error.Invalid.NoMatchingBulkStrategy.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoMatchingBulkStrategy` without raising it.","ref":"Ash.Error.Invalid.NoMatchingBulkStrategy.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoMatchingBulkStrategy.exception/1","doc":"- :resource\n- :action\n- :requested_strategies\n- :not_stream_reason\n- :not_atomic_batches_reason\n- :not_atomic_reason\n- :footer","ref":"Ash.Error.Invalid.NoMatchingBulkStrategy.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoPrimaryAction","doc":"Used when an action name is provided that doesn't exist","ref":"Ash.Error.Invalid.NoPrimaryAction.html"},{"type":"function","title":"Ash.Error.Invalid.NoPrimaryAction.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoPrimaryAction` without raising it.","ref":"Ash.Error.Invalid.NoPrimaryAction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoPrimaryAction.exception/1","doc":"- :resource\n- :type","ref":"Ash.Error.Invalid.NoPrimaryAction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoSuchAction","doc":"Used when an action name is provided that doesn't exist","ref":"Ash.Error.Invalid.NoSuchAction.html"},{"type":"function","title":"Ash.Error.Invalid.NoSuchAction.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoSuchAction` without raising it.","ref":"Ash.Error.Invalid.NoSuchAction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoSuchAction.exception/1","doc":"- :resource\n- :action\n- :type","ref":"Ash.Error.Invalid.NoSuchAction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoSuchInput","doc":"Used when an input is provided to an action or calculation that is not accepted","ref":"Ash.Error.Invalid.NoSuchInput.html"},{"type":"function","title":"Ash.Error.Invalid.NoSuchInput.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoSuchInput` without raising it.","ref":"Ash.Error.Invalid.NoSuchInput.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoSuchInput.exception/1","doc":"- :calculation\n- :resource\n- :action\n- :input\n- :inputs\n- :did_you_mean","ref":"Ash.Error.Invalid.NoSuchInput.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NoSuchResource","doc":"Used when a resource or alias is provided that doesn't exist","ref":"Ash.Error.Invalid.NoSuchResource.html"},{"type":"function","title":"Ash.Error.Invalid.NoSuchResource.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NoSuchResource` without raising it.","ref":"Ash.Error.Invalid.NoSuchResource.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NoSuchResource.exception/1","doc":"- :resource\n- :message","ref":"Ash.Error.Invalid.NoSuchResource.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NonCountableAction","doc":"Used when page[:count] option is passed but the action's pagination is not countable.","ref":"Ash.Error.Invalid.NonCountableAction.html"},{"type":"function","title":"Ash.Error.Invalid.NonCountableAction.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NonCountableAction` without raising it.","ref":"Ash.Error.Invalid.NonCountableAction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NonCountableAction.exception/1","doc":"- :resource\n- :action","ref":"Ash.Error.Invalid.NonCountableAction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.NonStreamableAction","doc":"Used when Ash.stream is used with an action that does not support keyset pagination","ref":"Ash.Error.Invalid.NonStreamableAction.html"},{"type":"function","title":"Ash.Error.Invalid.NonStreamableAction.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.NonStreamableAction` without raising it.","ref":"Ash.Error.Invalid.NonStreamableAction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.NonStreamableAction.exception/1","doc":"- :resource\n- :action\n- :for_bulk_update\n- :for_bulk_destroy\n- :types","ref":"Ash.Error.Invalid.NonStreamableAction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.PaginationRequired","doc":"Used when `page: false` is provided but pagination is required","ref":"Ash.Error.Invalid.PaginationRequired.html"},{"type":"function","title":"Ash.Error.Invalid.PaginationRequired.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.PaginationRequired` without raising it.","ref":"Ash.Error.Invalid.PaginationRequired.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.PaginationRequired.exception/1","doc":"","ref":"Ash.Error.Invalid.PaginationRequired.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.ResourceNotAllowed","doc":"Used when a resource or alias is provided that cannot be used with the given domain","ref":"Ash.Error.Invalid.ResourceNotAllowed.html"},{"type":"function","title":"Ash.Error.Invalid.ResourceNotAllowed.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.ResourceNotAllowed` without raising it.","ref":"Ash.Error.Invalid.ResourceNotAllowed.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.ResourceNotAllowed.exception/1","doc":"- :resource\n- :domain","ref":"Ash.Error.Invalid.ResourceNotAllowed.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.TenantRequired","doc":"Used when a tenant is not specified","ref":"Ash.Error.Invalid.TenantRequired.html"},{"type":"function","title":"Ash.Error.Invalid.TenantRequired.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.TenantRequired` without raising it.","ref":"Ash.Error.Invalid.TenantRequired.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.TenantRequired.exception/1","doc":"- :resource","ref":"Ash.Error.Invalid.TenantRequired.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.Timeout","doc":"Used when a request to a domain times out.","ref":"Ash.Error.Invalid.Timeout.html"},{"type":"function","title":"Ash.Error.Invalid.Timeout.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.Timeout` without raising it.","ref":"Ash.Error.Invalid.Timeout.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.Timeout.exception/1","doc":"- :name\n- :timeout","ref":"Ash.Error.Invalid.Timeout.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.TimeoutNotSupported","doc":"Used when timeouts are not supported by the data layer, but one is set","ref":"Ash.Error.Invalid.TimeoutNotSupported.html"},{"type":"function","title":"Ash.Error.Invalid.TimeoutNotSupported.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.TimeoutNotSupported` without raising it.","ref":"Ash.Error.Invalid.TimeoutNotSupported.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.TimeoutNotSupported.exception/1","doc":"- :resource","ref":"Ash.Error.Invalid.TimeoutNotSupported.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Invalid.Unavailable","doc":"Used when a given resource is unavailable.\n\nThis might happen due to locking at the data layer, or something\nyou implement yourself.","ref":"Ash.Error.Invalid.Unavailable.html"},{"type":"function","title":"Ash.Error.Invalid.Unavailable.exception/1","doc":"Create an `Elixir.Ash.Error.Invalid.Unavailable` without raising it.","ref":"Ash.Error.Invalid.Unavailable.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Invalid.Unavailable.exception/1","doc":"- :resource\n- :source\n- :reason","ref":"Ash.Error.Invalid.Unavailable.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Load.InvalidQuery","doc":"Used when an invalid query is provided in a load","ref":"Ash.Error.Load.InvalidQuery.html"},{"type":"function","title":"Ash.Error.Load.InvalidQuery.exception/1","doc":"Create an `Elixir.Ash.Error.Load.InvalidQuery` without raising it.","ref":"Ash.Error.Load.InvalidQuery.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Load.InvalidQuery.exception/1","doc":"- :resource\n- :relationship\n- :query\n- :load_path","ref":"Ash.Error.Load.InvalidQuery.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Load.NoSuchRelationship","doc":"Used when attempting to load a relationship that does not exist","ref":"Ash.Error.Load.NoSuchRelationship.html"},{"type":"function","title":"Ash.Error.Load.NoSuchRelationship.exception/1","doc":"Create an `Elixir.Ash.Error.Load.NoSuchRelationship` without raising it.","ref":"Ash.Error.Load.NoSuchRelationship.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Load.NoSuchRelationship.exception/1","doc":"- :resource\n- :relationship\n- :load_path","ref":"Ash.Error.Load.NoSuchRelationship.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Page.InvalidKeyset","doc":"Used when a value is provided for a keyset that cannot be Base64 decoded.","ref":"Ash.Error.Page.InvalidKeyset.html"},{"type":"function","title":"Ash.Error.Page.InvalidKeyset.exception/1","doc":"Create an `Elixir.Ash.Error.Page.InvalidKeyset` without raising it.","ref":"Ash.Error.Page.InvalidKeyset.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Page.InvalidKeyset.exception/1","doc":"- :value\n- :key","ref":"Ash.Error.Page.InvalidKeyset.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.AggregatesNotSupported","doc":"Used when the data_layer does not support aggregates, or filtering/sorting them","ref":"Ash.Error.Query.AggregatesNotSupported.html"},{"type":"function","title":"Ash.Error.Query.AggregatesNotSupported.exception/1","doc":"Create an `Elixir.Ash.Error.Query.AggregatesNotSupported` without raising it.","ref":"Ash.Error.Query.AggregatesNotSupported.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.AggregatesNotSupported.exception/1","doc":"- :resource\n- :feature\n- :type","ref":"Ash.Error.Query.AggregatesNotSupported.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.CalculationRequiresPrimaryKey","doc":"Used when a calculation requires a primary key but was not supplied with one","ref":"Ash.Error.Query.CalculationRequiresPrimaryKey.html"},{"type":"function","title":"Ash.Error.Query.CalculationRequiresPrimaryKey.exception/1","doc":"Create an `Elixir.Ash.Error.Query.CalculationRequiresPrimaryKey` without raising it.","ref":"Ash.Error.Query.CalculationRequiresPrimaryKey.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.CalculationRequiresPrimaryKey.exception/1","doc":"- :resource\n- :calculation","ref":"Ash.Error.Query.CalculationRequiresPrimaryKey.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.CalculationsNotSupported","doc":"Used when the data_layer does not support calculations, or filtering/sorting them","ref":"Ash.Error.Query.CalculationsNotSupported.html"},{"type":"function","title":"Ash.Error.Query.CalculationsNotSupported.exception/1","doc":"Create an `Elixir.Ash.Error.Query.CalculationsNotSupported` without raising it.","ref":"Ash.Error.Query.CalculationsNotSupported.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.CalculationsNotSupported.exception/1","doc":"- :resource\n- :feature","ref":"Ash.Error.Query.CalculationsNotSupported.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidArgument","doc":"Used when an invalid value is provided for an action argument","ref":"Ash.Error.Query.InvalidArgument.html"},{"type":"function","title":"Ash.Error.Query.InvalidArgument.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidArgument` without raising it.","ref":"Ash.Error.Query.InvalidArgument.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidArgument.exception/1","doc":"- :field\n- :message\n- :value","ref":"Ash.Error.Query.InvalidArgument.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidCalculationArgument","doc":"Used when an invalid value is provided for a calculation argument","ref":"Ash.Error.Query.InvalidCalculationArgument.html"},{"type":"function","title":"Ash.Error.Query.InvalidCalculationArgument.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidCalculationArgument` without raising it.","ref":"Ash.Error.Query.InvalidCalculationArgument.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidCalculationArgument.exception/1","doc":"- :calculation\n- :field\n- :message\n- :value","ref":"Ash.Error.Query.InvalidCalculationArgument.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidExpression","doc":"Used when an invalid expression is used in a filter","ref":"Ash.Error.Query.InvalidExpression.html"},{"type":"function","title":"Ash.Error.Query.InvalidExpression.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidExpression` without raising it.","ref":"Ash.Error.Query.InvalidExpression.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidExpression.exception/1","doc":"- :expression\n- :message","ref":"Ash.Error.Query.InvalidExpression.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidFilterReference","doc":"Used when an invalid reference is used in a filter","ref":"Ash.Error.Query.InvalidFilterReference.html"},{"type":"function","title":"Ash.Error.Query.InvalidFilterReference.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidFilterReference` without raising it.","ref":"Ash.Error.Query.InvalidFilterReference.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidFilterReference.exception/1","doc":"- :field\n- :simple_equality?","ref":"Ash.Error.Query.InvalidFilterReference.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidFilterValue","doc":"Used when an invalid value is provided for a filter","ref":"Ash.Error.Query.InvalidFilterValue.html"},{"type":"function","title":"Ash.Error.Query.InvalidFilterValue.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidFilterValue` without raising it.","ref":"Ash.Error.Query.InvalidFilterValue.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidFilterValue.exception/1","doc":"- :message\n- :value\n- :context","ref":"Ash.Error.Query.InvalidFilterValue.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidLimit","doc":"Used when an invalid limit is provided","ref":"Ash.Error.Query.InvalidLimit.html"},{"type":"function","title":"Ash.Error.Query.InvalidLimit.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidLimit` without raising it.","ref":"Ash.Error.Query.InvalidLimit.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidLimit.exception/1","doc":"- :limit","ref":"Ash.Error.Query.InvalidLimit.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidLoad","doc":"Used when an invalid load is provided","ref":"Ash.Error.Query.InvalidLoad.html"},{"type":"function","title":"Ash.Error.Query.InvalidLoad.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidLoad` without raising it.","ref":"Ash.Error.Query.InvalidLoad.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidLoad.exception/1","doc":"- :load","ref":"Ash.Error.Query.InvalidLoad.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidOffset","doc":"Used when an invalid offset is provided","ref":"Ash.Error.Query.InvalidOffset.html"},{"type":"function","title":"Ash.Error.Query.InvalidOffset.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidOffset` without raising it.","ref":"Ash.Error.Query.InvalidOffset.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidOffset.exception/1","doc":"- :offset","ref":"Ash.Error.Query.InvalidOffset.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidPage","doc":"Used when an invalid page option is provided","ref":"Ash.Error.Query.InvalidPage.html"},{"type":"function","title":"Ash.Error.Query.InvalidPage.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidPage` without raising it.","ref":"Ash.Error.Query.InvalidPage.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidPage.exception/1","doc":"- :page","ref":"Ash.Error.Query.InvalidPage.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidQuery","doc":"A generic error that can be used to add an error to a query for a specific field","ref":"Ash.Error.Query.InvalidQuery.html"},{"type":"function","title":"Ash.Error.Query.InvalidQuery.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidQuery` without raising it.","ref":"Ash.Error.Query.InvalidQuery.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidQuery.exception/1","doc":"- :field\n- :fields\n- :message\n- :value","ref":"Ash.Error.Query.InvalidQuery.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.InvalidSortOrder","doc":"Used when an invalid sort order is provided","ref":"Ash.Error.Query.InvalidSortOrder.html"},{"type":"function","title":"Ash.Error.Query.InvalidSortOrder.exception/1","doc":"Create an `Elixir.Ash.Error.Query.InvalidSortOrder` without raising it.","ref":"Ash.Error.Query.InvalidSortOrder.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.InvalidSortOrder.exception/1","doc":"- :order","ref":"Ash.Error.Query.InvalidSortOrder.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.LockNotSupported","doc":"Used when the data_layer does not support a given lock type","ref":"Ash.Error.Query.LockNotSupported.html"},{"type":"function","title":"Ash.Error.Query.LockNotSupported.exception/1","doc":"Create an `Elixir.Ash.Error.Query.LockNotSupported` without raising it.","ref":"Ash.Error.Query.LockNotSupported.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.LockNotSupported.exception/1","doc":"- :resource\n- :lock_type","ref":"Ash.Error.Query.LockNotSupported.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoComplexSortsWithKeysetPagination","doc":"Due to the filter-based implementation of keyset pagination, it cannot be used with sorts on calculations.\n\nWe could solve this problem by making the keyset only be the primary key of the record,\nand then fetching that value loading the calculations/aggregates that we need. If we do this\nwe should either: 1.) make it a new pagination mode or 2.) add an option like `mode: :strict | :fetch`\nto pagination options.\n\nLet me know if you're reading this and want to help implement it.","ref":"Ash.Error.Query.NoComplexSortsWithKeysetPagination.html"},{"type":"function","title":"Ash.Error.Query.NoComplexSortsWithKeysetPagination.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoComplexSortsWithKeysetPagination` without raising it.","ref":"Ash.Error.Query.NoComplexSortsWithKeysetPagination.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoComplexSortsWithKeysetPagination.exception/1","doc":"- :resource\n- :sort","ref":"Ash.Error.Query.NoComplexSortsWithKeysetPagination.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoReadAction","doc":"Used when a resource would be read but has no read action","ref":"Ash.Error.Query.NoReadAction.html"},{"type":"function","title":"Ash.Error.Query.NoReadAction.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoReadAction` without raising it.","ref":"Ash.Error.Query.NoReadAction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoReadAction.exception/1","doc":"- :resource\n- :when","ref":"Ash.Error.Query.NoReadAction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchAttribute","doc":"Used when an attribute that doesn't exist is used in a query","ref":"Ash.Error.Query.NoSuchAttribute.html"},{"type":"function","title":"Ash.Error.Query.NoSuchAttribute.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchAttribute` without raising it.","ref":"Ash.Error.Query.NoSuchAttribute.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchAttribute.exception/1","doc":"- :resource\n- :attribute","ref":"Ash.Error.Query.NoSuchAttribute.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchField","doc":"Used when a field(attribute, calculation, aggregate or relationship) that doesn't exist is used in a query","ref":"Ash.Error.Query.NoSuchField.html"},{"type":"function","title":"Ash.Error.Query.NoSuchField.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchField` without raising it.","ref":"Ash.Error.Query.NoSuchField.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchField.exception/1","doc":"- :resource\n- :field","ref":"Ash.Error.Query.NoSuchField.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchFilterPredicate","doc":"Used when a filter predicate that does not exist is referenced","ref":"Ash.Error.Query.NoSuchFilterPredicate.html"},{"type":"function","title":"Ash.Error.Query.NoSuchFilterPredicate.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchFilterPredicate` without raising it.","ref":"Ash.Error.Query.NoSuchFilterPredicate.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchFilterPredicate.exception/1","doc":"- :message\n- :value\n- :key\n- :resource","ref":"Ash.Error.Query.NoSuchFilterPredicate.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchFunction","doc":"Used when an function that doesn't exist is used in a query","ref":"Ash.Error.Query.NoSuchFunction.html"},{"type":"function","title":"Ash.Error.Query.NoSuchFunction.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchFunction` without raising it.","ref":"Ash.Error.Query.NoSuchFunction.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchFunction.exception/1","doc":"- :function\n- :arity\n- :resource\n- :could_be_calculation?","ref":"Ash.Error.Query.NoSuchFunction.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchOperator","doc":"Used when an operator that doesn't exist is used in a query","ref":"Ash.Error.Query.NoSuchOperator.html"},{"type":"function","title":"Ash.Error.Query.NoSuchOperator.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchOperator` without raising it.","ref":"Ash.Error.Query.NoSuchOperator.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchOperator.exception/1","doc":"- :operator","ref":"Ash.Error.Query.NoSuchOperator.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NoSuchRelationship","doc":"Used when an relationship that doesn't exist is used in a query","ref":"Ash.Error.Query.NoSuchRelationship.html"},{"type":"function","title":"Ash.Error.Query.NoSuchRelationship.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NoSuchRelationship` without raising it.","ref":"Ash.Error.Query.NoSuchRelationship.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NoSuchRelationship.exception/1","doc":"- :resource\n- :relationship","ref":"Ash.Error.Query.NoSuchRelationship.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.NotFound","doc":"Used when an entity that not exist is referenced","ref":"Ash.Error.Query.NotFound.html"},{"type":"function","title":"Ash.Error.Query.NotFound.exception/1","doc":"Create an `Elixir.Ash.Error.Query.NotFound` without raising it.","ref":"Ash.Error.Query.NotFound.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.NotFound.exception/1","doc":"- :primary_key\n- :resource","ref":"Ash.Error.Query.NotFound.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.ReadActionRequired","doc":"Used when a relationship is filtered and the destination does not have a default read action","ref":"Ash.Error.Query.ReadActionRequired.html"},{"type":"function","title":"Ash.Error.Query.ReadActionRequired.exception/1","doc":"Create an `Elixir.Ash.Error.Query.ReadActionRequired` without raising it.","ref":"Ash.Error.Query.ReadActionRequired.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.ReadActionRequired.exception/1","doc":"- :resource","ref":"Ash.Error.Query.ReadActionRequired.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.ReadActionRequiresActor","doc":"Used when an actor is referenced in a filter template, but no actor exists","ref":"Ash.Error.Query.ReadActionRequiresActor.html"},{"type":"function","title":"Ash.Error.Query.ReadActionRequiresActor.exception/1","doc":"Create an `Elixir.Ash.Error.Query.ReadActionRequiresActor` without raising it.","ref":"Ash.Error.Query.ReadActionRequiresActor.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.ReadActionRequiresActor.exception/1","doc":"","ref":"Ash.Error.Query.ReadActionRequiresActor.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.Required","doc":"Used when a filter or argument is required in a query","ref":"Ash.Error.Query.Required.html"},{"type":"function","title":"Ash.Error.Query.Required.exception/1","doc":"Create an `Elixir.Ash.Error.Query.Required` without raising it.","ref":"Ash.Error.Query.Required.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.Required.exception/1","doc":"- :field\n- :type\n- :resource","ref":"Ash.Error.Query.Required.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.UnsortableField","doc":"Used when attempting to sort on a field that cannot be used for sorting","ref":"Ash.Error.Query.UnsortableField.html"},{"type":"function","title":"Ash.Error.Query.UnsortableField.exception/1","doc":"Create an `Elixir.Ash.Error.Query.UnsortableField` without raising it.","ref":"Ash.Error.Query.UnsortableField.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.UnsortableField.exception/1","doc":"- :resource\n- :field\n- :reason","ref":"Ash.Error.Query.UnsortableField.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.Query.UnsupportedPredicate","doc":"Used when the data_layer does not support a provided predicate","ref":"Ash.Error.Query.UnsupportedPredicate.html"},{"type":"function","title":"Ash.Error.Query.UnsupportedPredicate.exception/1","doc":"Create an `Elixir.Ash.Error.Query.UnsupportedPredicate` without raising it.","ref":"Ash.Error.Query.UnsupportedPredicate.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Query.UnsupportedPredicate.exception/1","doc":"- :resource\n- :predicate\n- :type","ref":"Ash.Error.Query.UnsupportedPredicate.html#exception/1-keys"},{"type":"exception","title":"Ash.Error.SimpleDataLayer.NoDataProvided","doc":"Used when no data was provided to the simple data layer","ref":"Ash.Error.SimpleDataLayer.NoDataProvided.html"},{"type":"function","title":"Ash.Error.SimpleDataLayer.NoDataProvided.exception/1","doc":"Create an `Elixir.Ash.Error.SimpleDataLayer.NoDataProvided` without raising it.","ref":"Ash.Error.SimpleDataLayer.NoDataProvided.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.SimpleDataLayer.NoDataProvided.exception/1","doc":"- :resource\n- :message","ref":"Ash.Error.SimpleDataLayer.NoDataProvided.html#exception/1-keys"},{"type":"module","title":"Ash.Error.Stacktrace","doc":"A placeholder for a stacktrace so that we can avoid printing it everywhere","ref":"Ash.Error.Stacktrace.html"},{"type":"type","title":"Ash.Error.Stacktrace.t/0","doc":"","ref":"Ash.Error.Stacktrace.html#t:t/0"},{"type":"exception","title":"Ash.Error.Unknown","doc":"The top level unknown error container","ref":"Ash.Error.Unknown.html"},{"type":"function","title":"Ash.Error.Unknown.exception/1","doc":"Construction an exception using the arguments passed in. You can see\nElixir's doc on `Exception/1` for more information.","ref":"Ash.Error.Unknown.html#exception/1"},{"type":"function","title":"Ash.Error.Unknown.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Ash.Error.Unknown.html#message/1"},{"type":"type","title":"Ash.Error.Unknown.t/0","doc":"","ref":"Ash.Error.Unknown.html#t:t/0"},{"type":"exception","title":"Ash.Error.Unknown.UnknownError","doc":"Used when an unknown error occurs","ref":"Ash.Error.Unknown.UnknownError.html"},{"type":"function","title":"Ash.Error.Unknown.UnknownError.exception/1","doc":"Create an `Elixir.Ash.Error.Unknown.UnknownError` without raising it.","ref":"Ash.Error.Unknown.UnknownError.html#exception/1"},{"type":"function","title":"Keys - Ash.Error.Unknown.UnknownError.exception/1","doc":"- :error\n- :field\n- :value","ref":"Ash.Error.Unknown.UnknownError.html#exception/1-keys"},{"type":"type","title":"Ash.Error.Unknown.UnknownError.t/0","doc":"","ref":"Ash.Error.Unknown.UnknownError.html#t:t/0"},{"type":"module","title":"Ash.DataLayer.Verifiers.RequirePreCheckWith","doc":"Ensures that all identities have a `pre_check_with` configured, or raises.","ref":"Ash.DataLayer.Verifiers.RequirePreCheckWith.html"},{"type":"module","title":"Ash.Notifier.PubSub.Verifiers.VerifyActionNames","doc":"Verifies action names in configured publications","ref":"Ash.Notifier.PubSub.Verifiers.VerifyActionNames.html"},{"type":"function","title":"Ash.Notifier.PubSub.Verifiers.VerifyActionNames.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Notifier.PubSub.Verifiers.VerifyActionNames.html#verify/1"},{"type":"module","title":"Ash.Domain.Verifiers.EnsureNoEmbeds","doc":"Ensures that all resources for a given domain are not embeds.","ref":"Ash.Domain.Verifiers.EnsureNoEmbeds.html"},{"type":"module","title":"Ash.Domain.Verifiers.ValidateArgumentsToCodeInterface","doc":"Validate the arguments defined in the code interface\nand reject arguments that are not action attributes/arguments","ref":"Ash.Domain.Verifiers.ValidateArgumentsToCodeInterface.html"},{"type":"module","title":"Ash.Domain.Verifiers.ValidateRelatedResourceInclusion","doc":"Ensures that all related resources are included in a domain.","ref":"Ash.Domain.Verifiers.ValidateRelatedResourceInclusion.html"},{"type":"module","title":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies","doc":"Adds field policies for any missing fields","ref":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.html"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.html#after?/1"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.html#after_compile?/0"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.html#before?/1"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Policy.Authorizer.Transformers.AddMissingFieldPolicies.html#transform/1"},{"type":"module","title":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies","doc":"Cache field policies for each field","ref":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.html"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.html#after?/1"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.html#after_compile?/0"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.html#before?/1"},{"type":"function","title":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Policy.Authorizer.Transformers.CacheFieldPolicies.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.AttributesByName","doc":"Persists attribute_names and attributes_by_name.","ref":"Ash.Resource.Transformers.AttributesByName.html"},{"type":"function","title":"Ash.Resource.Transformers.AttributesByName.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.AttributesByName.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.AttributesByName.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.AttributesByName.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.AttributesByName.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.AttributesByName.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.AttributesByName.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.AttributesByName.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.BelongsToAttribute","doc":"Creates the attribute for belongs_to relationships that have `define_attribute?: true`","ref":"Ash.Resource.Transformers.BelongsToAttribute.html"},{"type":"function","title":"Ash.Resource.Transformers.BelongsToAttribute.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.BelongsToAttribute.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.BelongsToAttribute.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.BelongsToAttribute.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.BelongsToAttribute.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.BelongsToAttribute.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.BelongsToAttribute.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.BelongsToAttribute.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CacheActionInputs","doc":"Stores the set of valid input keys for each action","ref":"Ash.Resource.Transformers.CacheActionInputs.html"},{"type":"function","title":"Ash.Resource.Transformers.CacheActionInputs.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CacheActionInputs.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheActionInputs.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CacheActionInputs.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CacheActionInputs.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CacheActionInputs.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheActionInputs.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CacheActionInputs.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CacheCalculations","doc":"Persists commonly used calculation information.","ref":"Ash.Resource.Transformers.CacheCalculations.html"},{"type":"function","title":"Ash.Resource.Transformers.CacheCalculations.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CacheCalculations.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheCalculations.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CacheCalculations.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CacheCalculations.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CacheCalculations.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheCalculations.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CacheCalculations.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CachePrimaryKey","doc":"Validates and caches the primary key of a resource","ref":"Ash.Resource.Transformers.CachePrimaryKey.html"},{"type":"function","title":"Ash.Resource.Transformers.CachePrimaryKey.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CachePrimaryKey.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CachePrimaryKey.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CachePrimaryKey.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CachePrimaryKey.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CachePrimaryKey.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CachePrimaryKey.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CachePrimaryKey.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CacheRelationships","doc":"Persists commonly used relationship information.","ref":"Ash.Resource.Transformers.CacheRelationships.html"},{"type":"function","title":"Ash.Resource.Transformers.CacheRelationships.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CacheRelationships.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheRelationships.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CacheRelationships.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CacheRelationships.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CacheRelationships.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheRelationships.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CacheRelationships.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CacheUniqueKeys","doc":"Stores the set of unique keys for a resource","ref":"Ash.Resource.Transformers.CacheUniqueKeys.html"},{"type":"function","title":"Ash.Resource.Transformers.CacheUniqueKeys.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CacheUniqueKeys.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheUniqueKeys.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CacheUniqueKeys.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CacheUniqueKeys.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CacheUniqueKeys.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CacheUniqueKeys.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CacheUniqueKeys.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.CreateJoinRelationship","doc":"Creates an automatically named `has_many` relationship for each many_to_many.","ref":"Ash.Resource.Transformers.CreateJoinRelationship.html"},{"type":"function","title":"Ash.Resource.Transformers.CreateJoinRelationship.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.CreateJoinRelationship.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.CreateJoinRelationship.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.CreateJoinRelationship.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.CreateJoinRelationship.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.CreateJoinRelationship.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.CreateJoinRelationship.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.CreateJoinRelationship.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.DefaultAccept","doc":"Sets the default `accept` for each action","ref":"Ash.Resource.Transformers.DefaultAccept.html"},{"type":"function","title":"Ash.Resource.Transformers.DefaultAccept.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.DefaultAccept.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.DefaultAccept.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.DefaultAccept.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.DefaultAccept.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.DefaultAccept.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.DefaultAccept.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.DefaultAccept.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.GetByReadActions","doc":"Transform any read actions which contain a `get_by` option.","ref":"Ash.Resource.Transformers.GetByReadActions.html"},{"type":"function","title":"Ash.Resource.Transformers.GetByReadActions.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.GetByReadActions.html#after_compile?/0"},{"type":"module","title":"Ash.Resource.Transformers.HasDestinationField","doc":"Guesses the `destination_attribute` for has many and has one relationships unless provided","ref":"Ash.Resource.Transformers.HasDestinationField.html"},{"type":"function","title":"Ash.Resource.Transformers.HasDestinationField.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.HasDestinationField.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.HasDestinationField.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.HasDestinationField.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.HasDestinationField.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.HasDestinationField.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.HasDestinationField.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.HasDestinationField.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource","doc":"Guesses the `destination_attribute_on_join_resource` for many to many relationships unless provided.","ref":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.html"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.ManyToManyDestinationAttributeOnJoinResource.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource","doc":"Guesses the `source_attribute_on_join_resource` for many to many relationships unless provided.","ref":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.html"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.ManyToManySourceAttributeOnJoinResource.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.RequireUniqueActionNames","doc":"Ensures that all actions have unique names.","ref":"Ash.Resource.Transformers.RequireUniqueActionNames.html"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueActionNames.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.RequireUniqueActionNames.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueActionNames.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.RequireUniqueActionNames.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueActionNames.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.RequireUniqueActionNames.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueActionNames.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.RequireUniqueActionNames.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.RequireUniqueFieldNames","doc":"Confirms that a resource does not have multiple fields(attributes, calculations, aggregates, and relationships) with the same name.","ref":"Ash.Resource.Transformers.RequireUniqueFieldNames.html"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueFieldNames.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.RequireUniqueFieldNames.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueFieldNames.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.RequireUniqueFieldNames.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueFieldNames.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.RequireUniqueFieldNames.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.RequireUniqueFieldNames.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.RequireUniqueFieldNames.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.SetPrimaryActions","doc":"Validates the primary action configuration\n\nIf multiple primary actions exist this results in an error.","ref":"Ash.Resource.Transformers.SetPrimaryActions.html"},{"type":"function","title":"Ash.Resource.Transformers.SetPrimaryActions.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.SetPrimaryActions.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.SetPrimaryActions.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.SetPrimaryActions.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.SetPrimaryActions.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.SetPrimaryActions.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.SetPrimaryActions.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.SetPrimaryActions.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.SetRelationshipSource","doc":"Sets the `source` key on relationships to be the resource they were defined on","ref":"Ash.Resource.Transformers.SetRelationshipSource.html"},{"type":"function","title":"Ash.Resource.Transformers.SetRelationshipSource.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.SetRelationshipSource.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.SetRelationshipSource.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.SetRelationshipSource.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.SetRelationshipSource.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.SetRelationshipSource.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.SetRelationshipSource.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.SetRelationshipSource.html#transform/1"},{"type":"module","title":"Ash.Resource.Transformers.ValidationsAndChangesForType","doc":"Persists global changes/validations and what type they go on.","ref":"Ash.Resource.Transformers.ValidationsAndChangesForType.html"},{"type":"function","title":"Ash.Resource.Transformers.ValidationsAndChangesForType.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Resource.Transformers.ValidationsAndChangesForType.html#after?/1"},{"type":"function","title":"Ash.Resource.Transformers.ValidationsAndChangesForType.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Resource.Transformers.ValidationsAndChangesForType.html#after_compile?/0"},{"type":"function","title":"Ash.Resource.Transformers.ValidationsAndChangesForType.before?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.before?/1 .","ref":"Ash.Resource.Transformers.ValidationsAndChangesForType.html#before?/1"},{"type":"function","title":"Ash.Resource.Transformers.ValidationsAndChangesForType.transform/1","doc":"Callback implementation for  Spark.Dsl.Transformer.transform/1 .","ref":"Ash.Resource.Transformers.ValidationsAndChangesForType.html#transform/1"},{"type":"module","title":"Ash.Resource.Verifiers.EnsureAggregateFieldIsAttributeOrCalculation","doc":"Ensures that the field at the end of the path is an attribute or calculation.","ref":"Ash.Resource.Verifiers.EnsureAggregateFieldIsAttributeOrCalculation.html"},{"type":"function","title":"Ash.Resource.Verifiers.EnsureAggregateFieldIsAttributeOrCalculation.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.EnsureAggregateFieldIsAttributeOrCalculation.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.NoReservedFieldNames","doc":"Confirms that a resource does not use reserved names for field names.\n\nReserved field names are: [:__struct__, :__meta__, :__metadata__, :__order__, :__lateral_join_source__, :*, :calculations, :aggregates, :relationships, :as].","ref":"Ash.Resource.Verifiers.NoReservedFieldNames.html"},{"type":"function","title":"Ash.Resource.Verifiers.NoReservedFieldNames.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.NoReservedFieldNames.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateAccept","doc":"Validates that accept and reject lists only contain valid attributes","ref":"Ash.Resource.Verifiers.ValidateAccept.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateActionTypesSupported","doc":"Confirms that all action types declared on a resource are supported by its data layer","ref":"Ash.Resource.Verifiers.ValidateActionTypesSupported.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateAggregatesSupported","doc":"Confirms that all aggregates are supported by the data layer","ref":"Ash.Resource.Verifiers.ValidateAggregatesSupported.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateArgumentsToCodeInterface","doc":"Validate the arguments defined in the code interface\nand reject arguments that are not action attributes/arguments","ref":"Ash.Resource.Verifiers.ValidateArgumentsToCodeInterface.html"},{"type":"function","title":"Ash.Resource.Verifiers.ValidateArgumentsToCodeInterface.verify_interface!/4","doc":"","ref":"Ash.Resource.Verifiers.ValidateArgumentsToCodeInterface.html#verify_interface!/4"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateEagerIdentities","doc":"Confirms that eager identities are not declared on a resource with no primary read.","ref":"Ash.Resource.Verifiers.ValidateEagerIdentities.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateManagedRelationshipOpts","doc":"Confirms that all action types declared on a resource are supported by its data layer","ref":"Ash.Resource.Verifiers.ValidateManagedRelationshipOpts.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateMultitenancy","doc":"Ensures that the multitenancy configuration is valid for the given resource","ref":"Ash.Resource.Verifiers.ValidateMultitenancy.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidatePrimaryKey","doc":"Validates and caches the primary key of a resource","ref":"Ash.Resource.Verifiers.ValidatePrimaryKey.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateRelationshipAttributes","doc":"Validates that all relationships point to valid fields","ref":"Ash.Resource.Verifiers.ValidateRelationshipAttributes.html"},{"type":"module","title":"Ash.Resource.Verifiers.ValidateRelationshipAttributesMatch","doc":"Shows a warning on potentially incompatible relationship attributes.","ref":"Ash.Resource.Verifiers.ValidateRelationshipAttributesMatch.html"},{"type":"function","title":"Ash.Resource.Verifiers.ValidateRelationshipAttributesMatch.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.ValidateRelationshipAttributesMatch.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyActionsAtomic","doc":"Raises an error on update or destroy actions with `require_atomic?` set to\ntrue when it is known at compile time that they will not be atomic.","ref":"Ash.Resource.Verifiers.VerifyActionsAtomic.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyActionsAtomic.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyActionsAtomic.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyGenericActionReactorInputs","doc":"Returns an error if a generic action calls a Reactor module without specifying\nan argument for all expected inputs.","ref":"Ash.Resource.Verifiers.VerifyGenericActionReactorInputs.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyGenericActionReactorInputs.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyGenericActionReactorInputs.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyIdentityFields","doc":"Raises an error on potentially incompatible identity attributes.","ref":"Ash.Resource.Verifiers.VerifyIdentityFields.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyIdentityFields.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyIdentityFields.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyPrimaryKeyPresent","doc":"Raises an error when a required primary key is missing","ref":"Ash.Resource.Verifiers.VerifyPrimaryKeyPresent.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyPrimaryKeyPresent.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyPrimaryKeyPresent.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyPrimaryReadActionHasNoArguments","doc":"Verifies that primary read actions do not have any arguments","ref":"Ash.Resource.Verifiers.VerifyPrimaryReadActionHasNoArguments.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyPrimaryReadActionHasNoArguments.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyPrimaryReadActionHasNoArguments.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifyReservedCalculationArguments","doc":"Verifies that standard context keys are not used as calculation arguments","ref":"Ash.Resource.Verifiers.VerifyReservedCalculationArguments.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifyReservedCalculationArguments.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifyReservedCalculationArguments.html#verify/1"},{"type":"module","title":"Ash.Resource.Verifiers.VerifySelectedByDefault","doc":"Raises an error when a required primary key is missing","ref":"Ash.Resource.Verifiers.VerifySelectedByDefault.html"},{"type":"function","title":"Ash.Resource.Verifiers.VerifySelectedByDefault.verify/1","doc":"Callback implementation for  Spark.Dsl.Verifier.verify/1 .","ref":"Ash.Resource.Verifiers.VerifySelectedByDefault.html#verify/1"},{"type":"behaviour","title":"Ash.Filter.Predicate","doc":"Represents a predicate which can be simplified and/or compared with other predicates\n\nSimplification and comparison will need more documentation, but ultimately it\nis the logic that allows us to have a flexible and powerful authorization\nsystem.","ref":"Ash.Filter.Predicate.html"},{"type":"callback","title":"Ash.Filter.Predicate.bulk_compare/1","doc":"As long as at least one predicate of the type defined in your module,\n(and this callback is implemented), it will be called with all of the\nother predicates present in a filter. The return value is relatively\ncomplex, but it should be a list of boolean statements. E.g.\n`{op, left, right}` and `{:not, predicate}` (nested as deep as necessary).\n\nThe best way to do it is to find lists of predicates that are mutually\nexclusive or mutually inclusive, and pass those lists into\n`Ash.SatSolver.mutually_exclusive/1` and `Ash.SatSolver.mutually_inclusive/1`","ref":"Ash.Filter.Predicate.html#c:bulk_compare/1"},{"type":"function","title":"Ash.Filter.Predicate.compare/2","doc":"Checks with each predicate module to see if it has a comparison\nwith","ref":"Ash.Filter.Predicate.html#compare/2"},{"type":"callback","title":"Ash.Filter.Predicate.compare/2","doc":"Compare two predicates. If possible, use `c:bulk_compare/1` instead","ref":"Ash.Filter.Predicate.html#c:compare/2"},{"type":"type","title":"Ash.Filter.Predicate.comparison/0","doc":"","ref":"Ash.Filter.Predicate.html#t:comparison/0"},{"type":"type","title":"Ash.Filter.Predicate.predicate/0","doc":"","ref":"Ash.Filter.Predicate.html#t:predicate/0"},{"type":"callback","title":"Ash.Filter.Predicate.simplify/1","doc":"Simplify to a more primitive statement.\n\nFor example, `x in [1, 2]` simplifies to `x == 1 or x == 2`.\nSimplifying to filter expressions that already have comparisons\nlets you avoid writing that logic for a given predicate.","ref":"Ash.Filter.Predicate.html#c:simplify/1"},{"type":"module","title":"Ash.Query.BooleanExpression","doc":"Represents a boolean expression","ref":"Ash.Query.BooleanExpression.html"},{"type":"function","title":"Ash.Query.BooleanExpression.new/3","doc":"","ref":"Ash.Query.BooleanExpression.html#new/3"},{"type":"function","title":"Ash.Query.BooleanExpression.optimized_new/3","doc":"","ref":"Ash.Query.BooleanExpression.html#optimized_new/3"},{"type":"module","title":"Ash.Query.Call","doc":"Represents a function call/AST node in an Ash query expression","ref":"Ash.Query.Call.html"},{"type":"module","title":"Ash.Query.Exists","doc":"Determines if a given related entity exists.","ref":"Ash.Query.Exists.html"},{"type":"function","title":"Ash.Query.Exists.can_return_nil?/1","doc":"","ref":"Ash.Query.Exists.html#can_return_nil?/1"},{"type":"function","title":"Ash.Query.Exists.new/3","doc":"","ref":"Ash.Query.Exists.html#new/3"},{"type":"behaviour","title":"Ash.Query.Function","doc":"A function is a predicate with an arguments list.\n\nFor more information on being a predicate, see `Ash.Filter.Predicate`. Most of the complexities\nare there. A function must meet both behaviours.","ref":"Ash.Query.Function.html"},{"type":"type","title":"Ash.Query.Function.arg/0","doc":"","ref":"Ash.Query.Function.html#t:arg/0"},{"type":"callback","title":"Ash.Query.Function.args/0","doc":"The number and types of arguments supported.","ref":"Ash.Query.Function.html#c:args/0"},{"type":"callback","title":"Ash.Query.Function.can_return_nil?/1","doc":"Whether or not the function return nil.","ref":"Ash.Query.Function.html#c:can_return_nil?/1"},{"type":"callback","title":"Ash.Query.Function.eager_evaluate?/0","doc":"Whether or not the function can be evaluated eagerly. For example, `now()` cannot be.","ref":"Ash.Query.Function.html#c:eager_evaluate?/0"},{"type":"function","title":"Ash.Query.Function.evaluate/1","doc":"Evaluate the operator with provided inputs","ref":"Ash.Query.Function.html#evaluate/1"},{"type":"callback","title":"Ash.Query.Function.evaluate/1","doc":"Evaluate a function when all arguments are known valid values","ref":"Ash.Query.Function.html#c:evaluate/1"},{"type":"callback","title":"Ash.Query.Function.evaluate_nil_inputs?/0","doc":"If `true`, will be allowed to evaluate `nil` inputs.\n\nIf `false` (the default), any `nil` inputs will cause a `nil` return.","ref":"Ash.Query.Function.html#c:evaluate_nil_inputs?/0"},{"type":"callback","title":"Ash.Query.Function.name/0","doc":"The name of the function","ref":"Ash.Query.Function.html#c:name/0"},{"type":"callback","title":"Ash.Query.Function.new/1","doc":"Instantiate a new function with the provided arguments","ref":"Ash.Query.Function.html#c:new/1"},{"type":"function","title":"Ash.Query.Function.new/2","doc":"","ref":"Ash.Query.Function.html#new/2"},{"type":"function","title":"Ash.Query.Function.ordinal/1","doc":"Attaches the appropriate suffix to refer to an ordinal number, e.g 1 -> \"1st\"","ref":"Ash.Query.Function.html#ordinal/1"},{"type":"callback","title":"Ash.Query.Function.partial_evaluate/1","doc":"Evaluate a function when some or no arguments are known valid values","ref":"Ash.Query.Function.html#c:partial_evaluate/1"},{"type":"callback","title":"Ash.Query.Function.predicate?/0","doc":"Whether or not the function is a predicate (takes a reference as the first argument, a value as the second, and returns a boolean)","ref":"Ash.Query.Function.html#c:predicate?/0"},{"type":"callback","title":"Ash.Query.Function.private?/0","doc":"Whether or not the function should be usable when parsing input.","ref":"Ash.Query.Function.html#c:private?/0"},{"type":"callback","title":"Ash.Query.Function.returns/0","doc":"The return type for each corresponding set of args.","ref":"Ash.Query.Function.html#c:returns/0"},{"type":"function","title":"Ash.Query.Function.try_cast_arguments/2","doc":"","ref":"Ash.Query.Function.html#try_cast_arguments/2"},{"type":"module","title":"Ash.Query.Not","doc":"Represents the negation of the contained expression","ref":"Ash.Query.Not.html"},{"type":"function","title":"Ash.Query.Not.new/1","doc":"","ref":"Ash.Query.Not.html#new/1"},{"type":"behaviour","title":"Ash.Query.Operator","doc":"An operator is a predicate with a `left` and a `right`\n\nFor more information on being a predicate, see `Ash.Filter.Predicate`. Most of the complexities\nare there. An operator must meet both behaviours.","ref":"Ash.Query.Operator.html"},{"type":"callback","title":"Ash.Query.Operator.can_return_nil?/1","doc":"Whether or not the operator can evaluate to nil.","ref":"Ash.Query.Operator.html#c:can_return_nil?/1"},{"type":"function","title":"Ash.Query.Operator.evaluate/1","doc":"Evaluate the operator with provided inputs","ref":"Ash.Query.Operator.html#evaluate/1"},{"type":"callback","title":"Ash.Query.Operator.evaluate/1","doc":"Evaluates the operator in Elixir","ref":"Ash.Query.Operator.html#c:evaluate/1"},{"type":"callback","title":"Ash.Query.Operator.evaluate_nil_inputs?/0","doc":"If `true`, will be allowed to evaluate `nil` inputs.\n\nIf `false` (the default), any `nil` inputs will cause a `nil` return.","ref":"Ash.Query.Operator.html#c:evaluate_nil_inputs?/0"},{"type":"callback","title":"Ash.Query.Operator.new/2","doc":"Create a new predicate. There are various return types possible:\n\n  * `{:ok, left, right}` - Return the left/right values of the operator\n  * `{:ok, operator}` - Return the operator itself, this or the one above are acceptable\n  * `{:known, boolean}` - If the value is already known, e.g `1 == 1`\n  * `{:error, error}` - If there was an error creating the operator","ref":"Ash.Query.Operator.html#c:new/2"},{"type":"function","title":"Ash.Query.Operator.new/3","doc":"Create a new operator. Pass the module and the left and right values","ref":"Ash.Query.Operator.html#new/3"},{"type":"function","title":"Ash.Query.Operator.operator_overloads/1","doc":"Get type overloads for the given operator","ref":"Ash.Query.Operator.html#operator_overloads/1"},{"type":"function","title":"Ash.Query.Operator.operator_symbols/0","doc":"","ref":"Ash.Query.Operator.html#operator_symbols/0"},{"type":"function","title":"Ash.Query.Operator.operators/0","doc":"","ref":"Ash.Query.Operator.html#operators/0"},{"type":"callback","title":"Ash.Query.Operator.predicate?/0","doc":"","ref":"Ash.Query.Operator.html#c:predicate?/0"},{"type":"callback","title":"Ash.Query.Operator.returns/0","doc":"The types that the expression can return. Should be one entry in the list for each entry in `types`.","ref":"Ash.Query.Operator.html#c:returns/0"},{"type":"callback","title":"Ash.Query.Operator.to_string/2","doc":"The implementation of the inspect protocol.\n\nIf not defined, it will be inferred","ref":"Ash.Query.Operator.html#c:to_string/2"},{"type":"callback","title":"Ash.Query.Operator.types/0","doc":"The types accepted by the operator. Defaults to `[:same, :any]`, which is any values of the same type.","ref":"Ash.Query.Operator.html#c:types/0"},{"type":"module","title":"Ash.Query.Parent","doc":"Used to access values from the \"source\" of a given expression.\n\nThis is used in cases where expressions are given for some relationship path, for example:any()\n\n```elixir\n has_many :foo, Foo do\n   filter expr(priority == :foo and type == parent(foo_type))\n end\n```\n\nThis is supported on a case by case basis by a given data layer and in specific usages.","ref":"Ash.Query.Parent.html"},{"type":"function","title":"Ash.Query.Parent.new/1","doc":"","ref":"Ash.Query.Parent.html#new/1"},{"type":"module","title":"Ash.Query.Ref","doc":"Represents a relation/attribute reference","ref":"Ash.Query.Ref.html"},{"type":"function","title":"Ash.Query.Ref.name/1","doc":"Returns the referenced field","ref":"Ash.Query.Ref.html#name/1"},{"type":"module","title":"Ash.Query.Function.Ago","doc":"Subtracts the given interval or Duration from the current time in UTC.\n\nFor example:\n   deleted_at > ago(7, :day)\n   deleted_at > ago(Duration.new!(day: 7))\n\nDocumentation + available intervals inspired by the corresponding ecto interval implementation","ref":"Ash.Query.Function.Ago.html"},{"type":"function","title":"Ash.Query.Function.Ago.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Ago.html#args/0"},{"type":"function","title":"Ash.Query.Function.Ago.datetime_add/2","doc":"","ref":"Ash.Query.Function.Ago.html#datetime_add/2"},{"type":"module","title":"Ash.Query.Function.At","doc":"Gets an element in the list by index","ref":"Ash.Query.Function.At.html"},{"type":"function","title":"Ash.Query.Function.At.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.At.html#args/0"},{"type":"module","title":"Ash.Query.Function.CompositeType","doc":"Constructs a composite type in a way that is natively understood by the data layer\n\nTo do this, provide a tuple matching the format expected by the type in question.\nCheck that type's documentation for this information.","ref":"Ash.Query.Function.CompositeType.html"},{"type":"function","title":"Ash.Query.Function.CompositeType.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.CompositeType.html#args/0"},{"type":"module","title":"Ash.Query.Function.Contains","doc":"Returns true if the first string contains the second.\n\nCase insensitive strings are accounted for on either side.\n\n   contains(\"foo\", \"fo\")\n   true\n\n   contains(%Ash.CiString{:string \"foo\"}, \"FoO\")\n   true\n\n   contains(\"foo\", %Ash.CiString{:string \"FOO\"})\n   true","ref":"Ash.Query.Function.Contains.html"},{"type":"function","title":"Ash.Query.Function.Contains.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Contains.html#args/0"},{"type":"module","title":"Ash.Query.Function.CountNils","doc":"Returns the count of nil.\n\n    count_nil([nil, 1, nil]) # 2","ref":"Ash.Query.Function.CountNils.html"},{"type":"function","title":"Ash.Query.Function.CountNils.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.CountNils.html#args/0"},{"type":"module","title":"Ash.Query.Function.DateAdd","doc":"Adds the given interval or Duration to the current time in UTC\nAdds the given interval or Duration to the current time in UTC\n\nFor example:\n   activates_at < date_add(today(), 7, :day)\n   activates_at < date_add(today(), Duration.new!(day: 7))\n   activates_at < date_add(today(), Duration.new!(day: 7))\n\nDocumentation + available intervals inspired by the corresponding ecto interval implementation","ref":"Ash.Query.Function.DateAdd.html"},{"type":"function","title":"Ash.Query.Function.DateAdd.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.DateAdd.html#args/0"},{"type":"module","title":"Ash.Query.Function.DateTimeAdd","doc":"Adds the given interval or Duration to the current time in UTC\n\nFor example:\n   activates_at < datetime_add(now(), 7, :day)\n   activates_at < datetime_add(now(), Duration.new!(day:7))\n\nDocumentation + available intervals inspired by the corresponding ecto interval implementation","ref":"Ash.Query.Function.DateTimeAdd.html"},{"type":"function","title":"Ash.Query.Function.DateTimeAdd.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.DateTimeAdd.html#args/0"},{"type":"module","title":"Ash.Query.Function.Error","doc":"If the predicate is truthy, the provided exception is raised with the provided values.\n\nThis exception is not \"raised\" in the Elixir sense, but the entire expression fails to\nevaluate with the given error. Various data layers will handle this differently.","ref":"Ash.Query.Function.Error.html"},{"type":"function","title":"Ash.Query.Function.Error.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Error.html#args/0"},{"type":"module","title":"Ash.Query.Function.Fragment","doc":"Adds the given interval from the current time in UTC.\n\nFor example:\n   expires_at < from_now(7, :day)\n\nDocumentation + available intervals inspired by the corresponding ecto interval implementation","ref":"Ash.Query.Function.Fragment.html"},{"type":"function","title":"Ash.Query.Function.Fragment.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Fragment.html#args/0"},{"type":"function","title":"Ash.Query.Function.Fragment.casted_new/1","doc":"","ref":"Ash.Query.Function.Fragment.html#casted_new/1"},{"type":"module","title":"Ash.Query.Function.FromNow","doc":"Adds the given interval or Duration from the current time in UTC.\n\nFor example:\n   expires_at < from_now(7, :day)\n   expires_at < from_now(Duration.new!(day: 7))\n\nDocumentation + available intervals inspired by the corresponding ecto interval implementation","ref":"Ash.Query.Function.FromNow.html"},{"type":"function","title":"Ash.Query.Function.FromNow.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.FromNow.html#args/0"},{"type":"module","title":"Ash.Query.Function.GetPath","doc":"Gets the value at the provided path in the value, which must be a map or embed.\n\nIf you are using a datalayer that provides a `type` function (like AshPostgres), it is a good idea to\nwrap your call in that function, e.g `type(author[:bio][:title], :string)`, since data layers that depend\non knowing types may not be able to infer the type from the path. Ash may eventually be able to figure out\nthe type, in the case that the path consists of only embedded attributes.\n\nIf an atom key is provided, access is *indiscriminate* of atoms vs strings. The atom key is checked first.\nIf a string key is provided, that is the only thing that is checked. If the value will or may be a struct, be sure to use atoms.\n\nThe data layer may handle this differently, for example, AshPostgres only checks\nstrings at the data layer (because thats all it can be in the database anyway).\n\nAvailable in query expressions using bracket syntax, e.g `foo[:bar][:baz]`.","ref":"Ash.Query.Function.GetPath.html"},{"type":"function","title":"Ash.Query.Function.GetPath.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.GetPath.html#args/0"},{"type":"module","title":"Ash.Query.Function.If","doc":"If predicate is truthy, then the second argument is returned, otherwise the third.","ref":"Ash.Query.Function.If.html"},{"type":"function","title":"Ash.Query.Function.If.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.If.html#args/0"},{"type":"function","title":"Ash.Query.Function.If.partial_evaluate/1","doc":"Callback implementation for  Ash.Query.Function.partial_evaluate/1 .","ref":"Ash.Query.Function.If.html#partial_evaluate/1"},{"type":"module","title":"Ash.Query.Function.IsNil","doc":"true if the provided field is nil","ref":"Ash.Query.Function.IsNil.html"},{"type":"function","title":"Ash.Query.Function.IsNil.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.IsNil.html#args/0"},{"type":"module","title":"Ash.Query.Function.Lazy","doc":"Runs the provided MFA and returns the result as a known value.\n\nEvaluated just before running the query.","ref":"Ash.Query.Function.Lazy.html"},{"type":"function","title":"Ash.Query.Function.Lazy.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Lazy.html#args/0"},{"type":"module","title":"Ash.Query.Function.Length","doc":"Returns the length of a list attribute defined by the composite type `{:array, Type}`.\n\n    length(roles)\n\nIf the attribute allows nils:\n\n    length(roles || [])","ref":"Ash.Query.Function.Length.html"},{"type":"function","title":"Ash.Query.Function.Length.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Length.html#args/0"},{"type":"module","title":"Ash.Query.Function.Minus","doc":"Negates the value","ref":"Ash.Query.Function.Minus.html"},{"type":"function","title":"Ash.Query.Function.Minus.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Minus.html#args/0"},{"type":"module","title":"Ash.Query.Function.Now","doc":"Returns the current datetime","ref":"Ash.Query.Function.Now.html"},{"type":"function","title":"Ash.Query.Function.Now.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Now.html#args/0"},{"type":"module","title":"Ash.Query.Function.Rem","doc":"Rounds a float, decimal or integer to the given number of points","ref":"Ash.Query.Function.Rem.html"},{"type":"function","title":"Ash.Query.Function.Rem.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Rem.html#args/0"},{"type":"module","title":"Ash.Query.Function.Round","doc":"Rounds a float, decimal or integer to the given number of points","ref":"Ash.Query.Function.Round.html"},{"type":"function","title":"Ash.Query.Function.Round.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Round.html#args/0"},{"type":"module","title":"Ash.Query.Function.StartOfDay","doc":"Converts a date or datetime into the start of day\n\nAccepts an optional time zone, in the same format that can be passed to\n`DateTime.new/3`.\n\nFor example:\n   start_of_day(now()) < a_datetime()\n   start_of_day(now(), \"Europe/Copenhagen\") < a_datetime()","ref":"Ash.Query.Function.StartOfDay.html"},{"type":"function","title":"Ash.Query.Function.StartOfDay.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StartOfDay.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringDowncase","doc":"Downcase a string","ref":"Ash.Query.Function.StringDowncase.html"},{"type":"function","title":"Ash.Query.Function.StringDowncase.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringDowncase.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringJoin","doc":"Joins a list of values.\n\nIgnores `nil` values and concatenates the remaining non-nil values. An optional\njoiner can be provided.\n\n    string_join([first_name, last_name], \" \")\n\n    string_join([item_a, item_b])","ref":"Ash.Query.Function.StringJoin.html"},{"type":"function","title":"Ash.Query.Function.StringJoin.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringJoin.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringLength","doc":"Trims whitespace from a string","ref":"Ash.Query.Function.StringLength.html"},{"type":"function","title":"Ash.Query.Function.StringLength.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringLength.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringPosition","doc":"Returns the zero-based position of a substring within a string, nil if the string does not contain the substring.\n\nCase insensitive strings are accounted for on either side.\n\n   string_position(\"foo\", \"fo\")\n   0\n\n   string_position(%Ash.CiString{string: \"foo\"}, \"FoO\")\n   0\n\n   string_position(\"foo\", %Ash.CiString{string: \"FOO\"})\n   0","ref":"Ash.Query.Function.StringPosition.html"},{"type":"function","title":"Ash.Query.Function.StringPosition.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringPosition.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringSplit","doc":"Split a string into a list of strings\n\nSplits a string on the given delimiter. The delimiter defaults to a single space. Also supports options.\n\nKeep in mind, this function does *not* support regexes the way that `String.split/3` does, only raw strings.\n\n    string_split(employee_code)\n    string_split(full_name, \"foo\")\n    string_split(full_name, \"foo\", trim?: true)","ref":"Ash.Query.Function.StringSplit.html"},{"type":"module","title":"Options - Ash.Query.Function.StringSplit","doc":"* `:trim?` (`t:boolean/0`) - Whether or not to trim empty strings from the beginning or end of the result. Equivalent to the `trim` option to `String.split/3` The default value is `false`.","ref":"Ash.Query.Function.StringSplit.html#module-options"},{"type":"function","title":"Ash.Query.Function.StringSplit.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringSplit.html#args/0"},{"type":"module","title":"Ash.Query.Function.StringTrim","doc":"Trims whitespace from a string","ref":"Ash.Query.Function.StringTrim.html"},{"type":"function","title":"Ash.Query.Function.StringTrim.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.StringTrim.html#args/0"},{"type":"module","title":"Ash.Query.Function.Today","doc":"Returns the current date.","ref":"Ash.Query.Function.Today.html"},{"type":"function","title":"Ash.Query.Function.Today.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Today.html#args/0"},{"type":"module","title":"Ash.Query.Function.Type","doc":"Casts the value to a given type. Can also be used to provide type hints to data layers, where appropriate.","ref":"Ash.Query.Function.Type.html"},{"type":"function","title":"Ash.Query.Function.Type.args/0","doc":"Callback implementation for  Ash.Query.Function.args/0 .","ref":"Ash.Query.Function.Type.html#args/0"},{"type":"module","title":"Ash.Query.Operator.Eq","doc":"left == right\n\nThe simplest operator, matches if the left and right are equal.\n\nFor comparison, this compares as mutually exclusive with other equality\nand `is_nil` checks that have the same reference on the left side","ref":"Ash.Query.Operator.Eq.html"},{"type":"function","title":"Ash.Query.Operator.Eq.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.Eq.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.Eq.name/0","doc":"","ref":"Ash.Query.Operator.Eq.html#name/0"},{"type":"function","title":"Ash.Query.Operator.Eq.operator/0","doc":"","ref":"Ash.Query.Operator.Eq.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.GreaterThan","doc":"left > right\n\nIn comparison, simplifies to `not(left < right + 1)`, so it will never need to be compared against.","ref":"Ash.Query.Operator.GreaterThan.html"},{"type":"function","title":"Ash.Query.Operator.GreaterThan.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.GreaterThan.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.GreaterThan.name/0","doc":"","ref":"Ash.Query.Operator.GreaterThan.html#name/0"},{"type":"function","title":"Ash.Query.Operator.GreaterThan.operator/0","doc":"","ref":"Ash.Query.Operator.GreaterThan.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.GreaterThanOrEqual","doc":"left >= right\n\nIn comparison, simplifies to `not(left < right)`, so it will never need to be compared against.","ref":"Ash.Query.Operator.GreaterThanOrEqual.html"},{"type":"function","title":"Ash.Query.Operator.GreaterThanOrEqual.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.GreaterThanOrEqual.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.GreaterThanOrEqual.name/0","doc":"","ref":"Ash.Query.Operator.GreaterThanOrEqual.html#name/0"},{"type":"function","title":"Ash.Query.Operator.GreaterThanOrEqual.operator/0","doc":"","ref":"Ash.Query.Operator.GreaterThanOrEqual.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.In","doc":"left in [1, 2, 3]\n\nthis predicate matches if the left is in the list on the right\n\nFor comparison, this simplifies to a set of \"or equals\", e.g\n`{:or, {:or, {:or, left == 1}, left == 2}, left == 3}`","ref":"Ash.Query.Operator.In.html"},{"type":"function","title":"Ash.Query.Operator.In.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.In.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.In.name/0","doc":"","ref":"Ash.Query.Operator.In.html#name/0"},{"type":"function","title":"Ash.Query.Operator.In.operator/0","doc":"","ref":"Ash.Query.Operator.In.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.IsNil","doc":"left is_nil true/false\n\nThis predicate matches if the left is nil when the right is `true` or if the\nleft is not nil when the right is `false`","ref":"Ash.Query.Operator.IsNil.html"},{"type":"function","title":"Ash.Query.Operator.IsNil.name/0","doc":"","ref":"Ash.Query.Operator.IsNil.html#name/0"},{"type":"function","title":"Ash.Query.Operator.IsNil.operator/0","doc":"","ref":"Ash.Query.Operator.IsNil.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.LessThan","doc":"left < right\n\nDoes not simplify, but is used as the simplification value for\n`Ash.Query.Operator.LessThanOrEqual`, `Ash.Query.Operator.GreaterThan` and\n`Ash.Query.Operator.GreaterThanOrEqual`.\n\nWhen comparing predicates, it is mutually exclusive with `Ash.Query.Operator.IsNil`.\nAdditionally, it compares as mutually inclusive with any `Ash.Query.Operator.Eq` and\nany `Ash.Query.Operator.LessThan` who's right sides are less than it, and mutually\nexclusive with any `Ash.Query.Operator.Eq` or `Ash.Query.Operator.GreaterThan` who's\nright side's are greater than or equal to it.","ref":"Ash.Query.Operator.LessThan.html"},{"type":"function","title":"Ash.Query.Operator.LessThan.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.LessThan.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.LessThan.name/0","doc":"","ref":"Ash.Query.Operator.LessThan.html#name/0"},{"type":"function","title":"Ash.Query.Operator.LessThan.operator/0","doc":"","ref":"Ash.Query.Operator.LessThan.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.LessThanOrEqual","doc":"left <= right\n\nIn comparison, simplifies to `left < right + 1`, so it will never need to be compared against.","ref":"Ash.Query.Operator.LessThanOrEqual.html"},{"type":"function","title":"Ash.Query.Operator.LessThanOrEqual.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.LessThanOrEqual.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.LessThanOrEqual.name/0","doc":"","ref":"Ash.Query.Operator.LessThanOrEqual.html#name/0"},{"type":"function","title":"Ash.Query.Operator.LessThanOrEqual.operator/0","doc":"","ref":"Ash.Query.Operator.LessThanOrEqual.html#operator/0"},{"type":"module","title":"Ash.Query.Operator.NotEq","doc":"left != right\n\nIn comparison, simplifies to `not(left == right)`","ref":"Ash.Query.Operator.NotEq.html"},{"type":"function","title":"Ash.Query.Operator.NotEq.evaluate/1","doc":"Callback implementation for  Ash.Query.Operator.evaluate/1 .","ref":"Ash.Query.Operator.NotEq.html#evaluate/1"},{"type":"function","title":"Ash.Query.Operator.NotEq.name/0","doc":"","ref":"Ash.Query.Operator.NotEq.html#name/0"},{"type":"function","title":"Ash.Query.Operator.NotEq.operator/0","doc":"","ref":"Ash.Query.Operator.NotEq.html#operator/0"},{"type":"module","title":"Ash.Notifier.PubSub.Publication","doc":"Represents a configured publication from the pubsub notifier on an Ash.Resource","ref":"Ash.Notifier.PubSub.Publication.html"},{"type":"function","title":"Ash.Notifier.PubSub.Publication.publish_all_schema/0","doc":"","ref":"Ash.Notifier.PubSub.Publication.html#publish_all_schema/0"},{"type":"function","title":"Ash.Notifier.PubSub.Publication.schema/0","doc":"","ref":"Ash.Notifier.PubSub.Publication.html#schema/0"},{"type":"module","title":"Ash.Policy.FieldPolicy","doc":"Represents a field policy in an Ash.Resource","ref":"Ash.Policy.FieldPolicy.html"},{"type":"type","title":"Ash.Policy.FieldPolicy.t/0","doc":"","ref":"Ash.Policy.FieldPolicy.html#t:t/0"},{"type":"module","title":"Ash.Policy.Policy","doc":"Represents a policy on an Ash.Resource","ref":"Ash.Policy.Policy.html"},{"type":"function","title":"Ash.Policy.Policy.at_least_one_policy_expression/1","doc":"","ref":"Ash.Policy.Policy.html#at_least_one_policy_expression/1"},{"type":"function","title":"Ash.Policy.Policy.fetch_fact/2","doc":"","ref":"Ash.Policy.Policy.html#fetch_fact/2"},{"type":"function","title":"Ash.Policy.Policy.fetch_or_strict_check_fact/2","doc":"","ref":"Ash.Policy.Policy.html#fetch_or_strict_check_fact/2"},{"type":"function","title":"Ash.Policy.Policy.solve/1","doc":"","ref":"Ash.Policy.Policy.html#solve/1"},{"type":"type","title":"Ash.Policy.Policy.t/0","doc":"","ref":"Ash.Policy.Policy.html#t:t/0"},{"type":"module","title":"Ash.Resource.Actions","doc":"Types for Ash actions","ref":"Ash.Resource.Actions.html"},{"type":"type","title":"Ash.Resource.Actions.action/0","doc":"","ref":"Ash.Resource.Actions.html#t:action/0"},{"type":"type","title":"Ash.Resource.Actions.action_type/0","doc":"","ref":"Ash.Resource.Actions.html#t:action_type/0"},{"type":"module","title":"Ash.Resource.Actions.Action","doc":"Represents a custom action on a resource.","ref":"Ash.Resource.Actions.Action.html"},{"type":"type","title":"Ash.Resource.Actions.Action.t/0","doc":"","ref":"Ash.Resource.Actions.Action.html#t:t/0"},{"type":"function","title":"Ash.Resource.Actions.Action.transform/1","doc":"","ref":"Ash.Resource.Actions.Action.html#transform/1"},{"type":"module","title":"Ash.Resource.Actions.Argument","doc":"Represents an argument to an action","ref":"Ash.Resource.Actions.Argument.html"},{"type":"function","title":"Ash.Resource.Actions.Argument.schema/0","doc":"","ref":"Ash.Resource.Actions.Argument.html#schema/0"},{"type":"type","title":"Ash.Resource.Actions.Argument.t/0","doc":"","ref":"Ash.Resource.Actions.Argument.html#t:t/0"},{"type":"module","title":"Ash.Resource.Actions.Create","doc":"Represents a create action on a resource.","ref":"Ash.Resource.Actions.Create.html"},{"type":"type","title":"Ash.Resource.Actions.Create.t/0","doc":"","ref":"Ash.Resource.Actions.Create.html#t:t/0"},{"type":"module","title":"Ash.Resource.Actions.Destroy","doc":"Represents a destroy action on a resource.","ref":"Ash.Resource.Actions.Destroy.html"},{"type":"type","title":"Ash.Resource.Actions.Destroy.t/0","doc":"","ref":"Ash.Resource.Actions.Destroy.html#t:t/0"},{"type":"function","title":"Ash.Resource.Actions.Destroy.transform/1","doc":"","ref":"Ash.Resource.Actions.Destroy.html#transform/1"},{"type":"behaviour","title":"Ash.Resource.Actions.Implementation","doc":"An implementation of a [generic action](generic-actions.md).","ref":"Ash.Resource.Actions.Implementation.html"},{"type":"behaviour","title":"Example - Ash.Resource.Actions.Implementation","doc":"```\ndefmodule YourModule do\n  use Ash.Resource.Actions.Implementation\n\n  def run(input, opts, context) do\n    {:ok, \"Hello\"}\n  end\nend\n```","ref":"Ash.Resource.Actions.Implementation.html#module-example"},{"type":"callback","title":"Ash.Resource.Actions.Implementation.run/3","doc":"","ref":"Ash.Resource.Actions.Implementation.html#c:run/3"},{"type":"module","title":"Ash.Resource.Actions.Implementation.Context","doc":"The context passed into generic action functions","ref":"Ash.Resource.Actions.Implementation.Context.html"},{"type":"type","title":"Ash.Resource.Actions.Implementation.Context.t/0","doc":"","ref":"Ash.Resource.Actions.Implementation.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.Actions.Metadata","doc":"Represents metadata from an action","ref":"Ash.Resource.Actions.Metadata.html"},{"type":"function","title":"Ash.Resource.Actions.Metadata.schema/0","doc":"","ref":"Ash.Resource.Actions.Metadata.html#schema/0"},{"type":"type","title":"Ash.Resource.Actions.Metadata.t/0","doc":"","ref":"Ash.Resource.Actions.Metadata.html#t:t/0"},{"type":"module","title":"Ash.Resource.Actions.Read","doc":"Represents a read action on a resource.","ref":"Ash.Resource.Actions.Read.html"},{"type":"function","title":"Ash.Resource.Actions.Read.pagination_schema/0","doc":"","ref":"Ash.Resource.Actions.Read.html#pagination_schema/0"},{"type":"type","title":"Ash.Resource.Actions.Read.t/0","doc":"","ref":"Ash.Resource.Actions.Read.html#t:t/0"},{"type":"function","title":"Ash.Resource.Actions.Read.transform/1","doc":"","ref":"Ash.Resource.Actions.Read.html#transform/1"},{"type":"module","title":"Ash.Resource.Actions.Read.Pagination","doc":"Represents the pagination configuration of a read action","ref":"Ash.Resource.Actions.Read.Pagination.html"},{"type":"type","title":"Ash.Resource.Actions.Read.Pagination.t/0","doc":"","ref":"Ash.Resource.Actions.Read.Pagination.html#t:t/0"},{"type":"function","title":"Ash.Resource.Actions.Read.Pagination.transform/1","doc":"","ref":"Ash.Resource.Actions.Read.Pagination.html#transform/1"},{"type":"module","title":"Ash.Resource.Actions.Update","doc":"Represents a update action on a resource.","ref":"Ash.Resource.Actions.Update.html"},{"type":"type","title":"Ash.Resource.Actions.Update.t/0","doc":"","ref":"Ash.Resource.Actions.Update.html#t:t/0"},{"type":"function","title":"Ash.Resource.Actions.Update.transform/1","doc":"","ref":"Ash.Resource.Actions.Update.html#transform/1"},{"type":"module","title":"Ash.Resource.Aggregate","doc":"Represents a named aggregate on the resource that can be loaded","ref":"Ash.Resource.Aggregate.html"},{"type":"type","title":"Ash.Resource.Aggregate.t/0","doc":"","ref":"Ash.Resource.Aggregate.html#t:t/0"},{"type":"behaviour","title":"Ash.Resource.Aggregate.CustomAggregate","doc":"The root behavior for a custom aggregate.\n\nSee data layers for their implementation of custom aggregates.","ref":"Ash.Resource.Aggregate.CustomAggregate.html"},{"type":"callback","title":"Ash.Resource.Aggregate.CustomAggregate.describe/1","doc":"","ref":"Ash.Resource.Aggregate.CustomAggregate.html#c:describe/1"},{"type":"type","title":"Ash.Resource.Aggregate.CustomAggregate.t/0","doc":"","ref":"Ash.Resource.Aggregate.CustomAggregate.html#t:t/0"},{"type":"module","title":"Ash.Resource.Aggregate.JoinFilter","doc":"Represents a join filter on a resource aggregate","ref":"Ash.Resource.Aggregate.JoinFilter.html"},{"type":"module","title":"Ash.Resource.Attribute","doc":"Represents an attribute on a resource","ref":"Ash.Resource.Attribute.html"},{"type":"function","title":"Ash.Resource.Attribute.create_timestamp_schema/0","doc":"","ref":"Ash.Resource.Attribute.html#create_timestamp_schema/0"},{"type":"function","title":"Ash.Resource.Attribute.integer_primary_key_schema/0","doc":"","ref":"Ash.Resource.Attribute.html#integer_primary_key_schema/0"},{"type":"type","title":"Ash.Resource.Attribute.t/0","doc":"","ref":"Ash.Resource.Attribute.html#t:t/0"},{"type":"function","title":"Ash.Resource.Attribute.transform/1","doc":"","ref":"Ash.Resource.Attribute.html#transform/1"},{"type":"function","title":"Ash.Resource.Attribute.update_timestamp_schema/0","doc":"","ref":"Ash.Resource.Attribute.html#update_timestamp_schema/0"},{"type":"function","title":"Ash.Resource.Attribute.uuid_primary_key_schema/0","doc":"","ref":"Ash.Resource.Attribute.html#uuid_primary_key_schema/0"},{"type":"function","title":"Ash.Resource.Attribute.uuid_v7_primary_key_schema/0","doc":"","ref":"Ash.Resource.Attribute.html#uuid_v7_primary_key_schema/0"},{"type":"module","title":"Ash.Resource.Calculation.Argument","doc":"An argument to a calculation","ref":"Ash.Resource.Calculation.Argument.html"},{"type":"function","title":"Ash.Resource.Calculation.Argument.schema/0","doc":"","ref":"Ash.Resource.Calculation.Argument.html#schema/0"},{"type":"type","title":"Ash.Resource.Calculation.Argument.t/0","doc":"","ref":"Ash.Resource.Calculation.Argument.html#t:t/0"},{"type":"module","title":"Ash.Resource.Calculation.Context","doc":"The context and arguments of a calculation","ref":"Ash.Resource.Calculation.Context.html"},{"type":"type","title":"Ash.Resource.Calculation.Context.t/0","doc":"","ref":"Ash.Resource.Calculation.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.Calculation.LoadAttribute","doc":"Loads an attribute as a calculation.\n\nCan be used to load the same attribute with different load statements applied.","ref":"Ash.Resource.Calculation.LoadAttribute.html"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.calculate/3","doc":"Callback implementation for  Ash.Resource.Calculation.calculate/3 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#calculate/3"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.describe/1","doc":"Callback implementation for  Ash.Resource.Calculation.describe/1 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#describe/1"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.has_calculate?/0","doc":"","ref":"Ash.Resource.Calculation.LoadAttribute.html#has_calculate?/0"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.has_expression?/0","doc":"Callback implementation for  Ash.Resource.Calculation.has_expression?/0 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#has_expression?/0"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.init/1","doc":"Callback implementation for  Ash.Resource.Calculation.init/1 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#init/1"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.load/3","doc":"Callback implementation for  Ash.Resource.Calculation.load/3 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#load/3"},{"type":"function","title":"Ash.Resource.Calculation.LoadAttribute.strict_loads?/0","doc":"Callback implementation for  Ash.Resource.Calculation.strict_loads?/0 .","ref":"Ash.Resource.Calculation.LoadAttribute.html#strict_loads?/0"},{"type":"module","title":"Ash.Resource.Calculation.LoadRelationship","doc":"Loads a relationship as a calculation.\n\nCan be used to load the same relationship with a different query.","ref":"Ash.Resource.Calculation.LoadRelationship.html"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.calculate/3","doc":"Callback implementation for  Ash.Resource.Calculation.calculate/3 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#calculate/3"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.describe/1","doc":"Callback implementation for  Ash.Resource.Calculation.describe/1 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#describe/1"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.has_calculate?/0","doc":"","ref":"Ash.Resource.Calculation.LoadRelationship.html#has_calculate?/0"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.has_expression?/0","doc":"Callback implementation for  Ash.Resource.Calculation.has_expression?/0 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#has_expression?/0"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.init/1","doc":"Callback implementation for  Ash.Resource.Calculation.init/1 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#init/1"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.load/3","doc":"Callback implementation for  Ash.Resource.Calculation.load/3 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#load/3"},{"type":"function","title":"Ash.Resource.Calculation.LoadRelationship.strict_loads?/0","doc":"Callback implementation for  Ash.Resource.Calculation.strict_loads?/0 .","ref":"Ash.Resource.Calculation.LoadRelationship.html#strict_loads?/0"},{"type":"module","title":"Ash.Resource.CalculationInterface","doc":"Represents a function that evaluates a calculation in a resource's code interface","ref":"Ash.Resource.CalculationInterface.html"},{"type":"function","title":"Ash.Resource.CalculationInterface.schema/0","doc":"","ref":"Ash.Resource.CalculationInterface.html#schema/0"},{"type":"type","title":"Ash.Resource.CalculationInterface.t/0","doc":"","ref":"Ash.Resource.CalculationInterface.html#t:t/0"},{"type":"function","title":"Ash.Resource.CalculationInterface.transform/1","doc":"","ref":"Ash.Resource.CalculationInterface.html#transform/1"},{"type":"module","title":"Ash.Resource.Identity","doc":"Represents a unique constraint on a resource\n\nData layers should (and all built in ones do), discount `nil` or `null` (in the case of postgres) values\nwhen determining if a unique constraint matches. This often means that you should\nprefer to use identities with non-nullable columns.\n\nEventually, features could be added to support including `nil` or `null` values, but they would\nneed to include a corresponding feature for data layers.","ref":"Ash.Resource.Identity.html"},{"type":"function","title":"Ash.Resource.Identity.schema/0","doc":"","ref":"Ash.Resource.Identity.html#schema/0"},{"type":"type","title":"Ash.Resource.Identity.t/0","doc":"","ref":"Ash.Resource.Identity.html#t:t/0"},{"type":"module","title":"Ash.Resource.Interface","doc":"Represents a function in a resource's code interface\n\nSee the functions defined in this module for specifications of the options\nthat each type of code interface function supports in its options list.","ref":"Ash.Resource.Interface.html"},{"type":"function","title":"Ash.Resource.Interface.calculate_opts/0","doc":"Options supported by code interfaces for calculations","ref":"Ash.Resource.Interface.html#calculate_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.calculate_opts/0","doc":"* `:actor` (`t:term/0`) - The actor for handling `^actor/1` templates, supplied to calculation context.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant, supplied to calculation context.\n\n* `:context` (`t:map/0`) - Context to set on the calculation input.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the request is being authorized, provided to calculation context. The default value is `true`.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer, provided to the calculation context.\n\n* `:data_layer?` (`t:boolean/0`) - Set to `true` to require that the value be computed within the data layer. Only works for calculations that define an expression.\n\n* `:reuse_values?` (`t:boolean/0`) - Set to `true` to reuse existing values on any provided record. Only necessary if providing a record as the basis for calculation. The default value is `false`.","ref":"Ash.Resource.Interface.html#calculate_opts/0-options"},{"type":"function","title":"Ash.Resource.Interface.create_opts/0","doc":"Options supported by code interfaces for create actions","ref":"Ash.Resource.Interface.html#create_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.create_opts/0","doc":"* `:upsert?` (`t:boolean/0`) - If a conflict is found based on the primary key, the record is updated in the database (requires upsert support) The default value is `false`.\n\n* `:return_skipped_upsert?` (`t:boolean/0`) - If `true`, and a record was *not* upserted because its filter prevented the upsert, the original record (which was *not* upserted) will be returned. The default value is `false`.\n\n* `:upsert_identity` (`t:atom/0`) - The identity to use when detecting conflicts for `upsert?`, e.g. `upsert_identity: :full_name`. By default, the primary key is used. Has no effect if `upsert?: true` is not provided\n\n* `:upsert_fields` - The fields to upsert. If not set, the action's upsert_fields is used, and if that is not set, then any fields not being set to defaults are written.\n\n* `:upsert_condition` (`t:term/0`) - An expression to check if the record should be updated when there's a conflict.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset\n\n* `:changeset` (`t:term/0`) - A changeset to seed the action with.\n\n* `:bulk_options` (`t:keyword/0`) - Options passed to `Ash.bulk_create`, if a list or stream of inputs is provided.\n\n  * `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n  * `:after_action` (function of arity 2) - An after_action hook to be added to each processed changeset\n\n  * `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n  * `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n  * `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n  * `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. The default value is `:filter`.\n\n  * `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n  * `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n  * `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n  * `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n  * `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n    or `max_page_size` of the action, or 100.\n\n  * `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.\n\n    Potential elements:\n\n    `{:notification, notification}` - if `return_notifications?` is set to `true`\n    `{:ok, record}` - if `return_records?` is set to `true`\n    `{:error, error}` - an error that occurred. May be changeset or an individual error.\n\n    The default value is `false`.\n\n  * `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.\n\n    Only relevant if `return_stream?` is set to `true` and all other\n    `return_*?` options are set to `false`.\n\n    The default value is `false`.\n\n  * `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n    will be skipped. The default value is `true`.\n\n  * `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n    the results from each batch. This may be intensive for large bulk actions.\n\n    Notifications will be automatically sent unless `return_notifications?` is set to `true`.\n\n    The default value is `false`.\n\n  * `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.\n\n    Keep in mind:\n\n    `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n    *inside* the transaction if you choose `transaction: :all`.\n\n    The default value is `:batch`.\n\n  * `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set before validations and changes. The default value is `%{}`.","ref":"Ash.Resource.Interface.html#create_opts/0-options"},{"type":"function","title":"Ash.Resource.Interface.destroy_opts/0","doc":"Options supported by code interfaces for destroy actions","ref":"Ash.Resource.Interface.html#destroy_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.destroy_opts/0","doc":"* `:return_destroyed?` (`t:boolean/0`) - If true, the destroyed record is included in the return result, e.g `{:ok, destroyed}` or `{:ok, destroyed, notifications}` The default value is `false`.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset\n\n* `:bulk_options` (`t:keyword/0`) - Options passed to `Ash.bulk_destroy`, if a query, list, or stream of inputs is provided.\n\n  * `:stream_batch_size` (`t:integer/0`) - Batch size to use if provided a query and the query must be streamed\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records if the `:stream` strategy is chosen. See the `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:authorize_query?` (`t:boolean/0`) - If a query is given, determines whether or not authorization is run on that query. The default value is `true`.\n\n  * `:strategy` - The strategy or strategies to enable. :stream is used in all cases if the data layer does not support atomics. The default value is `:atomic`.\n\n  * `:filter` (`t:term/0`) - A filter to apply to records. This is also applied to a stream of inputs.\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more.\n\n  * `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n  * `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n  * `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.\n\n    The query goes through many potential changes during a request, potentially adding\n    authorization filters, or replacing relationships for other data layers with their\n    corresponding ids. This option can be used to get the true query that was sent to\n    the data layer.\n\n    The default value is `false`.\n\n  * `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n  * `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n      a list of fields to fetch on a relationship, which allows for more\n      optimized data-fetching.\n\n      See `Ash.Query.load/2`.\n\n    The default value is `false`.\n\n  * `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. The default value is `:filter`.\n\n  * `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n  * `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n  * `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n  * `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. The default value is `:filter`.\n\n  * `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n  * `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n  * `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n  * `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n  * `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n    or `max_page_size` of the action, or 100.\n\n  * `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.\n\n    Potential elements:\n\n    `{:notification, notification}` - if `return_notifications?` is set to `true`\n    `{:ok, record}` - if `return_records?` is set to `true`\n    `{:error, error}` - an error that occurred. May be changeset or an individual error.\n\n    The default value is `false`.\n\n  * `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.\n\n    Only relevant if `return_stream?` is set to `true` and all other\n    `return_*?` options are set to `false`.\n\n    The default value is `false`.\n\n  * `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n    will be skipped. The default value is `true`.\n\n  * `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n    the results from each batch. This may be intensive for large bulk actions.\n\n    Notifications will be automatically sent unless `return_notifications?` is set to `true`.\n\n    The default value is `false`.\n\n  * `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.\n\n    Keep in mind:\n\n    `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n    *inside* the transaction if you choose `transaction: :all`.\n\n    The default value is `:batch`.\n\n  * `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set before validations and changes. The default value is `%{}`.","ref":"Ash.Resource.Interface.html#destroy_opts/0-options"},{"type":"function","title":"Ash.Resource.Interface.generic_action_opts/0","doc":"Options supported by code interfaces for generic actions","ref":"Ash.Resource.Interface.html#generic_action_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.generic_action_opts/0","doc":"* `:actor` (`t:term/0`) - The actor for handling `^actor/1` templates, supplied to calculation context.\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol. Will overwrite any actor, tenant or context provided. See `Ash.Context` for more.\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - The tenant, supplied to calculation context.\n\n* `:authorize?` (`t:boolean/0`) - Whether or not the request should be authorized.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer, provided to the calculation context.\n\n* `:context` (`t:map/0`) - Context to set on the action input\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set before validations and changes. The default value is `%{}`.","ref":"Ash.Resource.Interface.html#generic_action_opts/0-options"},{"type":"function","title":"Ash.Resource.Interface.get_opts/0","doc":"Options supported by code interfaces for read actions with `get?` or `get_by` set.","ref":"Ash.Resource.Interface.html#get_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.get_opts/0","doc":"* `:page` - Pagination options, see `Ash.read/2` for more.\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:query` - A query to seed the action with.\n\n* `:not_found_error?` (`t:boolean/0`) - Whether or not to return or raise a `NotFound` error or to return `nil` when a get? action/interface is called.\n\n* `:stream?` (`t:boolean/0`) - If true, a stream of the results will be returned The default value is `false`.\n\n* `:stream_options` (`t:keyword/0`) - Options passed to `Ash.stream!`, if `stream?: true` is given\n\n  * `:batch_size` (`t:integer/0`) - How many records to request in each query run. Defaults to the pagination limits on the resource, or 250.\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more.","ref":"Ash.Resource.Interface.html#get_opts/0-options"},{"type":"function","title":"Ash.Resource.Interface.read_opts/0","doc":"Options supported by code interfaces for read actions","ref":"Ash.Resource.Interface.html#read_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.read_opts/0","doc":"* `:page` - Pagination options, see `Ash.read/2` for more.\n\n* `:load` (`t:term/0`) - A load statement to add onto the query\n\n* `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n* `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n* `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.  \n  The query goes through many potential changes during a request, potentially adding\n  authorization filters, or replacing relationships for other data layers with their\n  corresponding ids. This option can be used to get the true query that was sent to\n  the data layer. The default value is `false`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n* `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n    a list of fields to fetch on a relationship, which allows for more\n    optimized data-fetching.  \n    See `Ash.Query.load/2`. The default value is `false`.\n\n* `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. Valid values are :filter, :error The default value is `:filter`.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:query` - A query to seed the action with.\n\n* `:not_found_error?` (`t:boolean/0`) - Whether or not to return or raise a `NotFound` error or to return `nil` when a get? action/interface is called.\n\n* `:stream?` (`t:boolean/0`) - If true, a stream of the results will be returned The default value is `false`.\n\n* `:stream_options` (`t:keyword/0`) - Options passed to `Ash.stream!`, if `stream?: true` is given\n\n  * `:batch_size` (`t:integer/0`) - How many records to request in each query run. Defaults to the pagination limits on the resource, or 250.\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more.","ref":"Ash.Resource.Interface.html#read_opts/0-options"},{"type":"type","title":"Ash.Resource.Interface.t/0","doc":"","ref":"Ash.Resource.Interface.html#t:t/0"},{"type":"function","title":"Ash.Resource.Interface.update_opts/0","doc":"Options supported by code interfaces for update actions","ref":"Ash.Resource.Interface.html#update_opts/0"},{"type":"function","title":"Options - Ash.Resource.Interface.update_opts/0","doc":"* `:params` (`t:map/0`) - Parameters to supply, ignored if the input is a changeset, only used when an identifier is given.\n\n* `:atomic_upgrade?` (`t:boolean/0`) - If true the action will be done atomically if it can (and is configured to do so), ignoring the in memory transformations and validations. You should not generally need to disable this. The default value is `true`.\n\n* `:timeout` (`t:timeout/0`) - A positive integer, or `:infinity`. If none is provided, the timeout configured on the domain is used.\n\n* `:tracer` (one or a list of module that adopts `Ash.Tracer`) - A tracer that implements the `Ash.Tracer` behaviour. See that module for more.\n\n* `:action` (`t:term/0`) - The action to use, either an Action struct or the name of the action\n\n* `:authorize?` (`t:boolean/0`) - If an actor option is provided (even if it is `nil`), authorization happens automatically. If not, this flag can be used to authorize with no user.\n\n* `:context` (`t:map/0`) - Context to set on the query, changeset, or input\n\n* `:tenant` (value that implements the `Ash.ToTenant` protocol) - A tenant to set on the query or changeset\n\n* `:actor` (`t:term/0`) - If an actor is provided, it will be used in conjunction with the authorizers of a resource to authorize access\n\n* `:scope` (`t:term/0`) - A value that implements the `Ash.Scope.ToOpts` protocol, for passing around actor/tenant/context in a single value. See `Ash.Scope.ToOpts` for more.\n\n* `:return_notifications?` (`t:boolean/0`) - Use this if you're running ash actions in your own transaction and you want to manually handle sending notifications.  \n  If a transaction is ongoing, and this is false, notifications will be discarded, otherwise\n  the return value is `{:ok, result, notifications}` (or `{:ok, notifications}`)  \n  To send notifications later, use `Ash.Notifier.notify(notifications)`. It sends any notifications\n  that can be sent, and returns the rest. The default value is `false`.\n\n* `:rollback_on_error?` (`t:boolean/0`) - Whether or not to rollback the transaction on error, if the resource is in a transaction.  \n  If the action has `transaction? false` this option has no effect. If an error is returned from the\n  data layer and the resource is in a transaction, the transaction is always rolled back, regardless. The default value is `true`.\n\n* `:notification_metadata` (`t:term/0`) - Metadata to be merged into the metadata field for all notifications sent from this operation. The default value is `%{}`.\n\n* `:skip_unknown_inputs` - A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.\n\n* `:load` (`t:term/0`) - A load statement to add onto the changeset\n\n* `:bulk_options` (`t:keyword/0`) - Options passed to `Ash.bulk_update`, if a query, list, or stream of inputs is provided.\n\n  * `:atomic_update` (`t:map/0`) - A map of atomic updates to apply. See `Ash.Changeset.atomic_update/3` for more.\n\n  * `:stream_batch_size` (`t:integer/0`) - Batch size to use if provided a query and the query must be streamed\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records if the `:stream` strategy is chosen. See the `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:authorize_query?` (`t:boolean/0`) - If a query is given, determines whether or not authorization is run on that query. The default value is `true`.\n\n  * `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n  * `:filter` (`t:term/0`) - A filter to apply to records. This is also applied to a stream of inputs.\n\n  * `:strategy` - The strategy or strategies to enable. :stream is used in all cases if the data layer does not support atomics. The default value is `[:atomic]`.\n\n  * `:allow_stream_with` - The 'worst' strategy allowed to be used to fetch records. See `Ash.stream!/2` docs for more. The default value is `:keyset`.\n\n  * `:stream_with` - The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more.\n\n  * `:max_concurrency` (`t:non_neg_integer/0`) - The maximum number of processes allowed to be started for parallel loading of relationships and calculations. Defaults to `System.schedulers_online() * 2`\n\n  * `:lock` (`t:term/0`) - A lock statement to add onto the query\n\n  * `:return_query?` (`t:boolean/0`) - If `true`, the query that was ultimately used is returned as a third tuple element.\n\n    The query goes through many potential changes during a request, potentially adding\n    authorization filters, or replacing relationships for other data layers with their\n    corresponding ids. This option can be used to get the true query that was sent to\n    the data layer.\n\n    The default value is `false`.\n\n  * `:reuse_values?` (`t:boolean/0`) - Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer. The default value is `false`.\n\n  * `:strict?` (`t:boolean/0`) - If set to true, only specified attributes will be loaded when passing\n      a list of fields to fetch on a relationship, which allows for more\n      optimized data-fetching.\n\n      See `Ash.Query.load/2`.\n\n    The default value is `false`.\n\n  * `:authorize_with` - If set to `:error`, instead of applying authorization filters as a filter, any records not matching the authorization filter will cause an error to be returned. The default value is `:filter`.\n\n  * `:read_action` (`t:atom/0`) - The action to use when building the read query.\n\n  * `:assume_casted?` (`t:boolean/0`) - Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting The default value is `false`.\n\n  * `:select` (list of `t:atom/0`) - A select statement to apply to records. Ignored if `return_records?` is not true.\n\n  * `:authorize_query_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_changeset_with` - If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error. Uses `authorize_with` if not set.\n\n  * `:authorize_with` - If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error. The default value is `:filter`.\n\n  * `:private_arguments` (`t:map/0`) - Private argument values to set on each changeset before validations and changes are run. The default value is `%{}`.\n\n  * `:sorted?` (`t:boolean/0`) - Whether or not to sort results by their input position, in cases where `return_records?: true` was provided. The default value is `false`.\n\n  * `:return_records?` (`t:boolean/0`) - Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts. The default value is `false`.\n\n  * `:return_errors?` (`t:boolean/0`) - Whether to return all errors that occur during the operation. Defaults to the value of `:bulk_actions_default_to_errors?` in your config, or `false` if not set. Returning all errors may be expensive for large inserts. The default value is `true`.\n\n  * `:batch_size` (`t:pos_integer/0`) - The number of records to include in each batch. Defaults to the `default_limit`\n    or `max_page_size` of the action, or 100.\n\n  * `:return_stream?` (`t:boolean/0`) - If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.\n\n    Potential elements:\n\n    `{:notification, notification}` - if `return_notifications?` is set to `true`\n    `{:ok, record}` - if `return_records?` is set to `true`\n    `{:error, error}` - an error that occurred. May be changeset or an individual error.\n\n    The default value is `false`.\n\n  * `:return_nothing?` (`t:boolean/0`) - Mutes warnings about returning nothing.\n\n    Only relevant if `return_stream?` is set to `true` and all other\n    `return_*?` options are set to `false`.\n\n    The default value is `false`.\n\n  * `:stop_on_error?` (`t:boolean/0`) - If true, the first encountered error will stop the action and be returned. Otherwise, errors\n    will be skipped. The default value is `true`.\n\n  * `:notify?` (`t:boolean/0`) - Whether or not to generate any notifications. If this is set to `true` then the data layer must return\n    the results from each batch. This may be intensive for large bulk actions.\n\n    Notifications will be automatically sent unless `return_notifications?` is set to `true`.\n\n    The default value is `false`.\n\n  * `:transaction` - Whether or not to wrap the entire execution in a transaction, each batch, or not at all.\n\n    Keep in mind:\n\n    `before_transaction` and `after_transaction` hooks attached to changesets will have to be run\n    *inside* the transaction if you choose `transaction: :all`.\n\n    The default value is `:batch`.\n\n  * `:max_concurrency` (`t:non_neg_integer/0`) - If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously The default value is `0`.\n\n* `:private_arguments` (`t:map/0`) - Private argument values to set before validations and changes. The default value is `%{}`.","ref":"Ash.Resource.Interface.html#update_opts/0-options"},{"type":"module","title":"Ash.Resource.Interface.CustomInput","doc":"Represents a custom input to a code interface","ref":"Ash.Resource.Interface.CustomInput.html"},{"type":"function","title":"Ash.Resource.Interface.CustomInput.schema/0","doc":"","ref":"Ash.Resource.Interface.CustomInput.html#schema/0"},{"type":"type","title":"Ash.Resource.Interface.CustomInput.t/0","doc":"","ref":"Ash.Resource.Interface.CustomInput.html#t:t/0"},{"type":"module","title":"Ash.Resource.Interface.CustomInput.Transform","doc":"Represents a transformation applied to a custom input","ref":"Ash.Resource.Interface.CustomInput.Transform.html"},{"type":"module","title":"Ash.Resource.Relationships","doc":"Types Ash relationships","ref":"Ash.Resource.Relationships.html"},{"type":"type","title":"Ash.Resource.Relationships.cardinality/0","doc":"","ref":"Ash.Resource.Relationships.html#t:cardinality/0"},{"type":"type","title":"Ash.Resource.Relationships.relationship/0","doc":"","ref":"Ash.Resource.Relationships.html#t:relationship/0"},{"type":"type","title":"Ash.Resource.Relationships.type/0","doc":"","ref":"Ash.Resource.Relationships.html#t:type/0"},{"type":"module","title":"Ash.Resource.Relationships.BelongsTo","doc":"Represents a belongs_to relationship on a resource","ref":"Ash.Resource.Relationships.BelongsTo.html"},{"type":"type","title":"Ash.Resource.Relationships.BelongsTo.t/0","doc":"","ref":"Ash.Resource.Relationships.BelongsTo.html#t:t/0"},{"type":"module","title":"Ash.Resource.Relationships.HasMany","doc":"Represents a has_many relationship on a resource","ref":"Ash.Resource.Relationships.HasMany.html"},{"type":"function","title":"Ash.Resource.Relationships.HasMany.manual/1","doc":"","ref":"Ash.Resource.Relationships.HasMany.html#manual/1"},{"type":"type","title":"Ash.Resource.Relationships.HasMany.t/0","doc":"","ref":"Ash.Resource.Relationships.HasMany.html#t:t/0"},{"type":"function","title":"Ash.Resource.Relationships.HasMany.transform/1","doc":"","ref":"Ash.Resource.Relationships.HasMany.html#transform/1"},{"type":"module","title":"Ash.Resource.Relationships.HasOne","doc":"Represents a has_one relationship on a resource","ref":"Ash.Resource.Relationships.HasOne.html"},{"type":"type","title":"Ash.Resource.Relationships.HasOne.t/0","doc":"","ref":"Ash.Resource.Relationships.HasOne.html#t:t/0"},{"type":"function","title":"Ash.Resource.Relationships.HasOne.transform/1","doc":"","ref":"Ash.Resource.Relationships.HasOne.html#transform/1"},{"type":"module","title":"Ash.Resource.Relationships.ManyToMany","doc":"Represents a many_to_many relationship on a resource","ref":"Ash.Resource.Relationships.ManyToMany.html"},{"type":"type","title":"Ash.Resource.Relationships.ManyToMany.t/0","doc":"","ref":"Ash.Resource.Relationships.ManyToMany.html#t:t/0"},{"type":"module","title":"Ash.Actions.BulkManualActionHelpers","doc":"Helper functions used for handling manual actions when used in bulk operations.","ref":"Ash.Actions.BulkManualActionHelpers.html"},{"type":"function","title":"Ash.Actions.BulkManualActionHelpers.process_bulk_results/7","doc":"Processes the results of a manual action's bulk function.","ref":"Ash.Actions.BulkManualActionHelpers.html#process_bulk_results/7"},{"type":"function","title":"Ash.Actions.BulkManualActionHelpers.process_non_bulk_result/3","doc":"Used when a manual action does not export a bulk version of the action.","ref":"Ash.Actions.BulkManualActionHelpers.html#process_non_bulk_result/3"},{"type":"module","title":"Ash.Actions.Read.AsyncLimiter","doc":"A utility for limiting the number of concurrent async operations\n\nBecause this is an optimization, we opt to run something synchronously\nif there is no async task available in the slot. The idea here is that\nthe *vast* majority of things we do async will be fast enough not to\nwarrant always waiting for an async slot to be free. We may add in some\nsmarter heuristics later (i.e choosing to wait for a task instead of\ndoing the work sync), but for now this is a good start.","ref":"Ash.Actions.Read.AsyncLimiter.html"},{"type":"function","title":"Ash.Actions.Read.AsyncLimiter.async_or_inline/4","doc":"","ref":"Ash.Actions.Read.AsyncLimiter.html#async_or_inline/4"},{"type":"function","title":"Ash.Actions.Read.AsyncLimiter.await_all/1","doc":"","ref":"Ash.Actions.Read.AsyncLimiter.html#await_all/1"},{"type":"function","title":"Ash.Actions.Read.AsyncLimiter.await_at_least_one/1","doc":"","ref":"Ash.Actions.Read.AsyncLimiter.html#await_at_least_one/1"},{"type":"function","title":"Ash.Actions.Read.AsyncLimiter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ash.Actions.Read.AsyncLimiter.html#child_spec/1"},{"type":"function","title":"Ash.Actions.Read.AsyncLimiter.start_link/1","doc":"","ref":"Ash.Actions.Read.AsyncLimiter.html#start_link/1"},{"type":"module","title":"Ash.Can","doc":"Contains the Ash.can function logic.","ref":"Ash.Can.html"},{"type":"function","title":"Ash.Can.can/4","doc":"Returns a an ok tuple if the actor can perform the action, query, or changeset,\nan error tuple if an error happens, and a ok tuple with maybe if maybe is set to true\nor not set.\n\nYou should prefer to use `Ash.can/3` over this module, directly.\n\nNote: `is_maybe` is set to `true`, if not set.","ref":"Ash.Can.html#can/4"},{"type":"function","title":"Ash.Can.can?/4","doc":"Returns whether an actor can perform an action, query, or changeset.\n\nYou should prefer to use `Ash.can?/3` over this module, directly.\n\nCan raise an exception if return_forbidden_error is truthy in opts or there's an error.","ref":"Ash.Can.html#can?/4"},{"type":"type","title":"Ash.Can.subject/0","doc":"","ref":"Ash.Can.html#t:subject/0"},{"type":"module","title":"Ash.Changeset.OriginalDataNotAvailable","doc":"A value placed in changeset.data to indicate that the original data is not available","ref":"Ash.Changeset.OriginalDataNotAvailable.html"},{"type":"type","title":"Ash.Changeset.OriginalDataNotAvailable.t/0","doc":"Placeholder struct used when original data is not available.\n\nUsed in atomic operations where the original record data cannot be loaded.","ref":"Ash.Changeset.OriginalDataNotAvailable.html#t:t/0"},{"type":"module","title":"Ash.Context","doc":"Functions for working with the context provided to various callbacks in Ash.","ref":"Ash.Context.html"},{"type":"type","title":"Ash.Context.context_keyword_list/0","doc":"","ref":"Ash.Context.html#t:context_keyword_list/0"},{"type":"function","title":"Ash.Context.to_opts/2","doc":"Copies keys from the given context map into a keyword list. Does *not* copy the `:domain` key.\n\nKeys copied:\n\n* `:actor`\n* `:authorize?`\n* `:tracer`\n* `:tenant`\n* `context[:shared]` -> `:context`","ref":"Ash.Context.html#to_opts/2"},{"type":"behaviour","title":"Ash.CustomExpression","doc":"A module for defining custom functions that can be called in Ash expressions.\n\nFor example:\n\n```elixir\ndefmodule MyApp.Expressions.LevenshteinDistance do\n  use Ash.CustomExpression,\n    name: :levenshtein,\n    arguments: [\n      [:string, :string]\n    ]\n\n  def expression(AshPostgres.DataLayer, [left, right]) do\n    {:ok, expr(fragment(\"levenshtein(?, ?)\", ^left, ^right))}\n  end\n\n  # It is good practice to always define an expression for `Ash.DataLayer.Simple`,\n  # as that is what Ash will use to run your custom expression in Elixir.\n  # This allows us to completely avoid communicating with the database in some cases.\n\n  # Always use a fragment like this to evaluate code in simple data layers. The value you\n  # get here for arguments may be expressions at this point.\n  def expression(data_layer, [left, right]) when data_layer in [\n    Ash.DataLayer.Ets,\n    Ash.DataLayer.Simple\n  ] do\n    {:ok, expr(fragment(&__MODULE__.levenshtein/2, ^left, ^right))}\n  end\n\n  # always define this fallback clause as well\n  def expression(_data_layer, _args), do: :unknown\n\n  @doc \"Computes the levenshtein distance of two strings\"\n  def levenshtein(left, right) do\n    # ......\n  end\nend\n```","ref":"Ash.CustomExpression.html"},{"type":"behaviour","title":"Options - Ash.CustomExpression","doc":"- `name` - The name of the custom expression. This is the name that will be used in Ash expressions.\n- `arguments` - A list of lists of types that the custom expression accepts. Each list represents a set of arguments that the custom expression can accept.\n- `predicate?` - Whether this expression can be exposed as a predicate in filter interfaces. Defaults to `false`.","ref":"Ash.CustomExpression.html#module-options"},{"type":"behaviour","title":"Registering Your Expression - Ash.CustomExpression","doc":"Use compile-time configuration to register your custom expressions\n\n```elixir\nconfig :ash, :custom_expressions, [MyApp.Expressions.LevenshteinDistance]\n```","ref":"Ash.CustomExpression.html#module-registering-your-expression"},{"type":"callback","title":"Ash.CustomExpression.arguments/0","doc":"","ref":"Ash.CustomExpression.html#c:arguments/0"},{"type":"callback","title":"Ash.CustomExpression.expression/2","doc":"","ref":"Ash.CustomExpression.html#c:expression/2"},{"type":"callback","title":"Ash.CustomExpression.name/0","doc":"","ref":"Ash.CustomExpression.html#c:name/0"},{"type":"behaviour","title":"Ash.DataLayer","doc":"The behaviour for backing resource actions with persistence layers.","ref":"Ash.DataLayer.html"},{"type":"callback","title":"Ash.DataLayer.add_aggregate/3","doc":"","ref":"Ash.DataLayer.html#c:add_aggregate/3"},{"type":"function","title":"Ash.DataLayer.add_aggregates/3","doc":"","ref":"Ash.DataLayer.html#add_aggregates/3"},{"type":"callback","title":"Ash.DataLayer.add_aggregates/3","doc":"","ref":"Ash.DataLayer.html#c:add_aggregates/3"},{"type":"callback","title":"Ash.DataLayer.add_calculation/4","doc":"","ref":"Ash.DataLayer.html#c:add_calculation/4"},{"type":"function","title":"Ash.DataLayer.add_calculations/3","doc":"","ref":"Ash.DataLayer.html#add_calculations/3"},{"type":"callback","title":"Ash.DataLayer.add_calculations/3","doc":"","ref":"Ash.DataLayer.html#c:add_calculations/3"},{"type":"function","title":"Ash.DataLayer.bulk_create/3","doc":"","ref":"Ash.DataLayer.html#bulk_create/3"},{"type":"callback","title":"Ash.DataLayer.bulk_create/3","doc":"","ref":"Ash.DataLayer.html#c:bulk_create/3"},{"type":"type","title":"Ash.DataLayer.bulk_create_options/0","doc":"","ref":"Ash.DataLayer.html#t:bulk_create_options/0"},{"type":"type","title":"Ash.DataLayer.bulk_update_options/0","doc":"","ref":"Ash.DataLayer.html#t:bulk_update_options/0"},{"type":"function","title":"Ash.DataLayer.calculate/3","doc":"","ref":"Ash.DataLayer.html#calculate/3"},{"type":"callback","title":"Ash.DataLayer.calculate/3","doc":"","ref":"Ash.DataLayer.html#c:calculate/3"},{"type":"function","title":"Ash.DataLayer.can?/2","doc":"","ref":"Ash.DataLayer.html#can?/2"},{"type":"callback","title":"Ash.DataLayer.can?/2","doc":"","ref":"Ash.DataLayer.html#c:can?/2"},{"type":"function","title":"Ash.DataLayer.combination_of/3","doc":"","ref":"Ash.DataLayer.html#combination_of/3"},{"type":"callback","title":"Ash.DataLayer.combination_of/3","doc":"","ref":"Ash.DataLayer.html#c:combination_of/3"},{"type":"type","title":"Ash.DataLayer.combination_type/0","doc":"","ref":"Ash.DataLayer.html#t:combination_type/0"},{"type":"function","title":"Ash.DataLayer.create/2","doc":"","ref":"Ash.DataLayer.html#create/2"},{"type":"callback","title":"Ash.DataLayer.create/2","doc":"","ref":"Ash.DataLayer.html#c:create/2"},{"type":"function","title":"Ash.DataLayer.data_layer/1","doc":"The data layer of the resource, or nil if it does not have one","ref":"Ash.DataLayer.html#data_layer/1"},{"type":"function","title":"Ash.DataLayer.data_layer_can?/2","doc":"Whether or not the data layer supports a specific feature","ref":"Ash.DataLayer.html#data_layer_can?/2"},{"type":"function","title":"Ash.DataLayer.data_layer_functions/1","doc":"Custom functions supported by the data layer of the resource","ref":"Ash.DataLayer.html#data_layer_functions/1"},{"type":"type","title":"Ash.DataLayer.data_layer_query/0","doc":"","ref":"Ash.DataLayer.html#t:data_layer_query/0"},{"type":"function","title":"Ash.DataLayer.destroy/2","doc":"","ref":"Ash.DataLayer.html#destroy/2"},{"type":"callback","title":"Ash.DataLayer.destroy/2","doc":"","ref":"Ash.DataLayer.html#c:destroy/2"},{"type":"function","title":"Ash.DataLayer.destroy_query/3","doc":"","ref":"Ash.DataLayer.html#destroy_query/3"},{"type":"callback","title":"Ash.DataLayer.destroy_query/4","doc":"","ref":"Ash.DataLayer.html#c:destroy_query/4"},{"type":"function","title":"Ash.DataLayer.distinct/3","doc":"","ref":"Ash.DataLayer.html#distinct/3"},{"type":"callback","title":"Ash.DataLayer.distinct/3","doc":"","ref":"Ash.DataLayer.html#c:distinct/3"},{"type":"function","title":"Ash.DataLayer.distinct_sort/3","doc":"","ref":"Ash.DataLayer.html#distinct_sort/3"},{"type":"callback","title":"Ash.DataLayer.distinct_sort/3","doc":"","ref":"Ash.DataLayer.html#c:distinct_sort/3"},{"type":"type","title":"Ash.DataLayer.feature/0","doc":"","ref":"Ash.DataLayer.html#t:feature/0"},{"type":"function","title":"Ash.DataLayer.filter/3","doc":"","ref":"Ash.DataLayer.html#filter/3"},{"type":"callback","title":"Ash.DataLayer.filter/3","doc":"","ref":"Ash.DataLayer.html#c:filter/3"},{"type":"function","title":"Ash.DataLayer.functions/1","doc":"","ref":"Ash.DataLayer.html#functions/1"},{"type":"callback","title":"Ash.DataLayer.functions/1","doc":"","ref":"Ash.DataLayer.html#c:functions/1"},{"type":"function","title":"Ash.DataLayer.in_transaction?/1","doc":"","ref":"Ash.DataLayer.html#in_transaction?/1"},{"type":"callback","title":"Ash.DataLayer.in_transaction?/1","doc":"","ref":"Ash.DataLayer.html#c:in_transaction?/1"},{"type":"type","title":"Ash.DataLayer.lateral_join_link/0","doc":"","ref":"Ash.DataLayer.html#t:lateral_join_link/0"},{"type":"function","title":"Ash.DataLayer.limit/3","doc":"","ref":"Ash.DataLayer.html#limit/3"},{"type":"callback","title":"Ash.DataLayer.limit/3","doc":"","ref":"Ash.DataLayer.html#c:limit/3"},{"type":"function","title":"Ash.DataLayer.lock/3","doc":"","ref":"Ash.DataLayer.html#lock/3"},{"type":"callback","title":"Ash.DataLayer.lock/3","doc":"","ref":"Ash.DataLayer.html#c:lock/3"},{"type":"type","title":"Ash.DataLayer.lock_type/0","doc":"","ref":"Ash.DataLayer.html#t:lock_type/0"},{"type":"function","title":"Ash.DataLayer.offset/3","doc":"","ref":"Ash.DataLayer.html#offset/3"},{"type":"callback","title":"Ash.DataLayer.offset/3","doc":"","ref":"Ash.DataLayer.html#c:offset/3"},{"type":"callback","title":"Ash.DataLayer.prefer_lateral_join_for_many_to_many?/0","doc":"","ref":"Ash.DataLayer.html#c:prefer_lateral_join_for_many_to_many?/0"},{"type":"function","title":"Ash.DataLayer.prefer_lateral_join_for_many_to_many?/1","doc":"Whether or not lateral joins should be used for many to many relationships by default","ref":"Ash.DataLayer.html#prefer_lateral_join_for_many_to_many?/1"},{"type":"function","title":"Ash.DataLayer.prefer_transaction?/1","doc":"","ref":"Ash.DataLayer.html#prefer_transaction?/1"},{"type":"callback","title":"Ash.DataLayer.prefer_transaction?/1","doc":"","ref":"Ash.DataLayer.html#c:prefer_transaction?/1"},{"type":"function","title":"Ash.DataLayer.prefer_transaction_for_atomic_updates?/1","doc":"","ref":"Ash.DataLayer.html#prefer_transaction_for_atomic_updates?/1"},{"type":"callback","title":"Ash.DataLayer.prefer_transaction_for_atomic_updates?/1","doc":"","ref":"Ash.DataLayer.html#c:prefer_transaction_for_atomic_updates?/1"},{"type":"function","title":"Ash.DataLayer.resource_to_query/2","doc":"","ref":"Ash.DataLayer.html#resource_to_query/2"},{"type":"callback","title":"Ash.DataLayer.resource_to_query/2","doc":"","ref":"Ash.DataLayer.html#c:resource_to_query/2"},{"type":"function","title":"Ash.DataLayer.return_query/2","doc":"","ref":"Ash.DataLayer.html#return_query/2"},{"type":"callback","title":"Ash.DataLayer.return_query/2","doc":"","ref":"Ash.DataLayer.html#c:return_query/2"},{"type":"function","title":"Ash.DataLayer.rollback/2","doc":"Rolls back the current transaction","ref":"Ash.DataLayer.html#rollback/2"},{"type":"callback","title":"Ash.DataLayer.rollback/2","doc":"","ref":"Ash.DataLayer.html#c:rollback/2"},{"type":"function","title":"Ash.DataLayer.run_aggregate_query/3","doc":"","ref":"Ash.DataLayer.html#run_aggregate_query/3"},{"type":"callback","title":"Ash.DataLayer.run_aggregate_query/3","doc":"","ref":"Ash.DataLayer.html#c:run_aggregate_query/3"},{"type":"function","title":"Ash.DataLayer.run_aggregate_query_with_lateral_join/5","doc":"","ref":"Ash.DataLayer.html#run_aggregate_query_with_lateral_join/5"},{"type":"callback","title":"Ash.DataLayer.run_aggregate_query_with_lateral_join/5","doc":"","ref":"Ash.DataLayer.html#c:run_aggregate_query_with_lateral_join/5"},{"type":"function","title":"Ash.DataLayer.run_query/2","doc":"","ref":"Ash.DataLayer.html#run_query/2"},{"type":"callback","title":"Ash.DataLayer.run_query/2","doc":"","ref":"Ash.DataLayer.html#c:run_query/2"},{"type":"function","title":"Ash.DataLayer.run_query_with_lateral_join/4","doc":"","ref":"Ash.DataLayer.html#run_query_with_lateral_join/4"},{"type":"callback","title":"Ash.DataLayer.run_query_with_lateral_join/4","doc":"","ref":"Ash.DataLayer.html#c:run_query_with_lateral_join/4"},{"type":"function","title":"Ash.DataLayer.select/3","doc":"","ref":"Ash.DataLayer.html#select/3"},{"type":"callback","title":"Ash.DataLayer.select/3","doc":"","ref":"Ash.DataLayer.html#c:select/3"},{"type":"function","title":"Ash.DataLayer.set_context/3","doc":"","ref":"Ash.DataLayer.html#set_context/3"},{"type":"callback","title":"Ash.DataLayer.set_context/3","doc":"","ref":"Ash.DataLayer.html#c:set_context/3"},{"type":"function","title":"Ash.DataLayer.set_tenant/3","doc":"","ref":"Ash.DataLayer.html#set_tenant/3"},{"type":"callback","title":"Ash.DataLayer.set_tenant/3","doc":"","ref":"Ash.DataLayer.html#c:set_tenant/3"},{"type":"function","title":"Ash.DataLayer.sort/3","doc":"","ref":"Ash.DataLayer.html#sort/3"},{"type":"callback","title":"Ash.DataLayer.sort/3","doc":"","ref":"Ash.DataLayer.html#c:sort/3"},{"type":"function","title":"Ash.DataLayer.source/1","doc":"","ref":"Ash.DataLayer.html#source/1"},{"type":"callback","title":"Ash.DataLayer.source/1","doc":"","ref":"Ash.DataLayer.html#c:source/1"},{"type":"type","title":"Ash.DataLayer.t/0","doc":"","ref":"Ash.DataLayer.html#t:t/0"},{"type":"function","title":"Ash.DataLayer.transaction/4","doc":"Wraps the execution of the function in a transaction with the resource's data_layer","ref":"Ash.DataLayer.html#transaction/4"},{"type":"callback","title":"Ash.DataLayer.transaction/4","doc":"","ref":"Ash.DataLayer.html#c:transaction/4"},{"type":"type","title":"Ash.DataLayer.transaction_reason/0","doc":"","ref":"Ash.DataLayer.html#t:transaction_reason/0"},{"type":"function","title":"Ash.DataLayer.transform_query/1","doc":"","ref":"Ash.DataLayer.html#transform_query/1"},{"type":"callback","title":"Ash.DataLayer.transform_query/1","doc":"","ref":"Ash.DataLayer.html#c:transform_query/1"},{"type":"function","title":"Ash.DataLayer.update/2","doc":"","ref":"Ash.DataLayer.html#update/2"},{"type":"callback","title":"Ash.DataLayer.update/2","doc":"","ref":"Ash.DataLayer.html#c:update/2"},{"type":"function","title":"Ash.DataLayer.update_query/3","doc":"","ref":"Ash.DataLayer.html#update_query/3"},{"type":"callback","title":"Ash.DataLayer.update_query/4","doc":"","ref":"Ash.DataLayer.html#c:update_query/4"},{"type":"callback","title":"Ash.DataLayer.upsert/3","doc":"","ref":"Ash.DataLayer.html#c:upsert/3"},{"type":"function","title":"Ash.DataLayer.upsert/4","doc":"","ref":"Ash.DataLayer.html#upsert/4"},{"type":"callback","title":"Ash.DataLayer.upsert/4","doc":"","ref":"Ash.DataLayer.html#c:upsert/4"},{"type":"module","title":"Ash.Domain.Dsl.ResourceReference","doc":"A resource reference in a domain","ref":"Ash.Domain.Dsl.ResourceReference.html"},{"type":"type","title":"Ash.Domain.Dsl.ResourceReference.t/0","doc":"","ref":"Ash.Domain.Dsl.ResourceReference.html#t:t/0"},{"type":"module","title":"Ash.Domain.Igniter","doc":"Codemods for working with Ash.Domain modules","ref":"Ash.Domain.Igniter.html"},{"type":"function","title":"Ash.Domain.Igniter.add_new_code_interface/5","doc":"Adds a code interface if not present to the given resource on the given domain","ref":"Ash.Domain.Igniter.html#add_new_code_interface/5"},{"type":"function","title":"Ash.Domain.Igniter.add_resource_reference/3","doc":"Adds a resource reference to a domain's `resources` block","ref":"Ash.Domain.Igniter.html#add_resource_reference/3"},{"type":"function","title":"Ash.Domain.Igniter.list_domains/1","doc":"List all domain modules found in the project","ref":"Ash.Domain.Igniter.html#list_domains/1"},{"type":"function","title":"Ash.Domain.Igniter.remove_resource_reference/3","doc":"Removes a resource reference from a domain's `resources` block","ref":"Ash.Domain.Igniter.html#remove_resource_reference/3"},{"type":"behaviour","title":"Ash.Extension","doc":"A behavior of additional callbacks that extensions can implement, specific to Ash.\n\nIt is not necessary to adopt this behavior, but it is recommended to do so if you want to define these\nfunctions on your extension. These functions are invoked when their relevant Mix task is run.","ref":"Ash.Extension.html"},{"type":"type","title":"Ash.Extension.argv/0","doc":"","ref":"Ash.Extension.html#t:argv/0"},{"type":"callback","title":"Ash.Extension.codegen/1","doc":"","ref":"Ash.Extension.html#c:codegen/1"},{"type":"type","title":"Ash.Extension.igniter/0","doc":"","ref":"Ash.Extension.html#t:igniter/0"},{"type":"callback","title":"Ash.Extension.install/5","doc":"","ref":"Ash.Extension.html#c:install/5"},{"type":"callback","title":"Ash.Extension.migrate/1","doc":"","ref":"Ash.Extension.html#c:migrate/1"},{"type":"callback","title":"Ash.Extension.reset/1","doc":"","ref":"Ash.Extension.html#c:reset/1"},{"type":"callback","title":"Ash.Extension.rollback/1","doc":"","ref":"Ash.Extension.html#c:rollback/1"},{"type":"callback","title":"Ash.Extension.setup/1","doc":"","ref":"Ash.Extension.html#c:setup/1"},{"type":"callback","title":"Ash.Extension.tear_down/1","doc":"","ref":"Ash.Extension.html#c:tear_down/1"},{"type":"module","title":"Ash.Igniter","doc":"Codemods and utilities for working with Ash & Igniter","ref":"Ash.Igniter.html"},{"type":"function","title":"Ash.Igniter.codegen/2","doc":"Adds a codegen task, or updates the name to be ` _and_name`","ref":"Ash.Igniter.html#codegen/2"},{"type":"module","title":"Ash.Info","doc":"General introspection helpers for Ash applications.\n\nUse `Ash.Info.mermaid_overview(otp_app)` to generate a mermaid chart of the application's domains and resources.\nThis offers a high level view, but will not include information from extensions. Use `resource_report/2`\nfor a detailed report of a resource.\n\nUse `Ash.Info.mermaid_overview(otp_app, type: :entity_relationship)` to get a simplified entity relationship diagram.\n\nUse `Ash.Info.domains_and_resources(otp_app)` to get a map where the keys are the domains, and the value is the list\nof resources for that domain.","ref":"Ash.Info.html"},{"type":"function","title":"Ash.Info.domains/1","doc":"","ref":"Ash.Info.html#domains/1"},{"type":"function","title":"Ash.Info.domains_and_resources/1","doc":"Returns a map where the keys are the domains, and the value is the list of resources for that domain.","ref":"Ash.Info.html#domains_and_resources/1"},{"type":"function","title":"Ash.Info.mermaid_overview/2","doc":"Generate a mermaid chart of the application's domains and resources.\n\nThis offers a high level view, but will not include information from extensions.","ref":"Ash.Info.html#mermaid_overview/2"},{"type":"module","title":"Ash.Policy.PolicyGroup","doc":"Represents a policy group on an Ash.Resource","ref":"Ash.Policy.PolicyGroup.html"},{"type":"module","title":"Ash.Query.Combination","doc":"Represents one combination in a combination of queries.","ref":"Ash.Query.Combination.html"},{"type":"function","title":"Ash.Query.Combination.base/1","doc":"The initial combination of a combined query.","ref":"Ash.Query.Combination.html#base/1"},{"type":"function","title":"Ash.Query.Combination.except/1","doc":"Removes all rows that are present in the previous combinations *and* this one.","ref":"Ash.Query.Combination.html#except/1"},{"type":"function","title":"Ash.Query.Combination.intersect/1","doc":"Intersects the query with the previous combinations, keeping only rows that are present in the previous combinations and this one.","ref":"Ash.Query.Combination.html#intersect/1"},{"type":"type","title":"Ash.Query.Combination.t/0","doc":"","ref":"Ash.Query.Combination.html#t:t/0"},{"type":"function","title":"Ash.Query.Combination.union/1","doc":"Unions the query with the previous combinations, discarding duplicates when all fields are equal.","ref":"Ash.Query.Combination.html#union/1"},{"type":"function","title":"Ash.Query.Combination.union_all/1","doc":"Unions the query with the previous combinations, keeping all rows.","ref":"Ash.Query.Combination.html#union_all/1"},{"type":"module","title":"Ash.Query.UpsertConflict","doc":"Returns the conflicting new information.","ref":"Ash.Query.UpsertConflict.html"},{"type":"function","title":"Ash.Query.UpsertConflict.new/1","doc":"","ref":"Ash.Query.UpsertConflict.html#new/1"},{"type":"module","title":"Ash.Reactor.ActionStep","doc":"The Reactor step which is used to execute generic actions.","ref":"Ash.Reactor.ActionStep.html"},{"type":"module","title":"Ash.Reactor.AshStep","doc":"A reactor step which runs a step-module or an anonymous function, and enqueues any returned\nnotifications before returning.\n\nThe following return values are supported: `{:ok, result}`, `{:ok, result, notifications}`,\n`{:ok, result, notifications, new_steps}`\n\nExample:\n\n```elixir\nash_step :maybe_update_post do\n  run fn %{post: post, new_amount_of_likes: new_amount_of_likes}, ctx ->\n    opts = Ash.Context.to_opts(ctx, return_notifications?: true)\n    if post.amount_of_likes != new_amount_of_likes do\n      Post.update(post, %{amount_of_likes: new_amount_of_likes}, opts)\n    else\n      {:ok, post}\n    end\nend\n```","ref":"Ash.Reactor.AshStep.html"},{"type":"module","title":"Options - Ash.Reactor.AshStep","doc":"* `run` - a one or two arity function or MFA which will be called as the run\n  function of the step.\n* `compensate` - a one to three arity function or MFA which will be called as\n  the compensate function of the step.  Optional.\n* `undo` - a one to three arity function or MFA which will be called as the\n  undo function of this step.  Optional.","ref":"Ash.Reactor.AshStep.html#module-options"},{"type":"module","title":"Ash.Reactor.BulkCreateStep","doc":"The Reactor step which is used to execute create actions in bulk.","ref":"Ash.Reactor.BulkCreateStep.html"},{"type":"module","title":"Ash.Reactor.BulkUpdateStep","doc":"The Reactor step which is used to execute update actions in bulk.","ref":"Ash.Reactor.BulkUpdateStep.html"},{"type":"module","title":"Ash.Reactor.ChangeStep","doc":"The Reactor step which is used to execute change steps.","ref":"Ash.Reactor.ChangeStep.html"},{"type":"module","title":"Ash.Reactor.CreateStep","doc":"The Reactor step which is used to execute create actions.","ref":"Ash.Reactor.CreateStep.html"},{"type":"module","title":"Ash.Reactor.DestroyStep","doc":"The Reactor step which is used to execute update actions.","ref":"Ash.Reactor.DestroyStep.html"},{"type":"module","title":"Ash.Reactor.Dsl.Action","doc":"The `action` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Action.html"},{"type":"type","title":"Ash.Reactor.Dsl.Action.t/0","doc":"","ref":"Ash.Reactor.Dsl.Action.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.ActionLoad","doc":"Add a load statement to an action.","ref":"Ash.Reactor.Dsl.ActionLoad.html"},{"type":"type","title":"Ash.Reactor.Dsl.ActionLoad.t/0","doc":"","ref":"Ash.Reactor.Dsl.ActionLoad.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.ActionTransformer","doc":"Responsible for transforming actions.","ref":"Ash.Reactor.Dsl.ActionTransformer.html"},{"type":"function","title":"Ash.Reactor.Dsl.ActionTransformer.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Reactor.Dsl.ActionTransformer.html#after?/1"},{"type":"function","title":"Ash.Reactor.Dsl.ActionTransformer.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Reactor.Dsl.ActionTransformer.html#after_compile?/0"},{"type":"module","title":"Ash.Reactor.Dsl.Actor","doc":"Specify the actor used to execute an action.","ref":"Ash.Reactor.Dsl.Actor.html"},{"type":"type","title":"Ash.Reactor.Dsl.Actor.t/0","doc":"","ref":"Ash.Reactor.Dsl.Actor.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.AshStep","doc":"The ash_step DSL module.\n\nSee `d:Reactor.step`.","ref":"Ash.Reactor.Dsl.AshStep.html"},{"type":"type","title":"Ash.Reactor.Dsl.AshStep.t/0","doc":"","ref":"Ash.Reactor.Dsl.AshStep.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.BulkCreate","doc":"The `bulk_create` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.BulkCreate.html"},{"type":"type","title":"Ash.Reactor.Dsl.BulkCreate.t/0","doc":"","ref":"Ash.Reactor.Dsl.BulkCreate.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.BulkUpdate","doc":"The `bulk_update` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.BulkUpdate.html"},{"type":"type","title":"Ash.Reactor.Dsl.BulkUpdate.t/0","doc":"","ref":"Ash.Reactor.Dsl.BulkUpdate.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Change","doc":"The `change` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Change.html"},{"type":"type","title":"Ash.Reactor.Dsl.Change.t/0","doc":"","ref":"Ash.Reactor.Dsl.Change.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Context","doc":"Specify context to merge into an action's context.","ref":"Ash.Reactor.Dsl.Context.html"},{"type":"type","title":"Ash.Reactor.Dsl.Context.t/0","doc":"","ref":"Ash.Reactor.Dsl.Context.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Create","doc":"The `create` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Create.html"},{"type":"type","title":"Ash.Reactor.Dsl.Create.t/0","doc":"","ref":"Ash.Reactor.Dsl.Create.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Destroy","doc":"The `destroy` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Destroy.html"},{"type":"type","title":"Ash.Reactor.Dsl.Destroy.t/0","doc":"","ref":"Ash.Reactor.Dsl.Destroy.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Inputs","doc":"The `inputs` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Inputs.html"},{"type":"type","title":"Ash.Reactor.Dsl.Inputs.t/0","doc":"","ref":"Ash.Reactor.Dsl.Inputs.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Load","doc":"The `load` step entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Load.html"},{"type":"type","title":"Ash.Reactor.Dsl.Load.t/0","doc":"","ref":"Ash.Reactor.Dsl.Load.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.MiddlewareTransformer","doc":"Ensures that the required middlewares are added to the Reactor.","ref":"Ash.Reactor.Dsl.MiddlewareTransformer.html"},{"type":"function","title":"Ash.Reactor.Dsl.MiddlewareTransformer.after?/1","doc":"Callback implementation for  Spark.Dsl.Transformer.after?/1 .","ref":"Ash.Reactor.Dsl.MiddlewareTransformer.html#after?/1"},{"type":"function","title":"Ash.Reactor.Dsl.MiddlewareTransformer.after_compile?/0","doc":"Callback implementation for  Spark.Dsl.Transformer.after_compile?/0 .","ref":"Ash.Reactor.Dsl.MiddlewareTransformer.html#after_compile?/0"},{"type":"module","title":"Ash.Reactor.Dsl.Read","doc":"The `read` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Read.html"},{"type":"type","title":"Ash.Reactor.Dsl.Read.t/0","doc":"","ref":"Ash.Reactor.Dsl.Read.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.ReadOne","doc":"The `read_one` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.ReadOne.html"},{"type":"type","title":"Ash.Reactor.Dsl.ReadOne.t/0","doc":"","ref":"Ash.Reactor.Dsl.ReadOne.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Tenant","doc":"Specify the tenant used to execute an action.","ref":"Ash.Reactor.Dsl.Tenant.html"},{"type":"type","title":"Ash.Reactor.Dsl.Tenant.t/0","doc":"","ref":"Ash.Reactor.Dsl.Tenant.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Transaction","doc":"The `transaction` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Transaction.html"},{"type":"type","title":"Ash.Reactor.Dsl.Transaction.t/0","doc":"","ref":"Ash.Reactor.Dsl.Transaction.html#t:t/0"},{"type":"module","title":"Ash.Reactor.Dsl.Update","doc":"The `update` entity for the `Ash.Reactor` reactor extension.","ref":"Ash.Reactor.Dsl.Update.html"},{"type":"type","title":"Ash.Reactor.Dsl.Update.t/0","doc":"","ref":"Ash.Reactor.Dsl.Update.html#t:t/0"},{"type":"module","title":"Ash.Reactor.LoadStep","doc":"The Reactor step which is used to execute load steps.","ref":"Ash.Reactor.LoadStep.html"},{"type":"function","title":"Ash.Reactor.LoadStep.run/3","doc":"Callback implementation for  Reactor.Step.run/3 .","ref":"Ash.Reactor.LoadStep.html#run/3"},{"type":"module","title":"Ash.Reactor.MergeInputsStep","doc":"A custom step which merges any number of `inputs` results into a single map.","ref":"Ash.Reactor.MergeInputsStep.html"},{"type":"module","title":"Ash.Reactor.Notifications","doc":"Reactor middleware used to collect and emit notifications upon successful\ncompletion of the Reactor.","ref":"Ash.Reactor.Notifications.html"},{"type":"function","title":"Ash.Reactor.Notifications.complete/2","doc":"When the reactor completes successfully, publish any queued notifications.","ref":"Ash.Reactor.Notifications.html#complete/2"},{"type":"function","title":"Ash.Reactor.Notifications.enqueue_notifications/2","doc":"Add notifications to the queue to be published on reactor success.","ref":"Ash.Reactor.Notifications.html#enqueue_notifications/2"},{"type":"function","title":"Ash.Reactor.Notifications.error/2","doc":"When the reactor fails, discard any queued notifications.","ref":"Ash.Reactor.Notifications.html#error/2"},{"type":"function","title":"Ash.Reactor.Notifications.halt/1","doc":"When halting the reactor, store any queued notifications in the context for\neventual resumption.","ref":"Ash.Reactor.Notifications.html#halt/1"},{"type":"function","title":"Ash.Reactor.Notifications.init/1","doc":"When starting a reactor, start an agent to act as a temporary store of\nnotifications.","ref":"Ash.Reactor.Notifications.html#init/1"},{"type":"function","title":"Ash.Reactor.Notifications.publish/2","doc":"Dispatch notifications.","ref":"Ash.Reactor.Notifications.html#publish/2"},{"type":"module","title":"Ash.Reactor.ReadOneStep","doc":"The Reactor step which is used to execute get actions.","ref":"Ash.Reactor.ReadOneStep.html"},{"type":"function","title":"Ash.Reactor.ReadOneStep.run/3","doc":"Callback implementation for  Reactor.Step.run/3 .","ref":"Ash.Reactor.ReadOneStep.html#run/3"},{"type":"module","title":"Ash.Reactor.ReadStep","doc":"The Reactor step which is used to execute read actions.","ref":"Ash.Reactor.ReadStep.html"},{"type":"function","title":"Ash.Reactor.ReadStep.run/3","doc":"Callback implementation for  Reactor.Step.run/3 .","ref":"Ash.Reactor.ReadStep.html#run/3"},{"type":"module","title":"Ash.Reactor.Tracer","doc":"Reactor middleware which threads Ash's tracing information through to new\nprocesses spawned by Reactor.","ref":"Ash.Reactor.Tracer.html"},{"type":"module","title":"Ash.Reactor.TransactionStep","doc":"The Reactor step which is used to wrap other steps in an Ash data layer\ntransaction.","ref":"Ash.Reactor.TransactionStep.html"},{"type":"function","title":"Ash.Reactor.TransactionStep.run/3","doc":"Callback implementation for  Reactor.Step.run/3 .","ref":"Ash.Reactor.TransactionStep.html#run/3"},{"type":"module","title":"Ash.Reactor.UpdateStep","doc":"The Reactor step which is used to execute update actions.","ref":"Ash.Reactor.UpdateStep.html"},{"type":"module","title":"Ash.Resource.Dsl.Filter","doc":"Introspection target for a filter for read actions and relationships","ref":"Ash.Resource.Dsl.Filter.html"},{"type":"module","title":"Ash.Resource.Igniter","doc":"Codemods for working with Ash.Resource modules","ref":"Ash.Resource.Igniter.html"},{"type":"module","title":"Important Details - Ash.Resource.Igniter","doc":"This interrogates *the source code* of a resource, not its ultimate compiled state.\nWhat this means, is that things like `defines_attribute` will not return `true` if\nthe attribute is added by an extension. Only if it appears literally in the source code\nof the resource or one of its `Spark.Dsl.Fragment`s.","ref":"Ash.Resource.Igniter.html#module-important-details"},{"type":"function","title":"Ash.Resource.Igniter.add_action/3","doc":"Adds the given code block to the resource's `actions` block","ref":"Ash.Resource.Igniter.html#add_action/3"},{"type":"function","title":"Ash.Resource.Igniter.add_attribute/3","doc":"Adds the given code block to the resource's `attributes` block","ref":"Ash.Resource.Igniter.html#add_attribute/3"},{"type":"function","title":"Ash.Resource.Igniter.add_block/4","doc":"Adds the given code block to the block of the resource specified","ref":"Ash.Resource.Igniter.html#add_block/4"},{"type":"function","title":"Ash.Resource.Igniter.add_bypass/4","doc":"Adds a bypass to the top of the resource's `policies` block","ref":"Ash.Resource.Igniter.html#add_bypass/4"},{"type":"function","title":"Ash.Resource.Igniter.add_calculation/3","doc":"Adds the given code block to the resource's `calculations` block","ref":"Ash.Resource.Igniter.html#add_calculation/3"},{"type":"function","title":"Ash.Resource.Igniter.add_identity/3","doc":"Adds the given code block to the resource's `identities` block","ref":"Ash.Resource.Igniter.html#add_identity/3"},{"type":"function","title":"Ash.Resource.Igniter.add_new_action/4","doc":"Adds an action if it doesn't already exist","ref":"Ash.Resource.Igniter.html#add_new_action/4"},{"type":"function","title":"Ash.Resource.Igniter.add_new_attribute/4","doc":"Adds the given code block to the resource's `attributes` block if there is no existing attribute with the given name","ref":"Ash.Resource.Igniter.html#add_new_attribute/4"},{"type":"function","title":"Ash.Resource.Igniter.add_new_calculation/4","doc":"Adds the given code block to the resource's `calculations` block if there is no existing calculation with the given name","ref":"Ash.Resource.Igniter.html#add_new_calculation/4"},{"type":"function","title":"Ash.Resource.Igniter.add_new_identity/4","doc":"Adds the given code block to the resource's `identities` block if there is no existing identity with the given name","ref":"Ash.Resource.Igniter.html#add_new_identity/4"},{"type":"function","title":"Ash.Resource.Igniter.add_new_relationship/4","doc":"Adds the given code block to the resource's `relationships` block","ref":"Ash.Resource.Igniter.html#add_new_relationship/4"},{"type":"function","title":"Ash.Resource.Igniter.add_policy/4","doc":"Adds a policy to the bottom of the resource's `policies` block","ref":"Ash.Resource.Igniter.html#add_policy/4"},{"type":"function","title":"Ash.Resource.Igniter.add_relationship/3","doc":"Adds the given code block to the resource's `relationships` block","ref":"Ash.Resource.Igniter.html#add_relationship/3"},{"type":"function","title":"Ash.Resource.Igniter.add_resource_configuration/3","doc":"Adds the given code block to the resource's `resource` block","ref":"Ash.Resource.Igniter.html#add_resource_configuration/3"},{"type":"function","title":"Ash.Resource.Igniter.defines_action/3","doc":"Returns true if the given resource defines an action with the provided name","ref":"Ash.Resource.Igniter.html#defines_action/3"},{"type":"function","title":"Ash.Resource.Igniter.defines_attribute/3","doc":"Returns true if the given resource defines an attribute with the provided name","ref":"Ash.Resource.Igniter.html#defines_attribute/3"},{"type":"function","title":"Ash.Resource.Igniter.defines_calculation/3","doc":"Returns true if the given resource defines a calculation with the provided name","ref":"Ash.Resource.Igniter.html#defines_calculation/3"},{"type":"function","title":"Ash.Resource.Igniter.defines_identity/3","doc":"Returns true if the given resource defines an identity with the provided name","ref":"Ash.Resource.Igniter.html#defines_identity/3"},{"type":"function","title":"Ash.Resource.Igniter.defines_relationship/3","doc":"Returns true if the given resource defines a relationship with the provided name","ref":"Ash.Resource.Igniter.html#defines_relationship/3"},{"type":"function","title":"Ash.Resource.Igniter.domain/2","doc":"Gets the domain from the given resource module","ref":"Ash.Resource.Igniter.html#domain/2"},{"type":"function","title":"Ash.Resource.Igniter.ensure_primary_action/3","doc":"","ref":"Ash.Resource.Igniter.html#ensure_primary_action/3"},{"type":"function","title":"Ash.Resource.Igniter.ensure_timestamps/2","doc":"Ensures that created_at and updated_at timestamps exist on the resource","ref":"Ash.Resource.Igniter.html#ensure_timestamps/2"},{"type":"function","title":"Ash.Resource.Igniter.has_action_with_primary/3","doc":"","ref":"Ash.Resource.Igniter.html#has_action_with_primary/3"},{"type":"function","title":"Ash.Resource.Igniter.list_resources/1","doc":"List all resource modules found in the project","ref":"Ash.Resource.Igniter.html#list_resources/1"},{"type":"function","title":"Ash.Resource.Igniter.resource_mods/1","doc":"","ref":"Ash.Resource.Igniter.html#resource_mods/1"},{"type":"module","title":"Ash.Resource.ManualCreate.BulkContext","doc":"The context passed into manual bulk_create action functions","ref":"Ash.Resource.ManualCreate.BulkContext.html"},{"type":"type","title":"Ash.Resource.ManualCreate.BulkContext.t/0","doc":"","ref":"Ash.Resource.ManualCreate.BulkContext.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualCreate.Context","doc":"The context passed into manual create action functions","ref":"Ash.Resource.ManualCreate.Context.html"},{"type":"type","title":"Ash.Resource.ManualCreate.Context.t/0","doc":"","ref":"Ash.Resource.ManualCreate.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualDestroy.BulkContext","doc":"The context passed into manual bulk_update action functions","ref":"Ash.Resource.ManualDestroy.BulkContext.html"},{"type":"type","title":"Ash.Resource.ManualDestroy.BulkContext.t/0","doc":"","ref":"Ash.Resource.ManualDestroy.BulkContext.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualDestroy.Context","doc":"The context passed into manual update action functions","ref":"Ash.Resource.ManualDestroy.Context.html"},{"type":"type","title":"Ash.Resource.ManualDestroy.Context.t/0","doc":"","ref":"Ash.Resource.ManualDestroy.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualRelationship.Context","doc":"The context passed into manual relationship functions","ref":"Ash.Resource.ManualRelationship.Context.html"},{"type":"type","title":"Ash.Resource.ManualRelationship.Context.t/0","doc":"","ref":"Ash.Resource.ManualRelationship.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualUpdate.BulkContext","doc":"The context passed into manual update action functions","ref":"Ash.Resource.ManualUpdate.BulkContext.html"},{"type":"type","title":"Ash.Resource.ManualUpdate.BulkContext.t/0","doc":"","ref":"Ash.Resource.ManualUpdate.BulkContext.html#t:t/0"},{"type":"module","title":"Ash.Resource.ManualUpdate.Context","doc":"The context passed into manual update action functions","ref":"Ash.Resource.ManualUpdate.Context.html"},{"type":"type","title":"Ash.Resource.ManualUpdate.Context.t/0","doc":"","ref":"Ash.Resource.ManualUpdate.Context.html#t:t/0"},{"type":"module","title":"Ash.Resource.Preparation.Context","doc":"The context for a preparation.","ref":"Ash.Resource.Preparation.Context.html"},{"type":"type","title":"Ash.Resource.Preparation.Context.t/0","doc":"","ref":"Ash.Resource.Preparation.Context.html#t:t/0"},{"type":"module","title":"Ash.Scope","doc":"Determines how the `actor`, `tenant` and `context` are extracted from a data structure.\n\nThis is inspired by the same feature in `Phoenix`, however the `actor`, `tenant` and `context`\noptions will always remain available, as they are standardized representations of things that\nactions can use to do their work.\n\nWhen you have a scope, you can group up actor/tenant/context into one struct and pass that around,\nfor example:\n\n```elixir\nscope = %MyApp.Scope{current_user: user, current_tenant: tenant, locale: \"en\"}\n\n# instead of\nMyDomain.create_thing(actor: current_user, tenant: tenant)\n\n# you can do\nMyDomain.create_thing(scope: scope)\n```\n\n>","ref":"Ash.Scope.html"},{"type":"module","title":"Scope is left at the front door - Ash.Scope","doc":">\n> Your scope is \"left at the front door\". That is, when you pass a scope to an action, the options\n> are extracted and the scope is removed from those options. Within hooks, you are meant to use\n> the `context` provided to your functions as the new `scope`. This is very important, because\n> you don't want a bunch of your code or extension code having to switch on `if opts[:scope]`,\n> extracting the things that it needs, etc.\n>\n> See [the actions guide](/documentation/topics/actions/actions.md#context) for more information.","ref":"Ash.Scope.html#module-scope-is-left-at-the-front-door"},{"type":"module","title":"Setup - Ash.Scope","doc":"If you are using Phoenix, you will want to assign your `scope` module in a plug that runs\nafter your plugs that determine actor/tenant/context. Then, you will want to add an `on_mount`\nhook for LiveViews that sets your `scope` assign. This is especially true for `AshAuthentication`,\nas it does not currently have a concept of scopes.","ref":"Ash.Scope.html#module-setup"},{"type":"module","title":"Passing scope and options - Ash.Scope","doc":"For the `actor`, `tenant` and `authorize?`, extracted from scopes, the values from the scope are *discarded* if also present in `opts`.\n\ni.e `scope: scope, actor: nil` will remove the set actor. `scope: scope, actor: some_other_actor` will set the actor to `some_other_actor`.\n\nFor `context`, the values are deep merged.\n\nFor `tracer`, the value(s) are concatenated into a single list.","ref":"Ash.Scope.html#module-passing-scope-and-options"},{"type":"module","title":"Example - Ash.Scope","doc":"You would implement `Ash.Scope.ToOpts` for a module like so:\n\n```elixir\ndefmodule MyApp.Scope do\n  defstruct [:current_user, :current_tenant, :locale]\n\n  defimpl Ash.Scope.ToOpts do\n    def get_actor(%{current_user: current_user}), do: {:ok, current_user}\n    def get_tenant(%{current_tenant: current_tenant}), do: {:ok, current_tenant}\n    def get_context(%{locale: locale}), do: {:ok, %{shared: %{locale: locale}}}\n    # You typically configure tracers in config giles\n    # so this will typically return :error\n    def get_tracer(_), do: :error\n\n    # This should likely always return :error\n    # unless you want a way to bypass authorization configured in your scope\n    def get_authorize?(_), do: :error\n  end\nend\n```\n\nFor more on context, and what the `shared` key is used for, see the [actions guide](/documentation/topics/actions/actions.md#context)\n\nYou could then use this in various places by passing the `scope` option.\n\nFor example:\n\n```elixir\nscope = %MyApp.Scope{...}\n# with code interfaces\nMyApp.Blog.create_post!(\"new post\", scope: scope)\n\n# with changesets and queries\nMyApp.Blog\n|> Ash.Changeset.for_create(:create, %{title: \"new post\"}, scope: scope)\n|> Ash.create!()\n\n# with the context structs that we provide\n\ndef change(changeset, _, context) do\n  Ash.Changeset.after_action(changeset, fn changeset, result ->\n    MyApp.Domain.do_something_else(..., scope: context)\n    # if not using as a scope, the alternative is this\n    # in the future this will be deprecated\n    MyApp.Domain.do_somethign_else(..., Ash.Context.to_opts(context))\n  end)\nend\n```\n\n\nExtensions should not use this option, only end users.","ref":"Ash.Scope.html#module-example"},{"type":"type","title":"Ash.Scope.t/0","doc":"","ref":"Ash.Scope.html#t:t/0"},{"type":"function","title":"Ash.Scope.to_opts/2","doc":"","ref":"Ash.Scope.html#to_opts/2"},{"type":"protocol","title":"Ash.Scope.ToOpts","doc":"","ref":"Ash.Scope.ToOpts.html"},{"type":"function","title":"Ash.Scope.ToOpts.get_actor/1","doc":"Extracts the actor from the scope","ref":"Ash.Scope.ToOpts.html#get_actor/1"},{"type":"function","title":"Ash.Scope.ToOpts.get_authorize?/1","doc":"Extracts the `authorize?` option from the scope","ref":"Ash.Scope.ToOpts.html#get_authorize?/1"},{"type":"function","title":"Ash.Scope.ToOpts.get_context/1","doc":"Extracts the context from the scope","ref":"Ash.Scope.ToOpts.html#get_context/1"},{"type":"function","title":"Ash.Scope.ToOpts.get_tenant/1","doc":"Extracts the tenant from the scope","ref":"Ash.Scope.ToOpts.html#get_tenant/1"},{"type":"function","title":"Ash.Scope.ToOpts.get_tracer/1","doc":"Extracts the tracer(s) from the scope","ref":"Ash.Scope.ToOpts.html#get_tracer/1"},{"type":"type","title":"Ash.Scope.ToOpts.t/0","doc":"","ref":"Ash.Scope.ToOpts.html#t:t/0"},{"type":"protocol","title":"Ash.ToTenant","doc":"Converts a value to a tenant. To add this to a resource, implement the protocol like so:application\n\nWhat this should do is entirely dependent on how you've set up your tenants. This example assumes\nthat you want the tenant to be `org_#{organization_id}`, but it could also be something like\n`organization.schema`.\n\n```elixir\ndefmodule MyApp.Organization do\n  use Ash.Resource, ...\n\n  ...\n\n  defimpl Ash.ToTenant do\n    def to_tenant(%{id: id}, _resource), do: \"org_#{id}\"\n  end\nend\n```","ref":"Ash.ToTenant.html"},{"type":"type","title":"Ash.ToTenant.t/0","doc":"","ref":"Ash.ToTenant.html#t:t/0"},{"type":"function","title":"Ash.ToTenant.to_tenant/2","doc":"","ref":"Ash.ToTenant.html#to_tenant/2"},{"type":"module","title":"Comp","doc":"Provides utilities to implement and work with `Comparable` types","ref":"Comp.html"},{"type":"function","title":"Comp.compare/2","doc":"Compare left and right term","ref":"Comp.html#compare/2"},{"type":"function","title":"Examples - Comp.compare/2","doc":"iex> Comp.compare(1, 2)\n    :lt\n    iex> Comp.compare(2, 1)\n    :gt\n    iex> Comp.compare(1, 1)\n    :eq","ref":"Comp.html#compare/2-examples"},{"type":"macro","title":"Comp.defcomparable/3","doc":"Helper to define ordering relation for pair of types,\naccepts two `term :: type` pairs\nand block of code where relation is described.","ref":"Comp.html#defcomparable/3"},{"type":"macro","title":"Examples - Comp.defcomparable/3","doc":"iex> quote do\n    ...>   use Comp\n    ...>   defmodule Foo do\n    ...>     defstruct [:value, :meta]\n    ...>   end\n    ...>   defmodule Bar do\n    ...>     defstruct [:value, :meta]\n    ...>   end\n    ...>   defcomparable %Foo{value: left} :: Foo, %Foo{value: right} :: Foo do\n    ...>     Comp.compare(left, right)\n    ...>   end\n    ...>   defcomparable %Foo{value: left} :: Foo, %Bar{value: right} :: Bar do\n    ...>     Comp.compare(left, right)\n    ...>   end\n    ...>   defcomparable %Foo{value: left} :: Foo, right :: Integer do\n    ...>     Comp.compare(left, right)\n    ...>   end\n    ...> end\n    ...> |> Code.compile_quoted\n    iex> quote do\n    ...>   x = %Foo{value: 1, meta: 1}\n    ...>   y = %Foo{value: 1, meta: 2}\n    ...>   Comp.equal?(x, y) && Comp.equal?(y, x)\n    ...> end\n    ...> |> Code.eval_quoted\n    ...> |> elem(0)\n    true\n    iex> quote do\n    ...>   x = %Foo{value: 1, meta: 1}\n    ...>   y = %Bar{value: 1, meta: 2}\n    ...>   Comp.equal?(x, y) && Comp.equal?(y, x)\n    ...> end\n    ...> |> Code.eval_quoted\n    ...> |> elem(0)\n    true\n    iex> quote do\n    ...>   x = %Foo{value: 1, meta: 1}\n    ...>   y = 1\n    ...>   Comp.equal?(x, y) && Comp.equal?(y, x)\n    ...> end\n    ...> |> Code.eval_quoted\n    ...> |> elem(0)\n    true","ref":"Comp.html#defcomparable/3-examples"},{"type":"macro","title":"Comp.eq/0","doc":"","ref":"Comp.html#eq/0"},{"type":"function","title":"Comp.equal?/2","doc":"Is left term equal to right term?","ref":"Comp.html#equal?/2"},{"type":"function","title":"Examples - Comp.equal?/2","doc":"iex> Comp.equal?(1, 1)\n    true\n    iex> Comp.equal?(1, :hello)\n    false","ref":"Comp.html#equal?/2-examples"},{"type":"function","title":"Comp.greater_or_equal?/2","doc":"Is left term greater or equal to right term?","ref":"Comp.html#greater_or_equal?/2"},{"type":"function","title":"Examples - Comp.greater_or_equal?/2","doc":"iex> Comp.greater_or_equal?(1, 1)\n    true\n    iex> Comp.greater_or_equal?(1, 2)\n    false\n    iex> Comp.greater_or_equal?(2, 1)\n    true","ref":"Comp.html#greater_or_equal?/2-examples"},{"type":"function","title":"Comp.greater_than?/2","doc":"Is left term greater than right term?","ref":"Comp.html#greater_than?/2"},{"type":"function","title":"Examples - Comp.greater_than?/2","doc":"iex> Comp.greater_than?(1, 1)\n    false\n    iex> Comp.greater_than?(1, 2)\n    false\n    iex> Comp.greater_than?(2, 1)\n    true","ref":"Comp.html#greater_than?/2-examples"},{"type":"macro","title":"Comp.gt/0","doc":"","ref":"Comp.html#gt/0"},{"type":"type","title":"Comp.left/0","doc":"","ref":"Comp.html#t:left/0"},{"type":"function","title":"Comp.less_or_equal?/2","doc":"Is left term less or equal to right term?","ref":"Comp.html#less_or_equal?/2"},{"type":"function","title":"Examples - Comp.less_or_equal?/2","doc":"iex> Comp.less_or_equal?(1, 1)\n    true\n    iex> Comp.less_or_equal?(1, 2)\n    true\n    iex> Comp.less_or_equal?(2, 1)\n    false","ref":"Comp.html#less_or_equal?/2-examples"},{"type":"function","title":"Comp.less_than?/2","doc":"Is left term less than right term?","ref":"Comp.html#less_than?/2"},{"type":"function","title":"Examples - Comp.less_than?/2","doc":"iex> Comp.less_than?(1, 1)\n    false\n    iex> Comp.less_than?(1, 2)\n    true\n    iex> Comp.less_than?(2, 1)\n    false","ref":"Comp.html#less_than?/2-examples"},{"type":"macro","title":"Comp.lt/0","doc":"","ref":"Comp.html#lt/0"},{"type":"function","title":"Comp.max/2","doc":"Returns the biggest of the two given terms, if terms are equal - then the first one is returned","ref":"Comp.html#max/2"},{"type":"function","title":"Examples - Comp.max/2","doc":"iex> Comp.max(1, 1)\n    1\n    iex> Comp.max(1, 2)\n    2\n    iex> Comp.max(2, 1)\n    2","ref":"Comp.html#max/2-examples"},{"type":"function","title":"Comp.min/2","doc":"Returns the smallest of the two given terms, if terms are equal - then the first one is returned","ref":"Comp.html#min/2"},{"type":"function","title":"Examples - Comp.min/2","doc":"iex> Comp.min(1, 1)\n    1\n    iex> Comp.min(1, 2)\n    1\n    iex> Comp.min(2, 1)\n    1","ref":"Comp.html#min/2-examples"},{"type":"function","title":"Comp.not_equal?/2","doc":"Is left term not equal to right term?","ref":"Comp.html#not_equal?/2"},{"type":"function","title":"Examples - Comp.not_equal?/2","doc":"iex> Comp.not_equal?(1, 1)\n    false\n    iex> Comp.not_equal?(1, :hello)\n    true","ref":"Comp.html#not_equal?/2-examples"},{"type":"type","title":"Comp.right/0","doc":"","ref":"Comp.html#t:right/0"},{"type":"function","title":"Comp.type_of/1","doc":"","ref":"Comp.html#type_of/1"},{"type":"protocol","title":"Comparable","doc":"Protocol which describes ordering relation for pair of types","ref":"Comparable.html"},{"type":"function","title":"Comparable.compare/1","doc":"Accepts struct with fields :left and :right and returns ord value","ref":"Comparable.html#compare/1"},{"type":"type","title":"Comparable.ord/0","doc":"","ref":"Comparable.html#t:ord/0"},{"type":"type","title":"Comparable.t/0","doc":"","ref":"Comparable.html#t:t/0"},{"type":"module","title":"Comparable.Type.Any.To.Any","doc":"","ref":"Comparable.Type.Any.To.Any.html"},{"type":"module","title":"Comparable.Type.Ash.CiString.To.Ash.CiString","doc":"","ref":"Comparable.Type.Ash.CiString.To.Ash.CiString.html"},{"type":"module","title":"Comparable.Type.Ash.CiString.To.BitString","doc":"","ref":"Comparable.Type.Ash.CiString.To.BitString.html"},{"type":"module","title":"Comparable.Type.Atom.To.BitString","doc":"","ref":"Comparable.Type.Atom.To.BitString.html"},{"type":"module","title":"Comparable.Type.BitString.To.Ash.CiString","doc":"","ref":"Comparable.Type.BitString.To.Ash.CiString.html"},{"type":"module","title":"Comparable.Type.BitString.To.Atom","doc":"","ref":"Comparable.Type.BitString.To.Atom.html"},{"type":"module","title":"Comparable.Type.BitString.To.Decimal","doc":"","ref":"Comparable.Type.BitString.To.Decimal.html"},{"type":"module","title":"Comparable.Type.Date.To.Date","doc":"","ref":"Comparable.Type.Date.To.Date.html"},{"type":"module","title":"Comparable.Type.DateTime.To.DateTime","doc":"","ref":"Comparable.Type.DateTime.To.DateTime.html"},{"type":"module","title":"Comparable.Type.Decimal.To.BitString","doc":"","ref":"Comparable.Type.Decimal.To.BitString.html"},{"type":"module","title":"Comparable.Type.Decimal.To.Decimal","doc":"","ref":"Comparable.Type.Decimal.To.Decimal.html"},{"type":"module","title":"Comparable.Type.Decimal.To.Float","doc":"","ref":"Comparable.Type.Decimal.To.Float.html"},{"type":"module","title":"Comparable.Type.Decimal.To.Integer","doc":"","ref":"Comparable.Type.Decimal.To.Integer.html"},{"type":"module","title":"Comparable.Type.Float.To.Decimal","doc":"","ref":"Comparable.Type.Float.To.Decimal.html"},{"type":"module","title":"Comparable.Type.Integer.To.Decimal","doc":"","ref":"Comparable.Type.Integer.To.Decimal.html"},{"type":"module","title":"Comparable.Type.List.To.List","doc":"","ref":"Comparable.Type.List.To.List.html"},{"type":"module","title":"Comparable.Type.Map.To.Map","doc":"","ref":"Comparable.Type.Map.To.Map.html"},{"type":"module","title":"Comparable.Type.NaiveDateTime.To.NaiveDateTime","doc":"","ref":"Comparable.Type.NaiveDateTime.To.NaiveDateTime.html"},{"type":"module","title":"Comparable.Type.Time.To.Time","doc":"","ref":"Comparable.Type.Time.To.Time.html"},{"type":"module","title":"Comparable.Type.Tuple.To.Tuple","doc":"","ref":"Comparable.Type.Tuple.To.Tuple.html"},{"type":"task","title":"mix ash.codegen","doc":"Runs all codegen tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Codegen.html"},{"type":"task","title":"Flags - mix ash.codegen","doc":"* `--dry-run` - no files are created, instead the new generated code is printed to the console\n* `--check` - no files are created, returns an exit(1) code if any code would need to be generated\n* `--dev` - runs codegen tasks in dev mode. See the section on dev mode below\n* Individual extensions may use additional flags.","ref":"Mix.Tasks.Ash.Codegen.html#module-flags"},{"type":"task","title":"Dev Mode - mix ash.codegen","doc":"Some extensions that do `codegen` require providing a `name`. Those extensions (should) support a `--dev`\nflag, which indicates that the codegen can have a temporary name chosen by the extension. Then, once you\nare ready to commit the changes, you can run the codegen tasks again without the `--dev` flag and with a\nname to generate the final code.\n\nFor example, using AshPostgres:\n\n- First we add `first_name` to `MyApp.Accounts.User`\n- `mix ash.codegen --dev`, which generates a migration for adding `first_name` to the `\"users\"` table,\n  but the migration is suffixed with `_dev`\n- `mix ash.migrate` apply the migrations\n- Then we add `last_name` to `MyApp.Accounts.User`\n- `mix ash.codegen --dev` which generates a migration for adding `last_name` to the `\"users\"` table,\n  but the migration is suffixed with `_dev`\n- We review our changes, and are ready to save them as a unit\n- `mix ash.codegen add_name_to_user`, which rolls back and deletes the dev migrations & snapshots, and creates new\n  ones using the provided name","ref":"Mix.Tasks.Ash.Codegen.html#module-dev-mode"},{"type":"function","title":"Mix.Tasks.Ash.Codegen.run/1","doc":"Runs all codegen tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Codegen.html#run/1"},{"type":"task","title":"mix ash.extend","doc":"Adds an extension or extensions to the domain/resource\n\nExtensions can either be a fully qualified module name, or one of the following list, based on the thing being extended","ref":"Mix.Tasks.Ash.Extend.html"},{"type":"task","title":"Ash.Domain - mix ash.extend","doc":"- `json_api` - `AshJsonApi.Domain`\n- `graphql` - `AshGraphql.Domain`","ref":"Mix.Tasks.Ash.Extend.html#module-ash-domain"},{"type":"task","title":"Ash.Resource - mix ash.extend","doc":"- `postgres` - `AshPostgres.DataLayer`\n- `sqlite` - `AshSqlite.DataLayer`\n- `mysql` - `AshMysql.DataLayer`\n- `ets` - `Ash.DataLayer.Ets`\n- `mnesia` - `Ash.DataLayer.Mnesia`\n- `embedded` - `data_layer: :embedded`\n- `json_api` - `AshJsonApi.Resource`\n- `graphql` - `AshGraphql.Resource`","ref":"Mix.Tasks.Ash.Extend.html#module-ash-resource"},{"type":"task","title":"Example - mix ash.extend","doc":"```bash\nmix ash.extend My.Domain.Resource postgres,Ash.Policy.Authorizer\n```","ref":"Mix.Tasks.Ash.Extend.html#module-example"},{"type":"task","title":"mix ash.gen.base_resource","doc":"Generates a base resource\n\nSee the [writing extensions guide](/documentation/topics/advanced/writing-extensions.md#base-resources) for more.","ref":"Mix.Tasks.Ash.Gen.BaseResource.html"},{"type":"task","title":"Example - mix ash.gen.base_resource","doc":"```bash\nmix ash.gen.base_resource MyApp.Resource\n```","ref":"Mix.Tasks.Ash.Gen.BaseResource.html#module-example"},{"type":"task","title":"mix ash.gen.change","doc":"Generates a custom change\n\nSee [Custom Changes](https://hexdocs.pm/ash/changes.html#custom-changes) for more.","ref":"Mix.Tasks.Ash.Gen.Change.html"},{"type":"task","title":"Example - mix ash.gen.change","doc":"```bash\nmix ash.gen.change MyApp.Changes.Slugify\n```","ref":"Mix.Tasks.Ash.Gen.Change.html#module-example"},{"type":"task","title":"mix ash.gen.custom_expression","doc":"Generates a custom expression\n\nSee [Ash.CustomExpression](https://hexdocs.pm/ash/Ash.CustomExpression.html) for more.","ref":"Mix.Tasks.Ash.Gen.CustomExpression.html"},{"type":"task","title":"Example - mix ash.gen.custom_expression","doc":"```bash\nmix ash.gen.custom_expression MyApp.Expressions.LevenshteinDistance --args string,string\n```","ref":"Mix.Tasks.Ash.Gen.CustomExpression.html#module-example"},{"type":"task","title":"Options - mix ash.gen.custom_expression","doc":"* `--args` - A comma separated list of argument types. i.e `--args string`","ref":"Mix.Tasks.Ash.Gen.CustomExpression.html#module-options"},{"type":"task","title":"mix ash.gen.domain","doc":"Generates an Ash.Domain","ref":"Mix.Tasks.Ash.Gen.Domain.html"},{"type":"task","title":"Example - mix ash.gen.domain","doc":"```bash\nmix ash.gen.domain MyApp.Accounts\n```","ref":"Mix.Tasks.Ash.Gen.Domain.html#module-example"},{"type":"task","title":"mix ash.gen.enum","doc":"Generates an Ash.Type.Enum","ref":"Mix.Tasks.Ash.Gen.Enum.html"},{"type":"task","title":"Example - mix ash.gen.enum","doc":"```bash\nmix ash.gen.enum MyApp.Support.Ticket.Types.Status open,closed --short-name ticket_status\n```","ref":"Mix.Tasks.Ash.Gen.Enum.html#module-example"},{"type":"task","title":"Options - mix ash.gen.enum","doc":"- `--short-name`, `-s`: Register the type under the provided shortname, so it can be referenced like `:short_name` instead of the module name.\n- `--ignore-if-exists` - Does nothing if the resource already exists","ref":"Mix.Tasks.Ash.Gen.Enum.html#module-options"},{"type":"task","title":"mix ash.gen.preparation","doc":"Generates a custom preparation\n\nSee [Custom Preparations](https://hexdocs.pm/ash/preparations.html#custom-preparations) for more.","ref":"Mix.Tasks.Ash.Gen.Preparation.html"},{"type":"task","title":"Example - mix ash.gen.preparation","doc":"```bash\nmix ash.gen.preparation MyApp.Preparations.Top5\n```","ref":"Mix.Tasks.Ash.Gen.Preparation.html#module-example"},{"type":"task","title":"mix ash.gen.resource","doc":"Generate and configure an Ash.Resource.\n\nIf the domain does not exist, we create it. If it does, we add the resource to it if it is not already present.","ref":"Mix.Tasks.Ash.Gen.Resource.html"},{"type":"task","title":"Example - mix ash.gen.resource","doc":"```bash\nmix ash.gen.resource Helpdesk.Support.Ticket \\\n  --default-actions read \\\n  --uuid-primary-key id \\\n  --attribute subject:string:required:public \\\n  --relationship belongs_to:representative:Helpdesk.Support.Representative \\\n  --timestamps \\\n  --extend postgres,graphql\n\n```","ref":"Mix.Tasks.Ash.Gen.Resource.html#module-example"},{"type":"task","title":"Options - mix ash.gen.resource","doc":"* `--attribute` or `-a` - An attribute or comma separated list of attributes to add, as `name:type`. Modifiers: `primary_key`, `public`, `sensitive`, and `required`. i.e `-a name:string:required`\n* `--relationship` or `-r` - A relationship or comma separated list of relationships to add, as `type:name:dest`. Modifiers: `public`. `belongs_to` only modifiers: `primary_key`, `sensitive`, and `required`. i.e `-r belongs_to:author:MyApp.Accounts.Author:required`\n* `--default-actions` - A csv list of default action types to add. The `create` and `update` actions accept the public attributes being added.\n* `--uuid-primary-key` or `-u` - Adds a UUIDv4 primary key with that name. i.e `-u id`\n* `--uuid-v7-primary-key` - Adds a UUIDv7 primary key with that name.\n* `--integer-primary-key` or `-i` - Adds an integer primary key with that name. i.e `-i id`\n* `--domain` or `-d` - The domain module to add the resource to. i.e `-d MyApp.MyDomain`. This defaults to the resource's module name, minus the last segment.\n* `--extend` or `-e` - A comma separated list of modules or builtins to extend the resource with. i.e `-e postgres,Some.Extension`\n* `--base` or `-b` - The base module to use for the resource. i.e `-b Ash.Resource`. Requires that the module is in `config :your_app, :base_resources`\n* `--timestamps` or `-t` - If set adds `inserted_at` and `updated_at` timestamps to the resource.\n* `--ignore-if-exists` - Does nothing if the resource already exists","ref":"Mix.Tasks.Ash.Gen.Resource.html#module-options"},{"type":"task","title":"mix ash.gen.validation","doc":"Generates a custom validation\n\nSee [Custom Validations](https://hexdocs.pm/ash/validations.html#custom-validations) for more.","ref":"Mix.Tasks.Ash.Gen.Validation.html"},{"type":"task","title":"Example - mix ash.gen.validation","doc":"```bash\nmix ash.gen.validation MyApp.Validations.IsPrime\n```","ref":"Mix.Tasks.Ash.Gen.Validation.html#module-example"},{"type":"task","title":"mix ash.generate_livebook","doc":"Generates a Livebook for each Ash domain.","ref":"Mix.Tasks.Ash.GenerateLivebook.html"},{"type":"task","title":"Command line options - mix ash.generate_livebook","doc":"* `--filename` - Specify the name of the generated Livebook file. Default: `livebook.livemd`","ref":"Mix.Tasks.Ash.GenerateLivebook.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Ash.GenerateLivebook.run/1","doc":"Generates a Livebook for each Ash domain","ref":"Mix.Tasks.Ash.GenerateLivebook.html#run/1"},{"type":"task","title":"mix ash.generate_policy_charts","doc":"Generates a Mermaid Flow Chart for a given resource's policies.","ref":"Mix.Tasks.Ash.GeneratePolicyCharts.html"},{"type":"task","title":"Prerequisites - mix ash.generate_policy_charts","doc":"This mix task requires the Mermaid CLI to be installed on your system.\n\nSee https://github.com/mermaid-js/mermaid-cli","ref":"Mix.Tasks.Ash.GeneratePolicyCharts.html#module-prerequisites"},{"type":"task","title":"Command line options - mix ash.generate_policy_charts","doc":"* `--only` - only generates for the given file\n  * `--format` - Can be set to one of either:\n    * `plain` - Prints just the mermaid output as text. This is the default.\n    * `md` - Prints the mermaid diagram in a markdown code block.\n    * `svg` - Generates an SVG\n    * `pdf` - Generates a PDF\n    * `png` - Generates a PNG","ref":"Mix.Tasks.Ash.GeneratePolicyCharts.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Ash.GeneratePolicyCharts.run/1","doc":"Generates a Mermaid Flow Chart for a given resource's policies.","ref":"Mix.Tasks.Ash.GeneratePolicyCharts.html#run/1"},{"type":"task","title":"mix ash.generate_resource_diagrams","doc":"Generates a Mermaid Resource Diagram for each Ash domain.","ref":"Mix.Tasks.Ash.GenerateResourceDiagrams.html"},{"type":"task","title":"Prerequisites - mix ash.generate_resource_diagrams","doc":"This mix task requires the Mermaid CLI to be installed on your system.\n\nSee https://github.com/mermaid-js/mermaid-cli","ref":"Mix.Tasks.Ash.GenerateResourceDiagrams.html#module-prerequisites"},{"type":"task","title":"Command line options - mix ash.generate_resource_diagrams","doc":"* `--type` - `er` or `class` (defaults to `class`)\n  * `--only` - only generates for the given domain\n  * `--format` - Can be set to one of either:\n    * `plain` - Prints just the mermaid output as text. This is the default.\n    * `md` - Prints the mermaid diagram in a markdown code block.\n    * `svg` - Generates an SVG\n    * `pdf` - Generates a PDF\n    * `png` - Generates a PNG","ref":"Mix.Tasks.Ash.GenerateResourceDiagrams.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Ash.GenerateResourceDiagrams.run/1","doc":"Generates Mermaid Resource Diagrams for each Ash domain","ref":"Mix.Tasks.Ash.GenerateResourceDiagrams.html#run/1"},{"type":"task","title":"mix ash.install","doc":"Installs Ash into a project. Should be called with `mix igniter.install ash`","ref":"Mix.Tasks.Ash.Install.html"},{"type":"task","title":"Options - mix ash.install","doc":"- `--example` - Creates some example resources. When used, will pass\n  through options to `mix ash.gen.resource`. See that task docs for more.","ref":"Mix.Tasks.Ash.Install.html#module-options"},{"type":"task","title":"mix ash.migrate","doc":"Runs all migration tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Migrate.html"},{"type":"function","title":"Mix.Tasks.Ash.Migrate.run/1","doc":"Runs all migration tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Migrate.html#run/1"},{"type":"task","title":"mix ash.patch.extend","doc":"Adds an extension or extensions to the domain/resource. Use `ash.extend` instead.","ref":"Mix.Tasks.Ash.Patch.Extend.html"},{"type":"task","title":"mix ash.reset","doc":"Runs all tear down tasks for any extension on any resource/domain in your application, followed by setup tasks.","ref":"Mix.Tasks.Ash.Reset.html"},{"type":"function","title":"Mix.Tasks.Ash.Reset.run/1","doc":"Runs all tear down & setup tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Reset.html#run/1"},{"type":"task","title":"mix ash.rollback","doc":"Runs all rollback tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Rollback.html"},{"type":"function","title":"Mix.Tasks.Ash.Rollback.run/1","doc":"Runs all rollback tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Rollback.html#run/1"},{"type":"task","title":"mix ash.setup","doc":"Runs all setup tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Setup.html"},{"type":"function","title":"Mix.Tasks.Ash.Setup.run/1","doc":"Runs all setup tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.Setup.html#run/1"},{"type":"task","title":"mix ash.tear_down","doc":"Runs all tear down tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.TearDown.html"},{"type":"function","title":"Mix.Tasks.Ash.TearDown.run/1","doc":"Runs all tear_down tasks for any extension on any resource/domain in your application.","ref":"Mix.Tasks.Ash.TearDown.html#run/1"},{"type":"extras","title":"Home","doc":"![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-black-text.png?raw=true#gh-light-mode-only)\n![Logo](https://github.com/ash-project/ash/blob/main/logos/cropped-for-header-white-text.png?raw=true#gh-dark-mode-only)\n\n![Elixir CI](https://github.com/ash-project/ash/workflows/Ash%20CI/badge.svg)\n[![OpenSSF Scorecard](https://api.scorecard.dev/projects/github.com/ash-project/ash/badge)](https://scorecard.dev/viewer/?uri=github.com/ash-project/ash)\n[![OpenSSF Best Practices](https://www.bestpractices.dev/projects/10686/badge)](https://www.bestpractices.dev/projects/10686)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Hex version badge](https://img.shields.io/hexpm/v/ash.svg)](https://hex.pm/packages/ash)\n[![Hexdocs badge](https://img.shields.io/badge/docs-hexdocs-purple)](https://hexdocs.pm/ash)\n\n# Ash Framework\n\nWelcome! Here you will find everything you need to know to get started with and use Ash. This documentation is best viewed on [hexdocs](https://hexdocs.pm/ash).","ref":"readme.html"},{"type":"extras","title":"Dive In - Home","doc":"- [What is Ash?](documentation/topics/about_ash/what-is-ash.md)\n- [Getting Started Tutorial](documentation/tutorials/get-started.md)\n- [See the roadmap](https://github.com/orgs/ash-project/projects/3)","ref":"readme.html#dive-in"},{"type":"extras","title":"About the Documentation - Home","doc":"The [Get Started Livebook](documentation/tutorials/get-started.md) **Tutorial** introduces you to the core concepts of Ash like resources, actions, and relationships. It’s a great place to start if you are a beginner.\n\n---\n\n[**Topics**](#topics) provide a high level overview of a specific concept or feature of Ash. These are **understanding-oriented**, and are perfect for discovering design patterns, features, and tools.\n\n---\n\n[**How-to**](#how-to) guides are **goal-oriented** recipes for accomplishing specific popular tasks. Browse to get an idea of how Ash works and what is possible with it.\n\n---\n\n[**Reference**](#reference) documentation is **information-oriented**, covering every Ash module, function, expression, and DSL. It is produced automatically from our source code. Use the sidebar and the top search\nbar to find relevant reference information. Place the text `dsl` before your search to quickly jump to a particular DSL — e.g. try comparing the results of searching for `name` with the results for `dsl name`.\n\n---","ref":"readme.html#about-the-documentation"},{"type":"extras","title":"Topics - Home","doc":"","ref":"readme.html#topics"},{"type":"extras","title":"About Ash - Home","doc":"- [What is Ash?](documentation/topics/about_ash/what-is-ash.md)\n- [Our Design Principles](documentation/topics/about_ash/design-principles.md)\n- [Contributing to Ash](.github/CONTRIBUTING.md)\n- [Alternatives](documentation/topics/about_ash/alternatives.md)","ref":"readme.html#about-ash"},{"type":"extras","title":"Development - Home","doc":"- [Project Structure](documentation/topics/development/project-structure.md)\n- [Generators](documentation/topics/development/generators.md)\n- [Testing](documentation/topics/development/testing.md)\n- [Working with LLM Assistance](documentation/topics/development/working-with-llms.md)\n- [Development Utilities](documentation/topics/development/development-utilities.md)\n- [Backwards Compatibility Config](documentation/topics/development/backwards-compatibility-config.md)\n- [Upgrading to 3.0](documentation/topics/development/upgrading-to-3.0.md)\n- [Error Handling](documentation/topics/development/error-handling.md)","ref":"readme.html#development"},{"type":"extras","title":"Resources - Home","doc":"- [Domains](documentation/topics/resources/domains.md)\n- [Attributes](documentation/topics/resources/attributes.md)\n- [Relationships](documentation/topics/resources/relationships.md)\n- [Calculations](documentation/topics/resources/calculations.md)\n- [Aggregates](documentation/topics/resources/aggregates.md)\n- [Code Interfaces](documentation/topics/resources/code-interfaces.md)\n- [Identities](documentation/topics/resources/identities.md)\n- [Validations](documentation/topics/resources/validations.md)\n- [Changes](documentation/topics/resources/changes.md)\n- [Preparations](documentation/topics/resources/preparations.md)\n- [Embedded Resources](documentation/topics/resources/embedded-resources.md)\n- [Notifiers](documentation/topics/resources/notifiers.md)","ref":"readme.html#resources"},{"type":"extras","title":"Actions - Home","doc":"- [Actions](documentation/topics/actions/actions.md)\n- [Read Actions](documentation/topics/actions/read-actions.md)\n- [Create Actions](documentation/topics/actions/create-actions.md)\n- [Update Actions](documentation/topics/actions/update-actions.md)\n- [Destroy Actions](documentation/topics/actions/destroy-actions.md)\n- [Generic Actions](documentation/topics/actions/generic-actions.md)\n- [Manual Actions](documentation/topics/actions/manual-actions.md)","ref":"readme.html#actions"},{"type":"extras","title":"Security - Home","doc":"- [Actors & Authorization](documentation/topics/security/actors-and-authorization.md)\n- [Sensitive Data](documentation/topics/security/sensitive-data.md)\n- [Policies](documentation/topics/security/policies.md)","ref":"readme.html#security"},{"type":"extras","title":"Advanced - Home","doc":"- [Multi-step Actions](documentation/topics/advanced/multi-step-actions.md)\n- [Monitoring](documentation/topics/advanced/monitoring.md)\n- [Multitenancy](documentation/topics/advanced/multitenancy.md)\n- [Reactor](documentation/topics/advanced/reactor.md)\n- [Combination Queries](documentation/topics/advanced/combination-queries.md)\n- [Timeouts](documentation/topics/advanced/timeouts.md)\n- [Writing Extensions](documentation/topics/advanced/writing-extensions.md)\n\n---","ref":"readme.html#advanced"},{"type":"extras","title":"How-to - Home","doc":"- [Write Queries](documentation/how-to/write-queries.livemd)\n- [Test Resources](documentation/how-to/test-resources.livemd)\n- [Authorize Access to Resources](documentation/how-to/authorize-access-to-resources.livemd)\n- [Encrypt Attributes](documentation/how-to/encrypt-attributes.livemd)\n- [Prevent Concurrent Writes](documentation/how-to/prevent-concurrent-writes.livemd)\n- [Wrap External APIs](documentation/how-to/wrap-external-apis.livemd)\n- [Define Polymorphic Relationships](documentation/how-to/polymorphic-relationships.livemd)\n\n---","ref":"readme.html#how-to"},{"type":"extras","title":"Reference - Home","doc":"- [Ash.Resource DSL](documentation/dsls/DSL-Ash.Resource.md)\n- [Ash.Domain DSL](documentation/dsls/DSL-Ash.Domain.md)\n- [Ash.Reactor DSL](documentation/dsls/DSL-Ash.Reactor.md)\n- [Ash.Notifier.PubSub DSL](documentation/dsls/DSL-Ash.Notifier.PubSub.md)\n- [Ash.Policy.Authorizer DSL](documentation/dsls/DSL-Ash.Policy.Authorizer.md)\n- [Ash.DataLayer.Ets DSL](documentation/dsls/DSL-Ash.DataLayer.Ets.md)\n- [Ash.DataLayer.Mnesia DSL](documentation/dsls/DSL-Ash.DataLayer.Mnesia.md)\n- [Glossary](documentation/topics/reference/glossary.md)\n- [Expressions](documentation/topics/reference/expressions.md)\n- For other reference documentation, see the sidebar & search bar","ref":"readme.html#reference"},{"type":"extras","title":"Packages - Home","doc":"The Ash ecosystem consists of numerous packages, all of which have their own documentation. If you can't find something in this documentation, don't forget to search in any potentially relevant package.","ref":"readme.html#packages"},{"type":"extras","title":"Data Layers - Home","doc":"- [AshPostgres](https://hexdocs.pm/ash_postgres) | PostgreSQL data layer\n- [AshSqlite](https://hexdocs.pm/ash_sqlite) | SQLite data layer\n- [AshCsv](https://hexdocs.pm/ash_csv) | CSV data layer\n- [AshCubdb](https://hexdocs.pm/ash_cubdb) | CubDB data layer","ref":"readme.html#data-layers"},{"type":"extras","title":"API Extensions - Home","doc":"- [AshJsonApi](https://hexdocs.pm/ash_json_api) | JSON:API builder\n- [AshGraphql](https://hexdocs.pm/ash_graphql) | GraphQL builder","ref":"readme.html#api-extensions"},{"type":"extras","title":"AI - Home","doc":"- [AshAI](https://hexdocs.pm/ash_ai) | Structured Outpus, MCP, Vectorization and more","ref":"readme.html#ai"},{"type":"extras","title":"Web - Home","doc":"- [AshPhoenix](https://hexdocs.pm/ash_phoenix) | Phoenix integrations\n- [AshAuthentication](https://hexdocs.pm/ash_authentication) | Authenticate users with password, OAuth, and more\n- [AshAuthenticationPhoenix](https://hexdocs.pm/ash_authentication_phoenix) | Integrations for AshAuthentication and Phoenix","ref":"readme.html#web"},{"type":"extras","title":"Finance - Home","doc":"- [AshMoney](https://hexdocs.pm/ash_money) | A money data type for Ash\n- [AshDoubleEntry](https://hexdocs.pm/ash_double_entry) | A double entry system backed by Ash Resources","ref":"readme.html#finance"},{"type":"extras","title":"Resource Utilities - Home","doc":"- [AshOban](https://hexdocs.pm/ash_oban) | Background jobs and scheduled jobs for Ash, backed by Oban\n- [AshArchival](https://hexdocs.pm/ash_archival) | Archive resources instead of deleting them\n- [AshStateMachine](https://hexdocs.pm/ash_state_machine) | Create state machines for resources\n- [AshPaperTrail](https://hexdocs.pm/ash_paper_trail) | Keep a history of changes to resources\n- [AshCloak](https://hexdocs.pm/ash_cloak) | Encrypt attributes of a resource","ref":"readme.html#resource-utilities"},{"type":"extras","title":"Admin & Monitoring - Home","doc":"- [AshAdmin](https://hexdocs.pm/ash_admin) | A push-button admin interface\n- [AshAppsignal](https://hexdocs.pm/ash_appsignal) | Monitor your Ash resources with AppSignal","ref":"readme.html#admin-monitoring"},{"type":"extras","title":"Testing - Home","doc":"- [Smokestack](https://hexdocs.pm/smokestack) | Declarative test factories for Ash resources","ref":"readme.html#testing"},{"type":"extras","title":"Examples & External Resources - Home","doc":"- [Ash Real World](https://github.com/team-alembic/realworld) | Ash + Phoenix LiveView codebase containing real world examples (CRUD, auth, advanced patterns, etc)","ref":"readme.html#examples-external-resources"},{"type":"extras","title":"Get Started","doc":"# Get Started\n\n>","ref":"get-started.html"},{"type":"extras","title":"Learn with Livebook - Get Started","doc":">\n> We have a basic step by step tutorial in Livebook that introduces you to Ash. No prior Ash knowledge is required.\n> The Livebook tutorial is self contained and separate from the documentation below.\n> [![Run in Livebook](https://livebook.dev/badge/v1/pink.svg)](https://livebook.dev/run?url=https%3A%2F%2Fgithub.com%2Fash-project%2Fash_tutorial%2Fblob%2Fmaster%2Foverview.livemd)","ref":"get-started.html#learn-with-livebook"},{"type":"extras","title":"Goals - Get Started","doc":"In this guide we will:\n\n1. Create a new Elixir application and add Ash as a dependency\n2. Create a simple set of resources and see how they can be used\n3. Go over some core concepts of Ash\n4. Find out what material might be good to visit next","ref":"get-started.html#goals"},{"type":"extras","title":"Requirements - Get Started","doc":"If you want to follow along yourself, you will need the following things:\n\n1. [Elixir and Erlang installed](https://elixir-lang.org/install.html)\n2. A text editor to make the changes that we make\n3. A terminal to run the examples using `iex`","ref":"get-started.html#requirements"},{"type":"extras","title":"Steps - Get Started","doc":"For this tutorial, we'll use examples based around creating a help desk.\n\nWe will make the following resources:\n\n- `Helpdesk.Support.Ticket`\n- `Helpdesk.Support.Representative`\n\nThe actions we will be able to take on these resources include:\n\n- Opening a new Ticket\n- Closing a Ticket\n- Assigning a Ticket to a representative","ref":"get-started.html#steps"},{"type":"extras","title":"Create a new project - Get Started","doc":"This guide focuses on getting you introduced to Ash quickly. For that reason, we recommend starting a fresh\nproject to explore the concepts. You can, however, add Ash to your existing project if desired.\nSee the options below for more.\n\n\n>","ref":"get-started.html#create-a-new-project"},{"type":"extras","title":"What is igniter? - Get Started","doc":"> \n> Igniter is a code generation and project setup tool that automates the installation and configuration of Elixir packages. Instead of manually adding dependencies and writing boilerplate code, Igniter handles this for you. When you run `mix igniter.install ash`, it automatically adds Ash to your project and sets up the necessary configuration files.\n\n\n<!-- tabs-open -->","ref":"get-started.html#what-is-igniter"},{"type":"extras","title":"New project - Get Started","doc":"First, to use `mix igniter.new`, the archive must be installed.\n\n```bash\n# install igniter.new\nmix archive.install hex igniter_new\n\n# create a new application with Ash in it\nmix igniter.new helpdesk --install ash && cd helpdesk\n```","ref":"get-started.html#new-project"},{"type":"extras","title":"New Phoenix project - Get Started","doc":"If you already know that you want to use Phoenix and Ash together, you can use\n\n```bash\n# install the archive\nmix archive.install hex phx_new\nmix archive.install hex igniter_new\n\n# use the `--with` flag to generate the project with phx.new and add Ash\nmix igniter.new helpdesk --install ash,ash_phoenix --with phx.new && cd helpdesk\n```","ref":"get-started.html#new-phoenix-project"},{"type":"extras","title":"Existing Project - Get Started","doc":"You can use igniter to add Ash to your existing project as well.\n\n```bash\nmix archive.install hex igniter_new\nmix igniter.install ash\n```","ref":"get-started.html#existing-project"},{"type":"extras","title":"Manual Installation Instructions - Get Started","doc":"Finally, if you want to install Ash manually, step by step, follow the [manual installation guide](/documentation/topics/advanced/manual-installation.md).\n\n<!-- tabs-close -->\n\n>","ref":"get-started.html#manual-installation-instructions"},{"type":"extras","title":"Picosat installation issues? - Get Started","doc":">\n> If you have trouble compiling `picosat_elixir`, then alter your `mix.exs` file to replace `{:picosat_elixir, \"~> 0.2\"}` with `{:simple_sat, \"~> 0.1\"}` to use a simpler (but mildly slower) solver. You can always switch back to `picosat_elixir` later once you're done with the tutorial.\n> Then, run `mix deps.get && mix deps.compile ash --force`","ref":"get-started.html#picosat-installation-issues"},{"type":"extras","title":"Building your first Ash Domain - Get Started","doc":"The basic building blocks of an Ash application are Ash resources. They are tied together by a domain module, which will allow you to interact with those resources.","ref":"get-started.html#building-your-first-ash-domain"},{"type":"extras","title":"Creating our first resource - Get Started","doc":">","ref":"get-started.html#creating-our-first-resource"},{"type":"extras","title":"Generators - Get Started","doc":">\n> We have CLI commands that will do this for you, for example `mix ash.gen.resource`\n> In this getting started guide, we will create the resources by hand. This is primarily\n> because there are not actually very many steps, and we want you to be familiar with\n> each moving piece. For more on the generators, run `mix help ash.gen.resource`.\n\nLet's start by creating our first resource along with our first domain. We will create the following files:\n\n- The domain `Helpdesk.Support`, in `lib/helpdesk/support.ex`\n- Our Ticket resource `Helpdesk.Support.Ticket`, in `lib/helpdesk/support/ticket.ex`.\n\nTo create the required folders and files, you can use the following command in your terminal:\n\n```bash\nmkdir -p lib/helpdesk/support && \\\n  touch $_/ticket.ex && \\\n  touch lib/helpdesk/support.ex\n```\n\nYour project structure should now include the following files:\n\n```\nlib/\n├─ helpdesk/\n│  ├─ support/\n│  │  ├─ ticket.ex\n│  ├─ support.ex\n```\n\nAdd the following to the files we created\n\n```elixir\n# lib/helpdesk/support.ex\n\ndefmodule Helpdesk.Support do\n  use Ash.Domain\n\n  resources do\n    resource Helpdesk.Support.Ticket\n  end\nend\n```\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\ndefmodule Helpdesk.Support.Ticket do\n  # This turns this module into a resource\n  use Ash.Resource, domain: Helpdesk.Support\n\n  actions do\n    # Use the default implementation of the :read action\n    defaults [:read]\n\n    # and a create action, which we'll customize later\n    create :create\n  end\n\n  # Attributes are the simple pieces of data that exist on your resource\n  attributes do\n    # Add an autogenerated UUID primary key called `:id`.\n    uuid_primary_key :id\n\n    # Add a string type attribute called `:subject`\n    attribute :subject, :string\n  end\nend\n```\n\nNext, add your domain to your `config.exs`, and configure some backwards compatibility configuration.\n\nRun the following to create your `config.exs` if it doesn't already exist\n\n```elixir\nmkdir -p config\ntouch config/config.exs\n```\n\nand add the following contents to it.\n\n```elixir\n# in config/config.exs\nimport Config\n\n# This tells Ash about your domain, which is used to find\n# available resource and actions\nconfig :helpdesk, :ash_domains, [Helpdesk.Support]\n```","ref":"get-started.html#generators"},{"type":"extras","title":"Try our first resource out - Get Started","doc":"Run `iex -S mix` in your project's root directory and try out the following.\n\nTo create a ticket, we first make an `Ash.Changeset` for the `:create` action of the `Helpdesk.Support.Ticket` resource. Then we pass it to the `Ash.create!/1` function.\n\n```elixir\nHelpdesk.Support.Ticket\n|> Ash.Changeset.for_create(:create)\n|> Ash.create!()\n```\n\nThis returns what we call a `record` which is an instance of a resource.\n\n```elixir\n#Helpdesk.Support.Ticket<\n  ...,\n  id: \"c0f8dc32-a018-4eb4-8656-d5810118f4ea\",\n  subject: nil,\n  ...\n>\n```","ref":"get-started.html#try-our-first-resource-out"},{"type":"extras","title":"Customizing our Actions - Get Started","doc":"One thing you may have noticed earlier is that we created a ticket without providing any input, and as a result our ticket had a `subject` of `nil`. Additionally, we don't have any other data on the ticket. Lets add a `status` attribute, ensure that `subject` can't be `nil`, and provide a better interface by giving the `:create` action a better name, and accepting `:subject` as part of the action.\n\nWe'll start with the attribute changes:\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nattributes do\n  ...\n  attribute :subject, :string do\n    # Don't allow `nil` values\n    allow_nil? false\n\n    # Allow this attribute to be public. By default, all attributes are private.\n    public? true\n  end\n\n  # status is either `open` or `closed`. We can add more statuses later\n  attribute :status, :atom do\n    # Constraints allow you to provide extra rules for the value.\n    # The available constraints depend on the type\n    # See the documentation for each type to know what constraints are available\n    # Since atoms are generally only used when we know all of the values\n    # it provides a `one_of` constraint, that only allows those values\n    constraints [one_of: [:open, :closed]]\n\n    # The status defaulting to open makes sense\n    default :open\n\n    # We also don't want status to ever be `nil`\n    allow_nil? false\n  end\nend\n```\n\nAnd then replace the `:create` action with `:open`, and accept `:subject` as input.\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nactions do\n  ...\n  create :open do\n    accept [:subject]\n  end\nend\n```\n\nLet's try these changes in `iex`:\n\nWe use `create!` with an exclamation point here because that will raise the error which gives a nicer view of the error in `iex`\n\n```elixir\n# Use this to pick up changes you've made to your code, or restart your session\nrecompile()\n\nHelpdesk.Support.Ticket\n|> Ash.Changeset.for_create(:open, %{subject: \"My mouse won't click!\"})\n|> Ash.create!()\n```\n\nAnd we can see our newly created ticket with a subject and a status.\n\n```elixir\n#Helpdesk.Support.Ticket<\n  ...\n  id: \"3c94d310-7b5e-41f0-9104-5b193b831a5d\",\n  status: :open,\n  subject: \"My mouse won't click!\",\n  ...\n>\n```\n\nIf we didn't include a subject, or left off the arguments completely, we would see an error instead\n\n```text\n** (Ash.Error.Invalid) Invalid Error\n\n* attribute subject is required\n```","ref":"get-started.html#customizing-our-actions"},{"type":"extras","title":"Updates and validations - Get Started","doc":"Now let's add some logic to close a ticket. This time we'll add an `update` action.\n\nHere we will use a `change`. Changes allow you to customize how an action executes with very fine-grained control. There are built-in changes that are automatically available as functions, but you can define your own and pass it in as shown below. You can add multiple, and they will be run in order. See the `Ash.Changeset` module documentation for more.\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nactions do\n  ...\n  update :close do\n    # We don't want to accept any input here\n    accept []\n\n    validate attribute_does_not_equal(:status, :closed) do\n      message \"Ticket is already closed\"\n    end\n\n    change set_attribute(:status, :closed)\n    # A custom change could be added like so:\n    #\n    # change MyCustomChange\n    # change {MyCustomChange, opt: :val}\n  end\nend\n```\n\nTry out opening and closing a ticket in `iex`:\n\n```elixir\n# Use this to pick up changes you've made to your code, or restart your session\nrecompile()\n\n# parenthesis so you can paste into iex\nticket = (\n  Helpdesk.Support.Ticket\n  |> Ash.Changeset.for_create(:open, %{subject: \"My mouse won't click!\"})\n  |> Ash.create!()\n)\n\nticket = (\n  ticket\n  |> Ash.Changeset.for_update(:close)\n  |> Ash.update!()\n)\n\n#Helpdesk.Support.Ticket<\n  ...\n  status: :closed,\n  subject: \"My mouse won't click!\",\n  ...\n>\n```\n\nWhat if we would try to close the ticket again?\n\n```elixir\nticket\n|> Ash.Changeset.for_update(:close)\n|> Ash.update!()\n```\n\nThe application would halt with an error:\n\n```text\n...\n** (Ash.Error.Invalid)\nBread Crumbs:\n  > Returned from bulk query update: Helpdesk.Support.Ticket.close\n\nInvalid Error\n\n* Invalid value provided for status: Ticket is already closed.\n...\n```\nNote the 'Ticket is already closed' message that we have defined in the ```attribute_does_not_equal``` validation.\n\nAnd if we'd use the non-bang version:\n\n```elixir\nticket\n|> Ash.Changeset.for_update(:close)\n|> Ash.update()\n```\nwe get an error tuple with a %Ash.Error.Invalid struct that contains the message:\n\n```text\n{:error,\n %Ash.Error.Invalid{\n...\n       message: \"Ticket is already closed\",\n...\n```","ref":"get-started.html#updates-and-validations"},{"type":"extras","title":"Querying without persistence - Get Started","doc":"So far we haven't used a data layer that does any persistence, like storing records in a database. All that this simple resource does is return the record back to us. You can see this lack of persistence by attempting to use a `read` action:\n\n```elixir\nAsh.read!(Helpdesk.Support.Ticket)\n```\n\nWhich will raise an error explaining that there is no data to be read for that resource.\n\nIn order to save our data somewhere, we need to add a data layer to our resources. Before we do that, however, let's go over how Ash allows us to work against many different data layers (or even no data layer at all).\n\nResources without a data layer will implicitly be using `Ash.DataLayer.Simple`. This data is not persisted anywhere, and must be provided when running queries. It provides a utility for just this purpose, `Ash.DataLayer.Simple.set_data/2`.\n\nTry the following in `iex`. We will open some tickets, and close some of them, and then use `Ash.DataLayer.Simple.set_data/2` to use those tickets.\n\n```elixir\n# Ash.Query is a macro, so it must be required\nrequire Ash.Query\n\ntickets =\n  for i <- 0..5 do\n    ticket =\n      Helpdesk.Support.Ticket\n      |> Ash.Changeset.for_create(:open, %{subject: \"Issue #{i}\"})\n      |> Ash.create!()\n\n    if rem(i, 2) == 0 do\n      ticket\n      |> Ash.Changeset.for_update(:close)\n      |> Ash.update!()\n    else\n      ticket\n    end\n  end\n```\n\nFind the tickets where the subject contains `\"2\"`. Note that the we're setting the ticket data that we're querying using `set_data`.\n\n```elixir\nHelpdesk.Support.Ticket\n|> Ash.Query.filter(contains(subject, \"2\"))\n|> Ash.DataLayer.Simple.set_data(tickets)\n|> Ash.read!()\n```\n\nFind the tickets that are _closed_ and their subject does _not_ contain `\"4\"`\n\n```elixir\nHelpdesk.Support.Ticket\n|> Ash.Query.filter(status == :closed and not(contains(subject, \"4\")))\n|> Ash.DataLayer.Simple.set_data(tickets)\n|> Ash.read!()\n```\n\nThe examples above could be easily implemented with `Enum.filter`, but the real power here is to allow you to use the same tools when working with any data layer.\n\nEven though it doesn't persist data in any way, `Ash.DataLayer.Simple` can be useful to model static data, or be used for resources where all the actions are manual and inject data from other sources.","ref":"get-started.html#querying-without-persistence"},{"type":"extras","title":"Adding basic persistence - Get Started","doc":"Before we get into working with relationships, let's add some real persistence to our resource. This will let us add relationships and try out querying data.\n\nThere is a built in data layer that is useful for testing and prototyping, that uses [ETS](https://elixir-lang.org/getting-started/mix-otp/ets.html). ETS (Erlang Term Storage) is OTP's in-memory database, so the data won't actually stick around beyond the lifespan of your program, but it's a simple way to try things out.\n\nTo add it to your resource, modify it like so:\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nuse Ash.Resource,\n  domain: Helpdesk.Support,\n  data_layer: Ash.DataLayer.Ets\n```\n\nNow we can slightly modify our code above, by removing the `Ash.DataLayer.Simple.set_data/2` calls, and we can see our persistence in action. Remember, ETS is in-memory, meaning restarting your application/iex session will remove all of the data.\n\n```elixir\n# Use this to pick up changes you've made to your code, or restart your session\nrecompile()\n\nrequire Ash.Query\n\nfor i <- 0..5 do\n  ticket =\n    Helpdesk.Support.Ticket\n    |> Ash.Changeset.for_create(:open, %{subject: \"Issue #{i}\"})\n    |> Ash.create!()\n\n  if rem(i, 2) == 0 do\n    ticket\n    |> Ash.Changeset.for_update(:close)\n    |> Ash.update!()\n  end\nend\n\n# Show the tickets where the subject contains \"2\"\nHelpdesk.Support.Ticket\n|> Ash.Query.filter(contains(subject, \"2\"))\n|> Ash.read!()\n\n# Show the tickets that are closed and their subject does not contain \"4\"\nHelpdesk.Support.Ticket\n|> Ash.Query.filter(status == :closed and not(contains(subject, \"4\")))\n|> Ash.read!()\n```","ref":"get-started.html#adding-basic-persistence"},{"type":"extras","title":"Adding relationships - Get Started","doc":"Now we want to be able to assign a Ticket to a Representative. First, let's create the Representative resource:\n\n```elixir\n# lib/helpdesk/support/representative.ex\n\ndefmodule Helpdesk.Support.Representative do\n  # This turns this module into a resource using the in memory ETS data layer\n  use Ash.Resource,\n    domain: Helpdesk.Support,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    # Add the default simple actions\n    defaults [:read]\n\n    create :create do\n      accept [:name]\n    end\n  end\n\n  # Attributes are the simple pieces of data that exist on your resource\n  attributes do\n    # Add an autogenerated UUID primary key called `:id`.\n    uuid_primary_key :id\n\n    # Add a string type attribute called `:name`\n    attribute :name, :string do\n      # Make the attribute public in order to give a name when calling functions from `Ash.Changeset`.\n      public? true\n    end\n  end\n\n  relationships do\n    # `has_many` means that the destination attribute is not unique, therefore many related records could exist.\n    # We assume that the destination attribute is `representative_id` based\n    # on the module name of this resource and that the source attribute is `id`.\n    has_many :tickets, Helpdesk.Support.Ticket\n  end\nend\n```\n\nNow let's modify our Ticket resource to have the inverse relationship to the Representative.\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nrelationships do\n  # belongs_to means that the destination attribute is unique, meaning only one related record could exist.\n  # We assume that the destination attribute is `representative_id` based\n  # on the name of this relationship and that the source attribute is `representative_id`.\n  # We create `representative_id` automatically.\n  belongs_to :representative, Helpdesk.Support.Representative\nend\n```\n\nFinally, let's add our new Representative resource to our domain module\n\n```elixir\n# lib/helpdesk/support.ex\n\nresources do\n ...\n resource Helpdesk.Support.Representative\nend\n```\n\nYou may notice that if you don't add the resource to your domain, or if you don't add the `belongs_to` relationship, that you'll get helpful errors at compile time. Helpful compile time validations are a core concept of Ash as we really want to ensure that your application is valid.","ref":"get-started.html#adding-relationships"},{"type":"extras","title":"Working with relationships - Get Started","doc":"The simplest way to work with belongs to relationships is to allow directly editing the underlying id field.\n\n>","ref":"get-started.html#working-with-relationships"},{"type":"extras","title":"managing relationships - Get Started","doc":">\n> There are a wide array of options when managing relationships, and we won't cover all of them here. See the [Managing Relationships guide](/documentation/topics/resources/relationships.md#managing-relationships) for more.\n\nAdd the `assign` action to allow us to assign a Ticket to a Representative.\n\n```elixir\n# lib/helpdesk/support/ticket.ex\n\nupdate :assign do\n  accept [:representative_id]\nend\n```\n\nLet's try it out in our `iex` console!\n\nUse `recompile` to pick up changes you've made to your code, or just restart your session.\n\n```elixir\nrecompile()\n```","ref":"get-started.html#managing-relationships"},{"type":"extras","title":"Open a Ticket - Get Started","doc":"```elixir\nticket = (\n  Helpdesk.Support.Ticket\n  |> Ash.Changeset.for_create(:open, %{subject: \"I can't find my hand!\"})\n  |> Ash.create!()\n)\n```","ref":"get-started.html#open-a-ticket"},{"type":"extras","title":"Create a Representative - Get Started","doc":"```elixir\nrepresentative = (\n  Helpdesk.Support.Representative\n  |> Ash.Changeset.for_create(:create, %{name: \"Joe Armstrong\"})\n  |> Ash.create!()\n)\n```","ref":"get-started.html#create-a-representative"},{"type":"extras","title":"Assign that Representative to the Ticket - Get Started","doc":"```elixir\nticket\n|> Ash.Changeset.for_update(:assign, %{representative_id: representative.id})\n|> Ash.update!()\n```","ref":"get-started.html#assign-that-representative-to-the-ticket"},{"type":"extras","title":"What next? - Get Started","doc":"What you've seen above barely scratches the surface of what Ash can do. In a lot of ways, it will look very similar to other tools that you've seen. If all that you ever used was the above, then realistically you won't see much benefit to using Ash.\n\nWhere Ash shines however, is all of the tools that can work _with_ your resources. You have the ability to extend the framework yourself, and apply consistent design patterns that enable unparalleled efficiency, power and flexibility as your application grows.\n\n#### Get Help\n\n- Check out [ElixirForum](https://elixirforum.com/c/ash-framework-forum)\n- Join our [Discord server](https://discord.gg/D7FNG2q)\n- Open a [GitHub issue](https://github.com/ash-project/ash/issues/new/choose)\n\n#### Persist your data\n\nSee [The AshPostgres getting started guide](https://hexdocs.pm/ash_postgres) to see how to back your resources with Postgres.\nThis is highly recommended, as the Postgres data layer provides tons of advanced capabilities.\n\n#### Add a web API\n\nCheck out [AshJsonApi](https://hexdocs.pm/ash_json_api) and [AshGraphql](https://hexdocs.pm/ash_graphql) extensions to build APIs around your resource\n\n#### Authorize access and work with users\n\nSee [AshAuthentication](https://hexdocs.pm/ash_authentication) for setting up users and allowing them to\nlog in. It supports password, magic link, oauth (google, github, apple etc.) out of the box!\n\nSee the [Policies guide](/documentation/topics/security/policies.md) for information on how to authorize access to your resources using actors and policies.\n\n#### Clean up your code that uses Ash?\n\nCreating and using changesets manually can be verbose, and they all look very similar. Luckily, Ash has your back and can help you build high quality interfaces for you!\n\nCheck out the [Code Interface Guide](/documentation/topics/resources/code-interfaces.md) to derive things like `Helpdesk.Support.Ticket.assign!(representative.id)`","ref":"get-started.html#what-next"},{"type":"extras","title":"What is Ash?","doc":"# What is Ash?\n\nAsh is an opinionated, declarative application framework that brings the batteries-included experience to Elixir. It shines when building web apps, APIs and services, but can be used for any kind of Elixir application. It integrates with the best that the Elixir ecoystem has to offer, often used with Phoenix and PostgreSQL, slotting directly into a standard Elixir codebase. Ash is built for velocity at day 1, but also for maintainability at year 5, a place where many frameworks and tools leave you high and dry.\n\n> Through its declarative extensibility, Ash delivers more than you'd expect: Powerful APIs with filtering/sorting/pagination/calculations/aggregations, pub/sub, authorization, rich introspection, GraphQL... It's what empowers this solo developer to build an ambitious ERP!\n>\n> — Frank Dugan III, System Specialist, SunnyCor Inc.\n\nAt its heart, Ash is a framework for modeling your application's domain through **Resources** and their **Actions**. These are the fundamental abstractions that everything else builds upon.","ref":"what-is-ash.html"},{"type":"extras","title":"Why Ash? - What is Ash?","doc":"If you've ever built software professionally, you've almost certainly experienced one or more of the following:\n\n**The problem**:\n- **Repetitive work** - The same business logic scattered across our application\n- **Inconsistencies** - Different parts of your app handling the same data differently\n- **Maintenance burden** - Changing one thing requires updating five different places\n- **Knowledge silos** - Each developer builds everything slightly differently, leading to inconsistencies and inefficiencies.\n\n**Ash's solution**: Model your application's **behavior** first, as data, and derive everything else automatically. Ash resources center around **actions** that represent domain logic. Instead of exposing raw data models, you define meaningful operations like `:publish_post`, `:approve_order`, or `:calculate_shipping_cost` that encapsulate your business logic, validation, and authorization rules. This is coupled with a rich suite of extensions to fill the most common needs of Elixir applications.\n\n> Ash fills the gap that brings Phoenix up to feature parity with a batteries included framework like Django.\n> Ash Admin (Django admin), Ash Resource & Domain (Django models & ORM), AshJsonApi (Django REST Framework), Ash Authentication (Django Allauth), Ash Phoenix (Django Forms), Ash Policies (Django Permissions)\n>\n> But you aren't required to use Phoenix with an Ash project. Ash will happily work as a standalone CLI, terminal app or some other Elixir web framework that comes out tomorrow.\n>\n> Scott Woodall - Principal Software Engineer, Microsoft","ref":"what-is-ash.html#why-ash"},{"type":"extras","title":"Built for Flexibility - What is Ash?","doc":"Ash was born out of the battle-scars from inflexible abstractions that eventually paint you into a corner. That's why Ash is designed with **multi-tiered configurability** and **escape hatches all the way down**. Instead of deciding that abstraction was bad, we decided that it just needed to be done better. Elixir & the BEAM have our backs, providing a solid foundation upon which to build high quality applications.\n\n**You're never locked out of custom behavior**. Need to customize how an action works? Use preparations, changes, and validations. Need to override how data is fetched? Implement a manual action. Need to completely bypass Ash for a specific operation? Drop down to Ecto or raw SQL. Need to extend Ash itself? Use the same extension toolkit that powers AshPostgres and AshGraphql.\n\nThis isn't accidental - it's core to Ash's design. We provide powerful defaults that work for 80% of cases, extensive configuration options for the next 15%, and escape hatches for the remaining 5%. Your Ash application is just an Elixir application, so when you need to do something completely custom, Ash won't get in your way.\n\nThe framework acts as a **spinal cord** for your application: providing structure and coordination while allowing complete customization at every level.","ref":"what-is-ash.html#built-for-flexibility"},{"type":"extras","title":"Essential Context - What is Ash?","doc":"<!-- tabs-open -->","ref":"what-is-ash.html#essential-context"},{"type":"extras","title":"Elixir Developers - What is Ash?","doc":"**Ash builds on the best of the Elixir ecosystem** rather than replacing it. Ash leverages Ecto for database operations, Phoenix for web interfaces, Oban for background jobs, Absinthe for GraphQL, etc. Ash unifies all of this tooling and significantly simplifies and enhances they way you use them together.\n\n**Compile-time guarantees** come from Ash's declarative nature - many errors that would be runtime failures in other frameworks become compile-time checks in Ash.\n\n**The ecosystem advantage**: Instead of learning entirely new patterns, Ash enhances familiar Elixir concepts. Your existing knowledge of pattern matching, process supervision, and OTP principles all apply - Ash just provides structure and automation on top, acting as a spinal cord for your application.","ref":"what-is-ash.html#elixir-developers"},{"type":"extras","title":"Non-Elixir Developers - What is Ash?","doc":"**Elixir** is a functional programming language built on the Erlang VM, known for fault-tolerance and concurrency. It's used by WhatsApp, Discord, and Pinterest for systems that need high availability. If you know Ruby or Python, think of Elixir as optimized specifically for distributed, concurrent systems.\n\n**Functional vs Object-Oriented**: Unlike OOP frameworks, Elixir (and Ash) focus on transforming data through functions rather than encapsulating data in objects. This leads to more predictable, testable code.\n\n**Immutability** means data doesn't change in-place - instead, you create new versions. This eliminates many common bugs around shared state and makes concurrent operations safer.\n\n**Actor Model Concurrency**: Instead of threads and locks, Elixir uses lightweight processes (actors) that communicate via messages. This makes Ash naturally suited for high-concurrency applications.\n\n**Modules** (`defmodule`) are Elixir's equivalent to classes/namespaces - they group related functions together. Unlike OOP classes, they don't hold state.","ref":"what-is-ash.html#non-elixir-developers"},{"type":"extras","title":"New Programmers - What is Ash?","doc":"**Programming languages** like Elixir are tools for writing instructions that computers can follow. Elixir is specifically designed for building web applications that can handle lots of users at once.\n\n**Modules** (the `defmodule` blocks you'll see) are containers that group related code together. Think of them like chapters in a book - each chapter covers a specific topic.","ref":"what-is-ash.html#new-programmers"},{"type":"extras","title":"Business Leaders - What is Ash?","doc":"**Ash reduces software development costs** by eliminating repetitive code. When your development team defines a business process once, Ash automatically generates the database structure, API endpoints, and user interfaces - work that normally requires separate specialists.\n\n**Faster time-to-market** because features that typically take weeks can be built in days. Adding a new feature like \"customer reviews\" or \"order tracking\" requires defining the business rules once, rather than building separate systems for web, mobile, and internal tools.\n\n**Lower maintenance costs** because changes to business rules automatically update all related systems. When you change how orders work, your database, APIs, and documentation stay synchronized without manual updates.\n\n**Reduced technical risk** through built-in security, data validation, and error handling. Your team spends time building your competitive advantages instead of solving the same infrastructure problems every software company faces.\n\n**Developer productivity** increases because the framework handles the \"plumbing\" while your team focuses on what makes your business unique. This means you need fewer developers to build the same functionality.\n\n**Eliminates technical debt** through enforced consistency. When every part of your application follows the same patterns, there's no accumulation of \"quick fixes\" and inconsistent approaches that slow down future development and create maintenance headaches.\n\n**Faster developer onboarding** because new team members learn one set of patterns that apply everywhere. Instead of each developer building things differently, Ash provides a shared vocabulary and approach that new hires can quickly understand and contribute to. Ash is still niche, so developers may not know it right out of the gate, but if you think of Ash as a replacement for your internal framework, which are universally poorly documented and hard to train on, you can see the benefit of Ash being an open source, well documented project with a strong community.\n\n<!-- tabs-close -->","ref":"what-is-ash.html#business-leaders"},{"type":"extras","title":"Resources and Actions: The Core Abstractions - What is Ash?","doc":"The foremost abstraction in Ash is **Actions** - the things you can do in your domain like `:create_user`, `:publish_post`, `:approve_order`, or `:calculate_shipping`. These actions are organized into **Resources** that group related behaviors around domain concepts like `User`, `Post`, `Order`, or `Invoice`. Using resources, you can easily model actions, alongside the state that they operate on, or just actions in isolation.\n\nThese actions are **introspectable** and **fully typed**. This means the rest of your application (and **extensions** - add-on packages that enhance Ash) can automatically understand and build functionality around them. When you define a `create` action that accepts a `:title` string and `:content` text, extensions like AshGraphql can automatically generate GraphQL mutations, AshJsonApi can create REST endpoints, and AshPostgres can handle database persistence - all without additional configuration.\n\nThis **declarative approach** means your resources become the single source of truth for your entire application. Database schemas, API endpoints, authorization rules, state machines, background jobs, and more all stem directly from your resource definitions.","ref":"what-is-ash.html#resources-and-actions-the-core-abstractions"},{"type":"extras","title":"Beyond Simple CRUD - What is Ash?","doc":"Ash is not a web framework, like Phoenix or Rails. It is a framework for building your application layer, independent of how it is exposed or consumed. It is not an **alternative** to frameworks like Phoenix, rather a **complement** to them.\n\n> Ash Framework enabled us to build a robust platform for delivering financial services using bitcoin. Ash proved itself to our team by handling innovative use cases with ease and it continues to evolve ahead of our growing list of needs.\n>\n> — Yousef Janajri, CTO & Co-Founder, Coinbits\n\nThe intent behind Ash is _not_ to have you building simple CRUD-style applications, although we do provide conveniences for these cases. The real power comes from defining rich, domain-specific actions with meaningful names like `:publish_post`, `:approve_order`, or `:calculate_shipping`. These actions encapsulate your business logic and can be composed, validated, authorized, and extended in powerful ways.\n\n> #### Model your domain, derive the rest {: .info}\n>\n> Ash derives significant portions of your application directly from your resources, with little to no effort required. This allows you to focus on what matters most: your business logic.\n> We leverage the best of the Elixir ecosystem under the hood, providing a single unified tool-chain for our users.","ref":"what-is-ash.html#beyond-simple-crud"},{"type":"extras","title":"Community - What is Ash?","doc":"Ash has a vibrant community of developers who contribute to the project, provide support, and share knowledge. Join us on [Discord](https://discord.gg/w3AXeARR2p), [ElixirForum](https://elixirforum.com/ash) and [GitHub](https://github.com/ash-project/ash) to contribute, ask questions, and stay updated on the latest developments.\n\nOur community is one of the best features of Ash and **you should use it**. Lots of folks using Ash in production, with a shared mission of making better software.\n\n> I'm constantly blown away with the quality of work and support the Ash community has put into this project. It's gotten to the point that I can't imagine starting a new Elixir project that doesn't use Ash.\n>\n> — Brett Kolodny, Full stack engineer, MEW","ref":"what-is-ash.html#community"},{"type":"extras","title":"An Example: From Simple to Sophisticated - What is Ash?","doc":"<!-- tabs-open -->","ref":"what-is-ash.html#an-example-from-simple-to-sophisticated"},{"type":"extras","title":"Introduction - What is Ash?","doc":"If you're the \"just show me the code\" type, click through these tabs to see an example of evolving a blog post resource over time with Ash & its extensions to get a sense for how it works.\n\n> #### Don't worry about understanding all the code {: .info}\n>\n> You're not expected to understand every detail of the syntax - focus on the concepts and how it changes over time. If you want a gentler introduction, head over to the [getting started](/documentation/tutorials/get-started.md) guide.","ref":"what-is-ash.html#introduction"},{"type":"extras","title":"Actions - What is Ash?","doc":"```elixir\n# lib/my_blog/blog.ex\ndefmodule MyBlog.Blog do\n  use Ash.Domain\n\n  resources do\n    resource MyBlog.Blog.Post do\n      # Defines the `analyze_text/1` function which calls\n      # the action of the same name on the Post resource.\n      define :analyze_text, args: [:text]\n    end\n  end\nend\n\n# lib/my_blog/blog/post.ex\ndefmodule MyBlog.Blog.Post do\n  use Ash.Resource\n\n  actions do\n    # Start with pure behavior - a simple action that processes text\n    action :analyze_text, :map do\n      argument :text, :string, allow_nil?: false\n\n      run fn input, _context ->\n        text = input.arguments.text\n        words = String.split(text)\n\n        analysis = %{\n          word_count: length(words),\n          character_count: String.length(text),\n          estimated_reading_time: div(length(words), 200) + 1\n        }\n\n        {:ok, analysis}\n      end\n    end\n  end\nend\n```\n\nNotice how we have not defined any \"state\" here. A common misconception is that the purpose of Ash is to abstract state. Ash provides tons of useful features around state, but those are features *on top of* the core concept, which centers around typed actions.\n\n```elixir\n{:ok, analysis} = MyBlog.Blog.analyze_text(\"This is some sample blog content to analyze.\")\n# => {:ok, %{word_count: 9, character_count: 49, estimated_reading_time: 1}}\n```\n\n**Why not just write a regular function?** You could write `def analyze_text(text)` and get the same result. But here's what the Ash action gives you that a function doesn't:\n\n- **Type safety** - Arguments are automatically validated (try passing a number instead of string)\n- **Introspection** - Other tools can discover this action exists and what it does\n- **Extensibility** - You can add authorization, logging, or other behaviors later without changing callers\n- **API generation** - Extensions can automatically expose this as a REST endpoint or GraphQL query\n- **Consistent interface** - All actions work the same way, making your codebase predictable\n\nThe action is **typed** (it knows it takes a string and returns a map) and **introspectable** (your application can examine it at runtime). This means extensions can automatically understand and build on top of it.\n\n>","ref":"what-is-ash.html#actions"},{"type":"extras","title":"You could stop here - What is Ash?","doc":">\n> You could stop at this step and still derive significant value from Ash. You wouldn't be\n> using it wrong. You can build your own custom state system under the hood, use Phoenix contexts,\n> call directly into Ecto, totally up to you.","ref":"what-is-ash.html#you-could-stop-here"},{"type":"extras","title":"Persistence - What is Ash?","doc":"Now let's add state to support persistent storage, while keeping our existing behavior:\n\n```elixir\n# lib/my_blog/blog.ex\ndefmodule MyBlog.Blog do\n  use Ash.Domain\n\n  resources do\n    resource MyBlog.Blog.Post do\n      define :analyze_text, args: [:text]\n      define :create_post, action: :create, args: [:title, :content]\n    end\n  end\nend\n\n# lib/my_blog/blog/post.ex\ndefmodule MyBlog.Blog.Post do\n  use Ash.Resource,\n    domain: MyBlog.Blog,\n    data_layer: AshPostgres.DataLayer # data_layer tells Ash where to store data\n\n  postgres do\n    table \"posts\"\n    repo MyBlog.Repo\n  end\n\n  attributes do\n    uuid_primary_key :id\n    attribute :title, :string, allow_nil?: false, public?: true\n    attribute :content, :string, public?: true\n    attribute :status, :atom, constraints: [one_of: [:draft, :published]], default: :draft, public?: true\n\n    create_timestamp :created_at, public?: true\n    update_timestamp :updated_at, public?: true\n  end\n\n  actions do\n    action :analyze_text, :map do\n      argument :text, :string, allow_nil?: false\n\n      run fn input, _context ->\n        text = input.arguments.text\n        words = String.split(text)\n\n        analysis = %{\n          word_count: length(words),\n          character_count: String.length(text),\n          estimated_reading_time: div(length(words), 200) + 1\n        }\n\n        {:ok, analysis}\n      end\n    end\n\n    defaults [:read, :destroy, create: [:title, :content], update: [:title, :content, :status]]\n  end\nend\n```\n\nNow your resource combines behavior and state. The original `analyze_text` action still works, plus you can create and persist posts:\n\n```elixir\n# Behavior still works exactly the same\n{:ok, analysis} = MyBlog.Blog.analyze_text(\"Some text to analyze\")\n\n# Now we can also persist state\n{:ok, post} = MyBlog.Blog.create_post(\"My First Post\", \"This is some content\")\n```","ref":"what-is-ash.html#persistence"},{"type":"extras","title":"GraphQL - What is Ash?","doc":"This is just one example of an API extension. We also have [ash_json_api](https://hexdocs.pm/ash_json_api) with more on the way.\nAsh also comes with all the tools you need to build *your own* API extension.\n\n```elixir\n# Add to your domain\ndefmodule MyBlog.Blog do\n  use Ash.Domain,\n    extensions: [AshGraphql.Domain]\n\n  graphql do\n    queries do\n      action MyApp.Blog.Post, :analyze_text, :analyze_text\n    end\n\n    mutations do\n      create MyApp.Blog.Post, :create_post, :create\n    end\n  end\n\n  # ... resources\nend\n\n# Add to your resource\ndefmodule MyBlog.Blog.Post do\n  use Ash.Resource,\n    domain: MyBlog.Blog,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshGraphql.Resource]\n\n  graphql do\n    # you just tell us the name of the type\n    # we can take care of the rest\n    # there is deep configuration with plenty of escape hatches\n    # for when you need something bespoke\n    type :post\n  end\n\n  # ... rest of resource definition\nend\n```\n\nNow you have a full GraphQL API with queries, mutations, and custom actions automatically generated from your resource definition.\nThat is actually all of the code you need to do it.\n\n> The ease of defining our domain model and configuring Ash to generate a powerful GraphQL API has been a game-changer. What used to be complex and time-consuming has become simplicity itself.\n>\n> — Alan Heywood, CTO, HereTask","ref":"what-is-ash.html#graphql"},{"type":"extras","title":"Encryption - What is Ash?","doc":"[Cloak](https://hexdocs.pm/cloak) is a powerful library for encrypting data at rest and in transit. Ash provides a first class extension that integrates with it directly.\n\n```elixir\ndefmodule MyBlog.Blog.Post do\n  use Ash.Resource,\n    domain: MyBlog.Blog,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshGraphql.Resource, AshCloak.Resource]\n\n  cloak do\n    vault MyBlog.Vault\n    attributes [:content]  # Encrypt the content attribute\n  end\n\n  # ... rest of resource definition\nend\n```\n\nYour post content is now automatically encrypted at rest and decrypted when read, with no changes to your existing API or business logic.","ref":"what-is-ash.html#encryption"},{"type":"extras","title":"State Machines - What is Ash?","doc":"A state machine is a way to model the valid states for some piece of data. It allows you to define the states a record can be in, and the transitions between those states.\n\n```elixir\ndefmodule MyBlog.Blog.Post do\n  use Ash.Resource,\n    domain: MyBlog.Blog,\n    data_layer: AshPostgres.DataLayer,\n    extensions: [AshGraphql.Resource, AshCloak.Resource, AshStateMachine]\n\n  state_machine do\n    initial_states [:draft]\n    default_initial_state :draft\n\n    transitions do\n      transition :publish, from: :draft, to: :published\n      transition :unpublish, from: :published, to: :draft\n    end\n  end\n\n  actions do\n    # ... same actions as before\n    update :publish do\n      change transition_state(:published)\n      # ... additional custom logic on publish\n    end\n\n    update :unpublish do\n      change transition_state(:draft)\n      # ... additional custom logic on publish\n    end\n  end\n\n  # ... rest of resource definition\nend\n```\n\nNow your posts have a proper state machine with transition actions, state validation, and automatic GraphQL mutations for state changes.\n\n<!-- tabs-close -->","ref":"what-is-ash.html#state-machines"},{"type":"extras","title":"The Ash Advantage - What is Ash?","doc":"This example demonstrates Ash's core philosophy: **Model your domain, derive the rest**. Notice how:\n\n- The resource definition remained largely unchanged as we added each extension\n- Each extension automatically understood and enhanced the existing actions\n- Complex functionality (encryption, state machines, APIs) required minimal configuration\n- Everything remains introspectable and type-safe\n- Your business logic stays focused on the domain, not infrastructure concerns\n- These extensions are built with the same suite of tools we provide *to the user* for extending Ash.\n  Those using Ash will often end up crafting their own extensions, making the framework truly their own.\n\nExtensions work together seamlessly because they all operate on the same well-defined resource and action abstractions. In the example above, we went from a single action to a full-featured system with database persistence, GraphQL API, encryption, and state management - by adding configuration, not writing code.\n\n**The productivity gain is measurable**: What normally requires separate database migration files, API endpoint definitions, validation logic, GraphQL schema files, and state management code becomes a single, cohesive resource definition that stays automatically synchronized.","ref":"what-is-ash.html#the-ash-advantage"},{"type":"extras","title":"Design Principles","doc":"# Design Principles\n\nThe design principles behind Ash allows us to build an extremely flexible and powerful set of tools, without locking users into specific choices at any level. The framework acts as a spinal cord for your application, with extension points at every level to allow for custom behavior. What follows are the core tenets behind Ash Framework.","ref":"design-principles.html"},{"type":"extras","title":"Anything, not Everything - Design Principles","doc":"\"Anything, not Everything\" means building a framework capable of doing anything, not providing a framework that already does everything. The first is possible, the second is not. Our primary goal is to provide a framework that _unlocks_ potential, and frees developers to work on the things that make their application special.\n\nTo this end, there are many prebuilt extensions to use, but there is also a rich suite of tools to build your _own_ extensions. In this way, you can make the framework work for you, instead of struggling to fit your application to a strictly prescribed pattern. Use as much of Ash as you can, and leverage the amazing Elixir ecosystem for everything else.","ref":"design-principles.html#anything-not-everything"},{"type":"extras","title":"Declarative, Introspectable, Derivable - Design Principles","doc":"The real superpower behind Ash is the declarative design pattern. All behavior is driven by explicit, static declarations. A resource, for example, is really just a configuration file. On its own it does nothing. It is provided to code that reads that configuration and acts accordingly.\n\nYou can read more about some simple declarative design patterns outside of the context of Ash Framework in [An Incremental Approach to Declarative Design](https://zachdaniel.dev/incremental-declarative-design).","ref":"design-principles.html#declarative-introspectable-derivable"},{"type":"extras","title":"Configuration over Convention - Design Principles","doc":"While convention has value, we believe that explicit configuration ultimately leads to more discoverable, maintainable and flexible applications than a convention driven approach. This means that we never do things like assume that files with a given name are a certain type of thing, or that because a file is in a certain location, it should perform a specific function.","ref":"design-principles.html#configuration-over-convention"},{"type":"extras","title":"Pragmatism First - Design Principles","doc":"While Ash does have lofty goals and a roadmap, the priority for development is always what the _current_ users of Ash need or are having trouble with. We focus on simple, pragmatic, and integrated solutions that meld well with the rest of the framework.\n\nA high priority is placed on ensuring that our users don't experience feature whip-lash due to poorly thought out implementations, and that any breaking changes (a rare occurrence) have a clean and simple upgrade path. This is something made much easier by the declarative pattern.","ref":"design-principles.html#pragmatism-first"},{"type":"extras","title":"Community - Design Principles","doc":"The Ash community comes together and collaborates to make sure that we can all build our software quickly, effectively and in a way that will age gracefully. We have a strict code of conduct, and love working with people of any experience level or background. To experience this first-hand, participate on [ElixirForum](https://elixirforum.com/c/elixir-framework-forums/ash-framework-forum/123) or join our [discord](https://discord.gg/D7FNG2q)!\n\n>","ref":"design-principles.html#community"},{"type":"extras","title":"Domain Driven Design? - Design Principles","doc":">\n>  Ash is _not_ a Domain Driven Design framework, if we're talking about \"proper\" Domain Driven Design as it is taught and discussed today. Domain Driven Design comes with a considerable amount of baggage and unnecessary complexity. While we identify with the *goals* of Domain Driven Design, we believe that a simpler approach is more effective, and that much of what DDD teaches are actually _implementation_ details, and not _design_ concepts. If the name wasn't taken, we would surely have claimed it for ourselves. If you must have a similar term for Ash, consider it a \"Resource-oriented, Declarative Design Application Framework\".","ref":"design-principles.html#domain-driven-design"},{"type":"extras","title":"Contributing to Ash","doc":"# Contributing to Ash","ref":"contributing.html"},{"type":"extras","title":"Welcome! - Contributing to Ash","doc":"We are delighted to have anyone contribute to Ash, regardless of their skill level or background. We welcome contributions both large and small, from typos and documentation improvements, to bug fixes and features. There is a place for everyone's contribution here. Check the issue tracker or join the ElixirForum/discord server to see how you can help! Make sure to read the rules below as well.","ref":"contributing.html#welcome"},{"type":"extras","title":"Contributing to Documentation - Contributing to Ash","doc":"Documentation contributions are one of the most valuable kinds of contributions you can make! Good documentation helps everyone in the community understand and use Ash more effectively.","ref":"contributing.html#contributing-to-documentation"},{"type":"extras","title":"Protocol for Documentation Improvements - Contributing to Ash","doc":"**We prefer Pull Requests over issues for documentation improvements.** Here's why and how:\n\n- **Make a PR directly** - This is the preferred approach! Even if you're not 100% sure about your changes, submitting a PR with your suggested improvement is much more helpful than opening an issue to discuss it.\n- **PRs represent tangible suggestions** - They're easy to review, approve, reject, or modify. We can see exactly what you're proposing and act on it quickly.\n- **Issues are okay too** - If you're really unsure or want to discuss a larger documentation restructuring, you can open an issue first. But for most cases, just make the PR!\n- **Don't worry about rejection** - If a PR doesn't fit or needs changes, we'll provide feedback or close it with explanation. This is much more efficient than back-and-forth discussion in issues.","ref":"contributing.html#protocol-for-documentation-improvements"},{"type":"extras","title":"Making Documentation Changes - Contributing to Ash","doc":"The best way to contribute to documentation is often through GitHub's web interface, which allows you to make changes without having to clone the code locally:\n\n**For Guides:**\n- While viewing any guide on the documentation website, look for the `</>` button in the top right of the page\n- Clicking this button will take you directly to GitHub's editing interface for that file\n\n![Guide Edit Button](documentation/assets/images/guides-link.png)\n\n**For Module Documentation:**\n- When viewing module documentation, the `</>` button will also be in the top right of the page\n\n**For Function Documentation:**\n- When viewing individual functions, you'll find the `</>` button next to the function header\n\n![Function Edit Button](documentation/assets/images/functions-link.png)\n\nOnce you click the `</>` button, GitHub will:\n1. Fork the repository for you (if you haven't already)\n2. Open the file in GitHub's web editor\n3. Allow you to make your changes directly in the browser\n4. Help you create a pull request with your improvements\n\nThis workflow makes it incredibly easy to fix typos, clarify explanations, add examples, or improve any part of the documentation you encounter while using Ash.","ref":"contributing.html#making-documentation-changes"},{"type":"extras","title":"Important Note About DSL Documentation - Contributing to Ash","doc":"**DSL documentation cannot be edited directly on GitHub.** The documentation you see for DSL options (like those for `Ash.Resource`, `Ash.Domain`, etc.) is generated from the source code of the DSL definition modules.\n\nFor example, if you want to improve documentation for `Ash.Resource` options, you need to edit the source code in the `Ash.Resource.Dsl` module, not the generated documentation files. The DSL documentation is automatically generated from the `@doc` attributes and option definitions in these modules.\n\nTo find the right module to edit:\n- For `Ash.Resource` DSL docs → edit `lib/ash/resource/dsl.ex`\n- For `Ash.Domain` DSL docs → edit `lib/ash/domain/dsl.ex`\n- And so on for other DSL modules\n\nWhen making DSL documentation improvements, make sure to:\n1. Edit the appropriate DSL definition module (not generated docs)\n3. Test that your changes generate correctly by running `mix docs`","ref":"contributing.html#important-note-about-dsl-documentation"},{"type":"extras","title":"Rules - Contributing to Ash","doc":"* We have a zero tolerance policy for failure to abide by our code of conduct. It is very standard, but please make sure\n  you have read it.\n* Issues may be opened to propose new ideas, to ask questions, or to file bugs.\n* Before working on a feature, please talk to the core team/the rest of the community via a proposal. We are\n  building something that needs to be cohesive and well thought out across all use cases. Our top priority is\n  supporting real life use cases like yours, but we have to make sure that we do that in a sustainable way. The\n  best compromise there is to make sure that discussions are centered around the *use case* for a feature, rather\n  than the proposed feature itself.\n* Before starting work, please comment on the issue and/or ask in the discord if anyone is handling an issue. Be aware that if you've commented on an issue that you'd like to tackle it, but no one can reach you and/or demand/need arises sooner, it may still need to be done before you have a chance to finish. However, we will make all efforts to allow you to finish anything you claim.","ref":"contributing.html#rules"},{"type":"extras","title":"Local Development & Testing - Contributing to Ash","doc":"","ref":"contributing.html#local-development-testing"},{"type":"extras","title":"Setting Up Your Development Environment - Contributing to Ash","doc":"1. **Fork and clone the repository:**\n   ```bash\n   git clone https://github.com/your-username/ash.git\n   cd ash\n   ```\n\n2. **Install dependencies:**\n   ```bash\n   mix deps.get\n   ```\n\n3. **Compile the project:**\n   ```bash\n   mix compile\n   ```","ref":"contributing.html#setting-up-your-development-environment"},{"type":"extras","title":"Running Tests and Checks - Contributing to Ash","doc":"Before submitting any pull request, please run the full test suite and quality checks locally:\n\n```bash\nmix check\n```\n\nThis command runs a comprehensive suite of checks including:\n- Compilation\n- Tests\n- Code formatting (via `spark.formatter`)\n- Credo (static code analysis)\n- Dialyzer (type checking)\n- Documentation generation and validation\n- Sobelow (security analysis)\n- And other quality checks\n\nYou can also run individual checks:\n- `mix test` - Run the test suite\n- `mix format` - Format code\n- `mix credo` - Run static analysis\n- `mix dialyzer` - Run type checking\n- `mix docs` - Generate documentation","ref":"contributing.html#running-tests-and-checks"},{"type":"extras","title":"Testing Ash with Your Application - Contributing to Ash","doc":"If you want to test your Ash changes with your own application, you can use Ash as a local dependency. In your application's `mix.exs`, replace the hex dependency with a path dependency:\n\n```elixir\ndefp deps do\n  [\n    # Replace this:\n    # {:ash, \"~> 3.0\"}\n\n    # With this (adjust path as needed):\n    {:ash, path: \"../ash\"},\n\n    # Your other dependencies...\n  ]\nend\n```\n\nThen run:\n```bash\nmix deps.get\nmix compile\n```\n\nThis allows you to:\n- Test your changes against real-world usage\n- Verify that your changes don't break existing functionality\n- Develop features iteratively with immediate feedback\n\nTesting in your own application is not sufficient, you must also include automated tests.","ref":"contributing.html#testing-ash-with-your-application"},{"type":"extras","title":"Development Workflow - Contributing to Ash","doc":"1. **Create a feature branch:**\n   ```bash\n   git checkout -b feature/your-feature-name\n   ```\n\n2. **Make your changes** and write tests\n\n3. **Run the full check suite:**\n   ```bash\n   mix check\n   ```\n\n4. **Commit your changes:**\n   ```bash\n   git add .\n   git commit -m \"Add feature description\"\n   ```\n\n5. **Push and create a pull request**","ref":"contributing.html#development-workflow"},{"type":"extras","title":"Common Development Tasks - Contributing to Ash","doc":"- **Generate documentation:** `mix docs`\n- **Run tests in watch mode:** `mix test.watch`\n- **Check formatting:** `mix format --check-formatted`\n- **Run specific test file:** `mix test test/path/to/test_file.exs`\n- **Run tests with coverage:** `mix test --cover`","ref":"contributing.html#common-development-tasks"},{"type":"extras","title":"Alternatives","doc":"# Alternatives\n\nThere aren't really any alternatives to Ash that we are aware of that do all of the same things, but there are many different packages out there that do some of the things that Ash does.\n\nThis is a living document, and is not comprehensive. We are not *vouching* for any of these packages, but rather listing them here for your convenience to investigate on your own.\n\nWant to add or edit this list? Open a [pull request](https://github.com/ash-project/ash/edit/main/documentation/topics/about_ash/alternatives.md). Want a more comprehensive list? Check out the [Awesome Elixir](https://github.com/h4cc/awesome-elixir).","ref":"alternatives.html"},{"type":"extras","title":"Application Frameworks - Alternatives","doc":"These frameworks have similar overarching goals of helping you build your application layer.\n\n- [Commanded](https://hexdocs.pm/commanded) - An event sourced application framework.\n- [Sleeky](https://hexdocs.pm/sleeky) - Billed as a lightweight alternative to Ash. Inspired by Ash, but more tightly built on top of ecto.","ref":"alternatives.html#application-frameworks"},{"type":"extras","title":"Application Design - Alternatives","doc":"- [Boundary](https://hexdocs.pm/boundary) - A library for defining boundaries in your application.\n- [Phoenix Contexts](https://hexdocs.pm/phoenix/contexts.html) - Phoenix ships with a concept called \"contexts\", which provide some generators with application design guidance.","ref":"alternatives.html#application-design"},{"type":"extras","title":"Building APIs - Alternatives","doc":"- [Absinthe](https://hexdocs.pm/absinthe) - A GraphQL toolkit for Elixir. This is what `AshGraphql` uses under the hood, but you can also use Absinthe directly.\n- [Phoenix](https://hexdocs.pm/phoenix) - Phoenix is a web framework for Elixir. It is not necessarily an API framework, but has all the tools you need to build APIs by hand.\n- [JSONAPI Elixir](https://hexdocs.pm/jsonapi) - A library for building JSONAPI compliant APIs in Elixir on top of Ecto.\n- [Open API Spex](https://github.com/open-api-spex) - A library for generating OpenAPI (Swagger) documentation for your API. We generate these for you in AshJsonApi, but you can use this library to build open api specifications of your hand-written API","ref":"alternatives.html#building-apis"},{"type":"extras","title":"Working with Data - Alternatives","doc":"- [Ecto](https://hexdocs.pm/ecto) - Ecto is a database wrapper and query generator for Elixir. In many cases, Ash uses Ecto under the hood, but it is also available to build on top of directly.\n- [Flop](https://hexdocs.pm/flop) - A library designed to easily apply filtering, ordering, and pagination to Ecto queries.","ref":"alternatives.html#working-with-data"},{"type":"extras","title":"Authentication - Alternatives","doc":"- [mix phx.gen.auth](https://hexdocs.pm/phoenix/Mix.Tasks.Phx.Gen.Auth.html) - A mix task that generates authentication for a Phoenix application. Some folks prefer to use this over `AshAuthentication` even if they are using Ash.\n- [Assent](https://hexdocs.pm/assent) - Multi-provider authentication framework.","ref":"alternatives.html#authentication"},{"type":"extras","title":"Authorization - Alternatives","doc":"- [Bodyguard](https://hexdocs.pm/bodyguard) - A phoenix-context-based policy authorization framework.","ref":"alternatives.html#authorization"},{"type":"extras","title":"Validation - Alternatives","doc":"- [Ecto](https://hexdocs.pm/ecto) - Ecto can be used to validate data at the edge, using things like schema-less changesets.\n- [Drops](https://hexdocs.pm/drops) - a collection of small modules that provide useful extensions and functions that can be used to work with data effectively.\n- [GuardedStruct](https://github.com/mishka-group/guarded_struct) - validation, sanitization, and construction of structs, supporting nested structs.","ref":"alternatives.html#validation"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n<!-- changelog -->\n\n## [v3.5.21](https://github.com/ash-project/ash/compare/v3.5.20...v3.5.21) (2025-06-16)","ref":"changelog.html"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* support read actions and arguments in Ash.Generator.action_input/3 (#2137) by [@barnabasJ](https://github.com/barnabasJ)\n\n* combination_of typespec (#2135) by [@barnabasJ](https://github.com/barnabasJ)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Improvements: - Changelog","doc":"* allow update_query when `expr_error` is nto supported by [@zachdaniel](https://github.com/zachdaniel)\n\n* only require atomicity when update_query *and* expr_error supported by [@zachdaniel](https://github.com/zachdaniel)\n\n* make installer avoid protocol consolidation by [@zachdaniel](https://github.com/zachdaniel)\n\n## [v3.5.20](https://github.com/ash-project/ash/compare/v3.5.19...v3.5.20) (2025-06-13)","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* support private_arguments in code interface and bulk actions (#2133) by [@barnabasJ](https://github.com/barnabasJ)\n* combination_of typespec (#2135) by [@barnabasJ](https://github.com/barnabasJ)\n\n## [v3.5.19](https://github.com/ash-project/ash/compare/v3.5.18...v3.5.19) (2025-06-12)","ref":"changelog.html#bug-fixes-1"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* bad pattern match in query inspect logic by [@zachdaniel](https://github.com/zachdaniel)\n\n## [v3.5.18](https://github.com/ash-project/ash/compare/v3.5.17...v3.5.18) (2025-06-11)","ref":"changelog.html#bug-fixes-2"},{"type":"extras","title":"Improvements: - Changelog","doc":"* support regexes in match constraint/validation again by [@zachdaniel](https://github.com/zachdaniel)\n\n## [v3.5.17](https://github.com/ash-project/ash/compare/v3.5.16...v3.5.17) (2025-06-10)","ref":"changelog.html#improvements-1"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* set_tenant on combination query (#2123) by [@barnabasJ](https://github.com/barnabasJ)\n\n## [v3.5.16](https://github.com/ash-project/ash/compare/v3.5.15...v3.5.16) (2025-06-10)","ref":"changelog.html#bug-fixes-3"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ensure context is properly threaded through bulk update/atomic upgrades by [@zachdaniel](https://github.com/zachdaniel)\n\n* ensure `actor: nil` key is retained on scope to opts by [@zachdaniel](https://github.com/zachdaniel)\n\n* ensure stream_batch_size is properly set to batch_size on stream by [@zachdaniel](https://github.com/zachdaniel)\n\n* set batch size option when streaming by [@zachdaniel](https://github.com/zachdaniel)\n\n* keep union types in order by [@zachdaniel](https://github.com/zachdaniel)\n\n* resolve accidentally backwards incompatible inspect implementation by [@zachdaniel](https://github.com/zachdaniel)\n\n* better error message on mismatch action types in changesets by [@zachdaniel](https://github.com/zachdaniel)\n\n* add clause in Ash.Type.String.match/1 to handle the OTP 28 regex tuples (#2119) by Simon Bergström\n\n* underlying ecto type casting should use coercion, not cast_input by [@zachdaniel](https://github.com/zachdaniel)\n\n* make decimal type less strict by [@zachdaniel](https://github.com/zachdaniel)\n\n* Fix typespecs for Ash.get! and Ash.load! (#2117) by Moxley Stratton\n\n* implement Scope.to_opts for policy authorizer by [@zachdaniel](https://github.com/zachdaniel)\n\n* simplify relationship loading, to prevent losing set query info by [@zachdaniel](https://github.com/zachdaniel)\n\n* typo in Ash.Error.Framework module (#2108) by Samuel Wrenn\n\n* ash.gen.validation generates invalid callback (#2103) by KasparKipp","ref":"changelog.html#bug-fixes-4"},{"type":"extras","title":"Improvements: - Changelog","doc":"* show action when inspecting query by [@zachdaniel](https://github.com/zachdaniel)\n\n* add assert_stripped test helper (#2107) by Samuel Wrenn\n\n* fix misleading value in invalid attribute error by [@zachdaniel](https://github.com/zachdaniel)\n\n* better error message on unexpected argument inputs to code interfaces (#2102) by Abhishek Tripathi\n\n## [v3.5.15](https://github.com/ash-project/ash/compare/v3.5.14...v3.5.15) (2025-06-04)","ref":"changelog.html#improvements-2"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* only override bulk options if method is `:id` in code interfaces\n\n* handle missing primary keys more explicitly, and gracefully\n\n* Omit nil values in Ash.Scope.to_opts/2 (#2088)\n\n* Correct error message, example and doc for match constraint. (#2086)\n\n* handle context opt in Ash.calculate/2 (#2083)","ref":"changelog.html#bug-fixes-5"},{"type":"extras","title":"Improvements: - Changelog","doc":"* put __meta__ and relationships at the end when inspecting (#2096)\n\n* usage-rules: Add more guidance around codegen. (#2100)\n\n* update usage rules (#2095)\n\n## [v3.5.14](https://github.com/ash-project/ash/compare/v3.5.13...v3.5.14) (2025-06-01)","ref":"changelog.html#improvements-3"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ensure the proper dependency on igniter\n\n* ensure map module loaded when calling it\n\n* raise if integer primary key is not supplied in ETS (#2079)\n\n## [v3.5.13](https://github.com/ash-project/ash/compare/v3.5.12...v3.5.13) (2025-05-30)","ref":"changelog.html#bug-fixes-6"},{"type":"extras","title":"OTP 28 Compatibility - Changelog","doc":"`Ash.Type.String`, `Ash.Type.CiString` and the built in `match` validation all now warn on the use\nof regexes. OTP 28 no longer supports building regexes at compile time. The warning will explain.","ref":"changelog.html#otp-28-compatibility"},{"type":"extras","title":"Notable Features - Changelog","doc":"* `:shared` context key is now automatically passed down to child actions. See [the actions guide](/documentation/topics/actions/actions.md#context) for more information.\n\n* add `Ash.Scope`, mirroring the Phoenix pattern, and extend that to all Ash callback contexts. See `Ash.Scope` for more. (#2050)","ref":"changelog.html#notable-features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Query`] Allow non-list input to skip_unknown_inputs opt of query (#2074)\n\n* [`Ash.Query`] handle single tuple & string values in sort\n\n* [`Ash.Query`] use proper read action calls when building aggregates\n\n* [`Ash.Generator`] don't generate values for attributes that are `writable?: false` and `generated?: true`\n\n* [`:embedded` resources] ensure we detect all cases where embedded attributes can't be atomically updated\n\n* [`Ash.Type.NewType`] fix issue with expanding constraints for lazy initialized new types\n\n* [`Ash.Changeset`] properly pass tenant in bulk_update to managed_relationships (#2061)","ref":"changelog.html#bug-fixes-7"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`mix ash.codegen`] add `Ash.Error.Framework.PendingCodegen` error, used for the new `AshPhoenix.Plug.CheckCodegenStatus` plug\n\n* [`Ash.Generator`] support upsert/upsert_identity in changeset generator\n\n* [`usage-rules.md`] Improve LLM rules in `usage-rules.md`. See [usage_rules](https://hexdocs.pm/usage-rules) for more.\n\n* [`Ash.Type.Decimal`] - Add `precision` and `scale` constraints.\n\n\n\n\n## [v3.5.12](https://github.com/ash-project/ash/compare/v3.5.11...v3.5.12) (2025-05-22)","ref":"changelog.html#improvements-4"},{"type":"extras","title":"Features: - Changelog","doc":"* add duration type, functions and operator support (#2036)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* properly split lazy & non-lazy new type initialization\n\n* Ash.Reactor: Don't import `Ash.Expr` in the bulk update DSL. (#2055)\n\n* don't lift query info to aggregate info on aggregation\n\n* ensure managed relationship context is kept for belongs_to relationships","ref":"changelog.html#bug-fixes-8"},{"type":"extras","title":"Improvements: - Changelog","doc":"* Support manage relationship debug (#2021)\n\n* add `strict_load` key to `Ash.Query.build`\n\n* Add an Ash.OptionsHelpers.calculation_type/0 (#2051)\n\n## [v3.5.11](https://github.com/ash-project/ash/compare/v3.5.10...v3.5.11) (2025-05-20)","ref":"changelog.html#improvements-5"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ensure we fully initialize new types\n\n* Tuple loader and serializer (#2049)\n\n* make sure after_action is called in generate_many (#2047)\n\n* properly pass `select` into combinations\n\n* add_new_code_interface/5 when do block occurs after resource (#2020)","ref":"changelog.html#bug-fixes-9"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add experimental new tool `Ash.data_layer_query`\n\n## [v3.5.10](https://github.com/ash-project/ash/compare/v3.5.9...v3.5.10) (2025-05-15)","ref":"changelog.html#improvements-6"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ensure field policies are logged on success\n\n* various additional fixes for bulk action input ordering\n\n* Fix batch order of bulk_create (#2027)\n\n* make `lazy_init?` a callback so it can be checked on new types\n\n* don't raise error when no policies apply to request\n\n* ensure tenant is set on bulk created records.\n\n* don't try to cast input before cast atomic\n\n* properly prevent embedded attribute updates in atomics\n\n* preserve validation messages in non-atomic-bulk-update validations\n\n* add types for times operator\n\n* properly handle pre-expanded newtype constraints\n\n* shortcircuit queries properly\n\n* only print topic if present (#2013)","ref":"changelog.html#bug-fixes-10"},{"type":"extras","title":"Improvements: - Changelog","doc":"* support :time_usec (#2023)\n\n* support `limit` on has_many relationships (#2016)\n\n## [v3.5.9](https://github.com/ash-project/ash/compare/v3.5.8...v3.5.9) (2025-05-06)","ref":"changelog.html#improvements-7"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* shortcircuit queries properly\n\n* ensure that context is set on authorizers in nested field policies\n\n* accept `private_arguments` option in the same way as `defaults` in generators\n\n* handle case where atomic change isn't cleared when converted to static\n\n* handle case where atomic condition isn't applied to generated validations","ref":"changelog.html#bug-fixes-11"},{"type":"extras","title":"Improvements: - Changelog","doc":"* normalize authorizer context in more locations\n\n* aggressively prune ets logs while retaining important info\n\n* combination queries (#2009)\n\n## [v3.5.8](https://github.com/ash-project/ash/compare/v3.5.7...v3.5.8) (2025-04-30)","ref":"changelog.html#improvements-8"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* handle `nil` type or invalid types more gracefully in type determination\n\n* properly type expressions based on return values","ref":"changelog.html#bug-fixes-12"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add rem/2 expr (#2004)\n\n* add `Ash.Info` for general application information\n\n## [v3.5.7](https://github.com/ash-project/ash/compare/v3.5.6...v3.5.7) (2025-04-29)","ref":"changelog.html#improvements-9"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* Accept field opt in query aggregate (#2001)\n\n* only return the changeset (#2003)\n\n* expand types when typing expressions\n\n* Allow soft-deleting of many-to-many relationships (#1999)\n\n* properly show timeout errors on read transactions\n\n* handle values that generate as `nil` in map generators\n\n* sort lazy loaded records post-linking\n\n* properly attach nested transient calculation dependencies\n\n* be more lax with lazy_init? newtypes\n\n* properly return subtype constraints on constraint call\n\n* handle error cases in atomic changesets better\n\n* CodeInterface: logic error in `get_by` code interfaces. (#1961)\n\n* CodeInterface: logic error in `get_by` code interfaces.\n\n* crash when sorting by aggregates with non-attribute field (#1986)\n\n* handle atomic conditions on `{:atomic, ...}` changes\n\n* ensure data layer is loaded before using `function_exported?/3`\n\n* ensure that bulk callbacks are only called when appropriate\n\n* properly update belongs_to records on `relate_and_update`","ref":"changelog.html#bug-fixes-13"},{"type":"extras","title":"Improvements: - Changelog","doc":"* Skip reading query for filter false (#2002)\n\n* generator for union types\n\n* Add various DSL options for inspecting resources\n\n* hide calculations and aggregates when empty\n\n* hide calculation dep calculations while inspecting\n\n* Raise an `ArgumentError` when generating a changeset using a non-existent action (#1992)\n\n* add `:tuple` builtin type\n\n* set changeset.load on bulk destroy actions\n\n* return the error when atomically cascade destroying\n\n* add a description to map/keyword/struct types\n\n* Remove duplicate impl of default policy functions (#1985)\n\n* eagerly expand aggregates in `Ash.aggregate`\n\n* accept `context` option in generic action code interfaces\n\n## [v3.5.6](https://github.com/ash-project/ash/compare/v3.5.5...v3.5.6) (2025-04-15)","ref":"changelog.html#improvements-10"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* properly deduplicate on lazy relationship loading\n\n* use correct exceptions and add missing fields to them (#1960)\n\n* undo incorrect change to trimmed string casting","ref":"changelog.html#bug-fixes-14"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add `:__skip__` value to seed\n\n* Allow update action types in Ash.Generator.generate (#1967)\n\n## [v3.5.5](https://github.com/ash-project/ash/compare/v3.5.4...v3.5.5) (2025-04-14)","ref":"changelog.html#improvements-11"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* don't trim strings when `trim?: false` is set\n\n* remove flawed optimization about `is_nil: false`\n\n* properly evaluate ci_string concatenation in Elixir\n\n* don't validate types in gen.resource task.","ref":"changelog.html#bug-fixes-15"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add calculation tools to `Ash.Resource.Igniter`\n\n* support a `load` option on changeset building\n\n## [v3.5.4](https://github.com/ash-project/ash/compare/v3.5.3...v3.5.4) (2025-04-10)","ref":"changelog.html#improvements-12"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ensure after hooks force return_records on bulk update/destroy\n\n* add action to changeset for opts fetching in bulk create\n\n* avoid coercion of datetime to date in start_of_day function (#1958)\n\n* incorrect start_of_day value when timezone specified\n\n## [v3.5.3](https://github.com/ash-project/ash/compare/v3.5.2...v3.5.3) (2025-04-09)","ref":"changelog.html#bug-fixes-16"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* avoid defining default actions when actions w/ that name exist\n\n* use the notification logic from bulk update in create (#1951)\n\n* honor stream_options in read code interfaces\n\n* Fix filtering with aggregates refers calculation error (#1954)\n\n* properly provide the changeset to after action hooks\n\n* set calculation context in `Ash.can`\n\n* honor `:*` in skip_unknown_inputs in generic actions\n\n* honor action's `skip_unknown_inputs` in generic actions\n\n* set access_from in cascade_  (#1948)\n\n* maintain order of enum values (#1942)\n\n* carry context around to nested calculations better\n\n* fill templates in more necessary places\n\n* set tenant when building query in managed_relationships","ref":"changelog.html#bug-fixes-17"},{"type":"extras","title":"Improvements: - Changelog","doc":"* optimize args handling in code interfaces\n\n* tasks for generating custom modules (#1940)\n\n## [v3.5.2](https://github.com/ash-project/ash/compare/v3.5.1...v3.5.2) (2025-03-31)","ref":"changelog.html#improvements-13"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* match errors on cascade destroy/update\n\n* handle templated opts in bulk update after batch results\n\n* support `refs` and `args` option for `define_calculation`\n\n* don't cast arbitrary maps to structs in struct type","ref":"changelog.html#bug-fixes-18"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add `default_sort` on relationships and queries (#1928)\n\n* new type recursive validation (#1913)\n\n* task for generating custom changes module (#1926)\n\n* better error on `case` in ash expressions (#1927)\n\n* helpful error on incorrectly implemented change modules\n\n* support `custom_input`s in code interfaces\n\n* Ash.Type.Enum - Add optional description and label value â¦ (#1925)\n\n* support `exclude_inputs` for `define_calculation`\n\n* add `exclude_inputs` to code interface definitions\n\n* fix hint for read/generic actions in `NoSuchInput`\n\n## [v3.5.1](https://github.com/ash-project/ash/compare/v3.5.0...v3.5.1) (2025-03-27)","ref":"changelog.html#improvements-14"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* when reading a record to simulate an update, merge with data\n\n* more consistent error messaging from present validation\n\n* can't atomically update a query w/ after_action hooks","ref":"changelog.html#bug-fixes-19"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add `Ash.transaction(resources, func, opts)` (#1914)\n\n* set `query_for` context on queries\n\n## [v3.5.0](https://github.com/ash-project/ash/compare/v3.4.73...v3.5.0) (2025-03-26)","ref":"changelog.html#improvements-15"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* don't set `accessing_from` on lookups for managed relationships\n\nThis *may* be a breaking change for some cases, but ultimately we\nfelt it was too confusing to leave as it was. The semantics of\n`accessing_from` meant that \"all actions done as part of `manage_relationship`\nwere allowed to be performed. This was the intended design, but made it easy\nto implement an authorization related bug. Specifcaly, the `on_lookup` behavior\nwould set that context as well, potentially allowing a user to relate something\nto a resource that you did not intend them to be able to see.","ref":"changelog.html#bug-fixes-20"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add `tenant()` filter template expression (#1909)\n\n* `Ash.Domain.Igniter.add_new_code_interface/5`\n\n* add `unsafe_to_atom?` constraint for `Ash.Type.Atom`\n\n* support `expr(exists(relation))` (#1912)\n\n## [v3.4.73](https://github.com/ash-project/ash/compare/v3.4.72...v3.4.73) (2025-03-25)","ref":"changelog.html#improvements-16"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* use attribute names, not structs, for retaining ets update attrs\n\n* supply reactor with `nil` values for non-supplied arguments\n\n## [v3.4.72](https://github.com/ash-project/ash/compare/v3.4.71...v3.4.72) (2025-03-25)","ref":"changelog.html#bug-fixes-21"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* retain loaded fields on ETS data layer update\n\n* Fixes processing of manual bulk actions, with tests. (#1903)\n\n* Ensure batch_size is correctly set, return values are nil if not requested.\n\n* batch_size cond statement, fix batch -> changeset typo.\n\n* ensure consistent bulk result for `return_records?` and `return_errors?` types.\n\n* add action to base query if not present\n\n* update spark to get missing builders in `spark.formatter`\n\n* handle tenancy for built query in cascade destroy/update\n\n* Improve processing of manual actions during bulk operations (#1883)","ref":"changelog.html#bug-fixes-22"},{"type":"extras","title":"Improvements: - Changelog","doc":"* attach a limit to related queries if `from_many?: true`\n\n* support atomic actions in `can_` code interfaces\n\n* import the `:reactor` dep in `.formatter.exs` on install\n\n* maps,structs,keywords can be atomically updated\n\n* support anonymous functions in `error_handler` option\n\n## [v3.4.71](https://github.com/ash-project/ash/compare/v3.4.70...v3.4.71) (2025-03-21)","ref":"changelog.html#improvements-17"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* prefer new loads when loading relationships\n\n* put notifications in process context when inside an action\n\n* consider query tenant when validating aggregate multitenancy\n\n* update Validations.ActionIs to accept atom or list(atom) (#1893)","ref":"changelog.html#bug-fixes-23"},{"type":"extras","title":"Improvements: - Changelog","doc":"* set `bulk_actions_default_to_errors?` to `true` in installer\n\n## [v3.4.70](https://github.com/ash-project/ash/compare/v3.4.69...v3.4.70) (2025-03-20)","ref":"changelog.html#improvements-18"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* compose get_by and action filters properly\n\n* `fields` could be nil on exceptions\n\n* apply runtime supplied loads over top of action loads\n\n* return records from bulk soft destroy if requested (#1884)\n\n* ensure error classes are used in code interfaces","ref":"changelog.html#bug-fixes-24"},{"type":"extras","title":"Improvements: - Changelog","doc":"* validate multitenancy earlier in bulk actions\n\n* initialize all types properly at compile time\n\n## [v3.4.69](https://github.com/ash-project/ash/compare/v3.4.68...v3.4.69) (2025-03-18)","ref":"changelog.html#improvements-19"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* apply strict load in `Ash.get` properly (#1881)\n\n* honor tenant in `Ash.Seed.update!`\n\n* pattern match error in bulk result\n\n* match on `{:not_atomic` pattern in update actions\n\n* fix typo: `atomcis` -> `atomics`\n\n* properly evaluate `Exists{}` expressions in runtime filter\n\n* Fix upsert identity type error (#1872)\n\n* Fix bulk_create for manual create actions with bulk_create/3 (#1869)\n\n* ensure loading maps & structs properly load as keyword\n\n* make the --example installer flag idempotent\n\n* don't add tenant attribute for `all_tenant?` identities upsert keys\n\n* handle case clause error in `Ash.Filter`\n\n* validate return type of `Ash.Resource.Calculation.init/1`\n\n* make `lazy_init` work on `NewTypes`\n\n* properly parse maps with tuple values in filter parser\n\n* dump values to native storage type when doing atomic upgrades\n\n* fix required error deduplication logic\n\n* undo breaking change, use primary read action loads in `Ash.load!`\n\n* don't derive fields for resources in map types\n\n* make sequence unshrinkable in generators\n\n* properly handle bulk soft destroy (#1854)","ref":"changelog.html#bug-fixes-25"},{"type":"extras","title":"Improvements: - Changelog","doc":"* support a resource & attrs tuple in `seed_generator`\n\n* allow is_equal and is_not_equal for compare validation (#1853)\n\n* return errors and stop on errors by default in bulk actions\n\n* add config to do read after_action hooks in order\n\n* add compile flag requiring atomic for default actions\n\n* types support atomic update by default for non-expr values\n\n* add `actions.read.pagination.stable_sort` customization\n\n* support inferring struct types from a resource\n\n* show value in match validation errors\n\n* disambiguation message on NoSuchFunction\n\n## [v3.4.68](https://github.com/ash-project/ash/compare/v3.4.67...v3.4.68) (2025-03-11)","ref":"changelog.html#improvements-20"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* init nested types in map/keyword/struct types\n\n* properly handle change with `where` validations in bulk (#1843)\n\n* properly construct `parent_stack` for loaded relationships\n\n* update context tenant from changeset for each change (#1837)","ref":"changelog.html#bug-fixes-26"},{"type":"extras","title":"Improvements: - Changelog","doc":"* allow manual reads return `full_count` for pagination\n\n* validate action types in `Ash` functions\n\n## [v3.4.67](https://github.com/ash-project/ash/compare/v3.4.66...v3.4.67) (2025-03-04)","ref":"changelog.html#improvements-21"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* Pass tenant option when seeding resource given by the generator (#1834)\n\n## [v3.4.66](https://github.com/ash-project/ash/compare/v3.4.65...v3.4.66) (2025-03-03)","ref":"changelog.html#bug-fixes-27"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* handle unparseable relationship sorts & single atom sorts\n\n* don't allow modifying changeset with atomic conditions\n\n* don't prevent changing values to `nil` when original data is not available\n\n* propagate invalid reference error when adding calc context to sort (#1827)\n\n* ensure that we don't try to compare not loaded or forbidden values\n\n* use `filter` not `filters` in stale record error","ref":"changelog.html#bug-fixes-28"},{"type":"extras","title":"Improvements: - Changelog","doc":"* add `touching?` option to changing validation\n\n* don't show required errors for fields with other errors\n\n* validate aggregate multitenancy\n\n* ignore action-defined loads when using `Ash.load`\n\n## [v3.4.65](https://github.com/ash-project/ash/compare/v3.4.64...v3.4.65) (2025-02-25)","ref":"changelog.html#improvements-22"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* properly enumerate `:_pkey` in notifier\n\n* Always rollback input.resource when running generic actions. (#1817)\n\n* fix case where batch before/after action callbacks could be skipped\n\n* return NotFound error in proper cases on bulk interfaces\n\n* don't eagerly return records on bulk update/destroys\n\n* ensure actor templates are hydrated for aggregates\n\n* properly use operator overloads for evaluating operators at runtime\n\n* Missing case clause for bulk update/destroy with `get?: true` in interface (#1806)\n\n* always run update filter on skipped updates\n\n* don't change update defaults unless something changes","ref":"changelog.html#bug-fixes-29"},{"type":"extras","title":"Improvements: - Changelog","doc":"* better parent resource tracking in expressions\n\n* Generic actions to raise if they don't have return type but have an return value (#1805)\n\n## [v3.4.64](https://github.com/ash-project/ash/compare/v3.4.63...v3.4.64) (2025-02-17)","ref":"changelog.html#improvements-23"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* use undo action in generic action undo\n\n* handle generic actions with no return\n\n* ensure atomic `set_attribute` behaves the same as non-atomic\n\n* Missing actor on aggregate resource call (#1796)\n\n* Missing actor on aggregate call (#1793)","ref":"changelog.html#bug-fixes-30"},{"type":"extras","title":"Improvements: - Changelog","doc":"* support receiving the inputs when undoing generic actions\n\n* simplify & unify sort/sort_input logic\n\n* support related sorts everywhere (not just sort_input)\n\n* add field names to identities (#1786)\n\n## [v3.4.63](https://github.com/ash-project/ash/compare/v3.4.62...v3.4.63) (2025-02-11)","ref":"changelog.html#improvements-24"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* set `read_after_writes` to true if generated\n\n* type cast errors w/ floats & vectors\n\n* handle case clause error in filters\n\n* don't return invalid type from vector type\n\n* don't double process string interpolation expressions\n\n* Include warning for arguments only when 'things' are arguments (#1785)\n\n* empty bulk create inputs must still return a stream\n\n* don't use `authorize_with: :error` on data layers that can't do it\n\n* pass tenant to load in cascade destroy (#1775)\n\n* add tenant to load in cascade update (#1773)\n\n* raise errors on partial_success results in bulk actions\n\n* add `reuse_values?` opt to `Ash.can` and disable it automatically\n\n* compile `Ash.PlugHelpers` even without `Plug` available\n\n* expand `opts` when using `calculate/3`\n\n* handle base resources in `ash.extend`\n\n* type system warning on apps w/o solvers\n\n* Fix no read action exception for through relationship (#1750)\n\n* always recompile domain on resource changes\n\n* fix handling of generic action returns with transaction enabled (#1758)\n\n* type struct handle instance of return error tuple (#1756)\n\n* Compilation failure when using the `ash_step` Reactor DSL. (#1753)\n\n* pass `authorize?` option to `bulk_create` in `Ash.Generator.generate_many/2`","ref":"changelog.html#bug-fixes-31"},{"type":"extras","title":"Improvements: - Changelog","doc":"* Add string_position expression (#1782)\n\n* add `dimensions` to vector type\n\n* add filter & transform options for pubsub notifier\n\n* verify pub_sub actions at compile time\n\n* more clean boolean filter optimization for `or ==`\n\n* add `reuse_values?` option for calculate!\n\n* prefer `calculate/3` when reusing values\n\n* add `c:Ash.Type.coerce/2` callback\n\n## [v3.4.62](https://github.com/ash-project/ash/compare/v3.4.61...v3.4.62) (2025-01-31)","ref":"changelog.html#improvements-25"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Changeset`] always start transactions when managing relationships\n\n* [`Ash.Changeset`] handle parent in rel in managed belongs to (#1746)\n\n## [v3.4.61](https://github.com/ash-project/ash/compare/v3.4.60...v3.4.61) (2025-01-31)","ref":"changelog.html#bug-fixes-32"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Generator`] don't prevent setting manage_relationship inputs in generators\n\n* [`Ash.Reactor`] Fix referring to outer steps and inputs in `transaction` steps. (#1741)\n\n* [`Ash.Expr`] always return utc timestamp as result of `start_of_day`\n\n* [`Ash.Resource.Change.CascadeDestroy`] support `after_action?` option on cascade destroys & better error when it should be used (#1734)","ref":"changelog.html#bug-fixes-33"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash.Resource`] warning on args, preparations or filters on primary reads\n\n* [`Ash.Reactor`] Support guards in `Ash.Reactor` steps. (#1739)\n\n* [`mix ash.extend`] use `ash.extend` and use it instead of `ash.patch.extend`\n\n## [v3.4.60](https://github.com/ash-project/ash/compare/v3.4.59...v3.4.60) (2025-01-27)","ref":"changelog.html#improvements-26"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Expr`] traverse custom expressions when listing refs\n\n## [v3.4.59](https://github.com/ash-project/ash/compare/v3.4.58...v3.4.59) (2025-01-27)","ref":"changelog.html#bug-fixes-34"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Query`] better placed validations of aggregate support for data layers\n\n## [v3.4.58](https://github.com/ash-project/ash/compare/v3.4.57...v3.4.58) (2025-01-26)","ref":"changelog.html#bug-fixes-35"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Query.Aggregate`] properly check query aggregate support","ref":"changelog.html#bug-fixes-36"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash`] support `authorize_with` option in `Ash.get` (#1732)\n\n## [v3.4.57](https://github.com/ash-project/ash/compare/v3.4.56...v3.4.57) (2025-01-23)","ref":"changelog.html#improvements-27"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Resource.Validation.Compare`] ensure compare validation doesn't put functions in exceptions\n\n\n## [3.4.56](https://github.com/ash-project/ash/compare/v3.4.55...3.4.56) (2025-01-21)","ref":"changelog.html#bug-fixes-37"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash`] don't use `JSON` due to library compatibility issues\n\n* [`Ash.Changeset`] matching in managed_relationships handle_update (#1719)\n\n* [`Ash.Query.Calculation`] properly load doubly nested calculation's explicit dependencies\n\n* [`Ash.Query.Calculation`] handle related non-expr calculations referenced from expr calcs\n\n* [`Ash.Query.Calculation`] simplify and fix path generation for nested relationship path deps\n\n* [`Ash`] don't require multitenancy attribute in `get` (#1716)","ref":"changelog.html#bug-fixes-38"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash.Changeset`] make atomics work even if expr err is not supported (#1718)\n\n* [`Ash.Query`] support error shorthand for `Ash.Query.add_error/2-3`\n\n* [`Ash.Generator`] add `uses` option for `changeset_generator`\n\n* [`Ash.Generator`] add `uses` option for `seed_generator`\n\n* [`Ash.Changeset`] Use clearer error message for match validation atomic errors (#1721)\n\n* [`Ash.Type`] Add autogenerate_enabled? to Ash.Type for Ecto compatibility (#1715)\n\n* [`Ash.Policy.Authorizer`] warn when domain policies would be ignored by resources\n\n* [`Ash.Domain`] allow policy authorizer to be in authorizers key in domains\n\n## [v3.4.55](https://github.com/ash-project/ash/compare/v3.4.54...v3.4.55) (2025-01-13)","ref":"changelog.html#improvements-28"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [code interfaces] ensure can_* code interfaces pass arguments to actions\n\n* [`Ash`] case clause error in `Ash.can?`\n\n* [`Ash`] reset `ash_started_transaction?` on bulk create\n\n* [`Ash.Generator`] handle embedded attributes in attribute generator\n\n* [`Ash.Generator`] Fix typo in skipped import name (#1704)\n\n* [`Ash.Generator`] set max_concurrency to 0 for generate_many\n\n* [`Ash.Generator`] ensure that `once` and `sequence` behave predictably across tests","ref":"changelog.html#bug-fixes-39"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash.Changeset`] destroy missing records first in `manage_relationship`\n\n* [`Ash.Expr`] add start_of_day function\n\n* [`Ash.Type.DateTime`] add `cast_dates_as` constraint to `Ash.Type.DateTime`\n\n## [v3.4.54](https://github.com/ash-project/ash/compare/v3.4.53...v3.4.54) (2025-01-09)","ref":"changelog.html#improvements-29"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Generator`] Fix issues in `Ash.Generator.generate_many/2` (#1703)\n\n* [`Ash.Generator`] Don't error if no `after_action` is provided to `generate_many`\n\n* [`Ash.Generator`] Reuse the changeset actor when calling `bulk_create`","ref":"changelog.html#bug-fixes-40"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash.Generator`] run notifications for generators\n\n* [`Ash.Changeset`] `order_is_key` option for sorted relationships\n\n## [v3.4.53](https://github.com/ash-project/ash/compare/v3.4.52...v3.4.53) (2025-01-08)","ref":"changelog.html#improvements-30"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Generator`] properly delegate and handle conflicts in `Ash.Generator`\n\n* [`Ash.Generator`] Replace calls to `create` and `create_many` with `generate` and `generate_many` (#1701)\n\n* [calculations] use nested calculation dependencies from expr if not in expression\n\n* [`Ash.Changeset`] pattern match error on expression parse failure\n\n* [`Ash.Test.Resource.Validation.StringLengthTest`] handle `string_length` on arguments when atomic\n\n## [v3.4.52](https://github.com/ash-project/ash/compare/v3.4.51...v3.4.52) (2025-01-06)","ref":"changelog.html#bug-fixes-41"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Type.Map`]  handle keyword errors from map field type casting\n\n* [`mix ash.gen.resource`] ensure extensions & subjects args are unique","ref":"changelog.html#bug-fixes-42"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`ash.gen.resource`] validate that names given to `ash.gen.resource`\n\n* [`Ash.Generator`] add `Ash.Generator.changeset_generator/3`\n\n* [`Ash.Generator`] add `Ash.Generator.seed_generator/2`\n\n* [`Ash.Generator`] only use known keys in generators in `Ash.Generator`\n\n* [`Ash`] support `after_action` option to `Ash.bulk_create`\n\n* [`mix ash.install`] set `yes_to_deps` when fetching dependencies\n\n* [`Ash.Query`] better error message on non-resource in `Ash.Query.new/2`\n\n* [`Ash.bulk_destroy`] handle limited bulk destroys from streams\n\n* [Code interfaces] bulk actions use `full_read` from code interfaces given ids\n\n* [Code Interfaces] set `limit` in code interface to update or destroy one thing\n\n## [v3.4.51](https://github.com/ash-project/ash/compare/v3.4.50...v3.4.51) (2025-01-03)","ref":"changelog.html#improvements-31"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Resource`] handle ambiguous case of empty params in code interfaces (#1694)\n\n* [`Ash.Changeset`] discard manage_relationships added inside changes on atomic upgrade\n\n## [v3.4.50](https://github.com/ash-project/ash/compare/v3.4.49...v3.4.50) (2025-01-01)","ref":"changelog.html#bug-fixes-43"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.DataLayer.Ets`, `Ash.DataLayer.Mnesia`] properly handle aggregate defaults in ets/mnesia (#1684)\n\n* [`Ash.Resource.Validation.Changing`] use context message instead of default if provided in changing validation (#1677)\n\n* [`Ash.Changeset`] ensure that `changed?` context is set to true for atomics\n\n* [`Ash`] properly match on `return_query?` option, avoid raised pattern match error\n\n* [`Ash.Policy.Authorizer`] ensure that old config applies all aggregate policies","ref":"changelog.html#bug-fixes-44"},{"type":"extras","title":"Improvements: - Changelog","doc":"* [`Ash.Generator`] add `Ash.Generator.once/2`\n\n* [`Ash.Type.Map`, `Ash.Type.Keyword`, `Ash.Type.Struct`] define `generate/1` callback for maps, structs, keywords\n\n* [`Ash`] add `data_layer?` option to `Ash.calculate/3`\n\n* [`Ash.Resource`] Add default code interface options (#1681)\n\n* [`Ash.Resource`] add `allow_forbidden_field?` option to relationships\n\n* [`Ash.Resource`] add `authorize_read_with` option to relationships\n\n* [`Ash`] support `default` option in `Ash.first` (#1683)\n\n* [`Ash.Notifier.PubSub`] allow exclusion of certain actions from publish_all (#1680)\n\n* [`mix igniter.install ash`] no prompt about SAT solver unless user is on windows\n\n* [`Ash.Domain`] add otp_app option to use Ash.Domain\n\n* - [`Ash`] add support for `strict?` in read options (#1669)\n\n## [v3.4.49](https://github.com/ash-project/ash/compare/v3.4.48...v3.4.49) (2024-12-22)","ref":"changelog.html#improvements-32"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [read actions] - add support for `strict?` in `Ash.read` options. (#1669)","ref":"changelog.html#improvements-33"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* [`Ash.Policy.Authorizer`] ensure that old config applies all aggregate policies\n\nIf you've upgraded to the following configuration this does not affect you:\n\n```elixir\nconfig :ash, :policies, no_filter_static_forbidden_reads?: false\n```\n\nYou should upgrade regardless, and adopt that new configuration.\n\n## [v3.4.48](https://github.com/ash-project/ash/compare/v3.4.47...v3.4.48) (2024-12-20)","ref":"changelog.html#bug-fixes-45"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [calculations] properly update sort calculation expressions\n\n- [`Ash.Type.Module`] handle nil values in `Ash.Type.Module`\n\n- [`Ash.Resource`] ensure that `select_by_default?` is honored on loads\n\n- [`Ash.Type.Union`] Verify union types constraint on init\n\n- [loading data] ensure tenant is set on reselection query","ref":"changelog.html#bug-fixes-46"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Igniter] handle igniter not being compiled, and make it optional\n\n- [`Ash.Generator`] add `Ash.Generator.next_in_sequence/3`\n\n- [performance] don't reselect unnecessary attributes\n\n- [pagination] add `show_keysets_for_all_actions?` configuration\n\n  Set `config :ash, show_keysets_for_all_actions?, false` for significant performance\n  improvements when reading resources that support keyset pagination. This causes\n  keysets to only be shown for actions that are actively being paginated with\n  keyset pagination.\n\n## [v3.4.47](https://github.com/ash-project/ash/compare/v3.4.46...v3.4.47) (2024-12-17)","ref":"changelog.html#improvements-34"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Query`] handle indexed maps and string keys in calculation arguments\n\n- [`Ash.Changeset`] throw validation error when trying to set public arguments in private_arguments (#1663)\n\n- [`Ash.Policy.Authorizer`] include `changeset` in preflight authorization context\n\n- [embedded resources] include presence of authorizers in embedded resource optimization\n\n- [`Ash.DataLayer`] don't check data layer compatibility for manual actions","ref":"changelog.html#bug-fixes-47"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Reactor`]: Always add the notification middleware any time the extension is added. (#1657)\n\n## [v3.4.46](https://github.com/ash-project/ash/compare/v3.4.45...v3.4.46) (2024-12-12)","ref":"changelog.html#improvements-35"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Tracer`] use proper telemetry name for actions\n\n- [`Ash.Sort`] use atoms for paths in related sorts\n\n## [v3.4.45](https://github.com/ash-project/ash/compare/v3.4.44...v3.4.45) (2024-12-10)","ref":"changelog.html#bug-fixes-48"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash`] don't ignore tenant when calling aggregate functions","ref":"changelog.html#bug-fixes-49"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Authorizer`] don't log field policies unless logging successful policy breakdowns\n\n## [v3.4.44](https://github.com/ash-project/ash/compare/v3.4.43...v3.4.44) (2024-12-06)","ref":"changelog.html#improvements-36"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Changeset`] use `Ash.read` when eager validating relationships\n\n- [`Ash.Expr`] allow strings in `get_path/2`\n\n- [`Ash.Sort`] don't expand calculations until after authorization is complete\n\n- [`Ash.Resource.Change.GetAndLock`] don't automatically skip `get_and_lock` changes\n\n- [`Ash.Filter`] handle indexed maps in filter map syntax\n\n- [`Ash.Filter`] handle case where `%{or` is composing a single map\n\n- [`Ash.Policy.Authorizer`] ensure that `subject` is properly set when running field policies\n\n- [`Ash.Type`] fix logic errors in matches_type?/list logic\n\n- [pagination] add tenant to Aggregate opts when building count query (#1630)\n\n- [notifications] some notifications not being sent for bulk create actions\n\n- [validations] negate atomic validation expressions when used as `where` conditions (#1624)\n\n- [`Ash.Policy.Authorizer`] don't double apply action-filters when attaching policy filters (optimization)\n\n- [`Ash.read`] Fix not working skip_unknown_inputs opt of read action (#1596)\n\n- [`Ash.read`] Fix warning when actions.read.argument constraint is violated (#1607)\n\n- [aggregates] use last relationship's read action properly in aggregate queries\n\n- [`Ash.Changeset`] finish conversion from `append?` to `prepend?` option (#1601)","ref":"changelog.html#bug-fixes-50"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Type.NewType`] add `lazy_init?` option. Allows for recursive embedded types.\n\n- [`mix ash.gen.resource`] More Descriptive Error Messages for ash.gen.resource (#1645)\n\n- [`Ash.Expr`] better type signatures for division\n\n- [`Ash.Expr`] converge on known types better\n\n- [`Ash.Changeset`] add `Ash.Changeset.force_delete_argument/2`\n\n- [`Ash.Policy.Check.Builtins`] add `actor_absent` builtin check\n\n- [`Ash.Changeset`] Warn when `manage_relationship` is called without opts (#1408)\n\n- [`Ash.Resource.Validation.Builtins`] use Comp with attribute_equals and attribute_does_not_equal (#1623)\n\n## [v3.4.43](https://github.com/ash-project/ash/compare/v3.4.42...v3.4.43) (2024-11-20)","ref":"changelog.html#improvements-37"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [generic actions] store notifications from simple results (#1591)\n\n## [v3.4.42](https://github.com/ash-project/ash/compare/v3.4.41...v3.4.42) (2024-11-07)","ref":"changelog.html#bug-fixes-51"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.generate_livebook`] Allow multi-line descriptions to be rendered in Livebook without error (#1590)\n\n- [`Ash.Policy.Check.ChangingAttributes`] properly handle `changing_attributes` check with `from` in create (#1584)","ref":"changelog.html#bug-fixes-52"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Type.Union`] support map w/ keys `_union_type` and `_union_value` for union inputs\n\n## [v3.4.41](https://github.com/ash-project/ash/compare/v3.4.40...v3.4.41) (2024-11-05)","ref":"changelog.html#improvements-38"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Type.Struct`] don't double wrap casted struct instances in `{:ok, {:ok, ...}}`\n\n- [`Ash.Type.Struct`] support mixed key types in input maps for structs\n\n## [v3.4.40](https://github.com/ash-project/ash/compare/v3.4.39...v3.4.40) (2024-11-04)","ref":"changelog.html#bug-fixes-53"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Actions.Read`] don't call `.name` on a potentially `nil` action\n\n- [`Ash.Expr`] properly detect `Ash.CustomExpression` as an expr\n\n- [`Ash.Expr`] accept fragments pointing at non-aliases\n\n- [`Ash.Expr`] smarter type detection, preferring more concrete types\n\n## [v3.4.39](https://github.com/ash-project/ash/compare/v3.4.38...v3.4.39) (2024-11-01)","ref":"changelog.html#bug-fixes-54"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Changeset`] emit warnings about already validated actions in before_action hooks too\n\n- [`Ash.Changeset`] add missing case clause for `Changeset.atomic_update` type cast (#1569)\n\n- [`Ash.Type.Map`] handle case of invalid map/keyword key when type casting\n\n- [`Ash.Type.Keyword`] handle case of invalid map/keyword key when type casting","ref":"changelog.html#bug-fixes-55"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [error messages] more better error bread crumbs\n\n## [v3.4.38](https://github.com/ash-project/ash/compare/v3.4.37...v3.4.38) (2024-10-31)","ref":"changelog.html#improvements-39"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Changeset`] detect transaction hooks added by around_transaction and before_transaction & manual actions\n\n- [`Ash.Changeset`] don't special case `nil` change on `force_change_attribute`","ref":"changelog.html#bug-fixes-56"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Domain`] default backwards compatible interface to false for domains\n\n- [`Ash.Changeset`, create actions] more and better bread crumbs for changesets & create actions\n\n## [v3.4.37](https://github.com/ash-project/ash/compare/v3.4.36...v3.4.37) (2024-10-30)","ref":"changelog.html#improvements-40"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Type.Union`] handle nil union changing to nil\n\n- [multitenancy] enforce multitenancy on bulk creation\n\n- [`Ash.Changeset`] force_change_attribute no longer cares what the old value is","ref":"changelog.html#bug-fixes-57"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Changeset`] allow specifying `return_skipped_upsert?` as an option to changeset\n\n- [`Ash.DataLayer`] add `prefer_transaction_for_atomic_updates?` data layer callback\n\n- [`Ash.DataLayer`] support `prefer_transaction?` on DataLayer\n\n- [loading data] allow data loading when no primary read action exists\n\n## [v3.4.36](https://github.com/ash-project/ash/compare/v3.4.35...v3.4.36) (2024-10-24)","ref":"changelog.html#improvements-41"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.stream!`] ensure opts are passed through to constructed query in stream","ref":"changelog.html#bug-fixes-58"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Check.Builtins`] add `just_created_with_action/1` check\n\n## [v3.4.35](https://github.com/ash-project/ash/compare/v3.4.34...v3.4.35) (2024-10-22)","ref":"changelog.html#improvements-42"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [code interfaces] allow optional code interface args for fields with defaults\n\n- [atomic updates] write all attributes to atomics list before dispatching to data layer on update query\n\n- [`mix ash.gen.resource`] remove multichar aliases from `mix ash.gen.resource`\n\n- [`Ash.Type.Decimal`] check nil before calling Decimal.eq? in Type.Decimal.equal? (#1538)\n\n## [v3.4.34](https://github.com/ash-project/ash/compare/v3.4.33...v3.4.34) (2024-10-21)","ref":"changelog.html#bug-fixes-59"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.gen.resource`] properly accept options for `ash.gen.resource` in installer\n\n## [v3.4.33](https://github.com/ash-project/ash/compare/v3.4.32...v3.4.33) (2024-10-18)","ref":"changelog.html#bug-fixes-60"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk updates] apply attribute multitenancy on bulk update queries\n\n- [`Ash.Type.Decimal`] use Decimal.eq? in Ash.Type.Decimal (#1532)\n\n- [`Ash.Reactor`]: Don't validate `inputs` keys when being transformed. (#1527)\n\n- [atomic updates] set argument defaults in fully atomic changesets\n\n- [`Ash.Changeset`] ensure that default values are included in attribute changes\n\n- [manage_relationship] properly unrelate belongs_to relationships\n\n- [manage_relationship] ensure unrelated records are removed from the current records list","ref":"changelog.html#bug-fixes-61"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Resource`] Conditionally enable transactions on default actions. (#1525)\n\n- [`Ash.Seed`] ash seed upsert! function (#1522)\n\n- [code interfaces] Add compile-time checks for `code_interface` arguments in Resource and Domain (#1523)\n\n## [v3.4.32](https://github.com/ash-project/ash/compare/v3.4.31...v3.4.32) (2024-10-14)","ref":"changelog.html#improvements-43"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`mix ash.gen.resource`] use new `:csv` option type from igniter\n\n## [v3.4.31](https://github.com/ash-project/ash/compare/v3.4.30...v3.4.31) (2024-10-14)","ref":"changelog.html#improvements-44"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [all actions] allow strings in generic action skip_unknown_inputs\n\n## [v3.4.30](https://github.com/ash-project/ash/compare/v3.4.29...v3.4.30) (2024-10-14)","ref":"changelog.html#bug-fixes-62"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [all actions] add `tracer` option to generic action opts\n\n## [v3.4.29](https://github.com/ash-project/ash/compare/v3.4.28...v3.4.29) (2024-10-13)","ref":"changelog.html#bug-fixes-63"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Changeset`] clear change from atomics as well\n\n- [read actions] properly invoke notify callback in read actions","ref":"changelog.html#bug-fixes-64"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Changeset`] better ergonomics for atomic updates\n\n- [changeset, action inputs, queries] add `private_arguments` option\n\n- [`Ash.ActionInput`] validate `Ash.ActionInput.for_action` opts\n\n- [`Ash.Type.NewType`] allow additional callbacks in `Ash.Type.NewType`\n\n## [v3.4.28](https://github.com/ash-project/ash/compare/v3.4.27...v3.4.28) (2024-10-10)","ref":"changelog.html#improvements-45"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [upserts] support lazy evaluation of skipped upsert records\n\n## [v3.4.27](https://github.com/ash-project/ash/compare/v3.4.26...v3.4.27) (2024-10-10)","ref":"changelog.html#improvements-46"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [upserts] emit StaleRecordError on skipped upsert\n\n## [v3.4.26](https://github.com/ash-project/ash/compare/v3.4.25...v3.4.26) (2024-10-08)","ref":"changelog.html#improvements-47"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [query building] properly hydrate sort with parent context\n\n- [query building] handle nested parent references in runtime expression logic\n\n- [query building] set parent stack when hydrating references in related queries\n\n## [v3.4.25](https://github.com/ash-project/ash/compare/v3.4.24...v3.4.25) (2024-10-07)","ref":"changelog.html#bug-fixes-65"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [igniter] honor --yes or -y option when adding a satsolver\n\n- [query building] don't error on type casting against expressions\n\n## [v3.4.24](https://github.com/ash-project/ash/compare/v3.4.23...v3.4.24) (2024-10-07)","ref":"changelog.html#bug-fixes-66"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [policies] short-circuit policy condition evaluation when checking all conditions\n\n- [query building] properly hydrate aggregate calculations & fields","ref":"changelog.html#bug-fixes-67"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [policies] re-introduce removed behavior to short circuit policy conditions\n\n## [v3.4.23](https://github.com/ash-project/ash/compare/v3.4.22...v3.4.23) (2024-10-03)","ref":"changelog.html#improvements-48"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [loading data] don't rewrite calculation dependencies through not loaded/forbidden fields\n\n- [loading data] honor `reuse_values?` when lazy loading relationships\n\n## [v03.4.22](https://github.com/ash-project/ash/compare/v3.4.21...v03.4.22) (2024-10-01)","ref":"changelog.html#bug-fixes-68"},{"type":"extras","title":"Features: - Changelog","doc":"- [`Ash.Reactor`] Add the ability to specify action context in steps. (#1477)","ref":"changelog.html#features-1"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Query`] properly merge query calculations when one side is empty\n\n- [`Ash.Query`] remove expensive calculation reification step that is no longer necessary\n\n- [`Ash.Sort`] handle expression calculations that reference fields in input sorting\n\n- [`Ash.Sort`] properly apply field policies to all filter expressions\n\n- [`Ash.Type.Struct`] better error message on missing `instance_of` constraint on load-through\n\n- [authorization] select minimal data in authorization queries\n\n- [`Ash.Query`] calling `for_read/2..4` should raise an `ArgumentError` when the specified action doesn't exist. (#1479)\n\n- [`Ash.Changeset`] detect non-changing but setting attributes to honor `require_attributes` on update\n\n- [`Ash.Changeset`] set right defaults for `action_select` (#1476)\n\n- [`Ash.Expr`] don't resolve references when falling back to elixir handling for expressions","ref":"changelog.html#bug-fixes-69"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Authorizer`] disallow 2-tuple expression checks, to resolve ambiguity\n\n- [`Ash.Policy.Authorizer`] rewrite and drastically simplify policy -> solver expression logic\n\n- [`Ash.Changeset`] properly handle bypasses of atomic constraint casting\n\n- [`Ash.Changeset`] ensure that action_select sets attributes to `%Ash.NotLoaded{}`\n\n- [`Ash.Query.Calculation`] add `Ash.Query.Calculation.from_resource_calculation`\n\n- [`Ash.Query.Calculation`] fallback to runtime calculations when expressions aren't supported\n\n- [`Ash.Type.DateTime`] handle iso8601 dates in datetime cast\n\n- [`Ash.DataLayer`] add data layer capability for action select\n\n- [`Ash.Query.Calculation`] inspect calculations in queries more fluidly\n\n- [`Ash.Resource.Igniter`] add_identity for `Ash.Resource.Igniter`\n\n## [v3.4.21](https://github.com/ash-project/ash/compare/v3.4.20...v3.4.21) (2024-09-24)","ref":"changelog.html#improvements-49"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash`] handle `nil` result in `Ash.first`\n\n- [bulk actions] add checks for `around_transaction` and `around_action` in bulk (#1474)\n\n- [`Ash.Query.Aggregate`] include distinct from queries in aggregate query\n\n- [read actions] reselect required attributes unless `reuse_values?` is `true`\n\n- [`Ash.Changeset`] properly return `{:not_atomic` while applying atomic changes","ref":"changelog.html#bug-fixes-70"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Query.Aggregate`] proper error on unsupported aggregates\n\n## [v3.4.20](https://github.com/ash-project/ash/compare/v3.4.19...v3.4.20) (2024-09-23)","ref":"changelog.html#improvements-50"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [read actions] don't double-load data on bulk update reads","ref":"changelog.html#bug-fixes-71"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash`] support more formats in `Ash.can`\n\n- [`Ash`] add `validate?` option to `Ash.can`/`Ash.can?`\n\n## [v3.4.19](https://github.com/ash-project/ash/compare/v3.4.18...v3.4.19) (2024-09-21)","ref":"changelog.html#improvements-51"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Resource`] properly generate bypasses with `Ash.Resource.Igniter.add_bypass/2`","ref":"changelog.html#bug-fixes-72"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Sort`] support nested fields in input sorts\n\n- [optimization] optimize the reselection of necessary attributes on lazy loading\n\n- [`Ash.Resource`, optimization] optimize `Ash.Resource.selected?/2` in light of 3.0 changes\n\n## [v3.4.18](https://github.com/ash-project/ash/compare/v3.4.17...v3.4.18) (2024-09-20)","ref":"changelog.html#improvements-52"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Resource.Change.OptimisticLock`] properly increment version in optimist lock's non-atomic branch\n\n- [`Ash.Policy.Authorizer`] ensure that policy group compile time validations are enforced\n\n- [bulk updates] ensure that around_transaction and around_action hooks incur simple updates\n\n## [v3.4.17](https://github.com/ash-project/ash/compare/v3.4.16...v3.4.17) (2024-09-19)","ref":"changelog.html#bug-fixes-73"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Query`] handle more types in `Ash.Query.unload`\n\n- [`Ash.Changeset`] properly escape changeset.select in `Ash.Changeset.ensure_selected` (#1466)","ref":"changelog.html#bug-fixes-74"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Tracer`] add span & telemetry events for running calculations\n\n- [`Ash.Policy.Check.Builtins`] validate action types in `action_type` check\n\n## [v3.4.16](https://github.com/ash-project/ash/compare/v3.4.15...v3.4.16) (2024-09-18)","ref":"changelog.html#improvements-53"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Seed`] ensure Ash.Seed always sets action_select","ref":"changelog.html#bug-fixes-75"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Authorizer`] properly log successful policy breakdowns with extra info\n\n- [`Ash.Filter`] add `Ash.Filter.fetch_simple_equality_predicate`\n\n## [v3.4.15](https://github.com/ash-project/ash/compare/v3.4.14...v3.4.15) (2024-09-17)","ref":"changelog.html#improvements-54"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Query`] add `load` option to `Ash.Query.for_read`\n\n## [v3.4.14](https://github.com/ash-project/ash/compare/v3.4.13...v3.4.14) (2024-09-17)","ref":"changelog.html#improvements-55"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.gen.domain`] properly detect domains that don't exist yet in `ash.gen.domain`\n\n## [v3.4.13](https://github.com/ash-project/ash/compare/v3.4.12...v3.4.13) (2024-09-17)","ref":"changelog.html#bug-fixes-76"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Changeset`] honor `skip_global_validations?` on fully atomic changesets\n\n- [`Ash.Sort`] ensure calculation context is fully propagated to sort statements\n\n- [`Ash.Policy.Authorizer`] ensure that `resource` context is set for expanding filter descriptions\n\n## [v3.4.12](https://github.com/ash-project/ash/compare/v3.4.11...v3.4.12) (2024-09-16)","ref":"changelog.html#bug-fixes-77"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Seed`] ensure that action_select is set on seeding data","ref":"changelog.html#bug-fixes-78"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Resource.Igniter`] add more resource updating logic\n\n- [`Ash.Resource.Igniter`] add `_new` options for Ash.Resource.Igniter\n\n## [v3.4.11](https://github.com/ash-project/ash/compare/v3.4.10...v3.4.11) (2024-09-13)","ref":"changelog.html#improvements-56"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [igniter] update igniter and fix deprecation warnings\n\n## [v3.4.10](https://github.com/ash-project/ash/compare/v3.4.9...v3.4.10) (2024-09-13)","ref":"changelog.html#improvements-57"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.patch.extend`] properly add all types of extensions in `mix ash.patch.extend`\n\n## [v3.4.9](https://github.com/ash-project/ash/compare/v3.4.8...v3.4.9) (2024-09-13)","ref":"changelog.html#bug-fixes-79"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [field policies] ensure that field policies don't interfere with relationship loading\n\n- [bulk actions] properly merge provided context in atomic bulk actions\n\n- [managed relationships] properly handle rollbacks from `DBConnection` failures for belongs to relationships\n\n- [`Ash.Resource.Igniter`] don't generate doubly nested policies when adding policies in igniter\n\n- [`Ash.Changeset`] fix Ash.Changeset.manage_relationships/4 for list primary keys (#1455)\n\n- [`Ash.Filter`] Handle Ash.Query.filter for array values (#1452)\n\n- [`Ash.Type.Time`] cast embedded time properly (#1451)\n\n- [create actions] require private/non-accepted attributes _after_ before action hooks instead of _before_\n\n- [built in after_action change] we cannot assume that `after_action/1` can be done atomically\n\n  Previously, when you did `change after_action/3` in a resource, we would assume it was safe to be done atomically.\n  But because we cannot guarantee that your hook does not access `changeset.data`, it is not safe to make that assumption.\n\n  Instead, you must define a module change, and explicitly define `atomic/3`.","ref":"changelog.html#bug-fixes-80"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Error.Forbidden.Policy`] small improvements for policy breakdown formatting\n\n- [`Ash.Type.Union`] honor a `_union_type` type param when casting unions\n\n- [create/update/destroy actions] add system for `action_select`, which can limit selects from mutations\n\n  Callers can `select` when calling create/update/destroy actions, but those selects were not previously honored\n  by data layers. The reason for this is that often actions will require more fields than the fields that the caller\n  requests. Now, you can specify `action_select` in the action, and the data layer will honor that.\n\n  Additionally, the new `select_by_default?` flag on attributes causes the attribute to automatically not be selected\n  for update actions.\n\n- [attributes] support `select_by_default?` flag on attributes. This defaults to `true`.\n\n## [v3.4.8](https://github.com/ash-project/ash/compare/v3.4.7...v3.4.8) (2024-09-09)","ref":"changelog.html#improvements-58"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Policy.Authorizer`] support passing a forbidden error for policy breakdowns per the docs\n\n- [`Ash.Policy.Authorizer`] don't report the action as the actor for policy breakdowns\n\n- [`Ash.Changeset`] check `changeset.action` before raising a required primary action error\n\n- [bulk actions] ensure proper return types for `:stream` strategy bulk update/destroys\n\n## [v3.4.7](https://github.com/ash-project/ash/compare/v3.4.6...v3.4.7) (2024-09-06)","ref":"changelog.html#bug-fixes-81"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Resource.Igniter`] add `add_bypass` and `add_policy` igniter utilities\n\n## [v3.4.6](https://github.com/ash-project/ash/compare/v3.4.5...v3.4.6) (2024-09-06)","ref":"changelog.html#improvements-59"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [loading relationships] don't select destination attributes that don't exist\n\n- [`Ash.Filter.Runtime`] properly pass actor when running filters at runtime\n\n- [`Ash.Type.Struct`] misplaced curly bracket when handling struct type casting\n\n- [bulk/atomic updates] properly leverage atomic upgrade read action for an update action","ref":"changelog.html#bug-fixes-82"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Authorizer`] show informative error explaining the use of filter checks with create actions\n\n- [`Ash.Policy.Authorizer`] show the actor's primary key in policy breakdowns\n\n- [`Ash.Policy.Authorizer`] add an expanded description option to checks\n\n- [`Ash.Policy.Authorizer`] use expanded description to display filled in filter templates in policy breakdowns\n\n- [`Ash.Changeset` ] Add `Ash.Changeset.is_valid/1` guard. (#1437)\n\n## [v3.4.5](https://github.com/ash-project/ash/compare/v3.4.4...v3.4.5) (2024-09-05)","ref":"changelog.html#improvements-60"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [update actions] fix type definition for `atomic_upgrade_with`\n\n## [v3.4.4](https://github.com/ash-project/ash/compare/v3.4.3...v3.4.4) (2024-09-05)\n\n## [v3.4.3](https://github.com/ash-project/ash/compare/v3.4.2...v3.4.3) (2024-09-04)","ref":"changelog.html#bug-fixes-83"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [loading relationships] properly await tasks from lazy loading multiple relationships\n\n## [v3.4.2](https://github.com/ash-project/ash/compare/v3.4.1...v3.4.2) (2024-09-04)","ref":"changelog.html#bug-fixes-84"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [soft destroys] honor `return_destroyed?` in soft destroy actions\n\n- [`Ash.Resource.Change`] correctly handle return values of batch callbacks (#1424)\n\n- [read actions] ensure that async limiter is cleared up front\n\n- [bulk creates] honor bulk upsert condition (#1432)\n\n- [bulk updates] ensure that update_defaults are set on streaming updates\n\n- [bulk actions] honor `skip_global_validations?` in bulk actions\n\n- [pagination] honor the `countable` option in pagination\n\n- [read actions] return proper data shape when doing a read in a transaction\n\n- [notifications] ensure that `from` is properly set on all notifications\n\n- [notifications] fix typo in bulk destroy not clearing ash_started_transaction state\n\n- [calculations] traverse calculated relationships when rewriting transient calculation values\n\n- [calculations] don't unload calculation dependencies when `lazy?` is set\n\n- [`Ash.DataLayer.Ets`] handle no_attributes when joining lateral join relationship data\n\n- [`Ash.DataLayer.Ets`] fix ets lateral join source field usage\n\n- [`Ash.DataLayer.Ets`] properly apply distinct in ets","ref":"changelog.html#bug-fixes-85"},{"type":"extras","title":"Improvements with backwards compatibility configurations - Changelog","doc":"These configurations default to the current behavior, but in 4.0 (someday) will\nbe removed, and the new option will be the only option.\n\n- [pagination] make default page type configurable, defaulting to `:offset`.\n\n```elixir\n# set this configuration to adopt the new preferred behavior\nconfig :ash,\n  default_page_type: :keyset\n```\n\n- [`Ash.Policy.Authorizer`] make read policies more consistent, always preferring to filter over raise\n\nCurrently, some read actions can still return a `Forbidden` error, even\nthough policies are meant to filter out records by default. Now, it will always\nfilter, unless you set `access_type :strict` in the policy.\n\n```elixir\n# set this configuration to adopt the new preferred behavior\nconfig :ash, :policies,\n  no_filter_static_forbidden_reads?: false\n```","ref":"changelog.html#improvements-with-backwards-compatibility-configurations"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Policy.Authorizer`] show an explanation when a forbidden is because no policies applied\n\n- [`Ash.Policy.Authorizer`] error at compile for bypasses that will have no effect\n\n- [`Ash.Resource.ManualRead`] add `load_relationships/5` callback to manual reads\n\n- [`mix ash.gen.resource`] add `uuid-v7-primary-key` option to `mix ash.gen.resource`\n\n- [`Ash.Resource.Change.CascadeUpdate`] add cascade update built in change (#1398)\n\n- [`Ash.Resource.Change.CascadeDestroy`] add `read_action` option to `cascade_destroy`\n\n- [inline aggregates] support `expr` and `expr_type` options when building aggregates\n\n- [create actions] Implement condition for upsert (#1386)\n\n- [optimization] do not add relationship filter when building relationship authorization\n\n- [optimization] don't list telemetry handlers if app is compiling\n\n- [optimization] do not call tracer `set_metadata` with span type that it ignores (#1400)\n\n- [optimization] optimize filter expr traversal\n\n- [optimization] Add a case for handling mapsets in Filter.map (#1427)\n\n- [optimization] Cache always selected fields and use mapsets for building select list (#1428)\n\n- [optimization] add pattern for Ash.Query.Call in Filter.map (#1416)\n\n- [optimization] prevent unnecessary calls to `Ash.load`\n\n- [optimization] cache action known inputs individually\n\n- [optimization] cache action required inputs all together\n\n- [optimization] optimize to avoid inspects in changesets\n\n- [optimization] optimize to avoid expensive `String.valid?` check in uuid type\n\n- [optimization] add `async?` option to calculations, default to false\n\n- [optimization] optimize field checking for loading fields in query\n\n- [optimization] allow functions in tracers for lazy loading metadata\n\n- [optimization] don't start processes for single items in list\n\n- [optimization] Optimize option validation with compile time validators (#1387)\n\n## [v3.4.1](https://github.com/ash-project/ash/compare/v3.4.0...v3.4.1) (2024-08-13)","ref":"changelog.html#improvements-61"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [authorization] properly pass actor, action, tenant etc. to lazy loaded relationships\n\n## [v3.4.0](https://github.com/ash-project/ash/compare/v3.3.3...v3.4.0) (2024-08-12)","ref":"changelog.html#bug-fixes-86"},{"type":"extras","title":"Features: - Changelog","doc":"- [`Ash.Policy.Authorizer`] add policy groups\n\n- [authorization] support `authorize_with` option on `Ash.read`, allowing to error if any forbidden matching data exists\n\n- [`Ash.Resource`] Add `@type t` typespec to resource if it doesn't exist","ref":"changelog.html#features-2"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [calculations] remove pattern match error when exceptions come from calculations\n\n- [calculations] resolve nested expression calculation references in runtime filters\n\n- [arrays] Fix error with nil value on structure types (#1380)\n\n- [bulk actions] Pass options without :templated tuple to after_batch (#1376)\n\n- [bulk actions] `after_batch` arguments for `bulk_create` with `return_records?` disabled (#1371)\n\n- [bulk actions] set `upsert?` option when managing relationships in bulk creation\n\n- [`Ash.Resource`] check for `nil` resource_calculation in `Ash.Resource.loaded?/2`\n\n- [`Ash.Expr`] properly consider not-loaded record calculations as `:unknown`\n\n- [atomic updates] handle `nil` in atomic array casting\n\n- [aggregates] respect previously validated-for-action query for aggregates\n\n- [`mix ash.gen.resource`] use `timestamps()` instead of `timestamps`\n\n- [belongs_to relationships] prefer `source_attribute` is required, instead of relationship name","ref":"changelog.html#bug-fixes-87"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [performance] optimizations around allocating strings\n\n- [performance] optimizations around list operations for embedded resources\n\n- [performance] prune calculations made unnecessary by field policies\n\n- [performance] add optimized path for casting embeds when they are simple\n\n- [performance] add `include_embedded_source_by_default?` config to optimize embeds\n\n- [error messages] show proper error message when trying to accept non-writable attributes\n\n- [error messages] add \"hints\" to `NoSuchInput` errors to make any errors clearer\n\n- [`Ash.Expr`] warn on usage of `== nil`\n\n- [`Ash.Expr`] implement `Comp` for atoms & strings, comparing atoms as strings\n\n- [embedded resources] increase cases where embedded attribute can be updated atomically\n\n- [`Ash.Type.Struct`] support `:fields` constraint on `:struct` type, enabling persistence\n\n- [bulk actions] Warn on bulk action `return_stream?` without any other `return_*?` options enabled. (#1370)\n\n- [calculations] ensure the called calculation function is always in the stacktrace\n\n- [dependencies] remove `:comparable` as a dependency\n\n- [`Ash.Resource`] Add default values to resulting Resource struct (#1364)\n\n## [v3.3.3](https://github.com/ash-project/ash/compare/v3.3.2...v3.3.3) (2024-08-01)","ref":"changelog.html#improvements-62"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- delete ash_notifications from pdict after reading\n\n## [v3.3.2](https://github.com/ash-project/ash/compare/v3.3.1...v3.3.2) (2024-08-01)","ref":"changelog.html#bug-fixes-88"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`Ash.Igniter`] properly parse multiple occurrences of :keep arguments\n\n- [calculations] properly key nested calculations and add additional tests\n\n- [calculations] pass relationship path down when merging query loads\n\n- [`mix ash.codegen`] don't set `--name nil` when calling codegen tasks\n\n- [`Ash.Filter`] fix behavior of synthesized joins across data layers","ref":"changelog.html#bug-fixes-89"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`mix ash.gen.resource`] add `--timestamps` argument to add timestamps to the resource\n\n## [v3.3.1](https://github.com/ash-project/ash/compare/v3.3.0...v3.3.1) (2024-07-30)","ref":"changelog.html#improvements-63"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.gen.domain`] properly parse domain module in `mix ash.gen.domain`\n\n- [`Ash.Resource.Change`, `Ash.Resource.Validation`] properly handle mixed atomic & non-atomic validations/changes\n\n- [`Ash.Filter`] properly find data layer predicates when name is provided as a string\n\n- [relationships] set `accessing_from` and join read action for many_to_many relationships correctly (#1355)","ref":"changelog.html#bug-fixes-90"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Resource.Change`] implement `change/3` automatically if batch callbacks are defined\n\n## [v3.3.0](https://github.com/ash-project/ash/compare/v3.2.6...v3.3.0) (2024-07-27)","ref":"changelog.html#improvements-64"},{"type":"extras","title":"Features: - Changelog","doc":"- [`Ash.Type.File`] Add Ash.Type.File (#1337)","ref":"changelog.html#features-3"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk actions] ensure that statuses are set correctly on bulk actions\n\n- [bulk actions] properly transfer process context(tracers) for bulk actions\n\n- [embedded resources] properly display identity/primary key mismatch error on lists of embeds\n\n- [`Ash.Type.NewType`] apply constraints to NewType even when casting an array (#1341)\n\n- [`Ash.Query`] pass reuse_values? true when loading in Ash.Query.apply_to/2 (#1346)\n\n- [code interfaces] honor `skip_unknown_inputs` in code interfaces\n\n- [notifications] don't gather notifications except for in the top level transaction starter\n\n- [generic actions] support `skip_unknown_inputs` on generic actions\n\n- [atomic updates] ensure `changed?` context is set on atomic changesets (we assume it is `true`, its up to extensions to handle)\n\n- [`Ash.Type.CiString`, `Ash.Type.String`] Update string/ci_string generators to ensure min_length (#1335)\n\n- [`Ash`] handle `{record, action, input}` types in `Ash.can?`\n\n- [bulk actions] only call `batch_change` if it is defined, never `change` in bulk create","ref":"changelog.html#bug-fixes-91"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`mix ash.gen.resource`] better positional argument handling with igniter\n\n- [`Ash.Expr`] use `:utc_datetime_usec` for `now()` return type\n\n- [`mix ash.install`] don't install sat solver in initial installation\n\n- [`Ash.Policy.Authorizer`] validate that a solver exists at compile time when using policies\n\n- [`Ash.Type.Enum`] Expose type t() on Ash.Type.Enum implementations (#1338)\n\n- [`Ash.Resource`] add :\\* as a valid value in `skip_unknown_inputs`\n\n- [`Ash.Resource`] add `skip_unknown_inputs` to individual actions\n\n- [embedded resources] add `skip_unknown_inputs` constraint to embedded resources\n\n- [embedded resources] automatically fall back to a default domain when working with embeds\n\n- [`Ash`] handle 3 tuple in `Ash.can?`\n\n- [`Ash.Error`] add `Ash.Error.error_descriptions`\n\n## [v3.2.6](https://github.com/ash-project/ash/compare/v3.2.5...v3.2.6) (2024-07-22)","ref":"changelog.html#improvements-65"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk actions] fallback to `authorize_with` when authorizing bulk destroy actions\n\n- [bulk actions] don't refer to non-existent `batch_change/4`","ref":"changelog.html#bug-fixes-92"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [bulk actions] Replace incorrect function_exported?-checks in bulk-actions, add has-defs for change modules (#1330)\n\n## [v3.2.5](https://github.com/ash-project/ash/compare/v3.2.4...v3.2.5) (2024-07-22)","ref":"changelog.html#improvements-66"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [destroy actions] apply atomic validations on non-bulk destroy actions\n\n- [`Ash.Policy.Info`] add default to private_fields_policy in Ash.Policy.Info (#1329)\n\n- [relating_to_actor] make `relating_to_actor` built-in-check aware of atomics\n\n- [`Ash.Expr`] remove redundant overload of `Ash.Expr.get_path` (#1328)\n\n- [`Ash.Type.NewType`] cast_input/2 of Ash.Type.NewType returning :ok (#1324)\n\n- [`Ash.Reactor`] warnings emitted by removed reactor behaviour function. (#1325)","ref":"changelog.html#bug-fixes-93"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [bulk actions] add `authorize_with` fallback option to bulk actions\n\n- [`Ash.Policy.Authorizer`] allow policy conditions to be applied inside their block\n\n## [v3.2.4](https://github.com/ash-project/ash/compare/v3.2.3...v3.2.4) (2024-07-18)","ref":"changelog.html#improvements-67"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [transaction hooks] fix warning on transaction hooks violating their semantics\n\n## [v3.2.3](https://github.com/ash-project/ash/compare/v3.2.2...v3.2.3) (2024-07-18)","ref":"changelog.html#bug-fixes-94"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [`mix ash.patch.extend`] properly convert extension string into a module\n\n- [`mix ash.patch.extend`] only display available to extend\n\n- [`mix ash.install`] mix igniter.install ash --example case clause error (#1317)\n\n- [multitenancy] only use attribute for filtering when multitenancy strategy == :attribute","ref":"changelog.html#bug-fixes-95"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Resource.Igniter`] Add `Ash.Resource.Igniter.domain` to get the domain of a resource\n\n## [v3.2.2](https://github.com/ash-project/ash/compare/v3.2.1...v3.2.2) (2024-07-17)","ref":"changelog.html#improvements-68"},{"type":"extras","title":"Features: - Changelog","doc":"- [`Ash.Reactor`] Add ash_step wrapper (#1311)","ref":"changelog.html#features-4"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk destroys] honor atomic validations in destroy actions using `filter`\n\n- [Ash.Type.Vector] handle casting nil vectors (#1316)\n\n- [Ash.Type] don't override `nil` handling in `Ash.Type.cast_input/3`\n\n## [v3.2.1](https://github.com/ash-project/ash/compare/v3.2.0...v3.2.1) (2024-07-17)","ref":"changelog.html#bug-fixes-96"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- properly honor the `include_nil?` option\n\n- store after_action hooks added outside of changes for atomic upgrade\n\n- don't use `type/3` in string interpolation\n\n- properly pass `include_nil?` from when building query aggregates","ref":"changelog.html#bug-fixes-97"},{"type":"extras","title":"Improvements: - Changelog","doc":"- allow skipping initialization of types in unions\n\n## [v3.2.0](https://github.com/ash-project/ash/compare/v3.1.8...v3.2.0) (2024-07-15)","ref":"changelog.html#improvements-69"},{"type":"extras","title":"Features: - Changelog","doc":"- [field policies] Allow field policies to hide private fields (#1289)\n\nUse the `private_fields :include | :show | :hide` option in the `field_policies` section of your resource to control how private fields are handled by field policies.\n\nFor example:\n\n```elixir\n# hide all private fields when authorizing\nfield_policies :hide\n\n# the current behavior. Private fields are ignored by field policies\nfield_policies :show\n\n# private fields can have field policies like any other field\nfield_policies :include\n```","ref":"changelog.html#features-5"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [`Ash.Domain.Igniter`] add `Ash.Domain.Igniter.list_domains/1`\n\n- [`Ash.Resource.Igniter`] add `Ash.Resource.Igniter.list_resources/1`\n\n## [v3.1.8](https://github.com/ash-project/ash/compare/v3.1.7...v3.1.8) (2024-07-14)","ref":"changelog.html#improvements-70"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk actions] use `unpaginated_read` when simulating streaming for low limit queries\n\n## [v3.1.7](https://github.com/ash-project/ash/compare/v3.1.6...v3.1.7) (2024-07-14)","ref":"changelog.html#bug-fixes-98"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Query] don't use `:same` return type for most operators\n\n- [Ash.Query] don't use returns as basis type unless explicitly allowed\n\n## [v3.1.6](https://github.com/ash-project/ash/compare/v3.1.5...v3.1.6) (2024-07-14)","ref":"changelog.html#bug-fixes-99"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Query] ensure `today` has properly configured returns type\n\n- [Ash.Type] module type apply_constraints for nil fix (#1313)\n\n## [v3.1.5](https://github.com/ash-project/ash/compare/v3.1.4...v3.1.5) (2024-07-14)","ref":"changelog.html#bug-fixes-100"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Type] don't specify that `get_path` is a predicate function","ref":"changelog.html#bug-fixes-101"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Expr] add `Ash.Expr.determine_type(mod, children)`\n\n- [Ash.Query] add return typing to functions\n\n## [v3.1.4](https://github.com/ash-project/ash/compare/v3.1.3...v3.1.4) (2024-07-13)","ref":"changelog.html#improvements-71"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [code interface] properly omit destroyed result in code interfaces\n\n- [Ash.Type.Integer] properly compare `expr` to `min` with `min` integer constraint\n\n- [Ash.Reactor] Make action ctx-values from reactor-ctx take precedence if set. (#1308)","ref":"changelog.html#bug-fixes-102"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Resource.Change] support returning a list of atomics from atomic change callbacks\n\n- [Ash.Type] add `cast_atomic_constraints` callback and use it in core types\n\n- [Ash.Expr] improve type signature for `if/3`\n\n- [Ash.Expr] simpler and/or short circuiting\n\n## [v3.1.3](https://github.com/ash-project/ash/compare/v3.1.2...v3.1.3) (2024-07-11)","ref":"changelog.html#improvements-72"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk actions] ensure that errors in queries do not raise in atomic upgrades/single atomics\n\n- [Ash.Type.Integer] use correct constraint when validating min int (#1298)\n\n- [Ash.Filter] don't refer to private attributes when parsing filter inputs that refer to relationships (#1280)","ref":"changelog.html#bug-fixes-103"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Query] add `strict?` option to `Ash.Query.load` (#1302)\n\n## [v3.1.2](https://github.com/ash-project/ash/compare/v3.1.1...v3.1.2) (2024-07-10)","ref":"changelog.html#improvements-73"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk actions] ensure that manual action configurations are honored for bulk actions\n\n## [v3.1.1](https://github.com/ash-project/ash/compare/v3.1.0...v3.1.1) (2024-07-10)","ref":"changelog.html#bug-fixes-104"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [ash.install] installer doesn't need to add spark as a dependency, just run its installer\n\n## [v3.1.0](https://github.com/ash-project/ash/compare/v3.0.16...v3.1.0) (2024-07-09)","ref":"changelog.html#bug-fixes-105"},{"type":"extras","title":"Features: - Changelog","doc":"- [Generators] add `mix ash.install` (call with `mix igniter.install ash`)\n\n- [Generators] add `mix ash.gen.resource`\n\n- [Generators] add `mix ash.gen.base_resource`\n\n- [Generators] add `mix ash.gen.domain`\n\n- [Generators] add `mix ash.extend`\n\n- [Ash.Type.UUIDv7] Add built in `Ash.Type.UUIDv7` type, and `uuid_v7_primary_key` builder","ref":"changelog.html#features-6"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [atomics] sort primary key changes ahead of others in atomic changes\n\n- [Ash.Changeet] fix typespec for Changeset.around_transaction/2 (#1292)\n\n- [multitenancy] ensure tenancy is always enforced on create/update/destroy actions\n\n- [loading relationships] lateral join on `from_many? true` relationships\n\n- [calculations] don't reuse calculations/aggregates if `authorize?` is true\n\n- [aggregates] ensure aggregate context is fully configured in `Ash.aggregate`\n\n- [bulk actions] properly transfer changeset.context on streamed batch changesets\n\n- [bulk actions] ensure notifications are dispatched from bulk actions\n\n- [lazy loading] lazy-loading logic for calculations/aggregates was inversed (#1275)\n\n- [error handling] properly match on async task exceptions\n\n- [policies] ensure `context` is available in policy template expressions\n\n- [policies] ensure forbidden errors behave the same when using implicit bulk version of code interface functions\n\n- [manual relationships] compare keys in manual relationships when using 'complex' types (#1270)\n\n- [Ash.Filter] cover more cases in filter input parsing (#1261)\n\n- [has_one relationships] automatically set `from_many?` if a `has_one` has a `sort` applied\n\n- [Ash.Filter] fix match error when synthesizing joins across data layers\n\n- [Ash.DataLayer.Ets] properly support multitenancy when referencing relationships\n\n- [Ash.Type.Union] initialize & validate each subtype of a union","ref":"changelog.html#bug-fixes-106"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Type.Enum] allow overriding `cast_stored/2` and `dump_to_native/2`\n\n- [Ash.DataLayer.Simple] support offset in the simple data layer\n\n- [Ash.Changeset] allow after_action hooks in fully atomic changesets\n\n## [v3.0.16](https://github.com/ash-project/ash/compare/v3.0.15...v3.0.16) (2024-06-21)","ref":"changelog.html#improvements-74"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk updates] use the proper opts when calling manual updates in bulk updates\n\n- [pagination] apply pagination at runtime for non lateral join queries\n\n- [multitenancy] consider multitenancy when checking if through-join is unique\n\n- [Ash.Changeset] don't run any `before_action` hooks if changeset is invalidated in prior hook\n\n- [atomic upgrade] only prevent atomic upgrade when hooks were explicitly added","ref":"changelog.html#bug-fixes-107"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Error] retain error context on overridden messages\n\n## [v3.0.15](https://github.com/ash-project/ash/compare/v3.0.14...v3.0.15) (2024-06-18)","ref":"changelog.html#improvements-75"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Type] add optional `matches_type?/2` callback to `Ash.Type`\n\n- [Ash.Domain] add `backwards_compatible_interface?` option to `use Ash.Domain`\n\n## [v3.0.14](https://github.com/ash-project/ash/compare/v3.0.13...v3.0.14) (2024-06-18)","ref":"changelog.html#improvements-76"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [many-to-many relationships] apply join relationship filter when loading many_to_many relationships\n\n- [Ash.Query] ensure we honor any computed select changes when loading through attributes","ref":"changelog.html#bug-fixes-108"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Policy.Authorier] add `subject` and `context` keys to policy context\n\n## [v3.0.13](https://github.com/ash-project/ash/compare/v3.0.12...v3.0.13) (2024-06-17)","ref":"changelog.html#improvements-77"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [parallelism] don't start async limiter tasks if async is disabled\n\n- [Ash.Domain] properly set default timeout to `:infinity`\n\n- [upserts] pass down `identity` when doing upserts, for new feature support\n\n- [Ash.Changeset] ensure that `before_transaction` hook errors fail the operation\n\n- [Ash.Changeset] ensure that `before_transaction` hook errors still trigger `after_transaction` hooks\n\n- [bulk updates] abort bulk updates on before transaction hook errors\n\n## [v3.0.12](https://github.com/ash-project/ash/compare/v3.0.11...v3.0.12) (2024-06-14)","ref":"changelog.html#bug-fixes-109"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [atomic updates] fix expression interpolation for cast_atomic for integer, decimal, float\n\n- [generic actions] set default argument values on generic actions\n\n- [generic actions] support `^arg/1` and similar constructions in filter policies on generic actions","ref":"changelog.html#bug-fixes-110"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Resource] set a `module` when validating accepts\n\n## [v3.0.11](https://github.com/ash-project/ash/compare/v3.0.10...v3.0.11) (2024-06-11)","ref":"changelog.html#improvements-78"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [loading through attributes] only apply load through for attributes that are being selected directly\n\n- [relationship loading] ensure we lateral join with `from_many?: true` or any `:many` cardinality relationships\n\n- [create/update/destroy actions] correctly load paginated relationships after create, update, delete (#1229)\n\n- [bulk create/update/destroy] load relationships on bulk operations (#1234)\n\n- [Ash.Type.Atom] return proper `{:ok, value}` from `Ash.Type.Atom.apply_constraints/2`\n\n- [Ash.Filter] fix the compare/2 implementations (#1232)\n\n- [Ash.Filter] return proper value from short-circuit filter hydration\n\n- [Ash.Seed] fix seed not working when :**keep_nil** is generated using seed_input (#1228)\n\n- [Ash.Generator] pass resource to Ash.Seed.seed! in Ash.Generator.seed! (#1227)","ref":"changelog.html#bug-fixes-111"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Resource] validate `require_attributes` (error) and `allow_nil_input` (warning) at compile time\n\n- [Ash.Seed] add tenant option to Ash.Seed.seed! (#1230)\n\n## [v3.0.10](https://github.com/ash-project/ash/compare/v3.0.9...v3.0.10) (2024-06-06)","ref":"changelog.html#improvements-79"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Union] ensure that union types w/ explicit tags have constraints applied\n\n- [multitenancy] don't update tenant on update, instead enforce it\n\n- [compare/2 validation] Do not compare nil values in `compare` validation (#1223)\n\n- [bulk actions] ensure context is properly set on bulk manual action invocations","ref":"changelog.html#bug-fixes-112"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Resource] detect invalid resources placed in relationships on domains verifier\n\n- [Ash.Resource] warn at compile time on types that don't define `atomic_update/2`\n\n## [v3.0.9](https://github.com/ash-project/ash/compare/v3.0.8...v3.0.9) (2024-05-31)","ref":"changelog.html#improvements-80"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Filter] use correct boolean operation names in Filter.find/4 (#1214)\n\n- [aggregates] when hydrating nested aggregates, use correct related resource/path pair\n\n- [aggregates] retain `ref_path` when authorizing aggregates\n\n- [relationship loading] ensure that belongs_to relationships are properly not reloaded with `lazy?: true`\n\n- [bulk actions] implement rollback on after hooks for bulk actions\n\n- [bulk actions] check if in transaction before trying to roll it back","ref":"changelog.html#bug-fixes-113"},{"type":"extras","title":"Improvements: - Changelog","doc":"- compatibility with elixir 1.17\n\n## [v3.0.8](https://github.com/ash-project/ash/compare/v3.0.7...v3.0.8) (2024-05-28)","ref":"changelog.html#improvements-81"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk updates] missing `else` caused manual updates not to work with `bulk_update`\n\n- [Ash.Policy.Authorizer] properly compose multiple filter-checks in policy conditions\n\n- [Ash.Policy.Authorizer] properly honor trailing policies that are constantly false","ref":"changelog.html#bug-fixes-114"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Notifier] verify notifiers all use the `Ash.Notifier` behaviour\n\n## [v3.0.7](https://github.com/ash-project/ash/compare/v3.0.6...v3.0.7) (2024-05-24)","ref":"changelog.html#improvements-82"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [identities] support `nils_distinct?` on identities\n\n- [identities] support `where` option on `identities`\n\n- [identities] allow calculations in identity keys\n\n## [v3.0.6](https://github.com/ash-project/ash/compare/v3.0.5...v3.0.6) (2024-05-23)","ref":"changelog.html#improvements-83"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [policies] don't raise an error when authorizing against previous values without atomic upgrades\n\n- [calculations] handle subquery-requiring calculations in `calculate/2`, ensuring we have a primary key\n\n- [Ash.DataLayer] resolve mixup between atomic upsert vs update capability of datalayers (#1198)\n\n## [v3.0.5](https://github.com/ash-project/ash/compare/v3.0.4...v3.0.5) (2024-05-23)","ref":"changelog.html#bug-fixes-115"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [mix ash.*] only use `Mix.deps_tree` if defined (its only defined for elixir 1.15+)","ref":"changelog.html#bug-fixes-116"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [atomic upgrade] add `atomic_upgrade?` flag to update/destroy actions\n\n- [atomic upgrade] do not do atomic upgrade by default unless `require_atomic?` is `true`\n\n- [atomic upgrade] allow configuring the read action used by atomic upgrades\n\n## [v3.0.4](https://github.com/ash-project/ash/compare/v3.0.3...v3.0.4) (2024-05-22)","ref":"changelog.html#improvements-84"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [bulk update/destroy] ensure that all notifications are sent\n\n## [3.0.3](https://github.com/ash-project/ash/compare/v3.0.2...3.0.3) (2024-05-22)","ref":"changelog.html#bug-fixes-117"},{"type":"extras","title":"Features: - Changelog","doc":"- [relationship pagination] allow retrieving the count of paginated relationships (#1183)\n\n- [Ash.Reactor] Add `bulk_create` step type.\n- [Ash.Reactor] Add `bulk_update` step type. (#1185)","ref":"changelog.html#features-7"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Actions.Read] properly hydrate and scope sorts with query context\n\n- [Ash.Changeset] handle list of atomic conditions coming from atomic validation implementation (#1194) (#1195)\n\n- [embedded resources] handle nil value for old_values when casting arrays (#1191)\n\n- [Ash.Query] use `Ash.Sort.parse_input/3` in `Ash.Query.sort_input/2`\n\n- [Ash.Resource.Validation.Changing] works correctly in atomics, and can eagerly detect changing (#1178)\n\n- [atomic updates] check the where condition before checking validation atomicity (#1177)\n\n- [bulk actions] don't emit after batch notifications if `notify?: false`\n\n- [Ash.Resource] prefer resource domain over option domain (#1176)\n\n- [bulk update/destroy] don't require domain for empty stream bulk update and destroy (#1175)\n\n- [Ash.Generator] only return valid non nil values items from generator (#1121)\n\n- [bulk destroy] properly validate action when calling bulk destroy\n\n- [code interface] allow all strategies for bulk actions in code interfaces by default\n- [code interfaces] honor `get?` for bulk update/bulk destroy","ref":"changelog.html#bug-fixes-118"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Query] support anonymous aggregates and calculations in sorts\n\n- [sensitive fields] Implement `show_sensitive?` config (#1180)\n\n- [Ash.Query] support `filter_input` and `sort_input` in `Ash.Query.build/2`\n\n- [Ash.Changeset] add `template_requires_actor` check for changesets\n\n- [bulk update/destroy] don't use queries for streaming if they have hooks\n\n- [Ash.Policy.Check.ChangingAttributes] consider `from: nil` in `changing_attributes/1` check\n\n## [v3.0.2](https://github.com/ash-project/ash/compare/v3.0.1...v3.0.2) (2024-05-15)","ref":"changelog.html#improvements-85"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Expr] add pattern matching for clarity on values accepted by `ref/1` and `ref/2`\n\n- [Ash.Expr] add `can_return_nil?/1` callback to Ash expressions, allowing for various optimizations\n\n- [Ash.Type.NewType] raise argument error on unknown options in `Ash.Type.NewType`. Helps with typos & misunderstandings\n\n- [embedded resources] use the `source` configuration for attributes in embedded resources (it was previously just ignored)\n\n- [Ash.Policy.Authorizer] better type specification for checks, to get better autocomplete and compile time validation\n\n- [Ash.Error.Invalid.NoSuchInput] added a `did_you_mean` field and used it in the error message","ref":"changelog.html#improvements-86"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Resource] properly persist simple_notifiers (they were being ignored before)\n\n- [code interface] accept single ids in code interface as subject for destroy/update\n\n- [bulk update] ensure that the `changed?` context is set in after action hooks on batches\n\n- [relationships] allow for inferred domains when authorizing join queries\n\n- [Ash.Expr] don't treat `nil` as not a valid value when type casting lists\n\n- [atomic upgrade] keep data's metadata in atomic upgraded update (#1165)\n\n## [v3.0.1](https://github.com/ash-project/ash/compare/v3.0.0...v3.0.1) (2024-05-14)","ref":"changelog.html#bug-fixes-119"},{"type":"extras","title":"Features: - Changelog","doc":"- [Ash.Resource.Change.Builtins] Add `cascade_destroy` to builtin changes.","ref":"changelog.html#features-8"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [calculations] calculation eager evaluation bug caused `exists` to eager evaluate when we didn't actually have the related data\n\n- [field policies] fix field policy rewrite errors on non-success cases (#1163)\n\n- [embedded resources] fix embedded resource authorization (#1159) (#1160)\n\n- infinite recursion if query is empty (#1158)\n\n- [Ash.DataLayer.Ets] ensure that changeset filters are honored in ETS destroy/update_query\n\n- [update/destroy actions] don't rollback transactions on stale records, ignore stale records in bulk actions\n\n- [bulk creates] don't check required belongs to fields until after setting them in bulk creation\n\n- [code interface] check require_reference? when generating update code interface (#1152)\n\n## [v3.0.0](https://github.com/ash-project/ash/compare/v3.0.0...2.0)","ref":"changelog.html#bug-fixes-120"},{"type":"extras","title":"3.0 - Changelog","doc":"We are starting the changelog fresh. See `documentation/2.0-CHANGELOG.md` in GitHub for the old changelogs.","ref":"changelog.html#3-0"},{"type":"extras","title":"Breaking Changes: - Changelog","doc":"For a guide on adjusting to these breaking changes, see the [upgrade guide](/documentation/topics/development/upgrading-to-3.0.md)\n\n- [Ash.Api] has been renamed to `Ash.Domain`, and references to the concept have been renamed as well, i.e in options and in the DSL\n- [Ash] we now call functions on this, instead of the domain. i.e `Ash.create` and `Ash.read`. The generated functions are now marked as deprecated\n- [Ash] remove process context functionality. You can no longer store the actor/tenant in the context with `Ash.set_actor` and so on\n- [private?] deprecate `private?: false` in favor of the more explicit `public?: true`\n- [default_accept] default `default_accept` is now `[]`\n- [action lifecycle] after transaction hooks cannot be added from inside of other lifecycle hooks\n- [Ash.NotLoaded] use `%Ash.NotLoaded{}` for unselected values, instead of `nil`\n- [require_atomic?] now defaults to `true`, requiring opt-out of atomic behavior\n- [authorization] default `api.authorization.authorize` to `:by_default`\n- [Ash.Registry] has been removed\n- [actions] `domain` must always be known when constructing changesets\n- [Ash.Notifier] `requires_original_data?/2` callback defaults to `false`\n- [Ash.Notifier.PubSub] default to `previous_values?: false`, allowing notifications to be sent for atomic updates\n- [unknown inputs] all action invocations now use `UnknownInput` errors when given an input they don't accept\n- [policies] `requires_original_data?/2` callback on checks defaults to `false`\n- [Ash.Calculation] has been renamed to `Ash.Resource.Calculation`\n- [Ash.Resource.Calculation] \"strict mode\" has been added and defaults to `true`. This causes only explicitly requested fields from relationships to be loaded\n- [Ash.Query.Calculation] positional arguments are now an options list\n- [calculations] anonymous function calculations in a resource now take lists and return lists, instead of a single record (like standard calculations do)\n- [context] The context argument passed to many different callbacks is now a struct, tailored to that specific context. For example, in a calculation you will receive an `Ash.Resource.Calculation.Context`\n- [after_action/before_action] These builtin changes now accept a 3rd context argument\n- [picosat_elixir] is now optional (`simple_sat` is now an alternative)\n- [Ash.Changeset] `Ash.Changeset.new!` has been removed\n- [Ash.Changeset] `Ash.Changeset.new/2` has been removed (`Ash.Changeset.new/1` is still available)\n- [Ash.Changeset] `changeset.filters` is now `changeset.filter`\n- [Ash.Changeset] reverse order of before action & before transaction hooks. They now run in the action they are added. They used to run in reverse order.\n- [Ash.CiString] `Ash.CiString.new/1` returns `nil` on `nil` input\n- [belongs_to.attribute_writable?] add `attribute_public?` for controlling publicity, and default `attribute_writable?` to `true`.\n- [Ash.Filter.TemplateHelpers] removed, all functions needed for expressions are now defined in `Ash.Expr`\n- [Ash.Expr] keyword lists are no longer special cased in ash expressions, and requiring pinning like any other value.\n- [Ash.Resource] default read actions are now paginatable with keyset and offset pagination (but pagination is not required)\n- [Ash.Resource] default actions require explicit accept lists (or will use `default_accept`). i.e `defaults [:read, create: [:first_name, :last_name]]`\n- [Ash.Resource] `simple_notifiers` is now an option to `use Ash.Resource`, instead of being in the DSL at `resource.simple_notifiers`\n- [Ash.Flow] has been removed and put in its own package `ash_flow`. It is being deprecated in favor of `Reactor`\n- [Ash.Error] the implementation has been extracted out to `Splode`. Defining new `Ash.Error`s is now done by defining a new `Splode.Error`\n- [Ash.Query] swap position of sort order and arguments in calculation sorting, i.e instead of `calculation: {:asc, %{...args}}` it is now `calculation: {%{...args}, :asc}`\n- [Ash.Resource.Aggregate] add `include_nil?` aggregate option, and default it to `false` (so `list` and `first` aggregates do not consider `nil` values by default)\n- [Ash.Policy.FilterCheck] now accepts `context` arguments, like `Ash.Policy.FilterCheckWithContext`\n- [Ash.Policy.FilterCheckWithContext] has been removed, use `Ash.Policy.FilterCheck`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features: - Changelog","doc":"- [Ash.Type] add new remove_nil_items? array type constraint (#1116)\n- [Ash.Query] Paginatable relationships (#1050)\n- [Ash.DataLayer] new `calculate/3` callback that allows for data layers to compute the result of expressions outside the context of a query. Used to power `Ash.calculate/3`.\n- [validations] new builtin validations, `attributes_present/2` and `attributes_absent/2`\n- [multitenancy] configurable multitenancy behaviour on read actions (#1030)\n- [Ash.Reactor] Add new `change` step type which can be used to modify changesets.\n- [Ash.Changeset] add `Ash.Changeset.update_change/2` function and builtin change (#976)\n- [Ash.Domain] code interfaces can now be defined on the domain\n- [Ash.Domain] policies can now be defined on the domain, and will run before resource policies\n- [Ash.ToTenant] add `Ash.ToTenant`, allowing for passing arbitrary values as tenants\n- [Ash] add `Ash.read_first` (like `Ash.read_one`, but applies a limit automatically)\n- [Ash] support a second optional `input` option for `create`, `update` and `destroy`, allowing for things like `Ash.create!(Post, %{text: \"text\"}, opts)`\n- [sensitive?] support `sensitive?` option in query aggregate/calculation (#963)\n- [Ash.Resource] support `require_reference?: false` on code interfaces, for when an update or destroy action uniquely identifies a record (or for bulk update/destroy)\n- [Ash.Resource] notifiers can now be specified for specific actions, using the `notifiers` option\n- [mix ash.rollback] delegates to extensions to trigger their rollback tasks\n- [Ash.Query] add `Ash.Query.apply_to/3`, to \"apply\" the query to a set of records (i.e filter, sort, distinct, etc.)\n- [Ash.CustomExpression] Use `Ash.CustomExpression` to extend Ash's expression syntax in a data-layer agnostic way\n- [code interface] Code interface functions now support bulk actions, in a \"do what I mean\" way. For example: `Domain.deactive(post)` can also be `Post |> Ash.Query.filter(active == true) |> Domain.deactive()`","ref":"changelog.html#features-9"},{"type":"extras","title":"Improvements: - Changelog","doc":"- [Ash.Actions.Sort] allow providing a stream of records to sort, and performance improvements\n- [bulk actions] add `read_action` option to bulk actions (#1088)\n- [Ash.stream] support streaming with offset, or even _no_ pagination\n- [Ash.DataLayer.Ets] add debug logging, similar to ecto query debug logging\n- [Ash.DataLayer.Ets] support update_query, destroy_query and `Ash.Changeset.filter/2`\n- [Embedded resources] don't add `autogenerated_id` to embeds if they don't have a primary key\n- [Ash.Resource] you can now omit the return type of generic actions, indicating it either succeeds or fails, returning `:ok` or `{:error, error}`\n- [Ash.Resource] Generic actions can now accept a Reactor module, running it directly. (#993)\n- [Ash.Resource] support `allow_nil_input` dsl option in update/destroy actions (#964)\n- [Ash.Resource] The `filter` option can now be supplied multiple times in read actions and in relationships. They will be combined with `and`\n- [Ash.Resource] private attributes can now be accepted as action inputs\n- [Ash.Expr] is now imported automatically into places you will likely use it, like changes, validations, checks and calculations.\n- [Ash.Query] is now required automatically in places you will likely use it, as above\n- [sortable?] fields may mark themselves as unusable in sorts by using `sortable? false`\n- [sensitive?] calculations and aggregates may now also be marked as `sensitive?`","ref":"changelog.html#improvements-87"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"- [Ash.Type] apply array type `nil_items?` constraint after item constraints are applied (#1115)\n- [Ash.DataLayer.Ets] fix ETS data layer's support for lateral joining\n- [bulk actions] ensure transaction is rolled back on data layer errors during streaming\n- [bulk actions] set `notify?: true` when `return_notifications?: true` is set\n- [Ash.Changeset] `attributes_present?/2` -> `attribute_present?/2`\n- [Ash.Filter] don't eager evaluate `type/3` because data layers require type information\n- [Ash.Changeset] when comparing identities for `manage_relationship`, we now properly cast the values. Before, `\"1\"` and `1` were not considered equal for integer primary keys/identity fields\n- Many more bug fixes were added, but few are relevant enough to list here","ref":"changelog.html#bug-fixes-121"},{"type":"extras","title":"Project Structure","doc":"# Project Structure\n\nIn this guide we'll discuss some best practices for how to structure your project. These recommendations align well with [Elixir conventions](https://hexdocs.pm/elixir/1.16.2/naming-conventions.html#casing) around file and module naming. These conventions allow for a logical coupling of module and file names, and help keep your project organized and easy to navigate.\n\n>","ref":"project-structure.html"},{"type":"extras","title":"These are recommendations - Project Structure","doc":">\n> None of the things we show you here are _requirements_, only recommendations.\n> Feel free to plot your own course here. Ash avoids any pattern that requires\n> you to name a  file or module in a specific way, or put them in a specific\n> place. This ensures that all connections between one module and another\n> module are _explicit_ rather than _implicit_.\n\nThese recommendations all correspond to standard practice in most Elixir/Phoenix applications\n\n```\nlib/\n├── my_app/                    # Your application's main namespace\n│   ├── accounts.ex            # Accounts domain module\n│   ├── helpdesk.ex            # Helpdesk domain module\n│   │\n│   ├── accounts/               # Accounts context\n│   │   ├── user.ex             # User resource\n│   │   ├── user/               # User resource files\n│   │   ├── token.ex            # Token resource\n│   │   └── password_helper.ex  # Support module\n│   │\n│   └── helpdesk/            # Helpdesk context\n│       ├── ticket.ex        # Ticket resource\n│       ├── notification.ex  # Notification resource\n│       ├── other_file.ex    # Support module\n│       └── ticket/          # Ticket resource files\n│           ├── preparations/\n│           ├── changes/\n│           └── checks/\n```\n\nPlace your Ash application in the standard Elixir application directory `lib/my_app`. Your `Ash.Domain` modules should be at the root level of this directory. Each domain should have a directory named after it, containing the domain's `Ash.Resource` modules and any of the domain's supporting modules. All resource interaction ultimately goes through a domain module.\n\nFor resources that require additional files, create a dedicated folder in the domain context named after the resource. We suggest organizing these supplementary files into subdirectories by type (like `changes/`, `preparations/`, etc.), though this organization is optional.\n\n# Where do I put X thing\n\nThe purpose of Ash is to be both the model of and the interface to your domain logic (A.K.A business logic). Applying this generally looks like building as much of your domain logic \"behind\" your resources. This does not mean, however, that everything has to go _inside of_ your resources. For example, if you have a `Purchase` resource, and you want to be able to display a list of purchases that were taxable, and also calculate the percentage of the purchase that was taxable. You might have an action called `:taxable` and a calculation called `:percentage_tax`.","ref":"project-structure.html#these-are-recommendations"},{"type":"extras","title":"Example 1: Reads & Calculations - Project Structure","doc":"```elixir\nactions do\n  ...\n\n  read :taxable do\n    filter expr(taxable == true)\n  end\nend\n\ncalculations do\n  calculate :percentage_tax, :decimal, expr(\n    sum(line_items, field: :amount, query: [filter: tax == true]) /\n    sum(line_items, field: :amount)\n  )\nend\n```\n\nIn practice, you may not need the `taxable` action, i.e perhaps you simply want a \"taxable\" checkbox on a list view in your application, in which case you may use the primary read, or some other read like `:transaction_report`. You would then, on the consumer, provide the filter for `taxable == true`, and load the `:percentage_tax` calculation.","ref":"project-structure.html#example-1-reads-calculations"},{"type":"extras","title":"Example 2: Using external data in create actions - Project Structure","doc":"Lets say you want the user to fill in a github issue id, and you will fetch information from that github issue to use as part of creating a \"ticket\" in your system.. You might be tempted to do something like this in a LiveView:\n\n```elixir\ndef handle_event(\"link_ticket\", %{\"issue_id\" => issue_id}, socket) do\n  issue_info = GithubApi.get_issue(issue_id)\n\n  MyApp.Support.update_ticket(socket.assigns.ticket_id, %{issue_info: %{\n    title: issue_info.title,\n    body: issue_info.body\n  }})\nend\n```\n\nBut this is putting business logic inside of your UI/representation layer. Instead, you should write an action and put this logic inside of it.\n\n```elixir\ndefmodule MyApp.Ticket.FetchIssueInfo do\n  use Ash.Resource.Change\n\n  def change(changeset, _, _) do\n    Ash.Changeset.before_transaction(changeset, fn changeset ->\n      issue_info = GithubApi.get_issue(changeset.arguments.issue_id)\n\n      Ash.Changeset.force_change_attributes(changeset, %{\n        issue_info: %{\n          title: issue_info.title,\n          body: issue_info.body\n        }\n      })\n    end)\n  end\nend\n```\n\nThen you'd have an action like this:\n\n```elixir\nupdate :link_ticket do\n  argument :issue_id, :string, allow_nil?: false\n\n  change MyApp.Ticket.FetchIssueInfo\nend\n```\n\nThis cleanly encapsulates the operation behind the resource, even while the code for fetching the github issue still lives in a `GitHubApi` module.","ref":"project-structure.html#example-2-using-external-data-in-create-actions"},{"type":"extras","title":"Working with LLMs","doc":"# Working with LLMs\n\nLLMs are a new technology, and the patterns on how best to leverage them evolve every day. It is also *quite* debatable whether it is a good idea to use them at all. Nothing in Ash will ever be predicated on the usage of these tools, but we do want to provide at least some level of base guidance on what we think are the best practices for those that do. This is also to help those who are interested in trying these tools but don't yet know where to start.\n\nThis guide is about working with LLM dev assistants, not about building LLM-related features or integrating them into your application. For that, see [Ash AI](https://hexdocs.pm/ash_ai).\n\n>","ref":"working-with-llms.html"},{"type":"extras","title":"Getting Support with LLM generated code - Working with LLMs","doc":">\n> Please note that LLMs often hallucinate despite our best intentions. If you need help with something, and you come to our support channels, you *must* make it clear when the code you are asking for help with was generated by an LLM. You *must* first understand the code you've written yourself, and provide a detailed explanation of the code and the issue you are facing when requesting help. The discord and forums are not a place for others to debug LLM hallucinations.","ref":"working-with-llms.html#getting-support-with-llm-generated-code"},{"type":"extras","title":"What to know - Working with LLMs","doc":"To take advantage of LLMs, you will want to explore the following. You will have to make up your own mind on which avenues to explore and leverage in the following areas. This is essentially a \"big list of stuff you should research on your own\".\n\n- Language models - OpenAI, Anthropic, Gemini, etc. Choice of language model will make the most difference in your experience.\n- Agentic editors - Zed, Windsurf, Cursor\n- Agentic assistants - Claude Code, Aider, Codex\n- Rules files - The name of these files are often specific to the editor, but they essentially boil down to \"stuff to put in the system prompt\" to guide the LLM's behavior.","ref":"working-with-llms.html#what-to-know"},{"type":"extras","title":"Tools - Working with LLMs","doc":"We suggest setting up, where applicable, the following tools:\n\n- [Tidewave](https://tidewave.ai): Tidewave gives your LLMs the ability to interact with your running application. This can significantly improve the quality of the code generated by LLMs, and allows them to observe and interact with the running application, like reading logs and working with processes.\n- [Ash AI](https://hexdocs.pm/ash_ai): Ash AI contains tools for building AI enabled applications, but it also comes with a dev MCP server where we will experiment with tools similar to what tidewave offers, but tailored to Ash and the way we work.","ref":"working-with-llms.html#tools"},{"type":"extras","title":"Rules - Working with LLMs","doc":"We are working on establishing a pattern whereby packages can provide a `usage-rules.md` which you can then combine into your own rules file. The idea here is to democratize the process of building rules, allowing you to adopt well vetted and quality rules files from the maintainers of projects. This has only been done for a few packages so far.\n\nTo leverage these rules files, you can simply copy them yourself if you'd prefer something more manual, or you can use a new mix task provided by the [`usage_rules`](https://hexdocs.pm/usage_rules) package to combine them into your own rules file.","ref":"working-with-llms.html#rules"},{"type":"extras","title":"Combine all of your (direct) dependencies usage rules - Working with LLMs","doc":"```elixir\nmix usage_rules.sync .rules --all\n```","ref":"working-with-llms.html#combine-all-of-your-direct-dependencies-usage-rules"},{"type":"extras","title":"Pick specific dependencies - Working with LLMs","doc":"```elixir\nmix usage_rules.sync .rules \\\n  ash ash_postgres ash_phoenix ash_graphql ash_json_api ash_ai\n```\n\nYou can replace the `.rules` file with your own current rules file, and it will be appended to the contents. Repeated calls will only replace the package rules contents of the file, not the whole file contents.\n\nOnly dependencies of your current project will be added, and any dependencies that don't have rules are skipped.","ref":"working-with-llms.html#pick-specific-dependencies"},{"type":"extras","title":"Generators","doc":"# Generators\n\nAsh comes with multiple generators, packages as mix tasks, to help you generate and make modifications to your applications.\n\nSee the documentation for each mix task for more information. What is presented here is merely an overview.","ref":"generators.html"},{"type":"extras","title":"Installer - Generators","doc":"Ash can be installed into a project using `igniter`. Some examples of how this can work:\n\n- Install Ash & AshPostgres into your current project\n\n  ```\n  mix igniter.install ash ash_postgres\n  ```\n\n- Create a new mix project with Ash & AshPostgres installed\n\n  ```\n  mix archive.install hex igniter_new\n  mix igniter.new my_project --install ash,ash_postgres\n  ```\n\n- Create a new phoenix project with Ash & AshPostgres installed\n\n  ```\n  mix igniter.new my_project --install ash,ash_postgres,ash_phoenix --with phx.new\n  ```\n\n  >","ref":"generators.html#installer"},{"type":"extras","title":"install hex archives - Generators","doc":">\n  > The archives have to be installed to use them. This only needs to be done once, until you change elixir versions.\n  >\n  > ```elixir\n  > mix archive.install hex igniter_new\n  > mix archive.install hex phx_new\n  > ```","ref":"generators.html#install-hex-archives"},{"type":"extras","title":"Generators - Generators","doc":"- `mix ash.gen.resource` - Generates a new `Ash.Resource`.\n- `mix ash.gen.domain` - Generates a new `Ash.Domain`.\n- `mix ash.gen.enum` - Generates a new `Ash.Type.Enum`.\n- `mix ash.gen.base_resource` - Generates a new base resource.\n- `mix ash.gen.change` - Generates a new `Ash.Resource.Change`.\n- `mix ash.gen.validation` - Generates a new `Ash.Resource.Validation`.\n- `mix ash.gen.preparation` - Generates a new `Ash.Resource.Preparation`.\n- `mix ash.gen.custom_expression` - Generates a new `Ash.CustomExpression`.","ref":"generators.html#generators"},{"type":"extras","title":"Patchers - Generators","doc":"- `mix ash.extend` - Adds an extension or extensions to a domain or resource.","ref":"generators.html#patchers"},{"type":"extras","title":"Error Handling","doc":"# Error Handling\n\nAs of 3.0, Ash uses `Splode` to as our basis for errors. The documentation below still applies, but it is powered by `Splode` under the hood.\n\nThere is a difficult balance to cut between informative errors and enabling simple reactions to those errors. Since many extensions may need to work with and/or adapt their behavior based on errors coming from Ash, we need rich error messages. However, when you have a hundred different exceptions to represent the various kinds of errors a system can produce, it becomes difficult to say something like \"try this code, and if it is invalid, do x, if it is forbidden, do y. To this effect, exceptions in Ash have one of four classes mapping to the top level exceptions.","ref":"error-handling.html"},{"type":"extras","title":"Error Classes - Error Handling","doc":"- forbidden - `Ash.Error.Forbidden`\n- invalid - `Ash.Error.Invalid`\n- framework - `Ash.Error.Framework`\n- unknown - `Ash.Error.Unknown`\n\nSince many actions can be happening at once, we want to support the presence of multiple errors as a result of a request to Ash. We do this by grouping up the errors into one before returning or raising.\nWe choose an exception based on the order of the exceptions listed above. If there is a single forbidden, we choose `Ash.Error.Forbidden`, if there is a single invalid, we choose `Ash.Error.Invalid` and so on. The actual errors will be included in the `errors` key on the exception. The exception's message will contain a bulleted list of all the underlying exceptions that occurred. This makes it easy to react to specific kinds of errors, as well as to react to _any/all_ of the errors present.\n\nAn example of a single error being raised, representing multiple underlying errors:\n\n```elixir\nAshExample.Representative\n|> Ash.Changeset.for_create(:create, %{employee_id: \"the best\"})\n|> Ash.create!()\n ** (Ash.Error.Invalid) Invalid Error\n * employee_id: must be absent.\n * first_name, last_name: at least 1 must be present.\n```\n\nThis allows easy rescuing of the major error classes, as well as inspection of the underlying cases\n\n```elixir\ntry do\n  AshExample.Representative\n  |> Ash.Changeset.for_create(:create, %{employee_id: \"dabes\"})\n  |> Ash.create!()\nrescue\n  e in Ash.Error.Invalid ->\n    \"Encountered #{Enum.count(e.errors)} errors\"\nend\n\n\"Encountered 2 errors\"\n```\n\nThis pattern does add some additional overhead when you want to rescue specific kinds of errors. For example, you may need to do something like this:\n\n```elixir\ntry do\n  AshExample.Representative\n  |> Ash.Changeset.for_create(:create, %{employee_id: \"dabes\"})\n  |> Ash.create!()\nrescue\n  e in Ash.Error.Invalid ->\n    case Enum.find(e.errors, &(&1.__struct__ == A.Specific.Error)) do\n      nil ->\n        ...handle errors\n      error ->\n        ...handle specific error you found\n    end\nend\n```","ref":"error-handling.html#error-classes"},{"type":"extras","title":"Error Handlers - Error Handling","doc":"Create, update and destroy actions can be provided with an `error_handler`, which can be used to modify the errors\nbefore they are returned. This is not an error *recovery* mechanism, rather a way to control the shape of\nerrors that are returned. For more information on the callback itself, see `Ash.Changeset.handle_errors/2`.\n\nExample usage:\n\n```elixir\ncreate :upsert_article_by_slug do\n  upsert? true\n  accept [:slug, :title, :body]\n  upsert_identity :unique_slug\n  upsert_condition expr(user_id == ^actor(:id))\n  error_handler fn \n    _changeset, %Ash.Error.Changes.StaleRecord{} ->\n      Ash.Error.Changes.InvalidChanges.exception(field: :slug, message: \"has already been taken\")\n\n    _ changeset, other ->\n      # leave other errors untouched\n      other\n  end\nend\n```","ref":"error-handling.html#error-handlers"},{"type":"extras","title":"Generating Errors - Error Handling","doc":"When returning errors from behaviors or adding errors to a\nchangeset/query/action input, multiple formats are supported. You can return a\nsimple String, which will be converted into an `Ash.Error.Unknown` exception.\nYou can also return a keyword list containing `field` or `fields` and `message`,\nwhich will be used to construct an `Ash.Error.Invalid.InvalidChanges` error.\nFinally, you can pass an exception directly, which will be used as is if it is\nan Ash error, or wrapped in an `Ash.Error.Unknown` if it is not.\n\nTechnically *any* value can be used as an error, but will be wrapped in an\n`Ash.Error.Unknown` accordingly.\n\n>","ref":"error-handling.html#generating-errors"},{"type":"extras","title":"Use exception modules - Error Handling","doc":">\n> You should prefer to use the exception modules provided by Ash, or ones\n> that you have defined manually. This allows you to document your error\n> types, and to show those errors over API interfaces. See the section\n> on APIs below for more.","ref":"error-handling.html#use-exception-modules"},{"type":"extras","title":"Examples of using non standard errors - Error Handling","doc":"","ref":"error-handling.html#examples-of-using-non-standard-errors"},{"type":"extras","title":"Keyword list (Ash.Error.Changes.InvalidChanges) - Error Handling","doc":"```elixir\ndef change(changeset, _, _) do\n  if under_21?(changeset) do\n    Ash.Changeset.add_error(changeset, field: :age, message: \"must be 21 or older\")\n  else\n    changeset\n  end\nend\n```","ref":"error-handling.html#keyword-list-ash-error-changes-invalidchanges"},{"type":"extras","title":"String (Ash.Error.Unknown.UnknownError) - Error Handling","doc":"```elixir\ndef change(changeset, _, _) do\n  if under_21?(changeset) do\n    Ash.Changeset.add_error(changeset, \"must be 21 or older\")\n  else\n    changeset\n  end\nend\n```","ref":"error-handling.html#string-ash-error-unknown-unknownerror"},{"type":"extras","title":"Using an exception module - Error Handling","doc":"These are all modules under `Ash.Error.*`. You can create a new one with `error.exception(options)`, and the options are documented in each exception. This documentation is missing in some cases. Go to the source code of the exception to see its special options. All of them support the `vars` option, which are values to be interpolated into the message, useful for things like translation.\n\nFor example:\n\n```elixir\ndef change(changeset, _, _) do\n  if under_21?(changeset) do\n    error = Ash.Error.Changes.Required.exception(\n      field: :foo,\n      type: :attribute,\n      resource: changeset.resource\n    )\n\n    Ash.Changeset.add_error(changeset, error)\n  else\n    changeset\n  end\nend\n```","ref":"error-handling.html#using-an-exception-module"},{"type":"extras","title":"Using a Custom Exception - Error Handling","doc":"You can create a custom exception like so. This is an example of a builtin exception that you could mirror to build your own\n\n```elixir\ndefmodule MyApp.Errors.Invalid.TooYoung do\n  @moduledoc \"Used when a user who is too young is attempted to be created\"\n  use Splode.Error, fields: [:age], class: :invalid\n\n  def message(error) do\n    \"\"\"\n    Must be 21 or older, got: #{error.age}.\n    \"\"\"\n  end\nend\n\ndef change(changeset, _, _) do\n  if under_21?(changeset) do\n    error = MyApp.Errors.Invalid.TooYoung.exception(\n      age: Ash.Changeset.get_attribute(changeset, :age)\n    )\n\n    Ash.Changeset.add_error(changeset, error)\n  else\n    changeset\n  end\nend\n```","ref":"error-handling.html#using-a-custom-exception"},{"type":"extras","title":"Showing errors over APIs - Error Handling","doc":"AshJsonApi and AshGraphql both use a special protocol to determine how (and if) a raised or returned error should be displayed.\n\nSee the relevant docs:\n- [handling errors in AshGraphql](https://hexdocs.pm/ash_graphql/handle-errors.html)\n- [AshJsonApi.ToJsonApiError](https://hexdocs.pm/ash_json_api/AshJsonApi.ToJsonApiError.html)","ref":"error-handling.html#showing-errors-over-apis"},{"type":"extras","title":"Testing","doc":"# Testing\n\nTake a look at [the how-to guide](/documentation/how-to/test-resources.livemd) for a practical look at writing tests\n\nThe configuration you likely want to add to your `config/test.exs` is:\n\n```elixir\n# config/test.exs\nconfig :ash, :disable_async?, true\nconfig :ash, :missed_notifications, :ignore\n```\n\nEach option is explained in more detail below.","ref":"testing.html"},{"type":"extras","title":"Async tests - Testing","doc":"The first thing you will likely want to do, especially if you are using `AshPostgres`, is to add the following config to your `config/test.exs`.\n\n```elixir\n# config/test.exs\nconfig :ash, :disable_async?, true\n```\n\nThis ensures that Ash does not spawn tasks when executing your requests, which is necessary for doing transactional tests with `AshPostgres`.","ref":"testing.html#async-tests"},{"type":"extras","title":"Missed notifications - Testing","doc":"If you are using Ecto's transactional features to ensure that your tests all run in a transaction, Ash will detect that it had notifications to send (if you have any notifiers set up) but couldn't because it was still in a transaction. The default behavior when notifications are missed is to warn. However, this can get pretty noisy in tests. So we suggest adding the following config to your `config/test.exs`.\n\n```elixir\n# config/test.exs\nconfig :ash, :missed_notifications, :ignore\n```","ref":"testing.html#missed-notifications"},{"type":"extras","title":"Development Utilities","doc":"# Development Utilities","ref":"development-utilities.html"},{"type":"extras","title":"Formatting DSLs - Development Utilities","doc":"All Ash packages that ship with extensions provide exports in their `.formatter.exs`. This prevents the formatter from turning, for example, `attribute :name, :string` into `attribute(:name, :string)`. To enable this, add `:ash` (and any other Ash libraries you are using) to your `.formatter.exs` file:\n\n```elixir\n[\n  # ...\n  import_deps: [..., :ash],\n  # ...\n]\n```","ref":"development-utilities.html#formatting-dsls"},{"type":"extras","title":"ElixirSense Plugin - Development Utilities","doc":"Ash uses [Spark](https://hexdocs.pm/spark) to build all of our DSLs (like `Ash.Resource` and `Ash.Domain`) and to validate options lists to functions. `Spark` ships with an extension that is automatically picked up by ElixirLS to provide autocomplete for all of our DSLs, and options list. You don't need to do anything to enable this, but it only works with ElixirLS (not other language server tools).","ref":"development-utilities.html#elixirsense-plugin"},{"type":"extras","title":"Formatter plugin - Development Utilities","doc":"`Spark` also ships with a formatter plugin that can help you keep your resources formatted consistently. This plugin can sort the sections of your DSL to make your resources more consistent, and it can remove any accidentally added parentheses around DSL code.","ref":"development-utilities.html#formatter-plugin"},{"type":"extras","title":"Adding the plugin - Development Utilities","doc":"Add the following to your `.formatter.exs`\n\n```elixir\n[\n  plugins: [Spark.Formatter], # <- add the plugin here\n  inputs: ...\n]\n```","ref":"development-utilities.html#adding-the-plugin"},{"type":"extras","title":"Configuration - Development Utilities","doc":"#### Minimal config for your Ash Resources\n\n```elixir\nconfig :spark, :formatter,\n  remove_parens?: true,\n  \"Ash.Domain\": [],\n  \"Ash.Resource\": [\n    section_order: [\n      # any section not in this list is left where it is\n      # but these sections will always appear in this order in a resource\n      :actions,\n      :attributes,\n      :relationships,\n      :identities\n    ]\n  ]\n```\n\n#### If you `use` a different module than Ash.Resource\n\n```elixir\nconfig :spark, :formatter,\n  [\n    \"Ash.Resource\": [\n      section_order: [\n        :resource,\n        :identities,\n        :attributes,\n        :relationships,\n        ...\n      ]\n    ],\n    # If you use a different module than Ash.Resource\n    \"MyApp.Resource\": [\n      type: Ash.Resource,\n      # What extensions might be added by your base module\n      extensions: [...],\n      section_order: [\n        :resource,\n        :identities,\n        :attributes,\n        :relationships,\n        ...\n      ]\n    ]\n  ]\n```","ref":"development-utilities.html#configuration"},{"type":"extras","title":"Backwards Compatibility Config","doc":"# Backwards Compatibility Config\n\nAll of these configurations are potentially breaking changes when applied\nto your application. However, we *highly* encourage setting as many of\nthem as possible. In 4.0, some will be removed entirely, and any that remain\nwill have their defaults changed to the new value.\n\nThe ash installer automatically sets all of these.","ref":"backwards-compatibility-config.html"},{"type":"extras","title":"allow_forbidden_field_for_relationships_by_default? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, allow_forbidden_field_for_relationships_by_default?: true\n```","ref":"backwards-compatibility-config.html#allow_forbidden_field_for_relationships_by_default"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"Loaded relationships that produced a `Forbidden` error would fail the entire\nrequest. i.e in `Ash.load(post, [:comments, :author])`, if `author` returned\na `Forbidden` error, the entire request would fail with a forbidden error.","ref":"backwards-compatibility-config.html#old-behavior"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Now the relationships that produced a forbidden error are instead populated\nwith `%Ash.ForbiddenField{}`.","ref":"backwards-compatibility-config.html#new-behavior"},{"type":"extras","title":"include_embedded_source_by_default? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, include_embedded_source_by_default?: false\n```","ref":"backwards-compatibility-config.html#include_embedded_source_by_default"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"When working with embedded types, the `__source__` constraint is populated with\nthe original changeset. This can be very costly in terms of memory when working with \nlarge sets of embedded resources.","ref":"backwards-compatibility-config.html#old-behavior-1"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Now, the source is only included when you say `constraints: [include_source?: true]` on\nthe embedded resource's usage.","ref":"backwards-compatibility-config.html#new-behavior-1"},{"type":"extras","title":"show_keysets_for_all_actions? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, show_keysets_for_all_actions?: false\n```","ref":"backwards-compatibility-config.html#show_keysets_for_all_actions"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"For all actions, the records would be returned with `__metadata__.keyset` populated\nwith a keyset computed for the `sort` that was used to produce those records. This\nis expensive as it requires loading all things that are used by the sort.","ref":"backwards-compatibility-config.html#old-behavior-2"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Only when actually performing keyset pagination will the `__metadata__.keyset` be\ncomputed.","ref":"backwards-compatibility-config.html#new-behavior-2"},{"type":"extras","title":"default_page_type - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, default_page_type: :keyset\n```","ref":"backwards-compatibility-config.html#default_page_type"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"When an action supports `offset` and `keyset` pagination, and a page is requested\nwith only `limit` set, i.e `page: [limit: 10]`, you would get back an `%Ash.Page.Offset{}`.","ref":"backwards-compatibility-config.html#old-behavior-3"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Now we will return a `%Ash.Page.Keyset{}` choosing it whenever it is ambiguous.\nYou can always force returning an `%Ash.Page.Offset{}` by providing the offset option,\ni.e `page: [offset: 0]`","ref":"backwards-compatibility-config.html#new-behavior-3"},{"type":"extras","title":"policies.no_filter_static_forbidden_reads? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, policies: [no_filter_static_forbidden_reads?: false]\n```","ref":"backwards-compatibility-config.html#policies-no_filter_static_forbidden_reads"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"On read action policies, we can often tell statically that they cannot pass, for example:\n\n```elixir\npolicy action_type(:read) do\n  authorize_if actor_attribute_equals(:active, true)\nend\n```\n\nIn these cases, you would get an `Ash.Error.Forbidden`, despite the fact that the\ndefault `access_type` for a policy is `:filter`. If you instead had:\n\n```elixir\npolicy action_type(:read) do\n  authorize_if expr(private == false)\nend\n```\n\nYou would get a filter. This made it difficult to predict when you would get a forbidden\nerror and when the query results would  be filtered.","ref":"backwards-compatibility-config.html#old-behavior-4"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Now, we always filter the query even if we know statically that the request would be\nforbidden. For example the following policy:\n\n```elixir\npolicy action_type(:read) do\n  authorize_if actor_attribute_equals(:active, true)\nend\n```\n\nwould yield `filter: false`. This makes the behavior consistent and predictable.\nYou can always annotate that a given policy should result in a forbidden error\nby setting `access_type :strict` in the policy.","ref":"backwards-compatibility-config.html#new-behavior-4"},{"type":"extras","title":"keep_read_action_loads_when_loading? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, keep_read_action_loads_when_loading?: false\n```","ref":"backwards-compatibility-config.html#keep_read_action_loads_when_loading"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"If you had an action with a preparation, or a global preparation that loaded data, i.e\n\n```elixir\nprepare build(load: :comments)\n```\n\nthis wold be applied when using `Ash.load`, because we build a query for the primary\nread action as a basis for loading data. This could be expensive because now you are always\nloading `:comments` even if you only intended to load something else, and could also be\nunpredictable because it could \"overwrite\" the already loaded `comments` on the data you\npassed in.","ref":"backwards-compatibility-config.html#old-behavior-5"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"When using `Ash.load` *only* the explicitly provided load statement is applied.","ref":"backwards-compatibility-config.html#new-behavior-5"},{"type":"extras","title":"default_actions_require_atomic? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, default_actions_require_atomic?: true\n```","ref":"backwards-compatibility-config.html#default_actions_require_atomic"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"When building actions like so: `defaults [:read, create: :*, update: :*]` the default\naction is generated with `require_atomic? false`. This could make it difficult to spot\nactions that cannot safely be done asynchronously.","ref":"backwards-compatibility-config.html#old-behavior-6"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"The default generated actions are generated with `require_atomic? true`","ref":"backwards-compatibility-config.html#new-behavior-6"},{"type":"extras","title":"read_action_after_action_hooks_in_order? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, read_action_after_action_hooks_in_order?: true\n```","ref":"backwards-compatibility-config.html#read_action_after_action_hooks_in_order"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"In 3.0, we modified hooks on changesets to always be added in order instead of in\nreverse order. This was missed for `Ash.Query`. Meaning if you had something like this:\n\n```elixir\nread :read do\n  prepare fn query, _ -> \n    Ash.Query.after_action(query, fn query, results -> \n      IO.puts(\"hook 1\")\n      {:ok, results}\n    end)\n  end\n\n  prepare fn query, _ -> \n    Ash.Query.after_action(query, fn query, results -> \n      IO.puts(\"hook 2\")\n      {:ok, results}\n    end)\n  end\nend\n```\n\nrunning that action would print `hook 2` before `hook 1`.","ref":"backwards-compatibility-config.html#old-behavior-7"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Read action hooks are now run in the order they were added","ref":"backwards-compatibility-config.html#new-behavior-7"},{"type":"extras","title":"bulk_actions_default_to_errors? - Backwards Compatibility Config","doc":"```elixir\nconfig :ash, bulk_actions_default_to_errors?: true\n```","ref":"backwards-compatibility-config.html#bulk_actions_default_to_errors"},{"type":"extras","title":"Old Behavior - Backwards Compatibility Config","doc":"Bulk action options defaulted to `return_errors?: false`, and `stop_on_error?: false`,\nwhich was often a footgun for users unfamiliar to bulk actions, wondering \"why did I not\nget an error even though nothing was created?\"","ref":"backwards-compatibility-config.html#old-behavior-8"},{"type":"extras","title":"New Behavior - Backwards Compatibility Config","doc":"Now, `return_errors?` and `stop_on_error?` default to `true`","ref":"backwards-compatibility-config.html#new-behavior-8"},{"type":"extras","title":"Upgrade","doc":"# Upgrade","ref":"upgrading-to-3-0.html"},{"type":"extras","title":"Other Packages - Upgrade","doc":"Other packages have had a major version bump in addition to Ash core. While _all packages_ have been changed to refer to `domain` instead of `api`, they did not receive a major version bump because there were no special breaking changes to account for when using that package. You will also need to factor in the following upgrade guides, if you use those packages.\n\n- [AshPostgres](https://hexdocs.pm/ash_postgres/upgrading-to-2-0.html)\n- [AshJsonApi](https://hexdocs.pm/ash_json_api/upgrade.html)\n- [AshGraphql](https://hexdocs.pm/ash_graphql/upgrade.html)","ref":"upgrading-to-3-0.html#other-packages"},{"type":"extras","title":"Upgrading to 3.0 - Upgrade","doc":"This section contains each breaking change, and the steps required to address it in your application","ref":"upgrading-to-3-0.html#upgrading-to-3-0"},{"type":"extras","title":"Dependency Changes - Upgrade","doc":"#### Ash.Flow\n\nIf you use `Ash.Flow`, include `{:ash_flow, \"~> 0.1.0\"}` in your application.\n\n#### Picosat Elixir\n\nIn 2.0, Ash had a dependency on `picosat_elixir`. In 3.0, this is an optional dependency, to help folks handle certain compatibility issues. To upgrade, add `{:picosat_elixir, \"~> 0.2\"}` to your `mix.exs`.","ref":"upgrading-to-3-0.html#dependency-changes"},{"type":"extras","title":"Ash.Api is now Ash.Domain - Upgrade","doc":"The previous name was often confusing as this is an overloaded term for many. To that end, `Ash.Api` has been renamed to `Ash.Domain`, which better fits our usage and concepts.\n\n#### What you'll need to change\n\nTo make this change you will need to do two things:\n\n1. replace `Ash.Api` with `Ash.Domain` in your application\n2. replace places where an `:api` option is passed to a function with the `:domain` option. For example, `AshPhoenix.Form.for_create(..., api: MyApp.SomeApi)` should now be `AshPhoenix.Form.for_create(..., domain: MyApp.SomeDomain)`\n3. Update your application config to define `ash_domains` instead of `ash_apis`, eg. `config :my_app, ash_domains: [MyApp.MyDomain]`","ref":"upgrading-to-3-0.html#ash-api-is-now-ash-domain"},{"type":"extras","title":"DSL Changes - Upgrade","doc":"- `code_interface.define_for` is now `code_interface.domain`. Additionally, it is set automatically if the `domain` option is specified on `use Ash.Resource`.\n\n- `domain.execution.timeout` used to default to 30 seconds, but now it defaults to `:infinity`. This is because a timeout requires copying memory across process boundaries, and is an unnecessary expense a _vast_ majority of the time. We recommend putting timeouts on specific actions that may need them.\n\n- `actions.create.reject`, `actions.update.reject` and `actions.destroy.reject` have been removed. Blacklisting inputs makes it too easy to make mistakes. Instead, specify an explicit `accept` list.\n\n- `relationships.belongs_to.attribute_writable?` no longer makes the underlying attribute both public and writable. It defaults to the value of `writable?` on the relationship (which itself defaults to `true`), and only controls the generated attributes `writable? true` property. So now, by default, it will be `true`, which is safe when coupled with changes to the `default_accept`, discussed below. Generally, this means you should be safe to remove any occurrences of `attribute_writable? true`.\n\n- `relationships.belongs_to.attribute_public?` has been added, which controls the underlying attribute's `public?` value. This, similar to `attribute_writable?` defaults to the `public?` attribute of the relationship.\n\n- `resource.simple_notifiers` has been removed, in favor of specifying non-DSL notifiers in the `simple_notifiers` option to `use Ash.Resource`.\n\n- `resource.actions.read.filter` can now be specified multiple times. Multiple filters will be combined with `and`.\n\n### `Ash.Registry` has been removed\n\n`Ash.Registry` is no longer needed. Place each resource in the domain instead.\n\n```elixir\nresources do\n  resource Resource1\n  resource Resource2\nend\n```","ref":"upgrading-to-3-0.html#dsl-changes"},{"type":"extras","title":"Expression changes - Upgrade","doc":"When calling a calculation with arguments, this is done via passing a keyword list to the calculation, for example: `full_name(separator: \"\")`. In 2.0, keyword lists were not evaluated as part of the expression in the same way as other values, meaning two things:\n\n1. You did not have to pin usage of template functions, i.e `full_name(separator: arg(:separator))`. Now, you will need to do so: `full_name(separator: ^arg(:separator))`\n\n2. You had to use `expr` to pass an expression to a calculation argument (this only works if `allow_expr? true` is configured on the calculation argument). For example: `full_name(separator: expr(sep_1 <> sep_2))` would now be `full_name(separator: sep_1 <> sep_2)`\n\nIf you do not have any expression calculations that accept arguments, you likely need to do nothing. To make these changes, you will need to look at each place you build an expression that you may be calling a calculation w/ arguments, i.e `Ash.Query.filter`, and the `expression` callback in `Ash.Calculation`, and see if they must be modified as described above.","ref":"upgrading-to-3-0.html#expression-changes"},{"type":"extras","title":"Module/function changes - Upgrade","doc":"#### Ash.Policy.FilterCheck and Ash.Policy.FilterCheckWithContext are combined\n\n`Ash.Policy.FilterCheck` and `Ash.Policy.FilterCheckWithContext` have been combined into `Ash.Policy.FilterCheck`. If you have any usages of `FilterCheckWithContext`, you'll need to change that to `FilterCheck`. If you have usages of `FilterCheck`, you will need to add the `context` arguments to the callbacks. Compiler warnings will show you what callbacks mismatch.\n\n#### Ash.Filter\n\n`Ash.Filter.parse/5` is now `Ash.Filter.parse/3`. `Ash.Filter.parse_input/5` is now `Ash.Filter.parse_input/2` The third and fourth optional arguments are unnecessary and were previously ignored, and the fifth argument is not necessary for `parse_input`.\n\n`Ash.Filter.used_aggregates/3` no longer accepts `:all` as a relationship path, instead using `:*`. Its very unlikely that this is used in your application.\n\n#### Ash.Filter.TemplateHelpers\n\nTools for templating expressions were previously in `Ash.Filter.TemplateHelpers`. This often led to confusion because it was a hard to remember module name, and didn't really make sense to be separate from the rest of our utilities. Now, all the functions/macros you need for expressions are in `Ash.Expr`. This means that in any given file where you want to work with expressions, you only need to do `import Ash.Expr`. Additionally, this `import Ash.Expr` has been added to changes, preparations, validations and calculations automatically.\n\n#### Ash.CiString\n\n- `Ash.CiString.new(nil)` now returns `nil` instead of `%Ash.CiString{value: nil}`\n\n#### Ash.Resource.Validation\n\n`validate/2` is now `validate/3`, with the third argument being the context of the validation.\n\n#### Ash.Query.Calculation\n\nThe function signature of `Ash.Query.Calculation.new` has been changed. We use an options list over optional arguments, and now require constraints to be provided. You will need to adjust your calls to this function.\n\n#### Ash.Calculation\n\nThis module has been renamed to `Ash.Resource.Calculation`. You will need to rename your references to it.\n\n#### Ash.Query\n\nAsh.Query.to_query has been removed. Use `Ash.Query.new` instead.\n\nAsh.Query.expr has been removed. Use `Ash.Expr.expr` instead.\n\n#### Aggregates\n\n`first` and `list` aggregates have a new option called `include_nil?`, which _defaults to false_. You may need to add `include_nil?: true` to your resource aggregates if you wish to retain the old behavior.\n\n##### New format for sorting on calculations with arguments\n\nThe format for sorting on calculations that take input has been swapped. Previously, you would use `sort(calculation: {:desc, %{arg: :value}})`, but for the sake of consistency, you now use `sort(calculation: {%{arg: :value}, :desc})`.\n\n#### Ash.Changeset\n\n`Ash.Changeset.new/2` has been removed. `Ash.Changeset.new/1` is still available for creating a new changeset, but attributes and arguments should, with few exceptions, be passed to the relevant `Ash.Changeset.for_ ` functions, _not_ to `Ash.Changeset.new/2`. Removing the second argument helps clarify the purpose of `Ash.Changeset.new/1`.\n\n`Ash.Changeset.after_transaction/2` can no longer be called from within other lifecycle hooks. We need to know whether or not an after action hook, before we start processing any hooks.\n\n`Ash.Changeset.manage_relationship/4` no longer uses `:all` to signal that all changes will be sent to the join relationship. Instead, use `:*`.\n\n`Ash.Changeset.filter` now accepts expressions. The value of the filter is no longer a simple equality map, but rather a regular Ash expression. We add to it on successive calls to `Ash.Changeset.filter`. Additionally, this value is stored in `changeset.filter` instead of `changeset.filters`.\n\n#### Ash.Policy.FilterCheckWithContext\n\n`Ash.Policy.FilterCheck` and `Ash.Policy.FilterCheckWithContext` have been combined. The name is `Ash.Policy.FilterCheck`, but the callbacks take the extra arguments present in `Ash.Policy.FilterCheckWithContext`.\n\n#### Builtin Changes & Preparations\n\nThe functions provided to `after_action/1`, `after_transaction/1`, `before_transaction/1` and `before_action/1` must all now take an additional argument, which is the change context.\n\nFor example,\n\n```elixir\nchange after_action(fn changeset, result -> ... end)\n```\n\nis now\n\n```elixir\nchange after_action(fn changeset, result, context -> ... end)\n```\n\nThis is true for both preparations and changes.\n\n#### Expressions\n\nPreviously, in expressions, you could say `expr(ref(^some_atom))`. This is a tool for building dynamic references, but it was an exception to the standard pattern of prefixing \"external\" things in an expression, i.e `arg` with `^`. Now, you must do the same with `ref/1` and `ref/2`. You will need to search for `ref(` in your application, and ensure that if it is inside of an expression you have prefixed it with `^`. The original example becomes: `expr(^ref(some_atom))`.\n\n#### Exception changes\n\n##### Ash exceptions have been simplified and are now backed by `Splode`\n\nUsage of `def_ash_error/2` will show you what to change in its warnings.\n\nInstead of combining `def_ash_error` with `defimpl Ash.ErrorKind`, you create a custom error like so:\n\n```elixir\ndefmodule MyCustomError do\n  use Splode.Error, class: :invalid, fields: [:foo, :bar]\n\n  def message(error) do\n    \"Message: #{error.foo} - #{error.bar}\"\n  end\nend\n```\n\n##### Ash exception changes\n\nWhen sorting or filtering, if a field is not found, an `Ash.Query.Error.NoSuchField` is used, where it would have previously been an `Ash.Query.Error.NoSuchAttribute`. This was wrong as sometimes the field reference was not an attribute. Places that would previously return `Ash.Query.Error.NoSuchAttributeOrRelationship` now return `Ash.Query.Error.NoSuchField` as well.\n\nAdditionally, the following exceptions have had keys remapped:\n\n`NoSuchAttribute`: `name` -> `attribute`\n`NoSuchRelationship`: `name` -> `relationship`\n`NoSuchFunction`: `name` -> `function`\n`NoSuchOperator`: `name` -> `operator`\n\n---","ref":"upgrading-to-3-0.html#module-function-changes"},{"type":"extras","title":"Significant Changes - Upgrade","doc":"### `Ash.set_*` functions have been removed\n\nIn 2.0, a set of features allowed storing the actor, tenant and context in the process dictionary. There were fundamental issues with this pattern that manifested in subtle bugs. We suggest making this change _before_ you upgrade, as this change can be made and verified without upgrading to 3.0.\n\n#### What you'll need to change\n\nYou need to manually thread through your tenant, actor, and context values wherever you were using `Ash.set_*`. For example:\n\n```elixir\nAsh.set_actor(current_user)\nAsh.set_tenant(current_tenant)\n\nAsh.Changeset.for_create!(..)\nAsh.Query.for_read(..)\n```\n\nwould become\n\n```elixir\nAsh.Changeset.for_create!(.., tenant: current_tenant, actor: current_user)\nAsh.Query.for_read(.., tenant: current_tenant, actor: current_user)\n```","ref":"upgrading-to-3-0.html#significant-changes"},{"type":"extras","title":"The Domain of a resource must now be known when constructing a changeset, query or action input - Upgrade","doc":"In order to honor rules on the `Domain` module about authorization and timeouts, we have to know the `Domain` when building the changeset.\n\n#### What you'll need to change\n\n##### Embedded Resources\n\nThe domain for the calls to embedded resources is gotten from the parent changeset. No need to change them at all. a `domain` constraint has been added in case you wish to make a given embedded resource use a specific domain always.\n\nFor example:\n\n```elixir\nattribute :bio, MyApp.Bio do\n  constraints domain: MyApp.SomeDomain\nend\n```\n\n##### Single Domain resources\n\nWhile it is possible for resources to be used with multiple domains, it almost never happens in practice. Any resources that are only used from a single domain only (_not_ including embedded resources) should be modified to have a `domain` option specified in their call to `use Ash.Resource`. For example:\n\n```elixir\nuse Ash.Resource,\n  domain: MyApp.MyDomain\n```\n\n###### Using `Ash.*` to interact with your resources\n\nCalling functions on the domain has been deprecated. You must now use the functions defined in the `Ash` module to interact with your resources. They are the same as what was previously available in your domain module. For example:\n\n```elixir\nMyDomain1.create!(changeset)\nMyDomain2.read!(query)\nMyDomain3.calculate!(...)\n```\n\ncan now be written as\n\n```elixir\nAsh.create!(changeset)\nAsh.read!(query)\nAsh.calculate!(query)\n```\n\nThis makes refactoring resources easier, as you no longer need to change the call site, it remains the same regardless of what Domain a resource is in.\n\n##### Multi Domain resources\n\nFor these, you will need to include the `domain` option when you construct a changeset.\n\nFor example:\n\n```elixir\nMyResource\n|> Ash.Changeset.for_create(:create, input, domain: MyApp.MyDomain)\n```\n\n---","ref":"upgrading-to-3-0.html#the-domain-of-a-resource-must-now-be-known-when-constructing-a-changeset-query-or-action-input"},{"type":"extras","title":"Actions no longer default to accepting all public writable attributes - Upgrade","doc":"For more context, see the original discussion: https://github.com/ash-project/ash/issues/512\n\nIn 2.0, all public, writable attributes were accepted by each action by default. This made it very easy to accidentally expose writing to an attribute in an action where that was not the intent. Additionally, new attributes added were automatically writable across a wide array of actions, which was error prone for the same reason.\n\nIn 2.0, as well as 3.0, there is an option called `default_accept`, which modifies all actions that do not have an `accept` list. In 2.0, the default value for `default_accept` was \"all public, writable attributes\". In 3.0, the default value for `default_accept` is `[]`. This encourages a pattern of explicitly listing inputs to actions, and is safer and less error prone.\n\n#### What you'll need to change\n\nFor those who want to upgrade, you would use the new `:*` option to `default_accept` (also usable in an action's `accept` option) to accept all public attributes. Go to each resource and, inside the actions block, add:\n\n```elixir\nactions do\n  default_accept :*\n  ...\nend\n```\n\nThen mark the attributes and relationships you want to accept as `public?: true` (see [this section](#private-true-public-true) for more information on this change).\n\nFor those who want to be more explicit, or after your upgrade has complete if you wish to refactor existing resources and actions, the general best path forward is to copy the `default_accept` into each action (or put it in a module attribute and reference it) as the `accept` option. This way when a new action is added, it does not \"inherit\" some list of accepted attributes.\n\n> ### :\\* private attributes can now be accepted {: .info}\n>\n> In 2.0, accepting a private attribute as a change required adding an argument with the same name, and using `change set_attribute(...)`. Now that we require explicit accept lists, you can place private attributes in that list, which will allow them to be written to (but not read back).\n\n> ### :\\* includes belongs_to attributes! {: .warning}\n>\n> The change to explicit accepts also included a change that defaults belongs_to attributes to `writable?: true` and `public?: false`. You may want to add `attribute_writable?: false` to your belongs_to relationships if you are adding `default_accept :*` and don't currently have `attribute_writable?: true` on them currently.\n\n---","ref":"upgrading-to-3-0.html#actions-no-longer-default-to-accepting-all-public-writable-attributes"},{"type":"extras","title":"Default actions :create and :update can now have an accept list - Upgrade","doc":"For example:\n\n```elixir\ndefaults [:read, :destroy, create: :*, update: :*]\n```","ref":"upgrading-to-3-0.html#default-actions-create-and-update-can-now-have-an-accept-list"},{"type":"extras","title":"Default read actions are now paginatable - Upgrade","doc":"In 2.0, if you have `:read` in your default actions list, it would generate an action like this:\n\n```elixir\nread :read do\n  primary? true\nend\n```\n\nNow, it generates an action like this:\n\n```elixir\nread :read do\n  primary? true\n  pagination [keyset?: true, offset?: true, countable: true, required?: false]\nend\n```\n\n---","ref":"upgrading-to-3-0.html#default-read-actions-are-now-paginatable"},{"type":"extras","title":"What you will need to change - Upgrade","doc":"For most cases, this won't affect you. However, if you are using `AshGraphql`, and have any queries connected to a default `:read` action, it will default to making those queries paginatable with keyset pagination. To keep the old behavior, you will need to add `paginate_with nil` to the query, for example:\n\n```elixir\ngraphql do\n  queries do\n    list :list_things, :read, paginate_with: nil\n  end\nend\n```\n\n---","ref":"upgrading-to-3-0.html#what-you-will-need-to-change"},{"type":"extras","title":"Before action and before transaction hooks order has been reversed - Upgrade","doc":"In Ash 2.0, `before_action` and `before_transaction` hooks that were added to a changeset were prepended to the list of hooks by default. These hooks were then run in order. What this meant is that, given an action like the following:\n\n```elixir\ncreate :foo do\n  change before_action(fn changeset, _context ->\n    IO.puts(\"first\")\n    changeset\n  end)\n\n  change before_action(fn changeset, _context ->\n    IO.puts(\"second\")\n    changeset\n  end)\nend\n```\n\nYou would see `second` printed _before_ `first`.\n\n#### What you'll need to change\n\nIn many cases, this won't matter to you. However, if you have a situation where the order of your before action/transaction hooks matters, you can do one of two things:\n\n1. reorder the changes that add those before action/transaction hooks\n2. use the `:prepend` option to `Ash.Changeset.before_action/2` and `Ash.Changeset.before_transaction/2` to explicitly prepend the hook to the list of hooks\n\n---","ref":"upgrading-to-3-0.html#before-action-and-before-transaction-hooks-order-has-been-reversed"},{"type":"extras","title":"Context in changes, preparations, validations, calculations are now structs - Upgrade","doc":"To help make it clear what keys are available in the context provided to callbacks on these modules, they have been adjusted to provide a _struct_ instead of a `map`. This helps avoid potential ambiguity, and\nacts as documentation.\n\n#### What you'll need to change\n\nIf you are using something like `Keyword.new(context)` to generate options to pass into an action, change that to `Ash.Context.to_opts(context)`.\n\n---","ref":"upgrading-to-3-0.html#context-in-changes-preparations-validations-calculations-are-now-structs"},{"type":"extras","title":"Calculation arguments are now in context.arguments - Upgrade","doc":"Per the above change, we have specified the values available in the context of a calculation, with `Ash.Resource.Calculation.Context`. In Ash 2.0, context was merged with arguments, which was problematic in various ways. Now, arguments are in `context.arguments`.\n\n#### What you'll need to change\n\nYou will need to update your module-backed calculations to account for this.\n\n```elixir\ndef calculate(records, _opts, context) do\n  Enum.map(records, fn record ->\n    record.first_name <> context.delimiter <> record.last_name\n  end)\nend\n```\n\nwould need to be adjusted to access arguments in the context:\n\n```elixir\ndef calculate(records, _opts, %{arguments: arguments}) do\n  Enum.map(records, fn record ->\n    record.first_name <> arguments.delimiter <> record.last_name\n  end)\nend\n```\n\n---\n\n### `private?: true` -> `public?: true`\n\nThere is no longer a `private?` option for attributes, relationships, calculations and aggregates. Instead of attributes defaulting to `private?: false`, they now default to `public?: false`. It was too easy to add an attribute and not realize that you had exposed it over your api.\n\n#### What you'll need to change\n\nIf you are using api extensions (i.e `AshGraphql` and `AshJsonApi`), you will need to go to your resources and \"invert\" the definitions. i.e _remove_ `private?: true` and _add_ `public?: true` to _every other_ attribute, _relationship_ and _calculation_. Don't forget the relationships and calculations!\n\n>","ref":"upgrading-to-3-0.html#calculation-arguments-are-now-in-context-arguments"},{"type":"extras","title":"Embedded resources too! - Upgrade","doc":">\n> The above includes embedded resources as well! Don't forget to make sure that all fields on your embedded resources are also marked as `public?: true` (if applicable). The goal here is to have a clear visual indicator of what in your application can be shown publicly.\n\n---","ref":"upgrading-to-3-0.html#embedded-resources-too"},{"type":"extras","title":"Anonymous calculations now operate on a list, just like module calculations - Upgrade","doc":"Previously, anonymous function calculations were special cased to operate on a single record. For consistency, these anonymous functions now take the list of records.\n\n#### What you'll need to change\n\nUpdate any anonymous function calculations to take and return a list, for example:\n\n```elixir\ncalculate :full_name, :string, fn record, _context ->\n  record.first_name <> \" \" <> record.last_name\nend\n```\n\nwould become\n\n```elixir\ncalculate :full_name, :string, fn records, _context ->\n  # note, you can also return `{:ok, list}` or `{:error, error}`\n  Enum.map(records, fn record ->\n    record.first_name <> \" \" <> record.last_name\n  end)\nend\n```\n\n---","ref":"upgrading-to-3-0.html#anonymous-calculations-now-operate-on-a-list-just-like-module-calculations"},{"type":"extras","title":"Calculation loads do not select all related fields by default - Upgrade","doc":"In 2.0 relationship loads from the `load/3` callback in a calculation will select _all_ fields of that relationship and make them available to the calculation.\n\nFor example, the following calculation `load/3` callback expresses a dependency on _all_ fields of the relationship `:relationship`.\n\n```elixir\ndef load(_, _, _) do\n  [:relationship]\nend\n```\n\n#### What you'll need to change\n\n##### Refactor\n\nIn 3.0, relationship dependencies alone will only make the related primary keys available. You now need to select explicit fields that you want to use in your calculation, for example:\n\n```elixir\ndef load(_, _, _) do\n  [relationship: [:field1, :field2]]\nend\n```\n\n#### Keep the old behavior\n\nEach calculation can still opt into the old behavior by adding the callback `strict_loads/0` and returning `false`.\n\n```elixir\ndef load(_, _, _) do\n  [:relationship]\nend\n\ndef strict_loads, do: false\n```\n\n---","ref":"upgrading-to-3-0.html#calculation-loads-do-not-select-all-related-fields-by-default"},{"type":"extras","title":"Calculations do not have a select/3 callback any more - Upgrade","doc":"In 2.0 calculations had a `select/3` callback, but `load/3` is now a superset of `select/3` and so the former is no longer needed.\n\n#### What you'll need to change\n\nIf you have a `select/3` callback in your calculations, you will need to remove the `select/3` callback. You must then add those fields to the `load/3` callback.\n\nFor example:\n\n```elixir\ndef select(_, _, _), do: [:some_attribute]\ndef load(_, _, _), do: [:some_calculation, some_relationship: [:some_field1, :some_field2]]\n```\n\ncan now be written more simply as:\n\n```elixir\ndef load(_, _, _), do: [:some_attribute, :some_calculation, some_relationship: [:some_field1, :some_field2]]\n```\n\n---","ref":"upgrading-to-3-0.html#calculations-do-not-have-a-select-3-callback-any-more"},{"type":"extras","title":"Embedded resources no longer have autogenerated_id - Upgrade","doc":"A private primary key called `autogenerated_id` was added to embedded resources if no primary key was added manually.\n\nThis should have no real effect on your application, except for the fact that your embedded attributes will have `autogenerated_id` in the database which won't be reflected by an attribute any more. If it is updated, then the `autogenerated_id` field will go away.\n\nThis is listed as a breaking change in case someone is depending on this feature, but that should be very uncommon/unlikely.\n\n---","ref":"upgrading-to-3-0.html#embedded-resources-no-longer-have-autogenerated_id"},{"type":"extras","title":"PubSub notifier no longer publishes events for previous values by default - Upgrade","doc":"Previously, the Ash notifier would publish a message containing both the old _and_ new values for changing attributes. Typically, we use\nthings like IDs in notification topics, that do not change, so for most this will not have an impact.\n\nIf you wish to send a notification for the old value and the new value, then an action cannot be done atomically. Bulk actions must update each record in turn, and atomic updates can't be leveraged.\n\nIf you're comfortable with the performance implications, you can restore the previous behavior by adding `previous_values?: true` to your publications in your pub_sub notifier\n\n```elixir\npublish :update, [\"user:updated\", :email], previous_values?: true\n```\n\n---","ref":"upgrading-to-3-0.html#pubsub-notifier-no-longer-publishes-events-for-previous-values-by-default"},{"type":"extras","title":"Custom checks and notifiers will not have access to the original data by default - Upgrade","doc":"In your notifiers and policy checks, when you get a changeset you currently have access to the `data` field,\nwhich is the original record prior to being updated or destroyed. However, this is not compatible with atomic/bulk\nupdates/destroys, where we may be given a query and told to destroy it. In those cases, `changeset.data` will be\n`%Ash.Changeset.OriginalDataNotAvailable{}`. When you write a custom check or a custom notifier, if you need access to the original data, you must add the following function:\n\n```elixir\n# in custom checks\ndef requires_original_data?(_authorizer, _opts), do: true\n\n# in notifiers\ndef requires_original_data?(_resource, _action), do: true\n```\n\nKeep in mind, this will prevent the usage of these checks/notifiers with atomic actions.\n\n---\n\n### `Domain.authorization.authorize` now defaults to `:by_default`\n\nPreviously, the default was `:when_requested`. This meant that, unless you said `actor: some_actor` or `authorize?: true`, authorization was skipped. This has the obvious drawback of making it easy to accidentally bypass authorization unintentionally. In 3.0, this now defaults to `:by_default`.\n\n#### What you'll need to change\n\n##### Keep old behavior\n\nTo avoid making a significant refactor, and to keep your current behavior, you can go to your domain and set the configuration below. Otherwise skip to the refactor steps below. We advise that you take this route to start, but we _highly suggest_ that you change your domains to `authorize :by_default` in the future. `authorize :when_requested` will not be deprecated, so there is no time constraint.\n\n```elixir\nauthorization do\n  authorize :when_requested\nend\n```\n\n##### Refactor\n\nFor each domain that has the old configuration, after setting it to the new config, you'll need to revisit each call to that domain that doesn't set an actor or the `authorize?` option, and add `authorize?: false`.\n\nThis may be a good time to do the refactor from `YourDomain.func` to `Ash.func`, if you want to. See the section about domains being required when building changesets.\n\n---\n\n### `require_atomic?` defaults to `true`\n\nOn `:update` actions, and `:destroy` actions, they now default to `require_atomic? true`. This means that the following things will cause errors when attempting to run the action:\n\n1. changes or validations exist that do not have the `atomic` callback. _This includes anonymous function changes/validations_.\n2. attributes are being changed that do not support atomic updates. This most notably includes (for now) embedded resources.\n3. the action has a manual implementation\n4. the action has applicable notifiers that require the original data.\n\nUpdates and destroys that can be made fully atomic are always safe to do concurrently, and as such we now require that actions meet this criteria, or that it is explicitly stated that they do not have to. See the [update actions guide](/documentation/topics/actions/update-actions.md#fully-atomic-updates) for more.\n\n#### Upgrade config\n\nYou can set the following configuration, which _will be removed in Ash 3.1_. This configuration will retain the 2.0 default behavior of `require_atomic?` defaulting to `false`. You can then safely do the rest of the upgrade. Then, you can perform this one change after confirming that your system works as expected.\n\n```elixir\nconfig :ash, :require_atomic_by_default?, false\n```\n\n#### What you'll need to change\n\nThe vast majority of cases will be caught by warnings emitted at compile time.\n\n##### Anonymous function changes\n\nAnonymous function changes can never be made atomic, because we don't know what they contain. You will either need to transfer it to a module change and then follow the next section, or set `require_atomic? false`\n\n##### Module changes\n\nIf you have a module change, you can make it atomic by defining the `atomic/3` callback. This callback can replace the `change/3` callback, but it is very important to keep in mind that _later changes_ will no longer have access to the value. For example, if you have\n\n```elixir\ndef change(changeset, _, _) do\n  # this is not concurrency safe\n  Ash.Changeset.change_attribute(changeset, :value, changeset.data.value + 1)\nend\n```\n\nIf you have a subsequent change that does something like `Ash.Changeset.get_attribute(changeset, :value)` it will get the new value (i.e old value + 1). With atomics, `Ash.Changeset.get_attribute(changeset, :value)` would return the _old value_. This is because atomics are scheduling an update that happens when call the data layer. For example:\n\n```elixir\ndef atomic(changeset, _, _) do\n  {:atomic, %{value: expr(value + 1)}}\nend\n```\n\nThis should not typically matter unless you have complex actions w/ multiple changes where subsequent changes need to know the results of previous steps. In those cases, if you can't make them all atomic, then its best just not to worry about it and set `require_atomic? false`\n\n##### Non-atomic castable types\n\nIf you are using `change atomic_update/2` or `Ash.Changeset.atomic_update/2` or `Ash.Changeset.atomic_update/3`, and the type does not support atomic updates, you will get an error unless you do one of the following:\n\n1. for `change atomic_update/2` add the `cast_atomic?: false` option.\n2. for `Ash.Changeset.atomic_update`, pass the value as `{:atomic, expr}`, i.e `Ash.Changeset.atomic_update(changeset, :value, {:atomic, expr(value + 1)})`\n\nFor builtin types, the above applies to `:union`, `:map`, `:keyword`, embedded types. It also applies to `:string`, but only if the `match?` constraint is present.\n\n---\n\n### `Ash.Error.Invalid.NoSuchInput` errors on unknown action inputs\n\nIn 2.0, inputs to actions that don't match an accepted attribute or argument were silently ignored. This made it very easy to make certain kinds of mistakes, like assuming that an input is being used by an action when it actually is not. Now, unknown action inputs will cause an `Ash.Error.Invalid.NoSuchInput`.\n\n#### What you'll need to change\n\nIf you have action calls that are erroneously passing in extra values, you will need to do remove them.\n\nA logic error was fixed in this behavior for embedded resources. If you are using embedded resources in `{:array, _}` types, and are relying on including the primary key of that embedded resource to match records up for updating/destroy behavior, you will need to make sure that you do one of the following\n\n1. add the `writable?: true` flag to the uuid of the embedded resource (probably what you want)\n2. modify the actions to accept an `id` argument and set the argument to the provided value\n\n---\n\n### `%Ash.NotLoaded{}` for attributes\n\nIn 2.0, attributes that were not selected were replaced with `nil` values. This could lead to confusion when dealing with records that didn't have all attributes selected. If you passed these records to a function it might see that an attribute is `nil` when actually it just wasn't selected. To find out if it was selected, you could look into `record.__metadata__.selected`, but you'd have to know to do that. To alleviate these issues, attributes that are not selected are now filled in with `%Ash.NotLoaded{}`, just like calculations and aggregates.\n\n#### What you'll need to change\n\nIf you have logic that was looking at attribute values that may not be selected, you may have been accidentally working with non selected values. For example:\n\n```elixir\nif record.attribute do\n  handle_present_attribute(...)\nelse\n  # unselected attributes would have ended up in this branch\n  handle_not_present_attribute(...)\nend\n```\n\nNow, if it is possible for that attribute to have not been selected, you'll want to do something like this instead:\n\n```elixir\ncase record.attribute do\n  %Ash.NotLoaded{} ->\n    handle_not_selected(...)\n  nil ->\n    handle_not_present_attribute(...)\n  value ->\n    handle_present_attribute(...)\nend\n```\n\n---","ref":"upgrading-to-3-0.html#custom-checks-and-notifiers-will-not-have-access-to-the-original-data-by-default"},{"type":"extras","title":"Calculations do not reuse values by default - Upgrade","doc":"When loading data in 2.0 the option `reselect_all?` defaulted to `false`. What this would mean is that existing values for attributes would be reused, instead of visiting the data layer, by default. This can be an extremely valuable piece of behavior, but _defaulting_ to it often means accidentally using data as a cache that you did not intent to use as a cache. Take the following example:\n\n```elixir\nuser = %User{first_name: \"fred\", last_name: \"weasley\"}\n\nAsh.update!(user, first_name: \"george\")\n\nuser |> Ash.load!(:full_name)\n# in 2.0 -> fred weasley\n# in 3.0 -> george weasley\n```\n\nTo opt into the old behavior, which we recommend doing on a case-by-case basis, you can pass `reuse_values?: true`. For example:\n\n```elixir\nuser |> Ash.load!(:full_name, reuse_values?: true)\n```\n\n---","ref":"upgrading-to-3-0.html#calculations-do-not-reuse-values-by-default"},{"type":"extras","title":"Resources are not interchangeable with Ash.Type anymore - Upgrade","doc":"In 2.0 it was possible to pass an Ash resource in all places where some instance of `Ash.Type` was supported. In 3.0 resources (except for embedded resources) don't implement the `Ash.Type` behaviour anymore.\n\n#### What you'll need to change\n\nIf you were using a resource in one of the places that accept an `Ash.Type` (arguments, calculation return values or fields of a union) you have to refactor your code to use the `:struct` type together with an `instance_of` constraint\n\n```elixir\ncalculation :random_post, :struct do\n  constraints instance_of: Post\n  calculate Calculations.RandomPost\nend\n```","ref":"upgrading-to-3-0.html#resources-are-not-interchangeable-with-ash-type-anymore"},{"type":"DSL","title":"attributes - dsl-ash-resource","doc":"A section for declaring attributes on the resource.","ref":"dsl-ash-resource.html#attributes"},{"type":"DSL","title":"attributes.attribute - dsl-ash-resource","doc":"Declares an attribute on the resource.","ref":"dsl-ash-resource.html#attributes-attribute"},{"type":"DSL","title":"attributes.attribute.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-attribute-name"},{"type":"DSL","title":"attributes.attribute.type - dsl-ash-resource","doc":"The type of the attribute. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#attributes-attribute-type"},{"type":"DSL","title":"attributes.attribute.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more, see `Ash.Type`.","ref":"dsl-ash-resource.html#attributes-attribute-constraints"},{"type":"DSL","title":"attributes.attribute.description - dsl-ash-resource","doc":"An optional description for the attribute.","ref":"dsl-ash-resource.html#attributes-attribute-description"},{"type":"DSL","title":"attributes.attribute.sensitive? - dsl-ash-resource","doc":"Whether or not the attribute value contains sensitive information, like PII(Personally Identifiable Information). See the [Sensitive Data guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#attributes-attribute-sensitive?"},{"type":"DSL","title":"attributes.attribute.source - dsl-ash-resource","doc":"If the field should be mapped to a different name in the data layer. Support varies by data layer.","ref":"dsl-ash-resource.html#attributes-attribute-source"},{"type":"DSL","title":"attributes.attribute.select_by_default? - dsl-ash-resource","doc":"Whether or not the attribute is selected by default.","ref":"dsl-ash-resource.html#attributes-attribute-select_by_default?"},{"type":"DSL","title":"attributes.attribute.always_select? - dsl-ash-resource","doc":"Whether or not to ensure this attribute is always selected when reading from the database, regardless of applied select statements.","ref":"dsl-ash-resource.html#attributes-attribute-always_select?"},{"type":"DSL","title":"attributes.attribute.primary_key? - dsl-ash-resource","doc":"Whether the attribute is the primary key. Composite primary key is also possible by using `primary_key? true` in more than one attribute. If primary_key? is true, allow_nil? must be false.","ref":"dsl-ash-resource.html#attributes-attribute-primary_key?"},{"type":"DSL","title":"attributes.attribute.allow_nil? - dsl-ash-resource","doc":"Whether or not the attribute can be set to nil. If nil value is given error is raised.","ref":"dsl-ash-resource.html#attributes-attribute-allow_nil?"},{"type":"DSL","title":"attributes.attribute.generated? - dsl-ash-resource","doc":"Whether or not the value may be generated by the data layer.","ref":"dsl-ash-resource.html#attributes-attribute-generated?"},{"type":"DSL","title":"attributes.attribute.writable? - dsl-ash-resource","doc":"Whether or not the value can be written to. Non-writable attributes can still be written with `Ash.Changeset.force_change_attribute/3`.","ref":"dsl-ash-resource.html#attributes-attribute-writable?"},{"type":"DSL","title":"attributes.attribute.public? - dsl-ash-resource","doc":"Whether or not the attribute should be shown over public interfaces. See the [sensitive data guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#attributes-attribute-public?"},{"type":"DSL","title":"attributes.attribute.default - dsl-ash-resource","doc":"A value to be set on all creates, unless a value is being provided already.  Note: The default value is casted according to the type's `Ash.Type.*` module, before it is saved.  For `:string`, for example, if `constraints: [allow_empty?: _]` is false, the value `\"\"` will be cast to `nil`.  See the `:constraints` option, the `:allow_nil?` option, and the relevant `Ash.Type.*` documentation.","ref":"dsl-ash-resource.html#attributes-attribute-default"},{"type":"DSL","title":"attributes.attribute.update_default - dsl-ash-resource","doc":"A value to be set on all updates, unless a value is being provided already.","ref":"dsl-ash-resource.html#attributes-attribute-update_default"},{"type":"DSL","title":"attributes.attribute.filterable? - dsl-ash-resource","doc":"Whether or not the attribute can be referenced in filters.","ref":"dsl-ash-resource.html#attributes-attribute-filterable?"},{"type":"DSL","title":"attributes.attribute.sortable? - dsl-ash-resource","doc":"Whether or not the attribute can be referenced in sorts.","ref":"dsl-ash-resource.html#attributes-attribute-sortable?"},{"type":"DSL","title":"attributes.attribute.match_other_defaults? - dsl-ash-resource","doc":"Ensures that other attributes that use the same \"lazy\" default (a function or an mfa), use the same default value. Has no effect unless `default` is a zero argument function.","ref":"dsl-ash-resource.html#attributes-attribute-match_other_defaults?"},{"type":"DSL","title":"attributes.create_timestamp - dsl-ash-resource","doc":"Declares a non-writable attribute with a create default of `&DateTime.utc_now/0`\n\nAccepts all the same options as `d:Ash.Resource.Dsl.attributes.attribute`, except it sets\nthe following different defaults:\n\n```elixir\nwritable? false\ndefault &DateTime.utc_now/0\nmatch_other_defaults? true\ntype Ash.Type.UTCDatetimeUsec\nallow_nil? false\n```","ref":"dsl-ash-resource.html#attributes-create_timestamp"},{"type":"DSL","title":"attributes.create_timestamp.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-create_timestamp-name"},{"type":"DSL","title":"attributes.update_timestamp - dsl-ash-resource","doc":"Declares a non-writable attribute with a create and update default of `&DateTime.utc_now/0`\n\nAccepts all the same options as `d:Ash.Resource.Dsl.attributes.attribute`, except it sets\nthe following different defaults:\n\n```elixir\nwritable? false\ndefault &DateTime.utc_now/0\nmatch_other_defaults? true\nupdate_default &DateTime.utc_now/0\ntype Ash.Type.UTCDatetimeUsec\nallow_nil? false\n```","ref":"dsl-ash-resource.html#attributes-update_timestamp"},{"type":"DSL","title":"attributes.update_timestamp.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-update_timestamp-name"},{"type":"DSL","title":"attributes.integer_primary_key - dsl-ash-resource","doc":"Declares a generated, non writable, non-nil, primary key column of type integer.\n\nGenerated integer primary keys must be supported by the data layer.\n\nAccepts all the same options as `d:Ash.Resource.Dsl.attributes.attribute`, except for `allow_nil?`, but it sets\nthe following different defaults:\n\n```elixir\npublic? true\nwritable? false\nprimary_key? true\ngenerated? true\ntype :integer\n```","ref":"dsl-ash-resource.html#attributes-integer_primary_key"},{"type":"DSL","title":"attributes.integer_primary_key.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-integer_primary_key-name"},{"type":"DSL","title":"attributes.uuid_primary_key - dsl-ash-resource","doc":"Declares a non writable, non-nil, primary key column of type `uuid`, which defaults to `Ash.UUID.generate/0`.\n\nAccepts all the same options as `d:Ash.Resource.Dsl.attributes.attribute`, except for `allow_nil?`, but it sets\nthe following different defaults:\n\n```elixir\nwritable? false\npublic? true\ndefault &Ash.UUID.generate/0\nprimary_key? true\ntype :uuid\n```","ref":"dsl-ash-resource.html#attributes-uuid_primary_key"},{"type":"DSL","title":"attributes.uuid_primary_key.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-uuid_primary_key-name"},{"type":"DSL","title":"attributes.uuid_v7_primary_key - dsl-ash-resource","doc":"Declares a non writable, non-nil, primary key column of type `uuid_v7`, which defaults to `Ash.UUIDv7.generate/0`.\n\nAccepts all the same options as `d:Ash.Resource.Dsl.attributes.attribute`, except for `allow_nil?`, but it sets\nthe following different defaults:\n\n```elixir\nwritable? false\npublic? true\ndefault &Ash.UUIDv7.generate/0\nprimary_key? true\ntype :uuid_v7\n```","ref":"dsl-ash-resource.html#attributes-uuid_v7_primary_key"},{"type":"DSL","title":"attributes.uuid_v7_primary_key.name - dsl-ash-resource","doc":"The name of the attribute.","ref":"dsl-ash-resource.html#attributes-uuid_v7_primary_key-name"},{"type":"DSL","title":"relationships - dsl-ash-resource","doc":"A section for declaring relationships on the resource.\n\nRelationships are a core component of resource oriented design. Many components of Ash\nwill use these relationships. A simple use case is loading relationships (done via the `Ash.Query.load/2`).\n\nSee the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships"},{"type":"DSL","title":"relationships.has_one - dsl-ash-resource","doc":"Declares a `has_one` relationship. In a relational database, the foreign key would be on the *other* table.\n\nGenerally speaking, a `has_one` also implies that the destination table is unique on that foreign key.\n\nSee the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-has_one"},{"type":"DSL","title":"relationships.has_one.manual - dsl-ash-resource","doc":"A module that implements `Ash.Resource.ManualRelationship`. Also accepts a 2 argument function that takes the source records and the context.","ref":"dsl-ash-resource.html#relationships-has_one-manual"},{"type":"DSL","title":"relationships.has_one.no_attributes? - dsl-ash-resource","doc":"All existing entities are considered related, i.e this relationship is not based on any fields, and `source_attribute` and `destination_attribute` are ignored. See the See the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-has_one-no_attributes?"},{"type":"DSL","title":"relationships.has_one.allow_nil? - dsl-ash-resource","doc":"Marks the relationship as required. Has no effect on validations, but can inform extensions that there will always be a related entity.","ref":"dsl-ash-resource.html#relationships-has_one-allow_nil?"},{"type":"DSL","title":"relationships.has_one.from_many? - dsl-ash-resource","doc":"Signal that this relationship is actually a `has_many` where the first record is given via the `sort`. This will allow data layers to properly deduplicate when necessary.","ref":"dsl-ash-resource.html#relationships-has_one-from_many?"},{"type":"DSL","title":"relationships.has_one.name - dsl-ash-resource","doc":"The name of the relationship","ref":"dsl-ash-resource.html#relationships-has_one-name"},{"type":"DSL","title":"relationships.has_one.destination - dsl-ash-resource","doc":"The destination resource","ref":"dsl-ash-resource.html#relationships-has_one-destination"},{"type":"DSL","title":"relationships.has_one.description - dsl-ash-resource","doc":"An optional description for the relationship","ref":"dsl-ash-resource.html#relationships-has_one-description"},{"type":"DSL","title":"relationships.has_one.destination_attribute - dsl-ash-resource","doc":"The attribute on the related resource that should match the `source_attribute` configured on this resource.","ref":"dsl-ash-resource.html#relationships-has_one-destination_attribute"},{"type":"DSL","title":"relationships.has_one.validate_destination_attribute? - dsl-ash-resource","doc":"Whether or not to validate that the destination field exists on the destination resource","ref":"dsl-ash-resource.html#relationships-has_one-validate_destination_attribute?"},{"type":"DSL","title":"relationships.has_one.source_attribute - dsl-ash-resource","doc":"The field on this resource that should match the `destination_attribute` on the related resource.","ref":"dsl-ash-resource.html#relationships-has_one-source_attribute"},{"type":"DSL","title":"relationships.has_one.relationship_context - dsl-ash-resource","doc":"Context to be set on any queries or changesets generated for managing or querying this relationship.","ref":"dsl-ash-resource.html#relationships-has_one-relationship_context"},{"type":"DSL","title":"relationships.has_one.public? - dsl-ash-resource","doc":"Whether or not the relationship will appear in public interfaces","ref":"dsl-ash-resource.html#relationships-has_one-public?"},{"type":"DSL","title":"relationships.has_one.not_found_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on update or create, or when managing relationships.","ref":"dsl-ash-resource.html#relationships-has_one-not_found_message"},{"type":"DSL","title":"relationships.has_one.writable? - dsl-ash-resource","doc":"Whether or not the relationship may be managed.","ref":"dsl-ash-resource.html#relationships-has_one-writable?"},{"type":"DSL","title":"relationships.has_one.read_action - dsl-ash-resource","doc":"The read action on the destination resource to use when loading data and filtering.","ref":"dsl-ash-resource.html#relationships-has_one-read_action"},{"type":"DSL","title":"relationships.has_one.domain - dsl-ash-resource","doc":"The domain module to use when working with the related entity.","ref":"dsl-ash-resource.html#relationships-has_one-domain"},{"type":"DSL","title":"relationships.has_one.filterable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-has_one-filterable?"},{"type":"DSL","title":"relationships.has_one.sortable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-has_one-sortable?"},{"type":"DSL","title":"relationships.has_one.sort - dsl-ash-resource","doc":"A sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-has_one-sort"},{"type":"DSL","title":"relationships.has_one.default_sort - dsl-ash-resource","doc":"A default sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-has_one-default_sort"},{"type":"DSL","title":"relationships.has_one.could_be_related_at_creation? - dsl-ash-resource","doc":"Whether or not related values may exist for this relationship at creation.","ref":"dsl-ash-resource.html#relationships-has_one-could_be_related_at_creation?"},{"type":"DSL","title":"relationships.has_one.violation_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on destroy.","ref":"dsl-ash-resource.html#relationships-has_one-violation_message"},{"type":"DSL","title":"relationships.has_one.authorize_read_with - dsl-ash-resource","doc":"If set to `:error`, any authorization filter added to the relationship will result in an error if any record matches the filter in the database.","ref":"dsl-ash-resource.html#relationships-has_one-authorize_read_with"},{"type":"DSL","title":"relationships.has_one.allow_forbidden_field? - dsl-ash-resource","doc":"If set to `true`, the relationship will be set to `%Ash.ForbiddenField{}` if its query produces a forbidden error.","ref":"dsl-ash-resource.html#relationships-has_one-allow_forbidden_field?"},{"type":"DSL","title":"relationships.has_one.filter - dsl-ash-resource","doc":"Applies a filter. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-has_one-filter"},{"type":"DSL","title":"relationships.has_one.filter.filter - dsl-ash-resource","doc":"The filter to apply. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-has_one-filter-filter"},{"type":"DSL","title":"relationships.has_many - dsl-ash-resource","doc":"Declares a `has_many` relationship. There can be any number of related entities.\n\nSee the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-has_many"},{"type":"DSL","title":"relationships.has_many.manual - dsl-ash-resource","doc":"A module that implements `Ash.Resource.ManualRelationship`. Also accepts a 2 argument function that takes the source records and the context.","ref":"dsl-ash-resource.html#relationships-has_many-manual"},{"type":"DSL","title":"relationships.has_many.no_attributes? - dsl-ash-resource","doc":"All existing entities are considered related, i.e this relationship is not based on any fields, and `source_attribute` and `destination_attribute` are ignored. See the See the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-has_many-no_attributes?"},{"type":"DSL","title":"relationships.has_many.limit - dsl-ash-resource","doc":"An integer to limit entries from loaded relationship.","ref":"dsl-ash-resource.html#relationships-has_many-limit"},{"type":"DSL","title":"relationships.has_many.name - dsl-ash-resource","doc":"The name of the relationship","ref":"dsl-ash-resource.html#relationships-has_many-name"},{"type":"DSL","title":"relationships.has_many.destination - dsl-ash-resource","doc":"The destination resource","ref":"dsl-ash-resource.html#relationships-has_many-destination"},{"type":"DSL","title":"relationships.has_many.description - dsl-ash-resource","doc":"An optional description for the relationship","ref":"dsl-ash-resource.html#relationships-has_many-description"},{"type":"DSL","title":"relationships.has_many.destination_attribute - dsl-ash-resource","doc":"The attribute on the related resource that should match the `source_attribute` configured on this resource.","ref":"dsl-ash-resource.html#relationships-has_many-destination_attribute"},{"type":"DSL","title":"relationships.has_many.validate_destination_attribute? - dsl-ash-resource","doc":"Whether or not to validate that the destination field exists on the destination resource","ref":"dsl-ash-resource.html#relationships-has_many-validate_destination_attribute?"},{"type":"DSL","title":"relationships.has_many.source_attribute - dsl-ash-resource","doc":"The field on this resource that should match the `destination_attribute` on the related resource.","ref":"dsl-ash-resource.html#relationships-has_many-source_attribute"},{"type":"DSL","title":"relationships.has_many.relationship_context - dsl-ash-resource","doc":"Context to be set on any queries or changesets generated for managing or querying this relationship.","ref":"dsl-ash-resource.html#relationships-has_many-relationship_context"},{"type":"DSL","title":"relationships.has_many.public? - dsl-ash-resource","doc":"Whether or not the relationship will appear in public interfaces","ref":"dsl-ash-resource.html#relationships-has_many-public?"},{"type":"DSL","title":"relationships.has_many.not_found_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on update or create, or when managing relationships.","ref":"dsl-ash-resource.html#relationships-has_many-not_found_message"},{"type":"DSL","title":"relationships.has_many.writable? - dsl-ash-resource","doc":"Whether or not the relationship may be managed.","ref":"dsl-ash-resource.html#relationships-has_many-writable?"},{"type":"DSL","title":"relationships.has_many.read_action - dsl-ash-resource","doc":"The read action on the destination resource to use when loading data and filtering.","ref":"dsl-ash-resource.html#relationships-has_many-read_action"},{"type":"DSL","title":"relationships.has_many.domain - dsl-ash-resource","doc":"The domain module to use when working with the related entity.","ref":"dsl-ash-resource.html#relationships-has_many-domain"},{"type":"DSL","title":"relationships.has_many.filterable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-has_many-filterable?"},{"type":"DSL","title":"relationships.has_many.sortable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-has_many-sortable?"},{"type":"DSL","title":"relationships.has_many.sort - dsl-ash-resource","doc":"A sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-has_many-sort"},{"type":"DSL","title":"relationships.has_many.default_sort - dsl-ash-resource","doc":"A default sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-has_many-default_sort"},{"type":"DSL","title":"relationships.has_many.could_be_related_at_creation? - dsl-ash-resource","doc":"Whether or not related values may exist for this relationship at creation.","ref":"dsl-ash-resource.html#relationships-has_many-could_be_related_at_creation?"},{"type":"DSL","title":"relationships.has_many.violation_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on destroy.","ref":"dsl-ash-resource.html#relationships-has_many-violation_message"},{"type":"DSL","title":"relationships.has_many.authorize_read_with - dsl-ash-resource","doc":"If set to `:error`, any authorization filter added to the relationship will result in an error if any record matches the filter in the database.","ref":"dsl-ash-resource.html#relationships-has_many-authorize_read_with"},{"type":"DSL","title":"relationships.has_many.allow_forbidden_field? - dsl-ash-resource","doc":"If set to `true`, the relationship will be set to `%Ash.ForbiddenField{}` if its query produces a forbidden error.","ref":"dsl-ash-resource.html#relationships-has_many-allow_forbidden_field?"},{"type":"DSL","title":"relationships.has_many.filter - dsl-ash-resource","doc":"Applies a filter. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-has_many-filter"},{"type":"DSL","title":"relationships.has_many.filter.filter - dsl-ash-resource","doc":"The filter to apply. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-has_many-filter-filter"},{"type":"DSL","title":"relationships.many_to_many - dsl-ash-resource","doc":"Declares a `many_to_many` relationship. Many to many relationships require a join resource.\n\nA join resource is a resource that consists of a relationship to the source and destination of the many to many.\n\nSee the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-many_to_many"},{"type":"DSL","title":"relationships.many_to_many.source_attribute_on_join_resource - dsl-ash-resource","doc":"The attribute on the join resource that should line up with `source_attribute` on this resource. Defaults to ` _id`.","ref":"dsl-ash-resource.html#relationships-many_to_many-source_attribute_on_join_resource"},{"type":"DSL","title":"relationships.many_to_many.destination_attribute_on_join_resource - dsl-ash-resource","doc":"The attribute on the join resource that should line up with `destination_attribute` on the related resource. Defaults to ` _id`.","ref":"dsl-ash-resource.html#relationships-many_to_many-destination_attribute_on_join_resource"},{"type":"DSL","title":"relationships.many_to_many.through - dsl-ash-resource","doc":"The resource to use as the join resource.","ref":"dsl-ash-resource.html#relationships-many_to_many-through"},{"type":"DSL","title":"relationships.many_to_many.join_relationship - dsl-ash-resource","doc":"The has_many relationship to the join resource. Defaults to ` _join_assoc`.","ref":"dsl-ash-resource.html#relationships-many_to_many-join_relationship"},{"type":"DSL","title":"relationships.many_to_many.name - dsl-ash-resource","doc":"The name of the relationship","ref":"dsl-ash-resource.html#relationships-many_to_many-name"},{"type":"DSL","title":"relationships.many_to_many.destination - dsl-ash-resource","doc":"The destination resource","ref":"dsl-ash-resource.html#relationships-many_to_many-destination"},{"type":"DSL","title":"relationships.many_to_many.description - dsl-ash-resource","doc":"An optional description for the relationship","ref":"dsl-ash-resource.html#relationships-many_to_many-description"},{"type":"DSL","title":"relationships.many_to_many.destination_attribute - dsl-ash-resource","doc":"The attribute on the related resource that should match the `source_attribute` configured on this resource.","ref":"dsl-ash-resource.html#relationships-many_to_many-destination_attribute"},{"type":"DSL","title":"relationships.many_to_many.validate_destination_attribute? - dsl-ash-resource","doc":"Whether or not to validate that the destination field exists on the destination resource","ref":"dsl-ash-resource.html#relationships-many_to_many-validate_destination_attribute?"},{"type":"DSL","title":"relationships.many_to_many.source_attribute - dsl-ash-resource","doc":"The field on this resource that should match the `destination_attribute` on the related resource.","ref":"dsl-ash-resource.html#relationships-many_to_many-source_attribute"},{"type":"DSL","title":"relationships.many_to_many.relationship_context - dsl-ash-resource","doc":"Context to be set on any queries or changesets generated for managing or querying this relationship.","ref":"dsl-ash-resource.html#relationships-many_to_many-relationship_context"},{"type":"DSL","title":"relationships.many_to_many.public? - dsl-ash-resource","doc":"Whether or not the relationship will appear in public interfaces","ref":"dsl-ash-resource.html#relationships-many_to_many-public?"},{"type":"DSL","title":"relationships.many_to_many.not_found_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on update or create, or when managing relationships.","ref":"dsl-ash-resource.html#relationships-many_to_many-not_found_message"},{"type":"DSL","title":"relationships.many_to_many.writable? - dsl-ash-resource","doc":"Whether or not the relationship may be managed.","ref":"dsl-ash-resource.html#relationships-many_to_many-writable?"},{"type":"DSL","title":"relationships.many_to_many.read_action - dsl-ash-resource","doc":"The read action on the destination resource to use when loading data and filtering.","ref":"dsl-ash-resource.html#relationships-many_to_many-read_action"},{"type":"DSL","title":"relationships.many_to_many.domain - dsl-ash-resource","doc":"The domain module to use when working with the related entity.","ref":"dsl-ash-resource.html#relationships-many_to_many-domain"},{"type":"DSL","title":"relationships.many_to_many.filterable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-many_to_many-filterable?"},{"type":"DSL","title":"relationships.many_to_many.sortable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-many_to_many-sortable?"},{"type":"DSL","title":"relationships.many_to_many.sort - dsl-ash-resource","doc":"A sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-many_to_many-sort"},{"type":"DSL","title":"relationships.many_to_many.default_sort - dsl-ash-resource","doc":"A default sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-many_to_many-default_sort"},{"type":"DSL","title":"relationships.many_to_many.could_be_related_at_creation? - dsl-ash-resource","doc":"Whether or not related values may exist for this relationship at creation.","ref":"dsl-ash-resource.html#relationships-many_to_many-could_be_related_at_creation?"},{"type":"DSL","title":"relationships.many_to_many.violation_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on destroy.","ref":"dsl-ash-resource.html#relationships-many_to_many-violation_message"},{"type":"DSL","title":"relationships.many_to_many.authorize_read_with - dsl-ash-resource","doc":"If set to `:error`, any authorization filter added to the relationship will result in an error if any record matches the filter in the database.","ref":"dsl-ash-resource.html#relationships-many_to_many-authorize_read_with"},{"type":"DSL","title":"relationships.many_to_many.allow_forbidden_field? - dsl-ash-resource","doc":"If set to `true`, the relationship will be set to `%Ash.ForbiddenField{}` if its query produces a forbidden error.","ref":"dsl-ash-resource.html#relationships-many_to_many-allow_forbidden_field?"},{"type":"DSL","title":"relationships.many_to_many.filter - dsl-ash-resource","doc":"Applies a filter. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-many_to_many-filter"},{"type":"DSL","title":"relationships.many_to_many.filter.filter - dsl-ash-resource","doc":"The filter to apply. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-many_to_many-filter-filter"},{"type":"DSL","title":"relationships.belongs_to - dsl-ash-resource","doc":"Declares a `belongs_to` relationship. In a relational database, the foreign key would be on the *source* table.\n\nThis creates a field on the resource with the corresponding name and type, unless `define_attribute?: false` is provided.\n\nSee the [relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"dsl-ash-resource.html#relationships-belongs_to"},{"type":"DSL","title":"relationships.belongs_to.primary_key? - dsl-ash-resource","doc":"Whether the generated attribute is, or is part of, the primary key of a resource.","ref":"dsl-ash-resource.html#relationships-belongs_to-primary_key?"},{"type":"DSL","title":"relationships.belongs_to.allow_nil? - dsl-ash-resource","doc":"Whether this relationship must always be present, e.g: must be included on creation, and never removed (it may be modified). The generated attribute will not allow nil values.","ref":"dsl-ash-resource.html#relationships-belongs_to-allow_nil?"},{"type":"DSL","title":"relationships.belongs_to.attribute_writable? - dsl-ash-resource","doc":"Whether the generated attribute will be marked as writable. If not set, it will default to the relationship's `writable?` setting.","ref":"dsl-ash-resource.html#relationships-belongs_to-attribute_writable?"},{"type":"DSL","title":"relationships.belongs_to.attribute_public? - dsl-ash-resource","doc":"Whether or not the generated attribute will be public. If not set, it will default to the relationship's `public?` setting.","ref":"dsl-ash-resource.html#relationships-belongs_to-attribute_public?"},{"type":"DSL","title":"relationships.belongs_to.define_attribute? - dsl-ash-resource","doc":"If set to `false` an attribute is not created on the resource for this relationship, and one must be manually added in `attributes`, invalidating many other options.","ref":"dsl-ash-resource.html#relationships-belongs_to-define_attribute?"},{"type":"DSL","title":"relationships.belongs_to.attribute_type - dsl-ash-resource","doc":"The type of the generated created attribute. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#relationships-belongs_to-attribute_type"},{"type":"DSL","title":"relationships.belongs_to.name - dsl-ash-resource","doc":"The name of the relationship","ref":"dsl-ash-resource.html#relationships-belongs_to-name"},{"type":"DSL","title":"relationships.belongs_to.destination - dsl-ash-resource","doc":"The destination resource","ref":"dsl-ash-resource.html#relationships-belongs_to-destination"},{"type":"DSL","title":"relationships.belongs_to.description - dsl-ash-resource","doc":"An optional description for the relationship","ref":"dsl-ash-resource.html#relationships-belongs_to-description"},{"type":"DSL","title":"relationships.belongs_to.destination_attribute - dsl-ash-resource","doc":"The attribute on the related resource that should match the `source_attribute` configured on this resource.","ref":"dsl-ash-resource.html#relationships-belongs_to-destination_attribute"},{"type":"DSL","title":"relationships.belongs_to.validate_destination_attribute? - dsl-ash-resource","doc":"Whether or not to validate that the destination field exists on the destination resource","ref":"dsl-ash-resource.html#relationships-belongs_to-validate_destination_attribute?"},{"type":"DSL","title":"relationships.belongs_to.source_attribute - dsl-ash-resource","doc":"The field on this resource that should match the `destination_attribute` on the related resource. - Defaults to  _id","ref":"dsl-ash-resource.html#relationships-belongs_to-source_attribute"},{"type":"DSL","title":"relationships.belongs_to.relationship_context - dsl-ash-resource","doc":"Context to be set on any queries or changesets generated for managing or querying this relationship.","ref":"dsl-ash-resource.html#relationships-belongs_to-relationship_context"},{"type":"DSL","title":"relationships.belongs_to.public? - dsl-ash-resource","doc":"Whether or not the relationship will appear in public interfaces","ref":"dsl-ash-resource.html#relationships-belongs_to-public?"},{"type":"DSL","title":"relationships.belongs_to.not_found_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on update or create, or when managing relationships.","ref":"dsl-ash-resource.html#relationships-belongs_to-not_found_message"},{"type":"DSL","title":"relationships.belongs_to.writable? - dsl-ash-resource","doc":"Whether or not the relationship may be managed.","ref":"dsl-ash-resource.html#relationships-belongs_to-writable?"},{"type":"DSL","title":"relationships.belongs_to.read_action - dsl-ash-resource","doc":"The read action on the destination resource to use when loading data and filtering.","ref":"dsl-ash-resource.html#relationships-belongs_to-read_action"},{"type":"DSL","title":"relationships.belongs_to.domain - dsl-ash-resource","doc":"The domain module to use when working with the related entity.","ref":"dsl-ash-resource.html#relationships-belongs_to-domain"},{"type":"DSL","title":"relationships.belongs_to.filterable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-belongs_to-filterable?"},{"type":"DSL","title":"relationships.belongs_to.sortable? - dsl-ash-resource","doc":"If set to `false`, the relationship will not be usable in filters.","ref":"dsl-ash-resource.html#relationships-belongs_to-sortable?"},{"type":"DSL","title":"relationships.belongs_to.sort - dsl-ash-resource","doc":"A sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-belongs_to-sort"},{"type":"DSL","title":"relationships.belongs_to.default_sort - dsl-ash-resource","doc":"A default sort statement to be applied when loading the relationship.","ref":"dsl-ash-resource.html#relationships-belongs_to-default_sort"},{"type":"DSL","title":"relationships.belongs_to.violation_message - dsl-ash-resource","doc":"A message to show if there is a conflict with this relationship in the database on destroy.","ref":"dsl-ash-resource.html#relationships-belongs_to-violation_message"},{"type":"DSL","title":"relationships.belongs_to.authorize_read_with - dsl-ash-resource","doc":"If set to `:error`, any authorization filter added to the relationship will result in an error if any record matches the filter in the database.","ref":"dsl-ash-resource.html#relationships-belongs_to-authorize_read_with"},{"type":"DSL","title":"relationships.belongs_to.allow_forbidden_field? - dsl-ash-resource","doc":"If set to `true`, the relationship will be set to `%Ash.ForbiddenField{}` if its query produces a forbidden error.","ref":"dsl-ash-resource.html#relationships-belongs_to-allow_forbidden_field?"},{"type":"DSL","title":"relationships.belongs_to.filter - dsl-ash-resource","doc":"Applies a filter. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-belongs_to-filter"},{"type":"DSL","title":"relationships.belongs_to.filter.filter - dsl-ash-resource","doc":"The filter to apply. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#relationships-belongs_to-filter-filter"},{"type":"DSL","title":"actions - dsl-ash-resource","doc":"A section for declaring resource actions.\n\nAll manipulation of data through the underlying data layer happens through actions.\nThere are four types of action: `create`, `read`, `update`, and `destroy`. You may\nrecognize these from the acronym `CRUD`. You can have multiple actions of the same\ntype, as long as they have different names. This is the primary mechanism for customizing\nyour resources to conform to your business logic. It is normal and expected to have\nmultiple actions of each type in a large application.","ref":"dsl-ash-resource.html#actions"},{"type":"DSL","title":"actions.defaults - dsl-ash-resource","doc":"Creates a simple action of each specified type, with the same name as the type. These will be `primary?` unless one already exists for that type. Embedded resources, however, have a default of all resource types.","ref":"dsl-ash-resource.html#actions-defaults"},{"type":"DSL","title":"actions.default_accept - dsl-ash-resource","doc":"A default value for the `accept` option for each action. Use `:*` to accept all public attributes.","ref":"dsl-ash-resource.html#actions-default_accept"},{"type":"DSL","title":"actions.action - dsl-ash-resource","doc":"Declares a generic action. A combination of arguments, a return type and a run function.\n\nFor calling this action, see the `Ash.Domain` documentation.","ref":"dsl-ash-resource.html#actions-action"},{"type":"DSL","title":"actions.action.returns - dsl-ash-resource","doc":"The return type of the action. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-action-returns"},{"type":"DSL","title":"actions.action.constraints - dsl-ash-resource","doc":"Constraints for the return type. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-action-constraints"},{"type":"DSL","title":"actions.action.allow_nil? - dsl-ash-resource","doc":"Whether or not the action can return nil. Unlike attributes & arguments, this defaults to `false`.","ref":"dsl-ash-resource.html#actions-action-allow_nil?"},{"type":"DSL","title":"actions.action.run - dsl-ash-resource","doc":"Module may be an `Ash.Resource.Actions.Implementation` or `Reactor`.","ref":"dsl-ash-resource.html#actions-action-run"},{"type":"DSL","title":"actions.action.name - dsl-ash-resource","doc":"The name of the action","ref":"dsl-ash-resource.html#actions-action-name"},{"type":"DSL","title":"actions.action.primary? - dsl-ash-resource","doc":"Whether or not this action should be used when no action is specified by the caller.","ref":"dsl-ash-resource.html#actions-action-primary?"},{"type":"DSL","title":"actions.action.description - dsl-ash-resource","doc":"An optional description for the action","ref":"dsl-ash-resource.html#actions-action-description"},{"type":"DSL","title":"actions.action.transaction? - dsl-ash-resource","doc":"Whether or not the action should be run in transactions. Reads default to false, while create/update/destroy actions default to `true`.","ref":"dsl-ash-resource.html#actions-action-transaction?"},{"type":"DSL","title":"actions.action.touches_resources - dsl-ash-resource","doc":"A list of resources that the action may touch, used when building transactions.","ref":"dsl-ash-resource.html#actions-action-touches_resources"},{"type":"DSL","title":"actions.action.skip_unknown_inputs - dsl-ash-resource","doc":"A list of unknown fields to skip, or `:*` to skip all unknown fields.","ref":"dsl-ash-resource.html#actions-action-skip_unknown_inputs"},{"type":"DSL","title":"actions.action.argument - dsl-ash-resource","doc":"Declares an argument on the action","ref":"dsl-ash-resource.html#actions-action-argument"},{"type":"DSL","title":"actions.action.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#actions-action-argument-name"},{"type":"DSL","title":"actions.action.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-action-argument-type"},{"type":"DSL","title":"actions.action.argument.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#actions-action-argument-description"},{"type":"DSL","title":"actions.action.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#actions-action-argument-constraints"},{"type":"DSL","title":"actions.action.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#actions-action-argument-allow_nil?"},{"type":"DSL","title":"actions.action.argument.public? - dsl-ash-resource","doc":"Whether or not the argument should appear in public interfaces","ref":"dsl-ash-resource.html#actions-action-argument-public?"},{"type":"DSL","title":"actions.action.argument.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#actions-action-argument-sensitive?"},{"type":"DSL","title":"actions.action.argument.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-action-argument-default"},{"type":"DSL","title":"actions.create - dsl-ash-resource","doc":"Declares a `create` action. For calling this action, see the `Ash.Domain` documentation.","ref":"dsl-ash-resource.html#actions-create"},{"type":"DSL","title":"actions.create.manual - dsl-ash-resource","doc":"Override the creation behavior. Accepts a module or module and opts, or a function that takes the changeset and context. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-create-manual"},{"type":"DSL","title":"actions.create.upsert? - dsl-ash-resource","doc":"Forces all uses of this action to be treated as an upsert.","ref":"dsl-ash-resource.html#actions-create-upsert?"},{"type":"DSL","title":"actions.create.upsert_identity - dsl-ash-resource","doc":"The identity to use for the upsert. Cannot be overridden by the caller. Ignored  if `upsert?` is not set to `true`.","ref":"dsl-ash-resource.html#actions-create-upsert_identity"},{"type":"DSL","title":"actions.create.upsert_fields - dsl-ash-resource","doc":"The fields to overwrite in the case of an upsert. If not provided, all fields except for fields set by defaults will be overwritten.","ref":"dsl-ash-resource.html#actions-create-upsert_fields"},{"type":"DSL","title":"actions.create.upsert_condition - dsl-ash-resource","doc":"An expression to check if the record should be updated when there's a conflict.","ref":"dsl-ash-resource.html#actions-create-upsert_condition"},{"type":"DSL","title":"actions.create.return_skipped_upsert? - dsl-ash-resource","doc":"Returns the record that would have been upserted against but was skipped due to a filter or no fields being changed. How this works depends on the data layer. Keep in mind that read policies *are not applied* to the read of the record in question.","ref":"dsl-ash-resource.html#actions-create-return_skipped_upsert?"},{"type":"DSL","title":"actions.create.name - dsl-ash-resource","doc":"The name of the action","ref":"dsl-ash-resource.html#actions-create-name"},{"type":"DSL","title":"actions.create.primary? - dsl-ash-resource","doc":"Whether or not this action should be used when no action is specified by the caller.","ref":"dsl-ash-resource.html#actions-create-primary?"},{"type":"DSL","title":"actions.create.description - dsl-ash-resource","doc":"An optional description for the action","ref":"dsl-ash-resource.html#actions-create-description"},{"type":"DSL","title":"actions.create.transaction? - dsl-ash-resource","doc":"Whether or not the action should be run in transactions. Reads default to false, while create/update/destroy actions default to `true`.","ref":"dsl-ash-resource.html#actions-create-transaction?"},{"type":"DSL","title":"actions.create.touches_resources - dsl-ash-resource","doc":"A list of resources that the action may touch, used when building transactions.","ref":"dsl-ash-resource.html#actions-create-touches_resources"},{"type":"DSL","title":"actions.create.skip_unknown_inputs - dsl-ash-resource","doc":"A list of unknown fields to skip, or `:*` to skip all unknown fields.","ref":"dsl-ash-resource.html#actions-create-skip_unknown_inputs"},{"type":"DSL","title":"actions.create.accept - dsl-ash-resource","doc":"The list of attributes to accept. Use `:*` to accept all public attributes.","ref":"dsl-ash-resource.html#actions-create-accept"},{"type":"DSL","title":"actions.create.action_select - dsl-ash-resource","doc":"A list of attributes that the action requires to do its work. Defaults to all attributes except those with `select_by_default? false`. On actions with no changes/notifiers, it defaults to the externally selected attributes. Keep in mind that action_select is applied *before* notifiers.","ref":"dsl-ash-resource.html#actions-create-action_select"},{"type":"DSL","title":"actions.create.require_attributes - dsl-ash-resource","doc":"A list of attributes that would normally `allow_nil?`, to require for this action. No need to include attributes that already do not allow nil?","ref":"dsl-ash-resource.html#actions-create-require_attributes"},{"type":"DSL","title":"actions.create.allow_nil_input - dsl-ash-resource","doc":"A list of attributes that would normally be required, but should not be for this action. They will still be validated just before the data layer step.","ref":"dsl-ash-resource.html#actions-create-allow_nil_input"},{"type":"DSL","title":"actions.create.delay_global_validations? - dsl-ash-resource","doc":"If true, global validations will be done in a `before_action` hook, regardless of their configuration on the resource.","ref":"dsl-ash-resource.html#actions-create-delay_global_validations?"},{"type":"DSL","title":"actions.create.skip_global_validations? - dsl-ash-resource","doc":"If true, global validations will be skipped. Useful for manual actions.","ref":"dsl-ash-resource.html#actions-create-skip_global_validations?"},{"type":"DSL","title":"actions.create.error_handler - dsl-ash-resource","doc":"Sets the error handler on the changeset. See `Ash.Changeset.handle_errors/2` for more","ref":"dsl-ash-resource.html#actions-create-error_handler"},{"type":"DSL","title":"actions.create.notifiers - dsl-ash-resource","doc":"Notifiers that will be called specifically for this action.","ref":"dsl-ash-resource.html#actions-create-notifiers"},{"type":"DSL","title":"actions.create.manual? - dsl-ash-resource","doc":"Instructs Ash to *skip* the actual update/create/destroy step at the data layer. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-create-manual?"},{"type":"DSL","title":"actions.create.change - dsl-ash-resource","doc":"A change to be applied to the changeset.\n\nSee `Ash.Resource.Change` for more.","ref":"dsl-ash-resource.html#actions-create-change"},{"type":"DSL","title":"actions.create.change.only_when_valid? - dsl-ash-resource","doc":"If the change should only be run on valid changes. By default, all changes are run unless stated otherwise here.","ref":"dsl-ash-resource.html#actions-create-change-only_when_valid?"},{"type":"DSL","title":"actions.create.change.description - dsl-ash-resource","doc":"An optional description for the change","ref":"dsl-ash-resource.html#actions-create-change-description"},{"type":"DSL","title":"actions.create.change.change - dsl-ash-resource","doc":"The module and options for a change. Also accepts a function that takes the changeset and the context. See `Ash.Resource.Change.Builtins` for builtin changes.","ref":"dsl-ash-resource.html#actions-create-change-change"},{"type":"DSL","title":"actions.create.change.where - dsl-ash-resource","doc":"Validations that should pass in order for this change to apply. These validations failing will result in this change being ignored.","ref":"dsl-ash-resource.html#actions-create-change-where"},{"type":"DSL","title":"actions.create.change.always_atomic? - dsl-ash-resource","doc":"By default, changes are only run atomically if all changes will be run atomically or if there is no `change/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-create-change-always_atomic?"},{"type":"DSL","title":"actions.create.validate - dsl-ash-resource","doc":"Declares a validation to be applied to the changeset.\n\nSee `Ash.Resource.Validation.Builtins` or `Ash.Resource.Validation` for more.","ref":"dsl-ash-resource.html#actions-create-validate"},{"type":"DSL","title":"actions.create.validate.validation - dsl-ash-resource","doc":"The module (or module and opts) that implements the `Ash.Resource.Validation` behaviour. Also accepts a function that receives the changeset and its context.","ref":"dsl-ash-resource.html#actions-create-validate-validation"},{"type":"DSL","title":"actions.create.validate.where - dsl-ash-resource","doc":"Validations that should pass in order for this validation to apply. Any of these validations failing will result in this validation being ignored.","ref":"dsl-ash-resource.html#actions-create-validate-where"},{"type":"DSL","title":"actions.create.validate.only_when_valid? - dsl-ash-resource","doc":"If the validation should only run on valid changes. Useful for expensive validations or validations that depend on valid data.","ref":"dsl-ash-resource.html#actions-create-validate-only_when_valid?"},{"type":"DSL","title":"actions.create.validate.message - dsl-ash-resource","doc":"If provided, overrides any message set by the validation error","ref":"dsl-ash-resource.html#actions-create-validate-message"},{"type":"DSL","title":"actions.create.validate.description - dsl-ash-resource","doc":"An optional description for the validation","ref":"dsl-ash-resource.html#actions-create-validate-description"},{"type":"DSL","title":"actions.create.validate.before_action? - dsl-ash-resource","doc":"If set to `true`, the validation will be run in a before_action hook","ref":"dsl-ash-resource.html#actions-create-validate-before_action?"},{"type":"DSL","title":"actions.create.validate.always_atomic? - dsl-ash-resource","doc":"By default, validations are only run atomically if all changes will be run atomically or if there is no `validate/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-create-validate-always_atomic?"},{"type":"DSL","title":"actions.create.argument - dsl-ash-resource","doc":"Declares an argument on the action","ref":"dsl-ash-resource.html#actions-create-argument"},{"type":"DSL","title":"actions.create.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#actions-create-argument-name"},{"type":"DSL","title":"actions.create.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-create-argument-type"},{"type":"DSL","title":"actions.create.argument.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#actions-create-argument-description"},{"type":"DSL","title":"actions.create.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#actions-create-argument-constraints"},{"type":"DSL","title":"actions.create.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#actions-create-argument-allow_nil?"},{"type":"DSL","title":"actions.create.argument.public? - dsl-ash-resource","doc":"Whether or not the argument should appear in public interfaces","ref":"dsl-ash-resource.html#actions-create-argument-public?"},{"type":"DSL","title":"actions.create.argument.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#actions-create-argument-sensitive?"},{"type":"DSL","title":"actions.create.argument.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-create-argument-default"},{"type":"DSL","title":"actions.create.metadata - dsl-ash-resource","doc":"A special kind of attribute that is only added to specific actions. Nothing sets this value, it must be set in a custom\nchange after_action hook via `Ash.Resource.put_metadata/3`.","ref":"dsl-ash-resource.html#actions-create-metadata"},{"type":"DSL","title":"actions.create.metadata.name - dsl-ash-resource","doc":"The name of the metadata","ref":"dsl-ash-resource.html#actions-create-metadata-name"},{"type":"DSL","title":"actions.create.metadata.type - dsl-ash-resource","doc":"The type of the metadata. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-create-metadata-type"},{"type":"DSL","title":"actions.create.metadata.constraints - dsl-ash-resource","doc":"Type constraints on the metadata","ref":"dsl-ash-resource.html#actions-create-metadata-constraints"},{"type":"DSL","title":"actions.create.metadata.description - dsl-ash-resource","doc":"An optional description for the metadata.","ref":"dsl-ash-resource.html#actions-create-metadata-description"},{"type":"DSL","title":"actions.create.metadata.allow_nil? - dsl-ash-resource","doc":"Whether or not the metadata may return `nil`","ref":"dsl-ash-resource.html#actions-create-metadata-allow_nil?"},{"type":"DSL","title":"actions.create.metadata.default - dsl-ash-resource","doc":"The default value for the metadata to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-create-metadata-default"},{"type":"DSL","title":"actions.read - dsl-ash-resource","doc":"Declares a `read` action. For calling this action, see the `Ash.Domain` documentation.","ref":"dsl-ash-resource.html#actions-read"},{"type":"DSL","title":"actions.read.manual - dsl-ash-resource","doc":"Delegates running of the query to the provided module. Accepts a module or module and opts, or a function that takes the ash query, the data layer query, and context. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-read-manual"},{"type":"DSL","title":"actions.read.get? - dsl-ash-resource","doc":"Expresses that this action innately only returns a single result. Used by extensions to validate and/or modify behavior. Causes code interfaces to return a single value instead of a list. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#actions-read-get?"},{"type":"DSL","title":"actions.read.modify_query - dsl-ash-resource","doc":"Allows direct manipulation of the data layer query via an MFA. The ash query and the data layer query will be provided as additional arguments. The result must be `{:ok, new_data_layer_query} | {:error, error}`.","ref":"dsl-ash-resource.html#actions-read-modify_query"},{"type":"DSL","title":"actions.read.get_by - dsl-ash-resource","doc":"A helper to automatically generate a \"get by X\" action. Sets `get?` to true, add args for each of the specified fields, and adds a filter for each of the arguments.","ref":"dsl-ash-resource.html#actions-read-get_by"},{"type":"DSL","title":"actions.read.timeout - dsl-ash-resource","doc":"The maximum amount of time, in milliseconds, that the action is allowed to run for. Ignored if the data layer doesn't support transactions *and* async is disabled.","ref":"dsl-ash-resource.html#actions-read-timeout"},{"type":"DSL","title":"actions.read.multitenancy - dsl-ash-resource","doc":"This setting defines how this action handles multitenancy. `:enforce` requires a tenant to be set (the default behavior), `:allow_global` allows using this action both with and without a tenant, `:bypass` completely ignores the tenant even if it's set. This is useful to change the behaviour of selected read action without the need of marking the whole resource with `global? true`.","ref":"dsl-ash-resource.html#actions-read-multitenancy"},{"type":"DSL","title":"actions.read.name - dsl-ash-resource","doc":"The name of the action","ref":"dsl-ash-resource.html#actions-read-name"},{"type":"DSL","title":"actions.read.primary? - dsl-ash-resource","doc":"Whether or not this action should be used when no action is specified by the caller.","ref":"dsl-ash-resource.html#actions-read-primary?"},{"type":"DSL","title":"actions.read.description - dsl-ash-resource","doc":"An optional description for the action","ref":"dsl-ash-resource.html#actions-read-description"},{"type":"DSL","title":"actions.read.transaction? - dsl-ash-resource","doc":"Whether or not the action should be run in transactions. Reads default to false, while create/update/destroy actions default to `true`.","ref":"dsl-ash-resource.html#actions-read-transaction?"},{"type":"DSL","title":"actions.read.touches_resources - dsl-ash-resource","doc":"A list of resources that the action may touch, used when building transactions.","ref":"dsl-ash-resource.html#actions-read-touches_resources"},{"type":"DSL","title":"actions.read.skip_unknown_inputs - dsl-ash-resource","doc":"A list of unknown fields to skip, or `:*` to skip all unknown fields.","ref":"dsl-ash-resource.html#actions-read-skip_unknown_inputs"},{"type":"DSL","title":"actions.read.argument - dsl-ash-resource","doc":"Declares an argument on the action","ref":"dsl-ash-resource.html#actions-read-argument"},{"type":"DSL","title":"actions.read.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#actions-read-argument-name"},{"type":"DSL","title":"actions.read.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-read-argument-type"},{"type":"DSL","title":"actions.read.argument.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#actions-read-argument-description"},{"type":"DSL","title":"actions.read.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#actions-read-argument-constraints"},{"type":"DSL","title":"actions.read.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#actions-read-argument-allow_nil?"},{"type":"DSL","title":"actions.read.argument.public? - dsl-ash-resource","doc":"Whether or not the argument should appear in public interfaces","ref":"dsl-ash-resource.html#actions-read-argument-public?"},{"type":"DSL","title":"actions.read.argument.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#actions-read-argument-sensitive?"},{"type":"DSL","title":"actions.read.argument.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-read-argument-default"},{"type":"DSL","title":"actions.read.prepare - dsl-ash-resource","doc":"Declares a preparation, which can be used to prepare a query for a read action.","ref":"dsl-ash-resource.html#actions-read-prepare"},{"type":"DSL","title":"actions.read.prepare.preparation - dsl-ash-resource","doc":"The module and options for a preparation. Also accepts functions take the query and the context.","ref":"dsl-ash-resource.html#actions-read-prepare-preparation"},{"type":"DSL","title":"actions.read.pagination - dsl-ash-resource","doc":"Adds pagination options to a resource","ref":"dsl-ash-resource.html#actions-read-pagination"},{"type":"DSL","title":"actions.read.pagination.keyset? - dsl-ash-resource","doc":"Whether or not keyset based pagination is supported","ref":"dsl-ash-resource.html#actions-read-pagination-keyset?"},{"type":"DSL","title":"actions.read.pagination.offset? - dsl-ash-resource","doc":"Whether or not offset based pagination is supported","ref":"dsl-ash-resource.html#actions-read-pagination-offset?"},{"type":"DSL","title":"actions.read.pagination.default_limit - dsl-ash-resource","doc":"The default page size to apply, if one is not supplied","ref":"dsl-ash-resource.html#actions-read-pagination-default_limit"},{"type":"DSL","title":"actions.read.pagination.countable - dsl-ash-resource","doc":"Whether not a returned page will have a full count of all records. Use `:by_default` to do it automatically.","ref":"dsl-ash-resource.html#actions-read-pagination-countable"},{"type":"DSL","title":"actions.read.pagination.max_page_size - dsl-ash-resource","doc":"The maximum amount of records that can be requested in a single page","ref":"dsl-ash-resource.html#actions-read-pagination-max_page_size"},{"type":"DSL","title":"actions.read.pagination.stable_sort - dsl-ash-resource","doc":"A stable sort statement to add to a query (after any existing sorts). Only added if the sort does not already contain a stable sort (sorting on fields that uniquely identify a record). Defaults to the primary key.","ref":"dsl-ash-resource.html#actions-read-pagination-stable_sort"},{"type":"DSL","title":"actions.read.pagination.required? - dsl-ash-resource","doc":"Whether or not pagination can be disabled (by passing `page: false` to `Ash.Api.read!/2`, or by having `required?: false, default_limit: nil` set). Only relevant if some pagination configuration is supplied.","ref":"dsl-ash-resource.html#actions-read-pagination-required?"},{"type":"DSL","title":"actions.read.metadata - dsl-ash-resource","doc":"A special kind of attribute that is only added to specific actions. Nothing sets this value, it must be set in a custom\nchange after_action hook via `Ash.Resource.put_metadata/3`.","ref":"dsl-ash-resource.html#actions-read-metadata"},{"type":"DSL","title":"actions.read.metadata.name - dsl-ash-resource","doc":"The name of the metadata","ref":"dsl-ash-resource.html#actions-read-metadata-name"},{"type":"DSL","title":"actions.read.metadata.type - dsl-ash-resource","doc":"The type of the metadata. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-read-metadata-type"},{"type":"DSL","title":"actions.read.metadata.constraints - dsl-ash-resource","doc":"Type constraints on the metadata","ref":"dsl-ash-resource.html#actions-read-metadata-constraints"},{"type":"DSL","title":"actions.read.metadata.description - dsl-ash-resource","doc":"An optional description for the metadata.","ref":"dsl-ash-resource.html#actions-read-metadata-description"},{"type":"DSL","title":"actions.read.metadata.allow_nil? - dsl-ash-resource","doc":"Whether or not the metadata may return `nil`","ref":"dsl-ash-resource.html#actions-read-metadata-allow_nil?"},{"type":"DSL","title":"actions.read.metadata.default - dsl-ash-resource","doc":"The default value for the metadata to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-read-metadata-default"},{"type":"DSL","title":"actions.read.filter - dsl-ash-resource","doc":"Applies a filter. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#actions-read-filter"},{"type":"DSL","title":"actions.read.filter.filter - dsl-ash-resource","doc":"The filter to apply. Can use `^arg/1`, `^context/1` and `^actor/1` templates. Multiple filters are combined with *and*.","ref":"dsl-ash-resource.html#actions-read-filter-filter"},{"type":"DSL","title":"actions.update - dsl-ash-resource","doc":"Declares a `update` action. For calling this action, see the `Ash.Domain` documentation.","ref":"dsl-ash-resource.html#actions-update"},{"type":"DSL","title":"actions.update.manual - dsl-ash-resource","doc":"Override the update behavior. Accepts a module or module and opts, or a function that takes the changeset and context. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-update-manual"},{"type":"DSL","title":"actions.update.require_atomic? - dsl-ash-resource","doc":"Require that the update be atomic. This means that all changes and validations implement the `atomic` callback. See the guide on atomic updates for more.","ref":"dsl-ash-resource.html#actions-update-require_atomic?"},{"type":"DSL","title":"actions.update.atomic_upgrade? - dsl-ash-resource","doc":"If set to `true`, atomic upgrades will be performed. Ignored if `required_atomic?` is `true`. See the update actions guide for more.","ref":"dsl-ash-resource.html#actions-update-atomic_upgrade?"},{"type":"DSL","title":"actions.update.atomic_upgrade_with - dsl-ash-resource","doc":"Configure the read action used when performing atomic upgrades. Defaults to the primary read action.","ref":"dsl-ash-resource.html#actions-update-atomic_upgrade_with"},{"type":"DSL","title":"actions.update.name - dsl-ash-resource","doc":"The name of the action","ref":"dsl-ash-resource.html#actions-update-name"},{"type":"DSL","title":"actions.update.primary? - dsl-ash-resource","doc":"Whether or not this action should be used when no action is specified by the caller.","ref":"dsl-ash-resource.html#actions-update-primary?"},{"type":"DSL","title":"actions.update.description - dsl-ash-resource","doc":"An optional description for the action","ref":"dsl-ash-resource.html#actions-update-description"},{"type":"DSL","title":"actions.update.transaction? - dsl-ash-resource","doc":"Whether or not the action should be run in transactions. Reads default to false, while create/update/destroy actions default to `true`.","ref":"dsl-ash-resource.html#actions-update-transaction?"},{"type":"DSL","title":"actions.update.touches_resources - dsl-ash-resource","doc":"A list of resources that the action may touch, used when building transactions.","ref":"dsl-ash-resource.html#actions-update-touches_resources"},{"type":"DSL","title":"actions.update.skip_unknown_inputs - dsl-ash-resource","doc":"A list of unknown fields to skip, or `:*` to skip all unknown fields.","ref":"dsl-ash-resource.html#actions-update-skip_unknown_inputs"},{"type":"DSL","title":"actions.update.accept - dsl-ash-resource","doc":"The list of attributes to accept. Use `:*` to accept all public attributes.","ref":"dsl-ash-resource.html#actions-update-accept"},{"type":"DSL","title":"actions.update.action_select - dsl-ash-resource","doc":"A list of attributes that the action requires to do its work. Defaults to all attributes except those with `select_by_default? false`. On actions with no changes/notifiers, it defaults to the externally selected attributes. Keep in mind that action_select is applied *before* notifiers.","ref":"dsl-ash-resource.html#actions-update-action_select"},{"type":"DSL","title":"actions.update.require_attributes - dsl-ash-resource","doc":"A list of attributes that would normally `allow_nil?`, to require for this action. No need to include attributes that already do not allow nil?","ref":"dsl-ash-resource.html#actions-update-require_attributes"},{"type":"DSL","title":"actions.update.allow_nil_input - dsl-ash-resource","doc":"A list of attributes that would normally be required, but should not be for this action. They will still be validated just before the data layer step.","ref":"dsl-ash-resource.html#actions-update-allow_nil_input"},{"type":"DSL","title":"actions.update.delay_global_validations? - dsl-ash-resource","doc":"If true, global validations will be done in a `before_action` hook, regardless of their configuration on the resource.","ref":"dsl-ash-resource.html#actions-update-delay_global_validations?"},{"type":"DSL","title":"actions.update.skip_global_validations? - dsl-ash-resource","doc":"If true, global validations will be skipped. Useful for manual actions.","ref":"dsl-ash-resource.html#actions-update-skip_global_validations?"},{"type":"DSL","title":"actions.update.error_handler - dsl-ash-resource","doc":"Sets the error handler on the changeset. See `Ash.Changeset.handle_errors/2` for more","ref":"dsl-ash-resource.html#actions-update-error_handler"},{"type":"DSL","title":"actions.update.notifiers - dsl-ash-resource","doc":"Notifiers that will be called specifically for this action.","ref":"dsl-ash-resource.html#actions-update-notifiers"},{"type":"DSL","title":"actions.update.manual? - dsl-ash-resource","doc":"Instructs Ash to *skip* the actual update/create/destroy step at the data layer. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-update-manual?"},{"type":"DSL","title":"actions.update.change - dsl-ash-resource","doc":"A change to be applied to the changeset.\n\nSee `Ash.Resource.Change` for more.","ref":"dsl-ash-resource.html#actions-update-change"},{"type":"DSL","title":"actions.update.change.only_when_valid? - dsl-ash-resource","doc":"If the change should only be run on valid changes. By default, all changes are run unless stated otherwise here.","ref":"dsl-ash-resource.html#actions-update-change-only_when_valid?"},{"type":"DSL","title":"actions.update.change.description - dsl-ash-resource","doc":"An optional description for the change","ref":"dsl-ash-resource.html#actions-update-change-description"},{"type":"DSL","title":"actions.update.change.change - dsl-ash-resource","doc":"The module and options for a change. Also accepts a function that takes the changeset and the context. See `Ash.Resource.Change.Builtins` for builtin changes.","ref":"dsl-ash-resource.html#actions-update-change-change"},{"type":"DSL","title":"actions.update.change.where - dsl-ash-resource","doc":"Validations that should pass in order for this change to apply. These validations failing will result in this change being ignored.","ref":"dsl-ash-resource.html#actions-update-change-where"},{"type":"DSL","title":"actions.update.change.always_atomic? - dsl-ash-resource","doc":"By default, changes are only run atomically if all changes will be run atomically or if there is no `change/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-update-change-always_atomic?"},{"type":"DSL","title":"actions.update.validate - dsl-ash-resource","doc":"Declares a validation to be applied to the changeset.\n\nSee `Ash.Resource.Validation.Builtins` or `Ash.Resource.Validation` for more.","ref":"dsl-ash-resource.html#actions-update-validate"},{"type":"DSL","title":"actions.update.validate.validation - dsl-ash-resource","doc":"The module (or module and opts) that implements the `Ash.Resource.Validation` behaviour. Also accepts a function that receives the changeset and its context.","ref":"dsl-ash-resource.html#actions-update-validate-validation"},{"type":"DSL","title":"actions.update.validate.where - dsl-ash-resource","doc":"Validations that should pass in order for this validation to apply. Any of these validations failing will result in this validation being ignored.","ref":"dsl-ash-resource.html#actions-update-validate-where"},{"type":"DSL","title":"actions.update.validate.only_when_valid? - dsl-ash-resource","doc":"If the validation should only run on valid changes. Useful for expensive validations or validations that depend on valid data.","ref":"dsl-ash-resource.html#actions-update-validate-only_when_valid?"},{"type":"DSL","title":"actions.update.validate.message - dsl-ash-resource","doc":"If provided, overrides any message set by the validation error","ref":"dsl-ash-resource.html#actions-update-validate-message"},{"type":"DSL","title":"actions.update.validate.description - dsl-ash-resource","doc":"An optional description for the validation","ref":"dsl-ash-resource.html#actions-update-validate-description"},{"type":"DSL","title":"actions.update.validate.before_action? - dsl-ash-resource","doc":"If set to `true`, the validation will be run in a before_action hook","ref":"dsl-ash-resource.html#actions-update-validate-before_action?"},{"type":"DSL","title":"actions.update.validate.always_atomic? - dsl-ash-resource","doc":"By default, validations are only run atomically if all changes will be run atomically or if there is no `validate/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-update-validate-always_atomic?"},{"type":"DSL","title":"actions.update.metadata - dsl-ash-resource","doc":"A special kind of attribute that is only added to specific actions. Nothing sets this value, it must be set in a custom\nchange after_action hook via `Ash.Resource.put_metadata/3`.","ref":"dsl-ash-resource.html#actions-update-metadata"},{"type":"DSL","title":"actions.update.metadata.name - dsl-ash-resource","doc":"The name of the metadata","ref":"dsl-ash-resource.html#actions-update-metadata-name"},{"type":"DSL","title":"actions.update.metadata.type - dsl-ash-resource","doc":"The type of the metadata. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-update-metadata-type"},{"type":"DSL","title":"actions.update.metadata.constraints - dsl-ash-resource","doc":"Type constraints on the metadata","ref":"dsl-ash-resource.html#actions-update-metadata-constraints"},{"type":"DSL","title":"actions.update.metadata.description - dsl-ash-resource","doc":"An optional description for the metadata.","ref":"dsl-ash-resource.html#actions-update-metadata-description"},{"type":"DSL","title":"actions.update.metadata.allow_nil? - dsl-ash-resource","doc":"Whether or not the metadata may return `nil`","ref":"dsl-ash-resource.html#actions-update-metadata-allow_nil?"},{"type":"DSL","title":"actions.update.metadata.default - dsl-ash-resource","doc":"The default value for the metadata to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-update-metadata-default"},{"type":"DSL","title":"actions.update.argument - dsl-ash-resource","doc":"Declares an argument on the action","ref":"dsl-ash-resource.html#actions-update-argument"},{"type":"DSL","title":"actions.update.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#actions-update-argument-name"},{"type":"DSL","title":"actions.update.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-update-argument-type"},{"type":"DSL","title":"actions.update.argument.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#actions-update-argument-description"},{"type":"DSL","title":"actions.update.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#actions-update-argument-constraints"},{"type":"DSL","title":"actions.update.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#actions-update-argument-allow_nil?"},{"type":"DSL","title":"actions.update.argument.public? - dsl-ash-resource","doc":"Whether or not the argument should appear in public interfaces","ref":"dsl-ash-resource.html#actions-update-argument-public?"},{"type":"DSL","title":"actions.update.argument.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#actions-update-argument-sensitive?"},{"type":"DSL","title":"actions.update.argument.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-update-argument-default"},{"type":"DSL","title":"actions.destroy - dsl-ash-resource","doc":"Declares a `destroy` action. For calling this action, see the `Ash.Domain` documentation.","ref":"dsl-ash-resource.html#actions-destroy"},{"type":"DSL","title":"actions.destroy.soft? - dsl-ash-resource","doc":"If specified, the destroy action behaves as an update internally","ref":"dsl-ash-resource.html#actions-destroy-soft?"},{"type":"DSL","title":"actions.destroy.manual - dsl-ash-resource","doc":"Override the update behavior. Accepts a module or module and opts, or a function that takes the changeset and context. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-destroy-manual"},{"type":"DSL","title":"actions.destroy.require_atomic? - dsl-ash-resource","doc":"Require that the update be atomic. Only relevant if `soft?` is set to `true`. This means that all changes and validations implement the `atomic` callback. See the guide on atomic updates for more.","ref":"dsl-ash-resource.html#actions-destroy-require_atomic?"},{"type":"DSL","title":"actions.destroy.atomic_upgrade? - dsl-ash-resource","doc":"If set to `true`, atomic upgrades will be performed. See the update actions guide for more.","ref":"dsl-ash-resource.html#actions-destroy-atomic_upgrade?"},{"type":"DSL","title":"actions.destroy.atomic_upgrade_with - dsl-ash-resource","doc":"Configure the read action used when performing atomic upgrades. Defaults to the primary read action.","ref":"dsl-ash-resource.html#actions-destroy-atomic_upgrade_with"},{"type":"DSL","title":"actions.destroy.name - dsl-ash-resource","doc":"The name of the action","ref":"dsl-ash-resource.html#actions-destroy-name"},{"type":"DSL","title":"actions.destroy.primary? - dsl-ash-resource","doc":"Whether or not this action should be used when no action is specified by the caller.","ref":"dsl-ash-resource.html#actions-destroy-primary?"},{"type":"DSL","title":"actions.destroy.description - dsl-ash-resource","doc":"An optional description for the action","ref":"dsl-ash-resource.html#actions-destroy-description"},{"type":"DSL","title":"actions.destroy.transaction? - dsl-ash-resource","doc":"Whether or not the action should be run in transactions. Reads default to false, while create/update/destroy actions default to `true`.","ref":"dsl-ash-resource.html#actions-destroy-transaction?"},{"type":"DSL","title":"actions.destroy.touches_resources - dsl-ash-resource","doc":"A list of resources that the action may touch, used when building transactions.","ref":"dsl-ash-resource.html#actions-destroy-touches_resources"},{"type":"DSL","title":"actions.destroy.skip_unknown_inputs - dsl-ash-resource","doc":"A list of unknown fields to skip, or `:*` to skip all unknown fields.","ref":"dsl-ash-resource.html#actions-destroy-skip_unknown_inputs"},{"type":"DSL","title":"actions.destroy.accept - dsl-ash-resource","doc":"The list of attributes to accept. Use `:*` to accept all public attributes.","ref":"dsl-ash-resource.html#actions-destroy-accept"},{"type":"DSL","title":"actions.destroy.action_select - dsl-ash-resource","doc":"A list of attributes that the action requires to do its work. Defaults to all attributes except those with `select_by_default? false`. On actions with no changes/notifiers, it defaults to the externally selected attributes. Keep in mind that action_select is applied *before* notifiers.","ref":"dsl-ash-resource.html#actions-destroy-action_select"},{"type":"DSL","title":"actions.destroy.require_attributes - dsl-ash-resource","doc":"A list of attributes that would normally `allow_nil?`, to require for this action. No need to include attributes that already do not allow nil?","ref":"dsl-ash-resource.html#actions-destroy-require_attributes"},{"type":"DSL","title":"actions.destroy.allow_nil_input - dsl-ash-resource","doc":"A list of attributes that would normally be required, but should not be for this action. They will still be validated just before the data layer step.","ref":"dsl-ash-resource.html#actions-destroy-allow_nil_input"},{"type":"DSL","title":"actions.destroy.delay_global_validations? - dsl-ash-resource","doc":"If true, global validations will be done in a `before_action` hook, regardless of their configuration on the resource.","ref":"dsl-ash-resource.html#actions-destroy-delay_global_validations?"},{"type":"DSL","title":"actions.destroy.skip_global_validations? - dsl-ash-resource","doc":"If true, global validations will be skipped. Useful for manual actions.","ref":"dsl-ash-resource.html#actions-destroy-skip_global_validations?"},{"type":"DSL","title":"actions.destroy.error_handler - dsl-ash-resource","doc":"Sets the error handler on the changeset. See `Ash.Changeset.handle_errors/2` for more","ref":"dsl-ash-resource.html#actions-destroy-error_handler"},{"type":"DSL","title":"actions.destroy.notifiers - dsl-ash-resource","doc":"Notifiers that will be called specifically for this action.","ref":"dsl-ash-resource.html#actions-destroy-notifiers"},{"type":"DSL","title":"actions.destroy.manual? - dsl-ash-resource","doc":"Instructs Ash to *skip* the actual update/create/destroy step at the data layer. See the [manual actions guide](/documentation/topics/manual-actions.md) for more.","ref":"dsl-ash-resource.html#actions-destroy-manual?"},{"type":"DSL","title":"actions.destroy.change - dsl-ash-resource","doc":"A change to be applied to the changeset.\n\nSee `Ash.Resource.Change` for more.","ref":"dsl-ash-resource.html#actions-destroy-change"},{"type":"DSL","title":"actions.destroy.change.only_when_valid? - dsl-ash-resource","doc":"If the change should only be run on valid changes. By default, all changes are run unless stated otherwise here.","ref":"dsl-ash-resource.html#actions-destroy-change-only_when_valid?"},{"type":"DSL","title":"actions.destroy.change.description - dsl-ash-resource","doc":"An optional description for the change","ref":"dsl-ash-resource.html#actions-destroy-change-description"},{"type":"DSL","title":"actions.destroy.change.change - dsl-ash-resource","doc":"The module and options for a change. Also accepts a function that takes the changeset and the context. See `Ash.Resource.Change.Builtins` for builtin changes.","ref":"dsl-ash-resource.html#actions-destroy-change-change"},{"type":"DSL","title":"actions.destroy.change.where - dsl-ash-resource","doc":"Validations that should pass in order for this change to apply. These validations failing will result in this change being ignored.","ref":"dsl-ash-resource.html#actions-destroy-change-where"},{"type":"DSL","title":"actions.destroy.change.always_atomic? - dsl-ash-resource","doc":"By default, changes are only run atomically if all changes will be run atomically or if there is no `change/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-destroy-change-always_atomic?"},{"type":"DSL","title":"actions.destroy.validate - dsl-ash-resource","doc":"Declares a validation to be applied to the changeset.\n\nSee `Ash.Resource.Validation.Builtins` or `Ash.Resource.Validation` for more.","ref":"dsl-ash-resource.html#actions-destroy-validate"},{"type":"DSL","title":"actions.destroy.validate.validation - dsl-ash-resource","doc":"The module (or module and opts) that implements the `Ash.Resource.Validation` behaviour. Also accepts a function that receives the changeset and its context.","ref":"dsl-ash-resource.html#actions-destroy-validate-validation"},{"type":"DSL","title":"actions.destroy.validate.where - dsl-ash-resource","doc":"Validations that should pass in order for this validation to apply. Any of these validations failing will result in this validation being ignored.","ref":"dsl-ash-resource.html#actions-destroy-validate-where"},{"type":"DSL","title":"actions.destroy.validate.only_when_valid? - dsl-ash-resource","doc":"If the validation should only run on valid changes. Useful for expensive validations or validations that depend on valid data.","ref":"dsl-ash-resource.html#actions-destroy-validate-only_when_valid?"},{"type":"DSL","title":"actions.destroy.validate.message - dsl-ash-resource","doc":"If provided, overrides any message set by the validation error","ref":"dsl-ash-resource.html#actions-destroy-validate-message"},{"type":"DSL","title":"actions.destroy.validate.description - dsl-ash-resource","doc":"An optional description for the validation","ref":"dsl-ash-resource.html#actions-destroy-validate-description"},{"type":"DSL","title":"actions.destroy.validate.before_action? - dsl-ash-resource","doc":"If set to `true`, the validation will be run in a before_action hook","ref":"dsl-ash-resource.html#actions-destroy-validate-before_action?"},{"type":"DSL","title":"actions.destroy.validate.always_atomic? - dsl-ash-resource","doc":"By default, validations are only run atomically if all changes will be run atomically or if there is no `validate/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#actions-destroy-validate-always_atomic?"},{"type":"DSL","title":"actions.destroy.metadata - dsl-ash-resource","doc":"A special kind of attribute that is only added to specific actions. Nothing sets this value, it must be set in a custom\nchange after_action hook via `Ash.Resource.put_metadata/3`.","ref":"dsl-ash-resource.html#actions-destroy-metadata"},{"type":"DSL","title":"actions.destroy.metadata.name - dsl-ash-resource","doc":"The name of the metadata","ref":"dsl-ash-resource.html#actions-destroy-metadata-name"},{"type":"DSL","title":"actions.destroy.metadata.type - dsl-ash-resource","doc":"The type of the metadata. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-destroy-metadata-type"},{"type":"DSL","title":"actions.destroy.metadata.constraints - dsl-ash-resource","doc":"Type constraints on the metadata","ref":"dsl-ash-resource.html#actions-destroy-metadata-constraints"},{"type":"DSL","title":"actions.destroy.metadata.description - dsl-ash-resource","doc":"An optional description for the metadata.","ref":"dsl-ash-resource.html#actions-destroy-metadata-description"},{"type":"DSL","title":"actions.destroy.metadata.allow_nil? - dsl-ash-resource","doc":"Whether or not the metadata may return `nil`","ref":"dsl-ash-resource.html#actions-destroy-metadata-allow_nil?"},{"type":"DSL","title":"actions.destroy.metadata.default - dsl-ash-resource","doc":"The default value for the metadata to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-destroy-metadata-default"},{"type":"DSL","title":"actions.destroy.argument - dsl-ash-resource","doc":"Declares an argument on the action","ref":"dsl-ash-resource.html#actions-destroy-argument"},{"type":"DSL","title":"actions.destroy.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#actions-destroy-argument-name"},{"type":"DSL","title":"actions.destroy.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#actions-destroy-argument-type"},{"type":"DSL","title":"actions.destroy.argument.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#actions-destroy-argument-description"},{"type":"DSL","title":"actions.destroy.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#actions-destroy-argument-constraints"},{"type":"DSL","title":"actions.destroy.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#actions-destroy-argument-allow_nil?"},{"type":"DSL","title":"actions.destroy.argument.public? - dsl-ash-resource","doc":"Whether or not the argument should appear in public interfaces","ref":"dsl-ash-resource.html#actions-destroy-argument-public?"},{"type":"DSL","title":"actions.destroy.argument.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#actions-destroy-argument-sensitive?"},{"type":"DSL","title":"actions.destroy.argument.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#actions-destroy-argument-default"},{"type":"DSL","title":"code_interface - dsl-ash-resource","doc":"Functions that will be defined on the resource. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface"},{"type":"DSL","title":"code_interface.domain - dsl-ash-resource","doc":"Use the provided Domain instead of the resources configured domain when calling actions.","ref":"dsl-ash-resource.html#code_interface-domain"},{"type":"DSL","title":"code_interface.define? - dsl-ash-resource","doc":"Whether or not to define the code interface in the resource.","ref":"dsl-ash-resource.html#code_interface-define?"},{"type":"DSL","title":"code_interface.define - dsl-ash-resource","doc":"Defines a function with the corresponding name and arguments. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface-define"},{"type":"DSL","title":"code_interface.define.name - dsl-ash-resource","doc":"The name of the function that will be defined","ref":"dsl-ash-resource.html#code_interface-define-name"},{"type":"DSL","title":"code_interface.define.action - dsl-ash-resource","doc":"The name of the action that will be called. Defaults to the same name as the function.","ref":"dsl-ash-resource.html#code_interface-define-action"},{"type":"DSL","title":"code_interface.define.args - dsl-ash-resource","doc":"Map specific arguments to named inputs. Can provide any argument/attributes that the action allows.","ref":"dsl-ash-resource.html#code_interface-define-args"},{"type":"DSL","title":"code_interface.define.not_found_error? - dsl-ash-resource","doc":"If the action or interface is configured with `get?: true`, this determines whether or not an error is raised or `nil` is returned.","ref":"dsl-ash-resource.html#code_interface-define-not_found_error?"},{"type":"DSL","title":"code_interface.define.require_reference? - dsl-ash-resource","doc":"For update and destroy actions, require a resource or identifier to be passed in as the first argument. Not relevant for other action types.","ref":"dsl-ash-resource.html#code_interface-define-require_reference?"},{"type":"DSL","title":"code_interface.define.exclude_inputs - dsl-ash-resource","doc":"A list of action inputs to not accept in the defined interface","ref":"dsl-ash-resource.html#code_interface-define-exclude_inputs"},{"type":"DSL","title":"code_interface.define.get? - dsl-ash-resource","doc":"Expects to only receive a single result from a read action or a bulk update/destroy, and returns a single result instead of a list. Sets `require_reference?` to false automatically.","ref":"dsl-ash-resource.html#code_interface-define-get?"},{"type":"DSL","title":"code_interface.define.get_by - dsl-ash-resource","doc":"Takes a list of fields and adds those fields as arguments, which will then be used to filter. Sets `get?` to true and `require_reference?` to false automatically. Adds filters for read, update and destroy actions, replacing the `record` first argument.","ref":"dsl-ash-resource.html#code_interface-define-get_by"},{"type":"DSL","title":"code_interface.define.get_by_identity - dsl-ash-resource","doc":"Takes an identity, gets its field list, and performs the same logic as `get_by` with those fields. Adds filters for read, update and destroy actions, replacing the `record` first argument.","ref":"dsl-ash-resource.html#code_interface-define-get_by_identity"},{"type":"DSL","title":"code_interface.define.default_options - dsl-ash-resource","doc":"Default options to be merged with client-provided options. These can override domain or action defaults. `:load`, `:bulk_options`, and `:page` options will be deep merged.","ref":"dsl-ash-resource.html#code_interface-define-default_options"},{"type":"DSL","title":"code_interface.define.custom_input - dsl-ash-resource","doc":"Define or customize an input to the action.\n\nSee the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface-define-custom_input"},{"type":"DSL","title":"code_interface.define.custom_input.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#code_interface-define-custom_input-name"},{"type":"DSL","title":"code_interface.define.custom_input.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-type"},{"type":"DSL","title":"code_interface.define.custom_input.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-description"},{"type":"DSL","title":"code_interface.define.custom_input.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-constraints"},{"type":"DSL","title":"code_interface.define.custom_input.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-allow_nil?"},{"type":"DSL","title":"code_interface.define.custom_input.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-sensitive?"},{"type":"DSL","title":"code_interface.define.custom_input.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#code_interface-define-custom_input-default"},{"type":"DSL","title":"code_interface.define.custom_input.transform - dsl-ash-resource","doc":"A transformation to be applied to the custom input.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-transform"},{"type":"DSL","title":"code_interface.define.custom_input.transform.to - dsl-ash-resource","doc":"A key to rewrite the argument to. If the custom input is also a required positional argument, then the `to` is automatically added to the `exclude_inputs` list.","ref":"dsl-ash-resource.html#code_interface-define-custom_input-transform-to"},{"type":"DSL","title":"code_interface.define.custom_input.transform.using - dsl-ash-resource","doc":"A function to use to transform the value. Must return `value` or `nil`","ref":"dsl-ash-resource.html#code_interface-define-custom_input-transform-using"},{"type":"DSL","title":"code_interface.define_calculation - dsl-ash-resource","doc":"Defines a function with the corresponding name and arguments, that evaluates a calculation. Use `:_record` to take an instance of a record. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface-define_calculation"},{"type":"DSL","title":"code_interface.define_calculation.name - dsl-ash-resource","doc":"The name of the function that will be defined","ref":"dsl-ash-resource.html#code_interface-define_calculation-name"},{"type":"DSL","title":"code_interface.define_calculation.calculation - dsl-ash-resource","doc":"The name of the calculation that will be evaluated. Defaults to the same name as the function.","ref":"dsl-ash-resource.html#code_interface-define_calculation-calculation"},{"type":"DSL","title":"code_interface.define_calculation.exclude_inputs - dsl-ash-resource","doc":"A list of calculation inputs to not accept in the defined interface","ref":"dsl-ash-resource.html#code_interface-define_calculation-exclude_inputs"},{"type":"DSL","title":"code_interface.define_calculation.args - dsl-ash-resource","doc":"Supply field or argument values referenced by the calculation, in the form of :name, `{:arg, :name}` and/or `{:ref, :name}`. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface-define_calculation-args"},{"type":"DSL","title":"code_interface.define_calculation.custom_input - dsl-ash-resource","doc":"Define or customize an input to the action.\n\nSee the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-name"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-type"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.description - dsl-ash-resource","doc":"An optional description for the argument.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-description"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-constraints"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-allow_nil?"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.sensitive? - dsl-ash-resource","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-sensitive?"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.default - dsl-ash-resource","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-default"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.transform - dsl-ash-resource","doc":"A transformation to be applied to the custom input.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-transform"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.transform.to - dsl-ash-resource","doc":"A key to rewrite the argument to. If the custom input is also a required positional argument, then the `to` is automatically added to the `exclude_inputs` list.","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-transform-to"},{"type":"DSL","title":"code_interface.define_calculation.custom_input.transform.using - dsl-ash-resource","doc":"A function to use to transform the value. Must return `value` or `nil`","ref":"dsl-ash-resource.html#code_interface-define_calculation-custom_input-transform-using"},{"type":"DSL","title":"resource - dsl-ash-resource","doc":"General resource configuration","ref":"dsl-ash-resource.html#resource"},{"type":"DSL","title":"resource.description - dsl-ash-resource","doc":"A human readable description of the resource, to be used in generated documentation","ref":"dsl-ash-resource.html#resource-description"},{"type":"DSL","title":"resource.base_filter - dsl-ash-resource","doc":"A filter statement to be applied to any queries on the resource","ref":"dsl-ash-resource.html#resource-base_filter"},{"type":"DSL","title":"resource.default_context - dsl-ash-resource","doc":"Default context to apply to any queries/changesets generated for this resource.","ref":"dsl-ash-resource.html#resource-default_context"},{"type":"DSL","title":"resource.trace_name - dsl-ash-resource","doc":"The name to use in traces. Defaults to the short_name stringified. See the [monitoring guide](/documentation/topics/monitoring.md) for more.","ref":"dsl-ash-resource.html#resource-trace_name"},{"type":"DSL","title":"resource.short_name - dsl-ash-resource","doc":"A short identifier for the resource, which should be unique. See the [monitoring guide](/documentation/topics/monitoring.md) for more.","ref":"dsl-ash-resource.html#resource-short_name"},{"type":"DSL","title":"resource.inspect_private_fields? - dsl-ash-resource","doc":"Whether to include private fields in the inspect output. `show_inspect_fields` takes precedence over this option.","ref":"dsl-ash-resource.html#resource-inspect_private_fields?"},{"type":"DSL","title":"resource.hide_inspect_fields - dsl-ash-resource","doc":"A deny-list of fields to hide from the inspect output. Takes precedence over `show_inspect_fields`.","ref":"dsl-ash-resource.html#resource-hide_inspect_fields"},{"type":"DSL","title":"resource.show_inspect_fields - dsl-ash-resource","doc":"An allow-list of fields to show in the inspect output. Sensitive fields are _always_ hidden.","ref":"dsl-ash-resource.html#resource-show_inspect_fields"},{"type":"DSL","title":"resource.plural_name - dsl-ash-resource","doc":"A pluralized version of the resource short_name. May be used by generators or automated tooling.","ref":"dsl-ash-resource.html#resource-plural_name"},{"type":"DSL","title":"resource.require_primary_key? - dsl-ash-resource","doc":"Allow the resource to be used without any primary key fields. Warning: this option is experimental, and should not be used unless you know what you're doing.","ref":"dsl-ash-resource.html#resource-require_primary_key?"},{"type":"DSL","title":"identities - dsl-ash-resource","doc":"Unique identifiers for the resource","ref":"dsl-ash-resource.html#identities"},{"type":"DSL","title":"identities.identity - dsl-ash-resource","doc":"Represents a unique constraint on the resource.\n\nSee the [identities guide](/documentation/topics/resources/identities.md) for more.","ref":"dsl-ash-resource.html#identities-identity"},{"type":"DSL","title":"identities.identity.name - dsl-ash-resource","doc":"The name of the identity.","ref":"dsl-ash-resource.html#identities-identity-name"},{"type":"DSL","title":"identities.identity.keys - dsl-ash-resource","doc":"The names of the attributes that uniquely identify this resource.","ref":"dsl-ash-resource.html#identities-identity-keys"},{"type":"DSL","title":"identities.identity.where - dsl-ash-resource","doc":"A filter that expresses only matching records are unique on the provided keys. Ignored on embedded resources.","ref":"dsl-ash-resource.html#identities-identity-where"},{"type":"DSL","title":"identities.identity.nils_distinct? - dsl-ash-resource","doc":"Whether or not `nil` values are considered always distinct from eachother. `nil` values won't conflict with eachother unless you set this option to `false`.","ref":"dsl-ash-resource.html#identities-identity-nils_distinct?"},{"type":"DSL","title":"identities.identity.eager_check? - dsl-ash-resource","doc":"Whether or not this identity is validated to be unique at validation time.","ref":"dsl-ash-resource.html#identities-identity-eager_check?"},{"type":"DSL","title":"identities.identity.eager_check_with - dsl-ash-resource","doc":"Validates that the unique identity provided is unique at validation time, outside of any transactions, using the domain module provided. Will default to resource's domain.","ref":"dsl-ash-resource.html#identities-identity-eager_check_with"},{"type":"DSL","title":"identities.identity.pre_check? - dsl-ash-resource","doc":"Whether or not this identity is validated to be unique in a before_action hook.","ref":"dsl-ash-resource.html#identities-identity-pre_check?"},{"type":"DSL","title":"identities.identity.pre_check_with - dsl-ash-resource","doc":"Validates that the unique identity provided is unique in a before_action hook.","ref":"dsl-ash-resource.html#identities-identity-pre_check_with"},{"type":"DSL","title":"identities.identity.description - dsl-ash-resource","doc":"An optional description for the identity","ref":"dsl-ash-resource.html#identities-identity-description"},{"type":"DSL","title":"identities.identity.field_names - dsl-ash-resource","doc":"The field names to hold errors when unique identity is violated.","ref":"dsl-ash-resource.html#identities-identity-field_names"},{"type":"DSL","title":"identities.identity.message - dsl-ash-resource","doc":"An error message to use when the unique identity would be violated","ref":"dsl-ash-resource.html#identities-identity-message"},{"type":"DSL","title":"identities.identity.all_tenants? - dsl-ash-resource","doc":"Whether or not this identity is unique across all tenants. If the resource is not multitenant, has no effect.","ref":"dsl-ash-resource.html#identities-identity-all_tenants?"},{"type":"DSL","title":"changes - dsl-ash-resource","doc":"Declare changes that occur on create/update/destroy actions against the resource\n\nSee `Ash.Resource.Change` for more.","ref":"dsl-ash-resource.html#changes"},{"type":"DSL","title":"changes.change - dsl-ash-resource","doc":"A change to be applied to the changeset.\n\nSee `Ash.Resource.Change` for more.","ref":"dsl-ash-resource.html#changes-change"},{"type":"DSL","title":"changes.change.on - dsl-ash-resource","doc":"The action types the change should run on. Destroy actions are omitted by default as most changes don't make sense for a destroy.","ref":"dsl-ash-resource.html#changes-change-on"},{"type":"DSL","title":"changes.change.only_when_valid? - dsl-ash-resource","doc":"If the change should only be run on valid changes. By default, all changes are run unless stated otherwise here.","ref":"dsl-ash-resource.html#changes-change-only_when_valid?"},{"type":"DSL","title":"changes.change.description - dsl-ash-resource","doc":"An optional description for the change","ref":"dsl-ash-resource.html#changes-change-description"},{"type":"DSL","title":"changes.change.change - dsl-ash-resource","doc":"The module and options for a change. Also accepts a function that takes the changeset and the context. See `Ash.Resource.Change.Builtins` for builtin changes.","ref":"dsl-ash-resource.html#changes-change-change"},{"type":"DSL","title":"changes.change.where - dsl-ash-resource","doc":"Validations that should pass in order for this change to apply. These validations failing will result in this change being ignored.","ref":"dsl-ash-resource.html#changes-change-where"},{"type":"DSL","title":"changes.change.always_atomic? - dsl-ash-resource","doc":"By default, changes are only run atomically if all changes will be run atomically or if there is no `change/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#changes-change-always_atomic?"},{"type":"DSL","title":"preparations - dsl-ash-resource","doc":"Declare preparations that occur on all read actions for a given resource","ref":"dsl-ash-resource.html#preparations"},{"type":"DSL","title":"preparations.prepare - dsl-ash-resource","doc":"Declares a preparation, which can be used to prepare a query for a read action.","ref":"dsl-ash-resource.html#preparations-prepare"},{"type":"DSL","title":"preparations.prepare.preparation - dsl-ash-resource","doc":"The module and options for a preparation. Also accepts functions take the query and the context.","ref":"dsl-ash-resource.html#preparations-prepare-preparation"},{"type":"DSL","title":"validations - dsl-ash-resource","doc":"Declare validations prior to performing actions against the resource","ref":"dsl-ash-resource.html#validations"},{"type":"DSL","title":"validations.validate - dsl-ash-resource","doc":"Declares a validation for creates and updates.\n\nSee `Ash.Resource.Validation.Builtins` or `Ash.Resource.Validation` for more.","ref":"dsl-ash-resource.html#validations-validate"},{"type":"DSL","title":"validations.validate.validation - dsl-ash-resource","doc":"The module (or module and opts) that implements the `Ash.Resource.Validation` behaviour. Also accepts a function that receives the changeset and its context.","ref":"dsl-ash-resource.html#validations-validate-validation"},{"type":"DSL","title":"validations.validate.where - dsl-ash-resource","doc":"Validations that should pass in order for this validation to apply. Any of these validations failing will result in this validation being ignored.","ref":"dsl-ash-resource.html#validations-validate-where"},{"type":"DSL","title":"validations.validate.on - dsl-ash-resource","doc":"The action types the validation should run on. Many validations don't make sense in the context of deletion, so by default it is not included.","ref":"dsl-ash-resource.html#validations-validate-on"},{"type":"DSL","title":"validations.validate.only_when_valid? - dsl-ash-resource","doc":"If the validation should only run on valid changes. Useful for expensive validations or validations that depend on valid data.","ref":"dsl-ash-resource.html#validations-validate-only_when_valid?"},{"type":"DSL","title":"validations.validate.message - dsl-ash-resource","doc":"If provided, overrides any message set by the validation error","ref":"dsl-ash-resource.html#validations-validate-message"},{"type":"DSL","title":"validations.validate.description - dsl-ash-resource","doc":"An optional description for the validation","ref":"dsl-ash-resource.html#validations-validate-description"},{"type":"DSL","title":"validations.validate.before_action? - dsl-ash-resource","doc":"If set to `true`, the validation will be run in a before_action hook","ref":"dsl-ash-resource.html#validations-validate-before_action?"},{"type":"DSL","title":"validations.validate.always_atomic? - dsl-ash-resource","doc":"By default, validations are only run atomically if all changes will be run atomically or if there is no `validate/3` callback defined. Set this to `true` to run it atomically always.","ref":"dsl-ash-resource.html#validations-validate-always_atomic?"},{"type":"DSL","title":"aggregates - dsl-ash-resource","doc":"Declare named aggregates on the resource.\n\nThese are aggregates that can be loaded only by name using `Ash.Query.load/2`.\nThey are also available as top level fields on the resource.\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates"},{"type":"DSL","title":"aggregates.count - dsl-ash-resource","doc":"Declares a named count aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect the count)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-count"},{"type":"DSL","title":"aggregates.count.uniq? - dsl-ash-resource","doc":"Whether or not to count unique values only","ref":"dsl-ash-resource.html#aggregates-count-uniq?"},{"type":"DSL","title":"aggregates.count.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-count-name"},{"type":"DSL","title":"aggregates.count.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-count-read_action"},{"type":"DSL","title":"aggregates.count.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-count-relationship_path"},{"type":"DSL","title":"aggregates.count.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-count-field"},{"type":"DSL","title":"aggregates.count.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-count-filter"},{"type":"DSL","title":"aggregates.count.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-count-description"},{"type":"DSL","title":"aggregates.count.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-count-default"},{"type":"DSL","title":"aggregates.count.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-count-public?"},{"type":"DSL","title":"aggregates.count.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-count-filterable?"},{"type":"DSL","title":"aggregates.count.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-count-sortable?"},{"type":"DSL","title":"aggregates.count.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-count-sensitive?"},{"type":"DSL","title":"aggregates.count.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-count-authorize?"},{"type":"DSL","title":"aggregates.count.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-count-join_filter"},{"type":"DSL","title":"aggregates.count.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-count-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.count.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-count-join_filter-filter"},{"type":"DSL","title":"aggregates.exists - dsl-ash-resource","doc":"Declares a named `exists` aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect if something exists)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-exists"},{"type":"DSL","title":"aggregates.exists.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-exists-name"},{"type":"DSL","title":"aggregates.exists.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-exists-read_action"},{"type":"DSL","title":"aggregates.exists.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-exists-relationship_path"},{"type":"DSL","title":"aggregates.exists.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-exists-filter"},{"type":"DSL","title":"aggregates.exists.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-exists-description"},{"type":"DSL","title":"aggregates.exists.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-exists-default"},{"type":"DSL","title":"aggregates.exists.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-exists-public?"},{"type":"DSL","title":"aggregates.exists.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-exists-filterable?"},{"type":"DSL","title":"aggregates.exists.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-exists-sortable?"},{"type":"DSL","title":"aggregates.exists.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-exists-sensitive?"},{"type":"DSL","title":"aggregates.exists.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-exists-authorize?"},{"type":"DSL","title":"aggregates.exists.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-exists-join_filter"},{"type":"DSL","title":"aggregates.exists.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-exists-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.exists.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-exists-join_filter-filter"},{"type":"DSL","title":"aggregates.first - dsl-ash-resource","doc":"Declares a named `first` aggregate on the resource\n\nFirst aggregates return the first value of the related record\nthat matches. Supports both `filter` and `sort`.\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-first"},{"type":"DSL","title":"aggregates.first.include_nil? - dsl-ash-resource","doc":"Whether or not to include `nil` values in the aggregate. Only relevant for `list` and `first` aggregates.","ref":"dsl-ash-resource.html#aggregates-first-include_nil?"},{"type":"DSL","title":"aggregates.first.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-first-name"},{"type":"DSL","title":"aggregates.first.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-first-read_action"},{"type":"DSL","title":"aggregates.first.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-first-relationship_path"},{"type":"DSL","title":"aggregates.first.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-first-field"},{"type":"DSL","title":"aggregates.first.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-first-filter"},{"type":"DSL","title":"aggregates.first.sort - dsl-ash-resource","doc":"A sort to be applied to the aggregate","ref":"dsl-ash-resource.html#aggregates-first-sort"},{"type":"DSL","title":"aggregates.first.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-first-description"},{"type":"DSL","title":"aggregates.first.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-first-default"},{"type":"DSL","title":"aggregates.first.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-first-public?"},{"type":"DSL","title":"aggregates.first.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-first-filterable?"},{"type":"DSL","title":"aggregates.first.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-first-sortable?"},{"type":"DSL","title":"aggregates.first.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-first-sensitive?"},{"type":"DSL","title":"aggregates.first.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-first-authorize?"},{"type":"DSL","title":"aggregates.first.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-first-join_filter"},{"type":"DSL","title":"aggregates.first.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-first-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.first.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-first-join_filter-filter"},{"type":"DSL","title":"aggregates.sum - dsl-ash-resource","doc":"Declares a named `sum` aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect the sum)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-sum"},{"type":"DSL","title":"aggregates.sum.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-sum-name"},{"type":"DSL","title":"aggregates.sum.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-sum-read_action"},{"type":"DSL","title":"aggregates.sum.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-sum-relationship_path"},{"type":"DSL","title":"aggregates.sum.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-sum-field"},{"type":"DSL","title":"aggregates.sum.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-sum-filter"},{"type":"DSL","title":"aggregates.sum.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-sum-description"},{"type":"DSL","title":"aggregates.sum.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-sum-default"},{"type":"DSL","title":"aggregates.sum.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-sum-public?"},{"type":"DSL","title":"aggregates.sum.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-sum-filterable?"},{"type":"DSL","title":"aggregates.sum.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-sum-sortable?"},{"type":"DSL","title":"aggregates.sum.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-sum-sensitive?"},{"type":"DSL","title":"aggregates.sum.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-sum-authorize?"},{"type":"DSL","title":"aggregates.sum.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-sum-join_filter"},{"type":"DSL","title":"aggregates.sum.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-sum-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.sum.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-sum-join_filter-filter"},{"type":"DSL","title":"aggregates.list - dsl-ash-resource","doc":"Declares a named `list` aggregate on the resource.\n\nA list aggregate selects the list of all values for the given field\nand relationship combination.\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-list"},{"type":"DSL","title":"aggregates.list.include_nil? - dsl-ash-resource","doc":"Whether or not to include `nil` values in the aggregate. Only relevant for `list` and `first` aggregates.","ref":"dsl-ash-resource.html#aggregates-list-include_nil?"},{"type":"DSL","title":"aggregates.list.uniq? - dsl-ash-resource","doc":"Whether or not to count unique values only","ref":"dsl-ash-resource.html#aggregates-list-uniq?"},{"type":"DSL","title":"aggregates.list.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-list-name"},{"type":"DSL","title":"aggregates.list.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-list-read_action"},{"type":"DSL","title":"aggregates.list.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-list-relationship_path"},{"type":"DSL","title":"aggregates.list.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-list-field"},{"type":"DSL","title":"aggregates.list.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-list-filter"},{"type":"DSL","title":"aggregates.list.sort - dsl-ash-resource","doc":"A sort to be applied to the aggregate","ref":"dsl-ash-resource.html#aggregates-list-sort"},{"type":"DSL","title":"aggregates.list.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-list-description"},{"type":"DSL","title":"aggregates.list.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-list-default"},{"type":"DSL","title":"aggregates.list.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-list-public?"},{"type":"DSL","title":"aggregates.list.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-list-filterable?"},{"type":"DSL","title":"aggregates.list.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-list-sortable?"},{"type":"DSL","title":"aggregates.list.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-list-sensitive?"},{"type":"DSL","title":"aggregates.list.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-list-authorize?"},{"type":"DSL","title":"aggregates.list.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-list-join_filter"},{"type":"DSL","title":"aggregates.list.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-list-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.list.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-list-join_filter-filter"},{"type":"DSL","title":"aggregates.max - dsl-ash-resource","doc":"Declares a named `max` aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect the max)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-max"},{"type":"DSL","title":"aggregates.max.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-max-name"},{"type":"DSL","title":"aggregates.max.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-max-read_action"},{"type":"DSL","title":"aggregates.max.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-max-relationship_path"},{"type":"DSL","title":"aggregates.max.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-max-field"},{"type":"DSL","title":"aggregates.max.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-max-filter"},{"type":"DSL","title":"aggregates.max.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-max-description"},{"type":"DSL","title":"aggregates.max.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-max-default"},{"type":"DSL","title":"aggregates.max.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-max-public?"},{"type":"DSL","title":"aggregates.max.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-max-filterable?"},{"type":"DSL","title":"aggregates.max.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-max-sortable?"},{"type":"DSL","title":"aggregates.max.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-max-sensitive?"},{"type":"DSL","title":"aggregates.max.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-max-authorize?"},{"type":"DSL","title":"aggregates.max.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-max-join_filter"},{"type":"DSL","title":"aggregates.max.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-max-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.max.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-max-join_filter-filter"},{"type":"DSL","title":"aggregates.min - dsl-ash-resource","doc":"Declares a named `min` aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect the min)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-min"},{"type":"DSL","title":"aggregates.min.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-min-name"},{"type":"DSL","title":"aggregates.min.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-min-read_action"},{"type":"DSL","title":"aggregates.min.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-min-relationship_path"},{"type":"DSL","title":"aggregates.min.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-min-field"},{"type":"DSL","title":"aggregates.min.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-min-filter"},{"type":"DSL","title":"aggregates.min.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-min-description"},{"type":"DSL","title":"aggregates.min.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-min-default"},{"type":"DSL","title":"aggregates.min.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-min-public?"},{"type":"DSL","title":"aggregates.min.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-min-filterable?"},{"type":"DSL","title":"aggregates.min.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-min-sortable?"},{"type":"DSL","title":"aggregates.min.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-min-sensitive?"},{"type":"DSL","title":"aggregates.min.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-min-authorize?"},{"type":"DSL","title":"aggregates.min.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-min-join_filter"},{"type":"DSL","title":"aggregates.min.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-min-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.min.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-min-join_filter-filter"},{"type":"DSL","title":"aggregates.avg - dsl-ash-resource","doc":"Declares a named `avg` aggregate on the resource\n\nSupports `filter`, but not `sort` (because that wouldn't affect the avg)\n\nSee the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-avg"},{"type":"DSL","title":"aggregates.avg.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-avg-name"},{"type":"DSL","title":"aggregates.avg.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-avg-read_action"},{"type":"DSL","title":"aggregates.avg.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-avg-relationship_path"},{"type":"DSL","title":"aggregates.avg.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-avg-field"},{"type":"DSL","title":"aggregates.avg.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-avg-filter"},{"type":"DSL","title":"aggregates.avg.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-avg-description"},{"type":"DSL","title":"aggregates.avg.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-avg-default"},{"type":"DSL","title":"aggregates.avg.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-avg-public?"},{"type":"DSL","title":"aggregates.avg.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-avg-filterable?"},{"type":"DSL","title":"aggregates.avg.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-avg-sortable?"},{"type":"DSL","title":"aggregates.avg.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-avg-sensitive?"},{"type":"DSL","title":"aggregates.avg.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-avg-authorize?"},{"type":"DSL","title":"aggregates.avg.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-avg-join_filter"},{"type":"DSL","title":"aggregates.avg.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-avg-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.avg.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-avg-join_filter-filter"},{"type":"DSL","title":"aggregates.custom - dsl-ash-resource","doc":"Declares a named `custom` aggregate on the resource\n\nSupports `filter` and `sort`.\n\nCustom aggregates provide an `implementation` which must implement data layer specific callbacks.\n\nSee the relevant data layer documentation and the [aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"dsl-ash-resource.html#aggregates-custom"},{"type":"DSL","title":"aggregates.custom.implementation - dsl-ash-resource","doc":"The module that implements the relevant data layer callbacks","ref":"dsl-ash-resource.html#aggregates-custom-implementation"},{"type":"DSL","title":"aggregates.custom.type - dsl-ash-resource","doc":"The type of the value returned by the aggregate","ref":"dsl-ash-resource.html#aggregates-custom-type"},{"type":"DSL","title":"aggregates.custom.name - dsl-ash-resource","doc":"The field to place the aggregate in","ref":"dsl-ash-resource.html#aggregates-custom-name"},{"type":"DSL","title":"aggregates.custom.read_action - dsl-ash-resource","doc":"The read action to use when building the aggregate. Defaults to the primary read action. Keep in mind this action must not have any required arguments.","ref":"dsl-ash-resource.html#aggregates-custom-read_action"},{"type":"DSL","title":"aggregates.custom.relationship_path - dsl-ash-resource","doc":"The relationship or relationship path to use for the aggregate","ref":"dsl-ash-resource.html#aggregates-custom-relationship_path"},{"type":"DSL","title":"aggregates.custom.field - dsl-ash-resource","doc":"The field to aggregate. Defaults to the first field in the primary key of the resource","ref":"dsl-ash-resource.html#aggregates-custom-field"},{"type":"DSL","title":"aggregates.custom.filter - dsl-ash-resource","doc":"A filter to apply to the aggregate","ref":"dsl-ash-resource.html#aggregates-custom-filter"},{"type":"DSL","title":"aggregates.custom.sort - dsl-ash-resource","doc":"A sort to be applied to the aggregate","ref":"dsl-ash-resource.html#aggregates-custom-sort"},{"type":"DSL","title":"aggregates.custom.description - dsl-ash-resource","doc":"An optional description for the aggregate","ref":"dsl-ash-resource.html#aggregates-custom-description"},{"type":"DSL","title":"aggregates.custom.default - dsl-ash-resource","doc":"A default value to use in cases where nil would be used. Count defaults to `0`.","ref":"dsl-ash-resource.html#aggregates-custom-default"},{"type":"DSL","title":"aggregates.custom.public? - dsl-ash-resource","doc":"Whether or not the aggregate will appear in public interfaces","ref":"dsl-ash-resource.html#aggregates-custom-public?"},{"type":"DSL","title":"aggregates.custom.filterable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in filters.","ref":"dsl-ash-resource.html#aggregates-custom-filterable?"},{"type":"DSL","title":"aggregates.custom.sortable? - dsl-ash-resource","doc":"Whether or not the aggregate should be usable in sorts.","ref":"dsl-ash-resource.html#aggregates-custom-sortable?"},{"type":"DSL","title":"aggregates.custom.sensitive? - dsl-ash-resource","doc":"Whether or not the aggregate should be considered sensitive.","ref":"dsl-ash-resource.html#aggregates-custom-sensitive?"},{"type":"DSL","title":"aggregates.custom.authorize? - dsl-ash-resource","doc":"Whether or not the aggregate query should authorize based on the target action, if the parent query is authorized. Requires filter checks on the target action.","ref":"dsl-ash-resource.html#aggregates-custom-authorize?"},{"type":"DSL","title":"aggregates.custom.join_filter - dsl-ash-resource","doc":"Declares a join filter on an aggregate. See the aggregates guide for more.","ref":"dsl-ash-resource.html#aggregates-custom-join_filter"},{"type":"DSL","title":"aggregates.custom.join_filter.relationship_path - dsl-ash-resource","doc":"The relationship path on which to apply the join filter","ref":"dsl-ash-resource.html#aggregates-custom-join_filter-relationship_path"},{"type":"DSL","title":"aggregates.custom.join_filter.filter - dsl-ash-resource","doc":"The filter to apply. Can be an expression or a filter template.","ref":"dsl-ash-resource.html#aggregates-custom-join_filter-filter"},{"type":"DSL","title":"calculations - dsl-ash-resource","doc":"Declare named calculations on the resource.\n\nThese are calculations that can be loaded only by name using `Ash.Query.load/2`.\nThey are also available as top level fields on the resource.\n\nSee the [calculations guide](/documentation/topics/resources/calculations.md) for more.","ref":"dsl-ash-resource.html#calculations"},{"type":"DSL","title":"calculations.calculate - dsl-ash-resource","doc":"Declares a named calculation on the resource.\n\nTakes a module that must adopt the `Ash.Resource.Calculation` behaviour. See that module\nfor more information.\n\nTo ensure that the necessary fields are loaded:\n\n1.) Specifying the `load` option on a calculation in the resource.\n2.) Define a `load/3` callback in the calculation module\n3.) Set `always_select?` on the attribute in question\n\nSee the [calculations guide](/documentation/topics/resources/calculations.md) for more.","ref":"dsl-ash-resource.html#calculations-calculate"},{"type":"DSL","title":"calculations.calculate.name - dsl-ash-resource","doc":"The field name to use for the calculation value","ref":"dsl-ash-resource.html#calculations-calculate-name"},{"type":"DSL","title":"calculations.calculate.type - dsl-ash-resource","doc":"The type of the calculation. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#calculations-calculate-type"},{"type":"DSL","title":"calculations.calculate.async? - dsl-ash-resource","doc":"","ref":"dsl-ash-resource.html#calculations-calculate-async?"},{"type":"DSL","title":"calculations.calculate.constraints - dsl-ash-resource","doc":"Constraints to provide to the type. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#calculations-calculate-constraints"},{"type":"DSL","title":"calculations.calculate.calculation - dsl-ash-resource","doc":"The `module`, `{module, opts}` or `expr(...)` to use for the calculation. Also accepts a function that takes *a list of records* and the context, and produces a result for each record.","ref":"dsl-ash-resource.html#calculations-calculate-calculation"},{"type":"DSL","title":"calculations.calculate.description - dsl-ash-resource","doc":"An optional description for the calculation","ref":"dsl-ash-resource.html#calculations-calculate-description"},{"type":"DSL","title":"calculations.calculate.public? - dsl-ash-resource","doc":"Whether or not the calculation will appear in public interfaces.","ref":"dsl-ash-resource.html#calculations-calculate-public?"},{"type":"DSL","title":"calculations.calculate.sensitive? - dsl-ash-resource","doc":"Whether or not references to the calculation will be considered sensitive.","ref":"dsl-ash-resource.html#calculations-calculate-sensitive?"},{"type":"DSL","title":"calculations.calculate.load - dsl-ash-resource","doc":"A load statement to be applied if the calculation is used.","ref":"dsl-ash-resource.html#calculations-calculate-load"},{"type":"DSL","title":"calculations.calculate.allow_nil? - dsl-ash-resource","doc":"Whether or not the calculation can return nil.","ref":"dsl-ash-resource.html#calculations-calculate-allow_nil?"},{"type":"DSL","title":"calculations.calculate.filterable? - dsl-ash-resource","doc":"Whether or not the calculation should be usable in filters.","ref":"dsl-ash-resource.html#calculations-calculate-filterable?"},{"type":"DSL","title":"calculations.calculate.sortable? - dsl-ash-resource","doc":"Whether or not the calculation can be referenced in sorts.","ref":"dsl-ash-resource.html#calculations-calculate-sortable?"},{"type":"DSL","title":"calculations.calculate.argument - dsl-ash-resource","doc":"An argument to be passed into the calculation's arguments map\n\nSee the [calculations guide](/documentation/topics/resources/calculations.md) for more.","ref":"dsl-ash-resource.html#calculations-calculate-argument"},{"type":"DSL","title":"calculations.calculate.argument.name - dsl-ash-resource","doc":"The name of the argument","ref":"dsl-ash-resource.html#calculations-calculate-argument-name"},{"type":"DSL","title":"calculations.calculate.argument.type - dsl-ash-resource","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-resource.html#calculations-calculate-argument-type"},{"type":"DSL","title":"calculations.calculate.argument.default - dsl-ash-resource","doc":"A default value to use for the argument if not provided","ref":"dsl-ash-resource.html#calculations-calculate-argument-default"},{"type":"DSL","title":"calculations.calculate.argument.allow_nil? - dsl-ash-resource","doc":"Whether or not the argument value may be nil (or may be not provided)","ref":"dsl-ash-resource.html#calculations-calculate-argument-allow_nil?"},{"type":"DSL","title":"calculations.calculate.argument.allow_expr? - dsl-ash-resource","doc":"Allow passing expressions as argument values. Expressions cannot be type validated.","ref":"dsl-ash-resource.html#calculations-calculate-argument-allow_expr?"},{"type":"DSL","title":"calculations.calculate.argument.constraints - dsl-ash-resource","doc":"Constraints to provide to the type when casting the value. See the type's documentation and `Ash.Type` for more.","ref":"dsl-ash-resource.html#calculations-calculate-argument-constraints"},{"type":"DSL","title":"multitenancy - dsl-ash-resource","doc":"Options for configuring the multitenancy behavior of a resource.\n\nTo specify a tenant, use `Ash.Query.set_tenant/2` or\n`Ash.Changeset.set_tenant/2` before passing it to an operation.\n\nSee the [multitenancy guide](/documentation/topics/advanced/multitenancy.md)","ref":"dsl-ash-resource.html#multitenancy"},{"type":"DSL","title":"multitenancy.strategy - dsl-ash-resource","doc":"Determine if multitenancy is performed with attribute filters or using data layer features.","ref":"dsl-ash-resource.html#multitenancy-strategy"},{"type":"DSL","title":"multitenancy.attribute - dsl-ash-resource","doc":"If using the `attribute` strategy, the attribute to use, e.g `org_id`","ref":"dsl-ash-resource.html#multitenancy-attribute"},{"type":"DSL","title":"multitenancy.global? - dsl-ash-resource","doc":"Whether or not the data may be accessed without setting a tenant. For example, with attribute multitenancy, this allows accessing without filtering by the tenant attribute.","ref":"dsl-ash-resource.html#multitenancy-global?"},{"type":"DSL","title":"multitenancy.parse_attribute - dsl-ash-resource","doc":"An mfa ({module, function, args}) pointing to a function that takes a tenant and returns the attribute value","ref":"dsl-ash-resource.html#multitenancy-parse_attribute"},{"type":"DSL","title":"domain - dsl-ash-domain","doc":"General domain configuration","ref":"dsl-ash-domain.html#domain"},{"type":"DSL","title":"domain.description - dsl-ash-domain","doc":"A description for the domain.","ref":"dsl-ash-domain.html#domain-description"},{"type":"DSL","title":"resources - dsl-ash-domain","doc":"List the resources of this domain","ref":"dsl-ash-domain.html#resources"},{"type":"DSL","title":"resources.allow - dsl-ash-domain","doc":"Support a dynamic resource list by providing a callback that checks whether or not the resource should be allowed.","ref":"dsl-ash-domain.html#resources-allow"},{"type":"DSL","title":"resources.allow_unregistered? - dsl-ash-domain","doc":"Whether the domain will support only registered entries or not.","ref":"dsl-ash-domain.html#resources-allow_unregistered?"},{"type":"DSL","title":"resources.resource - dsl-ash-domain","doc":"A resource present in the domain","ref":"dsl-ash-domain.html#resources-resource"},{"type":"DSL","title":"resources.resource.resource - dsl-ash-domain","doc":"","ref":"dsl-ash-domain.html#resources-resource-resource"},{"type":"DSL","title":"resources.resource.define - dsl-ash-domain","doc":"Defines a function with the corresponding name and arguments. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define"},{"type":"DSL","title":"resources.resource.define.name - dsl-ash-domain","doc":"The name of the function that will be defined","ref":"dsl-ash-domain.html#resources-resource-define-name"},{"type":"DSL","title":"resources.resource.define.action - dsl-ash-domain","doc":"The name of the action that will be called. Defaults to the same name as the function.","ref":"dsl-ash-domain.html#resources-resource-define-action"},{"type":"DSL","title":"resources.resource.define.args - dsl-ash-domain","doc":"Map specific arguments to named inputs. Can provide any argument/attributes that the action allows.","ref":"dsl-ash-domain.html#resources-resource-define-args"},{"type":"DSL","title":"resources.resource.define.not_found_error? - dsl-ash-domain","doc":"If the action or interface is configured with `get?: true`, this determines whether or not an error is raised or `nil` is returned.","ref":"dsl-ash-domain.html#resources-resource-define-not_found_error?"},{"type":"DSL","title":"resources.resource.define.require_reference? - dsl-ash-domain","doc":"For update and destroy actions, require a resource or identifier to be passed in as the first argument. Not relevant for other action types.","ref":"dsl-ash-domain.html#resources-resource-define-require_reference?"},{"type":"DSL","title":"resources.resource.define.exclude_inputs - dsl-ash-domain","doc":"A list of action inputs to not accept in the defined interface","ref":"dsl-ash-domain.html#resources-resource-define-exclude_inputs"},{"type":"DSL","title":"resources.resource.define.get? - dsl-ash-domain","doc":"Expects to only receive a single result from a read action or a bulk update/destroy, and returns a single result instead of a list. Sets `require_reference?` to false automatically.","ref":"dsl-ash-domain.html#resources-resource-define-get?"},{"type":"DSL","title":"resources.resource.define.get_by - dsl-ash-domain","doc":"Takes a list of fields and adds those fields as arguments, which will then be used to filter. Sets `get?` to true and `require_reference?` to false automatically. Adds filters for read, update and destroy actions, replacing the `record` first argument.","ref":"dsl-ash-domain.html#resources-resource-define-get_by"},{"type":"DSL","title":"resources.resource.define.get_by_identity - dsl-ash-domain","doc":"Takes an identity, gets its field list, and performs the same logic as `get_by` with those fields. Adds filters for read, update and destroy actions, replacing the `record` first argument.","ref":"dsl-ash-domain.html#resources-resource-define-get_by_identity"},{"type":"DSL","title":"resources.resource.define.default_options - dsl-ash-domain","doc":"Default options to be merged with client-provided options. These can override domain or action defaults. `:load`, `:bulk_options`, and `:page` options will be deep merged.","ref":"dsl-ash-domain.html#resources-resource-define-default_options"},{"type":"DSL","title":"resources.resource.define.custom_input - dsl-ash-domain","doc":"Define or customize an input to the action.\n\nSee the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input"},{"type":"DSL","title":"resources.resource.define.custom_input.name - dsl-ash-domain","doc":"The name of the argument","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-name"},{"type":"DSL","title":"resources.resource.define.custom_input.type - dsl-ash-domain","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-type"},{"type":"DSL","title":"resources.resource.define.custom_input.description - dsl-ash-domain","doc":"An optional description for the argument.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-description"},{"type":"DSL","title":"resources.resource.define.custom_input.constraints - dsl-ash-domain","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-constraints"},{"type":"DSL","title":"resources.resource.define.custom_input.allow_nil? - dsl-ash-domain","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-allow_nil?"},{"type":"DSL","title":"resources.resource.define.custom_input.sensitive? - dsl-ash-domain","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-sensitive?"},{"type":"DSL","title":"resources.resource.define.custom_input.default - dsl-ash-domain","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-default"},{"type":"DSL","title":"resources.resource.define.custom_input.transform - dsl-ash-domain","doc":"A transformation to be applied to the custom input.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-transform"},{"type":"DSL","title":"resources.resource.define.custom_input.transform.to - dsl-ash-domain","doc":"A key to rewrite the argument to. If the custom input is also a required positional argument, then the `to` is automatically added to the `exclude_inputs` list.","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-transform-to"},{"type":"DSL","title":"resources.resource.define.custom_input.transform.using - dsl-ash-domain","doc":"A function to use to transform the value. Must return `value` or `nil`","ref":"dsl-ash-domain.html#resources-resource-define-custom_input-transform-using"},{"type":"DSL","title":"resources.resource.define_calculation - dsl-ash-domain","doc":"Defines a function with the corresponding name and arguments, that evaluates a calculation. Use `:_record` to take an instance of a record. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define_calculation"},{"type":"DSL","title":"resources.resource.define_calculation.name - dsl-ash-domain","doc":"The name of the function that will be defined","ref":"dsl-ash-domain.html#resources-resource-define_calculation-name"},{"type":"DSL","title":"resources.resource.define_calculation.calculation - dsl-ash-domain","doc":"The name of the calculation that will be evaluated. Defaults to the same name as the function.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-calculation"},{"type":"DSL","title":"resources.resource.define_calculation.exclude_inputs - dsl-ash-domain","doc":"A list of calculation inputs to not accept in the defined interface","ref":"dsl-ash-domain.html#resources-resource-define_calculation-exclude_inputs"},{"type":"DSL","title":"resources.resource.define_calculation.args - dsl-ash-domain","doc":"Supply field or argument values referenced by the calculation, in the form of :name, `{:arg, :name}` and/or `{:ref, :name}`. See the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-args"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input - dsl-ash-domain","doc":"Define or customize an input to the action.\n\nSee the [code interface guide](/documentation/topics/resources/code-interfaces.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.name - dsl-ash-domain","doc":"The name of the argument","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-name"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.type - dsl-ash-domain","doc":"The type of the argument. See `Ash.Type` for more.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-type"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.description - dsl-ash-domain","doc":"An optional description for the argument.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-description"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.constraints - dsl-ash-domain","doc":"Constraints to provide to the type when casting the value. For more information, see `Ash.Type`.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-constraints"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.allow_nil? - dsl-ash-domain","doc":"Whether or not the argument value may be nil (or may be not provided). If nil value is given error is raised.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-allow_nil?"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.sensitive? - dsl-ash-domain","doc":"Whether or not the argument value contains sensitive information, like PII(Personally Identifiable Information). See the [security guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-sensitive?"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.default - dsl-ash-domain","doc":"The default value for the argument to take. It can be a zero argument function e.g `&MyMod.my_fun/0` or a value","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-default"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.transform - dsl-ash-domain","doc":"A transformation to be applied to the custom input.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-transform"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.transform.to - dsl-ash-domain","doc":"A key to rewrite the argument to. If the custom input is also a required positional argument, then the `to` is automatically added to the `exclude_inputs` list.","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-transform-to"},{"type":"DSL","title":"resources.resource.define_calculation.custom_input.transform.using - dsl-ash-domain","doc":"A function to use to transform the value. Must return `value` or `nil`","ref":"dsl-ash-domain.html#resources-resource-define_calculation-custom_input-transform-using"},{"type":"DSL","title":"execution - dsl-ash-domain","doc":"Options for how requests are executed using this domain","ref":"dsl-ash-domain.html#execution"},{"type":"DSL","title":"execution.timeout - dsl-ash-domain","doc":"The default timeout in milliseconds to use for requests using this domain. See the [timeouts guide](/documentation/topics/timeouts.md) for more.","ref":"dsl-ash-domain.html#execution-timeout"},{"type":"DSL","title":"execution.trace_name - dsl-ash-domain","doc":"The name to use in traces. Defaults to the last part of the module. See the [monitoring guide](/documentation/topics/monitoring.md) for more","ref":"dsl-ash-domain.html#execution-trace_name"},{"type":"DSL","title":"authorization - dsl-ash-domain","doc":"Options for how requests are authorized using this domain. See the [Sensitive Data guide](/documentation/topics/security/sensitive-data.md) for more.","ref":"dsl-ash-domain.html#authorization"},{"type":"DSL","title":"authorization.require_actor? - dsl-ash-domain","doc":"Requires that an actor has been supplied.","ref":"dsl-ash-domain.html#authorization-require_actor?"},{"type":"DSL","title":"authorization.authorize - dsl-ash-domain","doc":"When to run authorization for a given request.","ref":"dsl-ash-domain.html#authorization-authorize"},{"type":"DSL","title":"pub_sub - dsl-ash-notifier-pubsub","doc":"A section for configuring how resource actions are published over pubsub","ref":"dsl-ash-notifier-pubsub.html#pub_sub"},{"type":"DSL","title":"pub_sub.module - dsl-ash-notifier-pubsub","doc":"The module to call `broadcast/3` on e.g module.broadcast(topic, event, message).","ref":"dsl-ash-notifier-pubsub.html#pub_sub-module"},{"type":"DSL","title":"pub_sub.prefix - dsl-ash-notifier-pubsub","doc":"A prefix for all pubsub messages, e.g `users`. A message with `created` would be published as `users:created`","ref":"dsl-ash-notifier-pubsub.html#pub_sub-prefix"},{"type":"DSL","title":"pub_sub.delimiter - dsl-ash-notifier-pubsub","doc":"A delimiter for building topics. Default is a colon (:)","ref":"dsl-ash-notifier-pubsub.html#pub_sub-delimiter"},{"type":"DSL","title":"pub_sub.filter - dsl-ash-notifier-pubsub","doc":"A filter for notifications. Receives a notification, and ignores it if the function returns a falsy value. Both this and filters on specific publications must return a truthy value for a notification to be emitted.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-filter"},{"type":"DSL","title":"pub_sub.transform - dsl-ash-notifier-pubsub","doc":"A transformer for notifications. Specific transformers on each publication *override* this option","ref":"dsl-ash-notifier-pubsub.html#pub_sub-transform"},{"type":"DSL","title":"pub_sub.broadcast_type - dsl-ash-notifier-pubsub","doc":"What shape the event payloads will be in. See","ref":"dsl-ash-notifier-pubsub.html#pub_sub-broadcast_type"},{"type":"DSL","title":"pub_sub.name - dsl-ash-notifier-pubsub","doc":"A named pub sub to pass as the first argument to broadcast.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-name"},{"type":"DSL","title":"pub_sub.publish - dsl-ash-notifier-pubsub","doc":"Configure a given action to publish its results over a given topic.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish"},{"type":"DSL","title":"pub_sub.publish.action - dsl-ash-notifier-pubsub","doc":"The name of the action that should be published","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-action"},{"type":"DSL","title":"pub_sub.publish.previous_values? - dsl-ash-notifier-pubsub","doc":"Whether or not to publish messages with both the new values and the old values for referencing changed attributes","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-previous_values?"},{"type":"DSL","title":"pub_sub.publish.filter - dsl-ash-notifier-pubsub","doc":"A filter for notifications. Receives a notification, and ignores it if the function returns a falsy value.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-filter"},{"type":"DSL","title":"pub_sub.publish.transform - dsl-ash-notifier-pubsub","doc":"A transformer for notifications. Receives a notification, and returns a new value to be broadcasted.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-transform"},{"type":"DSL","title":"pub_sub.publish.topic - dsl-ash-notifier-pubsub","doc":"The topic to publish","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-topic"},{"type":"DSL","title":"pub_sub.publish.event - dsl-ash-notifier-pubsub","doc":"The name of the event to publish. Defaults to the action name","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-event"},{"type":"DSL","title":"pub_sub.publish.dispatcher - dsl-ash-notifier-pubsub","doc":"The module to use as a dispatcher. If none is set, the pubsub module provided is used.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish-dispatcher"},{"type":"DSL","title":"pub_sub.publish_all - dsl-ash-notifier-pubsub","doc":"Works the same as `publish`, except that it takes a type and publishes all actions of that type.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all"},{"type":"DSL","title":"pub_sub.publish_all.except - dsl-ash-notifier-pubsub","doc":"Exclude these actions from notifications","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-except"},{"type":"DSL","title":"pub_sub.publish_all.type - dsl-ash-notifier-pubsub","doc":"Publish on all actions of a given type","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-type"},{"type":"DSL","title":"pub_sub.publish_all.action - dsl-ash-notifier-pubsub","doc":"The name of the action that should be published","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-action"},{"type":"DSL","title":"pub_sub.publish_all.previous_values? - dsl-ash-notifier-pubsub","doc":"Whether or not to publish messages with both the new values and the old values for referencing changed attributes","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-previous_values?"},{"type":"DSL","title":"pub_sub.publish_all.filter - dsl-ash-notifier-pubsub","doc":"A filter for notifications. Receives a notification, and ignores it if the function returns a falsy value.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-filter"},{"type":"DSL","title":"pub_sub.publish_all.transform - dsl-ash-notifier-pubsub","doc":"A transformer for notifications. Receives a notification, and returns a new value to be broadcasted.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-transform"},{"type":"DSL","title":"pub_sub.publish_all.topic - dsl-ash-notifier-pubsub","doc":"The topic to publish","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-topic"},{"type":"DSL","title":"pub_sub.publish_all.event - dsl-ash-notifier-pubsub","doc":"The name of the event to publish. Defaults to the action name","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-event"},{"type":"DSL","title":"pub_sub.publish_all.dispatcher - dsl-ash-notifier-pubsub","doc":"The module to use as a dispatcher. If none is set, the pubsub module provided is used.","ref":"dsl-ash-notifier-pubsub.html#pub_sub-publish_all-dispatcher"},{"type":"DSL","title":"policies - dsl-ash-policy-authorizer","doc":"A section for declaring authorization policies.\n\nEach policy that applies must pass independently in order for the\nrequest to be authorized.\n\nSee the [policies guide](/documentation/topics/security/policies.md) for more.","ref":"dsl-ash-policy-authorizer.html#policies"},{"type":"DSL","title":"policies.default_access_type - dsl-ash-policy-authorizer","doc":"The default access type of policies for this resource.","ref":"dsl-ash-policy-authorizer.html#policies-default_access_type"},{"type":"DSL","title":"policies.policy - dsl-ash-policy-authorizer","doc":"A policy has a name, a condition, and a list of checks.\n\nChecks apply logically in the order they are specified, from top to bottom.\nIf no check explicitly authorizes the request, then the request is forbidden.\nThis means that, if you want to \"blacklist\" instead of \"whitelist\", you likely\nwant to add an `authorize_if always()` at the bottom of your policy, like so:\n\n```elixir\npolicy action_type(:read) do\n  forbid_if not_logged_in()\n  forbid_if user_is_denylisted()\n  forbid_if user_is_in_denylisted_group()\n\n  authorize_if always()\nend\n```\n\nIf the policy should always run, use the `always()` check, like so:\n\n```elixir\npolicy always() do\n  ...\nend\n```\n\nSee the [policies guide](/documentation/topics/security/policies.md) for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy"},{"type":"DSL","title":"policies.policy.description - dsl-ash-policy-authorizer","doc":"A description for the policy, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy-description"},{"type":"DSL","title":"policies.policy.access_type - dsl-ash-policy-authorizer","doc":"Determines how the policy is applied. See the guide for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy-access_type"},{"type":"DSL","title":"policies.policy.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this policy to apply.","ref":"dsl-ash-policy-authorizer.html#policies-policy-condition"},{"type":"DSL","title":"policies.policy.authorize_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is authorized, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_if"},{"type":"DSL","title":"policies.policy.authorize_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_if-check"},{"type":"DSL","title":"policies.policy.authorize_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_if-name"},{"type":"DSL","title":"policies.policy.forbid_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is forbidden, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_if"},{"type":"DSL","title":"policies.policy.forbid_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_if-check"},{"type":"DSL","title":"policies.policy.forbid_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_if-name"},{"type":"DSL","title":"policies.policy.authorize_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is authorized.","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_unless"},{"type":"DSL","title":"policies.policy.authorize_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_unless-check"},{"type":"DSL","title":"policies.policy.authorize_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy-authorize_unless-name"},{"type":"DSL","title":"policies.policy.forbid_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is forbidden.","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_unless"},{"type":"DSL","title":"policies.policy.forbid_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_unless-check"},{"type":"DSL","title":"policies.policy.forbid_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy-forbid_unless-name"},{"type":"DSL","title":"policies.policy_group - dsl-ash-policy-authorizer","doc":"Groups a set of policies together by some condition.\n\nIf the condition on the policy group does not apply, then none of the policies within it apply.\n\nThis is primarily syntactic sugar. At compile time, the conditions from the policy group are\nadded to each policy it contains, and the list is flattened out. This exists primarily to make it\neasier to reason about and write policies.\n\nThe following are equivalent:\n\n```elixir\npolicy_group condition1 do\n  policy condition2 do\n    ...\n  end\n\n  policy condition3 do\n    ...\n  end\nend\n```\n\nand\n\n```elixir\npolicy [condition1, condition2] do\n  ...\nend\n\npolicy [condition1, condition3] do\n  ...\nend\n```","ref":"dsl-ash-policy-authorizer.html#policies-policy_group"},{"type":"DSL","title":"policies.policy_group.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this policy to apply.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-condition"},{"type":"DSL","title":"policies.policy_group.policy - dsl-ash-policy-authorizer","doc":"A policy has a name, a condition, and a list of checks.\n\nChecks apply logically in the order they are specified, from top to bottom.\nIf no check explicitly authorizes the request, then the request is forbidden.\nThis means that, if you want to \"blacklist\" instead of \"whitelist\", you likely\nwant to add an `authorize_if always()` at the bottom of your policy, like so:\n\n```elixir\npolicy action_type(:read) do\n  forbid_if not_logged_in()\n  forbid_if user_is_denylisted()\n  forbid_if user_is_in_denylisted_group()\n\n  authorize_if always()\nend\n```\n\nIf the policy should always run, use the `always()` check, like so:\n\n```elixir\npolicy always() do\n  ...\nend\n```\n\nSee the [policies guide](/documentation/topics/security/policies.md) for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy"},{"type":"DSL","title":"policies.policy_group.policy.description - dsl-ash-policy-authorizer","doc":"A description for the policy, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-description"},{"type":"DSL","title":"policies.policy_group.policy.access_type - dsl-ash-policy-authorizer","doc":"Determines how the policy is applied. See the guide for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-access_type"},{"type":"DSL","title":"policies.policy_group.policy.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this policy to apply.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-condition"},{"type":"DSL","title":"policies.policy_group.policy.authorize_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is authorized, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_if"},{"type":"DSL","title":"policies.policy_group.policy.authorize_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_if-check"},{"type":"DSL","title":"policies.policy_group.policy.authorize_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_if-name"},{"type":"DSL","title":"policies.policy_group.policy.forbid_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is forbidden, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_if"},{"type":"DSL","title":"policies.policy_group.policy.forbid_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_if-check"},{"type":"DSL","title":"policies.policy_group.policy.forbid_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_if-name"},{"type":"DSL","title":"policies.policy_group.policy.authorize_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is authorized.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_unless"},{"type":"DSL","title":"policies.policy_group.policy.authorize_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_unless-check"},{"type":"DSL","title":"policies.policy_group.policy.authorize_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-authorize_unless-name"},{"type":"DSL","title":"policies.policy_group.policy.forbid_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is forbidden.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_unless"},{"type":"DSL","title":"policies.policy_group.policy.forbid_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_unless-check"},{"type":"DSL","title":"policies.policy_group.policy.forbid_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-policy_group-policy-forbid_unless-name"},{"type":"DSL","title":"policies.bypass - dsl-ash-policy-authorizer","doc":"A policy that, if passed, will skip all following policies. If failed, authorization moves on to the next policy","ref":"dsl-ash-policy-authorizer.html#policies-bypass"},{"type":"DSL","title":"policies.bypass.description - dsl-ash-policy-authorizer","doc":"A description for the policy, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-bypass-description"},{"type":"DSL","title":"policies.bypass.access_type - dsl-ash-policy-authorizer","doc":"Determines how the policy is applied. See the guide for more.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-access_type"},{"type":"DSL","title":"policies.bypass.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this policy to apply.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-condition"},{"type":"DSL","title":"policies.bypass.authorize_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is authorized, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_if"},{"type":"DSL","title":"policies.bypass.authorize_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_if-check"},{"type":"DSL","title":"policies.bypass.authorize_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_if-name"},{"type":"DSL","title":"policies.bypass.forbid_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is forbidden, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_if"},{"type":"DSL","title":"policies.bypass.forbid_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_if-check"},{"type":"DSL","title":"policies.bypass.forbid_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_if-name"},{"type":"DSL","title":"policies.bypass.authorize_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is authorized.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_unless"},{"type":"DSL","title":"policies.bypass.authorize_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_unless-check"},{"type":"DSL","title":"policies.bypass.authorize_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-bypass-authorize_unless-name"},{"type":"DSL","title":"policies.bypass.forbid_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is forbidden.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_unless"},{"type":"DSL","title":"policies.bypass.forbid_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_unless-check"},{"type":"DSL","title":"policies.bypass.forbid_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#policies-bypass-forbid_unless-name"},{"type":"DSL","title":"field_policies - dsl-ash-policy-authorizer","doc":"Authorize access to specific fields via policies scoped to fields.\n\nIf *any* field policies exist then *all* fields must be authorized by a field policy.\nIf you want a \"deny-list\" style, then you can add policies for specific fields\nand add a catch-all policy using the special field name `:*`. All policies that apply\nto a field must be authorized.\n\nThe only exception to the above behavior is primary keys, which can always be read by everyone.\n\nAdditionally, keep in mind that adding `Ash.Policy.Authorizer` will require that all actions\npass policies. If you want to just add field policies, you will need to add a policy that allows\nall access explicitly, i.e\n\n```elixir\npolicies do\n  policy always() do\n    authorize_if always()\n  end\nend\n```\n\nUsing expressions: unlike in regular policies, expressions in field policies cannot refer\nto related entities currently. Instead, you will need to create aggregates or expression calculations\nthat return the results you want to reference.\n\nIn results, forbidden fields will be replaced with a special value: `%Ash.ForbiddenField{}`.\n\nWhen these fields are referred to in filters, they will be replaced with an expression that evaluates\nto `nil`. To support this behavior, only expression/filter checks are allowed in field policies.","ref":"dsl-ash-policy-authorizer.html#field_policies"},{"type":"DSL","title":"field_policies.private_fields - dsl-ash-policy-authorizer","doc":"How private fields should be handled by field policies in internal functions. See the [Policies guide](documentation/topics/security/policies.md#field-policies) for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-private_fields"},{"type":"DSL","title":"field_policies.field_policy_bypass - dsl-ash-policy-authorizer","doc":"A field policy that, if passed, will skip all following field policies for that field or fields. If failed, field authorization moves on to the next policy","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass"},{"type":"DSL","title":"field_policies.field_policy_bypass.description - dsl-ash-policy-authorizer","doc":"A description for the policy, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-description"},{"type":"DSL","title":"field_policies.field_policy_bypass.fields - dsl-ash-policy-authorizer","doc":"The field or fields that the policy applies to.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-fields"},{"type":"DSL","title":"field_policies.field_policy_bypass.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this field policy to apply. If not specified, it always applies.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-condition"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is authorized, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_if"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_if-check"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_if-name"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is forbidden, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_if"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_if-check"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_if-name"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is authorized.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_unless"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_unless-check"},{"type":"DSL","title":"field_policies.field_policy_bypass.authorize_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-authorize_unless-name"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is forbidden.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_unless"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_unless-check"},{"type":"DSL","title":"field_policies.field_policy_bypass.forbid_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy_bypass-forbid_unless-name"},{"type":"DSL","title":"field_policies.field_policy - dsl-ash-policy-authorizer","doc":"Field policies behave similarly to policies. See `d:Ash.Policy.Authorizer.field_policies`\nfor more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy"},{"type":"DSL","title":"field_policies.field_policy.description - dsl-ash-policy-authorizer","doc":"A description for the policy, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-description"},{"type":"DSL","title":"field_policies.field_policy.fields - dsl-ash-policy-authorizer","doc":"The field or fields that the policy applies to.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-fields"},{"type":"DSL","title":"field_policies.field_policy.condition - dsl-ash-policy-authorizer","doc":"A check or list of checks that must be true in order for this field policy to apply. If not specified, it always applies.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-condition"},{"type":"DSL","title":"field_policies.field_policy.authorize_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is authorized, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_if"},{"type":"DSL","title":"field_policies.field_policy.authorize_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_if-check"},{"type":"DSL","title":"field_policies.field_policy.authorize_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_if-name"},{"type":"DSL","title":"field_policies.field_policy.forbid_if - dsl-ash-policy-authorizer","doc":"If the check is true, the request is forbidden, otherwise run remaining checks.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_if"},{"type":"DSL","title":"field_policies.field_policy.forbid_if.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_if-check"},{"type":"DSL","title":"field_policies.field_policy.forbid_if.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_if-name"},{"type":"DSL","title":"field_policies.field_policy.authorize_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is authorized.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_unless"},{"type":"DSL","title":"field_policies.field_policy.authorize_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_unless-check"},{"type":"DSL","title":"field_policies.field_policy.authorize_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-authorize_unless-name"},{"type":"DSL","title":"field_policies.field_policy.forbid_unless - dsl-ash-policy-authorizer","doc":"If the check is true, run remaining checks, otherwise the request is forbidden.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_unless"},{"type":"DSL","title":"field_policies.field_policy.forbid_unless.check - dsl-ash-policy-authorizer","doc":"The check to run. See `Ash.Policy.Check` for more.","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_unless-check"},{"type":"DSL","title":"field_policies.field_policy.forbid_unless.name - dsl-ash-policy-authorizer","doc":"A short name or description for the check, used when explaining authorization results","ref":"dsl-ash-policy-authorizer.html#field_policies-field_policy-forbid_unless-name"},{"type":"DSL","title":"ets - dsl-ash-datalayer-ets","doc":"A section for configuring the ets data layer","ref":"dsl-ash-datalayer-ets.html#ets"},{"type":"DSL","title":"ets.private? - dsl-ash-datalayer-ets","doc":"Sets the ets table protection to private, and scopes it to only this process. The table name will not be used directly if this is true, to allow multiple processes to use this resource separately.","ref":"dsl-ash-datalayer-ets.html#ets-private?"},{"type":"DSL","title":"ets.table - dsl-ash-datalayer-ets","doc":"The name of the table. Defaults to the resource name.","ref":"dsl-ash-datalayer-ets.html#ets-table"},{"type":"DSL","title":"mnesia - dsl-ash-datalayer-mnesia","doc":"A section for configuring the mnesia data layer","ref":"dsl-ash-datalayer-mnesia.html#mnesia"},{"type":"DSL","title":"mnesia.table - dsl-ash-datalayer-mnesia","doc":"The table name to use, defaults to the name of the resource","ref":"dsl-ash-datalayer-mnesia.html#mnesia-table"},{"type":"DSL","title":"ash - dsl-ash-reactor","doc":"Ash-related configuration for the `Ash.Reactor` extension","ref":"dsl-ash-reactor.html#ash"},{"type":"DSL","title":"ash.default_domain - dsl-ash-reactor","doc":"A domain to use by default when calling actions","ref":"dsl-ash-reactor.html#ash-default_domain"},{"type":"DSL","title":"reactor.action - dsl-ash-reactor","doc":"Declares a step that will call a generic action on a resource.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-action"},{"type":"DSL","title":"reactor.action.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-action-action"},{"type":"DSL","title":"reactor.action.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-action-domain"},{"type":"DSL","title":"reactor.action.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-action-async?"},{"type":"DSL","title":"reactor.action.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-action-authorize?"},{"type":"DSL","title":"reactor.action.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-action-description"},{"type":"DSL","title":"reactor.action.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-action-name"},{"type":"DSL","title":"reactor.action.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-action-resource"},{"type":"DSL","title":"reactor.action.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-action-undo_action"},{"type":"DSL","title":"reactor.action.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-action-undo"},{"type":"DSL","title":"reactor.action.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-action-actor"},{"type":"DSL","title":"reactor.action.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-action-actor-source"},{"type":"DSL","title":"reactor.action.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-action-actor-transform"},{"type":"DSL","title":"reactor.action.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-action-context"},{"type":"DSL","title":"reactor.action.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-action-context-context"},{"type":"DSL","title":"reactor.action.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-action-context-transform"},{"type":"DSL","title":"reactor.action.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-action-guard"},{"type":"DSL","title":"reactor.action.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-action-guard-fun"},{"type":"DSL","title":"reactor.action.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-action-guard-description"},{"type":"DSL","title":"reactor.action.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-action-where"},{"type":"DSL","title":"reactor.action.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-action-where-predicate"},{"type":"DSL","title":"reactor.action.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-action-where-description"},{"type":"DSL","title":"reactor.action.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-action-inputs"},{"type":"DSL","title":"reactor.action.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-action-inputs-template"},{"type":"DSL","title":"reactor.action.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-action-inputs-transform"},{"type":"DSL","title":"reactor.action.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-action-tenant"},{"type":"DSL","title":"reactor.action.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-action-tenant-source"},{"type":"DSL","title":"reactor.action.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-action-tenant-transform"},{"type":"DSL","title":"reactor.action.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-action-wait_for"},{"type":"DSL","title":"reactor.action.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-action-wait_for-names"},{"type":"DSL","title":"reactor.action.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-action-wait_for-description"},{"type":"DSL","title":"reactor.ash_step - dsl-ash-reactor","doc":"Specifies a Ash.Reactor step.\n\nThis is basically a wrapper around `Reactor.step`, in order to handle\nany returned notifications from the run step/function.\n\nSee the `Reactor.Step` behaviour for more information.","ref":"dsl-ash-reactor.html#reactor-ash_step"},{"type":"DSL","title":"reactor.ash_step.name - dsl-ash-reactor","doc":"A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps.","ref":"dsl-ash-reactor.html#reactor-ash_step-name"},{"type":"DSL","title":"reactor.ash_step.impl - dsl-ash-reactor","doc":"A module that implements the `Reactor.Step` behaviour that provides the implementation.","ref":"dsl-ash-reactor.html#reactor-ash_step-impl"},{"type":"DSL","title":"reactor.ash_step.run - dsl-ash-reactor","doc":"Provide an anonymous function which implements the `run/3` callback. Cannot be provided at the same time as the `impl` argument.","ref":"dsl-ash-reactor.html#reactor-ash_step-run"},{"type":"DSL","title":"reactor.ash_step.undo - dsl-ash-reactor","doc":"Provide an anonymous function which implements the `undo/4` callback. Cannot be provided at the same time as the `impl` argument.","ref":"dsl-ash-reactor.html#reactor-ash_step-undo"},{"type":"DSL","title":"reactor.ash_step.compensate - dsl-ash-reactor","doc":"Provide an anonymous function which implements the `undo/4` callback. Cannot be provided at the same time as the `impl` argument.","ref":"dsl-ash-reactor.html#reactor-ash_step-compensate"},{"type":"DSL","title":"reactor.ash_step.max_retries - dsl-ash-reactor","doc":"The maximum number of times that the step can be retried before failing. Only used when the result of the `compensate/4` callback is `:retry`.","ref":"dsl-ash-reactor.html#reactor-ash_step-max_retries"},{"type":"DSL","title":"reactor.ash_step.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-ash_step-async?"},{"type":"DSL","title":"reactor.ash_step.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the entire argument map before it is passed to the step.","ref":"dsl-ash-reactor.html#reactor-ash_step-transform"},{"type":"DSL","title":"reactor.ash_step.argument - dsl-ash-reactor","doc":"Specifies an argument to a Reactor step.\n\nEach argument is a value which is either the result of another step, or an input value.\n\nIndividual arguments can be transformed with an arbitrary function before\nbeing passed to any steps.","ref":"dsl-ash-reactor.html#reactor-ash_step-argument"},{"type":"DSL","title":"reactor.ash_step.argument.description - dsl-ash-reactor","doc":"An optional description for the argument.","ref":"dsl-ash-reactor.html#reactor-ash_step-argument-description"},{"type":"DSL","title":"reactor.ash_step.argument.name - dsl-ash-reactor","doc":"The name of the argument which will be used as the key in the `arguments` map passed to the implementation.","ref":"dsl-ash-reactor.html#reactor-ash_step-argument-name"},{"type":"DSL","title":"reactor.ash_step.argument.source - dsl-ash-reactor","doc":"What to use as the source of the argument. See `Reactor.Dsl.Argument` for more information.","ref":"dsl-ash-reactor.html#reactor-ash_step-argument-source"},{"type":"DSL","title":"reactor.ash_step.argument.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the argument before it is passed to the step.","ref":"dsl-ash-reactor.html#reactor-ash_step-argument-transform"},{"type":"DSL","title":"reactor.ash_step.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-ash_step-wait_for"},{"type":"DSL","title":"reactor.ash_step.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-ash_step-wait_for-names"},{"type":"DSL","title":"reactor.ash_step.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-ash_step-wait_for-description"},{"type":"DSL","title":"reactor.ash_step.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-ash_step-guard"},{"type":"DSL","title":"reactor.ash_step.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-ash_step-guard-fun"},{"type":"DSL","title":"reactor.ash_step.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-ash_step-guard-description"},{"type":"DSL","title":"reactor.ash_step.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-ash_step-where"},{"type":"DSL","title":"reactor.ash_step.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-ash_step-where-predicate"},{"type":"DSL","title":"reactor.ash_step.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-ash_step-where-description"},{"type":"DSL","title":"reactor.bulk_create - dsl-ash-reactor","doc":"Declares a step which will call a create action on a resource with a collection of inputs.\n\n> ### Check the docs! {: .warning}\n>\n> Make sure to thoroughly read and understand the documentation in `Ash.bulk_create/4` before using. Read each option and note the default values. By default, bulk creates don't return records or errors, and don't emit notifications.\n\nCaveats/differences from `Ash.bulk_create/4`:\n\n1. `max_concurrency` specifies the number of tasks that Ash will start to process batches, and has no effect on Reactor concurrency targets.  It's could be possible to create a very large number of processes if a number of steps are running bulk actions with a high degree of concurrency.\n2. Setting `notify?` to `true` will cause both `notify?` and `return_notifications?` to be set to true in the underlying call to `Ash.bulk_create/4`. Notifications will then be managed by the `Ash.Reactor.Notifications` Reactor middleware.\n3. If you specify an undo action it must be a generic action which takes the bulk result as it's only argument.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-bulk_create"},{"type":"DSL","title":"reactor.bulk_create.assume_casted? - dsl-ash-reactor","doc":"Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting","ref":"dsl-ash-reactor.html#reactor-bulk_create-assume_casted?"},{"type":"DSL","title":"reactor.bulk_create.authorize_changeset_with - dsl-ash-reactor","doc":"If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error","ref":"dsl-ash-reactor.html#reactor-bulk_create-authorize_changeset_with"},{"type":"DSL","title":"reactor.bulk_create.authorize_query_with - dsl-ash-reactor","doc":"If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error","ref":"dsl-ash-reactor.html#reactor-bulk_create-authorize_query_with"},{"type":"DSL","title":"reactor.bulk_create.batch_size - dsl-ash-reactor","doc":"The number of records to include in each batch. Defaults to the `default_limit` or `max_page_size` of the action, or 100.","ref":"dsl-ash-reactor.html#reactor-bulk_create-batch_size"},{"type":"DSL","title":"reactor.bulk_create.initial - dsl-ash-reactor","doc":"A collection of inputs to pass to the create action. Must implement the `Enumerable` protocol.","ref":"dsl-ash-reactor.html#reactor-bulk_create-initial"},{"type":"DSL","title":"reactor.bulk_create.max_concurrency - dsl-ash-reactor","doc":"If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously.","ref":"dsl-ash-reactor.html#reactor-bulk_create-max_concurrency"},{"type":"DSL","title":"reactor.bulk_create.notification_metadata - dsl-ash-reactor","doc":"Metadata to be merged into the metadata field for all notifications sent from this operation.","ref":"dsl-ash-reactor.html#reactor-bulk_create-notification_metadata"},{"type":"DSL","title":"reactor.bulk_create.notify? - dsl-ash-reactor","doc":"Whether or not to generate any notifications. This may be intensive for large bulk actions.","ref":"dsl-ash-reactor.html#reactor-bulk_create-notify?"},{"type":"DSL","title":"reactor.bulk_create.read_action - dsl-ash-reactor","doc":"The action to use when building the read query.","ref":"dsl-ash-reactor.html#reactor-bulk_create-read_action"},{"type":"DSL","title":"reactor.bulk_create.return_errors? - dsl-ash-reactor","doc":"Whether or not to return all of the errors that occur. Defaults to false to account for large inserts.","ref":"dsl-ash-reactor.html#reactor-bulk_create-return_errors?"},{"type":"DSL","title":"reactor.bulk_create.return_records? - dsl-ash-reactor","doc":"Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts.","ref":"dsl-ash-reactor.html#reactor-bulk_create-return_records?"},{"type":"DSL","title":"reactor.bulk_create.return_stream? - dsl-ash-reactor","doc":"If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.","ref":"dsl-ash-reactor.html#reactor-bulk_create-return_stream?"},{"type":"DSL","title":"reactor.bulk_create.rollback_on_error? - dsl-ash-reactor","doc":"Whether or not to rollback the transaction on error, if the resource is in a transaction.","ref":"dsl-ash-reactor.html#reactor-bulk_create-rollback_on_error?"},{"type":"DSL","title":"reactor.bulk_create.select - dsl-ash-reactor","doc":"A select statement to apply to records. Ignored if `return_records?` is not `true`.","ref":"dsl-ash-reactor.html#reactor-bulk_create-select"},{"type":"DSL","title":"reactor.bulk_create.skip_unknown_inputs - dsl-ash-reactor","doc":"A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"dsl-ash-reactor.html#reactor-bulk_create-skip_unknown_inputs"},{"type":"DSL","title":"reactor.bulk_create.sorted? - dsl-ash-reactor","doc":"Whether or not to sort results by their input position, in cases where `return_records?` is set to `true`.","ref":"dsl-ash-reactor.html#reactor-bulk_create-sorted?"},{"type":"DSL","title":"reactor.bulk_create.stop_on_error? - dsl-ash-reactor","doc":"If `true`, the first encountered error will stop the action and be returned. Otherwise, errors will be skipped.","ref":"dsl-ash-reactor.html#reactor-bulk_create-stop_on_error?"},{"type":"DSL","title":"reactor.bulk_create.success_state - dsl-ash-reactor","doc":"Bulk results can be entirely or partially successful. Chooses the `Ash.BulkResult` state to consider the step a success.","ref":"dsl-ash-reactor.html#reactor-bulk_create-success_state"},{"type":"DSL","title":"reactor.bulk_create.timeout - dsl-ash-reactor","doc":"If none is provided, the timeout configured on the domain is used (which defaults to `30_000`).","ref":"dsl-ash-reactor.html#reactor-bulk_create-timeout"},{"type":"DSL","title":"reactor.bulk_create.transaction - dsl-ash-reactor","doc":"Whether or not to wrap the entire execution in a transaction, each batch, or not at all.","ref":"dsl-ash-reactor.html#reactor-bulk_create-transaction"},{"type":"DSL","title":"reactor.bulk_create.upsert_fields - dsl-ash-reactor","doc":"The fields to upsert. If not set, the action's `upsert_fields` is used.","ref":"dsl-ash-reactor.html#reactor-bulk_create-upsert_fields"},{"type":"DSL","title":"reactor.bulk_create.upsert_identity - dsl-ash-reactor","doc":"The identity to use for the upsert","ref":"dsl-ash-reactor.html#reactor-bulk_create-upsert_identity"},{"type":"DSL","title":"reactor.bulk_create.upsert? - dsl-ash-reactor","doc":"Whether or not this action should be executed as an upsert.","ref":"dsl-ash-reactor.html#reactor-bulk_create-upsert?"},{"type":"DSL","title":"reactor.bulk_create.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-bulk_create-action"},{"type":"DSL","title":"reactor.bulk_create.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-bulk_create-domain"},{"type":"DSL","title":"reactor.bulk_create.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-bulk_create-async?"},{"type":"DSL","title":"reactor.bulk_create.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-bulk_create-authorize?"},{"type":"DSL","title":"reactor.bulk_create.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-bulk_create-description"},{"type":"DSL","title":"reactor.bulk_create.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-bulk_create-name"},{"type":"DSL","title":"reactor.bulk_create.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-bulk_create-resource"},{"type":"DSL","title":"reactor.bulk_create.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-bulk_create-undo_action"},{"type":"DSL","title":"reactor.bulk_create.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-bulk_create-undo"},{"type":"DSL","title":"reactor.bulk_create.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-bulk_create-actor"},{"type":"DSL","title":"reactor.bulk_create.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-bulk_create-actor-source"},{"type":"DSL","title":"reactor.bulk_create.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_create-actor-transform"},{"type":"DSL","title":"reactor.bulk_create.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-bulk_create-context"},{"type":"DSL","title":"reactor.bulk_create.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-bulk_create-context-context"},{"type":"DSL","title":"reactor.bulk_create.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_create-context-transform"},{"type":"DSL","title":"reactor.bulk_create.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-bulk_create-guard"},{"type":"DSL","title":"reactor.bulk_create.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-bulk_create-guard-fun"},{"type":"DSL","title":"reactor.bulk_create.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-bulk_create-guard-description"},{"type":"DSL","title":"reactor.bulk_create.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-bulk_create-where"},{"type":"DSL","title":"reactor.bulk_create.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-bulk_create-where-predicate"},{"type":"DSL","title":"reactor.bulk_create.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-bulk_create-where-description"},{"type":"DSL","title":"reactor.bulk_create.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-bulk_create-load"},{"type":"DSL","title":"reactor.bulk_create.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-bulk_create-load-source"},{"type":"DSL","title":"reactor.bulk_create.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_create-load-transform"},{"type":"DSL","title":"reactor.bulk_create.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-bulk_create-tenant"},{"type":"DSL","title":"reactor.bulk_create.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-bulk_create-tenant-source"},{"type":"DSL","title":"reactor.bulk_create.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_create-tenant-transform"},{"type":"DSL","title":"reactor.bulk_create.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-bulk_create-wait_for"},{"type":"DSL","title":"reactor.bulk_create.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-bulk_create-wait_for-names"},{"type":"DSL","title":"reactor.bulk_create.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-bulk_create-wait_for-description"},{"type":"DSL","title":"reactor.bulk_update - dsl-ash-reactor","doc":"Declares a step which will call an update action on a resource with a collection of inputs.\n\n> ### Check the docs! {: .warning}\n>\n> Make sure to thoroughly read and understand the documentation in `Ash.bulk_update/4` before using.  Read each option and note the default values.  By default, bulk updates don't return records or errors, and don't emit notifications.\n\nCaveats/differences from `Ash.bulk_update/4`:\n\n1. `max_concurrency` specifies the number of tasks that Ash will start to process batches, and has no effect on Reactor concurrency targets.  It's could be possible to create a very large number of processes if a number of steps are running bulk actions with a high degree of concurrency.\n2. Setting `notify?` to `true` will cause both `notify?` and `return_notifications?` to be set to true in the underlying call to `Ash.bulk_create/4`. Notifications will then be managed by the `Ash.Reactor.Notifications` Reactor middleware.\n3. If you specify an undo action it must be a generic action which takes the bulk result as it's only argument.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-bulk_update"},{"type":"DSL","title":"reactor.bulk_update.allow_stream_with - dsl-ash-reactor","doc":"The 'worst' strategy allowed to be used to fetch records if the :stream strategy is chosen. See the `Ash.stream!/2` docs for more.","ref":"dsl-ash-reactor.html#reactor-bulk_update-allow_stream_with"},{"type":"DSL","title":"reactor.bulk_update.assume_casted? - dsl-ash-reactor","doc":"Whether or not to cast attributes and arguments as input. This is an optimization for cases where the input is already casted and/or not in need of casting","ref":"dsl-ash-reactor.html#reactor-bulk_update-assume_casted?"},{"type":"DSL","title":"reactor.bulk_update.atomic_update - dsl-ash-reactor","doc":"A map of atomic updates to apply. See `Ash.Changeset.atomic_update/3` for more.","ref":"dsl-ash-reactor.html#reactor-bulk_update-atomic_update"},{"type":"DSL","title":"reactor.bulk_update.authorize_changeset_with - dsl-ash-reactor","doc":"If set to `:error`, instead of filtering unauthorized changes, unauthorized changes will raise an appropriate forbidden error","ref":"dsl-ash-reactor.html#reactor-bulk_update-authorize_changeset_with"},{"type":"DSL","title":"reactor.bulk_update.authorize_query_with - dsl-ash-reactor","doc":"If set to `:error`, instead of filtering unauthorized query results, unauthorized query results will raise an appropriate forbidden error","ref":"dsl-ash-reactor.html#reactor-bulk_update-authorize_query_with"},{"type":"DSL","title":"reactor.bulk_update.authorize_query? - dsl-ash-reactor","doc":"If a query is given, determines whether or not authorization is run on that query.","ref":"dsl-ash-reactor.html#reactor-bulk_update-authorize_query?"},{"type":"DSL","title":"reactor.bulk_update.batch_size - dsl-ash-reactor","doc":"The number of records to include in each batch. Defaults to the `default_limit` or `max_page_size` of the action, or 100.","ref":"dsl-ash-reactor.html#reactor-bulk_update-batch_size"},{"type":"DSL","title":"reactor.bulk_update.filter - dsl-ash-reactor","doc":"A filter to apply to records. This is also applied to a stream of inputs.","ref":"dsl-ash-reactor.html#reactor-bulk_update-filter"},{"type":"DSL","title":"reactor.bulk_update.initial - dsl-ash-reactor","doc":"A collection of inputs to pass to the create action. Must implement the `Enumerable` protocol.","ref":"dsl-ash-reactor.html#reactor-bulk_update-initial"},{"type":"DSL","title":"reactor.bulk_update.lock - dsl-ash-reactor","doc":"A lock statement to add onto the query.","ref":"dsl-ash-reactor.html#reactor-bulk_update-lock"},{"type":"DSL","title":"reactor.bulk_update.max_concurrency - dsl-ash-reactor","doc":"If set to a value greater than 0, up to that many tasks will be started to run batches asynchronously.","ref":"dsl-ash-reactor.html#reactor-bulk_update-max_concurrency"},{"type":"DSL","title":"reactor.bulk_update.notification_metadata - dsl-ash-reactor","doc":"Metadata to be merged into the metadata field for all notifications sent from this operation.","ref":"dsl-ash-reactor.html#reactor-bulk_update-notification_metadata"},{"type":"DSL","title":"reactor.bulk_update.notify? - dsl-ash-reactor","doc":"Whether or not to generate any notifications. This may be intensive for large bulk actions.","ref":"dsl-ash-reactor.html#reactor-bulk_update-notify?"},{"type":"DSL","title":"reactor.bulk_update.page - dsl-ash-reactor","doc":"Pagination options, see `Ash.read/2` for more.","ref":"dsl-ash-reactor.html#reactor-bulk_update-page"},{"type":"DSL","title":"reactor.bulk_update.read_action - dsl-ash-reactor","doc":"The action to use when building the read query.","ref":"dsl-ash-reactor.html#reactor-bulk_update-read_action"},{"type":"DSL","title":"reactor.bulk_update.return_errors? - dsl-ash-reactor","doc":"Whether or not to return all of the errors that occur. Defaults to false to account for large inserts.","ref":"dsl-ash-reactor.html#reactor-bulk_update-return_errors?"},{"type":"DSL","title":"reactor.bulk_update.return_records? - dsl-ash-reactor","doc":"Whether or not to return all of the records that were inserted. Defaults to false to account for large inserts.","ref":"dsl-ash-reactor.html#reactor-bulk_update-return_records?"},{"type":"DSL","title":"reactor.bulk_update.return_stream? - dsl-ash-reactor","doc":"If set to `true`, instead of an `Ash.BulkResult`, a mixed stream is returned.","ref":"dsl-ash-reactor.html#reactor-bulk_update-return_stream?"},{"type":"DSL","title":"reactor.bulk_update.reuse_values? - dsl-ash-reactor","doc":"Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer.","ref":"dsl-ash-reactor.html#reactor-bulk_update-reuse_values?"},{"type":"DSL","title":"reactor.bulk_update.rollback_on_error? - dsl-ash-reactor","doc":"Whether or not to rollback the transaction on error, if the resource is in a transaction.","ref":"dsl-ash-reactor.html#reactor-bulk_update-rollback_on_error?"},{"type":"DSL","title":"reactor.bulk_update.select - dsl-ash-reactor","doc":"A select statement to apply to records. Ignored if `return_records?` is not `true`.","ref":"dsl-ash-reactor.html#reactor-bulk_update-select"},{"type":"DSL","title":"reactor.bulk_update.skip_unknown_inputs - dsl-ash-reactor","doc":"A list of inputs that, if provided, will be ignored if they are not recognized by the action. Use `:*` to indicate all unknown keys.","ref":"dsl-ash-reactor.html#reactor-bulk_update-skip_unknown_inputs"},{"type":"DSL","title":"reactor.bulk_update.sorted? - dsl-ash-reactor","doc":"Whether or not to sort results by their input position, in cases where `return_records?` is set to `true`.","ref":"dsl-ash-reactor.html#reactor-bulk_update-sorted?"},{"type":"DSL","title":"reactor.bulk_update.stop_on_error? - dsl-ash-reactor","doc":"If `true`, the first encountered error will stop the action and be returned. Otherwise, errors will be skipped.","ref":"dsl-ash-reactor.html#reactor-bulk_update-stop_on_error?"},{"type":"DSL","title":"reactor.bulk_update.strategy - dsl-ash-reactor","doc":"The strategy or strategies to enable. `:stream` is used in all cases if the data layer does not support atomics.","ref":"dsl-ash-reactor.html#reactor-bulk_update-strategy"},{"type":"DSL","title":"reactor.bulk_update.stream_batch_size - dsl-ash-reactor","doc":"Batch size to use if provided a query and the query must be streamed.","ref":"dsl-ash-reactor.html#reactor-bulk_update-stream_batch_size"},{"type":"DSL","title":"reactor.bulk_update.stream_with - dsl-ash-reactor","doc":"The specific strategy to use to fetch records. See `Ash.stream!/2` docs for more.","ref":"dsl-ash-reactor.html#reactor-bulk_update-stream_with"},{"type":"DSL","title":"reactor.bulk_update.success_state - dsl-ash-reactor","doc":"Bulk results can be entirely or partially successful. Chooses the `Ash.BulkResult` state to consider the step a success.","ref":"dsl-ash-reactor.html#reactor-bulk_update-success_state"},{"type":"DSL","title":"reactor.bulk_update.timeout - dsl-ash-reactor","doc":"If none is provided, the timeout configured on the domain is used (which defaults to `30_000`).","ref":"dsl-ash-reactor.html#reactor-bulk_update-timeout"},{"type":"DSL","title":"reactor.bulk_update.transaction - dsl-ash-reactor","doc":"Whether or not to wrap the entire execution in a transaction, each batch, or not at all.","ref":"dsl-ash-reactor.html#reactor-bulk_update-transaction"},{"type":"DSL","title":"reactor.bulk_update.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-bulk_update-action"},{"type":"DSL","title":"reactor.bulk_update.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-bulk_update-domain"},{"type":"DSL","title":"reactor.bulk_update.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-bulk_update-async?"},{"type":"DSL","title":"reactor.bulk_update.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-bulk_update-authorize?"},{"type":"DSL","title":"reactor.bulk_update.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-bulk_update-description"},{"type":"DSL","title":"reactor.bulk_update.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-bulk_update-name"},{"type":"DSL","title":"reactor.bulk_update.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-bulk_update-resource"},{"type":"DSL","title":"reactor.bulk_update.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-bulk_update-undo_action"},{"type":"DSL","title":"reactor.bulk_update.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-bulk_update-undo"},{"type":"DSL","title":"reactor.bulk_update.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-bulk_update-actor"},{"type":"DSL","title":"reactor.bulk_update.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-bulk_update-actor-source"},{"type":"DSL","title":"reactor.bulk_update.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_update-actor-transform"},{"type":"DSL","title":"reactor.bulk_update.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-bulk_update-context"},{"type":"DSL","title":"reactor.bulk_update.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-bulk_update-context-context"},{"type":"DSL","title":"reactor.bulk_update.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_update-context-transform"},{"type":"DSL","title":"reactor.bulk_update.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-bulk_update-guard"},{"type":"DSL","title":"reactor.bulk_update.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-bulk_update-guard-fun"},{"type":"DSL","title":"reactor.bulk_update.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-bulk_update-guard-description"},{"type":"DSL","title":"reactor.bulk_update.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-bulk_update-where"},{"type":"DSL","title":"reactor.bulk_update.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-bulk_update-where-predicate"},{"type":"DSL","title":"reactor.bulk_update.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-bulk_update-where-description"},{"type":"DSL","title":"reactor.bulk_update.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-bulk_update-inputs"},{"type":"DSL","title":"reactor.bulk_update.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-bulk_update-inputs-template"},{"type":"DSL","title":"reactor.bulk_update.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-bulk_update-inputs-transform"},{"type":"DSL","title":"reactor.bulk_update.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-bulk_update-tenant"},{"type":"DSL","title":"reactor.bulk_update.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-bulk_update-tenant-source"},{"type":"DSL","title":"reactor.bulk_update.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-bulk_update-tenant-transform"},{"type":"DSL","title":"reactor.bulk_update.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-bulk_update-wait_for"},{"type":"DSL","title":"reactor.bulk_update.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-bulk_update-wait_for-names"},{"type":"DSL","title":"reactor.bulk_update.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-bulk_update-wait_for-description"},{"type":"DSL","title":"reactor.change - dsl-ash-reactor","doc":"Declares a step that will modify a changeset.","ref":"dsl-ash-reactor.html#reactor-change"},{"type":"DSL","title":"reactor.change.name - dsl-ash-reactor","doc":"A unique name for this step.","ref":"dsl-ash-reactor.html#reactor-change-name"},{"type":"DSL","title":"reactor.change.initial - dsl-ash-reactor","doc":"The initial value to work from, either a resource or a changeset","ref":"dsl-ash-reactor.html#reactor-change-initial"},{"type":"DSL","title":"reactor.change.description - dsl-ash-reactor","doc":"An optional description for the change","ref":"dsl-ash-reactor.html#reactor-change-description"},{"type":"DSL","title":"reactor.change.only_when_valid? - dsl-ash-reactor","doc":"If the change should only be run on valid changes. By default, all changes are run unless stated otherwise here.","ref":"dsl-ash-reactor.html#reactor-change-only_when_valid?"},{"type":"DSL","title":"reactor.change.change - dsl-ash-reactor","doc":"The module and options for a change. Also accepts a function that takes the changeset and the context. See `Ash.Resource.Change.Builtins` for builtin changes.","ref":"dsl-ash-reactor.html#reactor-change-change"},{"type":"DSL","title":"reactor.change.where - dsl-ash-reactor","doc":"Validations that should pass in order for this change to apply. These validations failing will result in this change being ignored.","ref":"dsl-ash-reactor.html#reactor-change-where"},{"type":"DSL","title":"reactor.change.fail_if_invalid? - dsl-ash-reactor","doc":"Fail if the result of the change is an invalid changeset","ref":"dsl-ash-reactor.html#reactor-change-fail_if_invalid?"},{"type":"DSL","title":"reactor.change.argument - dsl-ash-reactor","doc":"Specifies an argument to a Reactor step.\n\nEach argument is a value which is either the result of another step, or an input value.\n\nIndividual arguments can be transformed with an arbitrary function before\nbeing passed to any steps.","ref":"dsl-ash-reactor.html#reactor-change-argument"},{"type":"DSL","title":"reactor.change.argument.description - dsl-ash-reactor","doc":"An optional description for the argument.","ref":"dsl-ash-reactor.html#reactor-change-argument-description"},{"type":"DSL","title":"reactor.change.argument.name - dsl-ash-reactor","doc":"The name of the argument which will be used as the key in the `arguments` map passed to the implementation.","ref":"dsl-ash-reactor.html#reactor-change-argument-name"},{"type":"DSL","title":"reactor.change.argument.source - dsl-ash-reactor","doc":"What to use as the source of the argument. See `Reactor.Dsl.Argument` for more information.","ref":"dsl-ash-reactor.html#reactor-change-argument-source"},{"type":"DSL","title":"reactor.change.argument.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the argument before it is passed to the step.","ref":"dsl-ash-reactor.html#reactor-change-argument-transform"},{"type":"DSL","title":"reactor.change.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-change-wait_for"},{"type":"DSL","title":"reactor.change.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-change-wait_for-names"},{"type":"DSL","title":"reactor.change.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-change-wait_for-description"},{"type":"DSL","title":"reactor.create - dsl-ash-reactor","doc":"Declares a step that will call a create action on a resource.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-create"},{"type":"DSL","title":"reactor.create.initial - dsl-ash-reactor","doc":"The initial value passed into the action.","ref":"dsl-ash-reactor.html#reactor-create-initial"},{"type":"DSL","title":"reactor.create.upsert_identity - dsl-ash-reactor","doc":"The identity to use for the upsert","ref":"dsl-ash-reactor.html#reactor-create-upsert_identity"},{"type":"DSL","title":"reactor.create.upsert? - dsl-ash-reactor","doc":"Whether or not this action should be executed as an upsert.","ref":"dsl-ash-reactor.html#reactor-create-upsert?"},{"type":"DSL","title":"reactor.create.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-create-action"},{"type":"DSL","title":"reactor.create.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-create-domain"},{"type":"DSL","title":"reactor.create.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-create-async?"},{"type":"DSL","title":"reactor.create.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-create-authorize?"},{"type":"DSL","title":"reactor.create.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-create-description"},{"type":"DSL","title":"reactor.create.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-create-name"},{"type":"DSL","title":"reactor.create.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-create-resource"},{"type":"DSL","title":"reactor.create.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-create-undo_action"},{"type":"DSL","title":"reactor.create.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-create-undo"},{"type":"DSL","title":"reactor.create.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-create-actor"},{"type":"DSL","title":"reactor.create.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-create-actor-source"},{"type":"DSL","title":"reactor.create.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-create-actor-transform"},{"type":"DSL","title":"reactor.create.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-create-context"},{"type":"DSL","title":"reactor.create.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-create-context-context"},{"type":"DSL","title":"reactor.create.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-create-context-transform"},{"type":"DSL","title":"reactor.create.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-create-guard"},{"type":"DSL","title":"reactor.create.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-create-guard-fun"},{"type":"DSL","title":"reactor.create.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-create-guard-description"},{"type":"DSL","title":"reactor.create.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-create-where"},{"type":"DSL","title":"reactor.create.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-create-where-predicate"},{"type":"DSL","title":"reactor.create.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-create-where-description"},{"type":"DSL","title":"reactor.create.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-create-inputs"},{"type":"DSL","title":"reactor.create.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-create-inputs-template"},{"type":"DSL","title":"reactor.create.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-create-inputs-transform"},{"type":"DSL","title":"reactor.create.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-create-load"},{"type":"DSL","title":"reactor.create.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-create-load-source"},{"type":"DSL","title":"reactor.create.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-create-load-transform"},{"type":"DSL","title":"reactor.create.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-create-tenant"},{"type":"DSL","title":"reactor.create.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-create-tenant-source"},{"type":"DSL","title":"reactor.create.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-create-tenant-transform"},{"type":"DSL","title":"reactor.create.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-create-wait_for"},{"type":"DSL","title":"reactor.create.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-create-wait_for-names"},{"type":"DSL","title":"reactor.create.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-create-wait_for-description"},{"type":"DSL","title":"reactor.destroy - dsl-ash-reactor","doc":"Declares a step that will call a destroy action on a resource.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-destroy"},{"type":"DSL","title":"reactor.destroy.initial - dsl-ash-reactor","doc":"The record to update.","ref":"dsl-ash-reactor.html#reactor-destroy-initial"},{"type":"DSL","title":"reactor.destroy.return_destroyed? - dsl-ash-reactor","doc":"Whether or not the step should return the destroyed record upon completion.","ref":"dsl-ash-reactor.html#reactor-destroy-return_destroyed?"},{"type":"DSL","title":"reactor.destroy.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-destroy-action"},{"type":"DSL","title":"reactor.destroy.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-destroy-domain"},{"type":"DSL","title":"reactor.destroy.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-destroy-async?"},{"type":"DSL","title":"reactor.destroy.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-destroy-authorize?"},{"type":"DSL","title":"reactor.destroy.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-destroy-description"},{"type":"DSL","title":"reactor.destroy.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-destroy-name"},{"type":"DSL","title":"reactor.destroy.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-destroy-resource"},{"type":"DSL","title":"reactor.destroy.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-destroy-undo_action"},{"type":"DSL","title":"reactor.destroy.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-destroy-undo"},{"type":"DSL","title":"reactor.destroy.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-destroy-actor"},{"type":"DSL","title":"reactor.destroy.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-destroy-actor-source"},{"type":"DSL","title":"reactor.destroy.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-destroy-actor-transform"},{"type":"DSL","title":"reactor.destroy.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-destroy-context"},{"type":"DSL","title":"reactor.destroy.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-destroy-context-context"},{"type":"DSL","title":"reactor.destroy.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-destroy-context-transform"},{"type":"DSL","title":"reactor.destroy.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-destroy-guard"},{"type":"DSL","title":"reactor.destroy.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-destroy-guard-fun"},{"type":"DSL","title":"reactor.destroy.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-destroy-guard-description"},{"type":"DSL","title":"reactor.destroy.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-destroy-where"},{"type":"DSL","title":"reactor.destroy.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-destroy-where-predicate"},{"type":"DSL","title":"reactor.destroy.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-destroy-where-description"},{"type":"DSL","title":"reactor.destroy.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-destroy-inputs"},{"type":"DSL","title":"reactor.destroy.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-destroy-inputs-template"},{"type":"DSL","title":"reactor.destroy.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-destroy-inputs-transform"},{"type":"DSL","title":"reactor.destroy.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-destroy-load"},{"type":"DSL","title":"reactor.destroy.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-destroy-load-source"},{"type":"DSL","title":"reactor.destroy.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-destroy-load-transform"},{"type":"DSL","title":"reactor.destroy.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-destroy-tenant"},{"type":"DSL","title":"reactor.destroy.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-destroy-tenant-source"},{"type":"DSL","title":"reactor.destroy.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-destroy-tenant-transform"},{"type":"DSL","title":"reactor.destroy.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-destroy-wait_for"},{"type":"DSL","title":"reactor.destroy.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-destroy-wait_for-names"},{"type":"DSL","title":"reactor.destroy.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-destroy-wait_for-description"},{"type":"DSL","title":"reactor.load - dsl-ash-reactor","doc":"Declares a step that will load additional data on a resource.","ref":"dsl-ash-reactor.html#reactor-load"},{"type":"DSL","title":"reactor.load.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-load-domain"},{"type":"DSL","title":"reactor.load.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-load-async?"},{"type":"DSL","title":"reactor.load.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-load-authorize?"},{"type":"DSL","title":"reactor.load.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-load-description"},{"type":"DSL","title":"reactor.load.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-load-name"},{"type":"DSL","title":"reactor.load.records - dsl-ash-reactor","doc":"The records upon which to add extra loaded data","ref":"dsl-ash-reactor.html#reactor-load-records"},{"type":"DSL","title":"reactor.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load statement before it is passed to the load.","ref":"dsl-ash-reactor.html#reactor-load-transform"},{"type":"DSL","title":"reactor.load.load - dsl-ash-reactor","doc":"An Ash load statement","ref":"dsl-ash-reactor.html#reactor-load-load"},{"type":"DSL","title":"reactor.load.lazy? - dsl-ash-reactor","doc":"If set to true, values will only be loaded if the related value isn't currently loaded.","ref":"dsl-ash-reactor.html#reactor-load-lazy?"},{"type":"DSL","title":"reactor.load.reuse_values? - dsl-ash-reactor","doc":"Whether calculations are allowed to reuse values that have already been loaded, or must refetch them from the data layer.","ref":"dsl-ash-reactor.html#reactor-load-reuse_values?"},{"type":"DSL","title":"reactor.load.strict? - dsl-ash-reactor","doc":"If set to true, only specified attributes will be loaded when passing a list of fields to fetch on a relationship, which allows for more optimized data-fetching.","ref":"dsl-ash-reactor.html#reactor-load-strict?"},{"type":"DSL","title":"reactor.load.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-load-actor"},{"type":"DSL","title":"reactor.load.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-load-actor-source"},{"type":"DSL","title":"reactor.load.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-load-actor-transform"},{"type":"DSL","title":"reactor.load.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-load-context"},{"type":"DSL","title":"reactor.load.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-load-context-context"},{"type":"DSL","title":"reactor.load.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-load-context-transform"},{"type":"DSL","title":"reactor.load.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-load-guard"},{"type":"DSL","title":"reactor.load.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-load-guard-fun"},{"type":"DSL","title":"reactor.load.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-load-guard-description"},{"type":"DSL","title":"reactor.load.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-load-where"},{"type":"DSL","title":"reactor.load.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-load-where-predicate"},{"type":"DSL","title":"reactor.load.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-load-where-description"},{"type":"DSL","title":"reactor.load.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-load-tenant"},{"type":"DSL","title":"reactor.load.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-load-tenant-source"},{"type":"DSL","title":"reactor.load.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-load-tenant-transform"},{"type":"DSL","title":"reactor.load.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-load-wait_for"},{"type":"DSL","title":"reactor.load.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-load-wait_for-names"},{"type":"DSL","title":"reactor.load.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-load-wait_for-description"},{"type":"DSL","title":"reactor.read_one - dsl-ash-reactor","doc":"Declares a step that will call a read action on a resource returning a single record.","ref":"dsl-ash-reactor.html#reactor-read_one"},{"type":"DSL","title":"reactor.read_one.fail_on_not_found? - dsl-ash-reactor","doc":"When set to true the step will fail if the resource is not found.","ref":"dsl-ash-reactor.html#reactor-read_one-fail_on_not_found?"},{"type":"DSL","title":"reactor.read_one.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-read_one-action"},{"type":"DSL","title":"reactor.read_one.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-read_one-domain"},{"type":"DSL","title":"reactor.read_one.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-read_one-async?"},{"type":"DSL","title":"reactor.read_one.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-read_one-authorize?"},{"type":"DSL","title":"reactor.read_one.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-read_one-description"},{"type":"DSL","title":"reactor.read_one.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-read_one-name"},{"type":"DSL","title":"reactor.read_one.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-read_one-resource"},{"type":"DSL","title":"reactor.read_one.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-read_one-actor"},{"type":"DSL","title":"reactor.read_one.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-read_one-actor-source"},{"type":"DSL","title":"reactor.read_one.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read_one-actor-transform"},{"type":"DSL","title":"reactor.read_one.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-read_one-context"},{"type":"DSL","title":"reactor.read_one.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-read_one-context-context"},{"type":"DSL","title":"reactor.read_one.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read_one-context-transform"},{"type":"DSL","title":"reactor.read_one.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-read_one-guard"},{"type":"DSL","title":"reactor.read_one.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-read_one-guard-fun"},{"type":"DSL","title":"reactor.read_one.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-read_one-guard-description"},{"type":"DSL","title":"reactor.read_one.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-read_one-where"},{"type":"DSL","title":"reactor.read_one.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-read_one-where-predicate"},{"type":"DSL","title":"reactor.read_one.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-read_one-where-description"},{"type":"DSL","title":"reactor.read_one.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-read_one-inputs"},{"type":"DSL","title":"reactor.read_one.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-read_one-inputs-template"},{"type":"DSL","title":"reactor.read_one.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-read_one-inputs-transform"},{"type":"DSL","title":"reactor.read_one.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-read_one-load"},{"type":"DSL","title":"reactor.read_one.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-read_one-load-source"},{"type":"DSL","title":"reactor.read_one.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read_one-load-transform"},{"type":"DSL","title":"reactor.read_one.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-read_one-tenant"},{"type":"DSL","title":"reactor.read_one.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-read_one-tenant-source"},{"type":"DSL","title":"reactor.read_one.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read_one-tenant-transform"},{"type":"DSL","title":"reactor.read_one.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-read_one-wait_for"},{"type":"DSL","title":"reactor.read_one.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-read_one-wait_for-names"},{"type":"DSL","title":"reactor.read_one.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-read_one-wait_for-description"},{"type":"DSL","title":"reactor.read - dsl-ash-reactor","doc":"Declares a step that will call a read action on a resource.","ref":"dsl-ash-reactor.html#reactor-read"},{"type":"DSL","title":"reactor.read.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-read-action"},{"type":"DSL","title":"reactor.read.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-read-domain"},{"type":"DSL","title":"reactor.read.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-read-async?"},{"type":"DSL","title":"reactor.read.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-read-authorize?"},{"type":"DSL","title":"reactor.read.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-read-description"},{"type":"DSL","title":"reactor.read.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-read-name"},{"type":"DSL","title":"reactor.read.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-read-resource"},{"type":"DSL","title":"reactor.read.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-read-actor"},{"type":"DSL","title":"reactor.read.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-read-actor-source"},{"type":"DSL","title":"reactor.read.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read-actor-transform"},{"type":"DSL","title":"reactor.read.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-read-context"},{"type":"DSL","title":"reactor.read.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-read-context-context"},{"type":"DSL","title":"reactor.read.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read-context-transform"},{"type":"DSL","title":"reactor.read.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-read-guard"},{"type":"DSL","title":"reactor.read.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-read-guard-fun"},{"type":"DSL","title":"reactor.read.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-read-guard-description"},{"type":"DSL","title":"reactor.read.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-read-where"},{"type":"DSL","title":"reactor.read.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-read-where-predicate"},{"type":"DSL","title":"reactor.read.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-read-where-description"},{"type":"DSL","title":"reactor.read.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-read-inputs"},{"type":"DSL","title":"reactor.read.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-read-inputs-template"},{"type":"DSL","title":"reactor.read.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-read-inputs-transform"},{"type":"DSL","title":"reactor.read.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-read-load"},{"type":"DSL","title":"reactor.read.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-read-load-source"},{"type":"DSL","title":"reactor.read.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read-load-transform"},{"type":"DSL","title":"reactor.read.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-read-tenant"},{"type":"DSL","title":"reactor.read.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-read-tenant-source"},{"type":"DSL","title":"reactor.read.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-read-tenant-transform"},{"type":"DSL","title":"reactor.read.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-read-wait_for"},{"type":"DSL","title":"reactor.read.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-read-wait_for-names"},{"type":"DSL","title":"reactor.read.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-read-wait_for-description"},{"type":"DSL","title":"reactor.transaction - dsl-ash-reactor","doc":"Creates a group of steps which will be executed inside a data layer transaction.","ref":"dsl-ash-reactor.html#reactor-transaction"},{"type":"DSL","title":"reactor.transaction.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-transaction-name"},{"type":"DSL","title":"reactor.transaction.resources - dsl-ash-reactor","doc":"A resource or list of resources to consider in the transaction.","ref":"dsl-ash-reactor.html#reactor-transaction-resources"},{"type":"DSL","title":"reactor.transaction.return - dsl-ash-reactor","doc":"The name of the step whose result will be returned as the return value of the transaction.","ref":"dsl-ash-reactor.html#reactor-transaction-return"},{"type":"DSL","title":"reactor.transaction.timeout - dsl-ash-reactor","doc":"How long to allow the transaction to run before timing out.","ref":"dsl-ash-reactor.html#reactor-transaction-timeout"},{"type":"DSL","title":"reactor.transaction.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-transaction-guard"},{"type":"DSL","title":"reactor.transaction.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-transaction-guard-fun"},{"type":"DSL","title":"reactor.transaction.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-transaction-guard-description"},{"type":"DSL","title":"reactor.transaction.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-transaction-where"},{"type":"DSL","title":"reactor.transaction.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-transaction-where-predicate"},{"type":"DSL","title":"reactor.transaction.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-transaction-where-description"},{"type":"DSL","title":"reactor.transaction.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-transaction-wait_for"},{"type":"DSL","title":"reactor.transaction.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-transaction-wait_for-names"},{"type":"DSL","title":"reactor.transaction.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-transaction-wait_for-description"},{"type":"DSL","title":"reactor.update - dsl-ash-reactor","doc":"Declares a step that will call an update action on a resource.\n\n> #### Undo behaviour {: .tip}\n>\n> This step has three different modes of undo.\n>\n> * `never` - The result of the action is never undone.  This is the default.\n> * `always` - The `undo_action` will always be called.\n> * `outside_transaction` - The `undo_action` will not be called when running inside a `transaction` block, but will be otherwise.","ref":"dsl-ash-reactor.html#reactor-update"},{"type":"DSL","title":"reactor.update.initial - dsl-ash-reactor","doc":"The record to update.","ref":"dsl-ash-reactor.html#reactor-update-initial"},{"type":"DSL","title":"reactor.update.action - dsl-ash-reactor","doc":"The name of the action to call on the resource.","ref":"dsl-ash-reactor.html#reactor-update-action"},{"type":"DSL","title":"reactor.update.domain - dsl-ash-reactor","doc":"The Domain to use when calling the action.  Defaults to the Domain set on the resource or in the `ash` section.","ref":"dsl-ash-reactor.html#reactor-update-domain"},{"type":"DSL","title":"reactor.update.async? - dsl-ash-reactor","doc":"When set to true the step will be executed asynchronously via Reactor's `TaskSupervisor`.","ref":"dsl-ash-reactor.html#reactor-update-async?"},{"type":"DSL","title":"reactor.update.authorize? - dsl-ash-reactor","doc":"Explicitly enable or disable authorization for the action.","ref":"dsl-ash-reactor.html#reactor-update-authorize?"},{"type":"DSL","title":"reactor.update.description - dsl-ash-reactor","doc":"A description for the step","ref":"dsl-ash-reactor.html#reactor-update-description"},{"type":"DSL","title":"reactor.update.name - dsl-ash-reactor","doc":"A unique name for the step.","ref":"dsl-ash-reactor.html#reactor-update-name"},{"type":"DSL","title":"reactor.update.resource - dsl-ash-reactor","doc":"The resource to call the action on.","ref":"dsl-ash-reactor.html#reactor-update-resource"},{"type":"DSL","title":"reactor.update.undo_action - dsl-ash-reactor","doc":"The name of the action to call on the resource when the step is to be undone.","ref":"dsl-ash-reactor.html#reactor-update-undo_action"},{"type":"DSL","title":"reactor.update.undo - dsl-ash-reactor","doc":"How to handle undoing this action","ref":"dsl-ash-reactor.html#reactor-update-undo"},{"type":"DSL","title":"reactor.update.actor - dsl-ash-reactor","doc":"Specifies the action actor","ref":"dsl-ash-reactor.html#reactor-update-actor"},{"type":"DSL","title":"reactor.update.actor.source - dsl-ash-reactor","doc":"What to use as the source of the actor.","ref":"dsl-ash-reactor.html#reactor-update-actor-source"},{"type":"DSL","title":"reactor.update.actor.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the actor before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-update-actor-transform"},{"type":"DSL","title":"reactor.update.context - dsl-ash-reactor","doc":"A map to be merged into the action's context","ref":"dsl-ash-reactor.html#reactor-update-context"},{"type":"DSL","title":"reactor.update.context.context - dsl-ash-reactor","doc":"A map to be merged into the action's context.","ref":"dsl-ash-reactor.html#reactor-update-context-context"},{"type":"DSL","title":"reactor.update.context.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the context before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-update-context-transform"},{"type":"DSL","title":"reactor.update.guard - dsl-ash-reactor","doc":"Provides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-ash-reactor.html#reactor-update-guard"},{"type":"DSL","title":"reactor.update.guard.fun - dsl-ash-reactor","doc":"The guard function.","ref":"dsl-ash-reactor.html#reactor-update-guard-fun"},{"type":"DSL","title":"reactor.update.guard.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-update-guard-description"},{"type":"DSL","title":"reactor.update.where - dsl-ash-reactor","doc":"Only execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-ash-reactor.html#reactor-update-where"},{"type":"DSL","title":"reactor.update.where.predicate - dsl-ash-reactor","doc":"Provide a function which takes the step arguments and optionally the context and returns a boolean value.","ref":"dsl-ash-reactor.html#reactor-update-where-predicate"},{"type":"DSL","title":"reactor.update.where.description - dsl-ash-reactor","doc":"An optional description of the guard.","ref":"dsl-ash-reactor.html#reactor-update-where-description"},{"type":"DSL","title":"reactor.update.inputs - dsl-ash-reactor","doc":"Specify the inputs for an action","ref":"dsl-ash-reactor.html#reactor-update-inputs"},{"type":"DSL","title":"reactor.update.inputs.template - dsl-ash-reactor","doc":"","ref":"dsl-ash-reactor.html#reactor-update-inputs-template"},{"type":"DSL","title":"reactor.update.inputs.transform - dsl-ash-reactor","doc":"An optional transformation function which will transform the inputs before executing the action.","ref":"dsl-ash-reactor.html#reactor-update-inputs-transform"},{"type":"DSL","title":"reactor.update.load - dsl-ash-reactor","doc":"Allows the addition of an Ash load statement to the action","ref":"dsl-ash-reactor.html#reactor-update-load"},{"type":"DSL","title":"reactor.update.load.source - dsl-ash-reactor","doc":"What to use as the source of the load","ref":"dsl-ash-reactor.html#reactor-update-load-source"},{"type":"DSL","title":"reactor.update.load.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the load before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-update-load-transform"},{"type":"DSL","title":"reactor.update.tenant - dsl-ash-reactor","doc":"Specifies the action tenant","ref":"dsl-ash-reactor.html#reactor-update-tenant"},{"type":"DSL","title":"reactor.update.tenant.source - dsl-ash-reactor","doc":"What to use as the source of the tenant.","ref":"dsl-ash-reactor.html#reactor-update-tenant-source"},{"type":"DSL","title":"reactor.update.tenant.transform - dsl-ash-reactor","doc":"An optional transformation function which can be used to modify the tenant before it is passed to the action.","ref":"dsl-ash-reactor.html#reactor-update-tenant-transform"},{"type":"DSL","title":"reactor.update.wait_for - dsl-ash-reactor","doc":"Wait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-ash-reactor.html#reactor-update-wait_for"},{"type":"DSL","title":"reactor.update.wait_for.names - dsl-ash-reactor","doc":"The name of the step to wait for.","ref":"dsl-ash-reactor.html#reactor-update-wait_for-names"},{"type":"DSL","title":"reactor.update.wait_for.description - dsl-ash-reactor","doc":"An optional description.","ref":"dsl-ash-reactor.html#reactor-update-wait_for-description"},{"type":"extras","title":"Glossary","doc":"# Glossary","ref":"glossary.html"},{"type":"extras","title":"Action - Glossary","doc":"An action describes an operation that can be performed for a given resource; it is the _verb_ to a resource's _noun_. Examples of actions:\n\n- User._create_\n- Comment._delete_\n- BlogPost._publish_\n- Article._search_\n\nAsh supports five different types of actions. `create`, `read`, `update` and `destroy` (collectively often abbreviated as CRUD), and `action`, referring to a generic action with a custom return type. A resource can define multiple actions per action type, eg. a _publish_ action would be considered an `update` because it is updating an existing instance of a resource. Actions are much more flexible than simple CRUD, but these five action types serve as templates for anything you might want to do.\n\nSee the [Actions guide](/documentation/topics/actions/actions.md) for more.","ref":"glossary.html#action"},{"type":"extras","title":"Actor - Glossary","doc":"The entity that performs an action. \n\nMost actions are run on direct user request, eg. if a user presses a Create button on a page then the actor is the user.\n\nThe actor can be anything that you want it to be. It is most typically a map or a struct containing information about the \"entity\" that is performing the action.\nIn the vast majority of cases, the actor will be something like `%MyApp.Accounts.User{}`. We recommend that the actor be a struct, but it could also be a map or any other kind of value.\n\nSome example actor types used in practice:\n\n- `%MyApp.Accounts.User{}`\n- `%MyApp.Accounts.Device{}`\n- `%MyApp.SystemUser{}`\n\nActors can be used in a number of places, from modifying the behavior of an action to auditing who did what in your system. They are most prominent, however, when writing [policies](#policy).\n\nSee the [Actors & Authorization guide](/documentation/topics/security/actors-and-authorization.md) for more.","ref":"glossary.html#actor"},{"type":"extras","title":"Aggregate - Glossary","doc":"An aggregate is a special type of field for a resource, one that summarizes related information of the record. A more specialized type of a [calculation](#calculation).\n\nIf a Project resource has_many Ticket resources, an example of an aggregate on the Project might be to count the tickets associated to each project.\n\nSee the [Aggregates guide](/documentation/topics/resources/aggregates.md) for more.","ref":"glossary.html#aggregate"},{"type":"extras","title":"Attribute - Glossary","doc":"A piece of data belonging to a resource. The most basic building block; an attribute has a type and a value. For resources backed by a data layer, they typically represent a column in a database table, or a key in an object store, for example.\n\nSee the [Attributes guide](/documentation/topics/resources/attributes.md) for more.","ref":"glossary.html#attribute"},{"type":"extras","title":"Authorizer - Glossary","doc":"An authorizer is an extension that can be added to a resource that will be given the opportunity to modify and/or prevent requests to a resource. In practice, you will almost always be using `Ash.Policy.Authorizer`, but you can still write your own if you need to.\n\nSee the [Actors & Authorization](documentation/topics/security/actors-and-authorization.md) and [Policies](documentation/topics/security/policies.md) guides for more.","ref":"glossary.html#authorizer"},{"type":"extras","title":"Calculation - Glossary","doc":"A calculation is a special type of field for a resource, one that is not directly stored in the data layer but generated on-demand. Typically it will derive from other information on the record, but it may come from some other data source entirely.\n\nSee the [Calculations guide](/documentation/topics/resources/calculations.md) for more.","ref":"glossary.html#calculation"},{"type":"extras","title":"Changeset - Glossary","doc":"Changesets encapsulate data changes made while creating or updating an instance of a resource. Similarly to [Ecto changesets](https://hexdocs.pm/ecto/Ecto.Changeset.html), they include data validations but they also have their own callback hook lifecycle.\n\nSee `Ash.Changeset` for more.","ref":"glossary.html#changeset"},{"type":"extras","title":"Domain - Glossary","doc":"A method of broadly separating resources into different [domains](https://en.wikipedia.org/wiki/Domain_(software_engineering)), A.K.A [bounded contexts](https://martinfowler.com/bliki/BoundedContext.html).\n\nSee the [Domains guide](/documentation/topics/domains.md) for more.","ref":"glossary.html#domain"},{"type":"extras","title":"Extension - Glossary","doc":"A packaged bundle of code that can be included in a resource to provide additional functionality. Built-in functionality such as the resource DSL itself is provided by an extension, and libraries like AshPostgres and AshAdmin also provide extensions that you can add to your resources with just one line of code.\n\nSee [Extending Resources](/documentation/topics/advanced/writing-extensions.md) for more.","ref":"glossary.html#extension"},{"type":"extras","title":"Filter - Glossary","doc":"Filters are applied to queries to limit the data returned. They can also be applied to changesets, to ensure only data matching a certain condition is updated. For example:\n\n- Fetching Articles that include a certain search term in the title\n- Fetching Posts created by a specific user\n- Fetching Tickets updated in the last week\n- Updating a record only if it's version matches your in memory version (optimistic locking)\n\nSee `Ash.Filter` for more.","ref":"glossary.html#filter"},{"type":"extras","title":"Identity - Glossary","doc":"A way to uniquely identify an instance of a resource. A primary key is an example of an identity that is automatically generated; you can manually add others such as a user's email address, or a URL slug for a post. If using AshPostgres, constraints will be created by the migration generator to enforce identities at the database level.\n\nSee the [Identities guide](/documentation/topics/resources/identities.md) for more.","ref":"glossary.html#identity"},{"type":"extras","title":"Notifier - Glossary","doc":"Notifiers are modules that are called for each action that occurs on a resource (except generic actions). They are called at the end of transactions, meaning that if a notifier is called, it is guaranteed that the action they pertain to has completed successfully.\n\nSee the [Notifiers guide](/documentation/topics/resources/notifiers.md) for more.","ref":"glossary.html#notifier"},{"type":"extras","title":"Policy - Glossary","doc":"A set of rules defining who is authorized to perform specific actions on a resource. Common policy checks include rules such as:\n\n- Forbidding anyone other than the user who wrote a blog post, from editing it\n- Allowing only admins to update site-wide settings\n\nSee the [Policies guide](/documentation/topics/security/policies.md) for more.","ref":"glossary.html#policy"},{"type":"extras","title":"Query - Glossary","doc":"The tools and functions used for reading and filtering stored data, from the data layer.\n\nSee `Ash.Query` for more.","ref":"glossary.html#query"},{"type":"extras","title":"Record - Glossary","doc":"A record is an \"instance\" of a resource. Keep in mind that this is not an \"instance\" in a mutable/object-oriented sense, but rather a snapshot of the data at a given point in time. When you call a read action, you get back records. You can provide a record to an update action to determine what data is being updated.","ref":"glossary.html#record"},{"type":"extras","title":"Relationship - Glossary","doc":"Relationships are named links between resources, that define how they relate to each other. Relationships can be used to signify ownership of a record, membership of a group, or can be used in filtering and querying data.\n\nSee the [Relationships guide](/documentation/topics/resources/relationships.md) for more.","ref":"glossary.html#relationship"},{"type":"extras","title":"Resource - Glossary","doc":"The central concept in Ash, a resource can be used to model all kinds of things. Most often, they will map to a data store, and represent things like rows of a database table. However, they can also be backed by external data sources, be used for validating data with no persistence at all, or even be simple containers for generic actions, completely stateless.\n\nSee the [Resource DSL docs](dsl-ash-resource.html) for DSL documentation.","ref":"glossary.html#resource"},{"type":"extras","title":"Tenant - Glossary","doc":"Multitenancy is the siloing of your app's data into discrete non-overlapping groups, typically by customer or organization (the tenant). Ash supports multitenancy both at the code level and the data layer level (depending on the data layer; for example, AshPostgres uses schemas to fully separate data per tenant.)\n\nSee the [Multitenancy guide](/documentation/topics/advanced/multitenancy.md) for more.","ref":"glossary.html#tenant"},{"type":"extras","title":"Expressions","doc":"# Expressions\n\nAsh expressions are used in various places like calculations, filters, and policies, and are meant to be portable representations of elixir expressions. You can create an expression using the `Ash.Expr.expr/1` macro, like so:\n\n```elixir\nAsh.Expr.expr(1 + 2)\nAsh.Expr.expr(x + y)\nAsh.Expr.expr(post.title <> \" | \" <> post.subtitle)\n```\n\n>","ref":"expressions.html"},{"type":"extras","title":"Ash Expressions are SQL-ish - Expressions","doc":">\n> Ash expressions have some interesting properties in their evaluation, primarily because they are made to be\n> portable, i.e executable in some data layer (like SQL) or executable in Elixir. In general, these expressions\n> will behave the same way they do in Elixir. The primary difference is how `nil` values work. They behave the way\n> that `NULL` values behave in SQL. This is primarily because this pattern is easier to replicate to various popular\n> data layers, and is generally safer when using expressions for things like authentication. The practical\n> implications of this are that `nil` values will \"poison\" many expressions, and cause them to return `nil`.\n> For example, `x + nil` would always evaluate to `nil`. Additionally, `true and nil` will always result in\n> `nil`, _this is also true with or and not_, i.e `true or nil` will return `nil`, and `not nil` will return `nil`.\n>\n> Additionally, atoms and strings compare as if the atom was a string. This is because most external data layers\n> do not know about atoms, and so they are converted to strings before comparison.","ref":"expressions.html#ash-expressions-are-sql-ish"},{"type":"extras","title":"Operators - Expressions","doc":"The following operators are available and they behave the same as they do in Elixir, except for the `nil` addendum above.\n\n- `==`\n- `!=`\n- `>`\n- `>=`\n- `<`\n- `<=`\n- `in`\n- `*`\n- `-`\n- `/`\n- `<>`\n- `and` - Boolean and operator\n- `or` - Boolean or operator\n- `||` - Elixir-ish or operator, if left is not `nil` or `false`, then left. Othewise right.\n- `&&` - Elixir-ish and operator, if left is not `nil` or `false`, then right. Otherwise left.\n- `is_nil` | Only works as an operator in maps/keyword list syntax. i.e `[x: [is_nil: true]]`","ref":"expressions.html#operators"},{"type":"extras","title":"Elixir-ish operators - Expressions","doc":"Prefer to use `and` and `or` if you are dealing with booleans, as they will typically perform much\nbetter in SQL data layers. `&&` and `||` should only be used when you want to deal with more than boolaens.\n\nFor example:\n\n```elixir\ncalculate :identifier, expr(manual_identifier || employee_id <> \" \" <> location_code)\n```","ref":"expressions.html#elixir-ish-operators"},{"type":"extras","title":"Functions - Expressions","doc":"The following functions are built in. Data Layers can add their own functions to expressions. For example, `AshPostgres` adds `trigram_similarity` function.\n\nThe following functions are built in:\n\n- `if` | Works like elixir's `if`.\n- `is_nil/1` | Works like elixir's `is_nil`\n- `get_path/2` | i.e `get_path(value, [\"foo\", \"bar\"])`. This is what expressions like `value[:foo][\"bar\"]` are turned into under the hood.\n- `contains/2` | if one string contains another string, i.e `contains(\"fred\", \"red\")`\n- `length/1` | the length of a list, i.e. `length([:foo, :bar])`\n- `type/2` | Cast a given value to a specific type, i.e `type(^arg(:id), :uuid)` or `type(integer_field, :string)`\n- `string_downcase/1` | Downcases a string\n- `string_join/1` | Concatenates a list of strings, and ignores any nil values\n- `string_join/2` | As above, but with a joiner\n- `string_position/2` | Returns the zero-based position of a substring within a string, or nil, i.e. `string_position(\"fred\", \"red\") == 1`\n- `string_split/1` | Splits a string on spaces\n- `string_split/2` | As above, but with a specific delimiter\n- `string_split/3` | As above, but with options. See the function for the available options.\n- `string_length/1` | Returns the length of a given string, as reported by `String.length/1`\n- `string_trim/1` | Trims unicode whitespace from the beginning and the end of a string\n- `at/2` | Get an element from a list, i.e `at(list, 1)`\n- `round/1` | Round a float, decimal or int to 0 precision, i.e `round(num)`\n- `round/2` | Round a float, decimal or int to the provided precision or less, i.e `round(1.1234, 3) == 1.1234` and `round(1.12, 3) == 1.12`\n- String interpolation | `\"#{first_name} #{last_name}\"`, is remapped to the equivalent usage of `<>`\n- `fragment/*` | Creates a fragment of a data layer expression. See the section on fragments below.","ref":"expressions.html#functions"},{"type":"extras","title":"Fragments - Expressions","doc":"Fragments come in two forms.","ref":"expressions.html#fragments"},{"type":"extras","title":"String Fragments - Expressions","doc":"For SQL/query-backed data layers, they will be a string with question marks for interpolation. For example: `fragment(\"(? + ?)\", foo, bar)`.","ref":"expressions.html#string-fragments"},{"type":"extras","title":"Function Fragments - Expressions","doc":"For elixir-backed data layers, they will be a function or an MFA that will be called with the provided arguments. For example: `fragment(&Module.add/2, foo, bar)` or `fragment({Module, :add, []}, foo, bar)`. When using anonymous functions, you can _only_ use the format `&Module.function/arity`. `&Module.add/2` is okay, but `fn a, b -> Module.add(a, b) end` is not.","ref":"expressions.html#function-fragments"},{"type":"extras","title":"Sub-expressions - Expressions","doc":"- `exists/2` | `exists(foo.bar, name == \"fred\")` takes an expression scoped to the destination resource, and checks if any related entry matches. See the section on `exists` below.\n- `path.exists/2` | Same as `exists` but the source of the relationship is itself a nested relationship. See the section on `exists` below.\n- `parent/1` | Allows an expression scoped to a resource to refer to the \"outer\" context. Used in relationship filters and `exists`","ref":"expressions.html#sub-expressions"},{"type":"extras","title":"DateTime Functions - Expressions","doc":"- `now/0` | Evaluates to the current time when the expression is evaluated\n- `today/0` | Evaluates to the current date when the expression is evaluated\n- `ago/2` | i.e `deleted_at > ago(7, :day)`. The available time intervals are documented in `Ash.Type.DurationName`\n- `from_now/2` | Same as `ago` but adds instead of subtracting\n- `datetime_add/3` | add an interval to a datetime, i.e `datetime_add(^datetime, 10, :hour)`\n- `date_add/3` | add an interval to a date, i.e `date_add(^date, 3, :day)`\n- `start_of_day/1-2` | Converts a date or a datetime to the correspond start of its day (at 00:00 time).","ref":"expressions.html#datetime-functions"},{"type":"extras","title":"Primitives - Expressions","doc":"- `cond` - `cond` is transformed to a series of `if` expressions under the hood\n- `item[:key] or item[\"key\"]` - accesses keys in a map. In both cases, it prefers a matching atom key, falling back to a matching string key. This is to aid with data stores that store embeds as JSON with string keys (like AshPostgres), so that this expression behaves the same in the data layer as it does in Elixir.","ref":"expressions.html#primitives"},{"type":"extras","title":"Escape Hatches - Expressions","doc":"- `lazy/1` - Takes an MFA and evaluates it just before running the query. This is important for calculations, because the `expression/2` callback should be _stable_ (returns the same value given the same input). For example `lazy({ULID, :generate, [timestamp_input]})`","ref":"expressions.html#escape-hatches"},{"type":"extras","title":"Inline Aggregates - Expressions","doc":"Aggregates can be referenced in-line, with their relationship path specified and any options provided that match the options given to `Ash.Query.Aggregate.new/4`. For example:\n\n```elixir\ncalculate :grade, :decimal, expr(\n  count(answers, query: [filter: expr(correct == true)]) /\n  count(answers, query: [filter: expr(correct == false)])\n)\n```\n\nThe available aggregate kinds can also be seen in the `Ash.Query.Aggregate` module documentation.","ref":"expressions.html#inline-aggregates"},{"type":"extras","title":"Templates - Expressions","doc":"Most of the time, when you are using an expression, you will actually be creating a *template*. In this template, you have a few references that can be used, which will be replaced before the expression is evaluated. The following references are available:\n\n```elixir\n^actor(:key) # equivalent to `get_in(actor || %{}, [:key])`\n^actor([:key1, :key2]) # equivalent to `get_in(actor || %{}, [:key, :key2])`\n^arg(:arg_name) # equivalent to `Map.get(arguments, :arg_name)`\n^context(:key) # equivalent to `get_in(context, :key)`\n^context([:key1, :key2]) # equivalent to `get_in(context, [:key1, :key2])`\n^ref(:key) # equivalent to referring to `key`. Allows for dynamic references\n^ref([:path], :key) # equivalent to referring to `path.key`. Allows for dynamic references with dynamic (or static) paths.\n```","ref":"expressions.html#templates"},{"type":"extras","title":"Custom Expressions - Expressions","doc":"Custom expressions allow you to extend Ash's expression language with your own expressions. To see more, see `Ash.CustomExpression`. To add a custom expression, configure it and recompile ash. For example:\n\n```elixir\nconfig :ash, :custom_expressions, [\n  MyApp.CustomExpression\n]\n```\n\n```\nmix deps.compile ash --force\n```\n\nThese expressions will be available across all usages of Ash expressions within your application.","ref":"expressions.html#custom-expressions"},{"type":"extras","title":"Filter semantics & joins - Expressions","doc":"The semantics of Ash filters are probably slightly different than what you are used to, and they are important to understand. Every filter expression is always talking about a single row, potentially \"joined\" to single related rows. By referencing relationships, you are implicitly doing a join. For those familiar with SQL terminology, it is equivalent to a left join, although AshPostgres can detect when it is safe to do an inner join (for performance reasons). Lets use an example of `posts` and `comments`.\n\nGiven a filter like the following:\n\n```elixir\nAsh.Query.filter(Post, comments.points > 10 and comments.tag.name == \"elixir\")\n```\n\nThe filter refers to a _single post/comment/tag combination_. So in english, this is \"posts where they have a comment with more than 10 points and _that same comment_ has a tag with the name `elixir`\". What this also means is that filters like the above do not compose nicely when new filters are added. For example:\n\n```elixir\ndef has_comment_with_more_points_than(query, score) do\n  Ash.Query.filter(query, comments.points > ^score)\nend\n\ndef has_comment_tagged(query, tag) do\n  Ash.Query.filter(query, comments.tag.name == ^tag)\nend\n\nPost\n|> has_comment_with_more_points_than(10)\n|> has_comment_tagged(\"elixir\")\n```\n\nThat code _seems_ like it ought to produce a filter over `Post` that would give us any post with a comment having more than 10 points, _and_ with a comment tagged `elixir`. That is not the same thing as having a _single_ comment that meets both those criteria. So how do we make this better?","ref":"expressions.html#filter-semantics-joins"},{"type":"extras","title":"Exists - Expressions","doc":"Lets rewrite the above using exists:\n\n```elixir\ndef has_comment_with_more_points_than(query, score) do\n  Ash.Query.filter(query, exists(comments, points > ^score))\nend\n\ndef has_comment_tagged(query, tag) do\n  Ash.Query.filter(query, exists(comments.tag, name == ^tag))\nend\n\nPost\n|> has_comment_with_more_points_than(10)\n|> has_comment_tagged(\"elixir\")\n```\n\nNow, they will compose properly! Generally speaking, you should use exists when you are filtering across any relationships that are `to_many` relationships \\*even if you don't expect your filter to be composed. Currently, the filter syntax does not minimize(combine) these `exists/2` statements, but doing so is not complex and can be added. While unlikely, please lodge an issue if you see any performance issues with `exists`.","ref":"expressions.html#exists"},{"type":"extras","title":"Exists at path - Expressions","doc":"Sometimes, you want the ability to say that some given row must have an existing related entry matching a filter. For example:\n\n```elixir\nAsh.Query.filter(Post, author.exists(roles, name == :admin) and author.active)\n```\n\nWhile the above is not common, it can be useful in some specific circumstances, and is used under the hood by the policy authorizer when combining the filters of various resources to create a single filter.","ref":"expressions.html#exists-at-path"},{"type":"extras","title":"Portability - Expressions","doc":"Ash expressions being portable is more important than it sounds. For example, if you were using AshPostgres and had the following calculation, which is an expression capable of being run in elixir or translated to SQL:\n\n```elixir\ncalculate :full_name, :string, expr(first_name <> \" \" <> last_name)\n```\n\nAnd you did something like the following:\n\n```elixir\nUser\n|> Ash.Query.load(:full_name)\n|> Ash.Query.sort(:full_name)\n|> Accounts.read!()\n```\n\nYou would see that it ran a SQL query with the `full_name` calculation as SQL. This allows for sorting on that value. However, if you had something like this:\n\n```elixir\n# data can be loaded in the query like above, or on demand later\nAccounts.load!(user, :full_name, reuse_values?: true)\n```\n\nyou would see that no SQL queries are run. The calculation is run directly in Elixir without needing to visit the database.","ref":"expressions.html#portability"},{"type":"extras","title":"Parent - Expressions","doc":"`Parent` is a way to \"jump out\" of a scoped expression. Here are some examples:\n\n```elixir\nAsh.Query.filter(exists(open_tickets, severity >= parent(severity_threshold)))\n```\n\n```elixir\nhas_many :relevant_tickets, Ticket do\n  no_attributes? true\n  # this says that there is no matching source_attribute and destination_attribute on this relationship\n  filter expr(status == :open and severity >= parent(severity_threshold))\nend\n```\n\n```elixir\ncount :count_of_relevant_tickets, :open_tickets do\n  filter expr(status == :open and severity >= parent(severity_threshold))\nend\n```","ref":"expressions.html#parent"},{"type":"extras","title":"Referencing related values - Expressions","doc":"Related values can be references using dot delimiters, i.e `Ash.Query.filter(user.first_name == \"fred\")`.\nWhen referencing related values in filters, if the reference is a `has_one` or `belongs_to`, the filter does exactly what it looks like (matches if the related value matches). If it is a `has_many` or a `many_to_many`, it matches if any of the related records match.","ref":"expressions.html#referencing-related-values"},{"type":"extras","title":"Referencing aggregates and calculations - Expressions","doc":"Aggregates are simple, as all aggregates can be referenced in filter expressions (if you are using a data layer that supports aggregates).\n\nFor calculations, only those that define an expression can be referenced in other expressions.\n\nHere are some examples:\n\n```elixir\n# given a `full_name` calculation\n\nAsh.Query.filter(User, full_name == \"Hob Goblin\")\n\n# given a `full_name` calculation that accepts an argument called `delimiter`\n\nAsh.Query.filter(User, full_name(delimiter: \"~\") == \"Hob~Goblin\")\n```","ref":"expressions.html#referencing-aggregates-and-calculations"},{"type":"extras","title":"Case vs Cond Expressions - Expressions","doc":"When working with conditional expressions in Ash, you should use `cond` instead of `case` statements. Here's an example:\n\n```elixir\n# This works - using cond\ncalculations do\n  calculate :user_order, :integer, expr(\n    cond do\n      role == :principal -> 1\n      role == :teacher -> 2\n      role == :student -> 3\n    end\n  )\nend\n\n# This doesn't work - using case\ncalculations do\n  calculate :user_order, :integer, expr(\n    case role do\n      :principal -> 1\n      :teacher -> 2\n      :student -> 3\n    end\n  )\nend\n```\n\nThe `cond` expression is the correct way to handle conditional logic in Ash expressions.","ref":"expressions.html#case-vs-cond-expressions"},{"type":"extras","title":"Domains","doc":"# Domains\n\nDomains serve three primary purposes:\n\n1. They group related resources together, providing organization and structure to your project.\n2. They allow you to define a centralized [code interface](/documentation/topics/resources/code-interfaces.md)\n3. They allow you to configure certain cross-cutting concerns of those resources in a single place.\n\nIf you are familiar with a [Phoenix Context](https://hexdocs.pm/phoenix/contexts.html), you can think of a domain as the Ash equivalent.","ref":"domains.html"},{"type":"extras","title":"Grouping Resources - Domains","doc":"In an `Ash.Domain`, you will typically see something like this:\n\n```elixir\ndefmodule MyApp.Tweets do\n  use Ash.Domain\n\n  resources do\n    resource MyApp.Tweets.Tweet\n    resource MyApp.Tweets.Comment\n  end\nend\n```\n\nWith this definition, you can do things like placing all of these resources into a GraphQL Api with AshGraphql. You'd see a line like this:\n\n```elixir\nuse AshGraphql, domains: [MyApp.Tweets]\n```","ref":"domains.html#grouping-resources"},{"type":"extras","title":"Centralized Code Interface - Domains","doc":"Working with our domain & resources in code *can* be done the long form way, by building changesets/queries/action inputs and calling the relevant function in `Ash`. However, we generally want to expose a well defined code API for working with our resources. This makes our code much clearer, and gives us nice things like auto complete and inline documentation.\n\n```elixir\ndefmodule MyApp.Tweets do\n  use Ash.Domain\n\n  resources do\n    resource MyApp.Tweets.Tweet do\n      # define a function called `tweet` that uses\n      # the `:create` action on MyApp.Tweets.Tweet\n      define :tweet, action: :create, args: [:text]\n    end\n\n    resource MyApp.Tweets.Comment do\n      # define a function called `comment` that uses\n      # the `:create` action on MyApp.Tweets.Comment\n      define :comment, action: :create, args: [:tweet_id, :text]\n    end\n  end\nend\n```\n\nWith these definitions, we can now do things like this:\n\n```elixir\ntweet = MyApp.Tweets.tweet!(\"My first tweet!\", actor: user1)\ncomment = MyApp.Tweets.comment!(tweet.id, \"What a cool tweet!\", actor: user2)\n```","ref":"domains.html#centralized-code-interface"},{"type":"extras","title":"Configuring Cross-cutting Concerns - Domains","doc":"","ref":"domains.html#configuring-cross-cutting-concerns"},{"type":"extras","title":"Built in configuration - Domains","doc":"`Ash.Domain` comes with a number of built-in configuration options. See `d:Ash.Domain` for more.\n\nFor example:\n\n```elixir\ndefmodule MyApp.Tweets do\n  use Ash.Domain\n\n  resources do\n    resource MyApp.Tweets.Tweet\n    resource MyApp.Tweets.Comment\n  end\n\n  execution do\n    # raise the default timeout for all actions in this domain from 30s to 60s\n    timeout :timer.seconds(60)\n  end\n\n  authorization do\n    # disable using the authorize?: false flag when calling actions\n    authorize :always\n  end\nend\n```","ref":"domains.html#built-in-configuration"},{"type":"extras","title":"Extensions - Domains","doc":"Extensions will often come with \"domain extensions\" to allow you to configure the behavior of all resources within a domain, as it pertains to that extension. For example:\n\n```elixir\ndefmodule MyApp.Tweets do\n  use Ash.Domain,\n    extensions: [AshGraphql.Domain]\n\n  graphql do\n    # skip authorization for these resources\n    authorize? false\n  end\n\n  resources do\n    resource MyApp.Tweets.Tweet\n    resource MyApp.Tweets.Comment\n  end\nend\n```","ref":"domains.html#extensions"},{"type":"extras","title":"Policies - Domains","doc":"You can also use `Ash.Policy.Authorizer` on your domains. This allows you to add policies that apply to *all* actions using this domain. For example:\n\n```elixir\ndefmodule MyApp.Tweets do\n  use Ash.Domain,\n    extensions: [Ash.Policy.Authorizer]\n\n  resources do\n    resource MyApp.Tweets.Tweet\n    resource MyApp.Tweets.Comment\n  end\n\n  policies do\n    # add a bypass up front to allow administrators to do whatever they want\n    bypass actor_attribute_equals(:is_admin, true) do\n      authorize_if always()\n    end\n\n    # forbid all access from disabled users\n    policy actor_attribute_equals(:disabled, true) do\n      forbid_if always()\n    end\n  end\nend\n```","ref":"domains.html#policies"},{"type":"extras","title":"Attributes","doc":"# Attributes\n\nAttributes specify the `name`, `type` and additional configuration of a simple property of a record. When using SQL data layers, for example, an attribute would correspond to a column in a database table. For information on types, see `Ash.Type`.\n\nTo see all of the options available when building attributes, see `d:Ash.Resource.Dsl.attributes.attribute`\n\nIf you are looking to compute values on demand, see the [Calculations guide](/documentation/topics/resources/calculations.md) and the [aggregates guide](/documentation/topics/resources/aggregates.md).","ref":"attributes.html"},{"type":"extras","title":"Special attributes - Attributes","doc":"In Ash there are 4 special attributes these are:\n\n- `create_timestamp`\n- `update_timestamp`\n- `integer_primary_key`\n- `uuid_primary_key`\n\nThese are really just shorthand for an attribute with specific options set. They're outlined below.\n\n### `create_timestamp`\n\nYou may recognise this if you have used Ecto before. This attribute will record the time at which each row is created, by default it uses `DateTime.utc_now/1`.\n\n`create_timestamp :inserted_at` is equivalent to an attribute with these options:\n\n```elixir\nattribute :inserted_at, :utc_datetime_usec do\n  writable? false\n  default &DateTime.utc_now/0\n  match_other_defaults? true\n  allow_nil? false\nend\n```\n\n### `update_timestamp`\n\nThis is also similar in Ecto. This attribute records the last time a row was updated, also using `DateTime.utc_now/1` by default.\n\n`update_timestamp :updated_at` is equivalent to:\n\n```elixir\nattribute :updated_at, :utc_datetime_usec do\n  writable? false\n  default &DateTime.utc_now/0\n  update_default &DateTime.utc_now/0\n  match_other_defaults? true\n  allow_nil? false\nend\n```\n\n### `uuid_primary_key`\n\nThis attribute is used in almost every resource. It generates a UUID every time a new record is made.\n`uuid_primary_key :id` is equivalent to:\n\n```elixir\nattribute :id, :uuid do\n  writable? false\n  default &Ash.UUID.generate/0\n  primary_key? true\n  allow_nil? false\nend\n```\n\n### `integer_primary_key`\n\nCreates a generated integer primary key. Keep in mind that not all data layers support auto incrementing ids, but for SQL data layers this is a very common practice. For those that don't, it is your own job to provide values for the primary key. We generally suggest using UUIDs over integers, as there are [a lot of good reasons to not use autoincrementing integer ids](https://www.clever-cloud.com/blog/engineering/2015/05/20/why-auto-increment-is-a-terrible-idea/).\n\n`integer_primary_key :id` is equivalent to:\n\n```elixir\nattribute :id, :integer do\n  writable? false\n  generated? true\n  primary_key? true\n  allow_nil? false\nend\n```","ref":"attributes.html#special-attributes"},{"type":"extras","title":"Relationships","doc":"# Relationships\n\nRelationships describe the connections between resources and are a core component of Ash. Defining relationships enables you to do things like\n\n- Loading related data\n- Filtering on related data\n- Managing related records through changes on a single resource\n- Authorizing based on the state of related data","ref":"relationships.html"},{"type":"extras","title":"Relationships Basics - Relationships","doc":"A relationship exists between a source resource and a destination resource. These are defined in the `relationships` block of the source resource. For example, if `MyApp.Tweet` is the source resource, and `MyApp.User` is the destination resource, we could define a relationship called `:owner` like this:\n\n```elixir\ndefmodule MyApp.Tweet do\n  use Ash.Resource,\n    data_layer: my_data_layer\n\n  attributes do\n    uuid_primary_key :id\n    attribute :body, :string\n  end\n\n  relationships do\n    belongs_to :owner, MyApp.User\n  end\nend\n```","ref":"relationships.html#relationships-basics"},{"type":"extras","title":"Kinds of relationships - Relationships","doc":"There are four kinds of relationships:\n\n- [`belongs_to`](#belongs-to)\n- [`has_one`](#has-one)\n- [`has_many`](#has-many)\n- [`many_to_many`](#many-to-many)\n\nEach of these relationships has a `source` resource and a `destination` resource with a corresponding attribute on the source resource (`source_attribute`), and destination resource (`destination_attribute`). Relationships will validate that their configured attributes exist at compile time.\n\nYou don't need to have a corresponding \"reverse\" relationship for every relationship, i.e if you have a `MyApp.Tweet` resource with `belongs_to :user, MyApp.User` you aren't required to have a `has_many :tweets, MyApp.Tweet` on `MyApp.User`. All that is required is that the attributes used by the relationship exist.","ref":"relationships.html#kinds-of-relationships"},{"type":"extras","title":"Belongs To - Relationships","doc":"```elixir\n# on MyApp.Tweet\nbelongs_to :owner, MyApp.User\n```\n\nA `belongs_to` relationship means that there is an attribute (`source_attribute`) on the source resource that uniquely identifies a record with a matching attribute (`destination_attribute`) in the destination. In the example above, the source attribute on `MyApp.Tweet` is `:owner_id` and the destination attribute on `MyApp.User` is `:id`.\n\n#### Attribute Defaults\n\nBy default, the `source_attribute` is defined as `: _id` of the type `:uuid` on the source resource and the `destination_attribute` is assumed to be `:id`. You can override the attribute names by specifying the `source_attribute` and `destination_attribute` options like so:\n\n```elixir\nbelongs_to :owner, MyApp.User do\n  # defaults to : _id (i.e. :owner_id)\n  source_attribute :custom_attribute_name\n\n  # defaults to :id\n  destination_attribute :custom_attribute_name\nend\n```\n\nYou can further customize the `source_attribute` using options such as:\n\n- `d:Ash.Resource.Dsl.relationships.belongs_to|define_attribute?` to define it yourself\n- `d:Ash.Resource.Dsl.relationships.belongs_to|attribute_type` to modify the default type\n- `d:Ash.Resource.Dsl.relationships.belongs_to|attribute_public?` to make the source attribute `public?: true`\n\nFor example:\n\n```elixir\nbelongs_to :owner, MyApp.User do\n  attribute_type :integer\n  attribute_writable? false\nend\n```\n\nOr if you wanted to define the attribute yourself,\n\n```elixir\nattributes do\n  attribute :owner_foo, MyApp.CustomType\nend\n\n...\nrelationships do\n  belongs_to :owner, MyApp.User do\n    define_attribute? false\n    source_attribute :owner_foo\n  end\nend\n```\n\n#### Customizing default belongs_to attribute type\n\nDestination attributes that are added by default are assumed to be `:uuid`. To change this, set the following configuration in `config.exs`:\n\n```elixir\nconfig :ash, :default_belongs_to_type, :integer\n```\n\nSee the docs for more: `d:Ash.Resource.Dsl.relationships.belongs_to`","ref":"relationships.html#belongs-to"},{"type":"extras","title":"Has One - Relationships","doc":"```elixir\n# on MyApp.User\nhas_one :profile, MyApp.Profile\n```\n\nA `has_one` relationship means that there is a unique attribute (`destination_attribute`) on the destination resource that identifies a record with a matching unique attribute (`source_resource`) in the source. In the example above, the source attribute on `MyApp.User` is `:id` and the destination attribute on `MyApp.Profile` is `:user_id`.\n\nA `has_one` is similar to a `belongs_to` except the reference attribute is on\nthe destination resource, instead of the source.\n\n#### Attribute Defaults\n\nBy default, the `source_attribute` is assumed to be `:id`, and `destination_attribute` defaults to ` _id`.\n\nSee the docs for more: `d:Ash.Resource.Dsl.relationships.has_one`","ref":"relationships.html#has-one"},{"type":"extras","title":"Has Many - Relationships","doc":"```elixir\n# on MyApp.User\nhas_many :tweets, MyApp.Tweet\n```\n\nA `has_many` relationship means that there is a non-unique attribute (`destination_attribute`) on the destination resource that identifies a record with a matching attribute (`source_attribute`) in the source. In the example above, the source attribute on `MyApp.User` is `:id` and the destination attribute on `MyApp.Tweet` is `:user_id`.\n\nA `has_many` relationship is similar to a `has_one` because the reference attribute exists on the destination resource. The only difference between this and `has_one` is that the destination attribute is not unique, and therefore will produce a list of related items. In the example above, `:tweets` corresponds to a list of `MyApp.Tweet` records.\n\n#### Attribute Defaults\n\nBy default, the `source_attribute` is assumed to be `:id`, and `destination_attribute` defaults to ` _id`.\n\nSee the docs for more: `d:Ash.Resource.Dsl.relationships.has_many`","ref":"relationships.html#has-many"},{"type":"extras","title":"Many To Many - Relationships","doc":"A `many_to_many` relationship can be used to relate many source resources to many destination resources. To achieve this, the `source_attribute` and `destination_attribute` are defined on a join resource. A `many_to_many` relationship can be thought of as a combination of a `has_many` relationship on the source/destination resources and a `belongs_to` relationship on the join resource.\n\nFor example, consider two resources `MyApp.Tweet` and `MyApp.Hashtag` representing tweets and hashtags. We want to be able to associate a tweet with many hashtags, and a hashtag with many tweets. To do this, we could define the following `many_to_many` relationship:\n\n```elixir\n# on MyApp.Tweet\nmany_to_many :hashtags, MyApp.Hashtag do\n  through MyApp.TweetHashtag\n  source_attribute_on_join_resource :tweet_id\n  destination_attribute_on_join_resource :hashtag_id\nend\n```\n\nThe `through` option specifies the \"join\" resource that will be used to store the relationship. We need to define this resource as well:\n\n```elixir\ndefmodule MyApp.TweetHashtag do\n  use Ash.Resource,\n    data_layer: your_data_layer\n\n  postgres do\n    table \"tweet_hashtags\"\n    repo MyApp.Repo\n  end\n\n  relationships do\n    belongs_to :tweet, MyApp.Tweet, primary_key?: true, allow_nil?: false\n    belongs_to :hashtag, MyApp.Hashtag, primary_key?: true, allow_nil?: false\n  end\n\n  actions do\n    defaults [:read, :destroy, create: :*, update: :*]\n  end\nend\n```\n\nIt is convention to name this resource `  ` however this is not required. The attributes on the join resource must match the `source_attribute_on_join_resource` and `destination_attribute_on_join_resource` options on the `many_to_many` relationship. The relationships on the join resource are standard `belongs_to` relationships, and can be configured as such. In this case, we have specified that the `:tweet_id` and `:hashtag_id` attributes form the primary key for the join resource, and that they cannot be `nil`.\n\nNow that we have a resource with the proper attributes, Ash will use this automatically under the hood when\nperforming relationship operations like filtering and loading.\n\nSee the docs for more: `d:Ash.Resource.Dsl.relationships.many_to_many`","ref":"relationships.html#many-to-many"},{"type":"extras","title":"Loading related data - Relationships","doc":"There are two ways to load relationships:\n\n- in the query using `Ash.Query.load/2`\n- directly on records using `Ash.load/3`","ref":"relationships.html#loading-related-data"},{"type":"extras","title":"On records - Relationships","doc":"Given a single record or a set of records, it is possible to load their relationships by calling the `load` function on the record's parent domain. For example:\n\n```elixir\n# user = %User{...}\nAsh.load(user, :tweets)\n\n# users = [%User{...}, %User{...}, ....]\nAsh.load(users, :tweets)\n```\n\nThis will fetch the tweets for each user, and set them in the corresponding `tweets` key.\n\n```elixir\n%User{\n  ...\n  tweets: [\n    %Tweet{...},\n    %Tweet{...},\n    ...\n  ]\n}\n```\n\nSee `Ash.load/3` for more information.","ref":"relationships.html#on-records"},{"type":"extras","title":"In the query - Relationships","doc":"The following will return a list of users with their tweets loaded identically to the previous example:\n\n```elixir\nUser\n|> Ash.Query.load(:tweets)\n|> Ash.read()\n```\n\nAt present, loading relationships in the query is fundamentally the same as loading on records. Eventually, data layers will be able to optimize these loads (potentially including them as joins in the main query).\n\nSee `Ash.Query.load/2` for more information.","ref":"relationships.html#in-the-query"},{"type":"extras","title":"More complex data loading - Relationships","doc":"Multiple relationships can be loaded at once, i.e\n\n```elixir\nAsh.load(users, [:tweets, :followers])\n```\n\nNested relationships can be loaded:\n\n```elixir\nAsh.load(users, followers: [:tweets, :followers])\n```\n\nThe queries used for loading can be customized by providing a query as the value.\n\n```elixir\nfollowers = Ash.Query.sort(User, follower_count: :asc)\n\nAsh.load(users, followers: followers)\n```\n\nNested loads will be included in the parent load.\n\n```elixir\nfollowers =\n  User\n  |> Ash.Query.sort(follower_count: :asc)\n  |> Ash.Query.load(:followers)\n\n# Will load followers and followers of those followers\nAsh.load(users, followers: followers)\n```","ref":"relationships.html#more-complex-data-loading"},{"type":"extras","title":"no_attributes? true - Relationships","doc":"This is really useful when creating customized relationships that aren't joined with simple attribute matches. For example:\n\n```elixir\nhas_many :higher_priority_tickets, __MODULE__ do\n  no_attributes? true\n  # parent/1 in this case puts the expression on this current resource\n  # so this is \"tickets with priority higher than this ticket\"\n  filter expr(priority > parent(priority))\nend\n```\n\nThis can also be useful when combined with schema-based multitenancy. Specifically, if you have a tenant resource like `Organization`,\nyou can use `no_attributes?` to do things like `has_many :employees, Employee, no_attributes?: true`, which lets you avoid having an\nunnecessary `organization_id` field on `Employee`. The same works in reverse: `has_one :organization, Organization, no_attributes?: true`\nallows relating the employee to their organization.\n\nYou can also use `no_attributes? true` with attribute-based multitenancy in the same situation described above, to avoid an unnecessary second\nfilter. If both resources have attribute multitenancy configured, they will already be filtered by `organization_id` by virtue of having\nset the tenant.\n\n>","ref":"relationships.html#no_attributes-true"},{"type":"extras","title":"Caveats for using no_attributes? - Relationships","doc":">\n> 1.  You can still manage relationships from one to the other, but \"relate\" and \"unrelate\" will have no effect, because there are no fields to change.\n> 2.  Loading the relationship on a list of resources will not behave as expected in all circumstances involving multitenancy. For example, if you get a list of `Organization` and then try to load `employees`, you would need to set a single tenant on the load query, meaning you'll get all organizations back with the set of employees from one tenant. This could eventually be solved, but for now it is considered an edge case.","ref":"relationships.html#caveats-for-using-no_attributes"},{"type":"extras","title":"Manual Relationships - Relationships","doc":"Manual relationships allow you to express complex or non-typical relationships between resources in a standard way. Individual data layers may interact with manual relationships in their own way, so see their corresponding guides. In general, you should try to use manual relationships sparingly, as you can do _a lot_ with filters on relationships, and the `no_attributes?` flag.","ref":"relationships.html#manual-relationships"},{"type":"extras","title":"Example - Relationships","doc":"In our Helpdesk example, we'd like to have a way to find tickets\n\nIn the `Representative` resource, define a `has_many` relationship as `manual` and point to the module where\nit will be implemented.\n\n```elixir\nrelationships do\n  has_many :tickets_above_threshold, Helpdesk.Support.Ticket do\n    manual Helpdesk.Support.Ticket.Relationships.TicketsAboveThreshold\n  end\nend\n```\n\nUsing Ash to get the destination records is ideal, so you can authorize access like normal\nbut if you need to use a raw ecto query here, you can. As long as you return the right structure.\n\nThe `TicketsAboveThreshold` module is implemented as follows.\n\n```elixir\ndefmodule Helpdesk.Support.Ticket.Relationships.TicketsAboveThreshold do\n  use Ash.Resource.ManualRelationship\n  require Ash.Query\n\n  def load(records, _opts, %{query: query} = context) do\n    # Use existing records to limit results\n    rep_ids = Enum.map(records, & &1.id)\n\n    {:ok,\n     query\n     |> Ash.Query.filter(representative_id in ^rep_ids)\n     |> Ash.Query.filter(priority > representative.priority_threshold)\n     |> Ash.read!(Ash.Context.to_opts(context))\n     # Return the items grouped by the primary key of the source, i.e representative.id => [...tickets above threshold]\n     |> Enum.group_by(& &1.representative_id)}\n  end\nend\n```","ref":"relationships.html#example"},{"type":"extras","title":"Reusing the Query - Relationships","doc":"Since you likely want to support things like filtering your relationship when being loaded, you will want to make sure that you use the query being provided. However, depending on how you're loading the relationship, you may need to do things like fetch extra records. To do this, you might do things like\n\n```elixir\ndef load(records, _opts, %{query: query, ..}) do\n  # unset some fields\n  fetch_query = Ash.Query.unset(query, [:limit, :offset])\n\n  # or, to be more safe/explicit, you might make a new query, explicitly setting only a few fields\n  fetch_query = query.resource |> Ash.Query.filter(^query.filter) |> Ash.Query.sort(query.sort)\n\n  ...\nend\n```","ref":"relationships.html#reusing-the-query"},{"type":"extras","title":"Query when loading with strict?: true - Relationships","doc":"When using `Ash.Query.load` or `Ash.load` with the `strict?: true` option, the query\nthat is provided to the load callback might be configured with a select-statement that doesn't\nload the attributes you want to group matching results by. If your codebase utilizes the strict\nloading functionality, it is therefore recommended to use `Ash.Query.ensure_selected` on the\nquery to ensure the required attributes are indeed fetched.\n\n```elixir\n\n# Here only :id & :priority is set, which will then configure the relationship query to only\n# select those attributes\n{:ok, rep} = Ash.load(representative, [tickets_above_threshold: [:id, :priority]], strict?: true)\n\ndefmodule Helpdesk.Support.Ticket.Relationships.TicketsAboveThreshold do\n  use Ash.Resource.ManualRelationship\n  require Ash.Query\n\n  def load(records, _opts, %{query: query, actor: actor, authorize?: authorize?}) do\n    rep_ids = Enum.map(records, & &1.id)\n\n    {:ok,\n     query\n     # If this isn't added, representative_id would be set to %Ash.NotLoaded, causing the\n     # Enum.group_by call below to fail mapping results to the correct records.\n     |> Ash.Query.ensure_selected([:representative_id])\n     |> Ash.Query.filter(representative_id in ^rep_ids)\n     |> Ash.Query.filter(priority > representative.priority_threshold)\n     |> Helpdesk.Support.read!(actor: actor, authorize?: authorize?)\n     |> Enum.group_by(& &1.representative_id)}\n  end\nend\n```","ref":"relationships.html#query-when-loading-with-strict-true"},{"type":"extras","title":"Fetching the records and then applying a query - Relationships","doc":"Lets say the records come from some totally unrelated source, or you can't just modify the query to fetch the records you need. You can fetch the records you need and then apply the query to them in memory.\n\n```elixir\ndef load(records, _opts, %{query: query, ..}) do\n  # fetch the data from the other source, which is capable of sorting\n  data = get_other_data(data, query.sort)\n\n  query\n  # unset the sort since we already applied that\n  |> Ash.Query.unset([:sort])\n  # apply the query in memory (filtering, distinct, limit, offset)\n  |> Ash.Query.apply_to(data)\nend\n```","ref":"relationships.html#fetching-the-records-and-then-applying-a-query"},{"type":"extras","title":"Managing Relationships - Relationships","doc":"Ash provides two primary approaches for managing related data, each suited to different scenarios:\n\n1. **Using `change manage_relationship/3` in actions** - When input comes from action arguments\n2. **Using `Ash.Changeset.manage_relationship/4` directly** - When building values programmatically in custom changes","ref":"relationships.html#managing-relationships"},{"type":"extras","title":"When to Use Which Approach - Relationships","doc":"**Use `change manage_relationship/3` when:**\n- Input comes from action arguments (API endpoints, form submissions)\n- You want portable logic across different interfaces (GraphQL, JSON API)\n- You need standard CRUD operations on relationships\n- The relationship management logic is straightforward\n\n**Use `Ash.Changeset.manage_relationship/4` when:**\n- Building relationship data programmatically in custom changes\n- You need complex logic or data transformation before managing relationships\n- Conditional relationship management based on changeset state\n- Integration with external APIs or complex business rules\n\n> #### Order of operations {: .warning}\n>\n> In destroy actions, relationships are managed **after** the main action is performed. This means if you're using `manage_relationship` to remove related records in a destroy action, and your database has foreign key constraints with \"no action\" or \"restrict\" settings, you may encounter constraint violations because Ash tries to destroy the primary resource first.\n>\n> To work around this, you can:\n> - Use the `cascade_destroy` builtin change instead of `manage_relationship`\n> - Configure your database constraints to be deferred\n> - Use different constraint settings that allow the operation order","ref":"relationships.html#when-to-use-which-approach"},{"type":"extras","title":"Using change manage_relationship/3 in Actions - Relationships","doc":"This is the most common approach for managing relationships through action arguments:\n\n```elixir\nactions do\n  update :update do\n    argument :add_comment, :map do\n      allow_nil? false\n    end\n\n    argument :tags, {:array, :uuid} do\n      allow_nil? false\n    end\n\n    # First argument is the name of the action argument to use\n    # Second argument is the relationship to be managed\n    # Third argument is options. For more, see `Ash.Changeset.manage_relationship/4`.\n    change manage_relationship(:add_comment, :comments, type: :create)\n\n    # Second argument can be omitted when argument name matches relationship name\n    change manage_relationship(:tags, type: :append_and_remove)\n  end\nend\n```\n\nWith this setup, you can use the arguments in action input:\n\n```elixir\npost\n|> Ash.Changeset.for_update(:update, %{\n  tags: [tag1.id, tag2.id],\n  add_comment: %{text: \"comment text\"}\n})\n|> Ash.update!()\n```\n\n#### Common Patterns with Actions\n\n**Creating with related data:**\n\n```elixir\ncreate :create_with_author do\n  argument :author, :map, allow_nil?: false\n  change manage_relationship(:author, type: :create)\nend\n\n# Usage\nPost\n|> Ash.Changeset.for_create(:create_with_author, %{\n  title: \"My Post\",\n  author: %{name: \"John Doe\", email: \"john@example.com\"}\n})\n|> Ash.create!()\n```\n\n**Managing many-to-many relationships:**\n\n```elixir\nupdate :manage_categories do\n  argument :category_names, {:array, :string}\n\n  change manage_relationship(:category_names, :categories,\n    type: :append_and_remove,\n    value_is_key: :name,\n    on_lookup: :relate,\n    on_no_match: :create\n  )\nend\n```\n\n**Different argument and relationship names:**\n\n```elixir\nupdate :assign_reviewer do\n  argument :reviewer_id, :uuid\n  change manage_relationship(:reviewer_id, :reviewer, type: :append_and_remove)\nend\n```","ref":"relationships.html#using-change-manage_relationship-3-in-actions"},{"type":"extras","title":"Using Ash.Changeset.manage_relationship/4 in Custom Changes - Relationships","doc":"For more complex scenarios, you can use `Ash.Changeset.manage_relationship/4` directly in custom changes:\n\n```elixir\ndefmodule MyApp.Changes.AssignProjectMembers do\n  use Ash.Resource.Change\n\n  def change(changeset, _opts, context) do\n    # Get the current user from context\n    current_user = context.actor\n\n    # Build relationship data based on business logic\n    members = determine_project_members(changeset, current_user)\n\n    # Manage the relationship directly\n    Ash.Changeset.manage_relationship(\n      changeset,\n      :members,\n      members,\n      type: :append_and_remove,\n      authorize?: true\n    )\n  end\n\n  defp determine_project_members(changeset, current_user) do\n    # Complex logic to determine who should be project members\n    # based on changeset data and business rules\n    # ...\n  end\nend\n```\n\n**Conditional relationship management:**\n\n```elixir\ndefmodule MyApp.Changes.ConditionalTagging do\n  use Ash.Resource.Change\n\n  def change(changeset, _opts, _context) do\n    # Only manage tags if certain conditions are met\n    if should_auto_tag?(changeset) do\n      tags = generate_auto_tags(changeset)\n\n      Ash.Changeset.manage_relationship(\n        changeset,\n        :tags,\n        tags,\n        type: :append,\n        on_no_match: :create\n      )\n    else\n      changeset\n    end\n  end\nend\n```\n\n**Data transformation before relationship management:**\n\n```elixir\ndefmodule MyApp.Changes.ProcessOrderItems do\n  use Ash.Resource.Change\n\n  def change(changeset, _opts, _context) do\n    case Ash.Changeset.fetch_argument(changeset, :raw_items) do\n      {:ok, raw_items} ->\n        # Transform and validate the raw item data\n        processed_items =\n          raw_items\n          |> validate_items()\n          |> calculate_pricing()\n          |> apply_discounts()\n\n        Ash.Changeset.manage_relationship(\n          changeset,\n          :items,\n          processed_items,\n          type: :direct_control\n        )\n\n      :error ->\n        changeset\n    end\n  end\nend\n```","ref":"relationships.html#using-ash-changeset-manage_relationship-4-in-custom-changes"},{"type":"extras","title":"Management Types and Options - Relationships","doc":"Ash provides several built-in management types that configure common relationship management patterns:\n\n#### Management Types\n\n**`:append`** - Add new related records, ignore existing ones\n```elixir\nchange manage_relationship(:tags, type: :append)\n# Equivalent to:\n# on_lookup: :relate, on_no_match: :error, on_match: :ignore, on_missing: :ignore\n```\n\n**`:append_and_remove`** - Add new related records, remove missing ones\n```elixir\nchange manage_relationship(:tags, type: :append_and_remove)\n# Equivalent to:\n# on_lookup: :relate, on_no_match: :error, on_match: :ignore, on_missing: :unrelate\n```\n\n**`:remove`** - Remove specified related records\n```elixir\nchange manage_relationship(:tags, type: :remove)\n# Equivalent to:\n# on_no_match: :error, on_match: :unrelate, on_missing: :ignore\n```\n\n**`:direct_control`** - Full CRUD control over the related records\n```elixir\nchange manage_relationship(:comments, type: :direct_control)\n# Equivalent to:\n# on_lookup: :ignore, on_no_match: :create, on_match: :update, on_missing: :destroy\n```\n\n**`:create`** - Only create new related records\n```elixir\nchange manage_relationship(:items, type: :create)\n# Equivalent to:\n# on_no_match: :create, on_match: :ignore\n```\n\n#### Key Options\n\n**`on_lookup`** - How to handle records that might exist elsewhere:\n- `:ignore` - Don't look up existing records\n- `:relate` - Look up and relate existing records\n- `{:relate, :action_name}` - Use specific action for relating\n\n**`on_no_match`** - What to do when no matching record exists:\n- `:ignore` - Skip these inputs\n- `:create` - Create new records\n- `{:create, :action_name}` - Use specific create action\n- `:error` - Raise an error\n\n**`on_match`** - What to do when a matching record is found:\n- `:ignore` - Leave the record as-is\n- `:update` - Update the existing record\n- `{:update, :action_name}` - Use specific update action\n- `:unrelate` - Remove the relationship\n- `:error` - Raise an error\n\n**`on_missing`** - What to do with related records not in the input:\n- `:ignore` - Leave them as-is\n- `:unrelate` - Remove the relationship\n- `:destroy` - Delete the records\n- `{:destroy, :action_name}` - Use specific destroy action\n\n#### Advanced Options\n\n**`value_is_key`** - Use a specific field as the key when providing simple values:\n```elixir\n# Allow using category names instead of IDs\nchange manage_relationship(:category_names, :categories,\n  value_is_key: :name,\n  type: :append_and_remove\n)\n```\n\n**`use_identities`** - Specify which identities to use for lookups:\n```elixir\nchange manage_relationship(:tags,\n  type: :append_and_remove,\n  use_identities: [:name, :_primary_key]\n)\n```\n\n**`join_keys`** - For many-to-many relationships, specify join table parameters:\n```elixir\nchange manage_relationship(:categories,\n  type: :append_and_remove,\n  join_keys: [:priority, :added_by]\n)\n```","ref":"relationships.html#management-types-and-options"},{"type":"extras","title":"Relationship Type Considerations - Relationships","doc":"#### belongs_to Relationships\n\nWhen managing `belongs_to` relationships, you're typically setting a parent:\n\n```elixir\ncreate :create_with_parent do\n  argument :parent, :map\n  change manage_relationship(:parent, type: :create)\nend\n\n# Or relating to existing parent\nupdate :assign_parent do\n  argument :parent_id, :uuid\n  change manage_relationship(:parent_id, :parent, type: :append_and_remove)\nend\n```\n\n#### has_one Relationships\n\nFor `has_one` relationships, you manage a single related record:\n\n```elixir\nupdate :update_profile do\n  argument :profile, :map\n  change manage_relationship(:profile, type: :direct_control)\nend\n```\n\n#### has_many Relationships\n\nWith `has_many`, you typically manage collections:\n\n```elixir\nupdate :manage_comments do\n  argument :comments, {:array, :map}\n  change manage_relationship(:comments, type: :direct_control)\nend\n```\n\n#### many_to_many Relationships\n\nMany-to-many relationships often involve join table management:\n\n```elixir\nupdate :update_post_tags do\n  argument :tags, {:array, :map}\n\n  change manage_relationship(:tags,\n    type: :append_and_remove,\n    join_keys: [:tagged_at, :tagged_by]\n  )\nend\n```","ref":"relationships.html#relationship-type-considerations"},{"type":"extras","title":"Advanced Patterns - Relationships","doc":"#### Multiple manage_relationship Calls\n\nYou can call `manage_relationship` multiple times, and they'll be processed in order:\n\n```elixir\nupdate :complex_update do\n  argument :add_tags, {:array, :string}\n  argument :remove_tags, {:array, :string}\n\n  change manage_relationship(:add_tags, :tags,\n    type: :append,\n    value_is_key: :name,\n    meta: [order: 1]\n  )\n\n  change manage_relationship(:remove_tags, :tags,\n    type: :remove,\n    value_is_key: :name,\n    meta: [order: 2]\n  )\nend\n```","ref":"relationships.html#advanced-patterns"},{"type":"extras","title":"Argument Types and Value Handling - Relationships","doc":"#### Map and List Inputs\n\nWhen providing maps or lists of maps, you're providing input for actions on the destination resource:\n\n```elixir\n# Maps become action input\nargument :comment, :map\nchange manage_relationship(:comment, :comments, type: :create)\n\n# Usage:\n%{comment: %{text: \"Great post!\", rating: 5}}\n```\n\n#### Simple Value Inputs\n\nYou can also provide simple values using `value_is_key`:\n\n```elixir\nargument :tag_names, {:array, :string}\nchange manage_relationship(:tag_names, :tags,\n  type: :append_and_remove,\n  value_is_key: :name,\n  on_lookup: :relate,\n  on_no_match: :create\n)\n\n# Usage:\n%{tag_names: [\"elixir\", \"phoenix\", \"ash\"]}\n```\n\n#### Using Existing Records\n\nYou can also pass existing record structs directly:\n\n```elixir\n# In a custom change\nexisting_tags = Ash.read!(Tag, actor: actor)\nAsh.Changeset.manage_relationship(changeset, :tags, existing_tags, type: :append)\n```\n\n> #### Authorization considerations {: .warning}\n>\n> When you pass existing record structs directly to `manage_relationship`, Ash assumes that the actor is already authorized to read those records. This bypasses the normal authorization checks that would occur if you had provided IDs instead.\n>\n> If you provide ids/maps, Ash will read the records and properly check authorization. Only pass existing record structs when you're certain the actor has appropriate read permissions for those records, or authorization is not relevant.","ref":"relationships.html#argument-types-and-value-handling"},{"type":"extras","title":"Integration with Tools - Relationships","doc":"The explicit nature of relationship management options enables rich integrations:\n\n- **AshPhoenix.Form** - Automatically derives nested form structures\n- **AshGraphQL** - Generates complex input objects for mutations\n- **AshJsonApi** - Creates appropriate API endpoints for relationship management\n\nThis works because Ash can determine exactly what actions might be called and what input is needed based on your `manage_relationship` configuration.\n\nFor complete documentation of all available options, see `Ash.Changeset.manage_relationship/4`.","ref":"relationships.html#integration-with-tools"},{"type":"extras","title":"Calculations","doc":"# Calculations\n\nCalculations in Ash allow for displaying complex values as a top level value of a resource.","ref":"calculations.html"},{"type":"extras","title":"Primer - Calculations","doc":"","ref":"calculations.html#primer"},{"type":"extras","title":"Declaring calculations on a resource - Calculations","doc":"","ref":"calculations.html#declaring-calculations-on-a-resource"},{"type":"extras","title":"Expression Calculations - Calculations","doc":"The simplest kind of calculation refers to an Ash expression. For example:\n\n```elixir\ncalculations do\n  calculate :full_name, :string, expr(first_name <> \" \" <> last_name)\nend\n```\n\nSee the [Expressions guide](/documentation/topics/reference/expressions.md) for more.","ref":"calculations.html#expression-calculations"},{"type":"extras","title":"Module Calculations - Calculations","doc":"When calculations require more complex code or can't be pushed down into the data layer, a module that uses `Ash.Resource.Calculation` can be used.\n\n```elixir\ndefmodule Concat do\n  # An example concatenation calculation, that accepts the delimiter as an argument,\n  #and the fields to concatenate as options\n  use Ash.Resource.Calculation\n\n  # Optional callback that verifies the passed in options (and optionally transforms them)\n  @impl true\n  def init(opts) do\n    if opts[:keys] && is_list(opts[:keys]) && Enum.all?(opts[:keys], &is_atom/1) do\n      {:ok, opts}\n    else\n      {:error, \"Expected a `keys` option for which keys to concat\"}\n    end\n  end\n\n  @impl true\n  # A callback to tell Ash what keys must be loaded/selected when running this calculation\n  # you can include related data here, but be sure to include the attributes you need from said related data\n  # i.e `posts: [:title, :body]`.\n  def load(_query, opts, _context) do\n    opts[:keys]\n  end\n\n  @impl true\n  def calculate(records, opts, %{arguments: %{separator: separator}}) do\n    Enum.map(records, fn record ->\n      Enum.map_join(opts[:keys], separator, fn key ->\n        to_string(Map.get(record, key))\n      end)\n    end)\n  end\n\n  # You can implement this callback to make this calculation possible in the data layer\n  # *and* in elixir. Ash expressions are already executable in Elixir or in the data layer, but this gives you fine grain control over how it is done\n  # See the expressions guide for more.\n  # @impl true\n  # def expression(opts, context) do\n  #   expr(your_expression_here)\n  # end\nend\n\n# Usage in a resource\ncalculations do\n  calculate :full_name, :string, {Concat, keys: [:first_name, :last_name]} do\n    # You need to use the [allow_empty?: true, trim?: false] constraints here.\n    # The separator could be an empty string or require a leading or trailing space,\n    # but would be trimmed or even set to `nil` without the constraints shown below.\n    argument :separator, :string do\n      allow_nil? false\n      constraints [allow_empty?: true, trim?: false]\n      default \"\"\n    end\n  end\nend\n```\n\nSee the documentation for the calculations section in [Resource DSL docs](dsl-ash-resource.html#calculations) and the `Ash.Resource.Calculation` docs for more information.\n\nThe calculations declared on a resource allow for declaring a set of named calculations that can be used by extensions.\nThey can also be loaded in the query using `Ash.Query.load/2`, or after the fact using `Ash.load/3`. Calculations declared on the resource will be keys in the resource's struct.","ref":"calculations.html#module-calculations"},{"type":"extras","title":"Custom calculations in the query - Calculations","doc":"Example:\n\n```elixir\nUser\n|> Ash.Query.calculate(:full_name, {Concat, keys: [:first_name, :last_name]}, :string, %{separator: \",\"})\n```\n\nSee the documentation for `Ash.Query.calculate/4` for more information.","ref":"calculations.html#custom-calculations-in-the-query"},{"type":"extras","title":"Arguments in calculations - Calculations","doc":"Using the above example with arguments, you can load a calculation with arguments like so:\n\n```elixir\nload(full_name: [separator: \",\"])\n```\n\nIf the calculation uses an expression, you can also filter and sort on it like so:\n\n```elixir\nquery\n|> Ash.Query.filter(full_name(separator: \" \") == \"Zach Daniel\")\n|> Ash.Query.sort(full_name: {%{separator: \" \"}, :asc})\n```","ref":"calculations.html#arguments-in-calculations"},{"type":"extras","title":"Loading Calculations - Calculations","doc":"When loading calculations, you specify them in the load statement just like relationships and aggregates.\n\n```elixir\n# load\nAsh.load!(user, [full_name: %{separator: \",\"}])\n# => %User{full_name: \"Zach,Daniel\"}\n```","ref":"calculations.html#loading-calculations"},{"type":"extras","title":"Loading with a custom name - Calculations","doc":"Every record in Ash also has a `calculations` field, where arbitrarily named calculations can live.\nSee `Ash.Query.calculate/4` for more. To do this with `load` statements, you use the reserved\n`as` key in the calculation arguments.\n\n```elixir\n# load\nAsh.load!(user, [\n  full_name: %{separator: \" \", as: :full_name_with_spaces},\n  full_name: %{separator: \",\", as: :full_name_with_commas}\n])\n# => %User{calculations: %{full_name_with_spaces: \"Zach Daniel\", full_name_with_commas: \"Zach,Daniel\"}}\n```","ref":"calculations.html#loading-with-a-custom-name"},{"type":"extras","title":"Loading \"through\" calculations - Calculations","doc":"If you have calculations that produce records, or loadable types like `Ash.Type.Map` and `Ash.Type.Struct`\nyou can load further things on those records by providing a tuple of calculation input and further load statements.\n\n\n```elixir\n# here is a map type that contains a user and a status\ndefmodule MyApp.Types.UserAndStatus do\n  use Ash.Type.NewType, subtype_of: :map, constraints: [\n    fields: [\n      user: [\n        type: :struct,\n        instance_of: MyApp.User\n      ],\n      status: [\n        type: :atom,\n        constraints: [one_of: [:active, :inactive]]\n      ]\n    ]\n  ]\nend\n\n# on our organization resource, we might have a calculation that returns a user and their status\ncalculate :user_statuses, {:array, MyApp.Types.UserAndStatus}, GetUsersAndTheirStatuses\n```\n\nYou could then load this calculation like so:\n\n```elixir\nAsh.load!(organization, :user_statuses)\n# => [%{user: %User{}, status: :active}, %{user: %User{}, status: :inactive}]\n```\n\nBut what if you wanted additional fields from the calculated user? To do this, we provide\na tuple of additional loads alongside their arguments. Maps support loading \"through\"\nfields by using the configured fields in the map and providing further loads.\n\n```elixir\n# {arguments, additional_load_statement}\nAsh.load!(organization, user_statuses: {%{}, [user: [full_name: %{separator: \" \"}]]}),\n# => [%{user: %User{full_name: \"Zach Daniel\"}, status: :active}, %{user: %User{full_name: \"Tobey Maguire\"}, status: :inactive}]\n```","ref":"calculations.html#loading-through-calculations"},{"type":"extras","title":"Aggregates","doc":"# Aggregates\n\nAggregates in Ash allow for retrieving summary information over groups of related data. A simple example might be to show the \"count of published posts for a user\". Aggregates allow us quick and performant access to this data, in a way that supports being filtered/sorted on automatically. More aggregate types can be added, but you will be restricted to only the supported types. In cases where aggregates don't suffice, use [Calculations](/documentation/topics/resources/calculations.md), which are intended to be much more flexible.","ref":"aggregates.html"},{"type":"extras","title":"Declaring aggregates on a resource - Aggregates","doc":"Aggregates are defined in an `aggregates` section. For all possible types, see below.\nFor a full reference, see `d:Ash.Resource.Dsl.aggregates`.\n\n```elixir\naggregates do\n  count :count_of_posts, :posts do\n    filter expr(published == true)\n  end\nend\n```","ref":"aggregates.html#declaring-aggregates-on-a-resource"},{"type":"extras","title":"Using an aggregate - Aggregates","doc":"Aggregates are loaded and filtered on in the same way that calculations are. Lets look at some examples:","ref":"aggregates.html#using-an-aggregate"},{"type":"extras","title":"Loading aggregates in a query or on records - Aggregates","doc":"```elixir\nUser\n|> Ash.Query.load(:count_of_posts)\n|> Map.get(:count_of_posts)\n# => 10\n\nusers\n|> Ash.load!(:count_of_posts)\n|> Enum.map(&(&1.count_of_posts))\n# => [3, 5, 2]\n```","ref":"aggregates.html#loading-aggregates-in-a-query-or-on-records"},{"type":"extras","title":"Filtering on aggregates - Aggregates","doc":"```elixir\nrequire Ash.Query\n\nUser\n|> Ash.Query.filter(count_of_posts > 10)\n|> Ash.read!()\n```","ref":"aggregates.html#filtering-on-aggregates"},{"type":"extras","title":"Sorting aggregates - Aggregates","doc":"```elixir\nUser\n|> Ash.Query.sort(count_of_posts: :asc)\n|> Ash.read!()\n```","ref":"aggregates.html#sorting-aggregates"},{"type":"extras","title":"Aggregate types - Aggregates","doc":"- `count` - counts related items meeting the criteria.\n- `exists` - checks if any related items meet the criteria.\n- `first` - gets the first related value matching the criteria. Must specify the `field`.\n- `sum` - sums the related items meeting the criteria. Must specify the `field`.\n- `list` - lists the related values. Must specify the `field`.\n- `max` - gets the maximum related value. Must specify the `field`.\n- `min` - gets the minimum related value. Must specify the `field`.\n- `avg` - gets the average related value. Must specify the `field`.\n- `custom` - allows for a custom aggregate. Implementation depends on the data layer. Must provide an `implementation`.\n\nThe declared set of named aggregates can be used by extensions and referred to throughout your application As an escape hatch, they can also be loaded in the query using `Ash.Query.load/2`, or after the fact using `Ash.load/3`. Aggregates declared on the resource will be keys in the resource's struct.\n\nSee the docs on `d:Ash.Resource.Dsl.aggregates` for more information.","ref":"aggregates.html#aggregate-types"},{"type":"extras","title":"Custom aggregates in the query - Aggregates","doc":"Custom aggregates can be added to the query and will be placed in the `aggregates` key of the results. This is an escape hatch, and is not the primary way that you should be using aggregates. It does, however, allow for dynamism, i.e if you are accepting user input that determines what the filter and/or field should be, that kind of thing.\n\nExample:\n\n```elixir\nUser\n|> Ash.Query.aggregate(\n  :count_of_posts,\n  :count,\n  :posts,\n  query: [\n    filter: [published: published?]\n  ]\n)\n```\n\nSee the documentation for `Ash.Query.aggregate/4` for more information.","ref":"aggregates.html#custom-aggregates-in-the-query"},{"type":"extras","title":"Join Filters - Aggregates","doc":"Join filters allows for more complex aggregate queries, including joining with predicates based on multiple related values.","ref":"aggregates.html#join-filters"},{"type":"extras","title":"Example - Aggregates","doc":"```elixir\n  aggregates do\n    sum :saved_money, [:redeems, :deal], :amount do\n      # where any redeem of the deal is redeemed\n      filter expr(redeems.redeemed == true)\n\n      # where the `redeems` are `redeemed`\n      join_filter :redeems, expr(redeemed == true)\n\n      # where the `redeems.deal.active` == `redeems.require_active`\n      join_filter [:redeems, :deal], expr(active == parent(require_active))\n    end\n  end\n```","ref":"aggregates.html#example"},{"type":"extras","title":"Inline Aggregates - Aggregates","doc":"Aggregates can be created in-line in expressions, with their relationship path specified and any options provided that match the options given to `Ash.Query.Aggregate.new/4`. For example:\n\n```elixir\ncalculate :grade, :decimal, expr(\n  count(answers, query: [filter: expr(correct == true)]) /\n  count(answers, query: [filter: expr(correct == false)])\n)\n```\n\nSee the [Expressions guide](/documentation/topics/reference/expressions.md#inline-aggregates) for more.","ref":"aggregates.html#inline-aggregates"},{"type":"extras","title":"Validations","doc":"# Validations\n\nValidations are similar to [changes](/documentation/topics/resources/changes.md), except they cannot modify the changeset. They can only continue, or add an error.","ref":"validations.html"},{"type":"extras","title":"Builtin Validations - Validations","doc":"There are a number of builtin validations that can be used, and are automatically imported into your resources. See `Ash.Resource.Validation.Builtins` for more.\n\nSome examples of usage of builtin validations\n\n```elixir\nvalidate match(:email, \"@\")\n\nvalidate compare(:age, greater_than_or_equal_to: 18) do\n  message \"must be over 18 to sign up\"\nend\n\nvalidate present(:last_name) do\n  where [present(:first_name), present(:middle_name)]\n  message \"must also be supplied if setting first name and middle_name\"\nend\n```","ref":"validations.html#builtin-validations"},{"type":"extras","title":"Custom Validations - Validations","doc":"```elixir\ndefmodule MyApp.Validations.IsPrime do\n  # transform and validate opts\n\n  use Ash.Resource.Validation\n\n  @impl true\n  def init(opts) do\n    if is_atom(opts[:attribute]) do\n      {:ok, opts}\n    else\n      {:error, \"attribute must be an atom!\"}\n    end\n  end\n\n  @impl true\n  def validate(changeset, opts, _context) do\n    value = Ash.Changeset.get_attribute(changeset, opts[:attribute])\n    # this is a function I made up for example\n    if is_nil(value) || Math.is_prime?(value) do\n      :ok\n    else\n      # The returned error will be passed into `Ash.Error.to_ash_error/3`\n      {:error, field: opts[:attribute], message: \"must be prime\"}\n    end\n  end\nend\n```\n\nThis could then be used in a resource via:\n\n```elixir\nvalidate {MyApp.Validations.IsPrime, attribute: :foo}\n```","ref":"validations.html#custom-validations"},{"type":"extras","title":"Anonymous Function Validations - Validations","doc":"You can also use anonymous functions for validations. Keep in mind, these cannot be made atomic. This is great for prototyping, but we generally recommend using a module, both for organizational purposes, and to allow adding atomic behavior.\n\n```elixir\nvalidate fn changeset, _context ->\n  # put your code here\nend\n```","ref":"validations.html#anonymous-function-validations"},{"type":"extras","title":"Where - Validations","doc":"The `where` can be used to perform validations conditionally.\n\nThe value of the `where` option can either be a validation or a list of validations. All of the `where`-validations must first pass for the main validation to be applied. For expressing complex conditionals, passing a list of built-in validations to `where` can serve as an alternative to writing a custom validation module.","ref":"validations.html#where"},{"type":"extras","title":"Examples - Validations","doc":"```elixir\nvalidate present(:other_number), where: absent(:that_number)\n```\n\n```elixir\nvalidate present(:other_number) do\n  where {MyApp.Validations.IsPrime, attribute: :foo}\nend\n```\n\n```elixir\nvalidate present(:other_number),\n  where: [\n    numericality(:large_number, greater_than: 100),\n    one_of(:magic_number, [7, 13, 123])\n  ]\n```","ref":"validations.html#examples"},{"type":"extras","title":"Action vs Global Validations - Validations","doc":"You can place a validation in any create, update, or destroy action. For example:\n\n```elixir\nactions do\n  create :create do\n    validate compare(:age, greater_than_or_equal_to: 18)\n  end\nend\n```\n\nOr you can use the global validations block to validate on all actions of a given type. Where statements can be used in either. Note the warning about running on destroy actions below.\n\n```elixir\nvalidations do\n  validate present([:foo, :bar], at_least: 1) do\n    on [:create, :update]\n    where present(:baz)\n  end\nend\n```\n\nThe validations section allows you to add validations across multiple actions of a changeset\n\n>","ref":"validations.html#action-vs-global-validations"},{"type":"extras","title":"Running on destroy actions - Validations","doc":">\n> By default, validations in the global `validations` block will run on create and update only. Many validations don't make sense in the context of destroys. To make them run on destroy, use `on: [:create, :update, :destroy]`","ref":"validations.html#running-on-destroy-actions"},{"type":"extras","title":"Examples - Validations","doc":"```elixir\nvalidations do\n  validate present([:foo, :bar]), on: :update\n  validate present([:foo, :bar, :baz], at_least: 2), on: :create\n  validate present([:foo, :bar, :baz], at_least: 2), where: [action_is([:action1, :action2])]\n  validate absent([:foo, :bar, :baz], exactly: 1), on: [:update, :destroy]\n  validate {MyCustomValidation, [foo: :bar]}, on: :create\nend\n```","ref":"validations.html#examples-1"},{"type":"extras","title":"Atomic Validations - Validations","doc":"To make a validation atomic, you have to implement the `c:Ash.Resource.Validation.atomic/3` callback. This callback returns an atomic instruction, or a list of atomic instructions, or an error/indication that the validation cannot be done atomically. For our `IsPrime` example above, this would look something like:\n\n```elixir\ndefmodule MyApp.Validations.IsPrime do\n  # transform and validate opts\n\n  use Ash.Resource.Validation\n\n  ...\n\n  def atomic(changeset, opts, context) do\n    # lets ignore that there is no easy/built-in way to check prime numbers in postgres\n    {:atomic,\n      # the list of attributes that are involved in the validation\n      [opts[:attribute]],\n      # the condition that should cause the error\n      # here we refer to the new value or the current value\n      expr(not(fragment(\"is_prime(?)\", ^atomic_ref(opts[:attribute])))),\n      # the error expression\n      expr(\n        error(^InvalidAttribute, %{\n          field: ^opts[:attribute],\n          # the value that caused the error\n          value: ^atomic_ref(opts[:attribute]),\n          # the message to display\n          message: ^(context.message || \"%{field} must be prime\"),\n          vars: %{field: ^opts[:attribute]}\n        })\n      )\n    }\n  end\nend\n```\n\nIn some cases, validations operate on arguments only and therefore have no need of atomic behavior. for this, you can call `validate/3` directly from `atomic/3`. The builtin `Ash.Resource.Validation.Builtins.argument_equals/2` validation does this, for example.\n\n```elixir\n@impl true\ndef atomic(changeset, opts, context) do\n  validate(changeset, opts, context)\nend\n```","ref":"validations.html#atomic-validations"},{"type":"extras","title":"Changes","doc":"# Changes\n\nChanges are the primary way of customizing create/update/destroy action behavior. If you are familiar with `Plug`, you can think of an `Ash.Resource.Change` as the equivalent of a `Plug` for changesets. At its most basic, a change will take a changeset and return a new changeset. Changes can be simple, like setting or modifying an attribute value, or more complex, attaching hooks to be executed within the lifecycle of the action.","ref":"changes.html"},{"type":"extras","title":"Builtin Changes - Changes","doc":"There are a number of builtin changes that can be used, and are automatically imported into your resources. See `Ash.Resource.Change.Builtins` for more.\n\nSome examples of usage of builtin changes\n\n```elixir\n# set the `owner` to the current actor\nchange relate_actor(:owner)\n\n# set `commited_at` to the current timestamp when the action is called\nchange set_attribute(:committed_at, &DateTime.utc_now/0)\n\n# optimistic lock using the `version` attribute\nchange optimistic_lock(:version)\n```","ref":"changes.html#builtin-changes"},{"type":"extras","title":"Custom Changes - Changes","doc":"```elixir\ndefmodule MyApp.Changes.Slugify do\n\n  use Ash.Resource.Change\n\n  # transform and validate opts\n  @impl true\n  def init(opts) do\n    if is_atom(opts[:attribute]) do\n      {:ok, opts}\n    else\n      {:error, \"attribute must be an atom!\"}\n    end\n  end\n\n  @impl true\n  def change(changeset, opts, _context) do\n    case Ash.Changeset.fetch_change(changeset, opts[:attribute]) do\n      {:ok, new_value} ->\n        slug = String.replace(new_value, ~r/\\s+/, \"-\")\n        Ash.Changeset.force_change_attribute(changeset, opts[:attribute], slug)\n      :error ->\n        changeset\n    end\n  end\nend\n```\n\nThis could then be used in a resource via:\n\n```elixir\nchange {MyApp.Changes.Slugify, attribute: :foo}\n```","ref":"changes.html#custom-changes"},{"type":"extras","title":"Anonymous Function Changes - Changes","doc":"You can also use anonymous functions for changes. Keep in mind, these cannot be made atomic, or support batching. This is great for prototyping, but we generally recommend using a module, both for organizational purposes, and to allow adding atomic/batch behavior.\n\n```elixir\nchange fn changeset, _context ->\n  # put your code here\nend\n```","ref":"changes.html#anonymous-function-changes"},{"type":"extras","title":"Where - Changes","doc":"The `where` can be used to perform changes conditionally. This functions by running the validations in the `where`, and if the validation returns an error, we discard the error and skip the operation. This means that even custom validations can be used in conditions.\n\nFor example:\n\n```elixir\nchange {Slugify, attribute: :foo} do\n  where [attribute_equals(:slugify, true)]\nend\n```","ref":"changes.html#where"},{"type":"extras","title":"Action vs Global Changes - Changes","doc":"You can place a change on any create, update, or destroy action. For example:\n\n```elixir\nactions do\n  create :create do\n    change {Slugify, attribute: :name}\n  end\nend\n```\n\nOr you can use the global changes block to apply to all actions of a given type. Where statements can be used in both cases. Use `on` to determine the types of actions the validation runs on. By default, it only runs on create and update actions.\n\n```elixir\nchanges do\n  change {Slugify, attribute: :name} do\n    on [:create]\n  end\nend\n```\n\nThe changes section allows you to add changes across multiple actions of a resource.\n\n>","ref":"changes.html#action-vs-global-changes"},{"type":"extras","title":"Running on destroy actions - Changes","doc":">\n> By default, changes in the global `changes` block will run on create and update only. Many changes don't make sense in the context of destroys. To make them run on destroy, use `on: [:create, :update, :destroy]`","ref":"changes.html#running-on-destroy-actions"},{"type":"extras","title":"Examples - Changes","doc":"```elixir\nchanges do\n  change relate_actor(:owner)\n  change set_attribute(:committed_at, &DateTime.utc_now/0)\n  change optimistic_lock(:version), on: [:create, :update, :destroy]\n  change {Slugify, [attribute: :foo]}, on: :create\nend\n```","ref":"changes.html#examples"},{"type":"extras","title":"Atomic Changes - Changes","doc":"To make a change atomic, you have to implement the `c:Ash.Resource.Change.atomic/3` callback. This callback returns a map of changes to attributes that should be changed atomically. We will also honor any `c:Ash.Resource.Change.after_batch/3` functionality to run code after atomic changes have been applied (only if `atomic/3` callback has also been defined). Note that `c:Ash.Resource.Change.before_batch/3` is not supported in this scenario and will be ignored.\n\n```elixir\ndefmodule MyApp.Changes.Slugify do\n  # transform and validate opts\n\n  use Ash.Resource.Change\n\n  ...\n\n  def atomic(changeset, opts, context) do\n    {:atomic, %{\n      opts[:attribute] => expr(\n        fragment(\"regexp_replace(?, ?, ?)\", ^ref(opts[:attribute]), ~r/\\s+/, \"-\")\n      )\n    }}\n  end\nend\n```\n\nIn some cases, changes operate only on arguments or context, or otherwise can do their work regardless of atomicity. In these cases, you can make your atomic callback call the `change/3` function\n\n```elixir\n@impl true\ndef atomic(changeset, opts, context) do\n  {:ok, change(changeset, opts, context)}\nend\n```\n\nIn other cases, a change may not be necessary in a fully atomic action. For this, you can simply return `:ok`\n\n```elixir\n@impl true\ndef atomic(_changeset, _opts, _context) do\n  :ok\nend\n```","ref":"changes.html#atomic-changes"},{"type":"extras","title":"Batches - Changes","doc":"Changes can support being run on batches of changesets, using the `c:Ash.Resource.Change.batch_change/3`, `c:Ash.Resource.Change.before_batch/3`, and `c:Ash.Resource.Change.after_batch/3` callbacks.\n\n>","ref":"changes.html#batches"},{"type":"extras","title":"When do we use batch callbacks? - Changes","doc":">\n> `c:Ash.Resource.Change.batch_change/3` must be defined for `c:Ash.Resource.Change.before_batch/3` and `c:Ash.Resource.Change.after_batch/3` to be called!\n>\n> The one exception is that `after_batch/3` is called after atomic changes as well as batches, so it will be called in cases that `atomic/3` is defined. `before_batch/3` is ignored when calling changes atomically.\n\nFor some changes, this may not be necessary at all, i.e the `Slugify` example has no benefit from batching. If no batch callbacks are added, your change will be run on a loop over the changesets. For the sake of example, however, we will show what it might look like to implement batching for our `Slugify` example.\n\n```elixir\ndefmodule MyApp.Changes.Slugify do\n  # transform and validate opts\n\n  use Ash.Resource.Change\n\n  @impl true\n  def init(opts) do\n    if is_atom(opts[:attribute]) do\n      {:ok, opts}\n    else\n      {:error, \"attribute must be an atom!\"}\n    end\n  end\n\n  @impl true\n  def batch_change(changeset, opts, context) do\n    # here we could run queries or do common work required\n    # for a given batch of changesets.\n    # in this example, however, we just return the changesets with\n    # the change logic applied.\n    Enum.map(changesets, &change(&1, opts, context))\n  end\nend\n```","ref":"changes.html#when-do-we-use-batch-callbacks"},{"type":"extras","title":"Preparations","doc":"# Preparations\n\nPreparations are the primary way of customizing read action behavior. If you are familiar with `Plug`, you can think of an `Ash.Resource.Preparation` as the equivalent of a `Plug` for queries. At its most basic, a preparation will take a query and return a new query. Queries can be simple, like adding a filter or a sort, or more complex, attaching hooks to be executed within the lifecycle of the action.","ref":"preparations.html"},{"type":"extras","title":"Builtin Preparations - Preparations","doc":"There are builtin preparations that can be used, and are automatically imported into your resources. See `Ash.Resource.Preparation.Builtins` for more.\n\nThe primary preparation you will use is `build/1`, which passes the arguments through to `Ash.Query.build/2` when the preparation is run. See that function for what options can be provided.\n\nSome examples of usage of builtin preparations\n\n```elixir\n# sort by inserted at descending\nprepare build(sort: [inserted_at: :desc])\n\n# only show the top 5 results\nprepare build(sort: [total_points: :desc], limit: 5)\n```","ref":"preparations.html#builtin-preparations"},{"type":"extras","title":"Custom Preparations - Preparations","doc":"```elixir\ndefmodule MyApp.Preparations.Top5 do\n  use Ash.Resource.Preparation\n\n  # transform and validate opts\n  @impl true\n  def init(opts) do\n    if is_atom(opts[:attribute]) do\n      {:ok, opts}\n    else\n      {:error, \"attribute must be an atom!\"}\n    end\n  end\n\n  @impl true\n  def prepare(query, opts, _context) do\n    attribute = opts[:attribute]\n\n    query\n    |> Ash.Query.sort([{attribute, :desc}])\n    |> Ash.Query.limit(5)\n  end\nend\n```\n\nThis could then be used in a resource via:\n\n```elixir\nprepare {MyApp.Preparations.Top5, attribute: :foo}\n```","ref":"preparations.html#custom-preparations"},{"type":"extras","title":"Anonymous Function Queries - Preparations","doc":"You can also use anonymous functions for preparations. This is great for prototyping, but we generally recommend using a module for organizational purposes.\n\n```elixir\nprepare fn query, _context ->\n  # put your code here\nend\n```","ref":"preparations.html#anonymous-function-queries"},{"type":"extras","title":"Action vs Global Preparations - Preparations","doc":"You can place a preparation on a read action, like so:\n\n```elixir\nactions do\n  read :read do\n    prepare {Top5, attribute: :name}\n  end\nend\n```\n\nOr you can use the global preparations block to apply to all read actions.\n\n```elixir\npreparations do\n  prepare {Top5, attribute: :name}\nend\n```\n\nThe preparations section allows you to add preparations across multiple actions of a resource.","ref":"preparations.html#action-vs-global-preparations"},{"type":"extras","title":"Code Interface","doc":"# Code Interface\n\nOne of the ways that we interact with our resources is via hand-written code. The general pattern for that looks like building a query or a changeset for a given action, and calling it via functions like `Ash.read/2` and `Ash.create/2`. This, however, is just one way to use Ash, and is designed to help you build tools that work with resources, and to power things like `AshPhoenix.Form`, `AshGraphql.Resource` and `AshJsonApi.Resource`. When working with your resources in code, we generally want something more idiomatic and simple. For example, on a domain called `Helpdesk.Support`.\n\n```elixir\nresources do\n  resource Ticket do\n    define :open_ticket, args: [:subject], action: :open\n  end\nend\n```\n\nThis simple setup now allows you to open a ticket with `Helpdesk.Support.open_ticket(subject)`. You can cause it to raise errors instead of return them with `Helpdesk.Support.open_ticket!(subject)`. For information on the options and additional inputs these defined functions take, look at the generated function documentation, which you can do in iex with `h Helpdesk.Support.open_ticket`. For more information on the code interface, read the DSL documentation: `d:Ash.Domain.Dsl.resources.resource.define`.","ref":"code-interfaces.html"},{"type":"extras","title":"Code interfaces on the resource - Code Interface","doc":"You can define a code interface on individual resources as well, using the `code_interface` block. The DSL is the same as the DSL for defining it in the `domain`. For example:\n\n```elixir\ncode_interface do\n  # the action open can be omitted because it matches the function name\n  define :open, args: [:subject]\nend\n```\n\nThese will then be called on the resource itself, i.e `Helpdesk.Support.Ticket.open(subject)`.","ref":"code-interfaces.html#code-interfaces-on-the-resource"},{"type":"extras","title":"Using the code interface - Code Interface","doc":"If the action is an update or destroy, it will take a record or a changeset as its _first_ argument.\nIf the action is a read action, it will take a starting query as an _opt in the last_ argument.\n\nAll functions will have an optional last argument that accepts options. See `Ash.Resource.Interface` for valid options.\n\nFor reads:\n\n- `:query` - a query to start the action with, can be used to filter/sort the results of the action. This can be a keyword list of any of the options that `Ash.Query.build/3` supports.\n\nFor creates:\n\n- `:changeset` - a changeset to start the action with\n\nThey will also have an optional second to last argument that is a freeform map to provide action input. It _must be a map_.\nIf it is a keyword list, it will be assumed that it is actually `options` (for convenience).\nThis allows for the following behaviour:\n\n```elixir\n# Because the 3rd argument is a keyword list, we use it as options\nAccounts.register_user(username, password, [tenant: \"organization_22\"])\n# Because the 3rd argument is a map, we use it as action input\nAccounts.register_user(username, password, %{key: \"val\"})\n# When all arguments are provided it is unambiguous\nAccounts.register_user(username, password, %{key: \"val\"}, [tenant: \"organization_22\"])\n```\n\nFor a full list of options, see the functions in `Ash.Resource.Interface`, or use iex help on your generated\nfunctions, i.e\n\n```elixir\niex> h Accounts.register_user/3\n```","ref":"code-interfaces.html#using-the-code-interface"},{"type":"extras","title":"get_by functions - Code Interface","doc":"A common pattern in Ash applications is the \"get by\" function for retrieving individual records. This pattern provides a clean alternative to using `Ash.get!/2` directly in your web modules.\n\n**Avoid this pattern:**\n\n```elixir\n# In a LiveView or Controller - DON'T DO THIS\ngroup =\n  MyApp.Ash.Dashboards.DashboardGroup\n  |> Ash.get!(group_id)\n  |> Ash.load!(students: [:user])\n```\n\nThis is similar to using `Repo.get/2` and `Repo.preload/2` directly outside of context modules, which is generally considered a bad practice.\n\n**Use this pattern instead:**\n\n```elixir\n# In your domain\nresource DashboardGroup do\n  define :get_by_id, action: :read, get_by: [:id]\nend\n\n# In your LiveView or Controller\ngroup = MyApp.Dashboards.get_dashboard_group_by_id!(group_id)\n```\n\nThe `get_by` option automatically creates a function that:\n- Uses the primary read action with an applied filter\n- Supports dynamic loading and filtering through the standard options\n- Provides both raising (`!`) and non-raising versions\n\n**Dynamic loading and filtering:**\n\nCode interfaces automatically support loading and filtering options:\n\n```elixir\n# Load relationships\nMyApp.Dashboards.get_dashboard_group_by_id!(id, load: [students: [:user]])\n\n# Apply additional filters\nMyApp.Dashboards.get_dashboard_group_by_id!(id, query: [filter: [status: :active]])\n\n# Combine both\nMyApp.Dashboards.get_dashboard_group_by_id!(id,\n  load: [students: [:user]],\n  query: [filter: [status: :active]]\n)\n```\n\nThe `query` option accepts either an `Ash.Query` struct or a keyword list that gets passed to `Ash.Query.build/2`.\n\n**When to use actions vs code interfaces:**\n\n- **Actions** define what operations are possible on your resource\n- **Code interfaces** provide function-based access to those actions\n- Actions don't require code interfaces and can be used by extensions like `AshJsonApi`\n- Code interfaces make actions callable as functions (e.g., `DashboardGroup.get_by_id/1`)\n\nThis pattern encourages proper separation of concerns by keeping domain logic in your resources and providing clean interfaces for your web layer.","ref":"code-interfaces.html#get_by-functions"},{"type":"extras","title":"Calculations - Code Interface","doc":"Resource calculations can be run dynamically using `Ash.calculate/3`, but\nyou can also expose them using the code_interface with `define_calculation`.\n\nFor example:\n\n```elixir\ncalculations do\n  calculate :full_name, :string, expr(first_name <> ^arg(:separator) <> last_name) do\n    argument :separator, :string do\n      allow_nil? false\n      default \" \"\n    end\n  end\nend\n\n# in your domain\nresource User do\n  define_calculation :full_name, args: [:first_name, :last_name, {:optional, :separator}]\n  # or if you want to take a record as an argument\n  define_calculation :full_name, args: [:_record]\nend\n```\n\nThis could now be used like so:\n\n```elixir\nAccounts.full_name(\"Jessie\", \"James\", \"-\")\n# or with a record as an argument\nAccounts.full_name(user)\n```\n\nThis allows for running calculations without an instance of a resource, normally done via `Ash.load(user, :full_name)`\n\nBy default, configured args will be provided for any matching named reference _or_ argument. This is normally fine, but in the case that you have an argument and a reference with the same name, you can specify it by supplying `{:arg, :name}` and `{:ref, :name}`. For example:\n\n```elixir\ndefine_calculation :id_matches, args: [{:arg, :id}, {:ref, :id}]\n```\n\nTo make arguments optional, wrap them in `{:optional, ..}`, for example:\n\n```elixir\ndefine_calculation :id_matches, args: [{:arg, :id}, {:optional, {:ref, :id}}]\n```","ref":"code-interfaces.html#calculations"},{"type":"extras","title":"Bulk & atomic actions - Code Interface","doc":"","ref":"code-interfaces.html#bulk-atomic-actions"},{"type":"extras","title":"Bulk Updates & Destroys - Code Interface","doc":"Updates support a list, stream, or query as the first argument. This allows for bulk updates. In this mode, an `%Ash.BulkResult{}` is returned.\n\n>","ref":"code-interfaces.html#bulk-updates-destroys"},{"type":"extras","title":"Valid inputs - Code Interface","doc":">\n> You cannot provide \"any enumerable\", only lists, streams (a function or a %Stream{}), and queries. We have to be able to distinguish the input as a bulk input and not input to the action itself.\n\nFor example:\n\n```elixir\nPost\n|> Ash.Query.filter(author_id == ^author_id)\n|> MyApp.Blog.archive_post!()\n# => %Ash.BulkResult{}\n\n[%Post{}, %Post{}]\n|> MyApp.Blog.destroy_post!()\n# => %Ash.BulkResult{}\nend\n```\n\nYou can pass options to the bulk operation with the `bulk_options` option to your code interface function.","ref":"code-interfaces.html#valid-inputs"},{"type":"extras","title":"Bulk Creates - Code Interface","doc":"For bulk creates, you can provide a list or stream of inputs. In this mode also, an `%Ash.BulkResult{}` is returned.\n\n>","ref":"code-interfaces.html#bulk-creates"},{"type":"extras","title":"Valid inputs - Code Interface","doc":">\n> You cannot provide \"any enumerable\", only lists, streams (a function or a %Stream{}). We have to be able to distinguish the input as a bulk input and not input to the action itself.\n\nAny arguments on the code interface will be applied to _all_ inputs given as a list, and the arguments will come first.\n\n```elixir\n[%{title: \"Post 1\"}, %{title: \"Post 2\"}, ...]\n# if `:special` is an action argument, it will be applied to all inputs\n|> MyApp.Blog.create_post!(:special, bulk_options: [batch_size: 10])\n```","ref":"code-interfaces.html#valid-inputs-1"},{"type":"extras","title":"Returning streams from read actions - Code Interface","doc":"The `:stream?` option allows you to return a stream to be enumerated later.\n\nFor example:\n\n```elixir\nMyApp.Blog.my_posts(stream?: true, actor: me)\n# => #Stream<...>\n```","ref":"code-interfaces.html#returning-streams-from-read-actions"},{"type":"extras","title":"Customizing the generated function - Code Interface","doc":"Often we want to have a slightly different interface when calling actions with functions,\nor we want to maintain backwards compatibility for callers of our code interface while\nchanging the underlying action implementation.\n\nYou can define `custom_input`s on your code interfaces to massage arguments from the function\ninto a shape expected by the action.\n\nFor example, lets say we have an action that accepts an `artist_id` as an argument. We want\nit to use `artist_id` for two reasons: it is the only part of the artist required to perform\nthe action, and accepting ids is better for an action supporting usage over an API. However,\nwe want the function itself to accept either an artist *or* an artist_id.\n\n```elixir\ndefine :follow_artist do\n  action :follow\n\n  # `artist` (from the custom input below) is a positional argument to the function\n  args [:artist]\n\n  # make a custom input called `artist`, that is a union type\n  custom_input :artist, :union do\n    # allow passing either an artist or an artist_id\n    constraints types: [\n      artist: [type: :struct, constraints: [instance_of: Artist]],\n      artist_id: [type: :uuid]\n    ]\n\n    transform do\n      # Pass it to the action as `artist_id`\n      to :artist_id\n\n      # Extracting the value using this function\n      using fn\n        %Ash.Union{type: :artist, value: value} ->\n          value.id\n        %Ash.Union{type: :artist_id, value: value} ->\n          value\n      end\n    end\n  end\nend\n```\n\nThe example above is a bit verbose. In practice we might create a type, called `ArtistOrId`,\nfor example, and extract that logic like so:\n\n```elixir\ndefmodule MyApp.Types.ArtistOrId do\n  use Ash.Type.NewType, subtype_of: :union, constraints: [\n    types: [\n      artist: [type: :struct, constraints: [instance_of: Artist]],\n      artist_id: [type: :uuid]\n    ]\n  ]\n\n  def to_artist_id(%Ash.Union{type: :artist, value: artist}), do: artist.id\n  def to_artist_id(%Ash.Union{type: :artist_id, value: artist_id}), do: artist_id\nend\n```\n\nAnd then we can refactor the above example like so:\n\n```elixir\ndefine :follow_artist do\n  action :follow\n  args [:artist]\n\n  custom_input :artist, MyApp.Types.ArtistOrId do\n    transform do\n      to :artist_id\n      using &MyApp.Types.ArtistOrId.to_artist_id/1\n    end\n  end\nend\n```","ref":"code-interfaces.html#customizing-the-generated-function"},{"type":"extras","title":"Authorization Functions - Code Interface","doc":"For each action defined in a code interface, Ash automatically generates corresponding authorization check functions:\n\n- `can_action_name?(actor, params \\\\ %{}, opts \\\\ [])` - Returns `true`/`false` for authorization checks\n- `can_action_name(actor, params \\\\ %{}, opts \\\\ [])` - Returns `{:ok, true/false}` or `{:error, reason}`\n\nExample usage:\n```elixir\n# Check if user can create a post\nif MyApp.Blog.can_create_post?(current_user) do\n  # Show create button\nend\n\n# Check if user can update a specific post\nif MyApp.Blog.can_update_post?(current_user, post) do\n  # Show edit button\nend\n\n# Check if user can destroy a specific comment\nif MyApp.Blog.can_destroy_comment?(current_user, comment) do\n  # Show delete button\nend\n```\n\nThese functions are particularly useful for conditional rendering of UI elements based on user permissions.","ref":"code-interfaces.html#authorization-functions"},{"type":"extras","title":"Embedded Resources","doc":"# Embedded Resources\n\nEmbedded resources are stored as maps in attributes of other resources. They are great for storing structured data, and support a whole range of useful features that resources support. For example, you can have calculations, validations, policies and even relationships on embedded resources. Here is an example of a simple embedded resource:\n\n```elixir\ndefmodule MyApp.Profile do\n  use Ash.Resource,\n    data_layer: :embedded # Use the atom `:embedded` as the data layer.\n\n  attributes do\n    attribute :first_name, :string, public?: true\n    attribute :last_name, :string, public?: true\n  end\nend\n```\n\n>","ref":"embedded-resources.html"},{"type":"extras","title":"Embedded resources can't do everything - Embedded Resources","doc":">\n> Embedded resources cannot have aggregates, or expression calculations that rely on data-layer-specific capabilities.","ref":"embedded-resources.html#embedded-resources-can-t-do-everything"},{"type":"extras","title":"Adding embedded resource attributes - Embedded Resources","doc":"Embedded resources define an `Ash.Type` under the hood, meaning you can use them anywhere you would use an Ash type.\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource, ...\n\n  attributes do\n    ...\n\n    attribute :profile, MyApp.Profile, public?: true\n    attribute :profiles, {:array, MyApp.Profile}, public?: true # You can also have an array of embeds\n  end\nend\n```","ref":"embedded-resources.html#adding-embedded-resource-attributes"},{"type":"extras","title":"Handling nil values - Embedded Resources","doc":"By default, all fields on an embedded resource will be included in the data layer, including keys with nil values. To prevent this, add the `embed_nil_values?` option to `use Ash.Resource`. For example:\n\n```elixir\ndefmodule YourEmbed do\n  use Ash.Resource,\n    data_layer: :embedded,\n    embed_nil_values?: false\nend\n```","ref":"embedded-resources.html#handling-nil-values"},{"type":"extras","title":"Editing embedded attributes - Embedded Resources","doc":"If you manually supply instances of the embedded structs, the structs you provide are used with no validation. For example:\n\n```elixir\nAsh.Changeset.for_update(user, :update, %{profile: %MyApp.Profile{first_name: \"first_name\", last_name: \"last_name\"}})\n```\n\nHowever, you can also treat embedded resources like regular resources that can be \"created\", \"updated\", and \"destroyed\".\nTo do this, provide maps as the input, instead of structs. In the example above, if you just wanted to change the `first_name`, you'd provide:\n\n```elixir\nAsh.Changeset.for_update(user, :update, %{profile: %{first_name: \"first_name\"}})\n```\n\nThis will call the primary `update` action on the resource. This allows you to define an action on the embed, and add validations to it. For example, you might have something like this:\n\n```elixir\ndefmodule MyApp.Profile do\n  use Ash.Resource,\n    data_layer: :embedded # Use the atom `:embedded` as the data layer.\n\n  attributes do\n    attribute :first_name, :string, public?: true\n    attribute :last_name, :string, public?: true\n  end\n\n  validations do\n    validate present([:first_name, :last_name], at_least: 1)\n  end\nend\n```","ref":"embedded-resources.html#editing-embedded-attributes"},{"type":"extras","title":"Calculations - Embedded Resources","doc":"Calculations can be added to embedded resources. When you use an embedded resource, you declare what calculations to load via a `constraint`.\nFor example:\n\n```elixir\ndefmodule MyApp.Profile do\n  use Ash.Resource,\n    data_layer: :embedded # Use the atom `:embedded` as the data layer.\n\n  attributes do\n    attribute :first_name, :string, public?: true\n    attribute :last_name, :string, public?: true\n  end\n\n  calculations do\n    calculate :full_name, :string, concat([:first_name, :last_name], \" \")\n  end\nend\n\ndefmodule MyApp.User do\n  use Ash.Resource,\n    ...\n\n  attributes do\n    attribute :profile, MyApp.Profile do\n      public? true\n      constraints [load: [:full_name]]\n    end\n  end\nend\n```","ref":"embedded-resources.html#calculations"},{"type":"extras","title":"Determining what action(s) will be called: - Embedded Resources","doc":"Remember: default actions are already implemented for a resource, with no need to add them. They are called `:create`, `:update`, `:destroy`, and `:read`. You can use those without defining them. You only need to define them if you wish to override their configuration.","ref":"embedded-resources.html#determining-what-action-s-will-be-called"},{"type":"extras","title":"Single Embeds - Embedded Resources","doc":"- If the current value is `nil` - a `create` with the provided values\n- If the current value is not `nil` - an `update` with the provided values\n- If the current value is not `nil` and the _new value_ is `nil` - a `destroy` with the original value","ref":"embedded-resources.html#single-embeds"},{"type":"extras","title":"Array Embeds - Embedded Resources","doc":"All values in the original array are destroyed, and all maps in the new array are used to `create` new records.","ref":"embedded-resources.html#array-embeds"},{"type":"extras","title":"Adding a primary key - Embedded Resources","doc":"Adding a primary key to your embedded resources is especially useful when managing lists of data. Specifically, it allows you to consider changes to elements with matching primary key values as `updates`.\n\nFor example:\n\n```elixir\ndefmodule MyApp.Tag do\n  use Ash.Resource,\n    data_layer: :embedded\n\n  attributes do\n    uuid_primary_key :id\n    attribute :name, :string, public?: true\n    attribute :counter, :integer, public?: true\n  end\n\n  validations do\n    validate {Increasing, field: :counter}, on: :update\n  end\nend\n```\n\nNow, you can accept input meant to `update` individual list items. The entire list must still be provided, but any items with a matching id will be considered an `update` instead of a `destroy` + `create`.","ref":"embedded-resources.html#adding-a-primary-key"},{"type":"extras","title":"Determining what action(s) will be called with a primary key: - Embedded Resources","doc":"#### Single Embeds with primary keys\n\n- If you provide a struct, instead of a map, the value provided is used as the new relationship value directly.\n- If the current value is `nil` - a `create` with the provided values\n- If the current value is not `nil` and the primary keys match - an `update` with the provided values\n- If the current value is not `nil` and the primary keys _don't_ match - a `destroy` of the original value and a `create` of the new value\n- If the current value is not `nil` and the _new value_ is `nil` - a `destroy` with the original value\n\n#### Array Embeds with primary keys\n\n- If you provide structs, instead of maps, the value provided is used as the new relationship value directly.\n- Any values in the original list with no primary key matching in the new list are `destroy`ed.\n- Any values in the new list with no primary key matching in the original list are `create`d.\n- Any values with a primary key match in the original list and the new list are `update`d","ref":"embedded-resources.html#determining-what-action-s-will-be-called-with-a-primary-key"},{"type":"extras","title":"Identities - Embedded Resources","doc":"Identities can be added on an embedded resource, which will ensure that for any given list, they are unique on that identity. For example, if you had an embedded resource called `Tag`, you could add an identity on `name` to ensure that nothing has duplicate tag names.","ref":"embedded-resources.html#identities"},{"type":"extras","title":"Usage in Extensions - Embedded Resources","doc":"The AshJsonApi extension exposes these attributes as maps. However, the AshGraphql extension allows you\nto specify a type (but not queries/mutations) for an embedded resource. If you do, instead of being treated as a `:json` type it will get its own named input object type and field type.","ref":"embedded-resources.html#usage-in-extensions"},{"type":"extras","title":"Accessing the source changeset - Embedded Resources","doc":"When building changesets for embedded resources, the source changeset will be available in action changes under `changeset.context.__source__`.\nThis allows you to customize the action based on the details of the parent changeset.","ref":"embedded-resources.html#accessing-the-source-changeset"},{"type":"extras","title":"Identities","doc":"# Identities\n\nIdentities are a way to declare that a record (an instance of a resource) can be uniquely identified by a set of attributes. This information can be used in various ways throughout the framework. The primary key of the resource does not need to be listed as an identity.","ref":"identities.html"},{"type":"extras","title":"Defining an identity - Identities","doc":"Identities are defined at the top level of a resource module, eg.\n\n```elixir\ndefmodule MyApp.MyResource do\n  use Ash.Resource #, ...\n  # ...\n\n  identities do\n    # If the `email` attribute must be unique across all records\n    identity :unique_email, [:email]\n\n    # If the `username` attribute must be unique for every record with a given `site` value\n    identity :special_usernames, [:username, :site]\n\n    # If the `user_id` field should hold the errors for the uniqueness violation\n    identity :unique_email, [:email], field_names: [:user_id]\n  end\nend\n```\n\nSee `d:Ash.Resource.Dsl.identities` for the full range of options available when defining identities.","ref":"identities.html#defining-an-identity"},{"type":"extras","title":"Using Ash.get - Identities","doc":"This will allow these fields to be passed to `Ash.get/3`, e.g `Ash.get(Resource, %{email: \"foo\"})`.","ref":"identities.html#using-ash-get"},{"type":"extras","title":"Using upserts - Identities","doc":"Create actions support the `upsert?: true` option, if the data layer supports it. An `upsert?` involves checking for a conflict on some set of attributes, and translating the behavior to an update in the case one is found. By default, the primary key is used when looking for duplicates, but you can set `[upsert?: true, upsert_identity: :identity_name]` to tell it to look for conflicts on a specific identity.","ref":"identities.html#using-upserts"},{"type":"extras","title":"Creating unique constraints - Identities","doc":"Tools like `AshPostgres` will create unique constraints in the database automatically for each identity. These unique constraints will honor other configuration on your resource, like the `base_filter` and [attribute multitenancy](/documentation/topics/multitenancy.md#attribute-multitenancy)","ref":"identities.html#creating-unique-constraints"},{"type":"extras","title":"Eager Checking - Identities","doc":"Setting `eager_check?: true` on an identity will allow that identity to be checked when building a create changeset over the resource. This allows for showing quick up-front validations about whether some value is taken, for example. If the resource does not have the domain configured, you can specify the domain to use with `eager_check_with: DomainName`.\n\nIf you are using `AshPhoenix.Form`, for example, this looks for a conflicting record on each call to `Form.validate/2`.\nFor updates, it is only checked if one of the involved fields is being changed.\n\nFor creates, The identity is checked unless your are performing an `upsert`, and the `upsert_identity` is this identity. Keep in mind that for this to work properly, you will need to pass the `upsert?: true, upsert_identity: :identity_name` _when creating the changeset_. The `primary?` read action is used to search for a record. This will error if you have not configured one.","ref":"identities.html#eager-checking"},{"type":"extras","title":"Pre Checking - Identities","doc":"`pre_check?` behaves the same as `eager_check?`, but it runs just prior to the action being committed. Useful for data layers that don't support transactions/unique constraints, or manual resources with identities. `Ash.DataLayer.Ets` will require you to set `pre_check?` since the ETS data layer has no built in support for unique constraints. The domain can be manually specified with `pre_check_with: DomainName`.","ref":"identities.html#pre-checking"},{"type":"extras","title":"Notifiers","doc":"# Notifiers","ref":"notifiers.html"},{"type":"extras","title":"What are notifiers for? - Notifiers","doc":"Notifiers allow you to tap into create, update and destroy actions on a resource. Notifiers are called after\nthe current transaction is committed, which solves a lot of problems that can happen from performing a certain\nkind of side effect in your action code.\n\nA common example of one such issue is using Phoenix PubSub to notify another part of your app (often a LiveView or\nphoenix channel) of a change. If you send a message to another process while your transaction is still open, and\nthat process tries to look up a record you just created, it won't find it yet, because your transaction is still open!\n\nNotifiers are a solution for a certain kind of side effect, what we call \"at most once\" effects. An example is\nsending an event to an analytics system, or our pubsub example above. It is \"okay\" if the event is fired and some\nerror in that process prevents it from being sent.","ref":"notifiers.html#what-are-notifiers-for"},{"type":"extras","title":"When you really need an event to happen - Notifiers","doc":"In these cases you are looking for something other than a notifier. For example, you may want to look into integrating\nhttps://hexdocs.pm/oban into your application, allowing you to commit a \"job\" in the same transaction as your changes, to be processed later.\n\nAlternatively, you could look into using `Reactor`, which is designed for writing \"sagas\" and has first-class support\nfor Ash via the `AshReactor` extension.","ref":"notifiers.html#when-you-really-need-an-event-to-happen"},{"type":"extras","title":"Including a notifier in a resource - Notifiers","doc":"If the notifier is also an extension, include it in the `notifiers` key:\n\n```elixir\ndefmodule MyResource do\n  use Ash.Resource,\n    notifiers: [ExampleNotifier]\nend\n```\n\nConfiguring a notifier for a specific action or actions can be a great way to avoid complexity in the implementation of a notifier. It allows you to avoid doing things like pattern matching on the action, and treat it more like a change module, that does its work whenever it is called.\n\n```elixir\ncreate :create do\n  notifiers [ExampleNotifier]\nend\n```\n\nWhen your notifier is not an extension, and you want it to run on all actions, include it this way to avoid unnecessary compile time dependencies:\n\n```elixir\ndefmodule MyResource do\n  use Ash.Resource,\n    simple_notifiers: [ExampleNotifier]\nend\n```","ref":"notifiers.html#including-a-notifier-in-a-resource"},{"type":"extras","title":"Built-in Notifiers - Notifiers","doc":"Ash comes with a builtin pub_sub notifier: `Ash.Notifier.PubSub`. See the module documentation for more.","ref":"notifiers.html#built-in-notifiers"},{"type":"extras","title":"Creating your own notifier - Notifiers","doc":"A notifier is a simple extension that must implement a single callback `notify/1`. Notifiers do not have to implement an Ash DSL extension, but they may in order to configure how that notifier should behave. See `Ash.Notifier.Notification` for the currently available fields on a notification.\n\nFor more information on creating a DSL extension to configure your notifier, see the docs for `Spark.Dsl.Extension`.\n\n>","ref":"notifiers.html#creating-your-own-notifier"},{"type":"extras","title":"Notifier performance - Notifiers","doc":">\n> Notifiers should not do intensive synchronous work. If any heavy work needs to be done, they should delegate to something else to handle the notification, like sending it to a GenServer or GenStage.","ref":"notifiers.html#notifier-performance"},{"type":"extras","title":"Example notifier - Notifiers","doc":"```elixir\ndefmodule ExampleNotifier do\n  use Ash.Notifier\n\n  def notify(%Ash.Notifier.Notification{resource: resource, action: %{type: :create}, actor: actor}) do\n    if actor do\n      Logger.info(\"#{actor.id} created a #{resource}\")\n    else\n      Logger.info(\"A non-logged in user created a #{resource}\")\n    end\n  end\nend\n```","ref":"notifiers.html#example-notifier"},{"type":"extras","title":"Transactions - Notifiers","doc":"Domain calls involving resources who's datalayer supports transactions (like Postgres), notifications are saved up and sent after the transaction is closed. For example, the domain call below ultimately results in many many database calls.\n\n```elixir\nPost\n|> Ash.Changeset.for_update(:update, %{})\n|> Ash.Changeset.manage_relationship(:related_posts, [1, 2, 3], type: :append)\n|> Ash.Changeset.manage_relationship(:related_posts, [4, 5], type: :remove)\n|> Ash.Changeset.manage_relationship(:comments, [10], type: :append)\n|> Ash.update!()\n```\n\n`Ash.Changeset.manage_relationship` doesn't leverage bulk operations yet, so it performs the following operations:\n\n- a read of the currently related posts\n- a read of the currently related comments\n- a creation of a post_link to relate to 1\n- a creation of a post_link to relate to 2\n- a creation of a post_link to relate to 3\n- a destruction of the post_link related to 4\n- a destruction of the post_link related to 5\n- an update to comment 10, to set its `post_id` to this post\n\nIf all three of these resources have notifiers configured, we need to send a notification for each operation (notifications are not sent for reads). For data consistency reasons, if a data layer supports transactions, all writes are done in a transaction. However, if you try to read the record from the database that you have just received a notification about before the transaction has been closed, in a different process, the information will be wrong. For this reason, Ash accumulates notifications until they can be sent.\n\nIf you need to perform multiple operations against your resources in your own transaction, you will have to handle that case yourself. To support this, `Ash.create/2`, `Ash.update/2` and `Ash.destroy/2` support a `return_notifications?: true` option. This causes the domain call to return `{:ok, result, notifications}` in the successful case. Here is an example of how you might use it.\n\n```elixir\nresult =\n  Ash.DataLayer.transaction(resource, fn ->\n    {:ok, something, notifications1} = create_something()\n    {:ok, result, notifications2} = create_another_thing(something)\n    {:ok, notifications3} = destroy_something(something)\n\n    {result, Enum.concat([notifications1, notifications2, notifications3])}\n  end)\n\ncase result do\n  {:ok, value, notifications} ->\n     Ash.Notifier.notify(notifications)\n\n     value\n  {:error, error} ->\n    handle_error(error)\nend\n```","ref":"notifiers.html#transactions"},{"type":"extras","title":"Actions","doc":"# Actions\n\nIn Ash, actions are the primary way to interact with your resources. There are five types of actions:\n\n- [Read](documentation/topics/actions/read-actions.md)\n- [Create](documentation/topics/actions/create-actions.md)\n- [Update](documentation/topics/actions/update-actions.md)\n- [Destroy](documentation/topics/actions/destroy-actions.md)\n- [Generic](documentation/topics/actions/generic-actions.md)\n\nAll actions can be run in a transaction. Create, update and destroy actions are run in a transaction by _default_, whereas read and generic actions require opting in with `transaction? true` in the action definition. Each action has its own set of options, ways of calling it, and ways of customizing it. See the relevant guide for specifics on each action type. This topic focuses on idiomatic ways to use actions, and concepts that cross all action types.","ref":"actions.html"},{"type":"extras","title":"Primary Actions - Actions","doc":"Primary actions are a way to inform the framework which actions should be used in certain \"automated\" circumstances, or in cases where an action has not been specified. If a primary action is attempted to be used but does not exist, you will get an error about it at runtime.\n\nThe place you typically need primary actions is when [Managing Relationships](/documentation/topics/resources/relationships.md#managing-relationships). When using the `defaults` option to add default actions, they are marked as primary.\n\nA simple example where a primary action would be used:\n\n```elixir\n# No action is specified, so we look for a primary read.\nAsh.get!(Resource, \"8ba0ab56-c6e3-4ab0-9c9c-df70e9945281\")\n```\n\nTo mark an action as primary, add the option, i.e\n\n```elixir\nread :action_name do\n  primary? true\nend\n```","ref":"actions.html#primary-actions"},{"type":"extras","title":"Accepting Inputs - Actions","doc":"[Create](documentation/topics/actions/create-actions.md) and [Update](documentation/topics/actions/update-actions.md) actions can accept attributes as input. There are two primary ways that you annotate this.","ref":"actions.html#accepting-inputs"},{"type":"extras","title":"Using accept in specific actions - Actions","doc":"Each action can define what it accepts, for example:\n\n```elixir\ncreate :create do\n  accept [:name, :description]\nend\n```\n\nYou could then pass in `%{name: \"a name\", description: \"a description\"}` to this action.","ref":"actions.html#using-accept-in-specific-actions"},{"type":"extras","title":"Using default_accept for all actions - Actions","doc":"The resource can have a `default_accept`, declared in its `actions` block, which will be used as the accept list for `create` and `update` actions, if they don't define one.\n\n```elixir\nactions do\n  default_accept [:name, :description]\n\n  create :create\n  update :update\n\n  update :special_update do\n    accept [:something_else]\n  end\nend\n```\n\nIn the example above, you can provide `%{name: \"a name\", description: \"a description\"}` to both the `:create` and `:update` actions, but only `%{something_else: \"some_value\"}` to `:special_update`.\n\n#### Using module attributes for action specific accept lists\n\nYou can also use module attributes to define the accept list.  This is useful if you have a lot of attributes and different variations for different actions.\n\n```elixir\n@accepts_special_update [:name, :description, :foo, :bar, :baz]\n\n@accepts_super_special_update @accepts_special_update ++ [:something_else, :another_thing]\n\nactions do\n  default_accept [:name, :description]\n\n  create :create\n  update :update\n\n  update :special_update do\n    accept @accepts_special_update\n  end\nend\n```\n\nThis is extremely simple example","ref":"actions.html#using-default_accept-for-all-actions"},{"type":"extras","title":"Context - Actions","doc":"There are two kinds of contexts in Ash:\n\n1. the context given to a changeset/action call, stored in `changeset.context`,\n2. the context given to a callback function like `c:Ash.Resource.Change.change/3`, which contains\n  the above context in it's `source_context` key, as well as additional information specific to the callback,\n  and/or commonly needed keys for callbacks (actor, tenant, etc.).\n\nActions accept a free-form map of context, which can be used for whatever you like. Whenever context is set, it is *deep merged*. I.e if you do `changeset |> Ash.Changeset.set_context(%{a: %{b: 1}}) |> Ash.Changeset.set_context(%{a: %{c: 2}})`, the resulting context will be `%{a: %{b: 1, c: 2}}`. Structs are not merged.\n\nThere are some special keys in context to note:\n\n### `:private`\n\nThe `:private` key is reserved for use by `Ash` itself. You shouldn't read from or write to it.\n\n### `:shared`\n\nThe `:shared` key will be passed to all nested actions built by Ash, and should be passed by you to any actions you call within changes/preparations etc. Whenever `:shared` context\nis set, it is also written to the outer context. For example `set_context(%{shared: %{locale: \"en\"}})` is equivalent to `set_context(%{shared: %{locale: \"en\"}, locale: \"en\"})`\n\nThis will generally happen automatically if you use one of the two abstractions provided by Ash for threading options through to nested action calls.\n\n>","ref":"actions.html#context"},{"type":"extras","title":"Careful with shared - Actions","doc":">\n> Shared context is passed to all nested actions, so don't pass massive values around, and also don't set context\n\n## `:query_for`\n\nThis is set on queries when they are being run for a \"special\" purpose. The values this can take are:\n\n- none, if a read action is being run, then no value is set for this context\n- `:bulk_update`, if the query is being built to power a bulk update action\n- `:bulk_destroy`, if the query is being built to power a bulk destroy action\n- `:load`, if the query is being built to power an `Ash.load` call\n\nYou can use this to adjust the behavior of your query preparations as needed.\n\n## `:bulk_create`, `:bulk_update`, `:bulk_destroy`\n\nThis is set on changesets when they are being run in bulk. The value will be a map with the following keys (more may be added in the future):\n\n`:index` -> The index of the changeset in the bulk operation.\n\n#### `Ash.Scope.ToOpts`\n\n`Ash.Scope.ToOpts` is newer and is the recommended way to do this. In action callbacks in Ash, you will be provided with a context, which can be passed down as a `scope` option when running nested actions or building nested changesets/queries. For example:\n\n```elixir\ndef change(changeset, opts, context) do\n  Ash.Changeset.after_action(changeset, fn changeset, result ->\n    # automatically passes the `shared` context to the nested action\n    MyApp.MyDomain.create_something_else(..., scope: context, other: :options)\n  end)\nend\n```\n\nTo get the opts for a given scope, you can use `Ash.Scope.to_opts(scope)`, but this is typically not\nnecessary.\n\n#### `Ash.Context.to_opts/2`\n\n`Ash.Context.to_opts/2` is a helper function that converts a context map into a list of options that can be passed to nested actions. It automatically passes the `shared` context to the nested action as well.\n\n```elixir\ndef change(changeset, opts, context) do\n  Ash.Changeset.after_action(changeset, fn changeset, result ->\n    # automatically passes the `shared` context to the nested action\n    MyApp.MyDomain.create_something_else(..., Ash.Context.to_opts(context, other: :options))\n  end)\nend\n```","ref":"actions.html#careful-with-shared"},{"type":"extras","title":"Idiomatic Actions - Actions","doc":"","ref":"actions.html#idiomatic-actions"},{"type":"extras","title":"Name Your Actions - Actions","doc":"The intent behind Ash is _not_ to have you building simple CRUD style applications. In a typical set up you may have a resource with four basic actions, there is even a shorthand to accomplish this:\n\n```elixir\nactions do\n  defaults [:read, :destroy, create: :*, update: :*]\nend\n```\n\nBut that is just a simple way to get started, or to create resources that really don't do anything beyond those four operations. You can have _as many actions as you want_. The best designed Ash applications will have numerous actions, named after the intent behind how they are used. They won't have all reads going through a single read action, and the same goes for the other action types. The richer the actions on the resource, the better interface you can have. With that said, many resources may only have those four basic actions, especially those that are \"managed\" through some parent resource. See the guide on [Managing Relationships](/documentation/topics/resources/relationships.md#managing-relationships) for more.","ref":"actions.html#name-your-actions"},{"type":"extras","title":"Put everything inside the action - Actions","doc":"Ash provides utilities to modify queries and changesets _outside_ of the actions on the resources. This is a very important tool in our tool belt, _but_ it is very easy to abuse. The intent is that as much behavior as possible is put into the action. Here is the \"wrong way\" to do it. There is a lot going on here, so don't hesitate to check out other relevant guides if you see something you don't understand.\n\n```elixir\ndef top_tickets(user_id) do\n  Ticket\n  |> Ash.Query.for_read(:read)\n  |> Ash.Query.filter(priority in [:medium, :high])\n  |> Ash.Query.filter(representative_id == ^user_id)\n  |> Ash.Query.filter(status == :open)\n  |> Ash.Query.sort(opened_at: :desc)\n  |> Ash.Query.limit(10)\n  |> Helpdesk.Support.read!()\nend\n\n# in the resource\n\nactions do\n  defaults [:read, ...]\nend\n```\n\nAnd here is the \"right way\", where the rules about getting the top tickets have been moved into the resource as a nicely named action, and included in the `code_interface` of that resource. The reality of the situation is that `top_tickets/1` is meant to be obsoleted by your Ash resource! Here is how it _should_ be done.\n\n```elixir\n# in the resource\n\ncode_interface do\n  define :top, args: [:user_id]\nend\n\nactions do\n  read :top do\n    argument :user_id, :uuid do\n      allow_nil? false\n    end\n\n    prepare build(limit: 10, sort: [opened_at: :desc])\n\n    filter expr(priority in [:medium, :high] and representative_id == ^arg(:user_id) and status == :open)\n  end\nend\n```\n\nNow, whatever code I had that would have called `top_tickets/1` can now call `Helpdesk.Support.Ticket.top(user.id)`. By doing it this way, you get the primary benefit of getting a nice simple interface to call into, but you _also_ have a way to modify how the action is invoked in any way necessary, by going back to the old way of building the query manually. For example, if I also only want to see top tickets that were opened in the last 10 minutes:\n\n```elixir\nTicket\n|> Ash.Query.for_read(:top, %{user_id: user.id})\n|> Ash.Query.filter(opened_at > ago(10, :minute))\n|> Helpdesk.Support.read!()\n```\n\nThat is the best of both worlds! These same lessons transfer to changeset based actions as well.","ref":"actions.html#put-everything-inside-the-action"},{"type":"extras","title":"Private Inputs - Actions","doc":"The concept of a \"private input\" can be somewhat paradoxical, but it can be used by actions that require something provided by the \"system\",\nas well as something provided by the caller. For example, you may want an `ip_address` input that can't be set by the user. For this,\nyou have two options.","ref":"actions.html#private-inputs"},{"type":"extras","title":"Private Options - Actions","doc":"```elixir\ncreate :create do\n  argument :ip_address, :string, allow_nil?: false, public?: false\n\n  ...\nend\n```\n\n```elixir\nAsh.Changeset.for_create(Resource, :create, %{}, private_arguments: %{ip_address: \" \"})\n```","ref":"actions.html#private-options"},{"type":"extras","title":"Context - Actions","doc":"You can also provide things to the action via `context`. Context is a map that is a free form map provided to the action.\nContext is occasionally used by callers to provide additional information that the action may or may not use.\n\nContext is _deep merged_ with any existing context, and also contains a `private` key that is reserved for use by Ash internals.\nYou should not remove or manipulate the `private` context key in any way.\n\n```elixir\ncreate :create do\n  ...\n  change fn changeset, _ ->\n    changeset.context # %{ip_address: \" \"}\n  end\nend\n```\n\n```elixir\nAsh.Changeset.for_create(Resource, :create, %{}, context: %{ip_address: \" \"})\n```","ref":"actions.html#context-1"},{"type":"extras","title":"Read Actions","doc":"# Read Actions\n\nRead actions operate on an `Ash.Query`. Read actions always return lists of data. The act of [pagination](#pagination), or returning a [single result](#ash-get), is handled as part of the interface, and is not a concern of the action itself. Here is an example of a read action:\n\n```elixir\n# Giving your actions informative names is always a good idea\nread :ticket_queue do\n  # Use arguments to take in values you need to run your read action.\n  argument :priorities, {:array, :atom} do\n    constraints items: [one_of: [:low, :medium, :high]]\n  end\n\n  # This action may be paginated,\n  # and returns a total count of records by default\n  pagination offset: true, countable: :by_default\n\n  # Arguments can be used in preparations and filters\n  filter expr(status == :open and priority in ^arg(:priorities))\nend\n```\n\nFor a full list of all of the available options for configuring read actions, see [the Ash.Resource.Dsl documentation](dsl-ash-resource.html#actions-read).","ref":"read-actions.html"},{"type":"extras","title":"Calling Read Actions - Read Actions","doc":"The basic formula for calling a read action looks like this:\n\n```elixir\nResource\n|> Ash.Query.for_read(:action_name, %{argument: :value}, ...opts)\n|> Ash.read!()\n```\n\nSee below for variations on action calling, and see the [code interface guide](/documentation/topics/code-interfaces.md) guide for how to\ndefine idiomatic and convenient functions that call your actions.","ref":"read-actions.html#calling-read-actions"},{"type":"extras","title":"Ash.get! - Read Actions","doc":"The `Ash.get!` function is a convenience function for running a read action, filtering by a unique identifier, and expecting only a single result. It is equivalent to the following code:\n\n```elixir\n# action can be omitted to use the primary read action\nAsh.get!(Resource, 1, action: :read_action)\n\n# is roughly equivalent to\n\nResource\n|> Ash.Query.filter(id == 1)\n|> Ash.Query.limit(2)\n|> Ash.Query.for_read(:read_action, %{})\n|> Ash.read!()\n|> case do\n  [] -> # raise not found error\n  [result] -> result\n  [_, _] -> # raise too many results error\nend\n```","ref":"read-actions.html#ash-get"},{"type":"extras","title":"Ash.read_one! - Read Actions","doc":"The `Ash.read_one!` function is a similar convenience function to `Ash.get!`, but it does not take a unique identifier. It is useful when you expect an action to return only a single result, and want to enforce that and return a single result.\n\n```elixir\nAsh.read_one!(query)\n\n# is roughly equivalent to\n\nquery\n|> Ash.Query.limit(2)\n|> Ash.read!()\n|> case do\n  [] -> nil\n  [result] -> result\n  [_, _] -> # raise too many results error\nend\n```","ref":"read-actions.html#ash-read_one"},{"type":"extras","title":"Pagination - Read Actions","doc":"Ash provides built-in support for pagination when reading resources and their relationships. You can find more information about this in the [pagination guide](/documentation/topics/advanced/pagination.livemd).\n\n>","ref":"read-actions.html#pagination"},{"type":"extras","title":"Pagination configuration on default vs custom read actions - Read Actions","doc":">\n> The default read action supports keyset pagination automatically. You need to explicitly enable pagination strategies you want to support when defining your own read actions.","ref":"read-actions.html#pagination-configuration-on-default-vs-custom-read-actions"},{"type":"extras","title":"What happens when you call Ash.Query.for_read/4 - Read Actions","doc":"The following steps are performed when you call `Ash.Query.for_read/4`.\n\n- Cast input arguments - `d:Ash.Resource.Dsl.actions.read.argument`\n- Set default argument values - `d:Ash.Resource.Dsl.actions.read.argument|default`\n- Add errors for missing required arguments | `d:Ash.Resource.Dsl.actions.read.argument|allow_nil?`\n- Run query preparations | `d:Ash.Resource.Dsl.actions.read.prepare`\n- Add action filter | `d:Ash.Resource.Dsl.actions.read|filter`","ref":"read-actions.html#what-happens-when-you-call-ash-query-for_read-4"},{"type":"extras","title":"What happens when you run the action - Read Actions","doc":"These steps are trimmed down, and are aimed at helping users understand the general flow. Some steps are omitted.\n\n- Run `Ash.Query.for_read/3` if it has not already been run\n- [Apply tenant filters for attribute](/documentation/topics/advanced/multitenancy.md)\n- Apply [pagination](/documentation/topics/advanced/pagination.livemd) options\n- Run before action hooks\n- Multi-datalayer filter is synthesized. We run queries in other data layers to fetch ids and translate related filters to `(destination_field in ^ids)`\n- Strict Check & Filter Authorization is run\n- Data layer query is built and validated\n- Field policies are added to the query\n- Data layer query is Run\n- Authorizer \"runtime\" checks are run (you likely do not have any of these)\n\nThe following steps happen while(asynchronously) or after the main data layer query has been run\n\n- If paginating and count was requested, the count is determined at the same time as the query is run.\n- Any calculations & aggregates that were able to be run outside of the main query are run\n- Relationships, calculations, and aggregates are loaded","ref":"read-actions.html#what-happens-when-you-run-the-action"},{"type":"extras","title":"Create Actions","doc":"# Create Actions\n\nCreate actions are used to create new records in the data layer. For example:\n\n```elixir\n# on a ticket resource\ncreate :open do\n  accept [:title]\n  change set_attribute(:status, :open)\nend\n```\n\nHere we have a create action called `:open` that allows setting the `title`, and sets the `status` to `:open`. It could be called like so:\n\n```elixir\nTicket\n|> Ash.Changeset.for_create(:open, %{title: \"Need help!\"})\n|> Ash.create!()\n```\n\nFor a full list of all of the available options for configuring create actions, see [the Ash.Resource.Dsl documentation](dsl-ash-resource.html#actions-create).\n\nSee the [Code Interface guide](documentation/topics/resources/code-interfaces.md) for creating an interface to call the action more elegantly, like so:\n\n```elixir\nSupport.open_ticket!(\"Need help!\")\n```","ref":"create-actions.html"},{"type":"extras","title":"Bulk creates - Create Actions","doc":"Bulk creates take a list or stream of inputs for a given action, and batches calls to the underlying data layer.\n\nGiven our example above, you could call `Ash.bulk_create` like so:\n\n```elixir\nAsh.bulk_create([%{title: \"Foo\"}, %{title: \"Bar\"}], Ticket, :open)\n```\n\n>","ref":"create-actions.html#bulk-creates"},{"type":"extras","title":"Check the docs! - Create Actions","doc":">\n> Make sure to thoroughly read and understand the documentation in `Ash.bulk_create/4` before using. Read each option and note the default values. By default, bulk creates don't return records or errors, and don't emit notifications.","ref":"create-actions.html#check-the-docs"},{"type":"extras","title":"Performance - Create Actions","doc":"Generally speaking, all regular Ash create actions are compatible (or can be made to be compatible) with bulk create actions. However, there are some important considerations.\n\n- `Ash.Resource.Change` modules can be optimized for bulk actions by implementing `batch_change/3`, `before_batch/3` and `after_batch/3`. If you implement `batch_change/3`, the `change` function will no longer be called, and you should swap any behavior implemented with `before_action` and `after_action` hooks to logic in the `before_batch` and `after_batch` callbacks.\n\n- Actions that reference arguments in changes, i.e `change set_attribute(:attr, ^arg(:arg))` will prevent us from using the `batch_change/3` behavior. This is usually not a problem, for instance that change is lightweight and would not benefit from being optimized with `batch_change/3`\n\n- If your action uses `after_action` hooks, or has `after_batch/3` logic defined for any of its changes, then we _must_ ask the data layer to return the records it inserted. Again, this is not generally a problem because we throw away the results of each batch by default. If you are using `return_records?: true` then you are already requesting all of the results anyway.","ref":"create-actions.html#performance"},{"type":"extras","title":"Returning a Stream - Create Actions","doc":"Returning a stream allows you to work with a bulk action as an Elixir Stream. For example:\n\n```elixir\ninput_stream()\n|> Ash.bulk_create(Resource, :action, return_stream?: true, return_records?: true)\n|> Stream.map(fn {:ok, result} ->\n  # process results\n  {:error, error} ->\n  # process errors\nend)\n|> Enum.reduce(%{}, fn {:ok, result}, acc ->\n   # process results\n   {:error, error} ->\n   # process errors\nend)\n```\n\n>","ref":"create-actions.html#returning-a-stream"},{"type":"extras","title":"Be careful with streams - Create Actions","doc":">\n> Because streams are lazily evaluated, if you were to do something like this:\n>\n> ```elixir\n> [input1, input2, ...] # has 300 things in it\n> |> Ash.bulk_create(\n>   Resource,\n>   :action,\n>   return_stream?: true,\n>   return_records?: true,\n>   batch_size: 100 #  default is 100\n> )\n> |> Enum.take(150) # stream has 300, but we only take 150\n> ```\n>\n> What would happen is that we would insert 200 records. The stream would end after we process the first two batches of 100. Be sure you aren't using things like `Stream.take` or `Enum.take` to limit the amount of things pulled from the stream, unless you actually want to limit the number of records created.","ref":"create-actions.html#be-careful-with-streams"},{"type":"extras","title":"Upserts - Create Actions","doc":"Upserting is the process of \"creating or updating\" a record, modeled with a single simple create. Both bulk creates and regular creates support upserts. Upserts can be declared in the action, like so:\n\n```elixir\ncreate :create_user do\n  accept [:email]\n  upsert? true\n  upsert_identity :unique_email\nend\n```\n\nOr they can be done with options when calling the create action.\n\n```elixir\nAsh.create!(changeset, upsert?: true, upsert_identity: :unique_email)\n```\n\n>","ref":"create-actions.html#upserts"},{"type":"extras","title":"Upserts do not use an update action - Create Actions","doc":">\n> While an upsert is conceptually a \"create or update\" operation, it does not result in an update action being called.\n> The data layer contains the upsert implementation. This means that if you have things like global changes that are only run on update,\n> they will not be run on upserts that result in an update. Additionally, notifications for updates will not be emitted from upserts.\n> Most importantly, there are no _read_ or _update_ policies applied! You must take care that an upsert can only target records that\n> the user has permission to update.\n\n#### Targeting Upserts\n\nLets imagine that you want a user to upsert an article by its slug, but only if it is their article:\n\nIf your action looked like this:\n\n```elixir\ncreate :upsert_article_by_slug do\n  upsert? true\n  accept [:slug, :title, :body]\n  upsert_identity :unique_slug\nend\n```\n\nAnd one way it could be called is like so:\n\n```elixir\nArticle\n|> Ash.Changeset.for_create(\n  :upsert_article_by_slug, \n  %{slug: \"foo\", title: \"new title\", slug: \"new slug\"},\n  actor: current_user\n)\n|> Ash.create!()\n```\n\nThis would create an article, unless there is an article with a matching slug in which case it would\nupdate the title and the body to match the provided input. Let's add the \"only if it is their article\"\nfunctionality.\n\nFor this we use a `filter` change to further scope the upsert:\n\n```elixir\ncreate :upsert_article_by_slug do\n  upsert? true\n  accept [:slug, :title, :body]\n  upsert_identity :unique_slug\n  upsert_condition expr(user_id == ^actor(:id))\nend\n```\n\n>","ref":"create-actions.html#upserts-do-not-use-an-update-action"},{"type":"extras","title":"What is ^actor(:id) ? - Create Actions","doc":">\n> Many places in Ash that support expression support *templates*. These are ways to refer\n> to certain things that are commonly available, like the actor, or action argument values.\n>\n> For more information, see [the expressions guide](/documentation/topics/reference/expressions.md#templates)\n\nNow, when we perform this upsert, there are three possible outcomes:\n\n- There is no article with that `slug`, in which case the article is created\n- There is an article with that `slug`, and the `user_id` matches the provided actor's `id`, so\n  it is updated with the new title and body.\n- There is an article with that `slug`, and the `user_id`  does not match the provided actor's,\n  `id`, in which case the action results in a `Ash.Error.Changes.StaleRecord` error. This is\n  the same error that would occur if the actor attempted to update something that had changed\n  in some unexpected way in the database.\n\n\n>","ref":"create-actions.html#what-is-actor-id"},{"type":"extras","title":"Improving the stale record error - Create Actions","doc":"> You may wish to transform this into an error message that can be displayed to the user, using\n> the `d:actions.create.error_handler` option. For example:\n>\n> ```elixir\n> create :upsert_article_by_slug do\n>   upsert? true\n>   accept [:slug, :title, :body]\n>   upsert_identity :unique_slug\n>   upsert_condition expr(user_id == ^actor(:id))\n>   error_handler fn \n>     _changeset, %Ash.Error.Changes.StaleRecord{} ->\n>       Ash.Error.Changes.InvalidChanges.exception(field: :slug, message: \"has already been taken\")\"\n> \n>     _ changeset, other ->\n>       # leave other errors untouched\n>       other\n>   end\n> end\n> ```","ref":"create-actions.html#improving-the-stale-record-error"},{"type":"extras","title":"Atomic Updates - Create Actions","doc":"Upserts support atomic updates. These atomic updates _do not apply to the data being created_. They are only applied in the case of an update. For example:\n\n```elixir\ncreate :create_game do\n  accept [:identifier]\n  upsert? true\n  upsert_identity :identifier\n  change set_attribute(:score, 0)\n  change atomic_update(:score, expr(score + 1))\nend\n```\n\nThis will result in creating a game with a score of 0, and if the game already exists, it will increment the score by 1.\n\nFor information on options configured in the action, see `d:Ash.Resource.Dsl.actions.create`.\nFor information on options when calling the action, see `Ash.create/2`.","ref":"create-actions.html#atomic-updates"},{"type":"extras","title":"What happens when you run a create Action - Create Actions","doc":"All actions are run in a transaction if the data layer supports it. You can opt out of this behavior by supplying `transaction?: false` when creating the action. When an action is being run in a transaction, all steps inside of it are serialized because transactions cannot be split across processes.\n\n- Authorization is performed on the changes\n- A before action hook is added to set up belongs_to relationships that are managed. This means potentially creating/modifying the destination of the relationship, and then changing the `destination_attribute` of the relationship.\n- `before_transaction` and `around_transaction` hooks are called (`Ash.Changeset.before_transaction/2`). Keep in mind, any validations that are marked as `before_action? true` (or all global validations if your action has `delay_global_validations? true`) will not have happened at this point.\n- A transaction is opened if the action is configured for it (by default they are) and the data layer supports transactions\n- `before_action` hooks are performed in order\n- The main action is sent to the data layer\n- `after_action` hooks are performed in order\n- Non-belongs-to relationships are managed, creating/updating/destroying related records.\n- The transaction is closed, if one was opened\n- `after_transaction` hooks are invoked with the result of the transaction (even if it was an error)","ref":"create-actions.html#what-happens-when-you-run-a-create-action"},{"type":"extras","title":"Update Actions","doc":"# Update Actions\n\nUpdate actions are used to update records in the data layer. For example:\n\n```elixir\n# on a ticket resource\nupdate :close do\n  accept [:close_reason]\n  change set_attribute(:status, :closed)\nend\n```\n\nHere we have an update action called `:close` that allows setting the `close_reason`, and sets the `status` to `:closed`. It could be called like so:\n\n```elixir\nticket # providing an initial ticket to close\n|> Ash.Changeset.for_update(:close, %{close_reason: \"I figured it out.\"})\n|> Ash.update!()\n```\n\nFor a full list of all of the available options for configuring update actions, see [the Ash.Resource.Dsl documentation](dsl-ash-resource.html#actions-update).\n\nSee the [Code Interface guide](documentation/topics/resources/code-interfaces.md) for creating an interface to call the action more elegantly, like so:\n\n```elixir\nSupport.close_ticket!(ticket, \"I figured it out.\")\n# You can also provide an id\nSupport.close_ticket!(ticket.id, \"I figured it out.\")\n```","ref":"update-actions.html"},{"type":"extras","title":"Atomics - Update Actions","doc":"Atomic updates can be added to a changeset, which will update the value of an attribute given by an expression. Atomics can be a very powerful way to model updating data in a simple way. An action does not have to be [fully atomic](#fully-atomic-updates) in order to leverage atomic updates. For example:\n\n```elixir\nupdate :add_to_name do\n  argument :to_add, :string, allow_nil? false\n  change atomic_update(:name, expr(\"#{name}_#{to_add}\"))\nend\n```\n\nChanging attributes in this way makes them safer to use in concurrent environments, and is typically more performant than doing it manually in memory.\n\n>","ref":"update-actions.html#atomics"},{"type":"extras","title":"Atomics are not stored with other changes - Update Actions","doc":">\n> While we recommend using atomics wherever possible, it is important to note that they are stored in their own map in the changeset,\n> i.e `changeset.atomics`, meaning if you need to do something later in the action with the new value for an attribute, you won't be\n> able to access the new value. This is because atomics are evaluated in the data layer. You can, however, access \"the old or new value\"\n> in a similar way to `Ash.Changeset.get_attribute`, using the template expression, `atomic_ref(:name)`. See the section below for more.\n\n### `atomic_ref/1`\n\nLets say that you have an action that may perform multiple atomic update on a single column, or for some other reason needs to refer to the new value.\nThe only way to access that new value is _also_ in an atomic update, change, or validation, using `atomic_ref/1`. There is no way to access the new value\nprior to the action being run with something like `Ash.Changeset.get_attribute/2`.\n\nFor example, lets say you have a postgres function that will slugify a string, and you want to make sure to always set it to the slugified version of `name`,\nwhenever `name` is changing.\n\n```elixir\nchanges do\n  change atomic_update(:slug, expr(fragment(\"slugify(?)\", ^atomic_ref(:name)))), where: changing(:name), on: [:update]\nend\n```\n\nBy using `atomic_ref/1` here, you are always referring to the new value of `name`, even if another atomic update has been made that modifies `name`.\n\nBecause the validation `changing/1` can be done atomically, and the change `atomic_update/2` (naturally) can be done atomically, this is a fully atomic update.\nLets say that you paired this with an action like this:\n\n```elixir\nupdate :add_to_name do\n  argument :to_add, :string, allow_nil? false\n  change atomic_update(:name, expr(\"#{name}_#{to_add}\"))\nend\n```\n\nand would produce a SQL update along the lines of:\n\n```sql\nUPDATE table\n  SET name = name || $1,\n  slug = CASE\n    WHEN name = name || $1 THEN\n      slug\n    ELSE\n      slugify(name || $1)\n  END\nWHERE id = $2\n```\n\nThis is a _fully atomic update_, because all changes are done atomically in the data layer. We now have the benefits of composable building blocks _and_\natomic updates.","ref":"update-actions.html#atomics-are-not-stored-with-other-changes"},{"type":"extras","title":"Fully Atomic updates - Update Actions","doc":"Atomic updates are a special case of update actions that can be done completely atomically. If your update action can't be done atomically, you will get an error unless you have set `require_atomic? false`. This is to encourage you to opt for atomic updates wherever reasonable. Not all actions can reasonably be made atomic, and not all non-atomic actions are problematic for concurrency. The goal is only to make sure that you are aware and have considered the implications.\n\n>","ref":"update-actions.html#fully-atomic-updates"},{"type":"extras","title":"What does atomic mean? - Update Actions","doc":">\n> An atomic update is one that can be done in a single operation in the data layer. This ensures that there are no issues with concurrent access to the record being updated, and that it is as performant as possible.\n> For example, the following action cannot be done atomically, because it has\n> an anonymous function change on it.\n>\n> ```elixir\n> update :increment_score do\n>   change fn changeset, _ ->\n>     Ash.Changeset.change_attribute(changeset, :score, changeset.data.score + 1)\n>   end\n> end\n> ```\n>\n> The action shown above is not safe to run concurrently. If two separate processes fetch the record with score `1`, and then call `increment_score`, they will both set the score to `2`, when what you almost certainly intended to do was end up with a score of `3`\n>\n> By contrast, the following action _can_ be done atomically\n>\n> ```elixir\n> update :increment_score do\n>   change atomic_update(:score, expr(score + 1))\n> end\n> ```\n>\n> In a SQL data layer, this would produce SQL along the lines of\n>\n> ```elixir\n> \"UPDATE table SET score = score + 1 WHERE id = post_id\"\n> ```","ref":"update-actions.html#what-does-atomic-mean"},{"type":"extras","title":"What makes an action not atomic? - Update Actions","doc":"","ref":"update-actions.html#what-makes-an-action-not-atomic"},{"type":"extras","title":"Types that can't be atomically casted - Update Actions","doc":"Not all types support being casted atomically. For instance, `:union` types, and embedded resources that have primary keys(and therefore may need to use an update action) cannot currently be casted atomically.","ref":"update-actions.html#types-that-can-t-be-atomically-casted"},{"type":"extras","title":"Changes without an atomic callback - Update Actions","doc":"Changes can be enhanced to support atomics by defining `c:Ash.Resource.Change.atomic/3`. This callback can return a map of atomic updates to be made to attributes. Here is a simplified example from the built in `Ash.Resource.Change.Builtins.increment/2` change:\n\n```elixir\n@impl true\ndef atomic(_changeset, opts, _context) do\n  # Set the requested attribute to its current value (atomic_ref) + the amount\n  {:atomic, %{opts[:attribute] => expr(^atomic_ref(opts[:attribute]) + ^opts[:amount])}}\nend\n```","ref":"update-actions.html#changes-without-an-atomic-callback"},{"type":"extras","title":"Validations without an atomic callback - Update Actions","doc":"Validations can be enhanced to support atomics by defining `c:Ash.Resource.Validation.atomic/3`. This callback can return an atomic validation (or a list of atomic validations), which is represented by a list of affected attributes (not currently used), an expression that should trigger an error, and the expression producing the error. Here is an example from the built in `Ash.Resource.Validations.Builtins.attribute_equals/2` validation:\n\n```elixir\n@impl true\ndef atomic(_changeset, opts, context) do\n  {:atomic, [opts[:attribute]], expr(^atomic_ref(opts[:attribute]) != ^opts[:value]),\n   expr(\n     error(^InvalidAttribute, %{\n       field: ^opts[:attribute],\n       value: ^atomic_ref(opts[:attribute]),\n       message: ^(context.message || \"must equal %{value}\"),\n       vars: %{field: ^opts[:attribute], value: ^opts[:value]}\n     })\n   )}\nend\n```","ref":"update-actions.html#validations-without-an-atomic-callback"},{"type":"extras","title":"Bulk updates - Update Actions","doc":"There are three strategies for bulk updating data. They are, in order of preference: `:atomic`, `:atomic_batches`, and `:stream`. When calling `Ash.bulk_update/4`, you can provide a strategy or strategies that can be used, and Ash will choose the best one available. The implementation of the update action and the capabilities of the data layer determine what strategies can be used.","ref":"update-actions.html#bulk-updates"},{"type":"extras","title":"Atomic - Update Actions","doc":"Atomic bulk updates are used when the subject of the bulk update is a query, and the update action [can be done atomically](#fully-atomic-updates) and the data layer supports updating a query. They map to a single statement to the data layer to update all matching records. The data layer must support updating a query.","ref":"update-actions.html#atomic"},{"type":"extras","title":"Example - Update Actions","doc":"```elixir\nTicket\n|> Ash.Query.filter(status == :open)\n|> Ash.bulk_update!(:close, %{reason: \"Closing all open tickets.\"})\n```\n\nIf using a SQL data layer, this would produce a query along the lines of\n\n```sql\nUPDATE tickets\nSET status = 'closed',\n    reason = 'Closing all open tickets.'\nWHERE status = 'open';\n```","ref":"update-actions.html#example"},{"type":"extras","title":"Atomic Batches - Update Actions","doc":"Atomic batches is used when the subject of the bulk update is an enumerable (i.e list or stream) of records and the update action [can be done atomically](#fully-atomic-updates) and the data layer supports updating a query. The records are pulled out in batches, and then each batch follows the logic described [above](#atomic). The batch size is controllable by the `batch_size` option.","ref":"update-actions.html#atomic-batches"},{"type":"extras","title":"Example - Update Actions","doc":"```elixir\n\nAsh.bulk_update!(one_hundred_tickets, :close, %{reason: \"Closing all open tickets.\"}, batch_size: 10)\n```\n\nIf using a SQL data layer, this would produce ten queries along the lines of\n\n```sql\nUPDATE tickets\nSET status = 'closed',\n    reason = 'Closing all open tickets.'\nWHERE id IN (...ids)\n```","ref":"update-actions.html#example-1"},{"type":"extras","title":"Stream - Update Actions","doc":"Stream is used when the update action [cannot be done atomically](#fully-atomic-updates) or if the data layer does not support updating a query. If a query is given, it is run and the records are used as an enumerable of inputs. If an enumerable of inputs is given, each one is updated individually. There is nothing inherently wrong with doing this kind of update, but it will naturally be slower than the other two strategies.\nThe benefit of having a single interface (`Ash.bulk_update/4`) is that the caller doesn't need to change based on the performance implications of the action.","ref":"update-actions.html#stream"},{"type":"extras","title":"Running a standard update action - Update Actions","doc":"All actions are run in a transaction if the data layer supports it. You can opt out of this behavior by supplying `transaction?: false` when creating the action. When an action is being run in a transaction, all steps inside of it are serialized because transactions cannot be split across processes.\n\n- Authorization is performed on the changes\n- A before action hook is added to set up belongs_to relationships that are managed. This means potentially creating/modifying the destination of the relationship, and then changing the `destination_attribute` of the relationship.\n- `before_transaction` and `around_transaction` hooks are called (`Ash.Changeset.before_transaction/2`). Keep in mind, any validations that are marked as `before_action? true` (or all global validations if your action has `delay_global_validations? true`) will not have happened at this point.\n- A transaction is opened if the action is configured for it (by default they are) and the data layer supports transactions\n- `before_action` hooks are performed in order\n- The main action is sent to the data layer\n- `after_action` hooks are performed in order\n- Non-belongs-to relationships are managed, creating/updating/destroying related records.\n- The transaction is closed, if one was opened\n- `after_transaction` hooks are invoked with the result of the transaction (even if it was an error)","ref":"update-actions.html#running-a-standard-update-action"},{"type":"extras","title":"Atomic Upgrade - Update Actions","doc":"Update actions that are run as \"normal\" update actions will, at the time of execution, be \"upgraded\" to an atomic action if possible. This means taking the original inputs and building a corresponding atomic action. This behavior is primarily useful for using things like `AshPhoenix.Form`, where you want to validate and see the effects of an action before running it, but want the ultimate invocation to be atomic (i.e concurrency safe).\n\nYou can disable this by adding `atomic_upgrade? false` to the action configuration. Additionally, you may want to configure the read action used for atomic upgrades (defaults to the primary read), with `atomic_upgrade_with` option, i.e `atomic_upgrade_with :list_all`","ref":"update-actions.html#atomic-upgrade"},{"type":"extras","title":"Destroy Actions","doc":"# Destroy Actions\n\nDestroy actions are comparatively simple. They expect to remove a given record, and by default return `:ok` in the successful case.\n\nMost destroy actions are one-liners, for example:\n\n```elixir\ndestroy :destroy\n# Can be added with the defaults\ndefaults [:read, :destroy]\n```","ref":"destroy-actions.html"},{"type":"extras","title":"Soft Destroy - Destroy Actions","doc":"You can mark a destroy action as `soft? true`, in which case it is handled by the [`update` action](/documentation/topics/actions/update-actions.md) logic.\n\nFor example:\n\n```elixir\ndestroy :archive do\n  soft? true\n  change set_attribute(:archived_at, &DateTime.utc_now/0)\nend\n```\n\nFor a full list of all of the available options for configuring destroy actions, see [the Ash.Resource.Dsl documentation](dsl-ash-resource.html#actions-destroy).","ref":"destroy-actions.html#soft-destroy"},{"type":"extras","title":"Calling Destroy Actions - Destroy Actions","doc":"The basic formula for calling a destroy action looks like this:\n\n```elixir\nrecord\n|> Ash.Changeset.for_destroy(:action_name, %{argument: :value}, ...opts)\n|> Ash.destroy!()\n```\n\nSee below for variations on action calling, and see the [code interface guide](/documentation/topics/code-interfaces.md) guide for how to\ndefine idiomatic and convenient functions that call your actions.","ref":"destroy-actions.html#calling-destroy-actions"},{"type":"extras","title":"Returning the destroyed record - Destroy Actions","doc":"You can use the `return_destroyed?` option to return the destroyed record.\n\n```elixir\n# when a resource is passed, or a query w/ no action, the primary destroy action is used.\nticket = Ash.get!(Ticket, 1)\nAsh.destroy!(ticket)\n# => :ok\nticket = Ash.get!(Ticket, 2)\nAsh.destroy!(ticket, return_destroyed?: true)\n# => {:ok, %Ticket{}}\n```\n\n>","ref":"destroy-actions.html#returning-the-destroyed-record"},{"type":"extras","title":"Loading on destroyed records - Destroy Actions","doc":">\n> Keep in mind that using `Ash.load` on destroyed data will produced mixed results. Relationships may appear as empty, or may be loaded as expected (depending on the data layer/relationship implementation) and calculations/aggregates may show as `nil` if they must be run in the data layer.","ref":"destroy-actions.html#loading-on-destroyed-records"},{"type":"extras","title":"Bulk Destroys - Destroy Actions","doc":"There are three strategies for bulk destroying data. They are, in order of preference: `:atomic`, `:atomic_batches`, and `:stream`. When calling `Ash.bulk_destroy/4`, you can provide a strategy or strategies that can be used, and Ash will choose the best one available. The capabilities of the data layer determine what strategies can be used.","ref":"destroy-actions.html#bulk-destroys"},{"type":"extras","title":"Atomic - Destroy Actions","doc":"Atomic bulk destroys are used when the subject of the bulk destroy is a query and the data layer supports destroying a query. They map to a single statement to the data layer to destroy all matching records.","ref":"destroy-actions.html#atomic"},{"type":"extras","title":"Example - Destroy Actions","doc":"```elixir\nTicket\n|> Ash.Query.filter(status == :open)\n|> Ash.bulk_destroy!(:close, %{})\n```\n\nIf using a SQL data layer, this would produce a query along the lines of\n\n```sql\nDELETE FROM tickets\nWHERE status = 'open';\n```","ref":"destroy-actions.html#example"},{"type":"extras","title":"Atomic Batches - Destroy Actions","doc":"Atomic batches are used when the subject of the bulk destroy is an enumerable (i.e list or stream) of records and the data layer supports destroying a query. The records are pulled out in batches, and then each batch follows the logic described [above](#atomic). The batch size is controllable by the `batch_size` option.","ref":"destroy-actions.html#atomic-batches"},{"type":"extras","title":"Example - Destroy Actions","doc":"```elixir\n\nAsh.bulk_destroy!(one_hundred_tickets, :close, %{}, batch_size: 10)\n```\n\nIf using a SQL data layer, this would produce ten queries along the lines of\n\n```sql\nDELETE FROM tickets\nWHERE id IN (...ids)\n```","ref":"destroy-actions.html#example-1"},{"type":"extras","title":"Stream - Destroy Actions","doc":"Stream is used when the data layer does not support destroying a query. If a query is given, it is run and the records are used as an enumerable of inputs. If an enumerable of inputs is given, each one is destroyed individually. There is nothing inherently wrong with doing this kind of destroy, but it will naturally be slower than the other two strategies.\nThe benefit of having a single interface (`Ash.bulk_destroy/4`) is that the caller doesn't need to change based on the performance implications of the action.\n\n>","ref":"destroy-actions.html#stream"},{"type":"extras","title":"Check the docs! - Destroy Actions","doc":">\n> Make sure to thoroughly read and understand the documentation in `Ash.bulk_destroy/4` before using. Read each option and note the default values. By default, bulk destroys don't return records or errors, and don't emit notifications.","ref":"destroy-actions.html#check-the-docs"},{"type":"extras","title":"Destroying records - Destroy Actions","doc":"If you provide an enumerable of records, they will be destroyed in batches. For example:\n\n```elixir\nAsh.bulk_destroy([%Ticket{}, %Ticket{}], :destroy, %{})\n```","ref":"destroy-actions.html#destroying-records"},{"type":"extras","title":"Destroying - Destroy Actions","doc":"","ref":"destroy-actions.html#destroying"},{"type":"extras","title":"Running the Destroy Action - Destroy Actions","doc":"All actions are run in a transaction if the data layer supports it. You can opt out of this behavior by supplying `transaction?: false` when creating the action. When an action is being run in a transaction, all steps inside of it are serialized because transactions cannot be split across processes.\n\n- Authorization is performed on the changes\n- A before action hook is added to set up belongs_to relationships that are managed. This means potentially creating/modifying the destination of the relationship, and then changing the `destination_attribute` of the relationship.\n- `before_transaction` and `around_transaction` hooks are called (`Ash.Changeset.before_transaction/2`). Keep in mind, any validations that are marked as `before_action? true` (or all global validations if your action has `delay_global_validations? true`) will not have happened at this point.\n- A transaction is opened if the action is configured for it (by default they are) and the data layer supports transactions\n- `before_action` hooks are performed in order\n- The main action is sent to the data layer\n- `after_action` hooks are performed in order\n- Non-belongs-to relationships are managed, creating/updating/destroying related records.\n- The transaction is closed, if one was opened\n- `after_transaction` hooks are invoked with the result of the transaction (even if it was an error)","ref":"destroy-actions.html#running-the-destroy-action"},{"type":"extras","title":"Generic Actions","doc":"# Generic Actions\n\nGeneric actions are so named because there are no special rules about how they work. A generic action takes arguments and returns a value. The struct used for building input for a generic action is `Ash.ActionInput`.\n\n```elixir\naction :say_hello, :string do\n  argument :name, :string, allow_nil?: false\n\n  run fn input, _ ->\n    {:ok, \"Hello: #{input.arguments.name}\"}\n  end\nend\n```\n\nA generic action declares its arguments, return type, and implementation, as illustrated above.\n\n>","ref":"generic-actions.html"},{"type":"extras","title":"No return? No problem! - Generic Actions","doc":">\n> Generic actions can omit a return type, in which case running them returns `:ok` if successful.\n>\n> ```elixir\n> action :schedule_job do\n>   argument :job_name, :string, allow_nil?: false\n>   run fn input, _ ->\n>     # Schedule the job\n>     :ok\n>   end\n> end\n> ```\n\nFor a full list of all of the available options for configuring generic actions, see [the Ash.Resource.Dsl documentation](dsl-ash-resource.html#actions-action).","ref":"generic-actions.html#no-return-no-problem"},{"type":"extras","title":"Calling Generic Actions - Generic Actions","doc":"The basic formula for calling a generic action looks like this:\n\n```elixir\nResource\n|> Ash.ActionInput.for_action(:action_name, %{argument: :value}, ...opts)\n|> Ash.run_action!()\n```\n\nSee the [code interface guide](/documentation/topics/code-interfaces.md) guide for how to\ndefine idiomatic and convenient functions that call your actions.","ref":"generic-actions.html#calling-generic-actions"},{"type":"extras","title":"Why use generic actions? - Generic Actions","doc":"The example above could be written as a normal function in elixir, i.e\n\n```elixir\ndef say_hello(name), do: \"Hello: #{name}\"\n```\n\nThe benefit of using generic actions instead of defining normal functions:\n\n- They can be used with api extensions like `ash_json_api` and `ash_graphql`\n- Their inputs are type checked and casted\n- They support Ash authorization patterns (i.e policies)\n- They can be included in the code interface of a resource\n- They can be made transactional with a single option (`transaction? true`)\n\nIf you don't need any of the above, then there is no problem with writing regular Elixir functions!","ref":"generic-actions.html#why-use-generic-actions"},{"type":"extras","title":"Return types and constraints - Generic Actions","doc":"Generic actions do not cast their return types. It is expected that the action return a valid value for the type that they declare. However, declaring additional constraints can inform API usage, and make the action more clear. For example:\n\n```elixir\naction :priority, :integer do\n  constraints [min: 1, max: 3]\n  argument :status, :atom, constraints: [one_of: [:high, :medium, :low]]\n\n  run fn input, _ ->\n    case input.arguments.status do\n      :high -> {:ok, 3}\n      :medium -> {:ok, 2}\n      :low -> {:ok, 1}\n    end\n  end\nend\n```\n\n> #### Returning resource instances {: .info}\n>\n> It sometimes happens that you want to make a generic action which returns an\n> instance or instances of the resource. It's natural to assume that you can\n> set your action's return type to the name of your resource. This won't work\n> as resources do not define a type, unless they are embedded. In embedded resources, this won't work because the module is still being compiled, so referencing yourself as a type causes a compile error. Instead, use the `:struct` type and the `instance_of` constraint, like so:\n>\n> ```elixir\n> action :get, :struct do\n>   constraints instance_of: __MODULE__\n>\n>   run # ...\n> end\n> ```\n>\n> For returning many instances of the resource, you can set your action's return type to\n> `{:array, :struct}` and set the `items` constraint to the name of your resource.\n>\n> ```elixir\n>  action :list_resources, {:array, :struct} do\n>    constraints items: [instance_of: __MODULE__]\n>\n>    run # ...\n>  end\n> ```","ref":"generic-actions.html#return-types-and-constraints"},{"type":"extras","title":"Calling Generic Actions - Generic Actions","doc":"To execute a generic action in Ash, follow these steps:\n\n1. **Prepare the action input:** Use `Ash.ActionInput.for_action/4` to specify the resource, the action and its arguments.\n2. **Run the action:** Use `Ash.run_action/2` to execute the action with the prepared input.","ref":"generic-actions.html#calling-generic-actions-1"},{"type":"extras","title":"Example Usage - Generic Actions","doc":"Consider an `Ash.Resource` with the action `:say_hello`:\n\n```elixir\naction :say_hello, :string do\n  argument :name, :string, allow_nil?: false\n\n  run fn input, _ ->\n    {:ok, \"Hello: #{input.arguments.name}\"}\n  end\nend\n```\n\nCall this action:\n\n```elixir\n{:ok, greeting} = Resource\n|> Ash.ActionInput.for_action(:say_hello, %{name: \"Alice\"})\n|> Ash.run_action()\n\nIO.puts(greeting)  # Output: Hello: Alice\n```","ref":"generic-actions.html#example-usage"},{"type":"extras","title":"Using Code Interface - Generic Actions","doc":"You can also use [Code Interfaces](documentation/topics/resources/code-interfaces.md) to call actions:\n\nGiven a definition like:\n\n```elixir\ndefine :say_hello, args: [:name]\n```\n\n```elixir\n{:ok, greeting} = Resource.say_hello(\"Alice\")\ngreeting = Resource.say_hello!(\"Alice\")\n```","ref":"generic-actions.html#using-code-interface"},{"type":"extras","title":"Manual Actions","doc":"# Manual Actions\n\nManual actions allow you to control how an action is performed instead of dispatching to a data layer. To do this, specify the `manual` option with a module that adopts the appropriate behavior.\n\nManual actions are a way to implement an action in a fully custom way. This can be a very useful escape hatch when you have something that you are finding difficult to model with Ash's builtin tools.","ref":"manual-actions.html"},{"type":"extras","title":"Manual Creates/Updates/Destroy - Manual Actions","doc":"For manual create, update and destroy actions, a module is passed that uses one of the following (`Ash.Resource.ManualCreate`, `Ash.Resource.ManualUpdate` and `Ash.Resource.ManualDestroy`).\n\nFor example:\n\n```elixir\ncreate :special_create do\n  manual MyApp.DoCreate\nend\n\n# The implementation\ndefmodule MyApp.DoCreate do\n  use Ash.Resource.ManualCreate\n\n  def create(changeset, _, _) do\n    record = create_the_record(changeset)\n    {:ok, record}\n\n    # An `{:error, error}` tuple should be returned if something failed\n  end\nend\n```\n\nThe underlying record can be retrieved from `changeset.data` for update and destroy manual actions. The changeset given to the manual action will be after any `before_action` hooks, and before any `after_action` hooks.","ref":"manual-actions.html#manual-creates-updates-destroy"},{"type":"extras","title":"Manual Read Actions - Manual Actions","doc":"Manual read actions work the same, except the will also get the \"data layer query\". For AshPostgres, this means you get the ecto query that would have been run. You can use `Ash.Query.apply_to/3` to apply a query to records in memory. This allows you to fetch the data in a way that is not possible with the data layer, but still honor the query that was provided to.\n\n```elixir\n# in the resource\nactions do\n  read :action_name do\n    manual MyApp.ManualRead\n    # or `{MyApp.ManualRead, ...opts}`\n  end\nend\n\n# the implementation\ndefmodule MyApp.ManualRead do\n  use Ash.Resource.ManualRead\n\n  def read(ash_query, ecto_query, _opts, _context) do\n    ...\n    {:ok, query_results} | {:error, error}\n  end\nend\n```\n\nIn addition to returning query results, you can return a `t:Ash.Resource.ManualRead.extra_info()`.\nThis contains a `full_count` key, which can be used when paginating to set the total count of records.\n\n```elixir\ndefmodule MyApp.ManualRead do\n  use Ash.Resource.ManualRead\n\n  def read(ash_query, ecto_query, _opts, _context) do\n    %{\"data\" => data, \"count\" => count} = make_some_api_request(...)\n    if ash_query.page[:count] do\n      {:ok, query_results}\n    else\n      {:ok, query_results, %{full_count: count}} \n    end\n  end\nend\n\n```","ref":"manual-actions.html#manual-read-actions"},{"type":"extras","title":"Modifying the query - Manual Actions","doc":"As an alternative to manual read actions, you can also provide the `modify_query` option, which takes an `MFA` and allows low level manipulation of the query just before it is dispatched to the data layer.\n\nFor example:\n\n```elixir\nread :read do\n  modify_query {MyApp.ModifyQuery, :modify, []}\nend\n\ndefmodule MyApp.ModifyQuery do\n  def modify(ash_query, data_layer_query) do\n    {:ok, modify_data_layer_query(data_layer_query)}\n  end\nend\n```\n\nThis can be used as a last-resort escape hatch when you want to still use resource actions but need to do something that you can't do easily with Ash tools. As with any low level escape hatch, here be dragons.","ref":"manual-actions.html#modifying-the-query"},{"type":"extras","title":"Actors & Authorization","doc":"# Actors & Authorization\n\nAuthorization in Ash involves three things:\n\n* actor - the entity (i.e user, organization, device) performing a given action. See [the glossary](/documentation/topics/reference/glossary.md#actor) for more.\n* authorize? - a flag that tells Ash to run authorization.\n* authorizers - the extensions on a resource that can modify or forbid the action.","ref":"actors-and-authorization.html"},{"type":"extras","title":"Setting actor and authorize? - Actors & Authorization","doc":"All functions in Ash that may perform authorization and/or wish to use the actor accept an `actor` and an `authorize?` option. For example:\n\nBuilding a changeset/query/input is the best time to provide the actor option\n\n```elixir\nAsh.Changeset.for_create(Post, %{title: \"Post Title\"}, actor: current_user, authorize?: true)\n```\n\nIf calling a function without changeset/query/input, you can provide the `actor` option at that point.\n\n```elixir\nAsh.count!(Post, actor: current_user, authorize?: true)\n```\n\nFunctions created with the code interface also accept an `actor` option.\n\n```elixir\nMyDomain.create_post!(Post, authorize?: true)\n```\n\n>","ref":"actors-and-authorization.html#setting-actor-and-authorize"},{"type":"extras","title":"Set the actor on the query/changeset/input - Actors & Authorization","doc":"> The hooks on a query/changeset/input to an action may need to know the actor, so you\n> need to set the actor when building them, not when calling the action.\n> ```elixir\n> # DO THIS\n>\n> Post\n> |> Ash.Query.for_read(:read, %{}, actor: current_user)\n> |> Ash.read!()\n>\n> # DON'T DO THIS\n>\n> Post\n> |> Ash.Query.for_read!(:read)\n> |> Ash.read!(actor: current_user)\n> ```","ref":"actors-and-authorization.html#set-the-actor-on-the-query-changeset-input"},{"type":"extras","title":"Default value of authorize? - Actors & Authorization","doc":"The default value of `authorize?` is determined by the `authorization` configuration of the relevant domain. By default, `authorize?` is set to `true` (and so can be omitted in all of the examples above). If a resource has no authorizers, then all requests will be allowed.","ref":"actors-and-authorization.html#default-value-of-authorize"},{"type":"extras","title":"Authorizers - Actors & Authorization","doc":"Authorizers are in control of what happens during authorization. Generally, you won't need to create your own authorizer, as the builtin policy authorizer `Ash.Policy.Authorizer` works well for any use case. See the [Policies guide](documentation/topics/security/policies.md) for more.","ref":"actors-and-authorization.html#authorizers"},{"type":"extras","title":"Domain Authorization Configuration - Actors & Authorization","doc":"### `d:Ash.Domain.Dsl.authorization|require_actor?`\n\nRequires that an actor is set for all requests.\n\nImportant: `nil` is still a valid actor, so this won't prevent providing `actor: nil`. It only requires that the option itself is provided.\n\n### `d:Ash.Domain.Dsl.authorization|authorize`\n\nWhen to run authorization for a given request.\n\n- `:by_default` sets `authorize?: true` if the `authorize?` option was not set (so it can be set to `false`). This is the default.\n- `:always` forces `authorize?: true` on all requests to the domain.\n- `:when_requested` sets `authorize?: true` whenever an actor is set or `authorize?: true` is explicitly passed.","ref":"actors-and-authorization.html#domain-authorization-configuration"},{"type":"extras","title":"Sensitive Data","doc":"# Sensitive Data","ref":"sensitive-data.html"},{"type":"extras","title":"Public & Private Attributes - Sensitive Data","doc":"By default, attributes, calculations, aggregates and relationships are _private_ (they are marked `public?: false`).  \nIf you are working with Ash in code, reading a resource, for example using `Ash.read/2`, the public/private status of an attribute is not relevant.  \nHowever, when working with api extensions like `AshGraphql` and `AshJsonApi`, they will only include public fields in their interfaces. This helps avoid accidentally exposing data over \"public\" interfaces.","ref":"sensitive-data.html#public-private-attributes"},{"type":"extras","title":"Public & Private Arguments - Sensitive Data","doc":"Public/private arguments work the same way as public/private fields, except that they default to `public?: true`.  \nThis is because arguments to an action being used in a public interface would naturally be expected to be `public`. If an argument is marked as `public?: false`, it can only be set using one of the following methods:\n1. `Ash.Query.set_argument/3`\n2. `Ash.Changeset.set_argument/3`\n3. By setting `private_argument` option when creating a changeset. For eg., `Ash.Changeset.for_create(:create_with_private_argument, %{public_title: \"title\"}, private_arguments: %{private_name: \"private\"}`","ref":"sensitive-data.html#public-private-arguments"},{"type":"extras","title":"Sensitive Attributes - Sensitive Data","doc":"Using `sensitive? true` will cause an attribute, calculation or argument to show as `\"** Redacted **\"` when inspecting records.  \nIn filter statements, any value used in the same expression as a sensitive field will also be redacted. For example, you might see: `email == \"** Redacted **\"` in a filter statement if `email` is marked as sensitive.","ref":"sensitive-data.html#sensitive-attributes"},{"type":"extras","title":"Show Sensitive Attributes - Sensitive Data","doc":"**IMPORTANT WARNING:** The following configuration should only ever be used in development mode!\n\nTo display sensitive attributes in their original form during development, use the following config.\n\n```elixir\nconfig :ash, show_sensitive?: true\n```","ref":"sensitive-data.html#show-sensitive-attributes"},{"type":"extras","title":"Field Policies - Sensitive Data","doc":"Field policies are a way to control the visibility of individual fields (except for relationships) as a part of authorization flow, for those using `Ash.Policy.Authorizer`.  \nIf a field is not visible, it will be populated with `%Ash.ForbiddenField{}`, or will be not shown (or may show an error) in public interfaces. See the [Policies guide](documentation/topics/security/policies.md#field-policies) for more.","ref":"sensitive-data.html#field-policies"},{"type":"extras","title":"Policies","doc":"# Policies\n\nPolicies determine what actions on a resource are permitted for a given actor, and can also filter the results of read actions to restrict the results to only records that should be visible.\n\nTo restrict access to specific fields (attributes, aggregates, calculations), see the section on field policies.\n\nRead and understand the [Actors & Authorization guide](/documentation/topics/security/actors-and-authorization.md) before proceeding, which explains actors, how to set them, and other relevant configurations.","ref":"policies.html"},{"type":"extras","title":"Setup - Policies","doc":"You'll need to add the extension to your resource, like so:\n\n```elixir\nuse Ash.Resource, authorizers: [Ash.Policy.Authorizer]\n```\n\nThen you can start defining policies for your resource.","ref":"policies.html#setup"},{"type":"extras","title":"Policies - Policies","doc":"","ref":"policies.html#policies"},{"type":"extras","title":"Anatomy of a Policy - Policies","doc":"Each policy defined in a resource has two parts -\n\n- a condition or a list of conditions, such as `action_type(:read)`, `[action_type(:read), actor_attribute_equals(:admin, true)]` or `always()`. If the condition, or all conditions if given a list are true for a given action request, then the policy will be applied to the request.\n- a set of policy checks, each of which will be evaluated individually if a policy applies to a request.\n\nIf more than one policy applies to any given request (eg. an admin actor calls a read action) then **all applicable policies must pass** for the action to be performed.\n\nA policy will produce one of three results: `:forbidden`, `:authorized`, or `:unknown`. `:unknown` is treated the same as `:forbidden`.","ref":"policies.html#anatomy-of-a-policy"},{"type":"extras","title":"The Simplest Policy - Policies","doc":"Let's start with the simplest (most permissive) policy:\n\n```elixir\npolicies do\n  policy always() do\n    authorize_if always()\n  end\nend\n```\n\nThe first argument to `policy` is the condition. In this case, the condition is `always()` - a built-in helper always returning true, meaning that the policy applies to every request.\n\nWithin this policy we have a single policy check, declared with `authorize_if`. Checks logically apply from top to bottom, based on their check type. In this case, we'd read the policy as \"this policy always applies, and authorizes always\".\n\nThere are four check types, all of which do what they sound like they do:\n\n- `authorize_if` - if the check is true, the whole policy is authorized.\n- `authorize_unless` - if the check is false, the whole policy is authorized.\n- `forbid_if` - if the check is true, the whole policy is forbidden.\n- `forbid_unless` - if the check is false, the whole policy is forbidden.\n\nIf a single check does not explicitly authorize or forbid the whole policy, then the flow moves to the next check. For example, if an `authorize_if` check does NOT return true, this _does not mean the whole policy is forbidden_ - it means that further checking is required.","ref":"policies.html#the-simplest-policy"},{"type":"extras","title":"Policy with condition inside do block - Policies","doc":"A condition or a list of conditions can also be moved inside the `policy` block.\n\nThis can make a really long list of conditions easier to read.\n\n```elixir\npolicies do\n  policy do\n    condition always()\n    authorize_if always()\n  end\nend\n```","ref":"policies.html#policy-with-condition-inside-do-block"},{"type":"extras","title":"How a Decision is Reached - Policies","doc":"**Not every check in a policy must pass!** This is described above, but is very important so another example is provided here. Checks go from top to bottom, are evaluated independently of each other, and _the first one that reaches a decision_ determines the overall _policy result_. For example:\n\n```elixir\npolicy action_type(:create) do\n  authorize_if IsSuperUser\n  forbid_if Deactivated\n  authorize_if IsAdminUser\n  forbid_if RegularUserCanCreate\n  authorize_if RegularUserAuthorized\nend\n```\n\nWe check those from top to bottom, so the first one of those that returns `:authorized` or `:forbidden` determines the entire outcome. For example:\n\n```elixir\nauthorize_if IsSuperUser # If this is true, the actor is a superuser\n\n# None of the rest of the checks matter, even if the actor is deactivated.\nforbid_if Deactivated\nauthorize_if IsAdminUser\nforbid_if RegularUserCanCreate\nauthorize_if RegularUserAuthorized\n```\n\nConversely:\n\n```elixir\nauthorize_if IsSuperUser # This can be false\nforbid_if Deactivated # This can be false\nauthorize_if IsAdminUser # If this is true, then the policy is still authorized.\n\n# And none of these checks matter\nforbid_if RegularUserCanCreate\nauthorize_if RegularUserAuthorized\n```","ref":"policies.html#how-a-decision-is-reached"},{"type":"extras","title":"Not all policy checks have yes/no answers - Policies","doc":"This will be covered in greater detail in [Checks](#checks), but will be briefly mentioned here.\n\nAsh provides two basic types of policy checks - _simple_ checks and _filter_ checks. Simple checks are what we commonly think of with authorization, and what the above example would suggest - is an actor allowed to perform a given operation, yes or no? But we can also use filter checks - given a list of resources, which ones is an actor allowed to perform the operation on?\n\nFilter checks are applied to all read actions, including those generated for bulk updates and destroys.","ref":"policies.html#not-all-policy-checks-have-yes-no-answers"},{"type":"extras","title":"Read Actions and Filtering Behavior - Policies","doc":"An important characteristic of read actions is that, by default, they are **filtered** by policies rather than returning authorization errors. This means:\n\n- When a user is not allowed to see certain records, those records are simply filtered out of the results\n- Instead of receiving a `Forbidden` error, users typically get a `NotFound` error (for single record queries) or an empty/reduced result set (for multi-record queries)\n- This filtering behavior applies to all read actions, including `get`, `read`, and any custom read actions you define\n\nFor example, if a policy restricts users to only see their own posts, a query for all posts will automatically filter to only return the current user's posts, rather than raising an authorization error. Similarly, attempting to fetch a specific post that belongs to another user will result in a `NotFound` error rather than `Forbidden`.\n\nThis design is a security feature that prevents **enumeration attacks** and **information disclosure**. By not distinguishing between \"record doesn't exist\" and \"record exists but you can't access it\", the system prevents attackers from probing to discover the existence of protected data, mapping out the system's data structure, or conducting reconnaissance attacks through systematic querying.\n\n#### Bypassing this behavior\n\nYou can bypass this behavior on a case-by-case basis with the `authorize_with` option, for data layers that support error expressions (all of the core ones except `AshSqlite`).\n\nFor example, given a post that the user cannot see:\n\n```elixir\nAsh.get!(Post, 123)\n# * Invalid:\n# not found\n\nAsh.get!(Post, 123, authorize_with: :error)\n# * Forbidden\n```","ref":"policies.html#read-actions-and-filtering-behavior"},{"type":"extras","title":"Bypass policies - Policies","doc":"A bypass policy is just like a regular policy, except if a bypass passes, then other policies after it _do not need to pass_. This can be useful for writing complex access rules, or for a simple rule like \"an admin can do anything\" without needing to specify it as part of every other policy.","ref":"policies.html#bypass-policies"},{"type":"extras","title":"A realistic policy - Policies","doc":"In this example, we use some of the provided built-in checks.\n\n```elixir\npolicies do\n  # Anything you can use in a condition, you can use in a check, and vice-versa\n  # This policy applies if the actor is a super_user\n  # Additionally, this policy is declared as a `bypass`. That means that this check is allowed to fail without\n  # failing the whole request, and that if this check *passes*, the entire request passes.\n  bypass actor_attribute_equals(:super_user, true) do\n    authorize_if always()\n  end\n\n  # This will likely be a common occurrence. Specifically, policies that apply to all read actions\n  policy action_type(:read) do\n    # unless the actor is an active user, forbid\n    forbid_unless actor_attribute_equals(:active, true)\n    # if the record is marked as public, authorize\n    authorize_if expr(public == true)\n    # if the actor is related to the data via that data's `owner` relationship, authorize\n    authorize_if relates_to_actor_via(:owner)\n  end\nend\n```","ref":"policies.html#a-realistic-policy"},{"type":"extras","title":"Policy Groups - Policies","doc":"Policy groups are a small abstraction over policies, that allow you to group policies together\nthat have shared conditions. Each policy inside of a policy group have the same conditions as\ntheir group.\n\n```elixir\npolicies do\n  policy_group actor_attribute_equals(:role, :owner) do\n    policy action_type(:read) do\n      authorize_if expr(owner_id == ^actor(:id))\n    end\n\n    policy action_type([:update, :destroy]) do\n      authorize_if expr(owner_id == ^actor(:id))\n    end\n\n    policy action_type(:create) do\n      authorize_if relating_to_actor(:owner)\n    end\n  end\nend\n```","ref":"policies.html#policy-groups"},{"type":"extras","title":"Nesting Policy groups - Policies","doc":"Policy groups can be nested. This can help when you have lots of policies and conditions.\n\n```elixir\npolicies do\n  policy_group condition do\n    policy_group condition2 do\n       policy condition3 do\n         # This policy applies if condition, condition2, and condition3 are all true\n       end\n    end\n  end\nend\n```","ref":"policies.html#nesting-policy-groups"},{"type":"extras","title":"Bypasses - Policies","doc":"Policy groups can _not_ contain bypass policies. The purpose of policy groups is to make it easier to reason\nabout the behavior of policies. When you see a policy group, you know that no policies inside that group will\ninteract with policies in other policy groups, unless they also apply.","ref":"policies.html#bypasses"},{"type":"extras","title":"Access Type - Policies","doc":"Policies have an \"access type\" that determines when they are applied. By default, `access_type` is `:filter`.\nWhen applied to a read action, `:filter` will result in a filtered read. For other action types, the filter will be evaluated\nto determine if a forbidden error should be raised.\n\nThere are three access types, and they determine the _latest point in the process_ that any check contained by a policy can be applied.\n\n- `strict` - All checks must be applied statically. These result in a forbidden error if they are not met.\n- `filter` - All checks must be applied either statically or as a filter. These result in a filtered read if they are not met, and a forbidden error for other action types.\n- `runtime` - This allows checks to be run _after_ the data has been read. It is exceedingly rare that you would need to use this access type.\n\nFor example, given this policy:\n\n```elixir\npolicy action(:read_hidden) do\n  authorize_if actor_attribute_equals(:is_admin, true)\nend\n```\n\nA non-admin using the `:read_hidden` action would see an empty list of records, rather than a forbidden error.\n\nHowever, with this policy\n\n```elixir\npolicy action(:read_hidden) do\n  access_type :strict\n\n  authorize_if actor_attribute_equals(:is_admin, true)\nend\n```\n\nA non-admin using the `:read_hidden` action would see a forbidden error.","ref":"policies.html#access-type"},{"type":"extras","title":"Relationships and Policies - Policies","doc":"A common point of confusion when working with relationships is when they return less results or no results due to policies.\nAdditionally, when requesting related data that produces a forbidden error, it forbids the *entire request*.\n\nFor example, if you have a `Post`, that `belongs_to` `:author`, and the user requesting data cannot see the `author` due to a **filter** policy,\nthen you may see something like this:\n\n```elixir\n%MyApp.Post{author: nil, ...}\n```\n\nEven though it is not possible for a `Post` to exist without an associated `:author`!\n\nAdditionally, if the user cannot read the `author` due to a `:strict` policy, if you attempt to load the `:author`, the result\nof the **entire operation** will be `{:error, %Ash.Error.Forbidden{}}`.\n\nThere are two ways that you can improve this behavior\n\n#### The `allow_forbidden_field?` Option\n\nThis option will **default to `true`** in 4.0. You can adopt this behavior now with the following configuration.\n\n```elixir\nconfig :ash, :allow_forbidden_field_for_relationships_by_default?, true\n```\n\nThis option adjusts the relationship reading logic such that, if running a related read action would produce a\nforbidden error, the relationship will be set to `%Ash.ForbiddenField{}`, instead of forbidding the entire request.\n\nSo in the example above where the **entire operation** fails, you would instead get:\n\n```elixir\n{:ok, %MyApp.Post{author: %Ash.ForbiddenField{}}}\n```\n\n#### The `authorize_read_with` Option\n\nThis option typically only makes sense to apply on `has_one` and `belongs_to` relationships. This alters the behavior\nof policy filtering when loading related records. In our above example, lets say there is a policy like the following\non `MyApp.Author`, that prevents us from reading an author that has been deactivated.\n\n```elixir\npolicy action_type(:read) do\n  access_type :filter # This is the default access type. It is here for example.\n  authorize_if expr(active == false)\nend\n```\n\nWhen running a normal read action against that resource, you want any deactivated authors to be filtered out.\nHowever, when reading the `:author` relationship, you don't want the author to appear as `nil`. This is especially\nuseful when combined with `allow_forbidden_field? true`.\n\nSo lets make our `belongs_to` relationship looks like this.\n\n```elixir\nbelongs_to :author, MyApp.Author do\n  allow_nil? false\n  allow_forbidden_field? true\n  authorize_read_with :error\nend\n```\n\nNow, that filter will be applied in such a way that produces an error if any record exists that matches `not(active == false)`.\n\nSo a forbidden read of the `:author` relationship will never produce a `nil` value, nor will it produce an `{:error, %Ash.Error.Forbidden{}}`\nresult. Instead, the value of `:author` will be `%Ash.ForbiddenField{}`!","ref":"policies.html#relationships-and-policies"},{"type":"extras","title":"Checks - Policies","doc":"Checks evaluate from top to bottom within a policy. A check can produce one of three results, the same that a policy can produce. While checks are not necessarily evaluated in order, they _logically apply_ in that order, so you may as well think of it in that way. It can be thought of as a step-through algorithm.\n\nFor each check, starting from the top:\n\n- Run the check.\n  - If it returns `:authorized`, the policy is `:authorized`\n  - If it returns `:forbidden`, the policy is `:forbidden`\n  - If it returns `:unknown`, the next check down is checked\n\nFor the example from earlier:\n\n- `authorize_if IsSuperUser`\n  - If this check succeeds, it returns `:authorized`, the whole policy is `:authorized`, and checks stop running\n  - If this check fails, it returns `:unknown` and the next check is checked\n- `forbid_if Deactivated`\n  - We only care about this result if the previous check failed, ie. the actor is not a super user.\n  - If this check succeeds, it returns `:forbidden`, the whole policy is `:forbidden`, and checks stop running\n  - If this check fails, it returns `:unknown` and the next check is checked\n- `authorize_if IsAdminUser`\n  - We only care about this result if the previous checks failed, ie. the actor is not a super user and is not deactivated.\n  - If this check succeeds, it returns `:authorized`, the whole policy is `:authorized` and checks stop running.\n  - If this check fails, it returns `:unknown` and the next check is checked\n- `authorize_if RegularUserAuthorized`\n  - We only care about this result if the previous checks failed, ie. the actor is not a super user, not deactivated and not an admin user.\n  - If this check succeeds, it returns `:authorized`, the whole policy is `:authorized` and checks stop running.\n  - If this check fails, it returns `:unknown`. As there are no more checks to run, the whole policy returns `:unknown`, which is treated as forbidden and the actor is not allowed to perform the action.","ref":"policies.html#checks"},{"type":"extras","title":"Types of checks - Policies","doc":"As mentioned earlier, there are two distinct types of checks - _simple_ checks and _filter_ checks. So far we've seen examples of both - let's look in a bit more detail.\n\n> #### Manual Checks {: .neutral}\n>\n> Both simple and filter checks are a subset of a third type of check - a _manual_ check - but you will almost always want to write simple or filter checks.\n\n#### Simple checks\n\nSimple checks are determined at the outset of a request, and can only cause a request to be authorized or forbidden. These are typically yes/no questions - is the actor an admin? Did the actor create the post they want to call the `update` action on? Is the actor old enough to drink alcohol?\n\nYou can write a simple check by creating a new module and using the `Ash.Policy.SimpleCheck` module:\n\n```elixir\ndefmodule MyApp.Checks.ActorIsOldEnough do\n  use Ash.Policy.SimpleCheck\n\n  # This is used when logging a breakdown of how a policy is applied - see Logging below.\n  def describe(_) do\n    \"actor is old enough\"\n  end\n\n  # The context here may have a changeset, query, resource, and domain module, depending\n  # on the action being run.\n  # `match?` should return true or false, and answer the statement being posed in the description,\n  # i.e \"is the actor old enough?\"\n  def match?(%MyApp.User{age: age} = _actor, %{resource: MyApp.Beer} = _context, _opts) do\n    age >= 21\n  end\n\n  def match?(_, _, _), do: false\nend\n```\n\nYou can then use this module as the check name, as part of a policy:\n\n```elixir\ndefmodule MyApp.Beer do\n  # ...\n\n  policies do\n    policy action(:drink) do\n      authorize_if MyApp.Checks.ActorIsOldEnough\n    end\n  end\n\n  # ...\nend\n```\n\nAsh will internally convert the true/false return value from `match?/3` to a `:authorized`/`:forbidden`/`:unknown` response, depending on how the check is being run (ie. whether it's part of an `authorize_if`/`forbid_if`/etc.)\n\n#### Filter checks\n\nMany checks won't return a status yes/no, but instead return a \"filter\" to apply to a collection of data. They are most commonly used for read actions, but can be used for all types of actions.\n\nFor update and destroy actions, they apply to the data _before_ the action is run.\n\nFor read actions, they will automatically restrict the returned data to be compliant with the filter. Using the drinking example from earlier, we could write a filter check to list only users that are old enough to drink alcohol.\n\nThere are two ways to write a filter check - by creating a module and using the `Ash.Policy.FilterCheck` module, or by using inline expression syntax.\n\n```elixir\ndefmodule MyApp.Checks.ActorOverAgeLimit do\n  use Ash.Policy.FilterCheck\n\n  # A description is not necessary, as it will be derived from the filter, but one could be added\n  # def describe(_opts), do: \"actor is over the age limit\"\n\n  def filter(actor, _authorizer, _opts) do\n    expr(age_limit <= ^actor.age)\n  end\nend\n```\n\nYou can then use this module as the check name, as part of a policy:\n\n```elixir\ndefmodule MyApp.User do\n  # ...\n\n  policies do\n    policy action(:of_drinking_age) do\n      authorize_if MyApp.Checks.ActorOverAgeLimit\n    end\n  end\n\n  # ...\nend\n```\n\n#### Inline checks\n\nInline checks are filter checks, but are different enough to warrant their own documentation. These are written directly in a policy, eg.\n\n```elixir\npolicy action_type(:read) do\n  # Allow records with the attribute `public` set to true to be read\n  authorize_if expr(public == true)\n\n  # Allow records with the attribute `level` less than the value of the `level`\n  # argument to the action to be read\n  authorize_if expr(level <= ^arg(:level))\nend\n```\n\n##### Inline checks for create actions\n\nWhen using expressions inside of policies that apply to create actions, you may not reference the data being created. For example:\n\n```elixir\npolicy action_type(:create) do\n  # This check is fine, as we only reference the actor\n  authorize_if expr(^actor(:admin) == true)\n  # This check is not, because it contains a reference to a field\n  authorize_if expr(status == :active)\nend\n```\n\n>","ref":"policies.html#types-of-checks"},{"type":"extras","title":"Why can't we reference data in creates? - Policies","doc":">\n> We cannot allow references to the data being created in create policies, because we do not yet know what the result of the action will be.\n> For updates and destroys, referencing the data always references the data _prior_ to the action being run, and so it is deterministic.\n\nIf a policy that applies to creates, would result in a filter, you will get a `Ash.Error.Forbidden.CannotFilterCreates` at runtime explaining\nthat you must change your check. Typically this means writing a custom `Ash.Policy.SimpleCheck` instead.\n\nAsh also comes with a set of built-in helpers for writing inline checks - see `Ash.Policy.Check.Builtins` for more information.\n\n##### Referencing the actor\n\nIn expression checks, the `actor` template can be used (other templates that may work in filter expressions, for example, are not available). For example:\n\n```elixir\n# Authorize records that have an author relationship with the author ID the same as the actor ID\n# ie. records authored by the actor\nauthorize_if expr(author.id == ^actor(:id))\n```\n\n##### Using `exists`\n\nA common mistake when using related data in filters is to be too restrictive. Imagine a scenario where you have an action like this:\n\n```elixir\nread :friends_of_ted do\n  filter expr(friends.first_name == \"ted\")\nend\n```\n\nIf this was in a User resource, it would return users that have a friend with the first name \"ted\". So far so good. Then someone calls it like so:\n\n```elixir\nResource\n|> Ash.Query.for_read(:friends_of_ted)\n|> Ash.Query.filter(friends.last_name == \"dansen\")\n```\n\nThe resulting filter is `friends.first_name == \"ted\" and friends.last_name == \"dansen\"`- this means that you'll get users that have a friend with the full name \"ted dansen\". That _might_ be what you meant, but more likely you would want \"users that have a friend with the first name \"ted\", that also have a friend with the last name 'dansen'\".\n\nTo accomplish that, we can use the `exists` helper and rework the example like so:\n\n```elixir\n# There exists a friend with the first name \"ted\"\nread :friends_of_ted do\n  filter expr(exists(friends, first_name == \"ted\"))\nend\n\n# And there also exists a friend with the last name \"dansen\"\n# They may be the same friend if the user is friends with Ted Dansen!\nResource\n|> Ash.Query.for_read(:friends_of_ted)\n|> Ash.Query.filter(exists(friends, last_name == \"dansen\"))\n```\n\nIn policies (and often any time you mean \"a related thing exists where some condition is true\"), it is advised to use `exists/2` when referring to relationships because of the way that the policy authorizer may mix & match your policies when building filters. This is also true when adding filters to actions. If you use `exists`, then your policies can be used in filters without excluding unnecessary data.","ref":"policies.html#why-can-t-we-reference-data-in-creates"},{"type":"extras","title":"Field Policies - Policies","doc":"Field policies allow you to authorize access to specific fields via policies scoped to fields.\n\nFor example:\n\n```elixir\nfield_policies do\n  field_policy :role do\n    authorize_if actor_attribute_equals(:role, :supervisor)\n  end\nend\n```\n\nIf _any_ field policies exist then _all_ fields must be authorized by a field policy.\nIf you want a \"deny-list\" style, then you can add policies for specific fields.\nand add a catch-all policy using the special field name `:*`. All policies that apply\nto a field must be authorized.\n\nThe only exception to the above behavior is primary keys, which can always be read by everyone.\n\nAdditionally, keep in mind that adding `Ash.Policy.Authorizer` will require that all actions\npass policies. If you want to just add field policies, you will need to add a policy that allows\nall access explicitly, i.e\n\n```elixir\npolicies do\n  policy always() do\n    authorize_if always()\n  end\nend\n```","ref":"policies.html#field-policies"},{"type":"extras","title":"Using Expressions In Field Policies - Policies","doc":"Unlike in regular policies, expressions in field policies cannot refer to related entities currently (except when using exists). Instead, you will need to create aggregates or expression calculations that return the results you want to reference.\n\nIn results, forbidden fields will be replaced with a special value: `%Ash.ForbiddenField{}`.\n\nWhen these fields are referred to in filters, they will be replaced with an expression that evaluates to `nil`. To support this behavior, only simple and filter checks are allowed in field policies.","ref":"policies.html#using-expressions-in-field-policies"},{"type":"extras","title":"Handling private fields in internal functions - Policies","doc":"When calling internal functions like `Ash.read!/1`, private fields will by default always be shown.\nEven if field policies apply to the resource. You can change the default behaviour by setting the\n`private_fields` option on field policies.\n\n```elixir\nfield_policies do\n  private_fields :include\nend\n```\n\nThe different options are:\n\n- `:show` will always show private fields\n- `:hide` will always hide private fields\n- `:include` will let you to write field policies for private fields and private fields\n  will be shown or hidden depending on the outcome of the policy\n\nIf you want to overwrite the default option that is `:show`, you can do that by setting a global flag:\n\n```elixir\nconfig :ash, :policies, private_fields: :include\n```","ref":"policies.html#handling-private-fields-in-internal-functions"},{"type":"extras","title":"References to related data & fields - Policies","doc":"","ref":"policies.html#references-to-related-data-fields"},{"type":"extras","title":"Calculations - Policies","doc":"The dependencies of a calculation do not have any authorization applied to them. This includes the dependencies loaded with `c:Ash.Resource.Calculation.load/3`, as well as any dependencies referenced in a calculation expression. To understand why this is the case, take the following calculation:\n\n```elixir\ncalculate :users_ssn_last_4, :string, expr(fragment(\"RIGHT(?, 4)\", user.ssn))\n```\n\nThe `user` record may not be visible to the actor loading this calculation, and if it is, the `ssn` field may not be visible due to field policies. If calculations authorized access to their dependencies, you would *never* be able to write a calculation like the above.\n\n#### Aggregates\n\nAggregates are the one exception to the above. Aggregates referenced *anywhere* will *always* authorize access to the related data they reference (unless `authorize?: false` is set when running the action). They will not, however, authorize references to the\nactual field that they refer to. For example, you may not allow managers to see the email\nof their team members, but they are allowed to know something like \"how many team members\nhave set up their email\". So in an aggregate like the following:\n\n```elixir\naggregate :users_with_email, :count, :email\n```\n\nif the field policies were applied to `:email`, that number would always show `0`,\nbecause all `:email` attributes would appear to be `nil` to the manager.","ref":"policies.html#calculations"},{"type":"extras","title":"Fields annotated as input - Policies","doc":"When you use `Ash.Query.filter_input` or `Ash.Query.sort_input` (which extensions like `AshGraphql` and `AshJsonApi` do when filters/sorts are provided by the user), the contained field references are annotated as `input`. The following rules are honored for field references annotated as `input` only. Per the above section, the *contents* of a calculation's expression referenced in filters are never annotated as `input`, although the calculation itself will be, so that you can write field policies on calculations.\n\n#### Relationship paths in filters\n\nWhen a field is referenced at a relationship path, the authorization rules for that related resource's primary read action (or the relationship's configured `:read_action`) will be applied to the filter as well. For example, say that you have a `User` resource, and a `Post` resource. The `User` resource can only be read by the actor if their id matches the user's id (i.e they can only read themselves).\n\nThese policies might look like this:\n\n```elixir\n# on `User`\npolicy action_type(:read) do\n  authorize_if expr(id == ^actor(:id))\nend\n```\n\nGiven a filter like `Ash.Query.filter_input(Post, %{user: %{email: \"example@example.com\"}})`, the resulting filter expression would be:\n\n```elixir\nexpr(\n  user.email == \"example@example.com\" and\n    user.id == ^actor(:id)\n)\n```\n\nThis prevents a common security vulnerability that would allow malicious actors to \"sniff\" related data by providing filters over `User`, and seeing what `Post` records are returned.\n\n#### Field references in filters & sorts\n\nWhen a field is referenced in filters or sorts, the field reference is replaced with a conditional, that evaluates to the field value if the actor is authorized to view the field, or `nil` otherwise (causing all conditions to evaluate to `false`).\n\nLets say that users have a field policy that only allows viewing the email address if the user's id matches the actor's id, similar to the abvoe example.\n\n```elixir\nfield_policies do\n  field_policy :email, always() do\n    authorize_if expr(user.id == ^actor(:id))\n  end\nend\n```\n\nWhen that field is referenced in filters or sorts, like so: `Ash.Query.filter_input(Post, %{user: %{email: \"example@example.com\"}})`, the resulting expression would become:\n\n```elixir\nexpr(\n  if user.id == ^actor(:id) do\n    user.email\n  else\n    nil\n  end == \"example@example.com\"\n)\n```\n\nThis prevents the same security vulnerability as described above, which would allow malicious actors to \"sniff\" field values that should not be visible to them by providing filters and/or sorts that reference that field.","ref":"policies.html#fields-annotated-as-input"},{"type":"extras","title":"Debugging and Logging - Policies","doc":"","ref":"policies.html#debugging-and-logging"},{"type":"extras","title":"Policy Breakdowns - Policies","doc":"Policy breakdowns can be fetched on demand for a given forbidden error (either an `Ash.Error.Forbidden` that contains one or more `Ash.Error.Forbidden.Policy` errors, or an `Ash.Error.Forbidden.Policy` error itself), via `Ash.Error.Forbidden.Policy.report/2`.\n\nAdditionally, you can request that they be provided in the error message for all raised forbidden errors (without the help text), by setting\n\n```elixir\nconfig :ash, :policies, show_policy_breakdowns?: true\n```\n\nHere is an example policy breakdown from tests.\n\n```text\nPolicy Breakdown\nA check status of `?` implies that the solver did not need to determine that check.\nSome checks may look like they failed when in reality there was no need to check them.\nLook for policies with `✘` and `✓` in check statuses.\n\nA check with a `⬇` means that it didn't determine if the policy was authorized or forbidden, and so moved on to the next check.\n`🌟` and `⛔` mean that the check was responsible for producing an authorized or forbidden (respectively) status.\n\nIf no check results in a status (they all have `⬇`) then the policy is assumed to have failed. In some cases, however, the policy\nmay have just been ignored, as described above.\n\n  Admins and managers can create posts | ⛔:\n    authorize if: actor.admin == true | ✘ | ⬇\n    authorize if: actor.manager == true | ✘ | ⬇\n```\n\nTo remove the help text, you can pass the `help_text?: false` option, which would leave you with:\n\n```text\nPolicy Breakdown\n  Admins and managers can create posts | ⛔:\n    authorize if: actor.admin == true | ✘ | ⬇\n    authorize if: actor.manager == true | ✘ | ⬇\n```","ref":"policies.html#policy-breakdowns"},{"type":"extras","title":"Including in error messages - Policies","doc":"**IMPORTANT WARNING:** The following configuration should only ever be used in development mode!\n\nFor security reasons, authorization errors don't include any extra information, aside from `forbidden`. To have authorization errors include a policy breakdown (without help text) use the following config.\n\n```elixir\nconfig :ash, :policies, show_policy_breakdowns?: true\n```","ref":"policies.html#including-in-error-messages"},{"type":"extras","title":"Logging - Policies","doc":"It is generally safe to log authorization error details, even in production. This can be very helpful when investigating certain classes of issue.\n\nTo have Ash automatically log each authorization failure, use\n\n```elixir\nconfig :ash, :policies, log_policy_breakdowns: :error # Use whatever log level you'd like to use here\n```\n\nTo have Ash log all policy breakdowns, even successful ones (this will be lots of noise, and should only be used for dev testing)\n\n```elixir\nconfig :ash, :policies, log_successful_policy_breakdowns: :error # Use whatever log level you'd like to use here\n```","ref":"policies.html#logging"},{"type":"extras","title":"Manual Installation","doc":"# Manual Installation\nThis guide will walk you through the process of manually installing Ash into your project.\nIf you are starting from scratch, you can use `mix new` or `mix igniter.new` and follow these instructions.\nThese installation instructions apply both to new projects and existing ones.","ref":"manual-installation.html"},{"type":"extras","title":"Install & Setup Dependencies - Manual Installation","doc":"See the readmes for `spark` and `reactor` for more information on their installation.\nWe've included their changes here for your convenience.\n\nCreate `.formatter.exs`:\n```\n# Used by \"mix format\"\n[\n  inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"],\n  plugins: [Spark.Formatter]\n]\n\n```\n\nCreate `config/config.exs`:\n```\nimport Config\nconfig :spark, formatter: [remove_parens?: true]\n\n```\n\nUpdate `mix.exs`:\n```diff\n...\n    defp deps do\n      [\n+       {:sourceror, \"~> 1.8\", only: [:dev, :test]}\n        # {:dep_from_hexpm, \"~> 0.3.0\"},\n        # {:dep_from_git, git: \"https://github.com/elixir-lang/my_dep.git\", tag: \"0.1.0\"}\n...\n```\n\nUpdate `.formatter.exs`:\n```diff\n...\n  [\n    inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"],\n-   plugins: [Spark.Formatter]\n+   plugins: [Spark.Formatter],\n+   import_deps: [:reactor]\n  ]\n```","ref":"manual-installation.html#install-setup-dependencies"},{"type":"extras","title":"Skip protocol consolidation - Manual Installation","doc":"To avoid warnings about protocol consolidation when recompiling in dev, we\nset protocolc onsolidation to happen only in non-dev environments.","ref":"manual-installation.html#skip-protocol-consolidation"},{"type":"extras","title":"Setup The Formatter - Manual Installation","doc":"Configure the DSL auto-formatter. This tells the formatter to remove excess parentheses\nand how to sort sections in your Ash.Resource & Ash.Domain modules for consistency.\n\nUpdate `.formatter.exs`:\n```diff\n...\n    inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"],\n    plugins: [Spark.Formatter],\n-   import_deps: [:reactor]\n+   import_deps: [:ash, :reactor]\n  ]\n```\n\nUpdate `config/config.exs`:\n```diff\n  import Config\n- config :spark, formatter: [remove_parens?: true]\n  \n+ config :spark,\n+   formatter: [\n+     remove_parens?: true,\n+     \"Ash.Resource\": [\n+       section_order: [\n+         :resource,\n+         :code_interface,\n+         :actions,\n+         :policies,\n+         :pub_sub,\n+         :preparations,\n+         :changes,\n+         :validations,\n+         :multitenancy,\n+         :attributes,\n+         :relationships,\n+         :calculations,\n+         :aggregates,\n+         :identities\n+       ]\n+     ],\n+     \"Ash.Domain\": [section_order: [:resources, :policies, :authorization, :domain, :execution]]\n+   ]\n+\n```","ref":"manual-installation.html#setup-the-formatter"},{"type":"extras","title":"Setup Backwards Compatibility Configurations - Manual Installation","doc":"Configure backwards compatibility settings. See the [backwards compatibility guide](https://hexdocs.pm/ash/backwards-compatibility-config.html)\nfor an explanation of each of the configurations.\n\nUpdate `config/config.exs`:\n```diff\n  import Config\n  \n+ config :ash,\n+   allow_forbidden_field_for_relationships_by_default?: true,\n+   include_embedded_source_by_default?: false,\n+   show_keysets_for_all_actions?: false,\n+   default_page_type: :keyset,\n+   policies: [no_filter_static_forbidden_reads?: false],\n+   keep_read_action_loads_when_loading?: false,\n+   default_actions_require_atomic?: true,\n+   read_action_after_action_hooks_in_order?: true,\n+   bulk_actions_default_to_errors?: true\n+ \n  config :spark,\n    formatter: [\n...\n```","ref":"manual-installation.html#setup-backwards-compatibility-configurations"},{"type":"extras","title":"Multi-Step Actions","doc":"# Multi-Step Actions\n\nActions in Ash allow you to create sophisticated workflows that coordinate multiple changes or processes. Often business logic crosses multiple resources, and we often want it to be transactional. By leveraging action lifecycle hooks, you can build powerful domain-specific operations. This guide will explore how to build and use multi-step actions using a helpdesk example.\n\nFor most use cases, hooks are the preferred approach due to their simplicity and tight integration with Ash's action lifecycle. [Reactor](/documentation/topics/advanced/reactor.md) is the comprehensive solution for truly complex orchestration scenarios. Additionally, you can write [generic actions](/documentation/topics/advanced/generic-actions.md) by hand, implementing an action with fully custom code. Reactors can be used as the `run` function for generic actions, giving them first class support in Ash extensions. See [below](#generic-action-example) for an example.","ref":"multi-step-actions.html"},{"type":"extras","title":"When to use hooks vs reactors vs generic actions - Multi-Step Actions","doc":"You should use hooks for most multi-step workflow scenarios as they provide simplicity and leverage Ash's transactional nature. The key decision point is whether you need compensation/rollback across external services:\n\n**Use hooks when:**\n- Coordinating changes within Ash resources (leverages database transactions)\n- Performing side effects that don't require rollback (logging, notifications)\n- Working with external services that don't need compensation logic\n- Building small-to-medium complexity workflows\n\n**Use [reactor](/documentation/topics/advanced/reactor.md) when:**\n- You need to compensate/undo changes across multiple external services\n- Building complex workflows that require sophisticated error handling and rollback logic\n- Coordinating long-running processes that span multiple systems\n\n**Use [generic actions](/documentation/topics/actions/generic-actions.md) when:**\n- You need a high-level action that works on multiple resources, and reactor or hooks are not fitting\n- There aren't side effects or external servies\n- Short transactional operations that can be understood at a glance\n\n>","ref":"multi-step-actions.html#when-to-use-hooks-vs-reactors-vs-generic-actions"},{"type":"extras","title":"Durable Workflows - Multi-Step Actions","doc":">\n> For durable workflows, we suggest to use Oban. We provide tools to integrate with Oban in [AshOban](https://hexdocs.pm/ash_oban). AshOban supports very specific types of common workflows, like \"triggers\" that run periodically for resources, and \"scheduled actions\" which run generic actions on a cron. You should not be afraid to write \"standard\" Oban jobs and code where possible. Don't bend over backwards trying to fit everything into AshOban.","ref":"multi-step-actions.html#durable-workflows"},{"type":"extras","title":"Action Lifecycle Hooks - Multi-Step Actions","doc":"At the core of Ash's multi-step action capability are action lifecycle hooks. These hooks allow you to run code at specific points during an action's execution:\n\n- **before_transaction**: Runs before the transaction is started. Useful for operations that should happen before the transaction, like external API calls.\n\n- **before_action**: Runs in the transaction, before the data layer is called. Perfect for side effects and expensive logic. This hook can be used with changesets and queries.\n\n- **after_action**: Runs in the transaction, after the data layer is called, only if the action is successful. Ideal for transactional side effects that should only happen on success. This hook can be used with changesets and queries.\n\n- **after_transaction**: Runs after the transaction completes, in both success and error cases. Ideal for operations that should happen regardless of the transaction outcome, and for operations that work with external services.\n\nThere are other hooks that we won't go into here, as they are rarely used. See the documentation in `Ash.Changeset` for more.\n\n- **around_action**: Runs code both before and after the action logic, within the transaction.\n- **around_transaction**: Runs code both before and after the transaction, outside the transaction.","ref":"multi-step-actions.html#action-lifecycle-hooks"},{"type":"extras","title":"Examples - Multi-Step Actions","doc":"Let's explore multi-step actions through a series of increasingly complex examples using a helpdesk system. Each example builds on concepts from the previous ones.","ref":"multi-step-actions.html#examples"},{"type":"extras","title":"Example 1: Simple Activity Logging - Multi-Step Actions","doc":"The simplest multi-step action uses a single hook to perform a transactional effect. Here's a basic example that logs ticket creation by inserting an activity log.\n\n```elixir\ndefmodule HelpDesk.Changes.LogActivity do\n  use Ash.Resource.Change\n\n  @impl true\n  def change(changeset, _opts, _context) do\n    # Log activity after the ticket is successfully created\n    Ash.Changeset.after_action(changeset, fn _changeset, ticket ->\n      HelpDesk.ActivityLog.log(\"Ticket #{ticket.id} created: #{ticket.title}\")\n      {:ok, ticket}\n    end)\n  end\nend\n```\n\nUse it in your Ticket resource:\n\n```elixir\nactions do\n  create :create do\n    accept [:title, :description]\n    change HelpDesk.Changes.LogActivity\n  end\nend\n```","ref":"multi-step-actions.html#example-1-simple-activity-logging"},{"type":"extras","title":"Example 2: Multi-Hook Ticket Assignment - Multi-Step Actions","doc":"Building on the first example, let's add ticket assignment logic that uses multiple hooks to coordinate a transactional workflow:\n\n```elixir\ndefmodule HelpDesk.Changes.AssignTicket do\n  use Ash.Resource.Change\n\n  @impl true\n  def change(changeset, _opts, _context) do\n    changeset\n    |> Ash.Changeset.before_action(&find_and_assign_agent/1)\n    |> Ash.Changeset.after_action(&notify_assignment/2)\n  end\n\n  defp find_and_assign_agent(changeset) do\n    case HelpDesk.AgentManager.find_available_agent() do\n      {:ok, agent} ->\n        changeset\n        |> Ash.Changeset.force_change_attribute(:agent_id, agent.id)\n        |> Ash.Changeset.force_change_attribute(:status, \"assigned\")\n        |> Ash.Changeset.put_context(:assigned_agent, agent)\n\n      {:error, reason} ->\n        Ash.Changeset.add_error(changeset, \"No agents available: #{reason}\")\n    end\n  end\n\n  defp notify_assignment(changeset, ticket) do\n    HelpDesk.Notifications.notify_assignment(changeset.context[:agent], ticket)\n\n    {:ok, ticket}\n  end\nend\n```","ref":"multi-step-actions.html#example-2-multi-hook-ticket-assignment"},{"type":"extras","title":"Example 3: Complex Workflow with External Services - Multi-Step Actions","doc":"This example shows a sophisticated workflow that interacts with external services and handles various error conditions:\n\n```elixir\ndefmodule HelpDesk.Changes.ProcessUrgentTicket do\n  use Ash.Resource.Change\n\n  @impl true\n  def change(changeset, _opts, _context) do\n    changeset\n    # uses before_transaction as it communicates with an external service\n    # and we don't want to keep a transaction longer than necessary\n    |> Ash.Changeset.before_transaction(&validate_external_services/1)\n    # Prepare for processing transactionally\n    |> Ash.Changeset.before_action(&prepare_urgent_processing/1)\n    # Complete the workflow transactionally\n    |> Ash.Changeset.after_action(&complete_urgent_workflow/2)\n    # Perform success or failure logic after the transaction\n    |> Ash.Changeset.after_transaction(&cleanup_and_notify/2)\n  end\n\n  defp validate_external_services(changeset) do\n    # Check external services before starting transaction\n    case HelpDesk.ExternalServices.health_check() do\n      :ok ->\n        changeset\n\n      {:error, service} ->\n        Ash.Changeset.add_error(changeset,\n          message: \"External service #{service} unavailable for urgent processing\")\n    end\n  end\n\n  defp prepare_urgent_processing(changeset) do\n    priority = Ash.Changeset.get_attribute(changeset, :priority)\n\n    if priority == \"urgent\" do\n      # Reserve resources for urgent processing\n      case HelpDesk.ResourceManager.reserve_urgent_slot() do\n        {:ok, slot_id} ->\n          changeset\n          |> Ash.Changeset.force_change_attribute(:status, \"urgent_processing\")\n          |> Ash.Changeset.force_change_attribute(:processing_slot_id, slot_id)\n          |> Ash.Changeset.put_context(:reserved_slot, slot_id)\n\n        {:error, :no_slots_available} ->\n          # Fallback to normal priority with notification\n          changeset\n          |> Ash.Changeset.force_change_attribute(:priority, \"high\")\n          |> Ash.Changeset.put_context(:priority_downgraded, true)\n      end\n    else\n      changeset\n    end\n  end\n\n  defp complete_urgent_workflow(_changeset, ticket) do\n    if ticket.status == \"urgent_processing\" do\n      # Create escalation path and update external tracking\n      with {:ok, _escalation} <- create_escalation_path(ticket),\n           {:ok, _external_ref} <- HelpDesk.ExternalServices.create_urgent_case(ticket) do\n\n        # Update ticket with external reference\n        ticket\n        |> Ash.Changeset.for_update(:add_external_reference, %{\n          external_ref: \"URG-#{ticket.id}-#{System.system_time(:second)}\"\n        })\n        |> HelpDesk.Ticket.update!()\n      else\n        {:error, reason} ->\n          # Handle errors gracefully\n          HelpDesk.Logger.error(\"Failed to complete urgent workflow for ticket #{ticket.id}: #{inspect(reason)}\")\n          {:ok, ticket}\n      end\n    end\n\n    {:ok, ticket}\n  end\n\n  defp cleanup_and_notify({:ok, ticket}, changeset) do\n    # Success case - perform cleanup and notifications\n    # Any more complex cleanup than this should be\n    # pushing you to consider reactor and/or oban\n    if slot_id = Ash.Changeset.get_context(changeset, :reserved_slot) do\n      HelpDesk.ResourceManager.release_slot(slot_id)\n    end\n\n    if Ash.Changeset.get_context(changeset, :priority_downgraded) do\n      HelpDesk.Notifications.notify_priority_downgrade(ticket)\n    end\n\n    HelpDesk.Metrics.increment_urgent_tickets()\n    {:ok, ticket}\n  end\n\n  defp cleanup_and_notify({:error, _reason} = error, changeset) do\n    # Error case - clean up resources\n    if slot_id = Ash.Changeset.get_context(changeset, :reserved_slot) do\n      HelpDesk.ResourceManager.release_slot(slot_id)\n    end\n\n    error\n  end\n\n  defp create_escalation_path(ticket) do\n    HelpDesk.Escalation\n    |> Ash.Changeset.for_create(:create, %{\n      ticket_id: ticket.id,\n      level: 1,\n      escalated_at: DateTime.utc_now()\n    })\n    |> HelpDesk.Escalation.create()\n  end\nend\n```\n\nThese examples demonstrate the progression from simple logging to complex cross-resource workflows. Each example builds on the previous concepts while introducing new patterns and considerations for multi-step actions.","ref":"multi-step-actions.html#example-3-complex-workflow-with-external-services"},{"type":"extras","title":"Shortcuts for hooks - Multi-Step Actions","doc":"There are multiple ways to add hooks to actions. What we showed above, defining an `Ash.Resource.Change` module, and using the functions in `Ash.Changeset` to add callbacks to the module is the most organized & idiomatic way. However, you can also use the following techniques as shorthand. In general, prefer to write your changes into modules to keep your resources clean and keep compile times down (more smaller modules is typically better than fewer larger modules).\n\n#### Anonymous Function Changes\n\nHere we use an anonymous function change, which is a shorthand for defining a change module with a single function. Just like in the `change/3` function, you can add hooks to the changeset.\n\n```elixir\ncreate :open_and_assign do\n  change fn changeset, context ->\n    Ash.Changeset.after_action(changeset, fn changeset, result ->\n      ...\n    end)\n  end\nend\n```\n\n#### Builtin Hook Changes\n\nIf you know that you just want to add a single hook to an action, you can use some of the functions in `Ash.Resource.Change.Builtins`, which are simple shorthands for the above form. For example:\n\n```elixir\ncreate :open_and_assign do\n  change after_action(changeset, result, context ->\n    ...\n  end)\nend\n```\n\nNotice how the anonymous function takes an extra argument. In the first format above, the `context` value came from `change fn changeset, context ->`, but in this format, it is provided as an argument at the end of the builtin function, i.e `change after_action(changeset, result, context -> .`","ref":"multi-step-actions.html#shortcuts-for-hooks"},{"type":"extras","title":"Batch Callbacks - Multi-Step Actions","doc":"When working with bulk actions (like `Ash.bulk_create/3`, `Ash.bulk_update/3`, etc.), you can optimize your changes by implementing batch callbacks. These allow you to process multiple changesets together, which can be much more efficient than processing them individually.","ref":"multi-step-actions.html#batch-callbacks"},{"type":"extras","title":"Understanding Batch Callbacks - Multi-Step Actions","doc":"Batch callbacks are optional methods you can implement in your change modules:\n\n- **`batch_change/3`**: Replaces `change/3` for batch operations, receiving a list of changesets\n- **`before_batch/3`**: Runs before the batch is sent to the data layer\n- **`after_batch/3`**: Runs after the batch completes, receiving changeset-result pairs\n\n>","ref":"multi-step-actions.html#understanding-batch-callbacks"},{"type":"extras","title":"Batch Operations and Transactions - Multi-Step Actions","doc":">\n> The batch logic is currently all within transactions. There is no `before_batch_transaction` or `after_batch_transaction` hooks yet.\n\n>","ref":"multi-step-actions.html#batch-operations-and-transactions"},{"type":"extras","title":"When are batch callbacks used? - Multi-Step Actions","doc":">\n> - `batch_change/3` must be defined for `before_batch/3` and `after_batch/3` to be called\n> - The exception is `after_batch/3`, which is also called after atomic changes when `atomic/3` is defined\n> - `before_batch/3` is ignored when calling changes atomically\n> - If you define `batch_change/3`, you can omit `change/3` entirely","ref":"multi-step-actions.html#when-are-batch-callbacks-used"},{"type":"extras","title":"Example: Batch Processing with External API - Multi-Step Actions","doc":"Here's an example that demonstrates efficient batch processing for ticket creation with external service integration:\n\n```elixir\ndefmodule HelpDesk.Changes.BatchNotifyExternalSystem do\n  use Ash.Resource.Change\n\n  @impl true\n  def batch_change(changesets, _opts, _context) do\n    # Process all changesets together - you can do expensive setup here\n    # that would be wasteful to repeat for each individual changeset\n    Enum.map(changesets, &prepare_for_external_notification/1)\n  end\n\n  @impl true\n  def before_batch(changesets, opts, context) do\n    # Validate external service availability before processing the batch\n    case HelpDesk.ExternalAPI.health_check() do\n      :ok ->\n        # Mark all changesets as ready for external notification\n        Enum.map(changesets, fn changeset ->\n          Ash.Changeset.put_context(changeset, :external_api_ready, true)\n        end)\n\n      {:error, reason} ->\n        # Add errors to all changesets if external service is down\n        Enum.map(changesets, fn changeset ->\n          Ash.Changeset.add_error(changeset,\n            message: \"External API unavailable: #{reason}\")\n        end)\n    end\n  end\n\n  @impl true\n  def after_batch(changesets_and_results, opts, context) do\n    # Efficiently batch notify external system\n    notifications =\n      changesets_and_results\n      |> Enum.filter(fn {changeset, _result} ->\n        Ash.Changeset.get_context(changeset, :external_api_ready, false)\n      end)\n      |> Enum.map(fn {_changeset, result} ->\n        %{\n          ticket_id: result.id,\n          title: result.title,\n          created_at: result.inserted_at\n        }\n      end)\n\n    # Single API call for entire batch instead of one per ticket\n    case HelpDesk.ExternalAPI.batch_notify_tickets(notifications) do\n      {:ok, _response} ->\n        # Return :ok to indicate success\n        :ok\n\n      {:error, error} ->\n        # You can return individual errors or notifications\n        Enum.map(changesets_and_results, fn {_changeset, result} ->\n          {:error, Ash.Error.Invalid.exception(\n            message: \"Failed to notify external system for ticket #{result.id}: #{error}\"\n          )}\n        end)\n    end\n  end\n\n  defp prepare_for_external_notification(changeset) do\n    # Add any metadata needed for external notification\n    changeset\n    |> Ash.Changeset.put_context(:needs_external_notification, true)\n  end\nend\n```","ref":"multi-step-actions.html#example-batch-processing-with-external-api"},{"type":"extras","title":"Example: Optimized Database Operations - Multi-Step Actions","doc":"This example shows how to use batch callbacks to optimize database operations:\n\n```elixir\ndefmodule HelpDesk.Changes.BatchAssignAgents do\n  use Ash.Resource.Change\n\n  @impl true\n  def before_batch(changesets, _opts, _context) do\n    # Pre-load all available agents once for the entire batch\n    available_agents =\n      HelpDesk.Agent\n      |> Ash.Query.filter(status == \"available\")\n      |> Ash.Query.sort(:workload)\n      |> Ash.read!()\n\n    # Distribute agents across the batch\n    {assigned_changesets, _remaining_agents} =\n      Enum.map_reduce(changesets, available_agents, fn changeset, [agent | rest] ->\n        updated_changeset =\n          changeset\n          |> Ash.Changeset.change_attribute(:agent_id, agent.id)\n          |> Ash.Changeset.change_attribute(:status, \"assigned\")\n          |> Ash.Changeset.put_context(:assigned_agent, agent)\n\n        {updated_changeset, rest ++ [%{agent | workload: agent.workload + 1}]}\n      end)\n\n    assigned_changesets\n  end\n\n  @impl true\n  def after_batch(changesets_and_results, _opts, _context) do\n    # Batch update agent workloads\n    agent_updates =\n      changesets_and_results\n      |> Enum.map(fn {changeset, _result} ->\n        Ash.Changeset.get_context(changeset, :assigned_agent)\n      end)\n      |> Enum.filter(& &1)\n      |> Enum.group_by(& &1.id)\n      |> Enum.map(fn {agent_id, assignments} ->\n        %{id: agent_id, workload_increment: length(assignments)}\n      end)\n\n    # Single bulk operation to update all agent workloads\n    HelpDesk.Agent\n    |> Ash.bulk_update(:increment_workload, agent_updates)\n\n    :ok\n  end\n\n  # Define batch_change/3 to enable batch callbacks\n  @impl true\n  def batch_change(changesets, _opts, _context) do\n    # The actual changeset modifications happen in before_batch\n    # This just returns the changesets as-is\n    changesets\n  end\nend\n```","ref":"multi-step-actions.html#example-optimized-database-operations"},{"type":"extras","title":"Example: Conditional Batch Processing - Multi-Step Actions","doc":"You can control when batch callbacks are used with the `batch_callbacks?/3` callback:\n\n```elixir\ndefmodule HelpDesk.Changes.ConditionalBatchProcessing do\n  use Ash.Resource.Change\n\n  @impl true\n  def batch_callbacks?(changesets, opts, context) do\n    # Only use batch processing for large batches\n    length(changesets) >= 10\n  end\n\n  @impl true\n  def change(changeset, opts, context) do\n    # This runs for individual changes or small batches\n    changeset\n    |> perform_individual_processing(opts, context)\n  end\n\n  @impl true\n  def batch_change(changesets, opts, context) do\n    # This runs for large batches (10+ items)\n    changesets\n    |> perform_optimized_batch_processing(opts, context)\n  end\n\n  defp perform_individual_processing(changeset, _opts, _context) do\n    # Simple processing for individual items\n    changeset\n  end\n\n  defp perform_optimized_batch_processing(changesets, _opts, _context) do\n    # Optimized processing for large batches\n    changesets\n  end\nend\n```","ref":"multi-step-actions.html#example-conditional-batch-processing"},{"type":"extras","title":"Best Practices for Batch Callbacks - Multi-Step Actions","doc":"1. **Use batch callbacks for expensive operations**: Database queries, external API calls, file I/O\n2. **Keep individual operations fast**: If your change is already fast, batch callbacks may not be worth the complexity\n3. **Handle errors gracefully**: Return appropriate error tuples from `after_batch/3` when things go wrong\n4. **Test both paths**: Ensure your change works correctly both individually and in batches","ref":"multi-step-actions.html#best-practices-for-batch-callbacks"},{"type":"extras","title":"Generic Action Example - Multi-Step Actions","doc":"```elixir\n# Define a plain-old elixir module/function to express the action\ndefmodule HelpDesk.Actions.AssignTicket do\n  def run(input, context) do\n    with {:ok, agent} <- HelpDesk.AgentManager.find_available_agent(),\n         {:ok, ticket} <- HelpDesk.get_ticket_by_id(input.arguments.ticket_id),\n         {:ok, ticket} <- HelpDesk.update_ticket(ticket, %{agent_id: agent.id, status: :assigned}, actor: input.actor)\n         :ok <- Helpdesk.Notifications.notify_assignment(agent, ticket)\n    end\n  end\nend\n\n# Invoke the action from Resource\nactions do\n  action :assign_to_available_agent do\n    transaction? true\n    argument :ticket_id, :uuid\n    run HelpDesk.Actions.AssignTicket\n  end\nend\n```","ref":"multi-step-actions.html#generic-action-example"},{"type":"extras","title":"Reactor","doc":"# Reactor\n\n`Ash.Reactor` is an extension for [`Reactor`](https://github.com/ash-project/reactor) which adds explicit support for interacting with resources via their defined actions.\n\nSee [Getting started with Reactor](https://hexdocs.pm/reactor/01-getting-started.html) to understand the core Reactor concepts first. Then return to this guide to see how `Ash.Reactor` adds conveniences for using Reactor from Ash.","ref":"reactor.html"},{"type":"extras","title":"Usage - Reactor","doc":"You can either add the `Ash.Reactor` extension to your existing reactors eg:\n\n```elixir\ndefmodule MyExistingReactor do\n  use Reactor, extensions: [Ash.Reactor]\nend\n```\n\nor for your convenience you can use `use Ash.Reactor` which expands to exactly the same as above.","ref":"reactor.html#usage"},{"type":"extras","title":"Running Reactors as an action - Reactor","doc":"Ash's [generic actions](actions.md#generic-actions) support providing a Reactor module directly as their `run` option.\nThis is the preferred way for you to initiate reactors in your application. These actions could be defined on your existing resources, your you could even have a resource w/ a single action on it that runs a reactor, and no attributes/data layer etc. for example.\n\nNotes:\n\n- Every Reactor input must have a corresponding action argument.\n- Ash's action context is passed in as the Reactor's context (including things like actor, tenant, etc).\n- [Reactor runtime options](`t:Reactor.options/0`) can be set by setting `run {MyReactor, opts}` instead of just `run MyReactor`.\n- If you set the `transaction?` action DSL option to true then the Reactor will be run synchronously - regardless of the value of the `async?` runtime option.","ref":"reactor.html#running-reactors-as-an-action"},{"type":"extras","title":"Example - Reactor","doc":">","ref":"reactor.html#example"},{"type":"extras","title":"Resources can just have generic actions - Reactor","doc":">\n> Below is a fully valid resource in its entirety. Not all resources need to have state/data layers associated with them\n\n\n```elixir\ndefmodule MyApp.Blog.Actions do\n  use Ash.Resource\n\n  action :create_post, :struct do\n    constraints instance_of: MyBlog.Post\n\n    argument :blog_title, :string, allow_nil?: false\n    argument :blog_body, :string, allow_nil?: false\n    argument :author_email, :ci_string, allow_nil?: false\n\n    run MyApp.Blog.Reactors.CreatePost\n  end\nend\n```","ref":"reactor.html#resources-can-just-have-generic-actions"},{"type":"extras","title":"Example - Reactor","doc":"An example is worth 1000 words of prose:\n\n```elixir\ndefmodule ExampleReactor do\n  use Ash.Reactor\n\n  ash do\n    default_domain ExampleDomain\n  end\n\n  input :customer_name\n  input :customer_email\n  input :plan_name\n  input :payment_nonce\n\n  create :create_customer, Customer do\n    inputs %{name: input(:customer_name), email: input(:customer_email)}\n  end\n\n  read_one :get_plan, Plan, :get_plan_by_name do\n    inputs %{name: input(:plan_name)}\n    fail_on_not_found? true\n  end\n\n  action :take_payment, PaymentProvider do\n    inputs %{\n      nonce: input(:payment_nonce),\n      amount: result(:get_plan, [:price])\n    }\n  end\n\n  create :subscription, Subscription do\n    inputs %{\n      plan_id: result(:get_plan, [:id]),\n      payment_provider_id: result(:take_payment, :id)\n    }\n  end\nend\n```","ref":"reactor.html#example-1"},{"type":"extras","title":"Actions - Reactor","doc":"For each action type there is a corresponding step DSL, which needs a name (used\nto refer to the result of the step by other steps), a resource and optional\naction name (defaults to the primary action if one is not provided).\n\nActions have several common options and some specific to their particular type.\nSee the [DSL documentation](dsl-ash-reactor.html) for\ndetails.","ref":"reactor.html#actions"},{"type":"extras","title":"Action inputs - Reactor","doc":"Ash actions take a map of input parameters which are usually a combination of\nresource attributes and action arguments. You can provide these values as a\nsingle map using the [`inputs` DSL entity](dsl-ash-reactor.html#reactor-action-inputs) with a map or keyword list which refers to Reactor inputs, results and hard-coded values via Reactor's [predefined template functions](https://hexdocs.pm/reactor/Reactor.Dsl.Argument.html#functions).\n\nFor action types that act on a specific resource (ie `update` and `destroy`) you can provide the value using the [`initial` DSL option](dsl-ash-reactor.html#reactor-update-initial).\n\n#### Example\n\n```elixir\ninput :blog_title\ninput :blog_body\ninput :author_email\n\nread :get_author, MyBlog.Author, :get_author_by_email do\n  inputs %{email: input(:author_email)}\nend\n\ncreate :create_post, MyBlog.Post, :create do\n  inputs %{\n    title: input(:blog, [:title]),\n    body: input(:blog, [:body]),\n    author_id: result(:get_author, [:email])\n  }\nend\n\nupdate :author_post_count, MyBlog.Author, :update_post_count do\n  wait_for :create_post\n  initial result(:get_author)\nend\n\nreturn :create_post\n```","ref":"reactor.html#action-inputs"},{"type":"extras","title":"Handling failure. - Reactor","doc":"Reactor is a saga executor, which means that when failure occurs it tries to\nclean up any intermediate state left behind. By default the `create`, `update`\nand `destroy` steps do not specify any behaviour for what to do when there is a\nfailure downstream in the reactor. This can be changed by providing both an\n`undo_action` and changing the step's `undo` option to either\n`:outside_transaction` or `:always` depending on your resource and datalayer\nsemantics.","ref":"reactor.html#handling-failure"},{"type":"extras","title":"The undo option. - Reactor","doc":"- `:never` - this is the default, and means that the reactor will never try and\n  undo the action's work. This is the most performant option, as it means that\n  the reactor doesn't need to store as many intermediate values.\n- `:outside_transaction` - this option allows the step to decide at runtime\n  whether it should support undo based on whether the action is being run within\n  a transaction. If it is, then no undo is required because the transaction\n  will rollback.\n- `:always` - this forces the step to always undo it's work on failure.","ref":"reactor.html#the-undo-option"},{"type":"extras","title":"The undo_action option. - Reactor","doc":"The behaviour of the `undo_action` is action specific:\n\n- For `create` actions, the `undo_action` should be the name of a `destroy`\n  action with no specific requirements.\n- For `update` actions, the `undo_action` should also be an `update` action\n  which takes a `changeset` argument, which will contain the `Ash.Changeset`\n  which was used to execute the original update.\n- For `destroy` actions, the `undo_action` should be the name of a `create`\n  action which takes a `record` argument, which will contain the\n  resource record which was used destroyed.","ref":"reactor.html#the-undo_action-option"},{"type":"extras","title":"Transactions - Reactor","doc":"You can use the `transaction` step type to wrap a group of steps inside a data layer transaction, however the following caveats apply:\n\n- All steps inside a transaction must happen in the same process, so the steps\n  inside the transaction will only ever be executed synchronously.\n- Notifications will be sent only when the transaction is committed.\n\n#### Example\n\n```elixir\ninput :blog_title\ninput :blog_body\ninput :author_email\n\nread :get_author, MyBlog.Author, :get_author_by_email do\n  inputs %{email: input(:author_email)}\nend\n\ntransaction :create_post_transaction, [MyBlog.Post, MyBlog.Author] do\n  create :create_post, MyBlog.Post, :create do\n    inputs %{\n      title: input(:blog, [:title]),\n      body: input(:blog, [:body]),\n      author_id: result(:get_author, [:email])\n    }\n  end\n\n  update :author_post_count, MyBlog.Author, :update_post_count do\n    wait_for :create_post\n    initial result(:get_author)\n  end\n\n  return :create_post\nend\n\nreturn :create_post_transaction\n```","ref":"reactor.html#transactions"},{"type":"extras","title":"Notifications - Reactor","doc":"Because a reactor has transaction-like semantics notifications are automatically batched and only sent upon successful completion of the reactor.","ref":"reactor.html#notifications"},{"type":"extras","title":"Monitoring","doc":"# Monitoring\n\nMonitoring in Ash has two primary components, `Ash.Tracer` and `:telemetry`. Monitoring might also be referred to as observability and instrumentation.","ref":"monitoring.html"},{"type":"extras","title":"Packages - Monitoring","doc":"If you want to integrate with [Appsignal](https://www.appsignal.com), use the [AshAppsignal](https://hexdocs.pm/ash_appsignal) package, which is maintained by the core team. We believe that Appsignal is a great way to get started quickly, is relatively cost effective, and provides a great user experience.","ref":"monitoring.html#packages"},{"type":"extras","title":"Telemetry - Monitoring","doc":"Ash emits the following telemetry events, suffixed with `:start` and `:stop`. Start events have `system_time` measurements, and stop events have `system_time` and `duration` measurements. All times will be in the native time unit.","ref":"monitoring.html#telemetry"},{"type":"extras","title":"Important - Monitoring","doc":"Note the mention of `:start` and `:stop` suffixes. The event below `[:ash, (domain_short_name), :create]`, is actually referring to two events, `[:ash, (domain_short_name), :create, :start]` and `[:ash, (domain_short_name), :create, :stop]`.\n\n\\_Replace `(domain_short_name)` with your domain short name, from `d:Ash.Domain.Info.short_name`.","ref":"monitoring.html#important"},{"type":"extras","title":"Events - Monitoring","doc":"- `[:ash, (domain_short_name), :create]` - The execution of a create action. Use `resource_short_name` and `action` metadata to break down measurements.\n- `[:ash, (domain_short_name), :update]` - The execution of a update action. Use `resource_short_name` and `action` metadata to break down measurements.\n- `[:ash, (domain_short_name), :read]` - The execution of a read action. Use `resource_short_name` and `action` metadata to break down measurements.\n- `[:ash, (domain_short_name), :destroy]` - The execution of a destroy action. Use `resource_short_name` and `action` metadata to break down measurements.\n- `[:ash, (domain_short_name), :action]` - The execution of a generic action. Use `resource_short_name` and `action` metadata to break down measurements.\n- `[:ash, :changeset]` - A changeset being processed for a given action, i.e with `Ash.Changeset.for_create`. Use `resource_short_name` metadata to break down measurements.\n- `[:ash, :query]` - A query being processed for an action, with `Ash.Query.for_read`. Use `resource_short_name` metadata to break down measurements.\n- `[:ash, :validation]` - A validation being run on a changeset. Use `resource_short_name` and `validation` metadata to break down measurements.\n- `[:ash, :change]` - A change being run on a changeset. Use `resource_short_name` and `change` metadata to break down measurements.\n- `[:ash, :calculation]` - A calculation being computed in the app. Use `resource_short_name` and `calculation` metadata to break down measurements.\n- `[:ash, :before_action]` - A before_action being run on a changeset. Use `resource_short_name` to break down measurements.\n- `[:ash, :after_action]` - An after_action being run on a changeset. Use `resource_short_name` to break down measurements.\n- `[:ash, :preparation]` - A preparation being run on a changeset. Use `resource_short_name` and `preparation` metadata to break down measurements.\n- `[:ash, :request_step]` - The resolution of an internal request. Ash breaks up its operations internally into multiple requests, this can give you a high resolution insight onto the execution of those internal requests resolution. Use `name` metadata to break down measurements.\n- `[:ash, :flow]` - The execution of an Ash flow. Use `flow_short_name` to break down measurements.\n- `[:ash, :flow, :custom_step]` - The execution of a custom flow step (only if using the built in runner, which is currently the only runner). Use `flow_short_name` and `name` metadata to break down measurements.","ref":"monitoring.html#events"},{"type":"extras","title":"Tracing - Monitoring","doc":"Tracing is very similar to telemetry, but gives you some additional hooks to `set_span_context()` and `get_span_context()`. This allows you to \"move\" some piece of context between two processes. Ash will call this whenever it starts a new process to do anything. What this means is that if you are using a tracing tool or library you can ensure that any processes spawned by Ash are properly included in the trace. Additionally, you should be able to integrate a tracing library to include Ash actions/spans relatively easily by implementing the other callbacks.\n\nA tracer can be configured globally in application config.\n\n```elixir\nconfig :ash, :tracer, MyApp.Tracer\n```\n\nAdditionally, one can be provide when creating changesets or calling an action, i.e\n\n```elixir\nResource\n# better to put it here, as changesets are traced as well. It will be carried over to the domain action\n|> Ash.Changeset.for_create(:create, %{}, tracer: MyApp.Tracer)\n# but you can also pass it here.\n|> Ash.create!(tracer: MyApp.Tracer)\n```\n\nFor customizing the names created for each span, see:\n\n- `d:Ash.Domain.Dsl.execution|trace_name`\n- `d:Ash.Resource.Dsl.resource|trace_name`","ref":"monitoring.html#tracing"},{"type":"extras","title":"Trace types - Monitoring","doc":"These are the list of trace types.\n\n- :custom\n- :action\n- :changeset\n- :validation\n- :change\n- :calculation\n- :before_transaction\n- :before_action\n- :after_transaction\n- :after_action\n- :request_step\n- :custom_flow_step\n- :flow\n- :query\n- :preparation","ref":"monitoring.html#trace-types"},{"type":"extras","title":"After/Before Action Hooks - Monitoring","doc":"Due to the way before/after action hooks run, their execution time won't be included in the span created for the change. In practice, before/after action hooks are where the long running operations tend to be. We start a corresponding `span` and emit a telemetry event for before and after hooks, but they are only so useful. In a trace, they can highlight that \"some hook\" took a long time. In telemetry metrics they are of even less use. The cardinality of the metric would be extremely high, and we don't have a \"name\" or anything to distinguish them. To that end, you can use the macros & functions available in `Ash.Tracer` to create custom spans and/or emit custom telemetry events from your hooks. They automatically handle cases where the provided tracer is `nil`, for convenience. For example:\n\n```elixir\ndefmodule MyApp.CustomChange do\n  use Ash.Resource.Change\n\n  require Ash.Tracer\n\n  def change(changeset, _, _) do\n    changeset\n    |> Ash.Changeset.before_action(fn changeset ->\n      Ash.Tracer.span(:custom, \"custom name\", changeset.context[:private][:tracer]) do\n        # optionally set some metadata\n        metadata = %{...}\n        Ash.Tracer.set_metadata(changeset.context[:private][:tracer], :custom, metadata)\n        # will get `:start` and `:stop` suffixed events emitted\n        Ash.Tracer.telemetry_span([:telemetry, :event, :name], metadata) do","ref":"monitoring.html#after-before-action-hooks"},{"type":"extras","title":"Your logic here - Monitoring","doc":"end\n      end\n    end)\n  end\nend\n```","ref":"monitoring.html#your-logic-here"},{"type":"extras","title":"Pagination","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Pagination\n\n```elixir\nMix.install([{:ash, \"~> 3.0\"}], consolidate_protocols: false)\nLogger.configure(level: :warning)\n```","ref":"pagination.html"},{"type":"extras","title":"Pagination in Ash - Pagination","doc":"Ash has built-in support for two kinds of pagination: `offset` and `keyset`. You can perform pagination by passing the `:page` option to read actions, or using `Ash.Query.page/2` on the query. The page options vary depending on which kind of pagination you want to perform.\n\nPagination support is configured on a per-action basis. A single action can support both kinds of pagination if desired, but typically you would use one or the other. Read actions generated with `defaults [:read]` support both offset and keyset pagination, for other `read` actions you have to configure the [`pagination` section](https://hexdocs.pm/ash/dsl-ash-resource.html#actions-read-pagination).\n\n>","ref":"pagination.html#pagination-in-ash"},{"type":"extras","title":"Check the updated query return type! - Pagination","doc":">\n> Pagination will modify the return type of calling the query action.\n>\n> Without pagination, Ash will return a list of records.\n>\n> But _with_ pagination, Ash will return an `Ash.Page.Offset` struct (for offset pagination) or `Ash.Page.Keyset` struct (for keyset pagination). Both structs contain the list of records in the `results` key of the struct.","ref":"pagination.html#check-the-updated-query-return-type"},{"type":"extras","title":"Offset Pagination - Pagination","doc":"Offset pagination is done via providing a `limit` and an `offset` when making queries.\n\n* The `limit` determines how many records should be returned in the query.\n* The `offset` describes how many records from the beginning should be skipped.","ref":"pagination.html#offset-pagination"},{"type":"extras","title":"Pros of offset pagination - Pagination","doc":"* Simple to think about\n* Possible to skip to a page by number. E.g the 5th page of 10 records is `offset: 40`\n* Easy to reason about what page you are currently on (if the total number of records is requested)\n* Can go to the last page (though data may have changed between calculating the last page details, and requesting it)","ref":"pagination.html#pros-of-offset-pagination"},{"type":"extras","title":"Cons of offset pagination - Pagination","doc":"* Does not perform well on large datasets (if you have to ask if your dataset is \"large\", it probably isn't)\n* When moving between pages, if data was created or deleted, individual records may be missing or appear on multiple pages","ref":"pagination.html#cons-of-offset-pagination"},{"type":"extras","title":"Keyset Pagination - Pagination","doc":"Keyset pagination, also known as cursor pagination, is done via providing an `after` or `before` option, as well as a `limit`.\n\n* The `limit` determines how many records should be returned in the query.\n* The `after` or `before` value should be a `keyset` value that has been returned from a previous request. Keyset values are returned whenever there is any read action on a resource that supports keyset pagination, and they are stored in the `__metadata__` key of each record.\n\n>","ref":"pagination.html#keyset-pagination"},{"type":"extras","title":"Keysets are directly tied to the sorting applied to the query - Pagination","doc":">\n> You can't change the sort applied to a request being paginated, and use the same keyset. If you want to change the sort, but *keep* the record who's keyset you are using in the `before` or `after` option, you must first request the individual record, with the new sort applied. Then, you can use the new keyset.","ref":"pagination.html#keysets-are-directly-tied-to-the-sorting-applied-to-the-query"},{"type":"extras","title":"Pros of keyset pagination - Pagination","doc":"* Performs very well on large datasets (assuming indices exist on the columns being sorted on)\n* Behaves well as data changes. The record specified will always be the first or last item in the page","ref":"pagination.html#pros-of-keyset-pagination"},{"type":"extras","title":"Cons of keyset paginations - Pagination","doc":"* A bit more complex to use\n* Can't go to a specific page number","ref":"pagination.html#cons-of-keyset-paginations"},{"type":"extras","title":"Counting records - Pagination","doc":"When calling an action that uses pagination, the full count of records can be requested by adding the option `count: true` to the page options.\nNote that this will perform a second query to fetch the count, which can be expensive on large data sets.","ref":"pagination.html#counting-records"},{"type":"extras","title":"Relationship pagination - Pagination","doc":"In addition to paginating root data, Ash is also capable of paginating relationships when you load them. To do this, pass a custom query in the load and call `Ash.Query.page/2` on it.\n\nThis can be leveraged by extensions to provide arbitrarily nested pagination, or it can be used directly in code to split data processing when dealing with relationship with a high cardinality.\n\n>","ref":"pagination.html#relationship-pagination"},{"type":"extras","title":"Pagination Defaults for Custom Read Actions - Pagination","doc":">\n> When you define a custom `read` action (instead of using the default generated by `defaults [:read]`), **pagination is not enabled by default**. This means features like streaming, offset, or keyset pagination will not work unless you explicitly configure them.\n>\n> To enable both offset and keyset pagination (matching the default behavior), add:\n>\n> ```elixir\n> read :read do\n>   primary? true\n>   pagination required? false, offset? true, keyset? true\n> end\n> ```\n>\n> If you omit this, you may see errors like:\n>\n> ```\n> Invalid Error\n> * ...read had no matching bulk strategy that could be used.\n> Requested strategies: [:stream]\n> ...\n> Action ... does not support streaming with one of [:keyset].\n> ```\n>\n> See the [pagination guide](/documentation/topics/advanced/pagination.livemd) for more details.","ref":"pagination.html#pagination-defaults-for-custom-read-actions"},{"type":"extras","title":"Pagination example - Pagination","doc":"Modify the setup block and configure the log level to `:debug` to see logs from the ETS data layer.\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nLogger.configure(level: :debug)\n```","ref":"pagination.html#pagination-example"},{"type":"extras","title":"Define some resources for our purpose - Pagination","doc":"```elixir\ndefmodule Post do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets\n\n  attributes do\n    uuid_primary_key(:id)\n    attribute(:title, :string, allow_nil?: false)\n    attribute(:text, :string, allow_nil?: false)\n  end\n\n  actions do\n    defaults(create: [:title, :text])\n\n    read :read do\n      primary?(true)\n      prepare(build(sort: :title))\n\n      pagination do\n        required?(false)\n        offset?(true)\n        keyset?(true)\n        countable(true)\n      end\n    end\n\n    read :keyset do\n      prepare(build(sort: :title))\n      pagination(keyset?: true)\n    end\n\n    update :add_comment do\n      require_atomic?(false)\n      argument(:comment, :string, allow_nil?: false)\n      change(manage_relationship(:comment, :comments, value_is_key: :text, type: :create))\n    end\n  end\n\n  relationships do\n    has_many(:comments, Comment, sort: [:created_at])\n  end\nend\n\ndefmodule Comment do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets\n\n  attributes do\n    uuid_primary_key(:id)\n    attribute(:text, :string, allow_nil?: false)\n    create_timestamp(:created_at)\n  end\n\n  actions do\n    defaults([:read, create: [:text, :post_id], update: [:text, :post_id]])\n  end\n\n  relationships do\n    belongs_to(:post, Post, sort: [:created_at])\n  end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource Post do\n      define(:list_posts, action: :read)\n      define(:list_posts_with_keyset, action: :keyset)\n      define(:create_post, action: :create, args: [:title, :text])\n      define(:add_comment_to_post, action: :add_comment, args: [:comment])\n    end\n\n    resource(Comment)\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 2, 31, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   Ash.Domain.Dsl,\n   %{opts: [], entities: [...]},\n   Ash.Domain.Dsl,\n   Ash.Domain.Dsl.Resources.Options,\n   ...\n ]}\n```","ref":"pagination.html#define-some-resources-for-our-purpose"},{"type":"extras","title":"Create 5 posts with 5 comments each - Pagination","doc":"```elixir\nfor post_idx <- 1..5 do\n  post = Domain.create_post!(\"post #{post_idx}\", \"text #{post_idx}\")\n\n  for comment_idx <- 1..5 do\n    Domain.add_comment_to_post!(post, \"comment #{comment_idx}\")\n  end\nend\n\nDomain.list_posts!(load: :comments)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #Post ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"6cdea87b-cb69-4dc5-9ff3-54fb46bd70b0\",\n        text: \"comment 1\",\n        created_at: ~U[2024-05-28 21:32:59.013913Z],\n        post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"c653e92c-fe2f-4011-84c8-ace28ebbb207\",\n        text: \"comment 2\",\n        created_at: ~U[2024-05-28 21:32:59.021204Z],\n        post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"aa207735-0a02-4b51-b5f6-69564a2a6365\",\n        text: \"comment 3\",\n        created_at: ~U[2024-05-28 21:32:59.022890Z],\n        post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"09f9cdfe-5a88-4f6a-a8d9-2f8aa312efb8\",\n        text: \"comment 4\",\n        created_at: ~U[2024-05-28 21:32:59.024526Z],\n        post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"9b92edf4-e79e-4870-9dd0-9130863a9715\",\n        text: \"comment 5\",\n        created_at: ~U[2024-05-28 21:32:59.026132Z],\n        post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >\n    ],\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n    title: \"post 1\",\n    text: \"text 1\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"9a87a3a0-6930-4920-9345-8227b861c2ed\",\n        text: \"comment 1\",\n        created_at: ~U[2024-05-28 21:32:59.028515Z],\n        post_id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"a5107151-5519-4925-ab73-aef75274cd4a\",\n        text: \"comment 2\",\n        created_at: ~U[2024-05-28 21:32:59.030176Z],\n        post_id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"0d7e9835-25a0-4df9-bb41-06aa964dc677\",\n        text: \"comment 3\",\n        created_at: ~U[2024-05-28 21:32:59.031780Z],\n        post_id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"7c6663a8-6a36-4c9a-a947-a70436add8be\",\n        text: \"comment 4\",\n        created_at: ~U[2024-05-28 21:32:59.033389Z],\n        post_id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"4a54ee7f-18fe-401b-9a86-7c768bc52a1d\",\n        text: \"comment 5\",\n        created_at: ~U[2024-05-28 21:32:59.034976Z],\n        post_id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >\n    ],\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n    title: \"post 2\",\n    text: \"text 2\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"9d01f179-4220-4796-bb7d-63527385e36b\",\n        text: \"comment 1\",\n        created_at: ~U[2024-05-28 21:32:59.037470Z],\n        post_id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"49b7a671-bc6f-4772-b8f8-73b2a4c75b34\",\n        text: \"comment 2\",\n        created_at: ~U[2024-05-28 21:32:59.039117Z],\n        post_id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"7d2ee81c-696f-4190-8d1a-45702bfaaef2\",\n        text: \"comment 3\",\n        created_at: ~U[2024-05-28 21:32:59.040795Z],\n        post_id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"98d80b57-b911-44f5-9a63-5db90c1a0d57\",\n        text: \"comment 4\",\n        created_at: ~U[2024-05-28 21:32:59.042457Z],\n        post_id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"a0f08863-ec7f-4eba-b9f5-f9d7764dc934\",\n        text: \"comment 5\",\n        created_at: ~U[2024-05-28 21:32:59.044061Z],\n        post_id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >\n    ],\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n    title: \"post 3\",\n    text: \"text 3\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"2a57aa5f-0431-4d7b-a054-3bf0fc6cb2e8\",\n        text: \"comment 1\",\n        created_at: ~U[2024-05-28 21:32:59.046395Z],\n        post_id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"1604094f-864e-4df3-9365-a16a77ace0ba\",\n        text: \"comment 2\",\n        created_at: ~U[2024-05-28 21:32:59.048111Z],\n        post_id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"6153ecb4-4668-4afb-94c8-b1e57ed1a187\",\n        text: \"comment 3\",\n        created_at: ~U[2024-05-28 21:32:59.049749Z],\n        post_id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"0844db43-39f7-41c3-aa74-41238b0882c9\",\n        text: \"comment 4\",\n        created_at: ~U[2024-05-28 21:32:59.051385Z],\n        post_id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"b22db803-cad0-4d90-ada1-944f0abdd304\",\n        text: \"comment 5\",\n        created_at: ~U[2024-05-28 21:32:59.053563Z],\n        post_id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >\n    ],\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n    title: \"post 4\",\n    text: \"text 4\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"7b5fc51b-8e68-441e-bcd0-e52a0158e779\",\n        text: \"comment 1\",\n        created_at: ~U[2024-05-28 21:32:59.056055Z],\n        post_id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"e3f01413-c79c-44ef-abd9-6cb27a3b31fc\",\n        text: \"comment 2\",\n        created_at: ~U[2024-05-28 21:32:59.057708Z],\n        post_id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"a461c559-5036-4113-93be-3f531af4d2f3\",\n        text: \"comment 3\",\n        created_at: ~U[2024-05-28 21:32:59.059418Z],\n        post_id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"a704a781-3ff2-479c-b428-d8a414223f00\",\n        text: \"comment 4\",\n        created_at: ~U[2024-05-28 21:32:59.061034Z],\n        post_id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >,\n      #Comment ,\n        __meta__: #Ecto.Schema.Metadata<:loaded>,\n        id: \"2d9e9279-ef21-4dd4-bdc6-adc3597fefb2\",\n        text: \"comment 5\",\n        created_at: ~U[2024-05-28 21:32:59.062631Z],\n        post_id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n        aggregates: %{},\n        calculations: %{},\n        ...\n      >\n    ],\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n    title: \"post 5\",\n    text: \"text 5\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"pagination.html#create-5-posts-with-5-comments-each"},{"type":"extras","title":"Offset pagination - Pagination","doc":"When using offset pagination, a `%Ash.Page.Offset{}` struct is returned from read actions.\n\n```elixir\npage = Domain.list_posts!(page: [limit: 2])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      title: \"post 1\",\n      text: \"text 1\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n      title: \"post 2\",\n      text: \"text 2\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 2,\n  offset: 0,\n  count: nil,\n  rerun: {#Ash.Query , [authorize?: true, reuse_values?: false, return_query?: false]},\n  more?: true\n}\n```\n\nYou can find the results in the `results` field of the page\n\n```elixir\npage.results\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #Post ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n    title: \"post 1\",\n    text: \"text 1\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n    title: \"post 2\",\n    text: \"text 2\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\nThe `more?` field contains a boolean indicating if there are more pages available\n\n```elixir\npage.more?\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\ntrue\n```","ref":"pagination.html#offset-pagination-1"},{"type":"extras","title":"Retrieving the next page - Pagination","doc":"You can calculate the next offset with the information available in the page and pass it in the page options to retrieve the following page\n\n```elixir\nnext_offset = page.offset + page.limit\nsecond_page = Domain.list_posts!(page: [limit: 2, offset: next_offset])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n      title: \"post 3\",\n      text: \"text 3\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n      title: \"post 4\",\n      text: \"text 4\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 2,\n  offset: 2,\n  count: nil,\n  rerun: {#Ash.Query , [authorize?: true, reuse_values?: false, return_query?: false]},\n  more?: true\n}\n```\n\nIf you have the current page in memory, you can also use `Ash.page!/2` to navigate between pages.\n\n```elixir\nlast_page = Ash.page!(second_page, :next)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"b0b20225-17f0-4bd1-8bd1-681e63ee26a8\",\n      title: \"post 5\",\n      text: \"text 5\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 2,\n  offset: 4,\n  count: nil,\n  rerun: {#Ash.Query , [authorize?: true, reuse_values?: false, return_query?: false]},\n  more?: false\n}\n```\n\nAnd since we had 5 posts, this should be the last page:\n\n```elixir\nlast_page.more?\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nfalse\n```","ref":"pagination.html#retrieving-the-next-page"},{"type":"extras","title":"Keyset pagination - Pagination","doc":"When using keyset pagination, a `%Ash.Page.Keyset{}` struct is returned from read actions.\n\n```elixir\npage = Domain.list_posts_with_keyset!(page: [limit: 2])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Keyset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      title: \"post 1\",\n      text: \"text 1\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n      title: \"post 2\",\n      text: \"text 2\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  count: nil,\n  before: nil,\n  after: nil,\n  limit: 2,\n  rerun: {#Ash.Query , [authorize?: true, reuse_values?: false, return_query?: false]},\n  more?: true\n}\n```\n\n`results` and `more?` work in the same way as offset pagination\n\n```elixir\npage.results\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #Post ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n    title: \"post 1\",\n    text: \"text 1\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Post ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"78cd10f0-a509-4602-861f-24652c68d54b\",\n    title: \"post 2\",\n    text: \"text 2\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"pagination.html#keyset-pagination-1"},{"type":"extras","title":"Retrieving the next page - Pagination","doc":"To retrieve the next page, you have to pass the keyset of the last record in the current page. The keyset is stored in `record.__metadata__.keyset`.\n\n```elixir\nlast_keyset =\n  page.results\n  |> List.last()\n  |> Map.get(:__metadata__)\n  |> Map.get(:keyset)\n\nsecond_page = Domain.list_posts_with_keyset!(page: [limit: 2, after: last_keyset])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Keyset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n      title: \"post 3\",\n      text: \"text 3\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n      title: \"post 4\",\n      text: \"text 4\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  count: nil,\n  before: nil,\n  after: \"g2wAAAACbQAAAAZwb3N0IDJtAAAAJDc4Y2QxMGYwLWE1MDktNDYwMi04NjFmLTI0NjUyYzY4ZDU0Ymo=\",\n  limit: 2,\n  rerun: {#Ash.Query \n\n```\nfalse\n```","ref":"pagination.html#retrieving-the-next-page-1"},{"type":"extras","title":"Actions supporting both offset and keyset pagination - Pagination","doc":"If an action supports both offset and keyset pagination (e.g. default read actions), offset pagination is used by default when page options only contain `limit`. However, the records will have the keyset in the metadata, so keyset pagination can be performed on next pages.\n\n```elixir\n%Ash.Page.Offset{results: [_, last]} = Domain.list_posts!(page: [limit: 2])\n\nDomain.list_posts!(page: [limit: 2, after: last.__metadata__.keyset])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Keyset{\n  results: [\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"ab8e9909-2a6c-42d7-bae9-09fad4356ea4\",\n      title: \"post 3\",\n      text: \"text 3\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Post ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"91d639c2-4a2c-4931-b446-543e118644f1\",\n      title: \"post 4\",\n      text: \"text 4\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  count: nil,\n  before: nil,\n  after: \"g2wAAAACbQAAAAZwb3N0IDJtAAAAJDc4Y2QxMGYwLWE1MDktNDYwMi04NjFmLTI0NjUyYzY4ZDU0Ymo=\",\n  limit: 2,\n  rerun: {#Ash.Query<\n     resource: Post,\n     sort: [title: :asc],\n     select: [:id, :title, :text],\n     page: [\n       limit: 2,\n       after: \"g2wAAAACbQAAAAZwb3N0IDJtAAAAJDc4Y2QxMGYwLWE1MDktNDYwMi04NjFmLTI0NjUyYzY4ZDU0Ymo=\"\n     ]\n   >, [authorize?: true, reuse_values?: false, return_query?: false]},\n  more?: true\n}\n```","ref":"pagination.html#actions-supporting-both-offset-and-keyset-pagination"},{"type":"extras","title":"Retrieving count - Pagination","doc":"Both `%Ash.Page.Offset{}` and `%Ash.Page.Keyset{}` have a `count` field that contains the total count of the items that are being paginated when `count: true` is passed in the page options.\n\n```elixir\npage = Domain.list_posts!(page: [limit: 2, count: true])\n\npage.count\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n5\n```","ref":"pagination.html#retrieving-count"},{"type":"extras","title":"Relationship pagination - Pagination","doc":"To paginate a relationship, pass a query customized with the page options to the load statement. This works both on paginated and unpaginated root data, and relationships can load arbitrarily nested paginated relationships.\n\n```elixir\npaginated_comments =\n  Comment\n  |> Ash.Query.page(limit: 2)\n\nfirst_post =\n  Domain.list_posts!(load: [comments: paginated_comments])\n  |> List.first()\n\nfirst_post.comments\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #Comment ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"6cdea87b-cb69-4dc5-9ff3-54fb46bd70b0\",\n      text: \"comment 1\",\n      created_at: ~U[2024-05-28 21:32:59.013913Z],\n      post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Comment ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"c653e92c-fe2f-4011-84c8-ace28ebbb207\",\n      text: \"comment 2\",\n      created_at: ~U[2024-05-28 21:32:59.021204Z],\n      post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 2,\n  offset: 0,\n  count: nil,\n  rerun: {#Ash.Query ,\n   [authorize?: true, actor: nil, tracer: []]},\n  more?: true\n}\n```\n\nYou can use all the methods describe above to navigate relationship pages and retrieve their count\n\n```elixir\nsecond = Ash.page!(first_post.comments, :next)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #Comment ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"aa207735-0a02-4b51-b5f6-69564a2a6365\",\n      text: \"comment 3\",\n      created_at: ~U[2024-05-28 21:32:59.022890Z],\n      post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    #Comment ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"09f9cdfe-5a88-4f6a-a8d9-2f8aa312efb8\",\n      text: \"comment 4\",\n      created_at: ~U[2024-05-28 21:32:59.024526Z],\n      post_id: \"6eb22ea7-184c-4cae-9054-0d1a0474db61\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 2,\n  offset: 2,\n  count: nil,\n  rerun: {#Ash.Query , [reuse_values?: false, return_query?: false, authorize?: true, actor: nil, tracer: []]},\n  more?: true\n}\n```","ref":"pagination.html#relationship-pagination-1"},{"type":"extras","title":"Combination Queries","doc":"# Combination Queries\n\nAsh Framework provides a powerful feature called \"combination queries\" that allows you to combine multiple queries into a single result set, giving you the ability to create complex data retrieval patterns with minimal effort. For SQL data-layers, this feature is implemented using SQL's UNION, INTERSECT, and EXCEPT operations.","ref":"combination-queries.html"},{"type":"extras","title":"Overview - Combination Queries","doc":"Combination queries let you:\n\n- Combine multiple distinct queries into a single result set\n- Apply different filters, sorting, limits, and calculations to each subquery\n- Use operations like union, intersection, and exclusion to define how results should be combined\n- Create complex composite queries that would otherwise require multiple separate database calls","ref":"combination-queries.html#overview"},{"type":"extras","title":"Syntax - Combination Queries","doc":"To use combination queries, you'll work with the following functions:\n\n```elixir\nAsh.Query.combination_of(query, combinations)\n```\n\nWhere `combinations` is a list of combination specifications starting with a base query, followed by additional operations:\n\n- `Ash.Query.Combination.base/1`: The starting point for your combined query\n- `Ash.Query.Combination.union/1`: Combine with the previous results, removing duplicates\n- `Ash.Query.Combination.union_all/1`: Combine with the previous results, keeping duplicates\n- `Ash.Query.Combination.intersect/1`: Keep only records that appear in both the previous results and this query\n- `Ash.Query.Combination.except/1`: Remove records from the previous results that appear in this query","ref":"combination-queries.html#syntax"},{"type":"extras","title":"Basic Example - Combination Queries","doc":"Here's a simple example that combines users who meet different criteria:\n\n```elixir\nread :best_and_worst_users do\n  description \"\"\"\n  Returns the top 10 active users who are not on a losing streak\n  (sorted by score descending) and the bottom 10 active users who are not on a\n  winning streak (sorted by score ascending)\n  \"\"\"\n\n  filter expr(active == true)\n\n  prepare fn query, _ ->\n    Ash.Query.combination_of(query, [\n      # Must always begin with a base combination\n      Ash.Query.Combination.base(\n        filter: expr(not(on_a_losing_streak)),\n        sort: [score: :desc],\n        limit: 10\n      ),\n      Ash.Query.Combination.union(\n        filter: expr(not(on_a_winning_streak)),\n        sort: [score: :asc],\n        limit: 10\n      )\n    ])\n  end\nend\n```\n\nThis query would return:\n- The top 10 active users who are not on a losing streak (sorted by score descending)\n- Union with the bottom 10 active users who are not on a winning streak (sorted by score ascending)","ref":"combination-queries.html#basic-example"},{"type":"extras","title":"Using Calculations in Combinations - Combination Queries","doc":"One of the most powerful features of combination queries is the ability to create calculations that can be referenced across the combinations:\n\n```elixir\nquery = \"fred\"\n\nUser\n|> Ash.Query.filter(active == true)\n|> Ash.Query.combination_of([\n  Ash.Query.Combination.base(\n    filter: expr(trigram_similarity(user_name, ^query) >= 0.5),\n    calculations: %{\n      match_score: calc(trigram_similarity(user_name, ^query), type: :float)\n    },\n    sort: [\n      {calc(trigram_similarity(user_name, ^query), type: :float), :desc}\n    ],\n    limit: 10\n  ),\n  Ash.Query.Combination.union(\n    filter: expr(trigram_similarity(email, ^query) >= 0.5),\n    calculations: %{\n      match_score: calc(trigram_similarity(email, ^query), type: :float)\n    },\n    sort: [\n      {calc(trigram_similarity(email, ^query), type: :float), :desc}\n    ],\n    limit: 10\n  )\n])\n|> Ash.read!()\n```\n\nWhen you add calculations to a combination query, they behave differently depending on the name of the calculation. If the name matches the name of an attribute, calculation or aggregate on the resource, then the value is placed in that key. Otherwise, it will be placed into the `calculations` key.\n\nThis example searches for users where either their name or email matches \"fred\" with a similarity score of at least 0.5, and returns the top 10 matches of each type sorted by their match score.","ref":"combination-queries.html#using-calculations-in-combinations"},{"type":"extras","title":"Accessing Combination Values - Combination Queries","doc":"To access values from combination queries in your main query, use the `combinations/1` function in your expressions:\n\n```elixir\nUser\n|> Ash.Query.combination_of([\n  Ash.Query.Combination.base(\n    filter: expr(organization.name == \"bar\"),\n    calculations: %{\n      domain: calc(\"bar\", type: :string),\n      full_name: calc(name <> \"@bar\", type: :string)\n    }\n  ),\n  Ash.Query.Combination.union_all(\n    filter: expr(organization.name == \"baz\"),\n    calculations: %{\n      domain: calc(\"baz\", type: :string),\n      full_name: calc(name <> \"@baz\", type: :string)\n    }\n  )\n])\n|> Ash.Query.calculate(:email_domain, :string, expr(^combinations(:domain)))\n|> Ash.Query.calculate(:display_name, :string, expr(^combinations(:full_name)))\n|> Ash.read!()\n```\n\nIn this example, the `combinations(:domain)` and `combinations(:full_name)` references allow the outer query to access the calculation values from the inner combinations.","ref":"combination-queries.html#accessing-combination-values"},{"type":"extras","title":"Sorting and Distinct Operations - Combination Queries","doc":"You can sort and filter the combined results using the calculations from your combinations:\n\n```elixir\nUser\n|> Ash.Query.combination_of([\n  Ash.Query.Combination.base(calculations: %{sort_order: calc(3, type: :integer)}),\n  Ash.Query.Combination.union_all(\n    filter: expr(name == \"alice\"),\n    calculations: %{sort_order: calc(1, type: :integer)}\n  ),\n  Ash.Query.Combination.union_all(\n    filter: expr(name == \"john\"),\n    calculations: %{sort_order: calc(2, type: :integer)}\n  )\n])\n|> Ash.Query.sort([{calc(^combinations(:sort_order)), :asc}, {:name, :asc}])\n|> Ash.Query.distinct(:name)\n|> Ash.read!()\n```\n\nThis will return results in the order: Alice, John, and then all other users, thanks to the custom sort_order calculation.","ref":"combination-queries.html#sorting-and-distinct-operations"},{"type":"extras","title":"Important Rules and Limitations - Combination Queries","doc":"1. **This is an internal power tool**: No public interfaces like `AshJsonApi`/`AshGraphql` will be\n    updated to allow this sort of query to be built \"from the outside\". It is designed to be implemented\n    within an action, \"under the hood\".\n\n2. **Base Combination Required**: Your list of combinations must always start with `Ash.Query.Combination.base/1`.\n\n3. **Field Consistency**: All combinations must produce the same set of fields. This means:\n   - If one combination has a calculation, all combinations need that calculation\n   - Select statements should be consistent across combinations\n   - If a calculation added to a combination has the same name as an attribute, then it will\n     be used by `combinations(:that_field)`, allowing for combinations to \"override\" attribute\n     values.\n\n4. **Primary Keys**: When adding runtime calculations or loading related data with `Ash.Query.load/2`, all fieldsets must include the primary key of the resource. If this is not the case, the query will fail.\n\n5. **Type Specification**: When referencing calculation values with `combinations/1`, the calculation must have been added with a specified type on the `base` query at a minimum:\n   ```elixir\n   # Correct - type is specified\n   calc(expression, type: :string)\n\n   # Incorrect - will raise an error when referenced\n   calc(expression)\n   ```","ref":"combination-queries.html#important-rules-and-limitations"},{"type":"extras","title":"Data Layer Support - Combination Queries","doc":"Combination queries depend on data layer support. The implementation in this release includes support for ETS data layer, with implementation for SQL and Postgres to be added in future releases.","ref":"combination-queries.html#data-layer-support"},{"type":"extras","title":"Performance Considerations - Combination Queries","doc":"Combination queries can be more efficient than multiple separate queries, especially when:\n\n- You need to apply complex ordering or pagination to combined datasets\n- You want to deduplicate results across multiple selection criteria\n- You need to perform operations like intersection or exclusion between sets\n\nHowever, be mindful that complex combinations can generate equally complex SQL queries, so monitor performance in production scenarios.","ref":"combination-queries.html#performance-considerations"},{"type":"extras","title":"Practical Examples - Combination Queries","doc":"","ref":"combination-queries.html#practical-examples"},{"type":"extras","title":"Example 1: Search across multiple fields - Combination Queries","doc":"```elixir\nPost\n|> Ash.Query.combination_of([\n  Ash.Query.Combination.base(\n    filter: expr(ilike(title, ^(\"%\" <> search_term <> \"%\"))),\n    calculations: %{match_type: calc(\"title\", type: :string)},\n    sort: [published_at: :desc],\n    limit: 10\n  ),\n  Ash.Query.Combination.union(\n    filter: expr(ilike(body, ^(\"%\" <> search_term <> \"%\"))),\n    calculations: %{match_type: calc(\"body\", type: :string)},\n    sort: [published_at: :desc],\n    limit: 10\n  )\n])\n|> Ash.Query.sort([published_at: :desc])\n|> Ash.Query.calculate(:matched_in, :string, expr(^combinations(:match_type)))\n|> Ash.read!()\n```","ref":"combination-queries.html#example-1-search-across-multiple-fields"},{"type":"extras","title":"Example 2: Complex filtering with intersection - Combination Queries","doc":"```elixir\nUser\n|> Ash.Query.combination_of([\n  Ash.Query.Combination.base(filter: expr(role == \"admin\")),\n  Ash.Query.Combination.intersect(filter: expr(last_login > ^one_month_ago))\n])\n|> Ash.read!()\n```\n\nThis returns all admin users who have logged in within the last month.","ref":"combination-queries.html#example-2-complex-filtering-with-intersection"},{"type":"extras","title":"Summary - Combination Queries","doc":"Combination queries provide a powerful tool for creating complex data retrieval patterns in Ash. By combining multiple queries with different filters, sorts, and calculations, you can build sophisticated interfaces that would otherwise require multiple database queries and application-level merging of results.\n\nThis feature is particularly valuable for search interfaces, reporting tools, and anywhere you need to blend data from different filter conditions in a single, cohesive result set.","ref":"combination-queries.html#summary"},{"type":"extras","title":"Timeouts","doc":"# Timeouts","ref":"timeouts.html"},{"type":"extras","title":"Ways to Specify Timeouts - Timeouts","doc":"You have a few options.\n\nYou can specify a timeout when you call an action. This takes the highest precedence.\n\n```elixir\nAsh.read!(query, timeout: :timer.seconds(30))\n```\n\nYou can specify one using `Ash.Changeset.timeout/2` or `Ash.Query.timeout/2`. This can be useful if you want to conditionally set a timeout based on the details of the request. For example, you might do something like this:\n\n```elixir\n# in your resource\ndefmodule MyApp.SetReportTimeout do\n  use Ash.Resource.Preparation\n\n  def prepare(query, _, _) do\n    if Ash.Query.get_argument(query, :full_report) do\n      Ash.Query.timeout(query, :timer.minutes(3))\n    else\n      Ash.Query.timeout(query, :timer.minutes(1))\n    end\n  end\nend\n\nactions do\n  read :report_items do\n    argument :full_report, :boolean, default: false\n\n    prepare MyApp.SetReportTimeout\n  end\nend\n```\n\nYou can also specify a default timeout on your domain modules.\n\n```elixir\nexecution do\n  timeout :timer.seconds(30) # the default is `:infinity`\nend\n```\n\nKeep in mind, you can't specify timeouts in a before_action or after_action hook, because at that point you are already \"within\" the code that should have a timeout applied.\n\n>","ref":"timeouts.html#ways-to-specify-timeouts"},{"type":"extras","title":"timeouts use processes - Timeouts","doc":">\n> Timeouts are implemented using processes. This means that potentially large query results will be copied across processes. Because of this, specifying timeouts globally or for things that you don't suspect would ever exceed that timeout is not recommended.","ref":"timeouts.html#timeouts-use-processes"},{"type":"extras","title":"How are timeouts handled? - Timeouts","doc":"Timeouts in Ash work a bit differently than other tools. The following considerations must be taken into account:\n\n1. If you run a resource action in a transaction, then the timeout applies to the entire transaction.\n2. If the resource action you are running, and any of its `touches_resources` is _already in a transaction_ then the timeout is ignored, as the outer transaction is handling the timeout.\n3. If the resource is not in a transaction, and supports async execution (ash_postgres does), then everything is run in a task and awaited with the provided timeout.\n4. If the data layer of the resource does not support timeouts, or async execution then timeouts are **ignored**.","ref":"timeouts.html#how-are-timeouts-handled"},{"type":"extras","title":"Multitenancy","doc":"# Multitenancy\n\nMultitenancy is the splitting up your data into discrete areas, typically by customer. One of the most common examples of this, is the idea of splitting up a postgres database into \"schemas\" one for each customer that you have. Then, when making any queries, you ensure to always specify the \"schema\" you are querying, and you never need to worry about data crossing over between customers. The biggest benefits of this kind of strategy are the simplification of authorization logic, and better performance. Instead of all queries from all customers needing to use the same large table, they are each instead all using their own smaller tables. Another benefit is that it is much easier to delete a single customer's data on request.\n\nIn Ash, there are two primary strategies for implementing multitenancy. The first (and simplest) works for any data layer that supports filtering, and requires very little maintenance/mental overhead. It is done via expecting a given attribute to line up with the `tenant`, and is called `:attribute`. The second, is based on the data layer backing your resource, and is called `:context`. For information on context based multitenancy, see the documentation of your data layer. For example, `AshPostgres` uses postgres schemas. While the `:attribute` strategy is simple to implement, it also offers fewer advantages, primarily acting as another way to ensure your data is filtered to the correct tenant.","ref":"multitenancy.html"},{"type":"extras","title":"Attribute Multitenancy - Multitenancy","doc":"```elixir\ndefmodule MyApp.Users do\n  use Ash.Resource, ...\n\n  multitenancy do\n    strategy :attribute\n    attribute :organization_id\n  end\n\n  ...\n\n  relationships do\n    belongs_to :organization, MyApp.Organization\n  end\nend\n```\n\nIn this case, if you were to try to run a query without specifying a tenant, you would get an error telling you that the tenant is required.\n\nSetting the tenant is done via `Ash.Query.set_tenant/2` and `Ash.Changeset.set_tenant/2`. If you are using a [code interface](/documentation/topics/resources/code-interfaces.md), you can pass `tenant: ` in the options list (the final parameter). If you are using an extension, such as `AshJsonApi` or `AshGraphql` the method of setting tenant context is explained in that extension's documentation.\n\nExample usage of the above:\n\n```elixir\n# Error when not setting a tenant\nMyApp.Users\n|> Ash.Query.filter(name == \"fred\")\n|> Ash.read!()\n** (Ash.Error.Invalid)\n\n* \"Queries against the Helpdesk.Accounts.User resource require a tenant to be specified\"\n    (ash 1.22.0) lib/ash/domain/domain.ex:944: Ash.Domain.unwrap_or_raise!/2\n\n# Automatically filtering by `organization_id == 1`\nMyApp.Users\n|> Ash.Query.filter(name == \"fred\")\n|> Ash.Query.set_tenant(1)\n|> Ash.read!()\n\n[...]\n\n# Automatically setting `organization_id` to `1`\nMyApp.Users\n|> Ash.Changeset.for_create(:create, %{name: \"fred\"})\n|> Ash.Changeset.set_tenant(1)\n|> Ash.create!()\n\n%MyApp.User{organization_id: 1}\n\n# Setting tenant with a code interface that exposes `list_all` as a read action\nMyApp.Users.list_all(tenant: 1)\n```\n\nIf you want to enable running queries _without_ a tenant as well as queries with a tenant, the `global?` option supports this. You will likely need to incorporate this ability into any authorization rules though, to ensure that users from one tenant can't access other tenant's data.\n\n```elixir\nmultitenancy do\n  strategy :attribute\n  attribute :organization_id\n  global? true\nend\n```\n\nYou can also provide the `parse_attribute?` option if the tenant being set doesn't exactly match the attribute value, e.g the tenant is `org_10` and the attribute is `organization_id`, which requires just `10`.","ref":"multitenancy.html#attribute-multitenancy"},{"type":"extras","title":"Tenant-Aware Identities - Multitenancy","doc":"When using identities in a multitenant resource, the tenant attribute is automatically included in the uniqueness constraints. This means that the same identity value can exist across different tenants, but must be unique within a single tenant. For example, if you have a `User` resource with an email identity, users in different organizations could have the same email address.\n\nIf you need an identity to be globally unique across all tenants (like a global user email system), you can set `all_tenants?: true` on the identity.\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource, ...\n\n  multitenancy do\n    strategy :attribute\n    attribute :organization_id\n  end\n\n  identities do\n    # This email must be unique within a tenant\n    identity :tenant_scoped_email, [:email]\n\n    # This username must be unique across all tenants\n    identity :global_username, [:username], all_tenants?: true\n  end\nend\n```\n\nExample implications:\n\n```elixir\n# These are valid because they're in different tenants\nUser\n|> Ash.Changeset.for_create(:create, %{email: \"fred@example.com\"})\n|> Ash.Changeset.set_tenant(1)\n|> Ash.create!()\n\nUser\n|> Ash.Changeset.for_create(:create, %{email: \"fred@example.com\"})\n|> Ash.Changeset.set_tenant(2)\n|> Ash.create!()\n\n# This would fail because usernames are global\nUser\n|> Ash.Changeset.for_create(:create, %{username: \"fred\"})\n|> Ash.Changeset.set_tenant(1)\n|> Ash.create!()\n\nUser\n|> Ash.Changeset.for_create(:create, %{username: \"fred\"})\n|> Ash.Changeset.set_tenant(2)\n|> Ash.create!() # Error: username already taken\n```","ref":"multitenancy.html#tenant-aware-identities"},{"type":"extras","title":"Context Multitenancy - Multitenancy","doc":"Context multitenancy allows for the data layer to dictate how multitenancy works. For example, a csv data layer might implement multitenancy via saving the file with different suffixes, or an API wrapping data layer might use different subdomains for the tenant.\n\nFor `AshPostgres` context multitenancy, which uses postgres schemas and is referred to ash \"Schema Based Multitenancy\", see the [guide](https://hexdocs.pm/ash_postgres/schema-based-multitenancy.html)","ref":"multitenancy.html#context-multitenancy"},{"type":"extras","title":"Possible Values for tenant - Multitenancy","doc":"By default, the tenant value is passed directly to the relevant implementation. For example, if you are using schema multitenancy with `ash_postgres`, you might provide a schema like `organization.subdomain`. In Ash, a tenant should be identifiable by a single value, like an integer or a string.\n\nYou can use the `Ash.ToTenant` protocol to automatically convert values into this simple value. The example below will allow you to use the same organization everywhere, and have it automatically converted into the correct schema for postgres, and the correct id for attribute-based multitenant resources. You can use this without looking up the relevant record as well, as long as the required fields used in your protocol are present.\n\n```elixir\nAsh.Changeset.for_create(..., tenant: %MyApp.Organization{id: id})\n```\n\n```elixir\n# in Organization resource\n\ndefimpl Ash.ToTenant do\n  def to_tenant(%{id: id}, resource) do\n    if Ash.Resource.Info.data_layer(resource) == AshPostgres.DataLayer\n      && Ash.Resource.Info.multitenancy_strategy(resource) == :context do\n      \"org_#{id}\"\n    else\n      id\n    end\n  end\nend\n```\n\nThis allows you to pass an `%Organization{}` or an organization_id around, and have that `organization_id` properly used with attribute and context-based multitenancy.","ref":"multitenancy.html#possible-values-for-tenant"},{"type":"extras","title":"Writing Extensions","doc":"# Writing Extensions\n\nExtensions allow you to make powerful modifications to DSL entities. If you are using `AshPostgres`, `AshGraphql` or `AshJsonApi`, they are all integrated into Ash using extensions. In this guide we will build a simple extension for `Ash.Resource` that adds timestamps to your resource. We'll also show some simple patterns that can help ensure that all of your resources are using your extension.","ref":"writing-extensions.html"},{"type":"extras","title":"Creating an extension - Writing Extensions","doc":"What we call an \"extension\" is typically one or more `Spark.Dsl.Extension`, and then any additional code that is used by that extension. For example, `AshGraphql` has a domain extension called `AshGraphql.Domain`, a resource extension called `AshGraphql.Resource`, and code to connect a GraphQL schema to your resources.","ref":"writing-extensions.html#creating-an-extension"},{"type":"extras","title":"DSL Extension - Writing Extensions","doc":"Here we create a DSL extension called `MyApp.Extensions.Base`, and configure a single transformer, called `MyApp.Extensions.Base.AddTimestamps`\n\n```elixir\ndefmodule MyApp.Extensions.Base do\n  use Spark.Dsl.Extension, transformers: [MyApp.Extensions.Base.AddTimestamps]\nend\n```","ref":"writing-extensions.html#dsl-extension"},{"type":"extras","title":"Transformers - Writing Extensions","doc":"Transformers are all run serially against a map of data called `dsl_state`, which is the data structure that we build as we use the DSL. For example:\n\n```elixir\nattributes do\n  attribute :name, :string\nend\n```\n\nWould, under the hood, look something like this:\n\n```elixir\n%{\n  [:attributes] => %{entities: [\n      %Ash.Resource.Attribute{name: :name, type: :string}\n    ]\n  },\n  ...\n}\n```\n\nA transformer exposes `transform/1`, which takes the `dsl_state` and returns either `{:ok, dsl_state}` or `{:error, error}`\n\n```elixir\ndefmodule MyApp.Extensions.Base.AddTimestamps do\n  use Spark.Dsl.Transformer\n  alias Spark.Dsl.Transformer\n\n  def transform(dsl_state) do\n    dsl_state\n    # Ash.Resource.Builder has utilities for extending resources\n    |> Ash.Resource.Builder.add_new_create_timestamp(:inserted_at)\n    |> Ash.Resource.Builder.add_new_update_timestamp(:updated_at)\n  end\nend\n```\n\nThis transformer builds adds a `create_timestamp` called `:inserted_at` and an `update_timestamp` called `:updated_at`, unless they already exist.","ref":"writing-extensions.html#transformers"},{"type":"extras","title":"Make the extension configurable - Writing Extensions","doc":"So far we've covered transformers, and using them to modify resources, but now lets say we want to make this behavior opt-out. Perhaps certain resources really shouldn't have timestamps, but we want it to be the default. Lets add a \"DSL Section\" to our extension.\n\n```elixir\ndefmodule MyApp.Extensions.Base do\n  @base %Spark.Dsl.Section{\n    name: :base,\n    describe: \"\"\"\n    Configure the behavior of our base extension.\n    \"\"\",\n    examples: [\n      \"\"\"\n      base do\n        timestamps? false\n      end\n      \"\"\"\n    ],\n    schema: [\n      timestamps?: [\n        type: :boolean,\n        doc: \"Set to false to skip adding timestamps\",\n        default: true\n      ]\n    ]\n  }\n\n\n\n  use Spark.Dsl.Extension,\n    transformers: [MyApp.Extensions.Base.AddTimestamps],\n    sections: [@base]\nend\n\ndefmodule MyApp.Extensions.Base.Info do\n  use Spark.InfoGenerator, extension: MyApp.Extensions.Base, sections: [:base]\n\n  # This will define `base_timestamps?/1`.\nend\n```\n\nNow we can use this configuration in our transformer, like so:\n\n```elixir\n  def transform(dsl_state) do\n    # Introspection functions can take a `dsl_state` *or* a module\n    if MyApp.Extensions.Base.Info.base_timestamps?(dsl_state) do\n      dsl_state\n      |> Ash.Resource.Builder.add_new_create_timestamp(:inserted_at)\n      |> Ash.Resource.Builder.add_new_update_timestamp(:updated_at)\n    else\n      {:ok, dsl_state}\n    end\n  end\n```\n\nAnd now we have a configurable base extension! For more information on writing DSLs, see `Spark`.  `Spark` is still lacking in documentation, unfortunately, as its something that mostly the adventurous/power users work with, and they often learn by way of examples, looking at `Ash` extensions. We would like to rectify this in the future. Please reach out if you're interested in assisting with that effort!","ref":"writing-extensions.html#make-the-extension-configurable"},{"type":"extras","title":"Ordering of transformers - Writing Extensions","doc":"In this case, this transformer can run in any order. However, as we start adding transformers and/or modify the behavior of this one, we may need to ensure that our transformer runs before or after specific transformers. As of the writing of this guide, the best way to look at the list of transformers is to look at the source of the extension, and see what transformers it has and what they do. The [Resource DSL](https://github.com/ash-project/ash/blob/main/lib/ash/resource/dsl.ex) for example.\n\nIf you need to affect the ordering, you can define `before?/1` and `after?/1` in your transformer, i.e\n\n```elixir\n# I go after every transformer\ndef after?(_), do: true\n\n# except I go before `SomeOtherTransformer`\ndef before?(SomeOtherTransformer), do: true\ndef before?(_), do: false\n```","ref":"writing-extensions.html#ordering-of-transformers"},{"type":"extras","title":"Using your extension - Writing Extensions","doc":"Now it can be used in a resource:\n\n```elixir\ndefmodule MyApp.Tweet do\n  use Ash.Resource,\n    extensions: [MyApp.Extensions.Base]\n\n  base do\n    # And you can configure it like so\n    timestamps? false\n  end\nend\n```\n\nYour extension will automatically support autocompletion if using `ElixirLS`, showing inline documentation and type-aware auto complete as you type. For more, see [Development Utilities](/documentation/topics/development/development-utilities.md)","ref":"writing-extensions.html#using-your-extension"},{"type":"extras","title":"Base Resources - Writing Extensions","doc":"Base resources are a \"quick and easy\" way to ensure that all or a set of your resources use the same behavior.\nIf we want to ensure that all of our resources have timestamps, we could create a module like so:\n\n```elixir\ndefmodule MyApp.Resource do\n  defmacro __using__(opts) do\n    quote do\n      use Ash.Resource, unquote(opts)\n\n      attributes do\n        timestamps()\n      end\n    end\n  end\nend\n```\n\nThen we would configure that this base resource exists, with\n`config :my_app, base_resources: [MyApp.Resource]`\n\nNow in our resources instead of `use Ash.Resource`, we say `use MyApp.Resource`.\n\nUse `mix ash.gen.base_resource` to create one of these.:w\n\nThis pattern is relatively limited, good for simple things like making sure that all\nof our resources use a particular extension. See the section above on extensions for the\nideal way of extending resources.","ref":"writing-extensions.html#base-resources"},{"type":"extras","title":"Write Queries","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Write Queries\n\n```elixir\nMix.install([{:ash, \"~> 3.0\"}],\n  consolidate_protocols: false\n)\n\nApplication.put_env(:ash, :validate_domain_resource_inclusion?, false)\nApplication.put_env(:ash, :validate_domain_config_inclusion?, false)\n\nExUnit.start()\n```","ref":"write-queries.html"},{"type":"extras","title":"Introduction - Write Queries","doc":"Here we will show practical examples of using `Ash.Query`. To understand more about its capabilities, limitations, and design, see the module docs of `Ash.Query`.\n\nThis guide is here to provide a slew of examples, for more information on any given function or option please search the documentation. Please propose additions\nfor any useful patterns that are not demonstrated here!","ref":"write-queries.html#introduction"},{"type":"extras","title":"Setup - Write Queries","doc":"First, lets create some resources and some data to query.\n\n```elixir\ndefmodule MyApp.Posts do\n  use Ash.Domain\n\n  resources do\n    resource MyApp.Posts.Post\n    resource MyApp.Posts.Comment\n  end\nend\n\ndefmodule MyApp.Posts.Post do\n  use Ash.Resource,\n    domain: MyApp.Posts,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, :destroy, create: :*]\n  end\n  \n  attributes do\n    uuid_primary_key :id\n    \n    attribute :text, :string do\n      allow_nil? false\n      public? true\n    end\n  end\n\n  calculations do\n    calculate :text_length, :integer, expr(string_length(text))\n  end\n\n  aggregates do\n    count :count_of_comments, :comments\n  end\n\n  relationships do\n    has_many :comments, MyApp.Posts.Comment do\n      public? true\n    end\n  end\nend\n\ndefmodule MyApp.Posts.Comment do\n  use Ash.Resource,\n    domain: MyApp.Posts,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, :destroy, create: :*]\n  end\n  \n  attributes do\n    uuid_primary_key :id\n    attribute :text, :string do\n      allow_nil? false\n      public? true\n    end\n  end\n\n  relationships do\n    belongs_to :post, MyApp.Posts.Post do\n      public? true\n    end\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, MyApp.Posts.Comment, <<70, 79, 82, 49, 0, 0, 110, ...>>,\n [\n   Ash.Expr,\n   Ash.Resource.Dsl.Relationships.BelongsTo,\n   Ash.Resource.Dsl.Relationships.ManyToMany,\n   Ash.Resource.Dsl.Relationships.HasMany,\n   Ash.Resource.Dsl.Relationships.HasOne,\n   %{...}\n ]}\n```\n\n```elixir\n# Get rid of any existing comments/posts\nAsh.bulk_destroy!(MyApp.Posts.Comment, :destroy, %{})\nAsh.bulk_destroy!(MyApp.Posts.Post, :destroy, %{})\n\n# Create some posts\npost1 =\n  Ash.create!(MyApp.Posts.Post, %{text: \"First post about Ash!\"})\n\npost2 =\n  Ash.create!(MyApp.Posts.Post, %{text: \"Learning to write queries\"})\n\ncomment1 =\n  Ash.create!(MyApp.Posts.Comment, %{text: \"Great post!\", post_id: post1.id})\n\ncomment2 =\n  Ash.create!(MyApp.Posts.Comment, %{text: \"Very helpful!\", post_id: post1.id})\n\ncomment3 =\n  Ash.create!(MyApp.Posts.Comment, %{text: \"Thanks for the explanation\", post_id: post2.id})\n\n# Store the created records in module attributes for later use\nposts = [post1, post2]\ncomments = [comment1, comment2, comment3]\n\nIO.puts(\"\\nCreated #{length(posts)} posts and #{length(comments)} comments!\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n23:17:15.097 [debug] ETS: Destroying MyApp.Posts.Comment\n\n\n23:17:15.104 [debug] ETS: Destroying MyApp.Posts.Post\n\n\n23:17:15.110 [debug] Creating MyApp.Posts.Post:\n\n%{id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\", text: \"First post about Ash!\"}\n\n\n23:17:15.110 [debug] Creating MyApp.Posts.Post:\n\n%{id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\", text: \"Learning to write queries\"}\n\n\n23:17:15.111 [debug] Creating MyApp.Posts.Comment:\n\n%{\n  id: \"05863bdd-38eb-4e0e-9ff7-f23f65639ec3\",\n  text: \"Great post!\",\n  post_id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\"\n}\n\n\n23:17:15.111 [debug] Creating MyApp.Posts.Comment:\n\n%{\n  id: \"437b6966-2929-4e12-94cc-5807adf60c3e\",\n  text: \"Very helpful!\",\n  post_id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\"\n}\n\n\n23:17:15.111 [debug] Creating MyApp.Posts.Comment:\n\n%{\n  id: \"09aaffc4-bca7-4848-8b05-2d1ca11aba33\",\n  text: \"Thanks for the explanation\",\n  post_id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\"\n}\n\n\nCreated 2 posts and 3 comments!\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"write-queries.html#setup"},{"type":"extras","title":"Basic Queries - Write Queries","doc":"Let's start with some basic query examples. To use `Ash.Query.filter/2`, we'll need to\n`require Ash.Query`.\n\n```elixir\nrequire Ash.Query\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nAsh.Query\n```","ref":"write-queries.html#basic-queries"},{"type":"extras","title":"Read everything - Write Queries","doc":"```elixir\n# with a lot of data, you probably shouldn't do this\nAsh.read!(MyApp.Posts.Post)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    text: \"Learning to write queries\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"write-queries.html#read-everything"},{"type":"extras","title":"Count all comments - Write Queries","doc":"```elixir\nMyApp.Posts.Comment\n|> Ash.count!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n3\n```","ref":"write-queries.html#count-all-comments"},{"type":"extras","title":"Filtering - Write Queries","doc":"```elixir\nMyApp.Posts.Post\n|> Ash.Query.filter(id == ^post1.id)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nMyApp.Posts.Post\n# you can filter on calculations\n|> Ash.Query.filter(text_length == 25)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    text: \"Learning to write queries\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nMyApp.Posts.Post\n# you can filter on aggregates\n|> Ash.Query.filter(count_of_comments == 2)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: 2,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nMyApp.Posts.Post\n# use `filter_input` to filter based on user input\n# it only allows accessing public fields\n|> Ash.Query.filter_input(%{count_of_comments: %{eq: 2}})\n|> Ash.read!()\n```","ref":"write-queries.html#filtering"},{"type":"extras","title":"Sorting - Write Queries","doc":"```elixir\nMyApp.Posts.Post\n|> Ash.Query.sort(:text)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    text: \"Learning to write queries\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\n# Apply multiple sorts\nMyApp.Posts.Post\n|> Ash.Query.sort(text: :asc, count_of_comments: :desc)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: 2,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Post ,\n    count_of_comments: 1,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    text: \"Learning to write queries\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\n# use `sort_input` to sort based on user input\n# it only allows accessing public fields\nMyApp.Posts.Post\n|> Ash.Query.sort_input(\"text,-count_of_comments\")\n|> Ash.read!()\n```","ref":"write-queries.html#sorting"},{"type":"extras","title":"Distinct - Write Queries","doc":"```elixir\nMyApp.Posts.Comment\n# only one comment per post\n|> Ash.Query.distinct(:post_id)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Comment ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"09aaffc4-bca7-4848-8b05-2d1ca11aba33\",\n    text: \"Thanks for the explanation\",\n    post_id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Comment ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"05863bdd-38eb-4e0e-9ff7-f23f65639ec3\",\n    text: \"Great post!\",\n    post_id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nMyApp.Posts.Comment\n# only one comment per post_id & text combination\n|> Ash.Query.distinct([:post_id, :text])\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Comment ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"09aaffc4-bca7-4848-8b05-2d1ca11aba33\",\n    text: \"Thanks for the explanation\",\n    post_id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Comment ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"05863bdd-38eb-4e0e-9ff7-f23f65639ec3\",\n    text: \"Great post!\",\n    post_id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #MyApp.Posts.Comment ,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"437b6966-2929-4e12-94cc-5807adf60c3e\",\n    text: \"Very helpful!\",\n    post_id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"write-queries.html#distinct"},{"type":"extras","title":"Load calculations/aggregates - Write Queries","doc":"```elixir\nMyApp.Posts.Post\n|> Ash.Query.load([:count_of_comments, :text_length])\n|> Ash.read!()\n|> Enum.map(&Map.take(&1, [:text, :count_of_comments, :text_length]))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{text: \"Learning to write queries\", text_length: 25, count_of_comments: 1},\n  %{text: \"First post about Ash!\", text_length: 21, count_of_comments: 2}\n]\n```","ref":"write-queries.html#load-calculations-aggregates"},{"type":"extras","title":"Load relationships - Write Queries","doc":"```elixir\nMyApp.Posts.Post\n|> Ash.Query.load(:comments)\n|> Ash.read!()\n|> Enum.at(0)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#MyApp.Posts.Post ,\n  count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n  comments: [\n    #MyApp.Posts.Comment ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"09aaffc4-bca7-4848-8b05-2d1ca11aba33\",\n      text: \"Thanks for the explanation\",\n      post_id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  __meta__: #Ecto.Schema.Metadata<:loaded>,\n  id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n  text: \"Learning to write queries\",\n  aggregates: %{},\n  calculations: %{},\n  ...\n>\n```","ref":"write-queries.html#load-relationships"},{"type":"extras","title":"Limit & Offset - Write Queries","doc":"```elixir\nMyApp.Posts.Post\n|> Ash.Query.limit(1)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n    text: \"Learning to write queries\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nMyApp.Posts.Post\n|> Ash.Query.offset(1)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #MyApp.Posts.Post ,\n    count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n    comments: #Ash.NotLoaded<:relationship, field: :comments>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n    text: \"First post about Ash!\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"write-queries.html#limit-offset"},{"type":"extras","title":"Pagination - Write Queries","doc":"```elixir\n# Offset Pagination\n\nMyApp.Posts.Post\n|> Ash.Query.page(limit: 1)\n|> Ash.read!()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #MyApp.Posts.Post ,\n      count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n      comments: #Ash.NotLoaded<:relationship, field: :comments>,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"2f22973b-f1cd-4d2d-b241-d7c53bf097d3\",\n      text: \"Learning to write queries\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 1,\n  offset: 0,\n  count: nil,\n  more?: true\n}\n```\n\n```elixir\n# Keyset pagination\n\nfirst_post = \n  MyApp.Posts.Post\n  # You can paginate using `Ash.Query.page/1`\n  |> Ash.Query.page(limit: 1)\n  |> Ash.read!()\n  |> Map.get(:results)\n  |> Enum.at(0)\n  \nMyApp.Posts.Post\n# Or using the `page` option\n|> Ash.read!(page: [limit: 1, after: first_post.__metadata__.keyset])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Keyset{\n  results: [\n    #MyApp.Posts.Post ,\n      count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n      comments: #Ash.NotLoaded<:relationship, field: :comments>,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n      text: \"First post about Ash!\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  count: nil,\n  before: nil,\n  after: \"g2wAAAABbQAAACQyZjIyOTczYi1mMWNkLTRkMmQtYjI0MS1kN2M1M2JmMDk3ZDNq\",\n  limit: 1,\n  more?: false\n}\n```\n\n```elixir\nMyApp.Posts.Post\n|> Ash.Query.page(limit: 1)\n|> Ash.read!()\n# you can ask for :next, :prev, :first, :last, or a page number\n|> Ash.page!(:next)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Ash.Page.Offset{\n  results: [\n    #MyApp.Posts.Post ,\n      count_of_comments: #Ash.NotLoaded<:aggregate, field: :count_of_comments>,\n      comments: #Ash.NotLoaded<:relationship, field: :comments>,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"eafcb80f-8c90-4c16-8a29-cf0c28964d9b\",\n      text: \"First post about Ash!\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >\n  ],\n  limit: 1,\n  offset: 1,\n  count: nil,\n  more?: false\n}\n```","ref":"write-queries.html#pagination"},{"type":"extras","title":"Define Polymorphic Relationships","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Define Polymorphic Relationships\n\n```elixir\nMix.install([{:ash, \"~> 3.0\"}], consolidate_protocols: false)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"polymorphic-relationships.html"},{"type":"extras","title":"Introduction - Define Polymorphic Relationships","doc":"Something that comes up in more complex domains is the idea of \"polymorphic relationships\". For this example, we will use the concept of a `BankAccount`, which can be either a `SavingsAccount` or a `CheckingAccount`. All accounts have an `account_number` and `transactions`, but checkings & savings accounts might have their own specific information. For example, a `SavingsAccount` has an `interest_rate`, and a `CheckingAccount` has many `debit_card`s.\n\nAsh does not support polymorphic relationships defined _as relationships_, but you can accomplish similar functionality via [calculations](documentation/topics/resources/calculations.md) with the type `Ash.Type.Union`.\n\nFor this tutorial, we will have a dedicated resource called `BankAccount`. I suggest taking that approach, as many things down the road will be simplified. With that said, you don't necessarily need to do that when there is no commonalities between the types. Instead of setting up the polymorphism on the `BankAccount` resource, you would define relationships to `SavingsAccount` and `CheckingAccount` directly.\n\nThis tutorial is not attempting to illustrate good design of accounting systems. We make many concessions for the sake of the simplicity of our example.","ref":"polymorphic-relationships.html#introduction"},{"type":"extras","title":"Defining our Resources - Define Polymorphic Relationships","doc":"```elixir\ndefmodule BankAccount do\n  use Ash.Resource,\n    domain: Finance,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, :destroy, create: [:account_number, :type]]\n  end\n\n  attributes do\n    uuid_primary_key :id\n\n    attribute :account_number, :integer, allow_nil?: false\n    attribute :type, :atom, constraints: [one_of: [:checking, :savings]]\n  end\n\n  # calculations do\n  #   calculate :implementation, AccountImplementation, GetAccountImplementation do\n  #    allow_nil? false\n  #  end\n  # end\n\n  relationships do\n    has_one :checking_account, CheckingAccount\n    has_one :savings_account, SavingsAccount\n  end\nend\n\ndefmodule CheckingAccount do\n  use Ash.Resource,\n    domain: Finance,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, :destroy, create: [:bank_account_id]]\n  end\n\n  attributes do\n    uuid_primary_key :id\n  end\n\n  identities do\n    identity :unique_bank_account, [:bank_account_id], pre_check?: true\n  end\n\n  relationships do\n    belongs_to :bank_account, BankAccount do\n      allow_nil? false\n    end\n  end\nend\n\ndefmodule SavingsAccount do\n  use Ash.Resource,\n    domain: Finance,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, :destroy, create: [:bank_account_id]]\n  end\n\n\n  attributes do\n    uuid_primary_key :id\n  end\n\n  identities do\n    identity :unique_bank_account, [:bank_account_id], pre_check?: true\n  end\n\n  relationships do\n    belongs_to :bank_account, BankAccount do\n      allow_nil? false\n    end\n  end\nend\n\ndefmodule Finance do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource BankAccount\n    resource SavingsAccount\n    resource CheckingAccount\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Finance, <<70, 79, 82, 49, 0, 0, 44, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   %{opts: [], entities: [%Ash.Domain.Dsl.ResourceReference{...}, ...]}\n ]}\n```\n\nWe haven't implemented the polymorphic part yet, but lets create a few of the above resources to show how they relate. Below we create a `BankAccount` for checkings, and a `BankAccount` for savings, and connect them to their \"specific\" types, i.e `CheckingAccount` and `SavingsAccount`.\n\nWe load the data, you can see that one `BankAccount` has a `:checking_account` but no `:savings_account`. For the other, the opposite is the case.\n\n```elixir\nbank_account1 = Ash.create!(BankAccount, %{account_number: 1, type: :checking})\nbank_account2 = Ash.create!(BankAccount, %{account_number: 2, type: :savings})\nchecking_account = Ash.create!(CheckingAccount, %{bank_account_id: bank_account1.id})\nsavings_account = Ash.create!(SavingsAccount, %{bank_account_id: bank_account2.id})\n\n[bank_account1, bank_account2] |> Ash.load!([:checking_account, :savings_account])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #BankAccount ,\n    savings_account: nil,\n    checking_account: #CheckingAccount ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"60f585f6-f352-410e-8c09-09ae49448851\",\n      bank_account_id: \"21d27a6c-49b8-4984-a1b7-f2ef030626af\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"21d27a6c-49b8-4984-a1b7-f2ef030626af\",\n    account_number: 1,\n    type: :checking,\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #BankAccount ,\n    savings_account: #SavingsAccount ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"d2bcc1cc-d709-418d-a9ff-0d21fd7d667b\",\n      bank_account_id: \"4d8dc988-3e09-4efb-a643-d822013abfba\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    checking_account: nil,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"4d8dc988-3e09-4efb-a643-d822013abfba\",\n    account_number: 2,\n    type: :savings,\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"polymorphic-relationships.html#defining-our-resources"},{"type":"extras","title":"Defining our union type - Define Polymorphic Relationships","doc":"Below we define an `Ash.Type.NewType`. This allows defining a new type that is the combination of an existing type and custom constraints.\n\n```elixir\ndefmodule AccountImplementation do\n  use Ash.Type.NewType, subtype_of: :union, constraints: [\n    types: [\n      checking: [\n        type: :struct,\n        constraints: [instance_of: CheckingAccount]\n      ],\n      savings: [\n        type: :struct,\n        constraints: [instance_of: SavingsAccount]\n      ]\n    ]\n  ]\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, AccountImplementation, <<70, 79, 82, 49, 0, 0, 44, ...>>, :ok}\n```","ref":"polymorphic-relationships.html#defining-our-union-type"},{"type":"extras","title":"Defining the calculation - Define Polymorphic Relationships","doc":"Next, we'll define a calculation resolves to the specific type of any given account.\n\n```elixir\ndefmodule GetAccountImplementation do\n  use Ash.Resource.Calculation\n\n  def load(_, _, _) do\n    [:checking_account, :savings_account]\n  end\n\n  # This ensures that all attributes are selected by default\n  # on the related loads we depend on.\n  def strict_loads?, do: false\n\n  def calculate(records, _, _) do\n    Enum.map(records, fn record ->\n      cond do\n        record.checking_account ->\n          %Ash.Union{type: :checking_account, value: record.checking_account}\n\n        record.savings_account ->\n          %Ash.Union{type: :savings_account, value: record.savings_account}\n\n        true ->\n          nil\n      end\n    end)\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, GetAccountImplementation, <<70, 79, 82, 49, 0, 0, 13, ...>>, {:calculate, 3}}\n```","ref":"polymorphic-relationships.html#defining-the-calculation"},{"type":"extras","title":"Adding the calculation to our resource - Define Polymorphic Relationships","doc":"Finally, we'll add the calculation to our `BankAccount` resource!\n\nFor those following along with the LiveBook, go back up and uncomment the commented out calculation.\n\nNow we can load `:implementation` and see that, for one account, it resolves to a `CheckingAccount` and for the other it resolves to a `SavingsAccount`.\n\n```elixir\nbank_account1 = Ash.create!(BankAccount, %{account_number: 1, type: :checking})\nbank_account2 = Ash.create!(BankAccount, %{account_number: 2, type: :savings})\nchecking_account = Ash.create!(CheckingAccount, %{bank_account_id: bank_account1.id})\nsavings_account = Ash.create!(SavingsAccount, %{bank_account_id: bank_account2.id})\n\n[bank_account1, bank_account2] |> Ash.load!([:implementation])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #BankAccount ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"1d1a7b6c-dd08-4b8c-9769-2c1155a41a40\",\n      bank_account_id: \"ce370381-303e-49dc-9950-a05b5052e7f8\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    savings_account: #Ash.NotLoaded<:relationship, field: :savings_account>,\n    checking_account: #Ash.NotLoaded<:relationship, field: :checking_account>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"ce370381-303e-49dc-9950-a05b5052e7f8\",\n    account_number: 1,\n    type: :checking,\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #BankAccount ,\n      __meta__: #Ecto.Schema.Metadata<:loaded>,\n      id: \"e537116d-c5b8-414a-a350-fa9b2afe0a4e\",\n      bank_account_id: \"c4739158-2a1f-4de8-bccb-ee1d1ee9e602\",\n      aggregates: %{},\n      calculations: %{},\n      ...\n    >,\n    savings_account: #Ash.NotLoaded<:relationship, field: :savings_account>,\n    checking_account: #Ash.NotLoaded<:relationship, field: :checking_account>,\n    __meta__: #Ecto.Schema.Metadata<:loaded>,\n    id: \"c4739158-2a1f-4de8-bccb-ee1d1ee9e602\",\n    account_number: 2,\n    type: :savings,\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"polymorphic-relationships.html#adding-the-calculation-to-our-resource"},{"type":"extras","title":"Taking it further - Define Polymorphic Relationships","doc":"One of the best things about using `Ash.Type.Union` is how it is *integrated*. Every extension (provided by the Ash team) supports working with unions. For example:\n\n* [Working with Unions in AshPhoenix.Form](https://hexdocs.pm/ash_phoenix/union-forms.html)\n* [AshGraphql & AshJsonApi Unions](https://hexdocs.pm/ash_graphql/use-unions-with-graphql.html)\n\nYou can also synthesize filterable fields with calculations. For example, if you wanted to allow filtering `BankAccount` by `:interest_rate`, and that field only existed on `SavingsAccount`, you might have a calculation like this on `BankAccount`:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\ncalculate :interest_rate, :decimal, expr(\n  if type == :savings do\n    savings_account.interest_rate\n  else\n    0\n  end\n)\n```\n\nThis would allow usage like the following:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nBankAccount\n|> Ash.Query.filter(interest_rate > 0.01)\n|> Ash.read!()\n```","ref":"polymorphic-relationships.html#taking-it-further"},{"type":"extras","title":"Test Resources","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Test Resources\n\n```elixir\nApplication.put_env(:stream_data, :max_runs, 10)\nMix.install([{:ash, \"~> 3.0\"}, {:simple_sat, \"~> 0.1\"}],\n  consolidate_protocols: false\n)\n\nLogger.configure(level: :warning)\nExUnit.start()\n```","ref":"test-resources.html"},{"type":"extras","title":"Introduction - Test Resources","doc":"We recommend testing your resources *thoroughly*. Often, folks think that testing an `Ash.Resource` is \"testing the framework\", and in some very simple cases this may be true, like a simple create that just accepts a few attributes.\n\nHowever, testing has two primary roles:\n\n1. Confirming our understanding of the way that our application behaves *now*\n2. Ensuring that our application does not change in unintended ways *later*\n\nTo this end, we highly recommend writing tests even for your simple actions. A single test that confirms that, with simple inputs, the action returns what you expect, can be very powerful.\n\nAdditionally, Ash offers unique ways of testing individual components of our resources, similar to a unit test.\n\nWhile you don't necessarily need to follow all steps below, we show the various ways you may want to go about testing your resources.","ref":"test-resources.html#introduction"},{"type":"extras","title":"Testing Resources - Test Resources","doc":"* Add tests for action inputs using property testing\n* Add tests for calling action invocation using property testing\n* Add explicit tests for action inputs/invocation.\n* Add tests for our calculations\n* Test policies \"in isolation\" using `Ash.can?` (or `Domain.can_*`, provided by code interfaces)","ref":"test-resources.html#testing-resources"},{"type":"extras","title":"Examples - Test Resources","doc":"```elixir\ndefmodule User do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, create: [:admin?]]\n  end\n\n  attributes do\n    uuid_primary_key :id\n    attribute :admin?, :boolean do\n      allow_nil? false\n      default false\n    end\n  end\nend\n\ndefmodule Tweet do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets,\n    authorizers: [Ash.Policy.Authorizer]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :text, :string do\n      allow_nil? false\n      constraints max_length: 144\n      public? true\n    end\n\n    attribute :hidden?, :boolean do\n      allow_nil? false\n      default false\n      public? true\n    end\n  end\n\n  calculations do\n    calculate :tweet_length, :integer, expr(string_length(text))\n  end\n\n  relationships do\n    belongs_to :user, User, allow_nil?: false\n  end\n\n  actions do\n    defaults [:read, update: [:text]]\n\n    create :create do\n      primary? true\n      accept [:text]\n      change relate_actor(:user)\n    end\n  end\n\n  policies do\n    policy action_type(:read) do\n      description \"If a tweet is hidden, only the author can read it. Otherwise, anyone can.\"\n      authorize_if relates_to_actor_via(:user)\n      forbid_if expr(hidden? == true)\n      authorize_if always()\n    end\n\n    policy action_type(:create) do\n      description \"Anyone can create a tweet\"\n      authorize_if always()\n    end\n\n    policy action_type(:update) do\n      description \"Only an admin or the user who tweeted can edit their tweet\"\n      authorize_if actor_attribute_equals(:admin?, true)\n      authorize_if relates_to_actor_via(:user)\n    end\n  end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource Tweet do\n      define :create_tweet, action: :create, args: [:text]\n      define :update_tweet, action: :update, args: [:text]\n      define :list_tweets, action: :read\n    end\n\n    resource User do\n      define :create_user, action: :create\n    end\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 1, 110, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   %{opts: [], entities: [%Ash.Domain.Dsl.ResourceReference{...}, ...]}\n ]}\n```","ref":"test-resources.html#examples"},{"type":"extras","title":"Create a generator - Test Resources","doc":"See `Ash.Generator` documentation for more examples and docs.\n\n```elixir\ndefmodule Generator do\n  use Ash.Generator\n\n  def user(opts \\\\ []) do\n    changeset_generator(\n      User,\n      :create,\n      overrides: opts,\n      actor: opts[:actor]\n    ) \n  end\n\n  def tweet(opts \\\\ []) do\n    changeset_generator(\n      Tweet,\n      :create,\n      overrides: opts,\n      actor: opts[:actor]\n    ) \n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Generator, <<70, 79, 82, 49, 0, 0, 10, ...>>, {:tweet, 1}}\n```","ref":"test-resources.html#create-a-generator"},{"type":"extras","title":"Write some tests - Test Resources","doc":"```elixir\ndefmodule ActionInvocationTest do\n  use ExUnit.Case\n  import ExUnitProperties\n  import Generator\n\n  describe \"valid inputs\" do\n    # now if our action inputs are invalid when we think they should be valid, we will find out here\n    property \"accepts all valid input\" do\n      user = generate(user())\n\n      check all(input <- Ash.Generator.action_input(Tweet, :create)) do\n        {text, other_inputs} = Map.pop!(input, :text)\n\n        assert Domain.changeset_to_create_tweet(\n                 text,\n                 other_inputs,\n                 authorize?: false,\n                 actor: user\n               ).valid?\n      end\n    end\n\n    # same as the above, but actually call the action. This tests the underlying action implementation\n    # not just initial validation\n    property \"succeeds on all valid input\" do\n      user = generate(user())\n\n      check all(input <- Ash.Generator.action_input(Tweet, :create)) do\n        {text, other_inputs} = Map.pop!(input, :text)\n        Domain.create_tweet!(text, other_inputs, authorize?: false, actor: user)\n      end\n    end\n\n    test \"can tweet some specific text, in addition to any other valid inputs\" do\n      user = generate(user())\n\n      check all(\n              input <- Ash.Generator.action_input(Tweet, :create, %{text: \"some specific text\"})\n            ) do\n        {text, other_inputs} = Map.pop!(input, :text)\n        Domain.create_tweet!(text, other_inputs, actor: user)\n      end\n    end\n  end\n\n  describe \"authorization\" do\n    test \"allows a user to update their own tweet\" do\n      user = generate(user())\n      tweet = generate(tweet(text: \"Hello world!\", actor: user))\n\n      assert Domain.can_update_tweet?(user, tweet, \"Goodbye world!\")\n    end\n\n    test \"does not allow a user to update someone elses tweet\" do\n      [user, user2] = generate_many(user(), 2)\n      tweet = Domain.create_tweet!(\"Hello world!\", actor: user)\n\n      refute Domain.can_update_tweet?(user2, tweet, \"Goodbye world!\")\n    end\n\n    test \"allows an admin user to update someone elses tweet\" do\n      user = generate(user())\n      admin = generate(user(admin?: true))\n      tweet = generate(tweet(actor: user))\n\n      assert Domain.can_update_tweet?(admin, tweet, \"Goodbye world!\")\n    end\n  end\n\n  describe \"calculations\" do\n    test \"text length calculation computes the length of the text\" do\n      user = generate(user())\n      tweet = generate(tweet(text: \"Hello world!\", actor: user))\n      assert Ash.calculate!(tweet, :tweet_length) == 12\n    end\n  end\nend\n\nExUnit.run()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nRunning ExUnit with seed: 462155, max_cases: 28\n\n.......\nFinished in 0.08 seconds (0.00s async, 0.08s sync)\n2 properties, 5 tests, 0 failures\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{total: 7, failures: 0, excluded: 0, skipped: 0}\n```","ref":"test-resources.html#write-some-tests"},{"type":"extras","title":"Authorize Access to Resources","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Authorize Access to Resources\n\n```elixir\nMix.install(\n  [\n    {:ash, \"~> 3.0\"},\n    {:simple_sat, \"~> 0.1\"},\n    {:kino, \"~> 0.12\"}\n  ],\n  consolidate_protocols: false\n)\n\nLogger.configure(level: :warning)\nApplication.put_env(:ash, :policies, show_policy_breakdowns?: true)\n```","ref":"authorize-access-to-resources.html"},{"type":"extras","title":"Introduction - Authorize Access to Resources","doc":"A key feature of Ash is the ability to build security directly into your resources. We do this with policies.\n\nBecause how you write policies is *extremely* situational, this how-to guide provides a list of \"considerations\" as opposed to \"instructions\".\n\nFor more context, read the [policies guide](https://hexdocs.pm/ash/policies.html#policies).","ref":"authorize-access-to-resources.html#introduction"},{"type":"extras","title":"Writing Policies - Authorize Access to Resources","doc":"1. Consider whether or not you want to adopt a specific style of authorization, like [ACL](https://en.wikipedia.org/wiki/Access-control_list), or [RBAC](https://en.wikipedia.org/wiki/Role-based_access_control). For standard RBAC, look into [AshRbac](https://hexdocs.pm/ash_rbac/getting_started.html), and you may not need to write any of your own policies at that point\n2. Determine if there are any `bypass` policies to add (admin users, super users, etc.). Consider placing this on the domain, instead of the resource\n3. Begin by making an inventory of each action on your resource, and under what conditions a given actor may be allowed to perform them. If all actions of a given type have the same criteria, we will typically use the `action_type(:type)` condition\n4. Armed with this inventory, begin to write policies. Start simple, write a policy per action type, and add a description of what your policy accomplishes.\n5. Find patterns, like cross-cutting checks that exist in all policies, that can be expressed as smaller, simpler policies\n6. Determine if any field policies are required to prohibit access to attributes/calculations/aggregates\n7. Finally, you can confirm your understanding of the authorization flow for a given resource by generating policy charts with `mix ash.generate_policy_charts` (field policies are not currently included in the generated charts)","ref":"authorize-access-to-resources.html#writing-policies"},{"type":"extras","title":"Example - Authorize Access to Resources","doc":"<!-- livebook:{\"disable_formatting\":true} -->\n\n```elixir\ndefmodule User do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets\n\n  actions do\n    defaults [:read, create: [:admin?]]\n  end\n\n  attributes do\n    uuid_primary_key :id\n    attribute :admin?, :boolean do\n      allow_nil? false\n      default false\n    end\n  end\nend\n\ndefmodule Tweet do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets,\n    authorizers: [Ash.Policy.Authorizer]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :text, :string do\n      allow_nil? false\n      constraints max_length: 144\n      public? true\n    end\n\n    attribute :hidden?, :boolean do\n      allow_nil? false\n      default false\n      public? true\n    end\n\n    attribute :private_note, :string do\n      sensitive? true\n      public? true\n    end\n  end\n\n  calculations do\n    calculate :tweet_length, :integer, expr(string_length(text)) do\n      public? true\n    end\n  end\n\n  relationships do\n    belongs_to :user, User, allow_nil?: false\n  end\n\n  actions do\n    defaults [:read, update: [:text, :hidden?, :private_note]]\n\n    create :create do\n      primary? true\n      accept [:text, :hidden?, :private_note]\n      change relate_actor(:user)\n    end\n  end\n\n  policies do\n    # action_type-based policies\n    policy action_type(:read) do\n      # each policy has a description\n      description \"If a tweet is hidden, only the author can read it. Otherwise, anyone can.\"\n      # first check this. If true, then this policy passes\n      authorize_if relates_to_actor_via(:user)\n      # then check this. If false, then this policy fails\n      forbid_if expr(hidden? == true)\n      # otherwise, this policy passes\n      authorize_if always()\n    end\n\n    # blanket allow-listing of creates\n    policy action_type(:create) do\n      description \"Anyone can create a tweet\"\n      authorize_if always()\n    end\n\n    policy action_type(:update) do\n      description \"Only an admin or the user who tweeted can edit their tweet\"\n      # first check this. If true, then this policy passes\n      authorize_if actor_attribute_equals(:admin?, true)\n      # then check this. If true, then this policy passes\n      authorize_if relates_to_actor_via(:user)\n      # otherwise, there is nothing left to check and no decision, so *this policy fails*\n    end\n  end\n\n\n  field_policies do\n    # anyone can see these fields\n    field_policy [:text, :tweet_length] do\n      description \"Public tweet fields are visible\"\n      authorize_if always()\n    end\n\n    field_policy [:hidden?, :private_note] do\n      description \"hidden? and private_note are only visible to the author\"\n      authorize_if relates_to_actor_via(:user)\n    end\n  end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource Tweet do\n      define :create_tweet, action: :create, args: [:text]\n      define :update_tweet, action: :update, args: [:text]\n      define :list_tweets, action: :read\n      define :get_tweet, action: :read, get_by: [:id]\n    end\n\n    resource User do\n      define :create_user, action: :create\n    end\n  end\nend\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 2, 117, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   Ash.Domain.Dsl,\n   %{opts: [], entities: [...]},\n   Ash.Domain.Dsl,\n   Ash.Domain.Dsl.Resources.Options,\n   ...\n ]}\n```","ref":"authorize-access-to-resources.html#example"},{"type":"extras","title":"Interacting with resources that have policies - Authorize Access to Resources","doc":"```elixir\n# doing forbidden things produces an `Ash.Error.Forbidden`\nuser = Domain.create_user!()\nother_user = Domain.create_user!()\n\ntweet = Domain.create_tweet!(\"hello world!\", actor: user)\nDomain.update_tweet!(tweet, \"Goodbye world\", actor: other_user)\n```\n\n```elixir\n# Reading data applies policies as filters\n\nuser = Domain.create_user!()\nother_user = Domain.create_user!()\n\nmy_hidden_tweet = Domain.create_tweet!(\"hello world!\", %{hidden?: true}, actor: user)\n\nother_users_hidden_tweet =\n  Domain.create_tweet!(\"hello world!\", %{hidden?: true}, actor: other_user)\n\nmy_tweet = Domain.create_tweet!(\"hello world!\", actor: user)\nother_users_tweet = Domain.create_tweet!(\"hello world!\", actor: other_user)\n\ntweet_ids = Domain.list_tweets!(actor: user) |> Enum.map(& &1.id)\n\n# I see my own hidden tweets, and other users non-hidden tweets\ntrue = my_hidden_tweet.id in tweet_ids\ntrue = other_users_tweet.id in tweet_ids\n\n# but not other users hidden tweets\nfalse = other_users_hidden_tweet.id in tweet_ids\n\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n```elixir\n# Field policies return hidden fields as `%Ash.ForbiddenField{}`\n\nuser = Domain.create_user!()\nother_user = Domain.create_user!()\n\nother_users_tweet =\n  Domain.create_tweet!(\"hello world!\", %{private_note: \"you can't see this!\"}, actor: other_user)\n\n%Ash.ForbiddenField{} = Domain.get_tweet!(other_users_tweet.id, actor: user).private_note\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Ash.ForbiddenField \n```\n\n```elixir\nTweet\n|> Ash.Policy.Chart.Mermaid.chart()\n|> Kino.Shorts.mermaid()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```mermaid\nflowchart TB\nsubgraph at least one policy applies\ndirection TB\nat_least_one_policy[\"action.type == :read\nor action.type == :create\nor action.type == :update\"]\nend\nat_least_one_policy--False-->Forbidden\nat_least_one_policy--True-->0_conditions\nsubgraph Policy 1[If a tweet is hidden, only the author can read it. Otherwise, anyone can.]\ndirection TB\n0_conditions{\"action.type == :read\"}\n0_checks_0{\"record.user == actor\"}\n0_checks_1{\"hidden? == true\"}\nend\n0_conditions--True-->0_checks_0\n0_conditions--False-->1_conditions\n0_checks_0--True-->1_conditions\n0_checks_0--False-->0_checks_1\n0_checks_1--True-->Forbidden\n0_checks_1--False-->1_conditions\nsubgraph Policy 2[Anyone can create a tweet]\ndirection TB\n1_conditions{\"action.type == :create\"}\nend\nsubgraph Policy 3[Only an admin or the user who tweeted can edit their tweet]\ndirection TB\n2_conditions{\"action.type == :update\"}\n2_checks_0{\"actor.admin? == true\"}\n2_checks_1{\"record.user == actor\"}\nend\n2_conditions--True-->2_checks_0\n2_conditions--False-->Authorized\n2_checks_0--True-->Authorized\n2_checks_0--False-->2_checks_1\n2_checks_1--True-->Authorized\n2_checks_1--False-->Forbidden\nsubgraph results[Results]\nAuthorized([Authorized])\nForbidden([Forbidden])\nend\n1_conditions--Or-->2_conditions\n```","ref":"authorize-access-to-resources.html#interacting-with-resources-that-have-policies"},{"type":"extras","title":"Encrypt Attributes","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Encrypt Attributes\n\n```elixir\nMix.install([{:ash, \"~> 3.0\"}, {:ash_cloak, \"~> 0.1.0\"}, {:cloak, \"~> 1.1\"}],\n  consolidate_protocols: false\n)\n\nApplication.put_env(:my_app, MyApp.Vault,\n  ciphers: [\n    default: {\n      Cloak.Ciphers.AES.GCM,\n      tag: \"AES.GCM.V1\",\n      key: Base.decode64!(\"ETpvtowVAL7JmcxfqJ+XVQWzKrt1ynAkC0vT7AxfyNU=\"),\n      iv_length: 12\n    }\n  ]\n)\n\ndefmodule MyApp.Vault do\n  use Cloak.Vault, otp_app: :my_app\nend\n\nMyApp.Vault.start_link()\n```","ref":"encrypt-attributes.html"},{"type":"extras","title":"Introduction - Encrypt Attributes","doc":"When dealing with PII(Personally Identifiable Information) or other sensitive data, we often want to encrypt this data, and control access to the decrypted values.\n\nTo do this in `Ash`, we do that with `AshCloak`. See the getting started guide in `AshCloak` for installation instructions.","ref":"encrypt-attributes.html#introduction"},{"type":"extras","title":"Encrypting attributes - Encrypt Attributes","doc":"1. If you have not yet, follow the getting started guide for `AshCloak` and `Cloak`\n2. Add the `AshCloak` extension to your resource\n3. Configure the attributes that should be encrypted\n4. Add any other additional desired configuration (provided by `AshCloak`)","ref":"encrypt-attributes.html#encrypting-attributes"},{"type":"extras","title":"Examples - Encrypt Attributes","doc":"<!-- livebook:{\"disable_formatting\":true} -->\n\n```elixir\ndefmodule User do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets,\n    extensions: [AshCloak]\n\n  cloak do\n    vault MyApp.Vault\n    attributes [:ssn]\n  end\n\n  attributes do\n    uuid_primary_key :id\n    attribute :ssn, :string, allow_nil?: false\n  end\n\n  actions do\n    defaults [:read, create: [:ssn], update: [:ssn]]\n  end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource User do\n      define(:create_user, action: :create, args: [:ssn])\n      define(:update_user, action: :update, args: [:ssn])\n      define(:list_users, action: :read)\n    end\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 1, 255, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   Ash.Domain.Dsl,\n   %{opts: [], entities: [...]},\n   Ash.Domain.Dsl,\n   Ash.Domain.Dsl.Resources.Options,\n   ...\n ]}\n```","ref":"encrypt-attributes.html#examples"},{"type":"extras","title":"Data is encrypted when modified and is not displayed when inspecting. - Encrypt Attributes","doc":"```elixir\nuser = Domain.create_user!(\"111-11-1111\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#User<\n  __meta__: #Ecto.Schema.Metadata<:loaded>,\n  id: \"bc5284fe-294a-485e-8585-06130a4bca4e\",\n  aggregates: %{},\n  calculations: %{},\n  ...\n>\n```\n\n```elixir\n# AshCloak turned ssn into a calculation\nuser.ssn\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Ash.NotLoaded<:calculation, field: :ssn>\n```\n\n```elixir\n# Load the value to decrypt it on-demand\nAsh.load!(user, :ssn).ssn\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"111-11-1111\"\n```","ref":"encrypt-attributes.html#data-is-encrypted-when-modified-and-is-not-displayed-when-inspecting"},{"type":"extras","title":"Prevent concurrent writes","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Prevent concurrent writes\n\n```elixir\nMix.install([{:ash, \"~> 3.0\"}], consolidate_protocols: false)\n# Set to `:debug` if you want to see ETS logs\nLogger.configure(level: :warning)\n```","ref":"prevent-concurrent-writes.html"},{"type":"extras","title":"Introduction - Prevent concurrent writes","doc":"Often, when working with resources, we want to ensure that a record has not been edited since we last read it. We may want this for UX reasons, or for ensuring data consistency, etc.\n\nTo ensure that a record hasn't been updated since the last time we read it, we use Optimistic Locking.\n\nFor more information, see the documentation for `Ash.Resource.Change.OptimisticLock`.","ref":"prevent-concurrent-writes.html#introduction"},{"type":"extras","title":"Add Optimistic Locking - Prevent concurrent writes","doc":"1. Add a `:version` attribute to your resource, of type `:integer`, with a default of `1` and `allow_nil?: false`\n2. Add `change optimistic_lock(:version)`. Where you add it depends on which actions you want to prevent concurrent writes for:\n   * If you want optimistic locking to occur for _specific actions_:\n     1. Add `change optimistic_lock(:version)` to those specific actions\n   * If you want optimistic locking to occur for _all action types_:\n     1. Add a global `changes` block to the resource, if you do not have one\n     2. Add `change optimistic_lock(:version), on: [:create, :update, :destroy]`\n   * If you want to apply optimistic locking to _many but not all actions_:\n     1. Add a global `changes` block to the resource, if you do not have one\n     2. Add `change optimistic_lock(:version), where: action_is([:action1, :action2, :action3])`","ref":"prevent-concurrent-writes.html#add-optimistic-locking"},{"type":"extras","title":"Examples - Prevent concurrent writes","doc":"<!-- livebook:{\"disable_formatting\":true} -->\n\n```elixir\ndefmodule Address do\n  use Ash.Resource,\n    domain: Domain,\n    data_layer: Ash.DataLayer.Ets\n\n  attributes do\n    uuid_primary_key :id\n    attribute :version, :integer, allow_nil?: false, default: 1\n    attribute :state, :string, allow_nil?: false\n    attribute :county, :string, allow_nil?: false\n  end\n\n  actions do\n    defaults [:read, create: [:state, :county]]\n\n    update :update do\n      accept [:state, :county]\n      change optimistic_lock(:version)\n    end\n  end\n\n  # apply to all actions\n  # changes do\n  #   change optimistic_lock(:version), on: [:create, :update, :destroy]\n  # end\n\n  # apply to a list of actions\n  # changes do\n  #   change optimistic_lock(:version), where: action_is([:action1, :action2, :action3])\n  # end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource Address do\n      define(:get_address, action: :read, get_by: [:id])\n      define(:create_address, action: :create, args: [:state, :county])\n      define(:update_address, action: :update)\n    end\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 2, 1, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   Ash.Domain.Dsl,\n   %{opts: [], entities: [...]},\n   Ash.Domain.Dsl,\n   Ash.Domain.Dsl.Resources.Options,\n   ...\n ]}\n```","ref":"prevent-concurrent-writes.html#examples"},{"type":"extras","title":"Triggering a StaleRecord error - Prevent concurrent writes","doc":"```elixir\naddress = Domain.create_address!(\"FL\", \"Pinellas\")\nDomain.update_address!(address, %{state: \"NC\", county: \"Guilford\"})\n\n# `address` still has a version of `1`, so our optimistic lock should catch it!\nDomain.update_address(address, %{county: \"Miami-Dade\"})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:error,\n %Ash.Error.Invalid{\n   changeset: \"#Changeset<>\",\n   errors: [\n     %Ash.Error.Changes.StaleRecord{\n       resource: \"Address\",\n       filter: %{\"version\" => 1},\n       splode: Ash.Error,\n       bread_crumbs: [],\n       vars: [],\n       path: [],\n       stacktrace: #Splode.Stacktrace<>,\n       class: :invalid\n     }\n   ]\n }}\n```","ref":"prevent-concurrent-writes.html#triggering-a-stalerecord-error"},{"type":"extras","title":"Refetching a record to get the latest version - Prevent concurrent writes","doc":"```elixir\naddress = Domain.create_address!(\"FL\", \"Pinellas\")\nDomain.update_address!(address, %{state: \"NC\", county: \"Guilford\"})\n\ncase Domain.update_address(address, %{county: \"Miami-Dade\"}) do\n  {:error, %Ash.Error.Invalid{errors: [%Ash.Error.Changes.StaleRecord{}]}} ->\n    # In a liveview, you wouldn't just refetch and resubmit\n    # you would show the user an error and have them submit the form again\n    address = Domain.get_address!(address.id)\n    Domain.update_address!(address, %{county: \"Miami-Dade\"})\n\n  {:ok, domain} ->\n    domain\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Address<\n  __meta__: #Ecto.Schema.Metadata<:loaded>,\n  id: \"91e552e1-3307-4d68-969f-1b7e6651e695\",\n  version: 3,\n  state: \"NC\",\n  county: \"Miami-Dade\",\n  aggregates: %{},\n  calculations: %{},\n  ...\n>\n```","ref":"prevent-concurrent-writes.html#refetching-a-record-to-get-the-latest-version"},{"type":"extras","title":"Wrap External APIs","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Wrap External APIs\n\n```elixir\nMix.install(\n  [\n    {:ash, \"~> 3.0\"},\n    {:req, \"~> 0.4.0\"}\n  ],\n  consolidate_protocols: false\n)\n```","ref":"wrap-external-apis.html"},{"type":"extras","title":"Introduction - Wrap External APIs","doc":"Wrapping external APIs in Ash resources can allow you to leverage the rich and consistent interface provided by `Ash.Resource` for interactions with external services.\n\nThere are a few approaches to how you might do this, including the still in progress [AshJsonApiWrapper](https://github.com/ash-project/ash_json_api_wrapper). Here we will leverage \"manual actions\" as this is fully supported by Ash, and is the commonly used approach.\n\nThis approach is most appropriate when you are working with an API that exposes some data, like entities, list of entities, etc. For this example, we will be interacting with https://openlibrary.org, which allows for us to search and list books.\n\nThis guide covers reading data from the external API, not creating/updating it. This can be implemented using manual actions of a different type, or generic actions.","ref":"wrap-external-apis.html#introduction"},{"type":"extras","title":"Wrapping External APIs - Wrap External APIs","doc":"1. Create a resource for interacting with the given API\n2. Create a manual read action\n3. In this manual action, we will:\n   1. call the target API\n   2. transform the results\n   3. apply query operations to simulate capabilities provided by Ash\n\nIn the example below, we are calling to a *paginated* API, and we want to continue fetching results until we have reached the amount of results requested by the `Ash.Query`. We show this to illustrate that you can do all kinds of creative things when working with external APIs in manual actions.","ref":"wrap-external-apis.html#wrapping-external-apis"},{"type":"extras","title":"Example - Wrap External APIs","doc":"<!-- livebook:{\"disable_formatting\":true} -->\n\n```elixir\ndefmodule Doc do\n  use Ash.Resource,\n    domain: Domain\n\n  attributes do\n    uuid_primary_key :id\n    attribute :author_name, :string\n    attribute :title, :string\n    attribute :type, :string\n  end\n\n  actions do\n    read :search do\n      primary? true\n      argument :query, :string, allow_nil?: false\n      prepare fn query, _ ->\n        # We require that they limit the results to some reasonable set\n        # (because this API is huge)\n        cond do\n          query.limit && query.limit > 250 ->\n            Ash.Query.add_error(query, \"must supply a limit that is less than or equal to 250\")\n          query.limit ->\n            query\n          true ->\n            # limit 5 by default\n            Ash.Query.limit(query, 5)\n        end\n      end\n\n      manual Doc.Actions.Read\n    end\n  end\nend\n\ndefmodule Domain do\n  use Ash.Domain,\n    validate_config_inclusion?: false\n\n  resources do\n    resource Doc do\n      define :search, args: [:query]\n    end\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Domain, <<70, 79, 82, 49, 0, 0, 250, ...>>,\n [\n   Ash.Domain.Dsl.Resources.Resource,\n   Ash.Domain.Dsl.Resources.Options,\n   Ash.Domain.Dsl,\n   %{opts: [], entities: [...]},\n   Ash.Domain.Dsl,\n   Ash.Domain.Dsl.Resources.Options,\n   ...\n ]}\n```\n\n```elixir\ndefmodule Doc.Actions.Read do\n  use Ash.Resource.ManualRead\n\n  def read(query, _, _opts, _context) do\n    # we aren't handling these query options to keep the example simple\n    # but you could on your own\n    if query.sort != [] || query.offset != 0 do\n      {:error, \"Cannot sort or offset documents\"}\n    end\n\n    if query.sort && query.sort != [] do\n      raise \"Cannot apply a sort to docs read\"\n    end\n\n    if query.offset && query.offset != 0 do\n      raise \"Cannot apply a sort to docs read\"\n    end\n\n    limit = query.limit || :infinity\n\n    query = Ash.Query.unset(query, :limit)\n\n    query_results =\n      Stream.resource(\n        fn ->\n          {limit, 0}\n        end,\n        fn\n          {remaining, page_number} when remaining <= 0 ->\n            {:halt, {0, page_number}}\n\n          {remaining, page_number} ->\n            api_results =\n              query.arguments.query\n              |> get!(page_number)\n              |> Enum.map(&to_doc/1)\n\n            case Ash.Query.apply_to(query, api_results) do\n              {:ok, []} ->\n                {:halt, remaining}\n\n              {:ok, results} ->\n                count_of_results = Enum.count(results)\n\n                cond do\n                  # the api gives us batches of 100\n                  remaining == :infinity && count_of_results == 100 ->\n                    {results, {:infinity, page_number + 1}}\n\n                  remaining == :infinity ->\n                    {results, {0, page_number + 1}}\n\n                  true ->\n                    still_remaining = remaining - count_of_results\n\n                    results =\n                      if still_remaining <= 0 do\n                        Enum.take(results, remaining)\n                      else\n                        results\n                      end\n\n                    {results, {still_remaining, page_number + 1}}\n                end\n\n              {:error, error} ->\n                raise Ash.Error.to_ash_error(error)\n            end\n        end,\n        fn _ -> :ok end\n      )\n      |> Enum.to_list()\n\n    {:ok, query_results}\n  end\n\n  defp to_doc(api_doc) do\n    %Doc{author_name: api_doc[\"author_name\"], type: api_doc[\"type\"], title: api_doc[\"title\"]}\n  end\n\n  defp get!(query, page) do\n    params =\n      if page == 0 do\n        [q: query]\n      else\n        [q: query, page: page]\n      end\n\n    Req.get!(\"https://openlibrary.org/search.json\", params: params).body[\"docs\"]\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Doc.Actions.Read, <<70, 79, 82, 49, 0, 0, 25, ...>>, {:get!, 2}}\n```","ref":"wrap-external-apis.html#example"},{"type":"extras","title":"Now we can use this like any other Ash resource - Wrap External APIs","doc":"```elixir\nDomain.search!(\"Lord of the rings\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Lord of the Rings\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Fellowship of the Ring\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Two Towers\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Return of the King\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Lord of the Rings\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```\n\n```elixir\nrequire Ash.Query\nquery = Doc |> Ash.Query.filter(contains(title, \"Two\"))\nDomain.search!(\"Lord of the rings\", query: query)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Two Towers\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"Alessio Cavatore\", \"Rick Priestley\"],\n    title: \"The Lord of the Rings - The Two Towers\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"The Two Towers\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"Alessio Cavatore\", \"Rick Priestley\"],\n    title: \"The Lord of the Rings - The Two Towers\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >,\n  #Doc<\n    __meta__: #Ecto.Schema.Metadata<:built, \"\">,\n    id: nil,\n    author_name: [\"J.R.R. Tolkien\"],\n    title: \"Two Towers : The Lord of the Rings\",\n    type: \"work\",\n    aggregates: %{},\n    calculations: %{},\n    ...\n  >\n]\n```","ref":"wrap-external-apis.html#now-we-can-use-this-like-any-other-ash-resource"},{"type":"extras","title":"Upgrade","doc":"# Upgrade\n\nThis document has been [moved](upgrading-to-3-0.html).","ref":"upgrade.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}